c-----------------------------------------------------------------------
c     file aux_surface.f:  contains subprograms that perform
c     computations directly on boundary of physical-field.  
c     these routines were formerly part of aux_field.f.
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c     code organization.
c-----------------------------------------------------------------------
c     1.  surface_exb.
c     2.  surface_b_radial 
c-----------------------------------------------------------------------
c     subprogram 1. surface_exb
c     set the n=0 flow at the surface according to the applied e and
c     local b.
c-----------------------------------------------------------------------
      SUBROUTINE surface_exb(vdum,vcent)
      USE local
      USE fields
      USE seam_storage_mod
      USE global
      USE input
      USE surface_ints
      USE coils
      USE rmf
      IMPLICIT NONE

      TYPE(cvector_type), DIMENSION(:), INTENT(INOUT) :: vdum
      REAL(r8), INTENT(IN) :: vcent

      REAL(r8), DIMENSION(2) :: normal,tangent,exb
      REAL(r8), DIMENSION(3) :: bes
      REAL(r8) :: ncomp,tcomp,bigr=1,dx,dy,gfac,lvolt,evert
      INTEGER(i4) :: ibe,ibl,imode,iv,ix,iy,ivp,ibase
c-----------------------------------------------------------------------
c     find the n=0 mode if there is one.
c-----------------------------------------------------------------------
      imode=1
      DO
        IF (imode>nmodes) RETURN
        IF (keff(imode)==0) EXIT
        imode=imode+1
      ENDDO
      IF (geom=='tor') THEN
        gfac=1/twopi
      ELSE
        gfac=1/per_length
      ENDIF
c-----------------------------------------------------------------------
c     center voltages according to vcent.
c-----------------------------------------------------------------------
      lvolt=vcent*volt+(1-vcent)*volt_old
      IF (coil_type(1:7)=='uniform') THEN
        lvolt=vcent*volt+(1-vcent)*volt_old
      ENDIF
      evert=vcent*e_vert+(1-vcent)*e_vert_old
c-----------------------------------------------------------------------
c     loop over all blocks touching the surface of the domain.
c-----------------------------------------------------------------------
      block: DO ibl=1,SIZE(exblock_list)
        ibe=exblock_list(ibl)
c-----------------------------------------------------------------------
c       loop over the block boundary, and find exb for points on the
c       external surface (the third component of be_eq includes a 
c       factor of R).
c-----------------------------------------------------------------------
        vert: DO iv=1,seam(ibe)%nvert
          IF (.NOT.seam(ibe)%expoint(iv).OR.
     $             seam(ibe)%r0point(iv).OR.
     $             seam(ibe)%excorner(iv)) CYCLE
          ix=seam(ibe)%vertex(iv)%intxy(1)
          iy=seam(ibe)%vertex(iv)%intxy(2)
          normal =seam(ibe)%vertex(iv)%norm
          tangent=seam(ibe)%vertex(iv)%tang
          bes=be_eq(ibe)%arr(:,ix,iy)
          IF (geom=='tor') THEN

            bigr=rb(ibe)%rz%fs(1,ix,iy)
            bes(3)=bes(3)/bigr
          ENDIF
          bes=bes+be(ibe)%arr(:,ix,iy,imode)
          IF (coil_type(1:11)=='theta_pinch') THEN
            lvolt=coil_voltage1(rb(ibe)%rz%fs(2,ix,iy),t+(vcent*dt))
          ENDIF
          exb(1)=evert*ABS(tangent(2))*bes(3)
     $            -lvolt*gfac*bes(2)/bigr
          exb(2)=lvolt*gfac*bes(1)/bigr
          ncomp=SUM(exb*normal)/SUM(bes**2)
          IF((coil_type(1:12)=='theta_pinch2') .OR.
     $       (coil_type(1:12)=='theta_pinch3')) THEN
            ncomp = min(0.0_r8, ncomp)
            ncomp = max(-1.e5_r8, ncomp)
          ENDIF
          tcomp=SUM(vdum(ibe)%arr(1:2,ix,iy,imode)*tangent)
          vdum(ibe)%arr(1:2,ix,iy,imode)=ncomp*normal+tcomp*tangent
        ENDDO vert
c-----------------------------------------------------------------------
c       side-centered bases:
c-----------------------------------------------------------------------
        IF (poly_degree>1) THEN
          seg: DO iv=1,seam(ibe)%nvert
            ivp=iv-1
            IF (ivp==0) ivp=seam(ibe)%nvert
            IF (.NOT.(seam(ibe)%expoint(iv).AND.seam(ibe)%expoint(ivp))
     $          .OR.(seam(ibe)%r0point(iv).AND.seam(ibe)%r0point(ivp)))
     $        CYCLE
            ix=seam(ibe)%segment(iv)%intxys(1)
            iy=seam(ibe)%segment(iv)%intxys(2)
            DO ibase=1,poly_degree-1
              normal =seam(ibe)%segment(iv)%norm(:,ibase)
              tangent=seam(ibe)%segment(iv)%tang(:,ibase)
c-PRE triangles
              IF (seam(ibe)%segment(iv)%h_side) THEN
                dx=rb(ibe)%be%dx(ibase+1)+
     $             MIN(seam(ibe)%segment(iv)%intxyn(1),
     $                 seam(ibe)%segment(iv)%intxyp(1))
                dy=iy
              ELSE
                dx=ix
                dy=rb(ibe)%be%dy(ibase+poly_degree)+
     $             MIN(seam(ibe)%segment(iv)%intxyn(2),
     $                 seam(ibe)%segment(iv)%intxyp(2))
              ENDIF
              CALL lagr_quad_eval(rb(ibe)%be_eq,dx,dy,0_i4)
              bes=rb(ibe)%be_eq%f
              IF (geom=='tor') THEN
                CALL lagr_quad_eval(rb(ibe)%rz,dx,dy,0_i4)
                bigr=rb(ibe)%rz%f(1)
                bes(3)=bes(3)/bigr
              ENDIF
              IF (coil_type(1:11)=='theta_pinch') THEN
                CALL lagr_quad_eval(rb(ibe)%rz,dx,dy,0_i4)
                lvolt=coil_voltage1(rb(ibe)%rz%f(2),t+(vcent*dt))
              ENDIF
              IF (seam(ibe)%segment(iv)%h_side) THEN
                bes=bes+be(ibe)%arrh(:,ibase,ix,iy,imode)
                exb(1)=evert*ABS(tangent(2))*bes(3)
     $                  -lvolt*gfac*bes(2)/bigr
                exb(2)=lvolt*gfac*bes(1)/bigr
                ncomp=SUM(exb*normal)/SUM(bes**2)
                IF((coil_type(1:12)=='theta_pinch2') .OR.
     $             (coil_type(1:12)=='theta_pinch3')) THEN
                  ncomp = min(0.0_r8, ncomp)
                  ncomp = max(-1.e5_r8, ncomp)
                ENDIF
                tcomp=SUM(vdum(ibe)%arrh(1:2,ibase,ix,iy,imode)*tangent)
                vdum(ibe)%arrh(1:2,ibase,ix,iy,imode)=ncomp*normal
     $                                               +tcomp*tangent
              ELSE
                bes=bes+be(ibe)%arrv(:,ibase,ix,iy,imode)
                exb(1)=evert*ABS(tangent(2))*bes(3)
     $                  -lvolt*gfac*bes(2)/bigr
                exb(2)=lvolt*gfac*bes(1)/bigr
                ncomp=SUM(exb*normal)/SUM(bes**2)
                IF((coil_type(1:12)=='theta_pinch2') .OR.
     $             (coil_type(1:12)=='theta_pinch3')) THEN
                  ncomp = min(0.0_r8, ncomp)
                  ncomp = max(-1.e5_r8, ncomp)
                ENDIF
                tcomp=SUM(vdum(ibe)%arrv(1:2,ibase,ix,iy,imode)*tangent)
                vdum(ibe)%arrv(1:2,ibase,ix,iy,imode)=ncomp*normal
     $                                               +tcomp*tangent
              ENDIF
            ENDDO
          ENDDO seg
        ENDIF
      ENDDO block
c-----------------------------------------------------------------------
c     terminate.
c-----------------------------------------------------------------------
      RETURN
      END SUBROUTINE surface_exb
c-----------------------------------------------------------------------
c     subprogram 5. surface_b_radial
c     set the n=0 magnetic field at the surface according to the theta-
c     pinch coils.
c-----------------------------------------------------------------------
      SUBROUTINE surface_b_radial(Bdum)
      USE fields
      USE seam_storage_mod
      USE global
      USE input
      USE surface_ints
      IMPLICIT NONE

      TYPE(cvector_type), DIMENSION(:), INTENT(INOUT) :: Bdum

      REAL(r8), DIMENSION(2) :: normal,tangent
      REAL(r8), DIMENSION(2) :: bpar, bperp
      REAL(r8):: bperp_abs
      REAL(r8) :: dx,dy,bigr=1,gfac
      INTEGER(i4) :: ibe,ibl,imode,iv,ix,iy,ivp,ibase
c-----------------------------------------------------------------------
c     find the n=0 mode if there is one.
c-----------------------------------------------------------------------
c- RDM / Disable subroutine surface_b_radial.
      IF((coil_type(1:12)=='theta_pinch2') .OR. 
     $   (coil_type(1:12)=='theta_pinch4')) THEN
          RETURN
      ENDIF
c-/ RDM
      IF(rmf_bomega .GT. 0.0) THEN
        CALL surface_b_radial_rmf(Bdum, 1.0_r8)  ! RDM
      ENDIF

      imode=1
      DO
        IF (imode>nmodes) RETURN
        IF (keff(imode)==0) EXIT
        imode=imode+1
      ENDDO
      IF (geom=='tor') THEN
        gfac=1/twopi
      ELSE
        gfac=1/per_length
      ENDIF
c-----------------------------------------------------------------------
c     loop over all blocks touching the surface of the domain.
c-----------------------------------------------------------------------
      block: DO ibl=1,SIZE(exblock_list)
        ibe=exblock_list(ibl)
c-----------------------------------------------------------------------
c       loop over the block boundary, and find exb for points on the
c       external surface (the third component of be_eq includes a 
c       factor of R).
c-----------------------------------------------------------------------
        vert: DO iv=1,seam(ibe)%nvert
          IF (.NOT.seam(ibe)%expoint(iv).OR.
     $             seam(ibe)%r0point(iv).OR.
     $             seam(ibe)%excorner(iv)) CYCLE
          ix=seam(ibe)%vertex(iv)%intxy(1)
          iy=seam(ibe)%vertex(iv)%intxy(2)
          normal =seam(ibe)%vertex(iv)%norm
          tangent=seam(ibe)%vertex(iv)%tang
          IF (geom=='tor') THEN
            bigr=rb(ibe)%rz%fs(1,ix,iy)
          ENDIF
          IF (coil_type(1:11)=='theta_pinch') THEN
            IF(normal(1) > 1.e-6) THEN   ! This is a radial wall
              bpar = DOT_PRODUCT(Bdum(ibe)%arr(1:2,ix,iy,imode),tangent)
     $                                                         *tangent
              bperp_abs = bperp_from_coil1(rb(ibe)%rz%fs(2,ix,iy),
     $                                            t+dt,gfac/bigr,normal)
              bperp = bperp_abs*normal
              Bdum(ibe)%arr(1:2,ix,iy,imode) = bpar + bperp
              Bdum(ibe)%arr(3,  ix,iy,imode) = 0.0
            ENDIF
          ENDIF
        ENDDO vert
c-----------------------------------------------------------------------
c       side-centered bases:
c-----------------------------------------------------------------------
        IF (poly_degree>1) THEN
          seg: DO iv=1,seam(ibe)%nvert
            ivp=iv-1
            IF (ivp==0) ivp=seam(ibe)%nvert
            IF (.NOT.(seam(ibe)%expoint(iv).AND.seam(ibe)%expoint(ivp))
     $          .OR.(seam(ibe)%r0point(iv).AND.seam(ibe)%r0point(ivp)))
     $        CYCLE
            IF (.NOT.seam(ibe)%segment(iv)%h_side) THEN
              ix=seam(ibe)%segment(iv)%intxys(1)
              iy=seam(ibe)%segment(iv)%intxys(2)
              DO ibase=1,poly_degree-1
                normal =seam(ibe)%segment(iv)%norm(:,ibase)
                tangent=seam(ibe)%segment(iv)%tang(:,ibase)
                IF (seam(ibe)%segment(iv)%h_side) THEN
                  dx=rb(ibe)%be%dx(ibase+1)+
     $               MIN(seam(ibe)%segment(iv)%intxyn(1),
     $                   seam(ibe)%segment(iv)%intxyp(1))
                  dy=iy
                ELSE
                  dx=ix
                  dy=rb(ibe)%be%dy(ibase+poly_degree)+
     $               MIN(seam(ibe)%segment(iv)%intxyn(2),
     $                   seam(ibe)%segment(iv)%intxyp(2))
                ENDIF
                IF (geom=='tor') THEN
                  CALL lagr_quad_eval(rb(ibe)%rz,dx,dy,0_i4)
                  bigr=rb(ibe)%rz%f(1)
                ENDIF

                CALL lagr_quad_eval(rb(ibe)%rz,dx,dy,0_i4)
                IF (coil_type(1:11)=='theta_pinch') THEN
                  IF(normal(1) > 1.e-6) THEN   ! This is a radial wall
                    bpar = DOT_PRODUCT(Bdum(ibe)%arrv(1:2,
     $                               ibase,ix,iy,imode),tangent)*tangent
                    bperp_abs = bperp_from_coil1(rb(ibe)%rz%f(2),
     $                                            t+dt,gfac/bigr,normal)
                    bperp = bperp_abs*normal
                    Bdum(ibe)%arrv(1:2,ibase,ix,iy,imode) = bpar + bperp
                    Bdum(ibe)%arrv(3,  ibase,ix,iy,imode) = 0.0
                  ENDIF
                ENDIF
              ENDDO
            ENDIF
          ENDDO seg
        ENDIF
      ENDDO block
c-----------------------------------------------------------------------
c     terminate
c-----------------------------------------------------------------------
      RETURN
      END SUBROUTINE surface_b_radial
