c#######################################################################
c
c     Nim_bound code suite
c     Copyright by Tibbar Plasma Technologies
c     2017
c
c     Programmed by D. C. Barnes
c     Tibbar Plasma Technologies
c
c#######################################################################
c
c     mesh and cut cells, etc. (mesh.f)
c
c#######################################################################
c
c     version 1.0 -- 04/06/15
c
c#######################################################################
c
      MODULE mesh_m
c
      USE glob_m
c
      IMPLICIT NONE
      INCLUDE 'params.f'
      INCLUDE 'priv.f'
c
      INTEGER::n_cut
      REAL(rn)::dth,dt
      REAL(rn),DIMENSION(:),ALLOCATABLE::th,t
c
      INTEGER,DIMENSION(:),ALLOCATABLE::c_i,c_j
      REAL(rn),DIMENSION(:),ALLOCATABLE::c_th,c_t,c_de,c_v
      CHARACTER(LEN=1),DIMENSION(:),ALLOCATABLE::c_d
      LOGICAL,DIMENSION(:,:),ALLOCATABLE::in
      REAL(rn),DIMENSION(:,:),ALLOCATABLE::vv_el
c
      CONTAINS
c
c#######################################################################
c     make mesh quantities
c-----------------------------------------------------------------------
      SUBROUTINE get_mesh
c local variables
      INTEGER::ith,it,nth1,nth2,nt1,nt2,i_c,it1,it2
      REAL(rn)::th1,th2,thof,tht,tt
      LOGICAL,DIMENSION(:,:),ALLOCATABLE::inx
c begin -- make zeta mesh
      dth=2*pi/nth; ALLOCATE(th(nth+1)); th=0
      DO ith=2,nth+1
         th(ith)=th(ith-1)+dth
      ENDDO
c make t (z) mesh
      dt=ll/(nt-1); ALLOCATE(t(nt)); t=-ll/2
      DO it=2,nt
         t(it)=t(it-1)+dt
      ENDDO
c get cut edge points
      nt1=g/dt+2; nt2=(ll-g)/dt+1
c
      ALLOCATE(in(nth+1,nt)); in=.TRUE.
      ALLOCATE(vv_el(nth+1,nt)); vv_el=0
c boundaries are outside
      in(:,1)=.FALSE.; in(:,nt)=.FALSE.; in(nth+1,:)=.FALSE.
      vv_el(:,1)=-vv_s; vv_el(:,nt)= vv_s
c mark interior of cathode as outside
      DO it=nt1,nt2
         thof=(pi-dl)/2-k*(-ll/2+g); tht=thof+k*t(it)
         DO WHILE(tht>2*pi)
            tht=tht-2*pi
         ENDDO
         it1=tht/dth+2
         thof=(pi+dl)/2-k*(-ll/2+g); tht=thof+k*t(it)
         DO WHILE(tht>2*pi)
            tht=tht-2*pi
         ENDDO
         it2=tht/dth+1
         IF(it2<it1) THEN
            in(it1:,it)=.FALSE.; in(:it2,it)=.FALSE.
            vv_el(it1:,it)=-vv_p; vv_el(:it2,it)=-vv_p
         ELSE
            in(it1:it2,it)=.FALSE.
            vv_el(it1:it2,it)=-vv_p
         ENDIF
      ENDDO
c mark interior of anode as outside
      DO it=nt1,nt2
         thof=(3*pi-dl)/2-k*(-ll/2+g); tht=thof+k*t(it)
         DO WHILE(tht>2*pi)
            tht=tht-2*pi
         ENDDO
         it1=tht/dth+2
         thof=(3*pi+dl)/2-k*(-ll/2+g); tht=thof+k*t(it)
         DO WHILE(tht>2*pi)
            tht=tht-2*pi
         ENDDO
         it2=tht/dth+1
         IF(it2<it1) THEN
            in(it1:,it)=.FALSE.; in(:it2,it)=.FALSE.
            vv_el(it1:,it)= vv_p; vv_el(:it2,it)= vv_p
         ELSE
            in(it1:it2,it)=.FALSE.
            vv_el(it1:it2,it)= vv_p
         ENDIF
      ENDDO
c count cut edge points
c     need periodic extension of inside array
      ALLOCATE(inx(nth+1,nt)); inx=in; inx(nth+1,:)=inx(1,:)
      n_cut=COUNT(inx(2:,2:nt-1).AND..NOT.inx(:nth,2:nt-1))
     .     +COUNT(inx(:nth,2:nt-1).AND..NOT.inx(2:,2:nt-1))
     .     +COUNT(inx(:nth,3:nt-1).AND..NOT.inx(:nth,2:nt-2))
     .     +COUNT(inx(:nth,2:nt-2).AND..NOT.inx(:nth,3:nt-1))
c allocate
      ALLOCATE(c_th(n_cut),c_t(n_cut),c_i(n_cut),c_j(n_cut)
     .        ,c_de(n_cut),c_v(n_cut),c_d(n_cut))
c assign cut edge points
      i_c=0
c left
      DO it=2,nt-1
         DO ith=2,nth+1
            IF(.NOT.inx(ith,it)) CYCLE; IF(inx(ith-1,it)) CYCLE
            i_c=i_c+1
c find tht (edge value)
            IF(vv_el(ith-1,it)>zero) THEN
c     anode right hand side
               tht=(3*pi+dl)/2-k*(-ll/2+g)+k*t(it)
            ELSE
c     cathode right hand side
               tht=(pi+dl)/2-k*(-ll/2+g)+k*t(it)
            ENDIF
            DO WHILE(tht>2*pi)
               tht=tht-2*pi
            ENDDO
            c_th(i_c)=tht; c_t(i_c)=t(it)
            c_i(i_c)=ith; c_j(i_c)=it
            c_de(i_c)=a*(th(ith)-tht); c_d(i_c)='l'
            c_v(i_c)=vv_el(ith-1,it)
         ENDDO
      ENDDO
c right
      DO it=2,nt-1
         DO ith=2,nth+1
            IF(inx(ith,it)) CYCLE; IF(.NOT.inx(ith-1,it)) CYCLE
            i_c=i_c+1
c find tht (edge value)
            IF(vv_el(ith,it)>zero) THEN
c     anode left hand side
               tht=(3*pi-dl)/2-k*(-ll/2+g)+k*t(it)
            ELSE
c     cathode left hand side
               tht=(pi-dl)/2-k*(-ll/2+g)+k*t(it)
            ENDIF
            DO WHILE(tht>2*pi)
               tht=tht-2*pi
            ENDDO
            c_th(i_c)=tht; c_t(i_c)=t(it)
            c_i(i_c)=ith-1; c_j(i_c)=it
            c_de(i_c)=a*(tht-th(ith-1)); c_d(i_c)='r'
            c_v(i_c)=vv_el(ith,it)
         ENDDO
      ENDDO
c down
      DO it=3,nt-1
         DO ith=1,nth
            IF(.NOT.inx(ith,it)) CYCLE; IF(inx(ith,it-1)) CYCLE
            i_c=i_c+1
c find tt (edge value)
            IF(vv_el(ith,it-1)>zero) THEN
c     anode left hand edge or top
               tht=th(ith)
 1             tt=(tht-(3*pi-dl)/2)/k-ll/2+g
               IF(k*(ll-2*g)>(pi-dl)/2) THEN
                  IF(tt<-ll/2) THEN
                     tht=tht+2*pi; GOTO 1
                  ELSEIF(tt>ll/2) THEN
                     tht=tht-2*pi; GOTO 1
                  ENDIF
               ENDIF
            ELSE
c     cathode left hand edge or top
               tht=th(ith)
 2             tt=(tht-(pi-dl)/2)/k-ll/2+g
               IF(k*(ll-2*g)>(3*pi-dl)/2) THEN
                  IF(tt<-ll/2) THEN
                     tht=tht+2*pi; GOTO 2
                  ELSEIF(tt>ll/2) THEN
                     tht=tht-2*pi; GOTO 2
                  ENDIF
               ENDIF
            ENDIF
            tt=MIN(tt,ll/2-g)
            c_th(i_c)=th(ith); c_t(i_c)=tt
            c_i(i_c)=ith; c_j(i_c)=it
            c_de(i_c)=t(it)-tt; c_d(i_c)='d'
            c_v(i_c)=vv_el(ith,it-1)
         ENDDO
      ENDDO
c up
      DO it=3,nt-1
         DO ith=1,nth
            IF(inx(ith,it)) CYCLE; IF(.NOT.inx(ith,it-1)) CYCLE
            i_c=i_c+1
c find tt (edge value)
            IF(vv_el(ith,it)>zero) THEN
c     anode right hand edge or bottom
               tht=th(ith)
 3             tt=(tht-(3*pi+dl)/2)/k-ll/2+g
               IF(k*(ll-2*g)>(pi-dl)/2) THEN
                  IF(tt<-ll/2) THEN
                     tht=tht+2*pi; GOTO 3
                  ELSEIF(tt>ll/2) THEN
                     tht=tht-2*pi; GOTO 3
                  ENDIF
               ENDIF
            ELSE
c     cathode right hand edge or bottom
               tht=th(ith)
 4             tt=(tht-(pi+dl)/2)/k-ll/2+g
               IF(k*(ll-2*g)>(3*pi-dl)/2) THEN
                  IF(tt<-ll/2) THEN
                     tht=tht+2*pi; GOTO 4
                  ELSEIF(tt>ll/2) THEN
                     tht=tht-2*pi; GOTO 4
                  ENDIF
               ENDIF
            ENDIF
            tt=MAX(tt,-ll/2+g)
            c_th(i_c)=th(ith); c_t(i_c)=tt
            c_i(i_c)=ith; c_j(i_c)=it-1
            c_de(i_c)=tt-t(it-1); c_d(i_c)='u'
            c_v(i_c)=vv_el(ith,it)
         ENDDO
      ENDDO
      DEALLOCATE(inx)
c periodic conditions
      WHERE(c_i==nth+1) c_i=1; WHERE(c_i==0) c_i=nth
c diagnostic plots
cc      CALL gnup_pointplot(c_th,c_t,n_cut,zero,2*pi,-ll/2,ll/2
cc     .                   ,'th','t','')
cc      CALL gnup_contplot(th,t,MERGE(one,zero,in),nth+1,nt,23
cc     .                  ,'th','t','vv_el')
cc      CALL gnup_contplot(th,t,vv_el,nth+1,nt,23
cc     .                  ,'th','t','vv_el')
c terminate
      RETURN
      END SUBROUTINE get_mesh
c
      END MODULE mesh_m
