c#######################################################################
c
c     Susie code suite
c     Copyright by Coronado Consulting
c     2015
c
c     Programmed by D. C. Barnes
c     Coronado Consulting
c
c#######################################################################
c
c     solvers for Susie (solv.f)
c
c#######################################################################
c
c     version 1.0 -- 07/12/15
c
c#######################################################################
c
      MODULE solv_m
c
      USE glob_m
      USE mesh_m
      USE matrix_m
c
      IMPLICIT NONE
      INCLUDE 'params.f'
      INCLUDE 'priv.f'
c
      CONTAINS
c
c#######################################################################
c     interface routine
c-----------------------------------------------------------------------
      SUBROUTINE solve(mat,rh,sl,guess)
c arguments
      TYPE(csr_t),INTENT(IN)::mat
      REAL(rn),DIMENSION(n_unk),INTENT(IN)::rh
      REAL(rn),DIMENSION(n_unk),INTENT(OUT)::sl
      REAL(rn),DIMENSION(n_unk),INTENT(IN),OPTIONAL::guess
c local variables
      TYPE(csr_t)::matl
      INTEGER::nrw,irw,i
      INTEGER,DIMENSION(:),ALLOCATABLE::un2rw,rw2un
      REAL(rn),DIMENSION(:),ALLOCATABLE::rhl,sll,gll
c begin -- squeeze out zero length rows
      nrw=SUM(MERGE(1,0,mat%ia(2:)/=mat%ia(:mat%nrw)))
      ALLOCATE(un2rw(n_unk),rw2un(nrw)); un2rw=0; rw2un=0; irw=0
      matl%nrw=nrw; matl%nel=mat%nel
      ALLOCATE(matl%ia(nrw+1),matl%ja(mat%nel),matl%a(mat%nel))
      matl%ia(1)=1
      DO i=1,mat%nrw
         IF(mat%ia(i+1)==mat%ia(i)) CYCLE
         irw=irw+1; matl%ia(irw+1)=mat%ia(i+1)
         un2rw(i)=irw; rw2un(irw)=i
      ENDDO
      matl%ja=un2rw(mat%ja); matl%a=mat%a
      ALLOCATE(rhl(nrw),sll(nrw)); rhl=rh(rw2un)
c don't change part of solution outside domain, if guess given
      IF(PRESENT(guess)) THEN
         ALLOCATE(gll(nrw)); gll=guess(rw2un)
         sll=gll
      ELSE
         sll=0
      ENDIF
c
      IF(csr==0) THEN
         CALL brew(matl,rhl,sll,guess)
      ELSE
         CALL agmg(matl,rhl,sll,guess)
      ENDIF
c
      DO i=1,nrw
         sl(rw2un(i))=sll(i)
      ENDDO
      DEALLOCATE(un2rw,rw2un,rhl,sll)
      IF(PRESENT(guess)) DEALLOCATE(gll)
      CALL clean(matl)
c terminate
      RETURN
      END SUBROUTINE solve
c
c#######################################################################
c     native CG with diagonal preconditioning
c-----------------------------------------------------------------------
      SUBROUTINE brew(mat,rh,sl,guess)
c arguments
      TYPE(csr_t),INTENT(IN)::mat
      REAL(rn),DIMENSION(:),INTENT(IN)::rh
      REAL(rn),DIMENSION(:),INTENT(OUT)::sl
      REAL(rn),DIMENSION(:),INTENT(IN),OPTIONAL::guess
c local variables
      INTEGER,PARAMETER::nrst=99999 ! disable restart for CG
      INTEGER::nn,l,it,irst
      REAL(rn)::rsmx,err,btd,btn,bet,al
      REAL(rn),DIMENSION(:),ALLOCATABLE::rs,rd,ad,p,dl
c begin -- allocate space and initialize iteration
      nn=mat%nrw; ALLOCATE(rs(nn),rd(nn),ad(nn),p(nn),dl(nn))
      rs=rh(:nn)
      IF(PRESENT(guess)) THEN
         sl(:nn)=guess(:nn); CALL matvec(mat,sl(:nn),dl); rs=rs-dl
      ELSE
         sl(:nn)=0
      ENDIF
      rd=0
      DO l=1,nn
         it=mat%ia(l); rd(l)=one/mat%a(it)
      ENDDO
      irst=nrst; rsmx=MAX(it_tol,SQRT(SUM(rs**2)/nn))
c iterate
      DO it=1,it_lim
         err=SQRT(SUM(rs**2)/nn)/rsmx
         IF(err<=it_tol) EXIT
c adjustment
         ad=-rs
c preconditioner
         ad=rd*ad
c start CG
         IF(irst>=nrst) THEN
            p=0; btd=1; irst=0
         ENDIF
         irst=irst+1
c dot products
         btn=SUM(rs*ad); bet=btn/btd; btd=btn
c update p
         p=ad+bet*p
c mat-vec
         CALL matvec(mat,p,dl)
c adjust
         al=btd/SUM(p*dl)
         sl(:nn)=sl(:nn)+al*p; rs=rs-al*dl
      ENDDO
      IF(it<=it_lim) RETURN
      WRITE(*,*) ' WARN: iteration limit in native CG'
      WRITE(*,*) 'err: ',err
c terminate
      RETURN
      END SUBROUTINE brew
c
c#######################################################################
c     agmg solver
c-----------------------------------------------------------------------
      SUBROUTINE agmg(mat,rh,sl,guess)
c arguments
      TYPE(csr_t),INTENT(IN)::mat
      REAL(rn),DIMENSION(:),INTENT(IN)::rh
      REAL(rn),DIMENSION(:),INTENT(OUT)::sl
      REAL(rn),DIMENSION(:),INTENT(IN),OPTIONAL::guess
c local variables
      INTEGER,PARAMETER::ijob=0,u_agmg=-6,nrst=1
      INTEGER::nn,ii,it
      REAL(rn)::ep
      INTEGER,DIMENSION(:),ALLOCATABLE::ia,ja
      REAL(rn),DIMENSION(:),ALLOCATABLE::a,b,dl
c begin -- buffer space
      nn=mat%nrw; ii=mat%nel
      ALLOCATE(ia(nn+1),ja(ii),a(ii),b(nn),dl(nn))
      ia=mat%ia; ja=mat%ja; a=mat%a; b=rh(:nn)
      IF(PRESENT(guess)) THEN
         sl(:nn)=guess(:nn); CALL matvec(mat,sl(:nn),dl); b=b-dl
      ELSE
         sl(:nn)=0
      ENDIF
c
      ep=it_tol; it=it_lim
      IF(SUM(b**2)>eps_mach) THEN
         CALL dagmg(nn,a,ja,ia,b,dl,ijob,u_agmg,nrst,it,ep)
         sl(:nn)=sl(:nn)+dl
      ELSE
         it=0
      ENDIF
      DEALLOCATE(ia,ja,a,b,dl)
c terminate
      RETURN
      END SUBROUTINE agmg
c
      END MODULE solv_m
