c#######################################################################
c        gnup.f                   -  routines to drive gnuplot graphics
c-----------------------------------------------------------------------
c     code organization.
c-----------------------------------------------------------------------
c      0. gnup_global
c      1. gnup_set.
c      2. fstrlen.
c      3. gnup_lineplot.
c      3.1 gnup_lineplotx.
c      4. gnup_mlineplot.
c      5. gnup_contplot.
c      6. gnup_contplotl.
c      7. gnup_zplot.
c      8. gnup_rzplot.
c      9. gnup_pointplot.
c     10. gnup_loopplot.
c     11. gnup_wrap.
c     12. gnup_stop.
c#######################################################################
c     module 0. gnup_global.
c     global storage for gnup routines
c-----------------------------------------------------------------------
      module gnup_global
         include 'params.f'
c
c     graphics file names
         integer,parameter::max_fam=100
         integer::nu_fam,i_fam=1,n_fam=0
         integer::char_fam,file_n
         character*80::gnup_fam
         character*16::time
         type::fam
            integer::char_fam,file_n,nu_fam
            character*80::gnup_fam
         endtype fam
c
         type(fam),dimension(max_fam)::gnp
c
      endmodule gnup_global
c
c#######################################################################
c     subprogram 1. gnup_set.
c     initialize file handling, etc. for gnuplot graphics
c-----------------------------------------------------------------------
      subroutine gnup_set(fam_name,t)
c
c modules
      use gnup_global
c
c arguments
      character*(*)::fam_name
      real(rn)::t
c
c local variables
      character*10::ta
      integer::char,i,nu
      integer,parameter::nuoff=60
c
c externals
      integer::fstrlen
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     save time string
c-----------------------------------------------------------------------
      write(ta,'(1pe10.3)') t
      time='  t = '//ta
c-----------------------------------------------------------------------
c     find last non-blank character from fam_name
c-----------------------------------------------------------------------
      char=fstrlen(fam_name)
c-----------------------------------------------------------------------
c     check for name already in table
c-----------------------------------------------------------------------
      do i=1,n_fam+1
         if(i.gt.n_fam)  exit
         if(char.ne.gnp(i)%char_fam)  cycle
         if(fam_name(1:char).eq.gnp(i)%gnup_fam(1:char))  exit
      enddo
c-----------------------------------------------------------------------
c     store incremented file number
c-----------------------------------------------------------------------
      gnp(i_fam)%file_n=file_n
      if(i.gt.n_fam)  then
c-----------------------------------------------------------------------
c     new file -- save family name in common, initialize file counter
c-----------------------------------------------------------------------
         n_fam=n_fam+1
         if(n_fam.gt.max_fam)
     .        call gnup_stop('stop',' ** max_fam exceeded **')
         char_fam=char
         file_n=0
         gnup_fam=fam_name(1:char)
         gnp(n_fam)%char_fam=char_fam
         gnp(n_fam)%file_n=file_n
         gnp(n_fam)%gnup_fam(1:char)=gnup_fam(1:char)
         nu=n_fam+nuoff
         nu_fam=nu
         gnp(n_fam)%nu_fam=nu_fam
         open(unit=nu,file=gnup_fam(1:char_fam)//'.plt')
         write(nu,'(a)') 'reset'
         CLOSE(nu)
      else
c-----------------------------------------------------------------------
c     old file -- set output stream
c-----------------------------------------------------------------------
         char_fam=gnp(i)%char_fam
         gnup_fam=gnp(i)%gnup_fam
         file_n=gnp(i)%file_n
         nu_fam=gnp(i)%nu_fam
         if(t==zero) then
            file_n=0
            nu=gnp(i)%nu_fam
            OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt',STATUS='OLD')
            rewind(nu)
            CLOSE(nu)
         endif
      endif
      i_fam=i
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      return
      end
c
c#######################################################################
c     subprogram 2. fstrlen.
c     find string length, excluding trailing blanks
c-----------------------------------------------------------------------
      function fstrlen(string)
c
c type function
      integer fstrlen
c
c arguments
      character*(*)::string
c
c local variables
      integer::len_string,i,nlast
      character*1,parameter::blank=char(32)
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
      len_string=len(string)
      nlast=0
      do i=1,len_string
         if(string(i:i).ne.blank)  nlast=i
      enddo
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      fstrlen=nlast
      return
      end
c
c#######################################################################
c     subprogram 3. gnup_lineplot.
c     line plot using gnuplot graphics -- one curve
c-----------------------------------------------------------------------
      subroutine gnup_lineplot(x,y,points,xmode,ymode,xtit,ytit,gtit)
c
c modules
      use gnup_global
c
c arguments
      integer::points
      real(rn),dimension(points)::x,y
      character*(*)::xmode,ymode,xtit,ytit,gtit
c
c local variables
      character*5::n_a
      character*1::j_a
      integer::i,levx,levy,nu
      integer,parameter::lev_max=99
      real(rn)::xscale,yscale
      real(rn),allocatable,dimension(:)::xp,yp
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     open data file in sequence
c-----------------------------------------------------------------------
      file_n=file_n+1
      write(n_a,'(i5)') 10000+file_n
      open(unit=51,file=gnup_fam(1:char_fam)//n_a(2:5)//'.dat')
c-----------------------------------------------------------------------
c     write data to file - find min/max
c-----------------------------------------------------------------------
      allocate(xp(points),yp(points))
      xp=x
      yp=y
      xscale=0
      yscale=0
      do i=1,points
         xscale=max(xscale,abs(xp(i)))
         yscale=max(yscale,abs(yp(i)))
      enddo
      if(xscale.eq.0)  xscale=1
      if(yscale.eq.0)  yscale=1
      levx=0
      if(xscale.gt.0)  then
         if(xscale.lt.1.e-3)  then
            do while(xscale.lt.1)
               xscale=10*xscale
               xp=10*xp
               levx=levx-1
               if(levx.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levx underflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         elseif(xscale.gt.1.e3)  then
            do while(xscale.gt.1)
               xscale=xscale/10
               xp=xp/10
               levx=levx+1
               if(levx.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levx overflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         endif
      endif
      levy=0
      if(yscale.gt.0)  then
         if(yscale.lt.1.e-3)  then
            do while(yscale.lt.1)
               yscale=10*yscale
               yp=10*yp
               levy=levy-1
               if(levy.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levy underflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         elseif(yscale.gt.1.e3)  then
            do while(yscale.gt.1)
               yscale=yscale/10
               yp=yp/10
               levy=levy+1
               if(levy.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levy overflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         endif
      endif
      WHERE(ABS(yp)<=1.d-6) yp=0
      do i=1,points
         write(51,'(1p9e13.5)') xp(i),yp(i)
      enddo
      close(51)
c-----------------------------------------------------------------------
c     write commands to file
c-----------------------------------------------------------------------
      nu=nu_fam
      OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .    ,STATUS='OLD',ACCESS='APPEND')
      write(nu,'(3a)')
     .     'set label 1 "',xtit,'" at screen 0.5,0.03 center'
      write(nu,'(4a)')
     .     'set label 2 "',ytit,'" at screen 0.03,0.5 center rotate'
      write(nu,'(a)') 'set nokey'
      if(gtit.eq.'')  then
         write(nu,'(3a)')
     .        'set label 3 "',gnup_fam(1:char_fam)//time
     .        ,'" at screen 0.5,0.99 center'
      else
         write(nu,'(3a)')
     .        'set label 3 "',gtit//time
     .        ,'" at screen 0.5,0.99 center'
      endif
      if(xmode.eq.'log')  write(nu,'(a)') 'set logscale x'
      if(levx.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 4 "E',levx,'" at screen 0.75,0.02'
      if(ymode.eq.'log')  write(nu,'(a)') 'set logscale y'
      if(levy.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 5 "E',levy,'" at screen 0.04,0.75 rotate'
cc      write(nu,'(a)') 'set data style lines'
      write(nu,'(a)') 'set style data lines'
      write(nu,'(3a)') 'plot "',gnup_fam(1:char_fam)//n_a(2:5)
     .        ,'.dat" using 1:2 \'
      write(nu,'(a)') ' '
      write(nu,'(a)') 'pause -1'
      write(nu,'(a)') 'reset'
      CLOSE(nu)
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
 1    continue
      deallocate(xp,yp)
      return
      end
c
c#######################################################################
c     subprogram 3.1 gnup_lineplotx.
c     line plot using gnuplot graphics -- one curve
c     fixed y scale
c-----------------------------------------------------------------------
      subroutine gnup_lineplotx(x,y,points,xmode,ymode,xtit,ytit,gtit
     .                         ,ymin,ymax)
c
c modules
      use gnup_global
c
c arguments
      integer::points
      real(rn),dimension(points)::x,y
      character*(*)::xmode,ymode,xtit,ytit,gtit
      real(rn)::ymin,ymax
c
c local variables
      character*5::n_a
      character*1::j_a
      integer::i,levx,levy,nu
      integer,parameter::lev_max=99
      real(rn)::xscale,yscale
      real(rn),allocatable,dimension(:)::xp,yp
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     open data file in sequence
c-----------------------------------------------------------------------
      file_n=file_n+1
      write(n_a,'(i5)') 10000+file_n
      open(unit=51,file=gnup_fam(1:char_fam)//n_a(2:5)//'.dat')
c-----------------------------------------------------------------------
c     write data to file - find min/max
c-----------------------------------------------------------------------
      allocate(xp(points),yp(points))
      xp=x
      yp=y
      xscale=0
      yscale=0
      do i=1,points
         xscale=max(xscale,abs(xp(i)))
         yscale=max(yscale,abs(yp(i)))
      enddo
      if(xscale.eq.0)  xscale=1
      if(yscale.eq.0)  yscale=1
      levx=0
      if(xscale.gt.0)  then
         if(xscale.lt.1.e-3)  then
            do while(xscale.lt.1)
               xscale=10*xscale
               xp=10*xp
               levx=levx-1
               if(levx.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levx underflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         elseif(xscale.gt.1.e3)  then
            do while(xscale.gt.1)
               xscale=xscale/10
               xp=xp/10
               levx=levx+1
               if(levx.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levx overflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         endif
      endif
      levy=0
      if(yscale.gt.0)  then
         if(yscale.lt.1.e-3)  then
            do while(yscale.lt.1)
               yscale=10*yscale
               yp=10*yp
               levy=levy-1
               if(levy.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levy underflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         elseif(yscale.gt.1.e3)  then
            do while(yscale.gt.1)
               yscale=yscale/10
               yp=yp/10
               levy=levy+1
               if(levy.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levy overflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         endif
      endif
      WHERE(ABS(yp)<=1.d-6) yp=0
      do i=1,points
         write(51,'(1p9e13.5)') xp(i),yp(i)
      enddo
      close(51)
c-----------------------------------------------------------------------
c     write commands to file
c-----------------------------------------------------------------------
      nu=nu_fam
      OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .    ,STATUS='OLD',ACCESS='APPEND')
      write(nu,'(3a)')
     .     'set label 1 "',xtit,'" at screen 0.5,0.03 center'
      write(nu,'(4a)')
     .     'set label 2 "',ytit,'" at screen 0.03,0.5 center rotate'
      write(nu,'(a)') 'set nokey'
      if(gtit.eq.'')  then
         write(nu,'(3a)')
     .        'set label 3 "',gnup_fam(1:char_fam)//time
     .        ,'" at screen 0.5,0.99 center'
      else
         write(nu,'(3a)')
     .        'set label 3 "',gtit//time
     .        ,'" at screen 0.5,0.99 center'
      endif
      if(xmode.eq.'log')  write(nu,'(a)') 'set logscale x'
      if(levx.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 4 "E',levx,'" at screen 0.75,0.02'
      if(ymode.eq.'log')  write(nu,'(a)') 'set logscale y'
      if(levy.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 5 "E',levy,'" at screen 0.04,0.75 rotate'
cc      write(nu,'(a)') 'set data style lines'
      write(nu,'(a)') 'set style data lines'
      write(nu,'(a,1pe10.3,a,e10.3,3a)') 
     .     'plot [ ] [',ymin,':',ymax,'] "'
     .     ,gnup_fam(1:char_fam)//n_a(2:5)
     .        ,'.dat" using 1:2 \'
      write(nu,'(a)') ' '
      write(nu,'(a)') 'pause -1'
      write(nu,'(a)') 'reset'
      CLOSE(nu)
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
 1    continue
      deallocate(xp,yp)
      return
      end
c
c#######################################################################
c     subprogram 4. gnup_mlineplot.
c     line plot using gnuplot graphics
c-----------------------------------------------------------------------
      subroutine gnup_mlineplot(x,y,points,curves,dim,xmode,ymode
     .                        ,xtit,ytit,gtit)
c
c modules
      use gnup_global
c
c arguments
      integer::points,curves,dim
      real(rn),dimension(points)::x
      real(rn),dimension(dim,curves)::y
      character*(*)::xmode,ymode,xtit,ytit,gtit
c
c local variables
      character*5::n_a
      character*2::j_a
      integer::curvs,i,j,levx,levy,nu
      integer,parameter::lev_max=99
      real(rn)::xscale,yscale
      real(rn),allocatable,dimension(:)::xp
      real(rn),allocatable,dimension(:,:)::yp
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     open data file in sequence
c-----------------------------------------------------------------------
      file_n=file_n+1
      write(n_a,'(i5)') 10000+file_n
      open(unit=51,file=gnup_fam(1:char_fam)//n_a(2:5)//'.dat')
c-----------------------------------------------------------------------
c     write data to file - find min/max
c-----------------------------------------------------------------------
      allocate(xp(points),yp(dim,curves))
      xp=x
      yp=y
      xscale=0
      yscale=0
      curvs=curves
      do i=1,points
         xscale=max(xscale,abs(xp(i)))
         do j=1,curvs
            yscale=max(yscale,abs(yp(i,j)))
         enddo
      enddo
      if(xscale.eq.0)  xscale=1
      if(yscale.eq.0)  yscale=1
      levx=0
      if(xscale.gt.0)  then
         if(xscale.lt.1.e-3)  then
            do while(xscale.lt.1)
               xscale=10*xscale
               xp=10*xp
               levx=levx-1
               if(levx.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levx underflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         elseif(xscale.gt.1.e3)  then
            do while(xscale.gt.1)
               xscale=xscale/10
               xp=xp/10
               levx=levx+1
               if(levx.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levx overflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         endif
      endif
      levy=0
      if(yscale.gt.0)  then
         if(yscale.lt.1.e-3)  then
            do while(yscale.lt.1)
               yscale=10*yscale
               yp=10*yp
               levy=levy-1
               if(levy.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levy underflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         elseif(yscale.gt.1.e3)  then
            do while(yscale.gt.1)
               yscale=yscale/10
               yp=yp/10
               levy=levy+1
               if(levy.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levy overflow in ** gnup_lineplot **')
                  goto 1
               endif
            enddo
         endif
      endif
      write(51,*)
      WHERE(ABS(yp)<=1.d-6) yp=0
      do i=1,points
         write(51,'(1p222222e13.5)')
     .        xp(i),(yp(i,j),j=1,curvs)
      enddo
      close(51)
c-----------------------------------------------------------------------
c     write commands to file
c-----------------------------------------------------------------------
      nu=nu_fam
      OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .    ,STATUS='OLD',ACCESS='APPEND')
      write(nu,'(3a)')
     .     'set label 1 "',xtit,'" at screen 0.5,0.03 center'
      write(nu,'(4a)')
     .     'set label 2 "',ytit,'" at screen 0.03,0.5 center rotate'
      write(nu,'(a)') 'set nokey'
      if(gtit.eq.'')  then
         write(nu,'(3a)')
     .        'set label 3 "',gnup_fam(1:char_fam)//time
     .        ,'" at screen 0.5,0.99 center'
      else
         write(nu,'(3a)')
     .        'set label 3 "',gtit//time
     .        ,'" at screen 0.5,0.99 center'
      endif
      if(xmode.eq.'log')  write(nu,'(a)') 'set logscale x'
      if(levx.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 4 "E',levx,'" at screen 0.75,0.02'
      if(ymode.eq.'log')  write(nu,'(a)') 'set logscale y'
      if(levy.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 5 "E',levy,'" at screen 0.04,0.75 rotate'
cc      write(nu,'(a)') 'set data style lines'
      write(nu,'(a)') 'set style data lines'
      write(nu,'(3a)') 'plot "',gnup_fam(1:char_fam)//n_a(2:5)
     .        ,'.dat" using 1:2 \'
      do j=2,curvs
         write(j_a,'(i2)') j+1
         write(nu,'(5a)') ', "',gnup_fam(1:char_fam)//n_a(2:5)
     .        ,'.dat" using 1:',j_a,' \'
      enddo
      write(nu,'(a)') ' '
      write(nu,'(a)') 'pause -1'
      write(nu,'(a)') 'reset'
      CLOSE(nu)
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
 1    continue
      deallocate(xp,yp)
      return
      end
c
c#######################################################################
c     subprogram 5. gnup_contplot.
c     contour plot using gnuplot graphics
c-----------------------------------------------------------------------
      subroutine gnup_contplot(x,y,f,xpoints,ypoints,conts
     .                        ,xtit,ytit,gtit)
c
c modules
      use gnup_global
c
c arguments
      integer::xpoints,ypoints,conts
      real(rn),dimension(xpoints)::x
      real(rn),dimension(ypoints)::y
      real(rn),dimension(xpoints,ypoints)::f
      character*(*)::xtit,ytit,gtit
c
c local variables
      integer::i,j,ch,levx,levy,nu
      integer,parameter::lev_max=99
      real(rn)::fmn,fmx,df,xscale,yscale
      real(rn),dimension(xpoints)::xp
      real(rn),dimension(ypoints)::yp
      character*5::n_a
      character*28::lev
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     open data file in sequence
c-----------------------------------------------------------------------
      file_n=file_n+1
      write(n_a,'(i5)') 10000+file_n
      open(unit=51,file=gnup_fam(1:char_fam)//n_a(2:5)//'.dat')
c-----------------------------------------------------------------------
c     write data to file - find min/max, or use optional limits
c-----------------------------------------------------------------------
      xp=x
      yp=y
      xscale=max(abs(x(1)),abs(x(xpoints)))
      if(xscale.eq.0)  xscale=1
      yscale=max(abs(y(1)),abs(y(ypoints)))
      if(yscale.eq.0)  yscale=1
      levx=0
      if(xscale.gt.0)  then
         if(xscale.lt.1.e-3)  then
            do while(xscale.lt.1)
               xscale=10*xscale
               xp=10*xp
               levx=levx-1
               if(levx.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levx underflow in ** gnup_contplot **')
                  return
               endif
            enddo
         elseif(xscale.gt.1.e3)  then
            do while(xscale.gt.1)
               xscale=xscale/10
               xp=xp/10
               levx=levx+1
               if(levx.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levx overflow in ** gnup_contplot **')
                  return
               endif
            enddo
         endif
      endif
      levy=0
      if(yscale.gt.0)  then
         if(yscale.lt.1.e-3)  then
            do while(yscale.lt.1)
               yscale=10*yscale
               yp=10*yp
               levy=levy-1
               if(levy.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levy underflow in ** gnup_contplot **')
                  return
               endif
            enddo
         elseif(yscale.gt.1.e3)  then
            do while(yscale.gt.1)
               yscale=yscale/10
               yp=yp/10
               levy=levy+1
               if(levy.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levy overflow in ** gnup_contplot **')
                  return
               endif
            enddo
         endif
      endif
      fmn=f(1,1)
      fmx=fmn
      do j=1,ypoints
         do i=1,xpoints
            write(51,'(1p3e13.5)') xp(i),yp(j),f(i,j)
            fmn=min(fmn,f(i,j))
            fmx=max(fmx,f(i,j))
         enddo
         write(51,'(a)') ''
      enddo
      close(51)
c-----------------------------------------------------------------------
c     write commands to file
c-----------------------------------------------------------------------
      nu=nu_fam
      OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .    ,STATUS='OLD',ACCESS='APPEND')
      write(nu,'(3a)')
     .     'set label 1 "',xtit,'" at screen 0.5,0.03 center'
      write(nu,'(4a)')
     .     'set label 2 "',ytit,'" at screen 0.03,0.5 center rotate'
      write(nu,'(a)') 'set nokey'
      if(gtit.eq.'')  then
         write(nu,'(3a)')
     .        'set label 3 "',gnup_fam(1:char_fam)//time
     .        ,'" at screen 0.5,0.95 center'
      else
         write(nu,'(3a)')
     .        'set label 3 "',gtit//time
     .        ,'" at screen 0.5,0.95 center'
      endif
      write(nu,'(a,1pe10.3,a,e10.3,a)')
     .     'set label 4 "min= ',fmn,'   max= ',fmx
     .     ,'" at screen 0.5,0.85 center'
      if(levx.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 5 "E',levx,'" at screen 0.75,0.21'
      if(levy.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 6 "E',levy,'" at screen 0.85,0.5'
cc      write(nu,'(a)') 'set data style lines'
      write(nu,'(a)') 'set style data lines'
      write(nu,'(a)') 'set nosurface'
      write(nu,'(a)') 'set contour'
      write(nu,'(a)') 'set view 0,0'
      if(conts.gt.0)  then
         df=(fmx-fmn)/conts
         fmn=fmn+df/2
         fmx=fmx-df/2
         if(df.eq.0)  df=1
         if(fmn.lt.0)  then
            if(fmx.lt.0)  then
               write(lev,'(1pe9.2,a,1pe8.2,a,1pe9.2)')
     .              fmn,',',df,',',fmx
               ch=28
            else
               write(lev,'(1pe9.2,a,1pe8.2,a,1pe8.2)')
     .              fmn,',',df,',',fmx
               ch=27
            endif
         else
            if(fmx.lt.0)  then
               write(lev,'(1pe8.2,a,1pe8.2,a,1pe9.2)')
     .              fmn,',',df,',',fmx
               ch=27
            else
               write(lev,'(1pe8.2,a,1pe8.2,a,1pe8.2)')
     .              fmn,',',df,',',fmx
               ch=26
            endif
         endif
         write(nu,'(2a)') 'set cntrparam levels incremental ',lev(1:ch)
      else
         write(nu,'(a)') 'set cntrparam levels discrete 0.'
      endif
      write(nu,'(3a)')
     .     'splot "',gnup_fam(1:char_fam)//n_a(2:5),'.dat" '
      write(nu,'(a)') 'pause -1'
      write(nu,'(a)') 'reset'
      CLOSE(nu)
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      return
      end
c
c#######################################################################
c     subprogram 6. gnup_contplotl.
c     contour plot using gnuplot graphics
c-----------------------------------------------------------------------
      subroutine gnup_contplotl(x,y,f,xpoints,ypoints,conts
     .                         ,xtit,ytit,gtit,fmn,fmx)
c
c modules
      use gnup_global
c
c arguments
      integer::xpoints,ypoints,conts
      real(rn),dimension(xpoints)::x
      real(rn),dimension(ypoints)::y
      real(rn),dimension(xpoints,ypoints)::f
      character*(*)::xtit,ytit,gtit
      real(rn)::fmn,fmx
c
c local variables
      integer::i,j,ch,levx,levy,nu
      integer,parameter::lev_max=99
      real(rn)::df,xscale,yscale
      real(rn),dimension(xpoints)::xp
      real(rn),dimension(ypoints)::yp
      character*5::n_a
      character*29::lev
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     open data file in sequence
c-----------------------------------------------------------------------
      file_n=file_n+1
      write(n_a,'(i5)') 10000+file_n
      open(unit=51,file=gnup_fam(1:char_fam)//n_a(2:5)//'.dat')
c-----------------------------------------------------------------------
c     write data to file - find min/max, or use optional limits
c-----------------------------------------------------------------------
      xp=x
      yp=y
      xscale=max(abs(x(1)),abs(x(xpoints)))
      if(xscale.eq.0)  xscale=1
      yscale=max(abs(y(1)),abs(y(ypoints)))
      if(yscale.eq.0)  yscale=1
      levx=0
      if(xscale.gt.0)  then
         if(xscale.lt.1.e-3)  then
            do while(xscale.lt.1)
               xscale=10*xscale
               xp=10*xp
               levx=levx-1
               if(levx.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levx underflow in ** gnup_contplot **')
                  return
               endif
            enddo
         elseif(xscale.gt.1.e3)  then
            do while(xscale.gt.1)
               xscale=xscale/10
               xp=xp/10
               levx=levx+1
               if(levx.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levx overflow in ** gnup_contplot **')
                  return
               endif
            enddo
         endif
      endif
      levy=0
      if(yscale.gt.0)  then
         if(yscale.lt.1.e-3)  then
            do while(yscale.lt.1)
               yscale=10*yscale
               yp=10*yp
               levy=levy-1
               if(levy.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levy underflow in ** gnup_contplot **')
                  return
               endif
            enddo
         elseif(yscale.gt.1.e3)  then
            do while(yscale.gt.1)
               yscale=yscale/10
               yp=yp/10
               levy=levy+1
               if(levy.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levy overflow in ** gnup_contplot **')
                  return
               endif
            enddo
         endif
      endif
      if(fmn>=fmx) then
         fmn=minval(f)
         fmx=maxval(f)
         df=(fmx-fmn)/MAX(1,conts)
         fmn=fmn+df/2
         fmx=fmx-df/2
      endif
      do j=1,ypoints
         do i=1,xpoints
            write(51,'(1p3e13.5)') xp(i),yp(j),f(i,j)
         enddo
         write(51,'(a)') ''
      enddo
      close(51)
c-----------------------------------------------------------------------
c     write commands to file
c-----------------------------------------------------------------------
      nu=nu_fam
      OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .    ,STATUS='OLD',ACCESS='APPEND')
      write(nu,'(3a)')
     .     'set label 1 "',xtit,'" at screen 0.5,0.03 center'
      write(nu,'(4a)')
     .     'set label 2 "',ytit,'" at screen 0.03,0.5 center rotate'
      write(nu,'(a)') 'set nokey'
      if(gtit.eq.'')  then
         write(nu,'(3a)')
     .        'set label 3 "',gnup_fam(1:char_fam)//time
     .        ,'" at screen 0.5,0.95 center'
      else
         write(nu,'(3a)')
     .        'set label 3 "',gtit//time
     .        ,'" at screen 0.5,0.95 center'
      endif
      write(nu,'(a,1pe10.3,a,e10.3,a)')
     .     'set label 4 "min= ',fmn,'   max= ',fmx
     .     ,'" at screen 0.5,0.85 center'
      if(levx.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 5 "E',levx,'" at screen 0.75,0.21'
      if(levy.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 6 "E',levy,'" at screen 0.85,0.5'
cc      write(nu,'(a)') 'set data style lines'
      write(nu,'(a)') 'set style data lines'
      write(nu,'(a)') 'set nosurface'
      write(nu,'(a)') 'set contour'
      write(nu,'(a)') 'set view 0,0'
      if(conts.gt.1)  then
         df=(fmx-fmn)/(conts-1)
         if(df.eq.0)  df=1
         if(fmn.lt.0)  then
            if(fmx.lt.0)  then
               write(lev,'(1pe9.2,a,1pe9.3,a,1pe9.2)')
     .              fmn,',',df,',',fmx
               ch=29
            else
               write(lev,'(1pe9.2,a,1pe9.3,a,1pe8.2)')
     .              fmn,',',df,',',fmx
               ch=28
            endif
         else
            if(fmx.lt.0)  then
               write(lev,'(1pe8.2,a,1pe9.3,a,1pe9.2)')
     .              fmn,',',df,',',fmx
               ch=28
            else
               write(lev,'(1pe8.2,a,1pe9.3,a,1pe8.2)')
     .              fmn,',',df,',',fmx
               ch=27
            endif
         endif
         write(nu,'(2a)') 'set cntrparam levels incremental ',lev(1:ch)
      else
         write(nu,'(a)') 'set cntrparam levels discrete 0.'
      endif
      write(nu,'(3a)')
     .     'splot "',gnup_fam(1:char_fam)//n_a(2:5),'.dat" '
      write(nu,'(a)') 'pause -1'
      write(nu,'(a)') 'reset'
      CLOSE(nu)
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      return
      end
c
c#######################################################################
c     subprogram 7. gnup_zplot.
c     plot zero coutours of real and imag parts of complex field
c-----------------------------------------------------------------------
      subroutine gnup_zplot(x,y,zf,xpoints,ypoints
     .                     ,xtit,ytit,gtit)
c
c modules
      use gnup_global
c
c arguments
      integer::xpoints,ypoints
      real(rn),dimension(xpoints)::x
      real(rn),dimension(ypoints)::y
      complex(rn),dimension(xpoints,ypoints)::zf
      character*(*)::xtit,ytit,gtit
c
c local variables
      integer::i,j,ch,levx,levy,nu
      integer,parameter::lev_max=99
      real(rn)::fmn,fmx,df,xscale,yscale
      real(rn),dimension(xpoints)::xp
      real(rn),dimension(ypoints)::yp
      character*5::n_a
      character*28::lev
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     open data file in sequence
c-----------------------------------------------------------------------
      file_n=file_n+1
      write(n_a,'(i5)') 10000+file_n
      open(unit=51,file=gnup_fam(1:char_fam)//n_a(2:5)//'.dat')
c-----------------------------------------------------------------------
c     write data to file - find min/max
c-----------------------------------------------------------------------
      xp=x
      yp=y
      xscale=max(abs(x(1)),abs(x(xpoints)))
      if(xscale.eq.0)  xscale=1
      yscale=max(abs(y(1)),abs(y(ypoints)))
      if(yscale.eq.0)  yscale=1
      levx=0
      if(xscale.gt.0)  then
         if(xscale.lt.1.e-3)  then
            do while(xscale.lt.1)
               xscale=10*xscale
               xp=10*xp
               levx=levx-1
               if(levx.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levx underflow in ** gnup_contplot **')
                  return
               endif
            enddo
         elseif(xscale.gt.1.e3)  then
            do while(xscale.gt.1)
               xscale=xscale/10
               xp=xp/10
               levx=levx+1
               if(levx.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levx overflow in ** gnup_contplot **')
                  return
               endif
            enddo
         endif
      endif
      levy=0
      if(yscale.gt.0)  then
         if(yscale.lt.1.e-3)  then
            do while(yscale.lt.1)
               yscale=10*yscale
               yp=10*yp
               levy=levy-1
               if(levy.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levy underflow in ** gnup_contplot **')
                  return
               endif
            enddo
         elseif(yscale.gt.1.e3)  then
            do while(yscale.gt.1)
               yscale=yscale/10
               yp=yp/10
               levy=levy+1
               if(levy.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levy overflow in ** gnup_contplot **')
                  return
               endif
            enddo
         endif
      endif
c
      do j=1,ypoints
         do i=1,xpoints
            write(51,'(1p4e13.5)')
     .           xp(i),yp(j),real(zf(i,j)),aimag(zf(i,j))
         enddo
         write(51,'(a)') ''
      enddo
      close(51)
c-----------------------------------------------------------------------
c     write commands to file
c-----------------------------------------------------------------------
      nu=nu_fam
      OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .    ,STATUS='OLD',ACCESS='APPEND')
      write(nu,'(3a)')
     .     'set label 1 "',xtit,'" at screen 0.5,0.03 center'
      write(nu,'(4a)')
     .     'set label 2 "',ytit,'" at screen 0.03,0.5 center rotate'
      write(nu,'(a)') 'set nokey'
      if(gtit.eq.'')  then
         write(nu,'(3a)')
     .        'set label 3 "',gnup_fam(1:char_fam)//time
     .        ,'" at screen 0.5,0.95 center'
      else
         write(nu,'(3a)')
     .        'set label 3 "',gtit//time
     .        ,'" at screen 0.5,0.95 center'
      endif
      if(levx.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 5 "E',levx,'" at screen 0.75,0.21'
      if(levy.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 6 "E',levy,'" at screen 0.85,0.5'
cc      write(nu,'(a)') 'set data style lines'
      write(nu,'(a)') 'set style data lines'
      write(nu,'(a)') 'set nosurface'
      write(nu,'(a)') 'set contour'
      write(nu,'(a)') 'set view 0,0'
      write(nu,'(a)') 'set cntrparam levels discrete 0.'
      write(nu,'(3a)')
     .     'splot "',gnup_fam(1:char_fam)//n_a(2:5)
     .     ,'.dat" using 1:2:3 \'
      write(nu,'(3a)')
     .     '     ,"',gnup_fam(1:char_fam)//n_a(2:5),'.dat" using 1:2:4'
      write(nu,'(a)') 'pause -1'
      write(nu,'(a)') 'reset'
      CLOSE(nu)
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      return
      end
c
c#######################################################################
c     subprogram 8. gnup_rzplot.
c     plot zero coutours of real field
c-----------------------------------------------------------------------
      subroutine gnup_rzplot(x,y,zf,xpoints,ypoints
     .                     ,xtit,ytit,gtit)
c
c modules
      use gnup_global
c
c arguments
      integer::xpoints,ypoints
      real(rn),dimension(xpoints)::x
      real(rn),dimension(ypoints)::y
      real(rn),dimension(xpoints,ypoints)::zf
      character*(*)::xtit,ytit,gtit
c
c local variables
      integer::i,j,ch,levx,levy,nu
      integer,parameter::lev_max=99
      real(rn)::fmn,fmx,df,xscale,yscale
      real(rn),dimension(xpoints)::xp
      real(rn),dimension(ypoints)::yp
      character*5::n_a
      character*28::lev
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     open data file in sequence
c-----------------------------------------------------------------------
      file_n=file_n+1
      write(n_a,'(i5)') 10000+file_n
      open(unit=51,file=gnup_fam(1:char_fam)//n_a(2:5)//'.dat')
c-----------------------------------------------------------------------
c     write data to file - find min/max
c-----------------------------------------------------------------------
      xp=x
      yp=y
      xscale=max(abs(x(1)),abs(x(xpoints)))
      if(xscale.eq.0)  xscale=1
      yscale=max(abs(y(1)),abs(y(ypoints)))
      if(yscale.eq.0)  yscale=1
      levx=0
      if(xscale.gt.0)  then
         if(xscale.lt.1.e-3)  then
            do while(xscale.lt.1)
               xscale=10*xscale
               xp=10*xp
               levx=levx-1
               if(levx.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levx underflow in ** gnup_contplot **')
                  return
               endif
            enddo
         elseif(xscale.gt.1.e3)  then
            do while(xscale.gt.1)
               xscale=xscale/10
               xp=xp/10
               levx=levx+1
               if(levx.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levx overflow in ** gnup_contplot **')
                  return
               endif
            enddo
         endif
      endif
      levy=0
      if(yscale.gt.0)  then
         if(yscale.lt.1.e-3)  then
            do while(yscale.lt.1)
               yscale=10*yscale
               yp=10*yp
               levy=levy-1
               if(levy.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levy underflow in ** gnup_contplot **')
                  return
               endif
            enddo
         elseif(yscale.gt.1.e3)  then
            do while(yscale.gt.1)
               yscale=yscale/10
               yp=yp/10
               levy=levy+1
               if(levy.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levy overflow in ** gnup_contplot **')
                  return
               endif
            enddo
         endif
      endif
c
      do j=1,ypoints
         do i=1,xpoints
            write(51,'(1p4e13.5)')
     .           xp(i),yp(j),zf(i,j)
         enddo
         write(51,'(a)') ''
      enddo
      close(51)
c-----------------------------------------------------------------------
c     write commands to file
c-----------------------------------------------------------------------
      nu=nu_fam
      OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .    ,STATUS='OLD',ACCESS='APPEND')
      write(nu,'(3a)')
     .     'set label 1 "',xtit,'" at screen 0.5,0.03 center'
      write(nu,'(4a)')
     .     'set label 2 "',ytit,'" at screen 0.03,0.5 center rotate'
      write(nu,'(a)') 'set nokey'
      if(gtit.eq.'')  then
         write(nu,'(3a)')
     .        'set label 3 "',gnup_fam(1:char_fam)//time
     .        ,'" at screen 0.5,0.95 center'
      else
         write(nu,'(3a)')
     .        'set label 3 "',gtit//time
     .        ,'" at screen 0.5,0.95 center'
      endif
      if(levx.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 5 "E',levx,'" at screen 0.75,0.21'
      if(levy.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 6 "E',levy,'" at screen 0.85,0.5'
cc      write(nu,'(a)') 'set data style lines'
      write(nu,'(a)') 'set style data lines'
      write(nu,'(a)') 'set nosurface'
      write(nu,'(a)') 'set contour'
      write(nu,'(a)') 'set view 0,0'
      write(nu,'(a)') 'set cntrparam levels discrete 0.'
      write(nu,'(3a)')
     .     'splot "',gnup_fam(1:char_fam)//n_a(2:5)
     .     ,'.dat" using 1:2:3'
      write(nu,'(a)') 'pause -1'
      write(nu,'(a)') 'reset'
      CLOSE(nu)
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      return
      end
c
c#######################################################################
c     subprogram 9. gnup_pointplot.
c     point plot using gnuplot graphics
c-----------------------------------------------------------------------
      subroutine gnup_pointplot(x,y,points,x0,x1,y0,y1,xtit,ytit,gtit)
c
c modules
      use gnup_global
c
c arguments
      integer::points
      real(rn),dimension(points)::x,y
      real(rn)::x0,x1,y0,y1
      character*(*)::xtit,ytit,gtit
c
c local variables
      integer::i,levx,levy,nu
      integer,parameter::lev_max=99
      real(rn)::xscale,yscale,xmn,xmx,ymn,ymx
      real(rn),dimension(points)::xp,yp
      character*5::n_a
      character*28::lev
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     open data file in sequence
c-----------------------------------------------------------------------
      file_n=file_n+1
      write(n_a,'(i5)') 10000+file_n
      open(unit=51,file=gnup_fam(1:char_fam)//n_a(2:5)//'.dat')
c-----------------------------------------------------------------------
c     write data to file - find min/max
c-----------------------------------------------------------------------
      xp=x
      xmn=x0
      xmx=x1
      yp=y
      ymn=y0
      ymx=y1
      xscale=abs(x(1))
      yscale=abs(y(1))
      do i=2,points
         xscale=max(abs(x(1)),abs(x(i)))
         yscale=max(abs(y(1)),abs(y(i)))
      enddo
      if(xscale.eq.0)  xscale=1
      if(yscale.eq.0)  yscale=1
      levx=0
      if(xscale.gt.0)  then
         if(xscale.lt.1.e-3)  then
            do while(xscale.lt.1)
               xscale=10*xscale
               xp=10*xp
               xmn=10*xmn
               xmx=10*xmx
               levx=levx-1
               if(levx.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levx underflow in ** gnup_pointplot **')
                  return
               endif
            enddo
         elseif(xscale.gt.1.e3)  then
            do while(xscale.gt.1)
               xscale=xscale/10
               xp=xp/10
               xmn=xmn/10
               xmx=xmx/10
               levx=levx+1
               if(levx.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levx overflow in ** gnup_pointplot **')
                  return
               endif
            enddo
         endif
      endif
      levy=0
      if(yscale.gt.0)  then
         if(yscale.lt.1.e-3)  then
            do while(yscale.lt.1)
               yscale=10*yscale
               yp=10*yp
               ymn=10*ymn
               ymx=10*ymx
               levy=levy-1
               if(levy.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levy underflow in ** gnup_pointplot **')
                  return
               endif
            enddo
         elseif(yscale.gt.1.e3)  then
            do while(yscale.gt.1)
               yscale=yscale/10
               yp=yp/10
               ymn=ymn/10
               ymx=ymx/10
               levy=levy+1
               if(levy.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levy overflow in ** gnup_pointplot **')
                  return
               endif
            enddo
         endif
      endif
      do i=1,points
         write(51,'(1p3e13.5)') xp(i),yp(i)
      enddo
      close(51)
c-----------------------------------------------------------------------
c     write commands to file
c-----------------------------------------------------------------------
      nu=nu_fam
      OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .    ,STATUS='OLD',ACCESS='APPEND')
      write(nu,'(3a)')
     .     'set label 1 "',xtit,'" at screen 0.5,0.03 center'
      write(nu,'(4a)')
     .     'set label 2 "',ytit,'" at screen 0.03,0.5 center rotate'
      write(nu,'(a)') 'set nokey'
      if(gtit.eq.'')  then
         write(nu,'(3a)')
     .        'set label 3 "',gnup_fam(1:char_fam)//time
     .        ,'" at screen 0.5,0.99 center'
      else
         write(nu,'(3a)')
     .        'set label 3 "',gtit//time
     .        ,'" at screen 0.5,0.99 center'
      endif
      if(levx.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 5 "E',levx,'" at screen 0.75,0.21'
      if(levy.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 6 "E',levy,'" at screen 0.85,0.5'
      if(xmn.lt.xmx)  write(nu,'(a,1pe13.5,a,e13.5,a)')
     .     'set xrange [',xmn,':',xmx,']'
      if(ymn.lt.ymx)  write(nu,'(a,1pe13.5,a,e13.5,a)')
     .     'set yrange [',ymn,':',ymx,']'
      write(nu,'(a)') 'set pointsize 0.5'
cc      write(nu,'(a)') 'set data style points'
      write(nu,'(a)') 'set style data points'
      write(nu,'(a)') 'set size 0.7296,1.'
      write(nu,'(3a)') 'plot "',gnup_fam(1:char_fam)//n_a(2:5),'.dat" '
      write(nu,'(a)') 'pause -1'
      write(nu,'(a)') 'reset'
      CLOSE(nu)
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      return
      end
c
c#######################################################################
c     subprogram 10. gnup_loopplot.
c     point plot using gnuplot graphics
c-----------------------------------------------------------------------
      subroutine gnup_loopplot(x,y,points,loops,x0,x1,y0,y1
     .                        ,xtit,ytit,gtit)
c
c modules
      use gnup_global
c
c arguments
      integer::points,loops
      real(rn),dimension(points,loops)::x,y
      real(rn)::x0,x1,y0,y1
      character*(*)::xtit,ytit,gtit
c
c local variables
      integer::i,j,levx,levy,nu
      integer,parameter::lev_max=99
      real(rn)::xscale,yscale,xmn,xmx,ymn,ymx
      real(rn),dimension(points,loops)::xp,yp
      character*5::n_a
      character*28::lev
      character*4::j_x,j_y
c-----------------------------------------------------------------------
c     begin program
c-----------------------------------------------------------------------
c     open data file in sequence
c-----------------------------------------------------------------------
      file_n=file_n+1
      write(n_a,'(i5)') 10000+file_n
      open(unit=51,file=gnup_fam(1:char_fam)//n_a(2:5)//'.dat')
c-----------------------------------------------------------------------
c     write data to file - find min/max
c-----------------------------------------------------------------------
      xp=x
      xmn=x0
      xmx=x1
      yp=y
      ymn=y0
      ymx=y1
      xscale=abs(x(1,1))
      yscale=abs(y(1,1))
      do j=1,loops
         do i=1,points
            xscale=max(xscale,abs(x(i,j)))
            yscale=max(yscale,abs(y(i,j)))
         enddo
      enddo
      if(xscale.eq.0)  xscale=1
      if(yscale.eq.0)  yscale=1
      levx=0
      if(xscale.gt.0)  then
         if(xscale.lt.1.e-3)  then
            do while(xscale.lt.1)
               xscale=10*xscale
               xp=10*xp
               xmn=10*xmn
               xmx=10*xmx
               levx=levx-1
               if(levx.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levx underflow in ** gnup_loopplot **')
                  return
               endif
            enddo
         elseif(xscale.gt.1.e3)  then
            do while(xscale.gt.1)
               xscale=xscale/10
               xp=xp/10
               xmn=xmn/10
               xmx=xmx/10
               levx=levx+1
               if(levx.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levx overflow in ** gnup_loopplot **')
                  return
               endif
            enddo
         endif
      endif
      levy=0
      if(yscale.gt.0)  then
         if(yscale.lt.1.e-3)  then
            do while(yscale.lt.1)
               yscale=10*yscale
               yp=10*yp
               ymn=10*ymn
               ymx=10*ymx
               levy=levy-1
               if(levy.le.-lev_max)  then
                  call gnup_stop('go',
     .                 ' levy underflow in ** gnup_loopplot **')
                  return
               endif
            enddo
         elseif(yscale.gt.1.e3)  then
            do while(yscale.gt.1)
               yscale=yscale/10
               yp=yp/10
               ymn=ymn/10
               ymx=ymx/10
               levy=levy+1
               if(levy.ge. lev_max)  then
                  call gnup_stop('go',
     .                 ' levy overflow in ** gnup_loopplot **')
                  return
               endif
            enddo
         endif
      endif
      do i=1,points
         do j=1,loops
            write(51,'(1p2e13.5)',advance='NO') xp(i,j),yp(i,j)
         enddo
         write(51,'(a)') ''
      enddo
      close(51)
c-----------------------------------------------------------------------
c     write commands to file
c-----------------------------------------------------------------------
      nu=nu_fam
      OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .    ,STATUS='OLD',ACCESS='APPEND')
      write(nu,'(3a)')
     .     'set label 1 "',xtit,'" at screen 0.5,0.03 center'
      write(nu,'(4a)')
     .     'set label 2 "',ytit,'" at screen 0.03,0.5 center rotate'
      write(nu,'(a)') 'set nokey'
      if(gtit.eq.'')  then
         write(nu,'(3a)')
     .        'set label 3 "',gnup_fam(1:char_fam)//time
     .        ,'" at screen 0.5,0.99 center'
      else
         write(nu,'(3a)')
     .        'set label 3 "',gtit//time
     .        ,'" at screen 0.5,0.99 center'
      endif
      if(levx.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 5 "E',levx,'" at screen 0.75,0.21'
      if(levy.ne.0)  write(nu,'(a,i3,a)')
     .     'set label 6 "E',levy,'" at screen 0.04,0.75 rotate'
      if(xmn.lt.xmx)  write(nu,'(a,1pe13.5,a,e13.5,a)')
     .     'set xrange [',xmn,':',xmx,']'
      if(ymn.lt.ymx)  write(nu,'(a,1pe13.5,a,e13.5,a)')
     .     'set yrange [',ymn,':',ymx,']'
cc      write(nu,'(a)') 'set data style lines'
      write(nu,'(a)') 'set style data lines'
      write(nu,'(3a)') 'plot "',gnup_fam(1:char_fam)//n_a(2:5)
     .        ,'.dat" using 1:2 \'
      do j=2,loops
         write(j_x,'(i4)') 2*j-1
         write(j_y,'(i4)') 2*j
         write(nu,'(7a)') ', "',gnup_fam(1:char_fam)//n_a(2:5)
     .        ,'.dat" using ',j_x,':',j_y,' \'
      enddo
      write(nu,'(a)') ' '
      write(nu,'(a)') 'pause -1'
      write(nu,'(a)') 'reset'
      CLOSE(nu)
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      return
      end
c
c#######################################################################
c     subprogram 11. gnup_wrap.
c     wrap up gnuplot graphics
c-----------------------------------------------------------------------
      subroutine gnup_wrap
c
c modules
      use gnup_global
c
c local variables
      integer::i,nu
c-----------------------------------------------------------------------
c     begin program - write final commands to file
c     close command file
c-----------------------------------------------------------------------
      do i=1,n_fam
         nu=gnp(i)%nu_fam
         OPEN(UNIT=nu,FILE=gnup_fam(1:char_fam)//'.plt'
     .       ,STATUS='OLD',ACCESS='APPEND')
         write(nu,'(a)') 'clear'
         close(nu)
      enddo
      n_fam=0
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      return
      end
c
c#######################################################################
c     subprogram 12. gnup_stop.
c     error stop routine for gnup routines
c-----------------------------------------------------------------------
      subroutine gnup_stop(mode,errmsg)
c
c modules
      use gnup_global
c
c arguements
      character*(*)::mode,errmsg
c-----------------------------------------------------------------------
c     begin program - write out error message
c     stop
c-----------------------------------------------------------------------
      write(*,'(2a)') ' ERROR IN GNUP : ',errmsg
      write(*,'(2a)') ' WRITING TO : ',gnup_fam(1:char_fam)
      if(mode.eq.'go')  return
      write(*,'(a)') 'STOPPING'
      call gnup_wrap
      stop
c-----------------------------------------------------------------------
c     terminate routine
c-----------------------------------------------------------------------
      return
      end
