c#######################################################################
c
c     NIMbnd code suite
c     Copyright by Coronado Consulting
c     2017
c
c     Programmed by D. C. Barnes
c     Coronado Consulting
c
c#######################################################################
c
c     boundary solution matrix (bmat.f)
c
c#######################################################################
c
c     version 1.0 -- 09/17/16
c
c     ddx now operates on B_n instead of phi
c     03/23/17
c
c#######################################################################
c
      MODULE bmat_m
c
      USE serv_m
      USE bint_m
      USE green_m
      USE seg_m
c
      IMPLICIT NONE
      INCLUDE 'params.f'
      INCLUDE 'priv.f'
c
      CONTAINS
c
c#######################################################################
c     form and return solution matrix for exterior B field problem
c-----------------------------------------------------------------------
      SUBROUTINE get_bmat(r,z,smat,ddx,dd)
c arguments
      REAL(rn),DIMENSION(:),INTENT(IN)::r,z
      REAL(rn),DIMENSION(:,:,:),INTENT(OUT)::smat,ddx
      REAL(rn),DIMENSION(:),INTENT(OUT)::dd
c local variables
      INTEGER::n_bn,n_seg,is,n2,i,j
      REAL(rn),DIMENSION(:),ALLOCATABLE::rp,zp,dp
      REAL(rn),DIMENSION(:,:),ALLOCATABLE::ll,mm,aa,li0,li1,ai
     .                                    ,lx,lr,ld,ddph
      REAL(rn)::drs,dzs
      LOGICAL::cyl
c begin -- find out if boundary is cyclic
      n_bn=SIZE(r); n_seg=n_bn-1
      IF(SIZE(z)/= n_bn) CALL bomb('bad data to get_bmat')
      IF(SIZE(smat,1)/=2) CALL bomb('bad data to get_bmat')
      IF(SIZE(smat,2)/=n_seg) CALL bomb('bad data to get_bmat')
      IF(SIZE(smat,3)/=n_seg) CALL bomb('bad data to get_bmat')
      IF(SIZE(dd)/=n_seg) CALL bomb('bad data to get_bmat')
      cyl=r(n_bn)==r(1).AND.z(n_bn)==z(1)
c make segment tables
      ALLOCATE(seg_r0(n_seg),seg_z0(n_seg),seg_r1(n_seg),seg_z1(n_seg))
      seg_r0=r(1:n_seg); seg_r1=r(2:n_bn)
      seg_z0=z(1:n_seg); seg_z1=z(2:n_bn)
c generate matrices
      ALLOCATE(ll(n_seg,n_seg),mm(n_seg,n_seg),aa(n_seg,n_seg)
     .        ,li0(n_seg,n_seg),li1(n_seg,n_seg),ai(n_seg,n_seg))
c diagonal matrix
      DO is=1,n_seg
         drs=r(is+1)-r(is); dzs=z(is+1)-z(is)
         dd(is)=(r(is+1)+r(is))/2*SQRT(drs**2+dzs**2)
      ENDDO
c n=0
      CALL sym_mat(ge0,ll); CALL asm_mat(ke0,mm); CALL inv_spd(-ll,li0)
c Plemlj
      DO is=1,n_seg
         mm(is,is)=mm(is,is)+0.5/dd(is)
      ENDDO
      aa=MATMUL(mm,li0)
      DO is=1,n_seg
         aa(is,:)=dd(is)*aa(is,:)
      ENDDO
      aa=(aa+TRANSPOSE(aa))/2; CALL inv_spd(aa,ai)
      DO is=1,n_seg
         smat(1,:,is)=-ai(:,is)*dd(is)
      ENDDO
c n=1
      CALL sym_mat(ge1,ll); CALL asm_mat(ke1,mm); CALL inv_spd(-ll,li1)
c Plemlj
      DO is=1,n_seg
         mm(is,is)=mm(is,is)+0.5/dd(is)
      ENDDO
      aa=MATMUL(mm,li1)
      DO is=1,n_seg
         aa(is,:)=dd(is)*aa(is,:)
      ENDDO
      aa=(aa+TRANSPOSE(aa))/2; CALL inv_spd(aa,ai)
      DO is=1,n_seg
         smat(2,:,is)=-ai(:,is)*dd(is)
      ENDDO
c tangential derivative via finite differences
c     construct the shifted L matrix
c divide segments by 2
      n2=2*n_seg
      ALLOCATE(rp(n2+1),zp(n2+1)); rp=0; zp=0
      rp(1:n2+1:2)=r; zp(1:n2+1:2)=z
      rp(2:n2  :2)=(r(:n_seg)+r(2:))/2
      zp(2:n2  :2)=(z(:n_seg)+z(2:))/2
c segment area
      ALLOCATE(dp(n2))
      DO is=1,n2
         drs=rp(is+1)-rp(is); dzs=zp(is+1)-zp(is)
         dp(is)=(rp(is+1)+rp(is))/2*SQRT(drs**2+dzs**2)
      ENDDO
c remake segment tables
      DEALLOCATE(seg_r0,seg_z0,seg_r1,seg_z1)
      ALLOCATE(lx(n2,n2),seg_r0(n2),seg_z0(n2),seg_r1(n2),seg_z1(n2))
      seg_r0=rp(:n2); seg_r1=rp(2:); seg_z0=zp(:n2); seg_z1=zp(2:)
c n=0
      CALL sym_mat(ge0,lx)
c compress
      ALLOCATE(lr(n2,n_seg))
      DO j=1,n_seg
         lr(:,j)=(dp(2*j-1)*lx(:,2*j-1)+dp(2*j)*lx(:,2*j))/dd(j)
      ENDDO
      DEALLOCATE(ll); ALLOCATE(ll(n_bn,n_seg))
      ll(   1,:)=(3*lr( 1,:)-lr(   2,:))/2
      DO i=2,n_seg
         ll(i,:)=(lr(2*i-2,:)+lr(2*i-1,:))/2
      ENDDO
      ll(n_bn,:)=(3*lr(n2,:)-lr(n2-1,:))/2
      DEALLOCATE(lr); ALLOCATE(ld(n_seg,n_seg))
      DO i=1,n_seg
         ld(i,:)=(ll(i+1,:)-ll(i,:))/
     .        SQRT((r(i+1)-r(i))**2+(z(i+1)-z(i))**2)
      ENDDO
      DEALLOCATE(ll); ALLOCATE(ddph(n_seg,n_seg)); ddph=MATMUL(ld,-li0)
      ddx(1,:,:)=MATMUL(ddph,smat(1,:,:))
c n=1
      CALL sym_mat(ge1,lx)
c compress
      DEALLOCATE(ld); ALLOCATE(lr(n2,n_seg))
      DO j=1,n_seg
         lr(:,j)=(dp(2*j-1)*lx(:,2*j-1)+dp(2*j)*lx(:,2*j))/dd(j)
      ENDDO
      DEALLOCATE(lx); ALLOCATE(ll(n_bn,n_seg))
      ll(   1,:)=(3*lr( 1,:)-lr(   2,:))/2
      DO i=2,n_seg
         ll(i,:)=(lr(2*i-2,:)+lr(2*i-1,:))/2
      ENDDO
      ll(n_bn,:)=(3*lr(n2,:)-lr(n2-1,:))/2
      DEALLOCATE(lr); ALLOCATE(ld(n_seg,n_seg))
      DO i=1,n_seg
         ld(i,:)=(ll(i+1,:)-ll(i,:))/
     .        SQRT((r(i+1)-r(i))**2+(z(i+1)-z(i))**2)
      ENDDO
      DEALLOCATE(ll); ddph=MATMUL(ld,-li1)
      ddx(2,:,:)=MATMUL(ddph,smat(2,:,:)); DEALLOCATE(ddph)
c terminate
      DEALLOCATE(mm,aa,li0,li1,ai,ld,dp)
      DEALLOCATE(seg_r0,seg_z0,seg_r1,seg_z1)
      RETURN
      END SUBROUTINE get_bmat
c
c#######################################################################
c     invert full spd matrix using row/column CG
c-----------------------------------------------------------------------
      RECURSIVE SUBROUTINE inv_spd(mm,mi)
c arguments
      REAL(rn),DIMENSION(:,:),INTENT(IN)::mm
      REAL(rn),DIMENSION(:,:),INTENT(OUT)::mi
c local variables
      INTEGER,PARAMETER::small_n=400
      INTEGER::nn,i,nh,nhp,ih
      REAL(rn),DIMENSION(:,:),ALLOCATABLE::d1i,d1b,d2i,d2b
c begin
      nn=SIZE(mm,1)
      IF(SIZE(mm,2)/=nn.OR.SIZE(mi,1)/=nn.OR.SIZE(mi,2)/=nn)
     .     CALL bomb('size inconsistency in **inv_spd**')
c bisect matrix
      IF(nn>small_n) THEN
         nh=nn/2; nhp=nn-nh
c$OMP PARALLEL DO
         DO ih=1,2
            SELECT CASE(ih)
            CASE (1)
               ALLOCATE(d2i(nhp,nhp),d1b(nh,nh))
               CALL inv_spd(mm(nh+1:,nh+1:),d2i)
               d1b=mm(:nh,:nh)-matmul(mm(:nh,nh+1:)
     .                               ,matmul(d2i,mm(nh+1:,:nh)))
               CALL inv_spd(d1b,mi(:nh,:nh)); DEALLOCATE(d2i,d1b)
            CASE (2)
               ALLOCATE(d1i(nh,nh),d2b(nhp,nhp))
               CALL inv_spd(mm(:nh,:nh),d1i)
               d2b=mm(nh+1:,nh+1:)-matmul(mm(nh+1:,:nh)
     .                                   ,matmul(d1i,mm(:nh,nh+1:)))
               CALL inv_spd(d2b,mi(nh+1:,nh+1:)); DEALLOCATE(d2b)
            END SELECT
         ENDDO
c$OMP END PARALLEL DO
         mi(:nh,nh+1:)=-matmul(d1i
     .                        ,matmul(mm(:nh,nh+1:),mi(nh+1:,nh+1:)))
         DEALLOCATE(d1i)
         mi(nh+1:,:nh)=TRANSPOSE(mi(:nh,nh+1:))
      ELSE
         DO i=nn,1,-1
            CALL inv_rc(mm,i,mi,nn)
         ENDDO
      ENDIF
c terminate
      RETURN
      END SUBROUTINE inv_spd
c
c#######################################################################
c     find inverse of last row/column in symmetic ll_wk matrix
c-----------------------------------------------------------------------
      SUBROUTINE inv_rc(mm,i,mm_i,nn)
c arguments
      INTEGER,INTENT(IN)::nn,i
      REAL(rn),DIMENSION(nn,nn),INTENT(IN)::mm
      REAL(rn),DIMENSION(nn,nn),INTENT(INOUT)::mm_i
c local variables
      INTEGER::j,l,it
      REAL(rn)::rsmx
      REAL(rn),dimension(i)::ib,res,rd,ad,p,dl
      INTEGER,PARAMETER::itmax=9999,nrst=99999
      REAL(rn),PARAMETER::tol=1.e-8
      INTEGER::irst
      REAL(rn)::err,btd,btn,bet,al
c begin -- residual for CG to give unit vector solution
      res=0; res(i)=1
      DO j=i+1,nn
         res=res-mm(:i,j)*mm_i(i,j)
      ENDDO
      ib=0; rsmx=MAX(tol,MAXVAL(ABS(res)))
c iteration
      DO l=1,i
         rd(l)=1/mm(l,l)
      ENDDO
      irst=nrst
c
      DO it=1,itmax
         err=SQRT(SUM(res**2)/i)/rsmx
         IF(err<=tol)  EXIT
c CG adjustment
         ad=-res
c preconditioner
         ad=rd*ad
c start CG
         IF(irst>=nrst)  THEN
            p=0; btd=1; irst=0
         ENDIF
         irst=irst+1
c dot products
         btn=SUM(res*ad); bet=btn/btd; btd=btn
c update p
         p=ad+bet*p
c mat-vec
         dl=MATMUL(mm(:i,:i),p)
c adjust solution and residual
         al=btd/SUM(p*dl)
         ib=ib+al*p; res=res-al*dl
      ENDDO
      IF(it>=itmax) CALL bomb('trouble inverting symm matrix')
      it=MIN(it,itmax)
c save solution
      mm_i(i,:i)=ib; mm_i(:i,i)=ib
c terminate
      RETURN
      END SUBROUTINE inv_rc
c
      END MODULE bmat_m
