c#######################################################################
c
c     NIMbnd code suite
c     Copyright by Coronado Consulting
c     2017
c
c     Programmed by D. C. Barnes
c     Coronado Consulting
c
c#######################################################################
c
c     Green's functions for magnetostatic boundary integrals (green.f)
c
c#######################################################################
c
c     version 2.0 -- 02/09/17
c     implemented Fukushima san's complete elliptic integrals
c
c#######################################################################
c
      MODULE green_m
c
      USE serv_m
c
      IMPLICIT NONE
      INCLUDE 'params.f'
      INCLUDE 'priv.f'
c
      TYPE::coef_t
         INTEGER::n
         REAL(rn),DIMENSION(:),POINTER::c
      END TYPE coef_t
c
      TYPE(coef_t),DIMENSION(:),POINTER::bn=>null(),dn
      TYPE(coef_t)::bx,dx,b0,d0
c
      REAL(rn),DIMENSION(0:11),PARAMETER::b01=(/0.790401414e0
     .                                         ,0.102006266e0
     .                                         ,0.039878396e0
     .                                         ,0.021737136e0
     .                                         ,0.01396098e0
     .                                         ,0.009892519e0
     .                                         ,0.007484612e0
     .                                         ,0.005934626e0
     .                                         ,0.004874249e0
     .                                         ,0.004114607e0
     .                                         ,0.003550453e0
     .                                         ,0.00311923e0/)
      REAL(rn),DIMENSION(0:11),PARAMETER::d01=(/0.80060204e0
     .                                         ,0.313994478e0
     .                                         ,0.205913119e0
     .                                         ,0.157744347e0
     .                                         ,0.130595077e0
     .                                         ,0.113308474e0
     .                                         ,0.101454199e0
     .                                         ,0.092918784e0
     .                                         ,0.08656538e0
     .                                         ,0.081727985e0
     .                                         ,0.077990666e0
     .                                         ,0.075080427e0/)
      REAL(rn),DIMENSION(0:11),PARAMETER::b12=(/0.801024064e0
     .                                         ,0.110695345e0
     .                                         ,0.047348747e0
     .                                         ,0.028484367e0
     .                                         ,0.020277811e0
     .                                         ,0.015965006e0
     .                                         ,0.01344132e0
     .                                         ,0.011871566e0
     .                                         ,0.01086364e0
     .                                         ,0.010231587e0
     .                                         ,0.009849586e0
     .                                         ,0.009656606e0/)
      REAL(rn),DIMENSION(0:11),PARAMETER::d12=(/0.834232668e0
     .                                         ,0.360495282e0
     .                                         ,0.262379664e0
     .                                         ,0.223723945e0
     .                                         ,0.206447812e0
     .                                         ,0.199809441e0
     .                                         ,0.199667452e0
     .                                         ,0.204157559e0
     .                                         ,0.212387468e0
     .                                         ,0.223948914e0
     .                                         ,0.238708097e0
     .                                         ,0.256707204e0/)
      REAL(rn),DIMENSION(0:12),PARAMETER::b23=(/0.812597773e0
     .                                         ,0.121109618e0
     .                                         ,0.057293377e0
     .                                         ,0.038509452e0
     .                                         ,0.03078343e0
     .                                         ,0.027290565e0
     .                                         ,0.025916369e0
     .                                         ,0.025847203e0
     .                                         ,0.026740924e0
     .                                         ,0.028464315e0
     .                                         ,0.030995446e0
     .                                         ,0.034384369e0
     .                                         ,0.038738002e0/)
      REAL(rn),DIMENSION(0:12),PARAMETER::d23=(/0.873152582e0
     .                                         ,0.420622231e0
     .                                         ,0.344231062e0
     .                                         ,0.331133022e0
     .                                         ,0.345277285e0
     .                                         ,0.377945322e0
     .                                         ,0.427378012e0
     .                                         ,0.494671744e0
     .                                         ,0.582685116e0
     .                                         ,0.695799208e0
     .                                         ,0.840018401e0
     .                                         ,1.023268504e0
     .                                         ,1.255859085e0/)
      REAL(rn),DIMENSION(0:13),PARAMETER::b34=(/0.825323558e0
     .                                         ,0.133862116e0
     .                                         ,0.071011294e0
     .                                         ,0.054178477e0
     .                                         ,0.049451745e0
     .                                         ,0.050222196e0
     .                                         ,0.054742913e0
     .                                         ,0.062746258e0
     .                                         ,0.074669881e0
     .                                         ,0.091480845e0
     .                                         ,0.114705092e0
     .                                         ,0.146571133e0
     .                                         ,0.190257137e0
     .                                         ,10.02004323e0/)
      REAL(rn),DIMENSION(0:13),PARAMETER::d34=(/0.919027039e0
     .                                         ,0.501002159e0
     .                                         ,0.468831271e0
     .                                         ,0.517714228e0
     .                                         ,0.620843391e0
     .                                         ,0.782364394e0
     .                                         ,1.019114535e0
     .                                         ,1.359345203e0
     .                                         ,1.845717302e0
     .                                         ,2.541071703e0
     .                                         ,3.537404655e0
     .                                         ,4.969296003e0
     .                                         ,7.03382287e0
     .                                         ,zero/)
      REAL(rn),DIMENSION(0:15),PARAMETER::b45=(/0.83947957e0
     .                                         ,0.14991644e0
     .                                         ,0.090831936e0
     .                                         ,0.080347033e0
     .                                         ,0.085638441e0
     .                                         ,0.101954726e0
     .                                         ,0.130574812e0
     .                                         ,0.176105076e0
     .                                         ,0.246835164e0
     .                                         ,0.356424477e0
     .                                         ,0.527002562e0
     .                                         ,0.794389634e0
     .                                         ,1.216762532e0
     .                                         ,88.38485474e0
     .                                         ,149.5663745e0
     .                                         ,254.3179084e0/)
      REAL(rn),DIMENSION(0:15),PARAMETER::d45=(/0.974404367e0
     .                                         ,0.613246805e0
     .                                         ,0.67109667e0
     .                                         ,0.87072762e0
     .                                         ,1.229542231e0
     .                                         ,1.826605968e0
     .                                         ,2.806934531e0
     .                                         ,4.418789329e0
     .                                         ,7.083236057e0
     .                                         ,11.51508812e0
     .                                         ,18.93151119e0
     .                                         ,31.41199694e0
     .                                         ,52.52072945e0
     .                                         ,zero,zero,zero/)
      REAL(rn),DIMENSION(0:16),PARAMETER::b56=(/0.855469615e0
     .                                         ,0.170896073e0
     .                                         ,0.121335229e0
     .                                         ,0.128201884e0
     .                                         ,0.164687281e0
     .                                         ,0.237418909e0
     .                                         ,0.369208105e0
     .                                         ,0.605658734e0
     .                                         ,1.033705562e0
     .                                         ,1.818988489e0
     .                                         ,3.279377651e0
     .                                         ,6.029888381e0
     .                                         ,11.26979686e0
     .                                         ,21.35457785e0
     .                                         ,2495.588727e0
     .                                         ,5184.692429e0
     .                                         ,10817.21334e0/)
      REAL(rn),DIMENSION(0:16),PARAMETER::d56=(/1.043455295e0
     .                                         ,0.779625722e0
     .                                         ,1.029742361e0
     .                                         ,1.622037223e0
     .                                         ,2.787989531e0
     .                                         ,5.048381487e0
     .                                         ,9.463277612e0
     .                                         ,18.18148995e0
     .                                         ,35.58098059e0
     .                                         ,70.63393546e0
     .                                         ,141.8285801e0
     .                                         ,287.4487513e0
     .                                         ,587.1153846e0
     .                                         ,1207.065435e0
     .                                         ,zero,zero,zero/)
      REAL(rn),DIMENSION(0:17),PARAMETER::b67=(/0.873920062e0
     .                                         ,0.199814057e0
     .                                         ,0.172769616e0
     .                                         ,0.228106913e0
     .                                         ,0.370468141e0
     .                                         ,0.679271253e0
     .                                         ,1.348008497e0
     .                                         ,2.827670977e0
     .                                         ,6.17946825e0
     .                                         ,13.93568601e0
     .                                         ,32.21892928e0
     .                                         ,76.00696296e0
     .                                         ,182.3214491e0
     .                                         ,443.5150764e0
     .                                         ,1091.854723e0
     .                                         ,2715.765866e0
     .                                         ,605941.5173e0
     .                                         ,1631082.6e0/)
      REAL(rn),DIMENSION(0:17),PARAMETER::d67=(/1.133678337e0
     .                                         ,1.048643174e0
     .                                         ,1.753465041e0
     .                                         ,3.523182727e0
     .                                         ,7.749476414e0
     .                                         ,17.98645006e0
     .                                         ,43.25591635e0
     .                                         ,106.6815345e0
     .                                         ,268.0984866e0
     .                                         ,683.6241149e0
     .                                         ,1763.497085e0
     .                                         ,4592.374754e0
     .                                         ,12053.44102e0
     .                                         ,31846.66302e0
     .                                         ,84621.22136e0
     .                                         ,225956.4232e0
     .                                         ,zero,zero/)
      REAL(rn),DIMENSION(0:20),PARAMETER::b78=(/0.895902821e0
     .                                         ,0.243140004e0
     .                                         ,0.273081876e0
     .                                         ,0.486280008e0
     .                                         ,1.082747437e0
     .                                         ,2.743445291e0
     .                                         ,7.555817829e0
     .                                         ,22.05194082e0
     .                                         ,67.15640645e0
     .                                         ,211.2722538e0
     .                                         ,681.9037843e0
     .                                         ,2246.956232e0
     .                                         ,7531.483866e0
     .                                         ,25608.5126e0
     .                                         ,88140.7474e0
     .                                         ,306564.4242e0
     .                                         ,1076036.078e0
     .                                         ,3807218.503e0
     .                                         ,13566382.24e0
     .                                         ,7160267534e0
     .                                         ,27223307947e0/)
      REAL(rn),DIMENSION(0:20),PARAMETER::d78=(/1.260612827e0
     .                                         ,1.548665638e0
     .                                         ,3.553669412e0
     .                                         ,9.900444676e0
     .                                         ,30.32056662e0
     .                                         ,98.18025866e0
     .                                         ,329.7710104e0
     .                                         ,1136.65599e0
     .                                         ,3993.834336e0
     .                                         ,14242.72959e0
     .                                         ,51394.75729e0
     .                                         ,187246.7029e0
     .                                         ,687653.0924e0
     .                                         ,2542385.536e0
     .                                         ,9453781.219e0
     .                                         ,35328363.02e0
     .                                         ,132593262.4e0
     .                                         ,499544968.2e0
     .                                         ,1888409347e0
     .                                         ,zero,zero/)
      REAL(rn),DIMENSION(0:17),PARAMETER::b85=(/0.915922053e0
     .                                         ,0.294714252e0
     .                                         ,0.435776709e0
     .                                         ,1.067328246e0
     .                                         ,3.327844119e0
     .                                         ,11.90406004e0
     .                                         ,46.4783882e0
     .                                         ,192.7556003e0
     .                                         ,835.3356299e0
     .                                         ,3743.124548e0
     .                                         ,17219.07731e0
     .                                         ,80904.60402e0
     .                                         ,386808.3293e0
     .                                         ,1876487.67e0
     .                                         ,9216559.909e0
     .                                         ,7469935793e0
     .                                         ,40041555958e0
     .                                         ,2.15463e+11/)
      REAL(rn),DIMENSION(0:17),PARAMETER::d85=(/1.402200569e0
     .                                         ,2.322205898e0
     .                                         ,7.462158366e0
     .                                         ,29.43506891e0
     .                                         ,128.1590924e0
     .                                         ,591.0807037e0
     .                                         ,2830.54623e0
     .                                         ,13917.76432e0
     .                                         ,69786.10525e0
     .                                         ,355234.142e0
     .                                         ,1830019.186e0
     .                                         ,9519610.812e0
     .                                         ,49920868.76e0
     .                                         ,263567701e0
     .                                         ,1399645765e0
     .                                         ,zero,zero,zero/)
      REAL(rn),DIMENSION(0:20),PARAMETER::b59=(/0.931906061e0
     .                                         ,0.34844803e0
     .                                         ,0.666809179e0
     .                                         ,2.210769136e0
     .                                         ,9.491765049e0
     .                                         ,47.09304791e0
     .                                         ,255.920046e0
     .                                         ,1480.029533e0
     .                                         ,8954.040905e0
     .                                         ,56052.48221e0
     .                                         ,360395.7242e0
     .                                         ,2367539.415e0
     .                                         ,15829949.57e0
     .                                         ,107415809.3e0
     .                                         ,738058546.e0
     .                                         ,5126022003.e0
     .                                         ,35935340655.e0
     .                                         ,2.53988e+11
     .                                         ,1.80818e+12
     .                                         ,3.77302e+15
     .                                         ,2.86826e+16/)
      REAL(rn),DIMENSION(0:20),PARAMETER::d59=(/1.541690113e0
     .                                         ,3.379176215e0
     .                                         ,14.94058386e0
     .                                         ,81.91773929e0
     .                                         ,497.4900547e0
     .                                         ,3205.18401e0
     .                                         ,21457.32237e0
     .                                         ,147557.0157e0
     .                                         ,1035045.29e0
     .                                         ,7371922.335e0
     .                                         ,53143443.95e0
     .                                         ,386882347.6e0
     .                                         ,2839458402.e0
     .                                         ,20982661229.e0
     .                                         ,1.55962e+11
     .                                         ,1.1651e+12
     .                                         ,8.74201e+12
     .                                         ,6.58473e+13
     .                                         ,4.9768e+14
     .                                         ,zero,zero/)
      REAL(rn),DIMENSION(0:13),PARAMETER::bsx=(/zero
     .                                         ,-.25e0
     .                                         ,-3.125000000000000e-02
     .                                         ,-1.171875000000000e-02
     .                                         ,-6.103515625000000e-03
     .                                         ,-3.738403320312500e-03
     .                                         ,-2.523422241210940e-03
     .                                         ,-1.817464828491210e-03
     .                                         ,-1.371212303638460e-03
     .                                         ,-1.071259612217550e-03
     .                                         ,-8.599834109190850e-04
     .                                         ,-7.055772985040680e-04
     .                                         ,-5.893174027278290e-04
     .                                         ,-4.995976058381760e-04/)

      REAL(rn),DIMENSION(0:13),PARAMETER::bs0=(/one
     .                                        ,-0.25e0
     .                                        ,4.68750000000000000e-02
     .                                        ,2.34375000000000000e-02
     .                                        ,1.35294596354167000e-02
     .                                        ,8.74074300130208000e-03
     .                                        ,6.09626770019531000e-03
     .                                        ,4.48911190032959000e-03
     .                                        ,3.44149707151311000e-03
     .                                        ,2.72140251971515000e-03
     .                                        ,2.20547845166284000e-03
     .                                        ,1.82333628910408000e-03
     .                                        ,1.53246703696644000e-03
     .                                        ,1.30598157535640000e-03/)
      REAL(rn),DIMENSION(0:12),PARAMETER::dsx=(/.5e0
     .                                        ,-1.2500000000000000e-01
     .                                        ,-2.3437500000000000e-02
     .                                        ,-9.7656250000000000e-03
     .                                        ,-5.3405761718750000e-03
     .                                        ,-3.3645629882812500e-03
     .                                        ,-2.3131370544433600e-03
     .                                        ,-1.6876459121704100e-03
     .                                        ,-1.2855115346610500e-03
     .                                        ,-1.0117451893165700e-03
     .                                        ,-8.1698424037313100e-04
     .                                        ,-6.7350560311751900e-04
     .                                        ,-5.6476251094750300e-04/)
      REAL(rn),DIMENSION(0:12),PARAMETER::ds0=(/-one
     .                                        ,zero
     .                                        ,3.9062500000000000e-02
     .                                        ,2.0182291666666700e-02
     .                                        ,1.2029012044270800e-02
     .                                        ,7.9414367675781300e-03
     .                                        ,5.6232929229736300e-03
     .                                        ,4.1870066097804500e-03
     .                                        ,3.2371161006657100e-03
     .                                        ,2.5768262044977500e-03
     .                                        ,2.0995044461342900e-03
     .                                        ,1.7433729755435800e-03
     .                                        ,1.4706604847412000e-03/)
c
      REAL(rn),DIMENSION(:),POINTER::md
c legacy
      INTEGER,PARAMETER::rsym=0,zsym=0
      CHARACTER(LEN=2),PARAMETER::geom='rz'
c
      CONTAINS
c
c#######################################################################
c     free space l=0 Green's function for ES problem
c-----------------------------------------------------------------------
      FUNCTION ge0(aa,n)
c type function
      REAL(rn)::ge0
c arguments
      INTEGER::n
      REAL(rn),DIMENSION(n)::aa
c local variables
      INTEGER::isym,jsym
      REAL(rn)::r1,z1,r2,z2,d,sgi,sgj
      REAL(rn)::rp,rsm,dlz,rr2,rr,k
      REAL(rn),PARAMETER::rmin=1.e-10
c begin -- check number of arguments
      IF(n/=7) THEN
         CALL bomb(' Wrong number of arguments to **ge0**')
      ENDIF
      r1=aa(1); r2=aa(3)
      z1=aa(2); z2=aa(4)
      d=aa(5)
c
      ge0=0
      sgi=1
      DO isym=1,2
         IF(geom=='rz') THEN
            IF(tooclose(r1,z1,r2,z2,d)) THEN
               rp=MAX(r1,rmin)
               ge0=ge0+sgi/2/pi*(log(d/16/rp)-1)
            ELSE
               rsm=r1+r2
               dlz=z1-z2
               rr2=dlz*dlz+rsm**2; rr=SQRT(rr2)
               k=SQRT(4.*r1*r2)/rr
               ge0=ge0-sgi/pi/rr*kk(k)
            ENDIF
         ELSE
c may need work on this
            sgj=1
            DO jsym=1,2
               IF(tooclose(r1,z1,r2,z2,d)) THEN
                  ge0=ge0+sgi*sgj*(1-log(d/2))/2
               ELSE
                  ge0=ge0-sgi*sgj*log((r2-r1)**2+(z2-z1)**2)/4
               ENDIF
               IF(rsym==0) EXIT
               r2=-r2
               IF(rsym<0) sgj=-sgj
            ENDDO
         ENDIF
         IF(zsym==0) EXIT
         z2=-z2
         IF(zsym<0)  sgi=-sgi
      ENDDO
c terminate
      RETURN
      END FUNCTION ge0
c
c#######################################################################
c     check if two points are closer than eps * d
c-----------------------------------------------------------------------
      FUNCTION tooclose(r0,z0,r1,z1,d)
c type function
      LOGICAL::tooclose
c arguments
      REAL(rn)::r0,z0,r1,z1,d
c local variables
      REAL(rn)::rsm,dlz,denom,ksq,eta
      REAL(rn),parameter::eps=1.e-10
c begin
      rsm=r0+r1; dlz=z1-z0
      denom=dlz*dlz+rsm**2
      ksq=4.*r0*r1/denom; eta=1.-ksq
      tooclose=(((r1-r0)**2+(z1-z0)**2<eps**2*d**2)
     .      .OR.(d<eps)).OR.(eta<eps)
c terminate
      RETURN
      END FUNCTION tooclose
c
c#######################################################################
c     compute normal derivative of
c     free space l=0 Green's function for ES problem
c-----------------------------------------------------------------------
      FUNCTION ke0(aa,n)
c type function
      REAL(rn)::ke0
c arguments
      INTEGER::n
      REAL(rn),DIMENSION(n)::aa
c local variables
      INTEGER::isym,jsym
      REAL(rn)::r1,z1,r2,z2,d,xrn,xzn,sgi,sgj
      REAL(rn)::rsm,dlz,rr2,rr,k
      REAL(rn)::drr,dk
      REAL(rn),PARAMETER::rmin=1.e-10
c begin -- check number of arguments
      IF(n/=7) THEN
         CALL bomb(' Wrong number of arguments to **ke0**')
      ENDIF
      r1=aa(1); r2=aa(3)
      z2=aa(4); z1=aa(2)
      d=aa(5)
      xrn=aa(6); xzn=aa(7)
c
      ke0=0
      sgi=1
      DO isym=1,2
         IF(geom=='rz') THEN
            IF(tooclose(r1,z1,r2,z2,d)) THEN
cc               ge0=ge0+sgi*facm*(-1-log(d/rr/16.))*2*rr
            ELSE
               rsm=r1+r2
               dlz=z1-z2
               rr2=dlz*dlz+rsm**2; rr=SQRT(rr2)
               k=SQRT(4.*r1*r2)/rr
               drr=(xrn*rsm+xzn*dlz)/rr
               dk=2*r2/k/rr2*(xrn*(1-2*r1*rsm/rr2)-xzn*2*r1*dlz/rr2)
               ke0=ke0-sgi/pi*(-drr*kk(k)/rr2+dk*dkk(k)/rr)
            ENDIF
         ELSE
c need work on this
            sgj=1
            DO jsym=1,2
               IF(tooclose(r1,z1,r2,z2,d)) THEN
cc                  ge0=ge0+sgi*sgj*(1-log(d/2))/2
               ELSE
                  ke0=ke0-sgi*sgj*log((r2-r1)**2+(z2-z1)**2)/4
               ENDIF
               IF(rsym==0) EXIT
               r2=-r2
               IF(rsym<0) sgj=-sgj
            ENDDO
         ENDIF
         IF(zsym==0) EXIT
         z2=-z2
         IF(zsym<0)  sgi=-sgi
      ENDDO
c terminate
      RETURN
      END FUNCTION ke0
c
c#######################################################################
c     free space l=1 Green's function for ES problem
c-----------------------------------------------------------------------
      FUNCTION ge1(aa,n)
c type function
      REAL(rn)::ge1
c arguments
      INTEGER::n
      REAL(rn),DIMENSION(n)::aa
c local variables
      INTEGER::isym,jsym
      REAL(rn)::r1,z1,r2,z2,d,sgi,sgj
      REAL(rn)::rsm,dlz,rr2,rr,k
      REAL(rn),PARAMETER::rmin=1.e-10
c begin -- check number of arguments
      IF(n/=7) THEN
         CALL bomb(' Wrong number of arguments to **ge1**')
      ENDIF
      r1=aa(1); r2=aa(3)
      z1=aa(2); z2=aa(4)
      d=aa(5)
c
      ge1=0
      sgi=1
      DO isym=1,2
         IF(geom=='rz') THEN
            IF(tooclose(r1,z1,r2,z2,d)) THEN
cc               ge0=ge0+sgi*facm*(-1-log(d/rr/16.))*2*rr
            ELSE
               rsm=r1+r2
               dlz=z1-z2
               rr2=dlz*dlz+rsm**2; rr=SQRT(rr2)
               k=SQRT(4.*r1*r2)/rr
               ge1=ge1+sgi*(b_fu(k)-d_fu(k))/pi/rr
            ENDIF
         ELSE
c need work on this
            sgj=1
            DO jsym=1,2
               IF(tooclose(r1,z1,r2,z2,d)) THEN
cc                  ge1=ge1+sgi*sgj*(1-log(d/2))/2
               ELSE
                  ge1=ge1-sgi*sgj*log((r2-r1)**2+(z2-z1)**2)/4
               ENDIF
               IF(rsym==0) EXIT
               r2=-r2
               IF(rsym<0) sgj=-sgj
            ENDDO
         ENDIF
         IF(zsym==0) EXIT
         z2=-z2
         IF(zsym<0)  sgi=-sgi
      ENDDO
c terminate
      RETURN
      END FUNCTION ge1
c
c#######################################################################
c     compute normal derivative of
c     free space l=1 Green's function for ES problem
c-----------------------------------------------------------------------
      FUNCTION ke1(aa,n)
c type function
      REAL(rn)::ke1
c arguments
      INTEGER::n
      REAL(rn),DIMENSION(n)::aa
c local variables
      INTEGER::isym,jsym
      REAL(rn)::r1,z1,r2,z2,d,xrn,xzn,sgi,sgj
      REAL(rn)::rsm,dlz,rr2,rr,k
      REAL(rn)::drr,dk
      REAL(rn),PARAMETER::rmin=1.e-10
c begin -- check number of arguments
      IF(n/=7) THEN
         CALL bomb(' Wrong number of arguments to **ke1**')
      ENDIF
      r1=aa(1); r2=aa(3)
      z2=aa(4); z1=aa(2)
      d=aa(5)
      xrn=aa(6); xzn=aa(7)
c
      ke1=0
      sgi=1
      DO isym=1,2
         IF(geom=='rz') THEN
            IF(tooclose(r1,z1,r2,z2,d)) THEN
cc               ge0=ge0+sgi*facm*(-1-log(d/rr/16.))*2*rr
            ELSE
               rsm=r1+r2
               dlz=z1-z2
               rr2=dlz*dlz+rsm**2; rr=SQRT(rr2)
               k=SQRT(4.*r1*r2)/rr
               drr=(xrn*rsm+xzn*dlz)/rr
               dk=2*r2/k/rr2*(xrn*(1-2*r1*rsm/rr2)-xzn*2*r1*dlz/rr2)
               ke1=ke1+sgi/pi/rr*(dk*(db_fu(k)-dd_fu(k))
     .                           -drr/rr*(b_fu(k)-d_fu(k)))
            ENDIF
         ELSE
c need work on this
            sgj=1
            DO jsym=1,2
               IF(tooclose(r1,z1,r2,z2,d)) THEN
cc                  ge0=ge0+sgi*sgj*(1-log(d/2))/2
               ELSE
                  ke1=ke1-sgi*sgj*log((r2-r1)**2+(z2-z1)**2)/4
               ENDIF
               IF(rsym==0) EXIT
               r2=-r2
               IF(rsym<0) sgj=-sgj
            ENDDO
         ENDIF
         IF(zsym==0) EXIT
         z2=-z2
         IF(zsym<0)  sgi=-sgi
      ENDDO
c terminate
      RETURN
      END FUNCTION ke1
c
c#######################################################################
c     complete elliptic integral of the 1st kind
c-----------------------------------------------------------------------
      FUNCTION kk(k)
c type function
      REAL(rn)::kk
c arguments
      REAL(rn),INTENT(IN)::k
c begin
      kk=b_fu(k)+d_fu(k)
c terminate
      RETURN
      END FUNCTION kk
c
c#######################################################################
c     complete elliptic integral of the 2st kind
c-----------------------------------------------------------------------
      FUNCTION ee(k)
c type function
      REAL(rn)::ee
c arguments
      REAL(rn),INTENT(IN)::k
c begin
      ee=b_fu(k)+(1-k*k)*d_fu(k)
c terminate
      RETURN
      END FUNCTION ee
c
c#######################################################################
c     derivative of complete elliptic integral of the 1st kind
c-----------------------------------------------------------------------
      FUNCTION dkk(k)
c type function
      REAL(rn)::dkk
c arguments
      REAL(rn),INTENT(IN)::k
c begin
      dkk=k*b_fu(k)/(1-k*k)
c terminate
      RETURN
      END FUNCTION dkk
c
c#######################################################################
c     derivative of complete elliptic integral of the 2nd kind
c-----------------------------------------------------------------------
      FUNCTION dee(k)
c type function
      REAL(rn)::dee
c arguments
      REAL(rn),INTENT(IN)::k
c begin
      dee=-k*d_fu(k)
c terminate
      RETURN
      END FUNCTION dee
c
c#######################################################################
      FUNCTION b_fu(k)
c type function
      REAL(rn)::b_fu
c arguments
      REAL(rn),INTENT(IN)::k
c local variables
      INTEGER::im
      REAL(rn)::m,mc,lg
c begin -- fill coefficient types, if not done
      IF(.NOT.ASSOCIATED(bn)) CALL get_coef
c find region
      m=k*k; mc=one-m
      IF(m<=0.9) THEN
         im=10*m+1
         IF(m>0.85) im=10
         b_fu=eval(m-md(im),bn(im))
      ELSE
         lg=LOG(16/mc)
         b_fu=(eval(mc,b0)+lg*eval(mc,bx))/m
      ENDIF
c terminate
      RETURN
      END FUNCTION b_fu
c
c#######################################################################
      FUNCTION d_fu(k)
c type function
      REAL(rn)::d_fu
c arguments
      REAL(rn),INTENT(IN)::k
c local variables
      INTEGER::im
      REAL(rn)::m,mc,lg
c begin -- fill coefficient types, if not done
      IF(.NOT.ASSOCIATED(bn)) CALL get_coef
c find region
      m=k*k; mc=one-m
      IF(m<=0.9) THEN
         im=10*m+1
         IF(m>0.85) im=10
         d_fu=eval(m-md(im),dn(im))
      ELSE
         lg=LOG(16/mc)
         d_fu=(eval(mc,d0)+lg*eval(mc,dx))/m
      ENDIF
c terminate
      RETURN
      END FUNCTION d_fu
c
c#######################################################################
      FUNCTION db_fu(k)
c type function
      REAL(rn)::db_fu
c arguments
      REAL(rn),INTENT(IN)::k
c begin
      db_fu=(d_fu(k)-b_fu(k))/k
c terminate
      RETURN
      END FUNCTION db_fu
c
c#######################################################################
      FUNCTION dd_fu(k)
c type function
      REAL(rn) dd_fu
c arguments
      REAL(rn),INTENT(IN)::k
c begin
      dd_fu=(b_fu(k)/(1-k*k)-d_fu(k))/k
c terminate
      RETURN
      END FUNCTION dd_fu
c
c#######################################################################
      SUBROUTINE get_coef
c begin
      ALLOCATE(bn(10),dn(10),md(10))
c
      bn(1)%n=11; ALLOCATE(bn(1)%c(0:11)); bn(1)%c=b01; md(1)=0.05
      dn(1)%n=11; ALLOCATE(dn(1)%c(0:11)); dn(1)%c=d01
      bn(2)%n=11; ALLOCATE(bn(2)%c(0:11)); bn(2)%c=b12; md(2)=0.15
      dn(2)%n=11; ALLOCATE(dn(2)%c(0:11)); dn(2)%c=d12
      bn(3)%n=12; ALLOCATE(bn(3)%c(0:12)); bn(3)%c=b23; md(3)=0.25
      dn(3)%n=12; ALLOCATE(dn(3)%c(0:12)); dn(3)%c=d23
      bn(4)%n=13; ALLOCATE(bn(4)%c(0:13)); bn(4)%c=b34; md(4)=0.35
      dn(4)%n=13; ALLOCATE(dn(4)%c(0:13)); dn(4)%c=d34
      bn(5)%n=15; ALLOCATE(bn(5)%c(0:15)); bn(5)%c=b45; md(5)=0.45
      dn(5)%n=15; ALLOCATE(dn(5)%c(0:15)); dn(5)%c=d45
      bn(6)%n=16; ALLOCATE(bn(6)%c(0:16)); bn(6)%c=b56; md(6)=0.55
      dn(6)%n=16; ALLOCATE(dn(6)%c(0:16)); dn(6)%c=d56
      bn(7)%n=17; ALLOCATE(bn(7)%c(0:17)); bn(7)%c=b67; md(7)=0.65
      dn(7)%n=17; ALLOCATE(dn(7)%c(0:17)); dn(7)%c=d67
      bn(8)%n=20; ALLOCATE(bn(8)%c(0:20)); bn(8)%c=b78; md(8)=0.75
      dn(8)%n=20; ALLOCATE(dn(8)%c(0:20)); dn(8)%c=d78
      bn(9)%n=17; ALLOCATE(bn(9)%c(0:17)); bn(9)%c=b85; md(9)=0.825
      dn(9)%n=17; ALLOCATE(dn(9)%c(0:17)); dn(9)%c=d85
      bn(10)%n=20; ALLOCATE(bn(10)%c(0:20)); bn(10)%c=b59; md(10)=0.875
      dn(10)%n=20; ALLOCATE(dn(10)%c(0:20)); dn(10)%c=d59
c
      bx%n=13; ALLOCATE(bx%c(0:13)); bx%c=bsx
      b0%n=13; ALLOCATE(b0%c(0:13)); b0%c=bs0
      dx%n=12; ALLOCATE(dx%c(0:12)); dx%c=dsx
      d0%n=12; ALLOCATE(d0%c(0:12)); d0%c=ds0
c terminate
      RETURN
      END SUBROUTINE get_coef
c
c#######################################################################
      FUNCTION eval(x,cf)
c type function
      REAL(rn)::eval
c arguments
      REAL(rn),INTENT(IN)::x
      TYPE(coef_t),INTENT(IN)::cf
c local variables
      INTEGER::n,i
c begin
      n=cf%n; eval=cf%c(n)
      DO i=1,n
         eval=x*eval+cf%c(n-i)
      ENDDO
c terminate
      RETURN
      END FUNCTION eval
c
      END MODULE green_m











