c#######################################################################
c
c     NIMbnd code suite
c     Copyright by Coronado Consulting
c     2017
c
c     Programmed by D. C. Barnes
c     Coronado Consulting
c
c#######################################################################
c
c     Legendre functions right our of Numerical Recipes (plm.f)
c
c#######################################################################
c
c     version 1.0 -- 01/06/17
c
c#######################################################################
c
      MODULE plm_m
c
      USE serv_m
c
      IMPLICIT NONE
      INCLUDE 'params.f'
      INCLUDE 'priv.f'
c
      CONTAINS
c
c#######################################################################
c     P^m_l
c-----------------------------------------------------------------------
      FUNCTION plm(l,m,x)
c type function
      REAL(rn)::plm
c arguments
      INTEGER,INTENT(IN)::l,m
      REAL(rn),INTENT(IN)::x
c local variables
      INTEGER::i,ll
      REAL(rn)::fact,pll,pmm,pmmp1,somx2
c begin -- test parameters
      IF(m<0.OR.m>l.OR.ABS(x)>one) CALL bomb('bad arguments to plm')
c get P^m_m
      pmm=1
      IF(m>0) THEN
         somx2=SQRT((1-x)*(1+x))
         fact=1
         DO i=1,m
            pmm=-pmm*fact*somx2
            fact=fact+2
         ENDDO
      ENDIF
c get P^m_l
      IF(l==m) THEN
         plm=pmm
      ELSE
         pmmp1=x*(2*m+1)*pmm
         IF(l==m+1) THEN
            plm=pmmp1
         ELSE
            DO ll=m+2,l
               pll=(x*(2*ll-1)*pmmp1-(ll+m-1)*pmm)/(ll-m)
               pmm=pmmp1; pmmp1=pll
            ENDDO
            plm=pll
         ENDIF
      ENDIF
c terminate
      RETURN
      END FUNCTION plm
c
c#######################################################################
c     d/dx P^m_l
c-----------------------------------------------------------------------
      FUNCTION dplm(l,m,x)
c type function
      REAL(rn)::dplm
c arguments
      INTEGER,INTENT(IN)::l,m
      REAL(rn),INTENT(IN)::x
c begin
      dplm=(-(l+1)*x*plm(l,m,x)+(l-m+1)*plm(l+1,m,x))/(x*x-1)
c terminate
      END FUNCTION dplm
c
      END MODULE plm_m
