c#######################################################################
c
c     NIMbnd code suite
c     Copyright by Coronado Consulting
c     2017
c
c     Programmed by D. C. Barnes
c     Coronado Consulting
c
c#######################################################################
c
c     test driver for cylindrical problem (test.f)
c
c#######################################################################
c
c     version 1.0 -- 01/06/17
c
c#######################################################################
c
      PROGRAM test
c
      USE bmat_m
      USE plm_m
c
      IMPLICIT NONE
      INCLUDE 'params.f'
c local variables
      INTEGER,PARAMETER::b_type=1    ! 1 for can, 2 for sphere, 3 for torus
      INTEGER,PARAMETER::mr=95,mz=190
      REAL(rn),PARAMETER::rmx=1.,zmn=-0.5,zmx=0.5
c
      INTEGER::nb,ib,nsg
      REAL(rn)::drr,dzz,dth,th
      REAL(rn),DIMENSION(:),ALLOCATABLE::rb,zb,dd
      REAL(rn),DIMENSION(:,:,:),ALLOCATABLE::smat
c
      INTEGER,PARAMETER::lmx=2
      REAL(rn),PARAMETER::zsr=0.2
c
      INTEGER::is,l
      REAL(rn)::rs,zs,drs,dzs,ds,nr,nz,sbl,dsl,rx,err,norm
      REAL(rn),DIMENSION(:),ALLOCATABLE::rob,dro,xb,dxb,sb
      REAL(rn),DIMENSION(:,:),ALLOCATABLE::phi,dphi,sphi,dif
      REAL(rn),DIMENSION(:,:,:),ALLOCATABLE::plmx,dplmx
      REAL(rn),DIMENSION(:),ALLOCATABLE::q
c begin -- initialize plots
      CALL gnup_set('test',zero)
c make boundary curve
      SELECT CASE (b_type)
      CASE (1)
c soup can
         nb=2*mr+mz+1; ALLOCATE(rb(nb),zb(nb)); rb=0; zb=0
c
         drr=rmx/mr; rb(1)=0; zb(1)=zmn
c bottom
         DO ib=2,mr
            rb(ib)=rb(ib-1)+drr; zb(ib)=zmn
         ENDDO
c curved side
         dzz=(zmx-zmn)/mz; rb(mr+1)=rmx; zb(mr+1)=zmn
         DO ib=2,mz
            rb(ib+mr)=rmx; zb(ib+mr)=zb(ib+mr-1)+dzz
         ENDDO
c top
         rb(mr+mz+1)=rmx; zb(mr+mz+1)=zmx
         DO ib=mr,1,-1
            rb(nb+1-ib)=rb(nb-ib)-drr; zb(nb+1-ib)=zmx
         ENDDO
c
      CASE (2)
c sphere
         nb=2*mr+mz+1; ALLOCATE(rb(nb),zb(nb)); rb=0; zb=0
c
         dth=-pi/(nb-1); th=pi
         DO ib=1,nb
            rb(ib)=SIN(th); zb(ib)=COS(th)
            th=th+dth
         ENDDO
c
      CASE (3)
c torus
         nb=2*mr+mz+1; ALLOCATE(rb(nb),zb(nb)); rb=0; zb=0
         dth=2*pi/(nb-1); th=0
         DO ib=1,nb
            rb(ib)=3+SIN(th); zb(ib)=COS(th)
            th=th+dth
         ENDDO
      END SELECT
c plot boundary
      CALL gnup_lineplot(rb,zb,nb,'lin','lin','r','z','bound')
c make solution matrices
      nsg=nb-1; ALLOCATE(smat(2,nsg,nsg),dd(nsg))
      CALL get_bmat(rb,zb,smat,dd)
c make test harmonic functions
      ALLOCATE(phi(3,nsg),dphi(3,nsg)); phi=0; dphi=0
c radius, cos polar angle, and normal derivative
      ALLOCATE(rob(nsg),dro(nsg),xb(nsg),dxb(nsg),sb(nsg))
      rob=0; dro=0; xb=0; dxb=0; sbl=0; dsl=0
      DO is=1,nsg
         rs=(rb(is)+rb(is+1))/2; zs=(zb(is)+zb(is+1))/2
         drs=rb(is+1)-rb(is); dzs=zb(is+1)-zb(is)
         ds=SQRT(drs**2+dzs**2)
         nr=dzs/ds; nz=-drs/ds
         rob(is)=SQRT(rs**2+(zs-zsr)**2)
         dro(is)=(rs*nr+(zs-zsr)*nz)/rob(is)
         xb(is)=(zs-zsr)/rob(is)
         dxb(is)=nz/rob(is)-dro(is)*(zs-zsr)/rob(is)**2
         sb(is)=sbl+(ds+dsl)/2
         sbl=sb(is); dsl=ds
      ENDDO
c diagnostic plots
      CALL gnup_lineplot(sb,xb,nsg,'lin','lin','s','xb','cos(theta)')
      CALL gnup_lineplot(sb,dxb,nsg,'lin','lin','s','Dxb','Dcos(theta)')
c Legendre functions and derivatives
      ALLOCATE(plmx(2,lmx,nsg),dplmx(2,lmx,nsg)); plmx=0; dplmx=0
      DO l=1,lmx
         DO is=1,nsg
            plmx(1,l,is)=plm(l,0,xb(is))
            dplmx(1,l,is)=dplm(l,0,xb(is))
            plmx(2,l,is)=plm(l,1,xb(is))
            dplmx(2,l,is)=dplm(l,1,xb(is))
         ENDDO
      ENDDO
c n=0
      DO l=1,lmx
         CALL random_number(rx); rx=rx/l
         phi(1,:)=phi(1,:)+rx*plmx(1,l,:)/rob**(l+1)
         dphi(1,:)=dphi(1,:)
     .        +rx*(dxb*dplmx(1,l,:)
     .            -(l+1)*dro*plmx(1,l,:)/rob)/rob**(l+1)
      ENDDO
c check consistency
      WRITE(*,*) MAXVAL(ABS(dphi(1,:))),SUM(dd*dphi(1,:))
c n=1
      DO l=1,lmx
         CALL random_number(rx); rx=rx/l
         phi(2,:)=phi(2,:)+rx*plmx(2,l,:)/rob**(l+1)
         dphi(2,:)=dphi(2,:)+rx*(dxb*dplmx(2,l,:)
     .                          -(l+1)*dro*plmx(2,l,:)/rob)/rob**(l+1)
         CALL random_number(rx); rx=rx/l
         phi(3,:)=phi(3,:)+rx*plmx(2,l,:)/rob**(l+1)
         dphi(3,:)=dphi(3,:)+rx*(dxb*dplmx(2,l,:)
     .                          -(l+1)*dro*plmx(2,l,:)/rob)/rob**(l+1)
      ENDDO
c diagnsotic plots
      CALL gnup_lineplot(sb,phi(1,:),nsg,'lin','lin','s','phi','n=0')
      CALL gnup_lineplot(sb,dphi(1,:),nsg,'lin','lin','s','Dphi','n=0')
      CALL gnup_mlineplot(sb,TRANSPOSE(phi(2:,:)),nsg,2,nsg,'lin','lin'
     .                   ,'s','phi','n=1')
      CALL gnup_mlineplot(sb,TRANSPOSE(dphi(2:,:)),nsg,2,nsg,'lin','lin'
     .                   ,'s','Dphi','n=1')
c ready for solution
      ALLOCATE(sphi(3,nsg))
      sphi(1,:)=MATMUL(smat(1,:,:),dphi(1,:))
      sphi(2,:)=MATMUL(smat(2,:,:),dphi(2,:))
      sphi(3,:)=MATMUL(smat(2,:,:),dphi(3,:))
c compare
      ALLOCATE(dif(nsg,2))
      dif(:,1)=phi(1,:); dif(:,2)=sphi(1,:)
      CALL gnup_mlineplot(sb,dif,nsg,2,nsg,'lin','lin','s','phi','n=0')
      CALL gnup_lineplot(sb,dd*(dif(:,1)-dif(:,2)),nsg
     .                  ,'lin','lin','s','wdiff','n=0')
      err=SQRT(SUM(dd*(dif(:,1)-dif(:,2))**2)/SUM(dd*dif(:,1)**2))
      WRITE(*,*) 'n=0 error: ',err
      dif(:,1)=phi(2,:); dif(:,2)=sphi(2,:)
      err=SUM(dd*(dif(:,1)-dif(:,2))**2)
      norm=SUM(dd*dif(:,1)**2)
      CALL gnup_mlineplot(sb,dif,nsg,2,nsg,'lin','lin'
     .                   ,'s','phi','n=1(Re)')
      dif(:,1)=phi(3,:); dif(:,2)=sphi(3,:)
      CALL gnup_mlineplot(sb,dif,nsg,2,nsg,'lin','lin'
     .                   ,'s','phi','n=1(Im)')
      err=SUM(dd*(dif(:,1)-dif(:,2))**2)+err
      norm=SUM(dd*dif(:,1)**2)+norm; err=SQRT(err/norm)
      WRITE(*,*) 'n=1 error: ',err
c terminate
      CALL gnup_wrap
      STOP
      END PROGRAM test
