c-----------------------------------------------------------------------
c       this external function evaluates R*A_phi from a set of
c       coils that are modeled as circular loops of wire.  it uses
c       the complete elliptic integral function (cel) that has been
c       adapted from Numerical Recipes.
c-----------------------------------------------------------------------
        FUNCTION aph_eval(r,z,ncoil,coilr,coilz,coili,mu0) RESULT(aph)
        USE local
        IMPLICIT NONE
 
        REAL(r8) :: aph
        REAL(r8), INTENT(IN) :: r,z,mu0
        REAL(r8), DIMENSION(*), INTENT(IN) :: coilr,coilz,coili
        INTEGER(i4), INTENT(IN) :: ncoil
 
        REAL(r8) :: xc,sinth,costh,k2d,k2,efac,kc
        REAL(r8), EXTERNAL :: cel
        REAL(r8), PARAMETER :: aph_tol=1.e-12_r8
        INTEGER(i4), PARAMETER :: aph_maxn=10000
        INTEGER(i4) :: icoil
 
        aph=0
        DO icoil=1,ncoil
          xc=SQRT(r**2+(z-coilz(icoil))**2)
          sinth=r/xc
          costh=(z-coilz(icoil))/xc
          k2d=coilr(icoil)**2+xc**2+2*coilr(icoil)*xc*sinth
          k2=4*coilr(icoil)*xc*sinth/k2d
          IF (k2<=0) THEN
            efac=1._r8
          ELSE
            kc=SQRT(1._r8-k2)
            efac=2._r8/(pi*k2)*(
     $        (2._r8-k2)*cel(kc,1._r8,1._r8,1._r8,aph_tol,aph_maxn)
     $         -2._r8*cel(kc,1._r8,1._r8,1._r8-k2,aph_tol,aph_maxn))
          ENDIF
          aph=aph+mu0*r*coili(icoil)*coilr(icoil)*
     $                efac/(2._r8*SQRT(k2d))
        ENDDO
 
        RETURN
        END FUNCTION aph_eval


