c-----------------------------------------------------------------------
c     file boundary.f:  contains all subprograms related to the external
c     boundary conditions for nimrod.
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c     module declaration.
c-----------------------------------------------------------------------
      MODULE boundary
      USE local
      IMPLICIT NONE

      CONTAINS
c-----------------------------------------------------------------------
c     code organization.
c-----------------------------------------------------------------------
c     1. dirichlet_rhs.
c     2. no_rhs_bc.
c     3. dirichlet_op.
c     4. no_mat_bc.
c     5. dirichlet_comp_op.
c     6. no_comp_mat_bc.
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c     subprogram 1. dirichlet_rhs.
c     apply external boundary conditions to blocks in exblock_list.
c
c     when the optional svess parameter is provided, the data that is
c     subtracted from rhs is saved in the seam_csave data structure.
c     note that while svess is a character, only its presence is tested.
c-----------------------------------------------------------------------
      SUBROUTINE dirichlet_rhs(rhs,blseam,component,nqty,symm,svess)
      USE edge_type_mod
      USE vector_type_mod

      INTEGER(i4), INTENT(IN) :: nqty
      TYPE(cvector_type), INTENT(INOUT) :: rhs
      TYPE(edge_type), INTENT(IN) :: blseam
      CHARACTER(*), INTENT(IN) :: component
      CHARACTER(*), INTENT(IN), OPTIONAL :: symm,svess

      COMPLEX(r8) :: tempn,tempt,tempp
      COMPLEX(r8), DIMENSION(3) :: cvec3
      REAL(r8), DIMENSION(2) :: tangent,normal
      INTEGER(i4) :: iv,ix,iy,imode,ivp,nside,is,isymm,iq,nm
c-----------------------------------------------------------------------
c     set the symmetry flag.  if symm starts with "t", the top boundary
c     is a symmetry condition.  if symm starts with "b", the bottom
c     boundary is a symmetry condition.
c-----------------------------------------------------------------------
      isymm=0
      IF (PRESENT(symm)) THEN
        SELECT CASE(symm(1:1))
        CASE('t','T')
          isymm=1
        CASE('b','B')
          isymm=-1
        END SELECT
      ENDIF
c-----------------------------------------------------------------------
c     if component is not 'all', nqty must be 3.
c     zero the desired components of the rhs vector for all modes.
c-----------------------------------------------------------------------
      nm=SIZE(rhs%arr,4)
      IF (.NOT.(nqty==3_i4.OR.nqty==6_i4) .AND. component/='all') THEN
        CALL nim_stop('Dirichlet_rhs component index error.')
      ENDIF
      vert: DO iv=1,blseam%nvert
        IF (.NOT.blseam%expoint(iv)) CYCLE
        ix=blseam%vertex(iv)%intxy(1)
        iy=blseam%vertex(iv)%intxy(2)
        IF (component=='all') THEN
          IF (isymm==-1) THEN
            IF (blseam%vertex(iv)%norm(2)==-1._r8) CYCLE
          ELSE IF (isymm==1) THEN
            IF (blseam%vertex(iv)%norm(2)==1._r8) CYCLE
          ENDIF
          IF (PRESENT(svess)) THEN
            iq=1
            DO imode=1,nm
              blseam%vertex(iv)%seam_csave(iq:iq+nqty-1)=
     $          blseam%vertex(iv)%seam_csave(iq:iq+nqty-1)+
     $          rhs%arr(1:nqty,ix,iy,imode)
              rhs%arr(1:nqty,ix,iy,imode)=0._r8
              iq=iq+nqty
            ENDDO
          ELSE
            rhs%arr(1:nqty,ix,iy,:)=0._r8
          ENDIF
        ELSE
          tangent=blseam%vertex(iv)%tang
          normal =blseam%vertex(iv)%norm
          iq=1
          DO imode=1,nm
            tempn=rhs%arr(1,ix,iy,imode)*normal(1)
     $           +rhs%arr(2,ix,iy,imode)*normal(2)
            tempt=rhs%arr(1,ix,iy,imode)*tangent(1)
     $           +rhs%arr(2,ix,iy,imode)*tangent(2)
            tempp=rhs%arr(3,ix,iy,imode)
            IF (component=='normal') THEN
              tempn=0._r8
              IF (blseam%excorner(iv)) tempt=0._r8
            ELSE
              tempt=0._r8
              IF (blseam%excorner(iv)) tempn=0._r8
              tempp=0._r8
            ENDIF
            cvec3(1:2)=tempn*normal+tempt*tangent
            cvec3(3)=tempp
            IF (PRESENT(svess))
     $        blseam%vertex(iv)%seam_csave(iq:iq+2)=
     $          blseam%vertex(iv)%seam_csave(iq:iq+2)+
     $          rhs%arr(1:3,ix,iy,imode)-cvec3
            rhs%arr(1:3,ix,iy,imode)=cvec3
c-----------------------------------------------------------------------
c           the following is for 6-vector systems.
c-----------------------------------------------------------------------
            IF (nqty==6_i4) THEN
              tempn=rhs%arr(4,ix,iy,imode)*normal(1)
     $             +rhs%arr(5,ix,iy,imode)*normal(2)
              tempt=rhs%arr(4,ix,iy,imode)*tangent(1)
     $             +rhs%arr(5,ix,iy,imode)*tangent(2)
              tempp=rhs%arr(6,ix,iy,imode)
              IF (component=='normal') THEN
                tempn=0._r8
                IF (blseam%excorner(iv)) tempt=0._r8
              ELSE
                tempt=0._r8
                IF (blseam%excorner(iv)) tempn=0._r8
                tempp=0._r8
              ENDIF
              cvec3(1:2)=tempn*normal+tempt*tangent
              cvec3(3)=tempp
              IF (PRESENT(svess))
     $          blseam%vertex(iv)%seam_csave(iq+3:iq+5)=
     $            blseam%vertex(iv)%seam_csave(iq+3:iq+5)+
     $            rhs%arr(4:6,ix,iy,imode)-cvec3
              rhs%arr(4:6,ix,iy,imode)=cvec3
            ENDIF
            iq=iq+nqty
          ENDDO
        ENDIF
      ENDDO vert
c-----------------------------------------------------------------------
c     side-centered nodes.
c-----------------------------------------------------------------------
      IF (ASSOCIATED(rhs%arrh)) THEN
        nside=SIZE(rhs%arrh,2)
        seg: DO iv=1,blseam%nvert
          ivp=iv-1
          IF (ivp==0) ivp=blseam%nvert
          IF (.NOT.(blseam%expoint(iv).AND.blseam%expoint(ivp)))
     $      CYCLE seg
          ix=blseam%segment(iv)%intxys(1)
          iy=blseam%segment(iv)%intxys(2)
          IF (component=='all') THEN
            IF (isymm==-1) THEN
              IF (blseam%segment(iv)%norm(2,1)==-1._r8) CYCLE
            ELSE IF (isymm==1) THEN
              IF (blseam%segment(iv)%norm(2,1)==1._r8) CYCLE
            ENDIF
            IF (PRESENT(svess)) THEN
              iq=1
              IF (blseam%segment(iv)%h_side) THEN
                DO imode=1,nm
                  DO is=1,nside
                    blseam%segment(iv)%seam_csave(iq:iq+nqty-1)=
     $                blseam%segment(iv)%seam_csave(iq:iq+nqty-1)+
     $                rhs%arrh(1:nqty,is,ix,iy,imode)
                    rhs%arrh(1:nqty,is,ix,iy,imode)=0._r8
                    iq=iq+nqty
                  ENDDO
                ENDDO
              ELSE
                DO imode=1,nm
                  DO is=1,nside
                    blseam%segment(iv)%seam_csave(iq:iq+nqty-1)=
     $                blseam%segment(iv)%seam_csave(iq:iq+nqty-1)+
     $                rhs%arrv(1:nqty,is,ix,iy,imode)
                    rhs%arrv(1:nqty,is,ix,iy,imode)=0._r8
                    iq=iq+nqty
                  ENDDO
                ENDDO
              ENDIF
            ELSE
              IF (blseam%segment(iv)%h_side) THEN
                rhs%arrh(1:nqty,:,ix,iy,:)=0._r8
              ELSE
                rhs%arrv(1:nqty,:,ix,iy,:)=0._r8
              ENDIF
            ENDIF
          ELSE
c-----------------------------------------------------------------------
c           component-specific horizontal side nodes:
c-----------------------------------------------------------------------
            iq=1
            IF (blseam%segment(iv)%h_side) THEN
              DO imode=1,nm
                DO is=1,nside
                  tangent=blseam%segment(iv)%tang(:,is)
                  normal =blseam%segment(iv)%norm(:,is)
                  tempn=rhs%arrh(1,is,ix,iy,imode)*normal(1)
     $                 +rhs%arrh(2,is,ix,iy,imode)*normal(2)
                  tempt=rhs%arrh(1,is,ix,iy,imode)*tangent(1)
     $                 +rhs%arrh(2,is,ix,iy,imode)*tangent(2)
                  tempp=rhs%arrh(3,is,ix,iy,imode)
                  IF (component=='normal') THEN
                    tempn=0._r8
                  ELSE
                    tempt=0._r8
                    tempp=0._r8
                  ENDIF
                  cvec3(1:2)=tempn*normal+tempt*tangent
                  cvec3(3)=tempp
                  IF (PRESENT(svess))
     $              blseam%segment(iv)%seam_csave(iq:iq+2)=
     $                blseam%segment(iv)%seam_csave(iq:iq+2)+
     $                rhs%arrh(1:3,is,ix,iy,imode)-cvec3
                  rhs%arrh(1:3,is,ix,iy,imode)=cvec3
c-----------------------------------------------------------------------
c                 6-vectors:
c-----------------------------------------------------------------------
                  IF (nqty==6_i4) THEN
                    tempn=rhs%arrh(4,is,ix,iy,imode)*normal(1)
     $                   +rhs%arrh(5,is,ix,iy,imode)*normal(2)
                    tempt=rhs%arrh(4,is,ix,iy,imode)*tangent(1)
     $                   +rhs%arrh(5,is,ix,iy,imode)*tangent(2)
                    tempp=rhs%arrh(6,is,ix,iy,imode)
                    IF (component=='normal') THEN
                      tempn=0._r8
                    ELSE
                      tempt=0._r8
                      tempp=0._r8
                    ENDIF
                    cvec3(1:2)=tempn*normal+tempt*tangent
                    cvec3(3)=tempp
                    IF (PRESENT(svess))
     $                blseam%segment(iv)%seam_csave(iq+3:iq+5)=
     $                  blseam%segment(iv)%seam_csave(iq+3:iq+5)+
     $                  rhs%arrh(4:6,is,ix,iy,imode)-cvec3
                    rhs%arrh(4:6,is,ix,iy,imode)=cvec3
                  ENDIF
                  iq=iq+nqty
                ENDDO
              ENDDO
c-----------------------------------------------------------------------
c           component-specific vertical side nodes:
c-----------------------------------------------------------------------
            ELSE
              DO imode=1,nm
                DO is=1,nside
                  tangent=blseam%segment(iv)%tang(:,is)
                  normal =blseam%segment(iv)%norm(:,is)
                  tempn=rhs%arrv(1,is,ix,iy,imode)*normal(1)
     $                 +rhs%arrv(2,is,ix,iy,imode)*normal(2)
                  tempt=rhs%arrv(1,is,ix,iy,imode)*tangent(1)
     $                 +rhs%arrv(2,is,ix,iy,imode)*tangent(2)
                  tempp=rhs%arrv(3,is,ix,iy,imode)
                  IF (component=='normal') THEN
                    tempn=0._r8
                  ELSE
                    tempt=0._r8
                    tempp=0._r8
                  ENDIF
                  cvec3(1:2)=tempn*normal+tempt*tangent
                  cvec3(3)=tempp
                  IF (PRESENT(svess))
     $              blseam%segment(iv)%seam_csave(iq:iq+2)=
     $                blseam%segment(iv)%seam_csave(iq:iq+2)+
     $                rhs%arrv(1:3,is,ix,iy,imode)-cvec3
                  rhs%arrv(1:3,is,ix,iy,imode)=cvec3
c-----------------------------------------------------------------------
c                 6-vectors:
c-----------------------------------------------------------------------
                  IF (nqty==6_i4) THEN
                    tempn=rhs%arrv(4,is,ix,iy,imode)*normal(1)
     $                   +rhs%arrv(5,is,ix,iy,imode)*normal(2)
                    tempt=rhs%arrv(4,is,ix,iy,imode)*tangent(1)
     $                   +rhs%arrv(5,is,ix,iy,imode)*tangent(2)
                    tempp=rhs%arrv(6,is,ix,iy,imode)
                    IF (component=='normal') THEN
                      tempn=0._r8
                    ELSE
                      tempt=0._r8
                      tempp=0._r8
                    ENDIF
                    cvec3(1:2)=tempn*normal+tempt*tangent
                    cvec3(3)=tempp
                    IF (PRESENT(svess))
     $                blseam%segment(iv)%seam_csave(iq+3:iq+5)=
     $                  blseam%segment(iv)%seam_csave(iq+3:iq+5)+
     $                  rhs%arrv(4:6,is,ix,iy,imode)-cvec3
                    rhs%arrv(4:6,is,ix,iy,imode)=cvec3
                  ENDIF
                  iq=iq+nqty
                ENDDO
              ENDDO
            ENDIF
          ENDIF
        ENDDO seg
      ENDIF
c-----------------------------------------------------------------------
c     terminate.
c-----------------------------------------------------------------------
      RETURN
      END SUBROUTINE dirichlet_rhs
c-----------------------------------------------------------------------
c     subprogram 2. no_rhs_bc.
c     a dummy boundary condition routine with the same interface
c     block as dirichlet_rhs (and future boundary condition routines).
c-----------------------------------------------------------------------
      SUBROUTINE no_rhs_bc(rhs,blseam,component,nqty,symm,svess)
      USE edge_type_mod
      USE vector_type_mod

      INTEGER(i4), INTENT(IN) :: nqty
      TYPE(cvector_type), INTENT(INOUT) :: rhs
      TYPE(edge_type), INTENT(IN) :: blseam
      CHARACTER(*), INTENT(IN) :: component
      CHARACTER(*), INTENT(IN), OPTIONAL :: symm,svess

      RETURN
      END SUBROUTINE no_rhs_bc
c-----------------------------------------------------------------------
c     subprogram 3. dirichlet_op.
c     apply Dirichlet boundary conditions to a Cartesian operator.
c     If the specified component is tangent, the resultant matrix is 
c     (I-tt-zz).M.(I-tt-zz)+tt+zz, where t is the surface tangent in 
c     the computational plane, z is the unit vector normal to the plane,
c     and I is the identity matrix.  If the specified component is 
c     normal, the resultant matrix is (I-nn).M.(I-nn)+nn, where n is 
c     the surface normal.  If the input is all, then the result is
c     (I-tt-zz-nn).M.(I-tt-zz-nn)+tt+zz+nn.
c
c     when the end of the component parameter is 'offdiag,' the passed
c     matrix structure contains only off-diagonal matrix entries, and
c     there is no diagonal entry to enter after couplings are
c     eliminated.
c-----------------------------------------------------------------------
      SUBROUTINE dirichlet_op(mat,component,dscale,symm)
      USE seam_storage_mod
      USE matrix_type_mod

      TYPE(global_matrix_type), INTENT(INOUT) :: mat
      CHARACTER(*), INTENT(IN) :: component
      REAL(r8), INTENT(IN) :: dscale
      CHARACTER(*), INTENT(IN), OPTIONAL :: symm

      INTEGER(i4) :: iv,nv,ip,ipn,ibl,ibe,ibv,jxmin,jxmax,jymin,jymax,
     $               jx,jy,ix,iy,ijx,ijy,mx,my,iq,jq,clim,elim,ixn,
     $               pd,pd2,imat,jmat,ivp,ix0,iy0,jx0,jy0,ibase_st,
     $               itype,jtype,iq0,iq1,jq0,jq1,pdm1,is,js,lcomp,isymm
      REAL(r8), DIMENSION(2) :: unit
      REAL(r8), DIMENSION(:), ALLOCATABLE :: product
      REAL(r8), DIMENSION(:,:,:,:,:,:), POINTER :: rmat
      TYPE(matrix_element_type3), DIMENSION(:), POINTER :: tmat
      LOGICAL :: complex,diag
c-----------------------------------------------------------------------
c     check vector component range.
c-----------------------------------------------------------------------
      IF (SIZE(mat%rbl_mat)>0) THEN
        clim=SIZE(mat%rbl_mat(1)%mat(1,1)%arr,1)
      ELSE
        clim=SIZE(mat%tbl_mat(1)%lmat(0)%element,1)
      ENDIF
      IF (MODULO(clim,3_i4)/=0.AND.component(1:3)/='all') CALL nim_stop
     $  ('Dirichlet_op '//component//' requires (r,z,phi)-vectors.')
c-----------------------------------------------------------------------
c     set the symmetry flag.  if symm starts with "t", the top boundary
c     is a symmetry condition.  if symm starts with "b", the bottom
c     boundary is a symmetry condition.
c-----------------------------------------------------------------------
      isymm=0
      IF (PRESENT(symm)) THEN
        SELECT CASE(symm(1:1))
        CASE('t','T')
          isymm=1
        CASE('b','B')
          isymm=-1
        END SELECT
      ENDIF
c-----------------------------------------------------------------------
c     save the diagonal scaling and essential-condition flag.
c-----------------------------------------------------------------------
      mat%diag_scale=dscale
      mat%essential_cond=component
c-----------------------------------------------------------------------
c     set vector index limits.
c-----------------------------------------------------------------------
      lcomp=LEN_TRIM(component)
      elim=3
      IF (lcomp>=6) THEN
        IF (component(1:6)=='normal') elim=2
      ENDIF
c-----------------------------------------------------------------------
c     check if this matrix is entirely off-diagonal.
c-----------------------------------------------------------------------
      diag=.true.
      IF (lcomp>6) THEN
        IF (component(lcomp-6:lcomp)=='offdiag') diag=.false.
      ENDIF
c-----------------------------------------------------------------------
c     loop over all external boundary points and zero the couplings
c     to the specified components along the boundary.
c-----------------------------------------------------------------------
      block: DO ibe=1,SIZE(exblock_list)
        ibl=exblock_list(ibe)
        nv=seam(ibl)%nvert
c-----------------------------------------------------------------------
c       begin rblock-specific coding.  rblock basis types indices are
c       1 == grid vertex centered
c       2 == horizontal side centered
c       3 == vertical side centered
c       4 == interior centered
c       1 : 3 are affected by boundary conditions.
c       note that itype is the to basis type and jtype is the from
c       basis type.
c-----------------------------------------------------------------------
        block_type: IF (ibl<=SIZE(mat%rbl_mat)) THEN
c-----------------------------------------------------------------------
c         connections to the rblock boundary vertices.
c-----------------------------------------------------------------------
          IF (mat%rbl_mat(ibl)%nbtype>1) THEN
            pd=SUM(mat%rbl_mat(ibl)%nb_type(1:2))
          ELSE
            pd=1
          ENDIF
          pdm1=pd-1
          rb_vert: DO iv=1,nv
            ivp=iv-1
            IF (ivp==0) ivp=seam(ibl)%nvert
            mx=SIZE(mat%rbl_mat(ibl)%mat(1,1)%arr,5)-1
            my=SIZE(mat%rbl_mat(ibl)%mat(1,1)%arr,6)-1
c-----------------------------------------------------------------------
c           eliminate all couplings if all components are zeroed.
c-----------------------------------------------------------------------
            IF (component(1:3)=='all') THEN
              DO itype=1,mat%rbl_mat(ibl)%nbtype
                ix0=mat%rbl_mat(ibl)%ix0(itype)
                iy0=mat%rbl_mat(ibl)%iy0(itype)
                DO jtype=1,MIN(mat%rbl_mat(ibl)%nbtype,3_i4)
                  IF (jtype==1) THEN
                    IF (.NOT.seam(ibl)%expoint(iv)) CYCLE
                    IF (isymm==-1) THEN
                      IF (seam(ibl)%vertex(iv)%norm(2)==-1._r8) CYCLE
                    ELSE IF (isymm==1) THEN
                      IF (seam(ibl)%vertex(iv)%norm(2)==1._r8) CYCLE
                    ENDIF
                    ix=seam(ibl)%vertex(iv)%intxy(1)
                    iy=seam(ibl)%vertex(iv)%intxy(2)
                  ELSE
                    IF (.NOT.(seam(ibl)%expoint(iv).AND.
     $                        seam(ibl)%expoint(ivp))) CYCLE
                    IF (seam(ibl)%segment(iv)%h_side.AND.jtype>2) CYCLE
                    IF (.NOT.seam(ibl)%segment(iv)%h_side.AND.
     $                  jtype<3) CYCLE
                    IF (isymm==-1) THEN
                      IF (seam(ibl)%segment(iv)%norm(2,1)==-1._r8) CYCLE
                    ELSE IF (isymm==1) THEN
                      IF (seam(ibl)%segment(iv)%norm(2,1)==1._r8) CYCLE
                    ENDIF
                    ix=seam(ibl)%segment(iv)%intxys(1)
                    iy=seam(ibl)%segment(iv)%intxys(2)
                  ENDIF
                  rmat=>mat%rbl_mat(ibl)%mat(jtype,itype)%arr
                  jx0=mat%rbl_mat(ibl)%ix0(jtype)
                  jy0=mat%rbl_mat(ibl)%iy0(jtype)
                  jxmin=MAX(ix0-1,ix0-ix)
                  jxmax=MIN(1-jx0,mx -ix)
                  jymin=MAX(iy0-1,iy0-iy)
                  jymax=MIN(1-jy0,my -iy)
                  DO jy=jymin,jymax
                    ijy=iy+jy
                    DO jx=jxmin,jxmax
                      ijx=ix+jx
                      rmat(:,-jx,-jy,:,ijx,ijy)=0
                    ENDDO
                  ENDDO
                ENDDO
              ENDDO
              DO itype=1,MIN(mat%rbl_mat(ibl)%nbtype,3_i4)
                IF (itype==1) THEN
                  IF (.NOT.seam(ibl)%expoint(iv)) CYCLE
                  IF (isymm==-1) THEN
                    IF (seam(ibl)%vertex(iv)%norm(2)==-1._r8) CYCLE
                  ELSE IF (isymm==1) THEN
                    IF (seam(ibl)%vertex(iv)%norm(2)==1._r8) CYCLE
                  ENDIF
                  ix=seam(ibl)%vertex(iv)%intxy(1)
                  iy=seam(ibl)%vertex(iv)%intxy(2)
                ELSE
                  IF (.NOT.(seam(ibl)%expoint(iv).AND.
     $                      seam(ibl)%expoint(ivp))) CYCLE
                  IF (seam(ibl)%segment(iv)%h_side.AND.itype>2) CYCLE
                  IF (.NOT.seam(ibl)%segment(iv)%h_side.AND.
     $                itype<3) CYCLE
                  IF (isymm==-1) THEN
                    IF (seam(ibl)%segment(iv)%norm(2,1)==-1._r8) CYCLE
                  ELSE IF (isymm==1) THEN
                    IF (seam(ibl)%segment(iv)%norm(2,1)==1._r8) CYCLE
                  ENDIF
                  ix=seam(ibl)%segment(iv)%intxys(1)
                  iy=seam(ibl)%segment(iv)%intxys(2)
                ENDIF
                DO jtype=1,mat%rbl_mat(ibl)%nbtype
                  rmat=>mat%rbl_mat(ibl)%mat(jtype,itype)%arr
                  rmat(:,:,:,:,ix,iy)=0
                  IF (jtype==itype.AND.diag) THEN
                    DO jq=1,mat%rbl_mat(ibl)%nq_type(itype)
                      rmat(jq,0,0,jq,ix,iy)=dscale
                    ENDDO
                  ENDIF
                ENDDO
              ENDDO
              CYCLE rb_vert
            ENDIF
c-----------------------------------------------------------------------
c           eliminate couplings to and from an external corner,
c           where only the grid vertex basis is appropriate.
c           tangent specified; for specified normal at a corner,
c           zero all but phi component.
c-----------------------------------------------------------------------
            IF (seam(ibl)%excorner(iv)) THEN
              ix=seam(ibl)%vertex(iv)%intxy(1)
              iy=seam(ibl)%vertex(iv)%intxy(2)
              DO itype=1,mat%rbl_mat(ibl)%nbtype
                rmat=>mat%rbl_mat(ibl)%mat(1,itype)%arr
                ix0=mat%rbl_mat(ibl)%ix0(itype)
                iy0=mat%rbl_mat(ibl)%iy0(itype)
                jxmin=MAX(ix0-1,ix0-ix)
                jxmax=MIN(1    ,mx -ix)
                jymin=MAX(iy0-1,iy0-iy)
                jymax=MIN(1    ,my -iy)
                DO jy=jymin,jymax
                  ijy=iy+jy
                  DO jx=jxmin,jxmax
                    ijx=ix+jx
                    rmat(1:elim,-jx,-jy,:,ijx,ijy)=0
                  ENDDO
                ENDDO
              ENDDO
              DO jtype=1,mat%rbl_mat(ibl)%nbtype
                rmat=>mat%rbl_mat(ibl)%mat(jtype,1)%arr
                rmat(:,:,:,1:elim,ix,iy)=0
                IF (jtype==1.AND.diag) THEN
                  DO jq=1,elim
                    rmat(jq,0,0,jq,ix,iy)=dscale
                  ENDDO
                ENDIF
              ENDDO
              ibase_st=2
            ELSE
              ibase_st=1
            ENDIF
c-----------------------------------------------------------------------
c           eliminate couplings from the specified component only.
c-----------------------------------------------------------------------
            DO itype=1,mat%rbl_mat(ibl)%nbtype
              ix0=mat%rbl_mat(ibl)%ix0(itype)
              iy0=mat%rbl_mat(ibl)%iy0(itype)
              ALLOCATE(product(mat%rbl_mat(ibl)%nq_type(itype)))
              DO jtype=ibase_st,MIN(mat%rbl_mat(ibl)%nbtype,3_i4)
                rmat=>mat%rbl_mat(ibl)%mat(jtype,itype)%arr
                jx0=mat%rbl_mat(ibl)%ix0(jtype)
                jy0=mat%rbl_mat(ibl)%iy0(jtype)
                DO jmat=1,mat%rbl_mat(ibl)%nb_type(jtype)
                  IF (jtype==1) THEN
                    IF (.NOT.seam(ibl)%expoint(iv)) CYCLE
                    ix=seam(ibl)%vertex(iv)%intxy(1)
                    iy=seam(ibl)%vertex(iv)%intxy(2)
                    IF (component(1:6)=='normal') THEN
                      unit=seam(ibl)%vertex(iv)%norm
                    ELSE
                      unit=seam(ibl)%vertex(iv)%tang
                    ENDIF
                  ELSE
                    IF (.NOT.(seam(ibl)%expoint(iv).AND.
     $                        seam(ibl)%expoint(ivp))) CYCLE
                    IF (seam(ibl)%segment(iv)%h_side) THEN
                      IF (jtype>2) CYCLE
                    ELSE
                      IF (jtype<3) CYCLE
                    ENDIF
                    ix=seam(ibl)%segment(iv)%intxys(1)
                    iy=seam(ibl)%segment(iv)%intxys(2)
                    IF (component(1:6)=='normal') THEN
                      unit=seam(ibl)%segment(iv)%norm(:,jmat)
                    ELSE
                      unit=seam(ibl)%segment(iv)%tang(:,jmat)
                    ENDIF
                  ENDIF
                  jxmin=MAX(ix0-1,ix0-ix)
                  jxmax=MIN(1-jx0,mx -ix)
                  jymin=MAX(iy0-1,iy0-iy)
                  jymax=MIN(1-jy0,my -iy)
c-----------------------------------------------------------------------
c                 find the vector elements of M.u then subtract 
c                 M.uu and (and M.zz for specified tangent) from M
c-----------------------------------------------------------------------
                  jq0=(jmat-1)*clim+1
                  DO jy=jymin,jymax
                    ijy=iy+jy
                    DO jx=jxmin,jxmax
                      ijx=ix+jx
                      IF (component(1:6)=='tangen')
     $                  rmat(jq0+2,-jx,-jy,:,ijx,ijy)=0
                      product=rmat(jq0  ,-jx,-jy,:,ijx,ijy)*unit(1)
     $                       +rmat(jq0+1,-jx,-jy,:,ijx,ijy)*unit(2)
                      rmat(jq0  ,-jx,-jy,:,ijx,ijy)=
     $                   rmat(jq0  ,-jx,-jy,:,ijx,ijy)-product*unit(1)
                      rmat(jq0+1,-jx,-jy,:,ijx,ijy)=
     $                   rmat(jq0+1,-jx,-jy,:,ijx,ijy)-product*unit(2)
                    ENDDO
                  ENDDO
                ENDDO
              ENDDO
              DEALLOCATE(product)
            ENDDO
c-----------------------------------------------------------------------
c           eliminate couplings to the specified component only.
c-----------------------------------------------------------------------
            DO itype=ibase_st,MIN(mat%rbl_mat(ibl)%nbtype,3_i4)
              ix0=mat%rbl_mat(ibl)%ix0(itype)
              iy0=mat%rbl_mat(ibl)%iy0(itype)
              DO jtype=1,mat%rbl_mat(ibl)%nbtype
                rmat=>mat%rbl_mat(ibl)%mat(jtype,itype)%arr
                jx0=mat%rbl_mat(ibl)%ix0(jtype)
                jy0=mat%rbl_mat(ibl)%iy0(jtype)
                ALLOCATE(product(mat%rbl_mat(ibl)%nq_type(jtype)))
                DO imat=1,mat%rbl_mat(ibl)%nb_type(itype)
                  IF (itype==1) THEN
                    IF (.NOT.seam(ibl)%expoint(iv)) CYCLE
                    ix=seam(ibl)%vertex(iv)%intxy(1)
                    iy=seam(ibl)%vertex(iv)%intxy(2)
                    IF (component(1:6)=='normal') THEN
                      unit=seam(ibl)%vertex(iv)%norm
                    ELSE
                      unit=seam(ibl)%vertex(iv)%tang
                    ENDIF
                  ELSE
                    IF (.NOT.(seam(ibl)%expoint(iv).AND.
     $                        seam(ibl)%expoint(ivp))) CYCLE
                    IF (seam(ibl)%segment(iv)%h_side) THEN
                      IF (itype>2) CYCLE
                    ELSE
                      IF (itype<3) CYCLE
                    ENDIF
                    ix=seam(ibl)%segment(iv)%intxys(1)
                    iy=seam(ibl)%segment(iv)%intxys(2)
                    IF (component(1:6)=='normal') THEN
                      unit=seam(ibl)%segment(iv)%norm(:,imat)
                    ELSE
                      unit=seam(ibl)%segment(iv)%tang(:,imat)
                    ENDIF
                  ENDIF
                  jxmin=MAX(jx0-1,jx0-ix)
                  jxmax=MIN(1-ix0,mx -ix)
                  jymin=MAX(jy0-1,jy0-iy)
                  jymax=MIN(1-iy0,my -iy)
c-----------------------------------------------------------------------
c                 find the vector elements of u.M then subtract 
c                 uu.M and (and zz.M for specified tangent) from M
c-----------------------------------------------------------------------
                  iq0=(imat-1)*clim+1
                  DO jy=jymin,jymax
                    DO jx=jxmin,jxmax
                      IF (component(1:6)=='tangen')
     $                  rmat(:,jx,jy,iq0+2,ix,iy)=0
                      product=unit(1)*rmat(:,jx,jy,iq0  ,ix,iy)
     $                       +unit(2)*rmat(:,jx,jy,iq0+1,ix,iy)
                      rmat(:,jx,jy,iq0  ,ix,iy)=
     $                   rmat(:,jx,jy,iq0  ,ix,iy)-unit(1)*product
                      rmat(:,jx,jy,iq0+1,ix,iy)=
     $                   rmat(:,jx,jy,iq0+1,ix,iy)-unit(2)*product
                    ENDDO
                  ENDDO
c-----------------------------------------------------------------------
c                 take care of uu and zz.
c-----------------------------------------------------------------------
                  IF (jtype==itype.AND.diag) THEN
                    DO jq=0,1
                      rmat(iq0:iq0+1,0,0,iq0+jq,ix,iy)=
     $                   rmat(iq0:iq0+1,0,0,iq0+jq,ix,iy)
     $                  +unit*unit(jq+1)*dscale
                    ENDDO
                    IF (component(1:6)=='tangen')
     $                rmat(iq0+2,0,0,iq0+2,ix,iy)=dscale
                  ENDIF
                ENDDO
                DEALLOCATE(product)
              ENDDO
            ENDDO
          ENDDO rb_vert
c-----------------------------------------------------------------------
c       begin tblock-specific coding.
c-----------------------------------------------------------------------
        ELSE block_type
          tb_vert: DO iv=1,nv
            IF (.NOT.seam(ibl)%expoint(iv)) CYCLE tb_vert
            ix=seam(ibl)%vertex(iv)%intxy(1)
            iy=seam(ibl)%vertex(iv)%intxy(2)
            tmat=>mat%tbl_mat(ibl)%lmat
c-----------------------------------------------------------------------
c           eliminate all couplings if this is an external corner with
c           tangent specified or if all components are zeroed.  for
c           specified normal at a corner, zero all but phi component.
c-----------------------------------------------------------------------
            IF (component(1:3)=='all') THEN
              tmat(ix)%element=0
              DO ip=0,SIZE(tmat(ix)%from_vert)-1
                ixn=tmat(ix)%from_vert(ip)
                DO ipn=0,SIZE(tmat(ixn)%from_vert)-1
                  IF (tmat(ixn)%from_vert(ipn)==ix)
     $              tmat(ixn)%element(:,:,ipn)=0
                ENDDO
              ENDDO
              IF (diag) THEN
                DO jq=1,clim
                  tmat(ix)%element(jq,jq,0)=dscale
                ENDDO
              ENDIF
              CYCLE tb_vert
            ELSE IF (seam(ibl)%excorner(iv)) THEN
              tmat(ix)%element(:,1:elim,:)=0
              DO ip=0,SIZE(tmat(ix)%from_vert)-1
                ixn=tmat(ix)%from_vert(ip)
                DO ipn=0,SIZE(tmat(ixn)%from_vert)-1
                  IF (tmat(ixn)%from_vert(ipn)==ix)
     $              tmat(ixn)%element(1:elim,:,ipn)=0
                ENDDO
              ENDDO
              IF (diag) THEN
                DO jq=1,elim
                  tmat(ix)%element(jq,jq,0)=dscale
                ENDDO
              ENDIF
              CYCLE tb_vert
            ENDIF
c-----------------------------------------------------------------------
c           otherwise, loop over couplings.
c-----------------------------------------------------------------------
            IF (component(1:6)=='normal') THEN
              unit=seam(ibl)%vertex(iv)%norm
            ELSE
              unit=seam(ibl)%vertex(iv)%tang
            ENDIF
            ALLOCATE(product(SIZE(tmat(ix)%element,1)))
c-----------------------------------------------------------------------
c           find the vector elements of M.u then subtract 
c           M.uu and (and M.zz for specified tangent) from M
c-----------------------------------------------------------------------
            DO ip=0,SIZE(tmat(ix)%from_vert)-1
              ixn=tmat(ix)%from_vert(ip)
              DO ipn=0,SIZE(tmat(ixn)%from_vert)-1
                IF (tmat(ixn)%from_vert(ipn)==ix) THEN
                  IF (component(1:6)=='tangen') THEN
                    tmat(ixn)%element(3,:,ipn)=0
                  ENDIF
                  product=tmat(ixn)%element(1,:,ipn)*unit(1)
     $                   +tmat(ixn)%element(2,:,ipn)*unit(2)
                  tmat(ixn)%element(1,:,ipn)=
     $               tmat(ixn)%element(1,:,ipn)-product*unit(1)
                  tmat(ixn)%element(2,:,ipn)=
     $               tmat(ixn)%element(2,:,ipn)-product*unit(2)
                ENDIF
              ENDDO
            ENDDO
c-----------------------------------------------------------------------
c           find the vector elements of u.M then subtract 
c           uu.M and (and zz.M for specified tangent) from M
c-----------------------------------------------------------------------
            DO ip=0,SIZE(tmat(ix)%from_vert)-1
              IF (component(1:6)=='tangen') tmat(ix)%element(:,3,ip)=0
              product=unit(1)*tmat(ix)%element(:,1,ip)
     $               +unit(2)*tmat(ix)%element(:,2,ip)
              tmat(ix)%element(:,1,ip)=
     $           tmat(ix)%element(:,1,ip)-unit(1)*product
              tmat(ix)%element(:,2,ip)=
     $           tmat(ix)%element(:,2,ip)-unit(2)*product
            ENDDO
c-----------------------------------------------------------------------
c           take care of uu and zz.
c-----------------------------------------------------------------------
            IF (diag) THEN
              DO jq=1,2
                tmat(ix)%element(1:2,jq,0)=
     $             tmat(ix)%element(1:2,jq,0)+unit*unit(jq)*dscale
              ENDDO
              IF (component(1:6)=='tangen')
     $          tmat(ix)%element(3,3,0)=dscale
            ENDIF
            DEALLOCATE(product)
          ENDDO tb_vert
        ENDIF block_type
      ENDDO block
c-----------------------------------------------------------------------
c     terminate.
c-----------------------------------------------------------------------
      RETURN
      END SUBROUTINE dirichlet_op
c-----------------------------------------------------------------------
c     subprogram 4. no_mat_bc.
c     a dummy boundary condition routine with the same interface
c     block as dirichlet_op (and future boundary condition routines).
c-----------------------------------------------------------------------
      SUBROUTINE no_mat_bc(mat,component,dscale,symm)
      USE seam_storage_mod
      USE matrix_type_mod

      TYPE(global_matrix_type), INTENT(INOUT) :: mat
      CHARACTER(*), INTENT(IN) :: component
      REAL(r8), INTENT(IN) :: dscale
      CHARACTER(*), INTENT(IN), OPTIONAL :: symm

      RETURN
      END SUBROUTINE no_mat_bc
c-----------------------------------------------------------------------
c     subprogram 5. dirichlet_comp_op.
c     apply Dirichlet boundary conditions to a Cartesian operator.
c     If the specified component is tangent, the resultant matrix is 
c     (I-tt-zz).M.(I-tt-zz)+tt+zz, where t is the surface tangent in 
c     the computational plane, z is the unit vector normal to the plane,
c     and I is the identity matrix.  If the specified component is 
c     normal, the resultant matrix is (I-nn).M.(I-nn)+nn, where n is 
c     the surface normal.  If the input is all, then the result is
c     (I-tt-zz-nn).M.(I-tt-zz-nn)+tt+zz+nn.
c
c     this version handles complex 3-vectors directly.
c
c     when the end of the component parameter is 'offdiag,' the passed
c     matrix structure contains only off-diagonal matrix entries, and
c     there is no diagonal entry to enter after couplings are
c     eliminated.
c
c     capability for 6-vectors in rblocks, i.e. two 3-vectors solved
c     together, is reinstalled.
c-----------------------------------------------------------------------
      SUBROUTINE dirichlet_comp_op(mat,component,dscale)
      USE seam_storage_mod
      USE matrix_type_mod

      TYPE(complex_matrix_type), INTENT(INOUT) :: mat
      CHARACTER(*), INTENT(IN) :: component
      REAL(r8), INTENT(IN) :: dscale

      INTEGER(i4) :: iv,nv,ip,ipn,ibl,ibe,ibv,jxmin,jxmax,jymin,jymax,
     $               jx,jy,ix,iy,ijx,ijy,mx,my,iq,jq,clim,elim,ixn,
     $               pd,pd2,imat,jmat,ivp,ix0,iy0,jx0,jy0,ibase_st,
     $               itype,jtype,iq0,iq1,jq0,jq1,pdm1,is,js,lcomp
      REAL(r8), DIMENSION(2) :: unit
      COMPLEX(r8), DIMENSION(:), ALLOCATABLE :: product
      COMPLEX(r8), DIMENSION(:,:,:,:,:,:), POINTER :: rmat
      TYPE(comp_matrix_element_type3), DIMENSION(:), POINTER :: tmat
      LOGICAL :: diag
c-----------------------------------------------------------------------
c     check vector component range.
c-----------------------------------------------------------------------
      IF (SIZE(mat%rbl_mat)>0) THEN
        clim=SIZE(mat%rbl_mat(1)%mat(1,1)%arr,1)
      ELSE
        clim=SIZE(mat%tbl_mat(1)%lmat(0)%element,1)
      ENDIF
      IF (MODULO(clim,3_i4)/=0.AND.component(1:3)/='all') CALL nim_stop
     $ ('Dirichlet_comp_op '//component//' requires (r,z,phi)-vectors.')
c-----------------------------------------------------------------------
c     save the diagonal scaling and essential-condition flag.
c-----------------------------------------------------------------------
      mat%diag_scale=dscale
      mat%essential_cond=component
c-----------------------------------------------------------------------
c     set vector index limits.
c-----------------------------------------------------------------------
      lcomp=LEN_TRIM(component)
      elim=3
      IF (lcomp>=6) THEN
        IF (component(1:6)=='normal') elim=2
      ENDIF
c-----------------------------------------------------------------------
c     check if this matrix is entirely off-diagonal.
c-----------------------------------------------------------------------
      diag=.true.
      IF (lcomp>6) THEN
        IF (component(lcomp-6:lcomp)=='offdiag') diag=.false.
      ENDIF
c-----------------------------------------------------------------------
c     loop over all external boundary points and zero the couplings
c     to the specified components along the boundary.
c-----------------------------------------------------------------------
      block: DO ibe=1,SIZE(exblock_list)
        ibl=exblock_list(ibe)
        nv=seam(ibl)%nvert
c-----------------------------------------------------------------------
c       begin rblock-specific coding.  rblock basis types indices are
c       1 == grid vertex centered
c       2 == horizontal side centered
c       3 == vertical side centered
c       4 == interior centered
c       1 : 3 are affected by boundary conditions.
c       note that itype is the to basis type and jtype is the from
c       basis type.
c-----------------------------------------------------------------------
        block_type: IF (ibl<=SIZE(mat%rbl_mat)) THEN
c-----------------------------------------------------------------------
c         connections to the rblock boundary vertices.
c-----------------------------------------------------------------------
          IF (mat%rbl_mat(ibl)%nbtype>1) THEN
            pd=SUM(mat%rbl_mat(ibl)%nb_type(1:2))
          ELSE
            pd=1
          ENDIF
          pdm1=pd-1
          rb_vert: DO iv=1,nv
            ivp=iv-1
            IF (ivp==0) ivp=seam(ibl)%nvert
            mx=SIZE(mat%rbl_mat(ibl)%mat(1,1)%arr,5)-1
            my=SIZE(mat%rbl_mat(ibl)%mat(1,1)%arr,6)-1
c-----------------------------------------------------------------------
c           eliminate all couplings if all components are zeroed.
c-----------------------------------------------------------------------
            IF (component(1:3)=='all') THEN
              DO itype=1,mat%rbl_mat(ibl)%nbtype
                ix0=mat%rbl_mat(ibl)%ix0(itype)
                iy0=mat%rbl_mat(ibl)%iy0(itype)
                DO jtype=1,MIN(mat%rbl_mat(ibl)%nbtype,3_i4)
                  IF (jtype==1) THEN
                    IF (.NOT.seam(ibl)%expoint(iv)) CYCLE
                    ix=seam(ibl)%vertex(iv)%intxy(1)
                    iy=seam(ibl)%vertex(iv)%intxy(2)
                  ELSE
                    IF (.NOT.(seam(ibl)%expoint(iv).AND.
     $                        seam(ibl)%expoint(ivp))) CYCLE
                    IF (seam(ibl)%segment(iv)%h_side.AND.jtype>2) CYCLE
                    IF (.NOT.seam(ibl)%segment(iv)%h_side.AND.
     $                  jtype<3) CYCLE
                    ix=seam(ibl)%segment(iv)%intxys(1)
                    iy=seam(ibl)%segment(iv)%intxys(2)
                  ENDIF
                  rmat=>mat%rbl_mat(ibl)%mat(jtype,itype)%arr
                  jx0=mat%rbl_mat(ibl)%ix0(jtype)
                  jy0=mat%rbl_mat(ibl)%iy0(jtype)
                  jxmin=MAX(ix0-1,ix0-ix)
                  jxmax=MIN(1-jx0,mx -ix)
                  jymin=MAX(iy0-1,iy0-iy)
                  jymax=MIN(1-jy0,my -iy)
                  DO jy=jymin,jymax
                    ijy=iy+jy
                    DO jx=jxmin,jxmax
                      ijx=ix+jx
                      rmat(:,-jx,-jy,:,ijx,ijy)=0
                    ENDDO
                  ENDDO
                ENDDO
              ENDDO
              DO itype=1,MIN(mat%rbl_mat(ibl)%nbtype,3_i4)
                IF (itype==1) THEN
                  IF (.NOT.seam(ibl)%expoint(iv)) CYCLE
                  ix=seam(ibl)%vertex(iv)%intxy(1)
                  iy=seam(ibl)%vertex(iv)%intxy(2)
                ELSE
                  IF (.NOT.(seam(ibl)%expoint(iv).AND.
     $                      seam(ibl)%expoint(ivp))) CYCLE
                  IF (seam(ibl)%segment(iv)%h_side.AND.itype>2) CYCLE
                  IF (.NOT.seam(ibl)%segment(iv)%h_side.AND.
     $                itype<3) CYCLE
                  ix=seam(ibl)%segment(iv)%intxys(1)
                  iy=seam(ibl)%segment(iv)%intxys(2)
                ENDIF
                DO jtype=1,mat%rbl_mat(ibl)%nbtype
                  rmat=>mat%rbl_mat(ibl)%mat(jtype,itype)%arr
                  rmat(:,:,:,:,ix,iy)=0
                  IF (jtype==itype.AND.diag) THEN
                    DO jq=1,mat%rbl_mat(ibl)%nq_type(itype)
                      rmat(jq,0,0,jq,ix,iy)=dscale
                    ENDDO
                  ENDIF
                ENDDO
              ENDDO
              CYCLE rb_vert
            ENDIF
c-----------------------------------------------------------------------
c           eliminate couplings to and from an external corner,
c           where only the grid vertex basis is appropriate.
c           for specified normal at a corner, zero all but phi
c           component.
c-----------------------------------------------------------------------
            IF (seam(ibl)%excorner(iv)) THEN
              ix=seam(ibl)%vertex(iv)%intxy(1)
              iy=seam(ibl)%vertex(iv)%intxy(2)
              DO itype=1,mat%rbl_mat(ibl)%nbtype
                rmat=>mat%rbl_mat(ibl)%mat(1,itype)%arr
                ix0=mat%rbl_mat(ibl)%ix0(itype)
                iy0=mat%rbl_mat(ibl)%iy0(itype)
                jxmin=MAX(ix0-1,ix0-ix)
                jxmax=MIN(1    ,mx -ix)
                jymin=MAX(iy0-1,iy0-iy)
                jymax=MIN(1    ,my -iy)
                DO jy=jymin,jymax
                  ijy=iy+jy
                  DO jx=jxmin,jxmax
                    ijx=ix+jx
                    rmat(1:elim,-jx,-jy,:,ijx,ijy)=0
                    IF (clim==6_i4)
     $                rmat(4:3+elim,-jx,-jy,:,ijx,ijy)=0
                  ENDDO
                ENDDO
              ENDDO
              DO jtype=1,mat%rbl_mat(ibl)%nbtype
                rmat=>mat%rbl_mat(ibl)%mat(jtype,1)%arr
                rmat(:,:,:,1:elim,ix,iy)=0
                IF (jtype==1.AND.diag) THEN
                  DO jq=1,elim
                    rmat(jq,0,0,jq,ix,iy)=dscale
                  ENDDO
                ENDIF
                IF (clim==6_i4) THEN
                  rmat(:,:,:,4:3+elim,ix,iy)=0
                  IF (jtype==1.AND.diag) THEN
                    DO jq=4,3+elim
                      rmat(jq,0,0,jq,ix,iy)=dscale
                    ENDDO
                  ENDIF
                ENDIF
              ENDDO
              ibase_st=2
            ELSE
              ibase_st=1
            ENDIF
c-----------------------------------------------------------------------
c           eliminate couplings from the specified component only.
c-----------------------------------------------------------------------
            DO itype=1,mat%rbl_mat(ibl)%nbtype
              ix0=mat%rbl_mat(ibl)%ix0(itype)
              iy0=mat%rbl_mat(ibl)%iy0(itype)
              ALLOCATE(product(mat%rbl_mat(ibl)%nq_type(itype)))
              DO jtype=ibase_st,MIN(mat%rbl_mat(ibl)%nbtype,3_i4)
                rmat=>mat%rbl_mat(ibl)%mat(jtype,itype)%arr
                jx0=mat%rbl_mat(ibl)%ix0(jtype)
                jy0=mat%rbl_mat(ibl)%iy0(jtype)
                DO jmat=1,mat%rbl_mat(ibl)%nb_type(jtype)
                  IF (jtype==1) THEN
                    IF (.NOT.seam(ibl)%expoint(iv)) CYCLE
                    ix=seam(ibl)%vertex(iv)%intxy(1)
                    iy=seam(ibl)%vertex(iv)%intxy(2)
                    IF (component(1:6)=='normal') THEN
                      unit=seam(ibl)%vertex(iv)%norm
                    ELSE
                      unit=seam(ibl)%vertex(iv)%tang
                    ENDIF
                  ELSE
                    IF (.NOT.(seam(ibl)%expoint(iv).AND.
     $                        seam(ibl)%expoint(ivp))) CYCLE
                    IF (seam(ibl)%segment(iv)%h_side) THEN
                      IF (jtype>2) CYCLE
                    ELSE
                      IF (jtype<3) CYCLE
                    ENDIF
                    ix=seam(ibl)%segment(iv)%intxys(1)
                    iy=seam(ibl)%segment(iv)%intxys(2)
                    IF (component(1:6)=='normal') THEN
                      unit=seam(ibl)%segment(iv)%norm(:,jmat)
                    ELSE
                      unit=seam(ibl)%segment(iv)%tang(:,jmat)
                    ENDIF
                  ENDIF
                  jxmin=MAX(ix0-1,ix0-ix)
                  jxmax=MIN(1-jx0,mx -ix)
                  jymin=MAX(iy0-1,iy0-iy)
                  jymax=MIN(1-jy0,my -iy)
c-----------------------------------------------------------------------
c                 find the vector elements of M.u then subtract 
c                 M.uu and (and M.zz for specified tangent) from M
c-----------------------------------------------------------------------
                  jq0=(jmat-1)*clim+1
                  DO jy=jymin,jymax
                    ijy=iy+jy
                    DO jx=jxmin,jxmax
                      ijx=ix+jx
                      IF (component(1:6)=='tangen')
     $                  rmat(jq0+2,-jx,-jy,:,ijx,ijy)=0
                      product=rmat(jq0  ,-jx,-jy,:,ijx,ijy)*unit(1)
     $                       +rmat(jq0+1,-jx,-jy,:,ijx,ijy)*unit(2)
                      rmat(jq0  ,-jx,-jy,:,ijx,ijy)=
     $                   rmat(jq0  ,-jx,-jy,:,ijx,ijy)-product*unit(1)
                      rmat(jq0+1,-jx,-jy,:,ijx,ijy)=
     $                   rmat(jq0+1,-jx,-jy,:,ijx,ijy)-product*unit(2)
                      IF (clim==6_i4) THEN
                        IF (component(1:6)=='tangen')
     $                    rmat(jq0+5,-jx,-jy,:,ijx,ijy)=0
                        product=rmat(jq0+3,-jx,-jy,:,ijx,ijy)*unit(1)
     $                         +rmat(jq0+4,-jx,-jy,:,ijx,ijy)*unit(2)
                        rmat(jq0+3,-jx,-jy,:,ijx,ijy)=
     $                     rmat(jq0+3,-jx,-jy,:,ijx,ijy)-product*unit(1)
                        rmat(jq0+4,-jx,-jy,:,ijx,ijy)=
     $                     rmat(jq0+4,-jx,-jy,:,ijx,ijy)-product*unit(2)
                      ENDIF
                    ENDDO
                  ENDDO
                ENDDO
              ENDDO
              DEALLOCATE(product)
            ENDDO
c-----------------------------------------------------------------------
c           eliminate couplings to the specified component only.
c-----------------------------------------------------------------------
            DO itype=ibase_st,MIN(mat%rbl_mat(ibl)%nbtype,3_i4)
              ix0=mat%rbl_mat(ibl)%ix0(itype)
              iy0=mat%rbl_mat(ibl)%iy0(itype)
              DO jtype=1,mat%rbl_mat(ibl)%nbtype
                rmat=>mat%rbl_mat(ibl)%mat(jtype,itype)%arr
                jx0=mat%rbl_mat(ibl)%ix0(jtype)
                jy0=mat%rbl_mat(ibl)%iy0(jtype)
                ALLOCATE(product(mat%rbl_mat(ibl)%nq_type(jtype)))
                DO imat=1,mat%rbl_mat(ibl)%nb_type(itype)
                  IF (itype==1) THEN
                    IF (.NOT.seam(ibl)%expoint(iv)) CYCLE
                    ix=seam(ibl)%vertex(iv)%intxy(1)
                    iy=seam(ibl)%vertex(iv)%intxy(2)
                    IF (component(1:6)=='normal') THEN
                      unit=seam(ibl)%vertex(iv)%norm
                    ELSE
                      unit=seam(ibl)%vertex(iv)%tang
                    ENDIF
                  ELSE
                    IF (.NOT.(seam(ibl)%expoint(iv).AND.
     $                        seam(ibl)%expoint(ivp))) CYCLE
                    IF (seam(ibl)%segment(iv)%h_side) THEN
                      IF (itype>2) CYCLE
                    ELSE
                      IF (itype<3) CYCLE
                    ENDIF
                    ix=seam(ibl)%segment(iv)%intxys(1)
                    iy=seam(ibl)%segment(iv)%intxys(2)
                    IF (component(1:6)=='normal') THEN
                      unit=seam(ibl)%segment(iv)%norm(:,imat)
                    ELSE
                      unit=seam(ibl)%segment(iv)%tang(:,imat)
                    ENDIF
                  ENDIF
                  jxmin=MAX(jx0-1,jx0-ix)
                  jxmax=MIN(1-ix0,mx -ix)
                  jymin=MAX(jy0-1,jy0-iy)
                  jymax=MIN(1-iy0,my -iy)
c-----------------------------------------------------------------------
c                 find the vector elements of u.M then subtract 
c                 uu.M and (and zz.M for specified tangent) from M
c-----------------------------------------------------------------------
                  iq0=(imat-1)*clim+1
                  DO jy=jymin,jymax
                    DO jx=jxmin,jxmax
                      IF (component(1:6)=='tangen')
     $                  rmat(:,jx,jy,iq0+2,ix,iy)=0
                      product=unit(1)*rmat(:,jx,jy,iq0  ,ix,iy)
     $                       +unit(2)*rmat(:,jx,jy,iq0+1,ix,iy)
                      rmat(:,jx,jy,iq0  ,ix,iy)=
     $                   rmat(:,jx,jy,iq0  ,ix,iy)-unit(1)*product
                      rmat(:,jx,jy,iq0+1,ix,iy)=
     $                   rmat(:,jx,jy,iq0+1,ix,iy)-unit(2)*product
                      IF (clim==6_i4) THEN
                        IF (component(1:6)=='tangen')
     $                    rmat(:,jx,jy,iq0+5,ix,iy)=0
                        product=unit(1)*rmat(:,jx,jy,iq0+3,ix,iy)
     $                         +unit(2)*rmat(:,jx,jy,iq0+4,ix,iy)
                        rmat(:,jx,jy,iq0+3,ix,iy)=
     $                     rmat(:,jx,jy,iq0+3,ix,iy)-unit(1)*product
                        rmat(:,jx,jy,iq0+4,ix,iy)=
     $                     rmat(:,jx,jy,iq0+4,ix,iy)-unit(2)*product
                      ENDIF
                    ENDDO
                  ENDDO
c-----------------------------------------------------------------------
c                 take care of uu and zz.
c-----------------------------------------------------------------------
                  IF (jtype==itype.AND.diag) THEN
                    DO jq=0,1
                      rmat(iq0:iq0+1,0,0,iq0+jq,ix,iy)=
     $                   rmat(iq0:iq0+1,0,0,iq0+jq,ix,iy)
     $                  +unit*unit(jq+1)*dscale
                    ENDDO
                    IF (component(1:6)=='tangen')
     $                rmat(iq0+2,0,0,iq0+2,ix,iy)=dscale
                    IF (clim==6_i4) THEN
                      DO jq=3,4
                        rmat(iq0+3:iq0+4,0,0,iq0+jq,ix,iy)=
     $                     rmat(iq0+3:iq0+4,0,0,iq0+jq,ix,iy)
     $                    +unit*unit(jq-2)*dscale
                      ENDDO
                      IF (component(1:6)=='tangen')
     $                  rmat(iq0+5,0,0,iq0+5,ix,iy)=dscale
                    ENDIF
                  ENDIF
                ENDDO
                DEALLOCATE(product)
              ENDDO
            ENDDO
          ENDDO rb_vert
c-----------------------------------------------------------------------
c       begin tblock-specific coding.
c-----------------------------------------------------------------------
        ELSE block_type
          tb_vert: DO iv=1,nv
            IF (.NOT.seam(ibl)%expoint(iv)) CYCLE tb_vert
            ix=seam(ibl)%vertex(iv)%intxy(1)
            iy=seam(ibl)%vertex(iv)%intxy(2)
            tmat=>mat%tbl_mat(ibl)%lmat
c-----------------------------------------------------------------------
c           eliminate all couplings if this is an external corner with
c           tangent specified or if all components are zeroed.  for
c           specified normal at a corner, zero all but phi component.
c-----------------------------------------------------------------------
            IF (component(1:3)=='all') THEN
              tmat(ix)%element=0
              DO ip=0,SIZE(tmat(ix)%from_vert)-1
                ixn=tmat(ix)%from_vert(ip)
                DO ipn=0,SIZE(tmat(ixn)%from_vert)-1
                  IF (tmat(ixn)%from_vert(ipn)==ix)
     $              tmat(ixn)%element(:,:,ipn)=0
                ENDDO
              ENDDO
              IF (diag) THEN
                DO jq=1,clim
                  tmat(ix)%element(jq,jq,0)=dscale
                ENDDO
              ENDIF
              CYCLE tb_vert
            ELSE IF (seam(ibl)%excorner(iv)) THEN
              tmat(ix)%element(:,1:elim,:)=0
              DO ip=0,SIZE(tmat(ix)%from_vert)-1
                ixn=tmat(ix)%from_vert(ip)
                DO ipn=0,SIZE(tmat(ixn)%from_vert)-1
                  IF (tmat(ixn)%from_vert(ipn)==ix)
     $              tmat(ixn)%element(1:elim,:,ipn)=0
                ENDDO
              ENDDO
              IF (diag) THEN
                DO jq=1,elim
                  tmat(ix)%element(jq,jq,0)=dscale
                ENDDO
              ENDIF
              CYCLE tb_vert
            ENDIF
c-----------------------------------------------------------------------
c           otherwise, loop over couplings.
c-----------------------------------------------------------------------
            IF (component(1:6)=='normal') THEN
              unit=seam(ibl)%vertex(iv)%norm
            ELSE
              unit=seam(ibl)%vertex(iv)%tang
            ENDIF
            ALLOCATE(product(SIZE(tmat(ix)%element,1)))
c-----------------------------------------------------------------------
c           find the vector elements of M.u then subtract 
c           M.uu and (and M.zz for specified tangent) from M
c-----------------------------------------------------------------------
            DO ip=0,SIZE(tmat(ix)%from_vert)-1
              ixn=tmat(ix)%from_vert(ip)
              DO ipn=0,SIZE(tmat(ixn)%from_vert)-1
                IF (tmat(ixn)%from_vert(ipn)==ix) THEN
                  IF (component(1:6)=='tangen') THEN
                    tmat(ixn)%element(3,:,ipn)=0
                  ENDIF
                  product=tmat(ixn)%element(1,:,ipn)*unit(1)
     $                   +tmat(ixn)%element(2,:,ipn)*unit(2)
                  tmat(ixn)%element(1,:,ipn)=
     $               tmat(ixn)%element(1,:,ipn)-product*unit(1)
                  tmat(ixn)%element(2,:,ipn)=
     $               tmat(ixn)%element(2,:,ipn)-product*unit(2)
                ENDIF
              ENDDO
            ENDDO
c-----------------------------------------------------------------------
c           find the vector elements of u.M then subtract 
c           uu.M and (and zz.M for specified tangent) from M
c-----------------------------------------------------------------------
            DO ip=0,SIZE(tmat(ix)%from_vert)-1
              IF (component(1:6)=='tangen') tmat(ix)%element(:,3,ip)=0
              product=unit(1)*tmat(ix)%element(:,1,ip)
     $               +unit(2)*tmat(ix)%element(:,2,ip)
              tmat(ix)%element(:,1,ip)=
     $           tmat(ix)%element(:,1,ip)-unit(1)*product
              tmat(ix)%element(:,2,ip)=
     $           tmat(ix)%element(:,2,ip)-unit(2)*product
            ENDDO
c-----------------------------------------------------------------------
c           take care of uu and zz.
c-----------------------------------------------------------------------
            IF (diag) THEN
              DO jq=1,2
                tmat(ix)%element(1:2,jq,0)=
     $             tmat(ix)%element(1:2,jq,0)+unit*unit(jq)*dscale
              ENDDO
              IF (component(1:6)=='tangen')
     $          tmat(ix)%element(3,3,0)=dscale
            ENDIF
            DEALLOCATE(product)
          ENDDO tb_vert
        ENDIF block_type
      ENDDO block
c-----------------------------------------------------------------------
c     terminate.
c-----------------------------------------------------------------------
      RETURN
      END SUBROUTINE dirichlet_comp_op
c-----------------------------------------------------------------------
c     subprogram 6. no_comp_mat_bc.
c     a dummy boundary condition routine with the same interface
c     block as dirichlet_op (and future boundary condition routines).
c-----------------------------------------------------------------------
      SUBROUTINE no_comp_mat_bc(mat,component,dscale)
      USE seam_storage_mod
      USE matrix_type_mod

      TYPE(complex_matrix_type), INTENT(INOUT) :: mat
      CHARACTER(*), INTENT(IN) :: component
      REAL(r8), INTENT(IN) :: dscale

      RETURN
      END SUBROUTINE no_comp_mat_bc
c-----------------------------------------------------------------------
c     close module.
c-----------------------------------------------------------------------
      END MODULE boundary
