!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
       MODULE DAGMG_MUMPS_STRUC_DF
!
!
      TYPE DAGMG_MUMPS_ROOT_STRUC
        SEQUENCE
        INTEGER MBLOCK, NBLOCK, NPROW, NPCOL
        INTEGER MYROW, MYCOL
        INTEGER ROOT_SIZE, TOT_ROOT_SIZE
        INTEGER :: CNTXT_BLACS, truc
        INTEGER, DIMENSION(:), POINTER :: RG2L_ROW
        INTEGER, DIMENSION(:), POINTER :: RG2L_COL
        INTEGER , DIMENSION(:), POINTER :: IPIV
        INTEGER, DIMENSION( 9 ) :: DESCRIPTOR, DESCB
        LOGICAL yes, gridinit_done
        INTEGER LPIV, brol
!       Used to access Schur easily from root structure
        DOUBLE PRECISION, DIMENSION(:), POINTER :: SCHUR_POINTER
        INTEGER SCHUR_MLOC, SCHUR_NLOC, SCHUR_LLD, machin
!
!      Data for nullspace/QR
!
        DOUBLE PRECISION, DIMENSION(:), POINTER :: QR_TAU
        DOUBLE PRECISION     QR_RCOND
!
!      Givens rotations
!
        INTEGER MAXG, GIND
        DOUBLE PRECISION, DIMENSION(:),POINTER::GROW, GCOS, GSIN
!
!      RRRLU data
!
        INTEGER ELG_MAX,NULL_MAX
        INTEGER ELIND,EUIND,NLUPDATE,NUUPDATE
        INTEGER,DIMENSION(:),POINTER::PERM_ROW,PERM_COL
        INTEGER,DIMENSION(:),POINTER::ELROW, EUROW, PTREL, PTREU
        DOUBLE PRECISION, DIMENSION(:), POINTER :: ELELG, EUELG, DL
!
      END TYPE DAGMG_MUMPS_ROOT_STRUC
      TYPE DAGMG_MUMPS_STRUC
        SEQUENCE
!
! This structure contains all parameters
! for the interface to the user, plus internal
! information
!
! *****************
! INPUT PARAMETERS
! *****************
!    -----------------
!    MPI Communicator
!    -----------------
        INTEGER COMM
!    ------------------
!    Problem definition
!    ------------------
!    Solver (SYM=0 unsymmetric,SYM=1 symmetric Positive Definite,
!        SYM=2 general symmetric)
!    Type of parallelism (PAR=1 host working, PAR=0 host not working)
        INTEGER SYM, PAR
        INTEGER JOB
!    --------------------
!    Order of Input matrix
!    --------------------
        INTEGER N
!
!    ----------------------------------------
!    Assembled input matrix : User interface
!    ----------------------------------------
        INTEGER NZ
        DOUBLE PRECISION, DIMENSION(:), POINTER :: A
        INTEGER, DIMENSION(:), POINTER :: IRN, JCN
        DOUBLE PRECISION, DIMENSION(:), POINTER :: COLSCA, ROWSCA, pad0
!
!       ------------------------------------
!       Case of distributed assembled matrix
!       matrix on entry:
!       ------------------------------------
        INTEGER NZ_loc, pad1
        INTEGER, DIMENSION(:), POINTER :: IRN_loc, JCN_loc
        DOUBLE PRECISION, DIMENSION(:), POINTER :: A_loc, pad2
!
!    ----------------------------------------
!    Unassembled input matrix: User interface
!    ----------------------------------------
        INTEGER NELT, pad3
        INTEGER, DIMENSION(:), POINTER :: ELTPTR
        INTEGER, DIMENSION(:), POINTER :: ELTVAR
        DOUBLE PRECISION, DIMENSION(:), POINTER :: A_ELT, pad4
!
!    ---------------------------------------------
!    Symmetric permutation :
!               PERM_IN if given by user (optional)
!    ---------------------------------------------
        INTEGER, DIMENSION(:), POINTER :: PERM_IN
!
!
! ******************
! INPUT/OUTPUT data
! ******************
!    --------------------------------------------------------
!    RHS / SOL_LOC
!    -------------
!       right-hand side and solution
!    -------------------------------------------------------
        DOUBLE PRECISION, DIMENSION(:), POINTER :: RHS, REDRHS
        DOUBLE PRECISION, DIMENSION(:), POINTER :: RHS_SPARSE
        DOUBLE PRECISION, DIMENSION(:), POINTER :: SOL_LOC
        INTEGER, DIMENSION(:), POINTER :: IRHS_SPARSE
        INTEGER, DIMENSION(:), POINTER :: IRHS_PTR
        INTEGER, DIMENSION(:), POINTER :: ISOL_LOC
        INTEGER LRHS, NRHS, NZ_RHS, LSOL_LOC, LREDRHS
        INTEGER pad5
!    ----------------------------
!    Control parameters,
!    statistics and output data
!    ---------------------------
        INTEGER ICNTL(40)
        INTEGER INFO(40)
        INTEGER INFOG(40)
        DOUBLE PRECISION COST_SUBTREES
        DOUBLE PRECISION CNTL(15)
        DOUBLE PRECISION RINFO(20)
        DOUBLE PRECISION RINFOG(20)
!    ---------------------------------------------------------
!    Permutations computed during analysis:
!       SYM_PERM: Symmetric permutation
!       UNS_PERM: Column permutations (optionnal)
!    ---------------------------------------------------------
        INTEGER, DIMENSION(:), POINTER :: SYM_PERM, UNS_PERM
!
!    -------------------------------------
!    Case of distributed matrix on entry:
!    DMUMPS potentially provides mapping
!    -------------------------------------
        INTEGER, DIMENSION(:), POINTER :: MAPPING
!
!    -------------------------------
!    Deficiency and null space basis
!    -------------------------------
        DOUBLE PRECISION, DIMENSION(:,:), POINTER :: NULL_SPACE
        INTEGER Deficiency, pad6
!    -----
!    Schur
!    -----
        INTEGER NPROW, NPCOL, MBLOCK, NBLOCK
        INTEGER SCHUR_MLOC, SCHUR_NLOC, SCHUR_LLD
        INTEGER SIZE_SCHUR
        INTEGER, DIMENSION(:), POINTER :: LISTVAR_SCHUR
        DOUBLE PRECISION, DIMENSION(:), POINTER :: SCHUR
        DOUBLE PRECISION, DIMENSION(:), POINTER :: SCHUR_CINTERFACE
!    --------------
!    Version number
!    --------------
        CHARACTER(LEN=16) VERSION_NUMBER
!    -----------
!    Out-of-core
!    -----------
        CHARACTER(LEN=256) :: OOC_TMPDIR
        CHARACTER(LEN=64) :: OOC_PREFIX
!    ------------------------------------------
!    To save the matrix in matrix market format
!    ------------------------------------------
        CHARACTER(LEN=256) WRITE_PROBLEM
!
!
! **********************
! INTERNAL Working data
! *********************
        INTEGER INST_Number
!       For MPI
        INTEGER COMM_NODES, MYID_NODES, COMM_LOAD
        INTEGER  MYID, NPROCS, NSLAVES
        INTEGER ASS_IRECV
        INTEGER, DIMENSION(:), POINTER :: POIDS
        INTEGER LBUFR
        INTEGER LBUFR_BYTES
        INTEGER, DIMENSION(:), POINTER ::  BUFR
!       For analysis/facto/solve phases
        INTEGER MAXIS1, pad7
        INTEGER KEEP(500)
        INTEGER*8 KEEP8(150)
!       IS is used for the factors + workspace for contrib. blocks
        INTEGER, DIMENSION(:), POINTER :: IS
!       is1 (maxis1) contains working arrays computed
!       and used only during analysis
        INTEGER, DIMENSION(:), POINTER :: IS1
!       The following data/arrays are computed during the analysis
!       phase and used during the factorization and solve phases.
        INTEGER LNA
        INTEGER NBSA
        INTEGER,POINTER,DIMENSION(:)::STEP, NE_STEPS, ND_STEPS
!  Info for pruning tree
        INTEGER,POINTER,DIMENSION(:)::Step2node
!  ---------------------
        INTEGER,POINTER,DIMENSION(:)::FRERE_STEPS, DAD_STEPS
        INTEGER,POINTER,DIMENSION(:)::FILS, PTRAR, FRTPTR, FRTELT
        INTEGER,POINTER,DIMENSION(:)::NA, PROCNODE_STEPS
!       The two pointer arrays computed in facto and used by the solve
!          (except the factors) are PTLUST_S and PTRFAC.
        INTEGER, DIMENSION(:), POINTER :: PTLUST_S
        INTEGER(8), DIMENSION(:), POINTER :: PTRFAC
!       main real working arrays for factorization/solve phases
        DOUBLE PRECISION, DIMENSION(:), POINTER :: S
!       Information on mapping
        INTEGER, DIMENSION(:), POINTER :: PROCNODE
!       Input matrix ready for numerical assembly
!           -arrowhead format in case of assembled matrix
!           -element format otherwise
        INTEGER, DIMENSION(:), POINTER :: INTARR
        DOUBLE PRECISION, DIMENSION(:), POINTER :: DBLARR
!       Element entry: internal data
        INTEGER NELT_LOC, LELTVAR, NA_ELT, pad8
        INTEGER, DIMENSION(:), POINTER :: ELTPROC
!       Candidates and node partitionning
        INTEGER, DIMENSION(:,:), POINTER :: CANDIDATES
        INTEGER, DIMENSION(:),   POINTER :: ISTEP_TO_INIV2
        INTEGER, DIMENSION(:),   POINTER :: FUTURE_NIV2
        INTEGER, DIMENSION(:,:), POINTER :: TAB_POS_IN_PERE
        LOGICAL, DIMENSION(:), POINTER :: I_AM_CAND
!       For heterogeneous architecture
        INTEGER, DIMENSION(:), POINTER :: MEM_DIST
!       Compressed RHS
        INTEGER, DIMENSION(:),   POINTER :: POSINRHSCOMP
        DOUBLE PRECISION, DIMENSION(:), POINTER :: RHSCOMP
!       For C interface
!   Info on the subtrees to be used during factorization
        DOUBLE PRECISION, DIMENSION(:),   POINTER :: MEM_SUBTREE
        INTEGER, DIMENSION(:),   POINTER :: MY_ROOT_SBTR
        INTEGER, DIMENSION(:),   POINTER :: MY_FIRST_LEAF
        INTEGER, DIMENSION(:),   POINTER :: MY_NB_LEAF
        INTEGER, DIMENSION(:),   POINTER :: DEPTH_FIRST
        DOUBLE PRECISION, DIMENSION(:),   POINTER :: COST_TRAV
        INTEGER NBSA_LOCAL, zwave
        INTEGER(8) :: MAX_SURF_MASTER
        INTEGER :: LWK_USER, zozo
        DOUBLE PRECISION, DIMENSION(:), POINTER :: WK_USER
!    For simulating parallel out-of-core stack.
        DOUBLE PRECISION, DIMENSION(:),POINTER ::CB_SON_SIZE
!   Instance number used/managed by the C/F77 interface
        INTEGER INSTANCE_NUMBER
!    OOC management data that must persist from factorization to solve.
        INTEGER OOC_MAX_NB_NODES_FOR_ZONE
        INTEGER, DIMENSION(:,:),   POINTER :: OOC_INODE_SEQUENCE
        INTEGER(8),DIMENSION(:,:), POINTER :: OOC_SIZE_OF_BLOCK
        INTEGER*8, DIMENSION(:,:),   POINTER :: OOC_VADDR
        INTEGER,DIMENSION(:), POINTER :: OOC_TOTAL_NB_NODES
        INTEGER,DIMENSION(:), POINTER :: OOC_NB_FILES
        CHARACTER,DIMENSION(:,:), POINTER :: OOC_FILE_NAMES
        INTEGER,DIMENSION(:), POINTER :: OOC_FILE_NAME_LENGTH
!    Indices of nul pivots
        INTEGER,DIMENSION(:), POINTER :: PIVNUL_LIST
!    Internal control array
        DOUBLE PRECISION DKEEP(30)
!    Array needed to manage additionnal candidate processor
        INTEGER, DIMENSION(:,:), POINTER :: SUP_PROC
!   ------------------------
!   Root structure(internal)
!   ------------------------
        TYPE (DAGMG_MUMPS_ROOT_STRUC) :: root
      END TYPE DAGMG_MUMPS_STRUC
       END MODULE DAGMG_MUMPS_STRUC_DF
       ! merge of 1 & 2
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      MODULE TOOLS_COMM
      INTERFACE AGMG_MUMPS_733
      SUBROUTINE AGMG_MUMPS_754(ARRAY, MINSIZE, INFO, LP, FORCE, COPY,       &
     &     STRING, MEMCNT, ERRCODE)
      INTEGER, POINTER    :: ARRAY(:)
      INTEGER             :: MINSIZE, LP
      INTEGER             :: INFO(*)
      LOGICAL, OPTIONAL   :: FORCE
      LOGICAL, OPTIONAL   :: COPY
      CHARACTER, OPTIONAL :: STRING*(*)
      INTEGER, OPTIONAL   :: ERRCODE, MEMCNT
      END SUBROUTINE AGMG_MUMPS_754
      SUBROUTINE AGMG_MUMPS_752(ARRAY, MINSIZE, INFO, LP, FORCE, COPY,       &
     &     STRING, MEMCNT, ERRCODE)
      REAL(KIND(1.D0)), POINTER    :: ARRAY(:)
      INTEGER             :: MINSIZE, LP
      INTEGER             :: INFO(*)
      LOGICAL, OPTIONAL   :: FORCE
      LOGICAL, OPTIONAL   :: COPY
      CHARACTER, OPTIONAL :: STRING*(*)
      INTEGER, OPTIONAL   :: ERRCODE, MEMCNT
      END SUBROUTINE AGMG_MUMPS_752
      SUBROUTINE AGMG_MUMPS_750(ARRAY, MINSIZE, INFO, LP, FORCE, COPY,       &
     &     STRING, MEMCNT, ERRCODE)
      REAL(KIND(1.E0)), POINTER    :: ARRAY(:)
      INTEGER             :: MINSIZE, LP
      INTEGER             :: INFO(*)
      LOGICAL, OPTIONAL   :: FORCE
      LOGICAL, OPTIONAL   :: COPY
      CHARACTER, OPTIONAL :: STRING*(*)
      INTEGER, OPTIONAL   :: ERRCODE, MEMCNT
      END SUBROUTINE AGMG_MUMPS_750
      SUBROUTINE AGMG_MUMPS_753(ARRAY, MINSIZE, INFO, LP, FORCE, COPY,       &
     &     STRING, MEMCNT, ERRCODE)
      COMPLEX(KIND((1.D0,1.D0))), POINTER    :: ARRAY(:)
      INTEGER             :: MINSIZE, LP
      INTEGER             :: INFO(*)
      LOGICAL, OPTIONAL   :: FORCE
      LOGICAL, OPTIONAL   :: COPY
      CHARACTER, OPTIONAL :: STRING*(*)
      INTEGER, OPTIONAL   :: ERRCODE, MEMCNT
      END SUBROUTINE AGMG_MUMPS_753
      SUBROUTINE AGMG_MUMPS_751(ARRAY, MINSIZE, INFO, LP, FORCE, COPY,       &
     &     STRING, MEMCNT, ERRCODE)
      COMPLEX(KIND((1.E0,1.E0))), POINTER    :: ARRAY(:)
      INTEGER             :: MINSIZE, LP
      INTEGER             :: INFO(*)
      LOGICAL, OPTIONAL   :: FORCE
      LOGICAL, OPTIONAL   :: COPY
      CHARACTER, OPTIONAL :: STRING*(*)
      INTEGER, OPTIONAL   :: ERRCODE, MEMCNT
      END SUBROUTINE AGMG_MUMPS_751
      END INTERFACE
      END MODULE
      ! grand merge of 1, 2, and all 3's
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      MODULE AGMG_MUMPS_STATIC_MAPPING
      IMPLICIT NONE
      PRIVATE
      PUBLIC :: AGMG_MUMPS_369, AGMG_MUMPS_393,                                   &
     &     AGMG_MUMPS_427,AGMG_MUMPS_494
      integer,pointer,dimension(:,:),SAVE::cv_cand
      integer,pointer,dimension(:),SAVE::cv_par2_nodes
      integer,SAVE::cv_slavef,cv_nb_niv2,cv_lp,cv_mp
      DOUBLE PRECISION,SAVE::cv_stack_peak
      integer,SAVE::cv_mem_strat
      integer,parameter::cv_invalid=-9999
      DOUBLE PRECISION,parameter::cv_d_invalid=-9999.D0
      integer,parameter::cv_equilib_flops=1
      integer,parameter::cv_equilib_mem=2
      integer,parameter::cv_error_memalloc = -13
      integer,parameter::cv_error_memdeloc = -96
      integer,dimension(:),allocatable,save :: mem_distribtmp
      integer, dimension(:),allocatable, save :: table_of_process
      integer,dimension(:),allocatable,save :: mem_distribmpi
      integer, save ::ke69,nb_arch_nodes
      logical,dimension(:),allocatable,save :: allowed_nodes
      integer,dimension(:),allocatable,save :: score
      type nodelist
         integer::nodenumber
         type(nodelist),pointer::next
      end type nodelist
      type alloc_arraytype
         integer, pointer, dimension(:)::t2_nodenumbers
         integer, pointer, dimension(:,:)::t2_cand
         DOUBLE PRECISION, pointer, dimension(:)::t2_candcostw(:),      &
     &                                            t2_candcostm(:)
         integer:: nmb_t2s
      end type alloc_arraytype
      type splitting_data
         integer:: new_ison,new_ifather,old_keep2
         DOUBLE PRECISION:: ncostw_oldinode,ncostm_oldinode,            &
     &                      tcostw_oldinode,tcostm_oldinode
      end type splitting_data
      type procs4node_t
         integer, dimension(:), pointer :: ind_proc
      end type procs4node_t
      DOUBLE PRECISION, pointer, dimension(:) ::                        &
     &      cv_proc_workload,                                           &
     &      cv_proc_maxwork,                                            &
     &      cv_proc_memused,                                            &
     &      cv_proc_maxmem
      type(splitting_data)::cv_last_splitting
      integer::cv_n,cv_nsteps,cv_maxlayer,                              &
     &         cv_nbsa,cv_maxnsteps,cv_maxcut,cv_maxdepth,              &
     &         cv_maxnodenmb,cv_total_amalg,cv_total_split,             &
     &         cv_bitsize_of_int,cv_size_ind_proc                       &
     &         ,cv_mixed_strat_bound,cv_dist_L0_mixed_strat_bound       &
     &         ,cv_layerl0_end,cv_layerl0_start
      integer :: layerL0_endforarrangeL0
      DOUBLE PRECISION :: mincostw
      DOUBLE PRECISION:: cv_costw_upper,cv_costm_upper,                 &
     &      cv_costw_layer0,cv_costm_layer0,cv_relax,                   &
     &      cv_costw_total,cv_costm_total,cv_l0wthresh,cv_splitthresh
      logical::cv_constr_work,cv_constr_mem
      integer,pointer,dimension(:):: cv_nodetype,cv_nodelayer,          &
     &          cv_layerl0_array,cv_proc_sorted,cv_potsplit,cv_depth
      integer,dimension(:),pointer::                                    &
     &    cv_ne,cv_nfsiz,cv_frere,cv_fils,cv_keep,cv_info,              &
     &    cv_procnode,cv_ssarbr,cv_icntl
      integer*8,dimension(:),pointer::cv_keep8
      type(alloc_arraytype),pointer,dimension(:)::cv_layer_p2node
      DOUBLE PRECISION,dimension(:),pointer:: cv_ncostw,                &
     & cv_tcostw,cv_ncostm,cv_tcostm,cv_layerworkload,cv_layermemused   &
     & ,cv_layerl0_sorted_costw
      type(procs4node_t),dimension(:),pointer :: cv_prop_map
      contains
      subroutine AGMG_MUMPS_369(n,slavef,icntl,info,                         &
     &                        ne,nfsiz,frere,fils,keep,KEEP8,           &
     &                        procnode,ssarbr,nbsa,peak,istat           &
     &     )
      implicit none
      integer,intent(in)::n,slavef
      integer, intent(inout),TARGET:: ne(n),nfsiz(n),                   &
     &         procnode(n),ssarbr(n),frere(n),fils(n),keep(500),        &
     &         icntl(40),info(40)
      INTEGER*8 KEEP8(150)
      integer,intent(out)::nbsa,istat
      integer ierr,nmb_thislayer,layernmb,mapalgo,allocok,i
      integer,pointer,dimension(:)::thislayer
      integer,parameter::memonly=1,floponly=2,hybrid=3
      DOUBLE PRECISION::                                                &
     &         maxwork,minwork,maxmem,minmem,workbalance,membalance
      DOUBLE PRECISION:: cost_root_node
      DOUBLE PRECISION,dimension(:),allocatable:: work_per_proc
      integer,dimension(:),allocatable::id_son
      logical::cont
      character (len=48):: err_rep,subname
      DOUBLE PRECISION peak
      istat=-1
      subname='DISTRIBUTE'
      cv_lp=icntl(1)
      cv_mp=icntl(3)
      nullify(thislayer)
      err_rep='INITPART1'
      call AGMG_MUMPS_478(n,slavef,                                          &
     &                   frere,fils,nfsiz,ne,keep,KEEP8,icntl,info,     &
     &                   procnode,ssarbr,peak,ierr                      &
     &     )
      if (ierr.ne.0) goto 99999
      err_rep='PROCINIT'
      call AGMG_MUMPS_391(istat=ierr)
      if (ierr.ne.0) goto 99999
      err_rep='CALCCOST'
      call AGMG_MUMPS_417(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='ROOTLIST'
      call AGMG_MUMPS_394(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='LAYERL0'
      call AGMG_MUMPS_381(ierr)
      if (ierr.ne.0) goto 99999
      if(cv_keep(82) .eq. 0) then
         err_rep='FIND_POTSPLIT'
         call AGMG_MUMPS_375(cv_equilib_flops,ierr)
      endif
      if (ierr.ne.0) goto 99999
      err_rep='INITPART2'
      call AGMG_MUMPS_479(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='WORKMEM_'
      call AGMG_MUMPS_408(                                                   &
     &     cv_proc_workload,cv_proc_memused,                            &
     &     maxwork,minwork,maxmem,minmem)
      if(maxwork.gt.0.0D0) then
         workbalance=minwork/maxwork
      else
         workbalance=0.0D0
      endif
      if(maxmem.gt.0.0D0) then
         membalance=minmem/maxmem
      else
         membalance=0.0D0
      endif
      err_rep='mem_alloc'
      allocate(thislayer(cv_maxnodenmb),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 2*cv_maxnsteps+cv_maxnodenmb
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'memory allocation error in ',subname
         ierr = cv_error_memalloc
         goto 99999
      end if
      cont=.TRUE.
      layernmb=0
      mapalgo=floponly
      err_rep='SELECT_TYPE3'
      call AGMG_MUMPS_396(ierr)
      if (ierr.ne.0) goto 99999
      IF (cv_keep(38) .ne. 0 .and. cv_keep(60) .eq. 0 ) THEN
        call AGMG_MUMPS_511(cv_nfsiz(keep(38)),                              &
     &               cv_nfsiz(keep(38)), cv_nfsiz(keep(38)),            &
     &               cv_keep(50), 3, cost_root_node)
        cost_root_node = cost_root_node / dble(cv_slavef)
        do i=1, cv_slavef
          cv_proc_memused(i)=cv_proc_memused(i)+                        &
     &        dble(cv_nfsiz(keep(38)))*dble(cv_nfsiz(keep(38)))/        &
     &        dble(cv_slavef)
          cv_proc_workload(i)=cv_proc_workload(i)+dble(cost_root_node)
        enddo
      ENDIF
      do while((cont).OR.(layernmb.le.cv_maxlayer))
         err_rep='FIND_THIS'
         call AGMG_MUMPS_376(layernmb,thislayer,nmb_thislayer,               &
     &                                ierr)
         if (ierr.ne.0) goto 99999
         err_rep='DO_SPLITTING'
         if(cv_keep(82) .gt. 0) then
            if(layernmb.gt.0) call AGMG_MUMPS_527                            &
     &           (layernmb,thislayer,nmb_thislayer,ierr)
         else
            if(layernmb.gt.0) call AGMG_MUMPS_370                            &
     &           (layernmb,thislayer,nmb_thislayer,ierr)
         endif
         if (ierr.ne.0) goto 99999
         err_rep='ASSIGN_TYPES'
         call AGMG_MUMPS_416(layernmb,thislayer,nmb_thislayer,               &
     &                              ierr)
         if (ierr.ne.0) goto 99999
         if(layernmb.gt.0) then
            if ((cv_keep(24).eq.1).OR.(cv_keep(24).eq.2).OR.            &
     &          (cv_keep(24).eq.4).OR.(cv_keep(24).eq.6)) then
               err_rep='COSTS_LAYER_T2'
               call AGMG_MUMPS_367(layernmb,nmb_thislayer,ierr)
            elseif((cv_keep(24).eq.8).OR.(cv_keep(24).eq.10)            &
     &             .OR.(cv_keep(24).eq.12).OR.(cv_keep(24).eq.14)       &
     &             .OR.(cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
               err_rep='COSTS_LAYER_T2PM'
               call AGMG_MUMPS_489(layernmb,nmb_thislayer,ierr)
            else
               err_rep='wrong strategy for COSTS_LAYER_T2'
               ierr = -9999
            endif
            if (ierr.ne.0) goto 99999
            err_rep='WORKMEM_'
            call AGMG_MUMPS_408(                                             &
     &                          cv_proc_workload,cv_proc_memused,       &
     &                                   maxwork,minwork,maxmem,minmem)
            if(maxwork.gt.0.0D0) then
               workbalance=minwork/maxwork
            else
               workbalance=0.0D0
            endif
            if(maxmem.gt.0.0D0) then
               membalance=minmem/maxmem
            else
               membalance=0.0D0
            endif
            if(mapalgo.eq.memonly) then
               err_rep='MAP_LAYER'
               call AGMG_MUMPS_387(layernmb,thislayer,                       &
     &              nmb_thislayer,cv_equilib_mem,ierr)
               if (ierr.ne.0) goto 99999
            elseif(mapalgo.eq.floponly) then
               err_rep='MAP_LAYER'
               call AGMG_MUMPS_387(layernmb,thislayer,                       &
     &              nmb_thislayer,cv_equilib_flops,ierr)
               if (ierr.ne.0) goto 99999
            elseif(mapalgo.eq.hybrid) then
               if (workbalance <= membalance) then
                  err_rep='MAP_LAYER'
                  call AGMG_MUMPS_387(layernmb,thislayer,                    &
     &                 nmb_thislayer,cv_equilib_flops,ierr)
                  if (ierr.ne.0) goto 99999
               else
                  err_rep='MAP_LAYER'
                  call AGMG_MUMPS_387(layernmb,thislayer,                    &
     &                 nmb_thislayer,cv_equilib_mem,ierr)
                  if (ierr.ne.0) goto 99999
               endif
            else
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Unknown mapalgo in ',subname
               return
            endif
      endif
         layernmb=layernmb+1
         err_rep='HIGHER_LAYER'
         call AGMG_MUMPS_377(layernmb,thislayer,                             &
     &                 nmb_thislayer,cont,ierr)
         if (ierr.ne.0) goto 99999
      end do
       if(cv_slavef.gt.4)                                               &
     &   call AGMG_MUMPS_431()
      err_rep='SETUP_CAND'
      call AGMG_MUMPS_397(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='ENCODE_PROC'
      call AGMG_MUMPS_371(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='STORE_GLOB'
      call AGMG_MUMPS_402(ne,nfsiz,frere,fils,keep,KEEP8,                    &
     &                         info,procnode,ssarbr,nbsa)
      err_rep='mem_dealloc'
      deallocate(thislayer,STAT=ierr)
      if (ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'Memory deallocation error in ',subname
         ierr = cv_error_memdeloc
         goto 99999
      endif
      err_rep='TERMGLOB'
      call AGMG_MUMPS_403(ierr)
      if (ierr.ne.0) goto 99999
      istat=0
      return
99999 continue
      if(cv_lp.gt.0) then
         write(cv_lp,*)'Error in ',subname,', layernmb=',layernmb
         write(cv_lp,*)'procedure reporting the error: ',err_rep
      endif
      if(ierr.eq.cv_error_memalloc) then
         info(1) = cv_info(1)
         info(2) = cv_info(2)
      endif
      istat=ierr
      return
      CONTAINS
      subroutine AGMG_MUMPS_413(                                             &
     &                       map_strat,workload,memused,accepted,       &
     &                              istat)
      implicit none
      integer,intent(in)::map_strat
      DOUBLE PRECISION,dimension(cv_slavef),intent(in)::workload,       &
     &                                                   memused
      logical,intent(out)::accepted
      integer,intent(out)::istat
      DOUBLE PRECISION maxi,mini,mean,stddev
      integer i,nmb
      intrinsic maxval,minval,count,sum
      character (len=48):: subname
      logical alternative_criterion
      DOUBLE PRECISION::                                                &
     &          MINFLOPS , MINMEM,                                      &
     &          CL_RATE, DV_RATE
      istat=-1
      if ( cv_keep(72) .EQ. 1) then
       MINFLOPS = 2.0D0
       MINMEM=50.0D0
       CL_RATE =0.8D0
       DV_RATE=0.2D0
      else
       MINFLOPS = 5.0D7
       MINMEM=5.0D6
       CL_RATE =0.8D0
       DV_RATE=0.2D0
      endif
      subname='ACCEPT_L0'
      accepted=.FALSE.
      alternative_criterion=.FALSE.
      if(map_strat.eq.cv_equilib_flops) then
         maxi=maxval(workload)
         mini=minval(workload)
         if (maxi.lt.MINFLOPS) then
            accepted=.TRUE.
         elseif(maxi.le.(dble(cv_keep(102))/dble(100))*mini)then
            accepted=.TRUE.
         endif
         if ((.NOT.accepted).AND.(alternative_criterion)) then
            mean=sum(workload)/max(dble(cv_slavef),dble(1))
            stddev=dble(0)
            do i=1,cv_slavef
               stddev=stddev+                                           &
     &               (abs(workload(i)-mean)*abs(workload(i)-mean))
            enddo
            stddev=sqrt(stddev/max(dble(cv_slavef),dble(1)))
            nmb=count(mask=abs(workload-mean)<stddev)
            if((dble(nmb)/max(dble(cv_slavef),dble(1)).gt.CL_RATE)      &
     &       .AND.(stddev.lt.DV_RATE*mean)) accepted=.TRUE.
         endif
      elseif(map_strat.eq.cv_equilib_mem) then
         maxi=maxval(memused)
         mini=minval(memused)
         if (maxi.lt.MINMEM) then
            accepted=.TRUE.
         else if(cv_slavef.lt.48) then
            if (maxi.le.dble(2)*mini) accepted=.TRUE.
         else if(cv_slavef.lt.128) then
            if (maxi.le.dble(4)*mini) accepted=.TRUE.
         else if(cv_slavef.lt.256) then
            if (maxi.le.dble(6)*mini) accepted=.TRUE.
         else if(cv_slavef.lt.512) then
            if (maxi.le.dble(8)*mini) accepted=.TRUE.
         else if(cv_slavef.gt.512) then
            if (maxi.le.dble(10)*mini) accepted=.TRUE.
         end if
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_413
      subroutine AGMG_MUMPS_415(map_strat,layerL0end,workload,memused,       &
     &                           procnode,istat,respect_prop)
      implicit none
      integer, intent(in)::map_strat, layerL0end
      DOUBLE PRECISION,dimension(cv_slavef),intent(out)::workload,      &
     &                                                   memused
      integer, intent(out)::procnode(cv_n),istat
      logical, intent(in), OPTIONAL:: respect_prop
      integer nmb_listnodes,i,j,ierr,                                   &
     &        nodenumber,proc
      DOUBLE PRECISION tot_work,tot_mem,work,mem
      character (len=48):: err_rep,subname
      istat=-1
      subname='ARRANGEL0'
      if ((.NOT.associated(cv_tcostw)).OR.(.NOT.associated(cv_tcostm))) &
     &   then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:tcost must be allocated in ',subname
         return
      end if
      if((map_strat.ne.cv_equilib_flops).and.                           &
     &   (map_strat.ne.cv_equilib_mem)) return
      do i=1,cv_n
         procnode(i)=cv_invalid
      end do
      do i=1,cv_slavef
         workload(i)=cv_proc_workload(i)
         memused(i)=cv_proc_memused(i)
      end do
      do i=cv_layerl0_start,layerl0end
         nodenumber=cv_layerl0_array(i)
         work=cv_tcostw(nodenumber)
         mem=cv_tcostm(nodenumber)
         err_rep='FIND_BEST_PROC'
         if(present(respect_prop)) then
            call AGMG_MUMPS_374(nodenumber,map_strat,work,mem,               &
     &           workload,memused,proc,ierr,respect_prop)
         else
            call AGMG_MUMPS_374(nodenumber,map_strat,work,mem,               &
     &           workload,memused,proc,ierr)
         endif
         if(ierr.eq.0) then
            procnode(nodenumber)=proc
         else
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            do j=1,cv_slavef
               workload(j)=cv_proc_workload(j)
               memused(j)=cv_proc_memused(j)
            end do
            do j=1,cv_n
               procnode(j)=cv_invalid
            end do
            return
         end if
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_415
      subroutine AGMG_MUMPS_416(                                             &
     &                              layernmb,thislayer,nmb_thislayer,   &
     &                                 istat)
      implicit none
      integer,intent(in)::layernmb,thislayer(cv_maxnodenmb),            &
     &                    nmb_thislayer
      integer,intent(out)::istat
      integer i,in,npiv,nfront,inode,inoderoot,par_nodes_in_layer,      &
     &        dummy,allocok
      character (len=48):: subname
      istat=-1
      subname='ASSIGN_TYPES'
      if((layernmb.lt.0).or.(layernmb.gt.cv_maxlayer)) return
      if(cv_slavef.eq.1) then
         if(layernmb.eq.0) then
            do inode=1,cv_n
               cv_nodetype(inode)=0
            end do
         end if
      else if(layernmb.eq.0) then
         do i=1,nmb_thislayer
            inode=thislayer(i)
            inoderoot=inode
            if(cv_nodetype(inode).ne.cv_invalid) cycle
            cv_nodetype(inode)=0
   30       continue
            in = inode
            do while (in .ne. 0)
               inode = in
               do while (in .gt. 0)
                  in = cv_fils(in)
               end do
               if (in.lt.0) in=-in
            end do
   10       continue
            if ( inode .ne. inoderoot ) then
               cv_nodetype(inode)=-1
               in = cv_frere(inode)
               inode = abs(in)
               if (in .lt. 0) then
                  go to 10
               else
                  go to 30
               end if
            end if
         end do
      else
         do i=1,nmb_thislayer
            inode=thislayer(i)
            in = inode
            npiv = 0
            do while (in.gt.0)
               in = cv_fils(in)
               npiv = npiv + 1
            end do
            nfront = cv_nfsiz(inode)
            if(cv_nodetype(inode).ne.cv_invalid) cycle
            if( ( AGMG_MUMPS_359(nfront,npiv)) .AND.                         &
     &           (in.ne.0)) then
               cv_nodetype(inode)=2
            else
               cv_nodetype(inode)=1
            end if
         end do
      end if
      if(layernmb.gt.0) then
         par_nodes_in_layer=0
         do i=1,nmb_thislayer
            inode=thislayer(i)
            if (cv_nodetype(inode).eq.2)                                &
     &         par_nodes_in_layer=par_nodes_in_layer+1
         enddo
         if(par_nodes_in_layer.gt.0) then
            allocate(                                                   &
     &cv_layer_p2node(layernmb)%t2_nodenumbers(par_nodes_in_layer),     &
     &cv_layer_p2node(layernmb)%t2_cand(par_nodes_in_layer,cv_slavef+1),&
     &cv_layer_p2node(layernmb)%t2_candcostw(par_nodes_in_layer),       &
     &cv_layer_p2node(layernmb)%t2_candcostm(par_nodes_in_layer),       &
     &               STAT=allocok)
            if (allocok.gt.0) then
               cv_info(1) = cv_error_memalloc
               cv_info(2) = (3+cv_slavef+1)*par_nodes_in_layer
               istat = cv_error_memalloc
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'memory allocation error in ',subname
               return
            end if
            cv_layer_p2node(layernmb)%nmb_t2s=par_nodes_in_layer
            dummy=1
            do i=1,nmb_thislayer
               inode=thislayer(i)
               if (cv_nodetype(inode).eq.2) then
                  cv_layer_p2node(layernmb)%t2_nodenumbers(dummy)=inode
                  cv_layer_p2node(layernmb)%t2_cand(dummy,:)=0
                  cv_layer_p2node(layernmb)%t2_candcostw(dummy)         &
     &                                                    =cv_d_invalid
                  cv_layer_p2node(layernmb)%t2_candcostm(dummy)         &
     &                                                    =cv_d_invalid
                  dummy=dummy+1
               endif
            enddo
         else
            nullify(cv_layer_p2node(layernmb)%t2_nodenumbers,           &
     &              cv_layer_p2node(layernmb)%t2_cand,                  &
     &              cv_layer_p2node(layernmb)%t2_candcostw,             &
     &              cv_layer_p2node(layernmb)%t2_candcostm)
         end if
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_416
      function AGMG_MUMPS_480(procs4node,procnumber)
      implicit none
      integer,intent(in)::procs4node(cv_size_ind_proc)
      integer,intent(in)::procnumber
      logical :: AGMG_MUMPS_480
      integer pos1,pos2
      pos1 = (procnumber-1)/cv_bitsize_of_int +1
      pos2 = mod(procnumber-1,cv_bitsize_of_int)
      AGMG_MUMPS_480=btest(procs4node(pos1),pos2)
      return
      end function AGMG_MUMPS_480
      function AGMG_MUMPS_481(inode,procnumber)
      implicit none
      integer, intent(in)::inode,procnumber
      logical :: AGMG_MUMPS_481
      integer pos1,pos2
      AGMG_MUMPS_481=.FALSE.
      if((procnumber.lt.1).or.(procnumber.gt.cv_slavef)) return
      if(.not.associated(cv_prop_map(inode)%ind_proc)) return
      pos1 = (procnumber-1)/cv_bitsize_of_int +1
      pos2 = mod(procnumber-1,cv_bitsize_of_int)
      AGMG_MUMPS_481=btest                                                   &
     &               (cv_prop_map(inode)%ind_proc(pos1),pos2)
      return
      end function AGMG_MUMPS_481
      subroutine AGMG_MUMPS_482(procs4node,procnumber,istat)
      implicit none
      integer, intent(inout)::procs4node(cv_size_ind_proc)
      integer,intent(in)::procnumber
      integer, intent(out)::istat
      integer pos1,pos2
      istat = -1
      if((procnumber.lt.1).or.(procnumber.gt.cv_slavef)) return
      if(cv_bitsize_of_int.le.0) return
      pos1 = (procnumber-1)/cv_bitsize_of_int +1
      pos2 = mod(procnumber-1,cv_bitsize_of_int)
      procs4node(pos1)=ibset(procs4node(pos1),pos2)
      istat = 0
      return
      end subroutine AGMG_MUMPS_482
      subroutine AGMG_MUMPS_417(istat)
      implicit none
      integer,intent(out)::istat
      integer i,ierr
      DOUBLE PRECISION :: maxcostw_root
      character (len=48):: err_rep,subname
      istat = -1
      subname='CALCCOSTS'
      if ((.NOT.associated(cv_tcostw)).OR.(.NOT.associated(cv_tcostm))) &
     &   then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:tcost must be allocated in ',subname
         return
      end if
      maxcostw_root = 0D0
      do i=1,cv_n
         if (cv_frere(i).eq.cv_n+1) then
            cv_tcostw(i)=0.0D0
            cv_ncostw(i)=0.0D0
            cv_tcostm(i)=0.0D0
            cv_ncostm(i)=0.0D0
         elseif (cv_frere(i).eq.0) then
            err_rep='TREECOSTS'
            cv_depth(i)=1
            call AGMG_MUMPS_404(i,ierr)
            maxcostw_root = max(maxcostw_root,cv_tcostw(i))
            if (ierr.ne.0) then
          if(cv_lp.gt.0)                                                &
     &    write(cv_lp,*)'Failure reported by ',err_rep, ' in ',subname
               return
            end if
         end if
      end do
      istat = 0
      mincostw = 1.0D0+maxcostw_root/(dble(cv_maxnsteps)*               &
     &              dble(10*cv_slavef) )
      return
      end subroutine AGMG_MUMPS_417
      subroutine AGMG_MUMPS_418(npiv,nfront,costw,costm)
      implicit none
      integer,intent(in)::npiv,nfront
      DOUBLE PRECISION,intent(out)::costw,costm
      character (len=48):: subname
      subname='CALCNODECOSTS'
      if((npiv.le.1).and.(nfront.le.1)) then
         costw = dble(0)
         costm = dble(1)
      else
         if(cv_keep(50).eq.0) then
            costw= 2.0D0*dble(nfront)*dble(npiv)*dble(nfront-npiv-1)    &
     &      + dble(npiv)*dble(npiv+1)*dble(2*npiv+1)/dble(3)            &
     &           + dble(2*nfront-npiv-1) * dble(npiv) / dble(2)
            costm= dble(npiv)*(dble(2*nfront)-dble(npiv))
         else
            costw= dble(npiv) *                                         &
     &            (dble(nfront)*dble(nfront)+dble(2*nfront) -           &
     &             dble(nfront+1) * dble(npiv+1) +                      &
     &             dble(npiv+1) * dble(2*npiv+1) / dble(6))
            costm= dble(npiv) * dble(nfront)
         end if
      end if
      if((costw.lt.0.0D0).or.(costm.lt.0.0D0)) then
      endif
      return
      end subroutine AGMG_MUMPS_418
      subroutine AGMG_MUMPS_367(layernmb,nmb_thislayer,istat)
      implicit none
      integer,intent(in)::layernmb,nmb_thislayer
      integer,intent(out)::istat
      integer in,inode,j,kmax,npiv,nfront,ncb,ncol,                     &
     &        min_needed,max_needed,more_than_needed,total_nmb_cand,    &
     &        nmb_type2_thislayer,fraction,                             &
     &        total_cand_layer,cand_strat, keep48_loc, WHAT_LOC
      DOUBLE PRECISION flop1,work_type2_thislayer,                      &
     &        relative_weight,workmaster,nrow
      logical force_cand
      intrinsic count,max
      character (len=48):: subname
      integer AGMG_MUMPS_497, AGMG_MUMPS_50,                                      &
     &        AGMG_MUMPS_52
      external AGMG_MUMPS_497, AGMG_MUMPS_50,                                     &
     &        AGMG_MUMPS_52
      istat=-1
      subname='COSTS_LAYER_T2'
      if (cv_keep(24).lt.1) then
          if(cv_lp.gt.0)                                                &
     &    write(cv_lp,*)'Error in ',subname,'. Wrong keep24'
         return
      endif
      force_cand=(mod(cv_keep(24),2).eq.0)
      cand_strat=cv_keep(24)/2
      nmb_type2_thislayer=cv_layer_p2node(layernmb)%nmb_t2s
      if (nmb_type2_thislayer.gt.0) then
         work_type2_thislayer=0.0D0
         do j=1,nmb_type2_thislayer
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(j)
            work_type2_thislayer=work_type2_thislayer+cv_ncostw(inode)
         end do
         if(cv_relax.le.0.0D0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Error in ',subname,'. Wrong cv_relax'
            return
         endif
         total_cand_layer=int(cv_relax*dble(cv_slavef))
         do j=1,nmb_type2_thislayer
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(j)
            nfront=cv_nfsiz(inode)
            npiv=0
            in=inode
            do while(in.gt.0)
               npiv=npiv+1
               in=cv_fils(in)
            end do
            ncb=nfront-npiv
            kmax = AGMG_MUMPS_497(cv_keep8(21),ncb)
            if (force_cand) then
              if (cv_keep(50) ==  0) then
                keep48_loc=0
              else
                keep48_loc=3
              endif
              if (cv_keep(48).EQ.5) keep48_loc = 5
               min_needed = AGMG_MUMPS_50(                                   &
     &             cv_slavef, keep48_loc,cv_keep8(21),                  &
     &             cv_keep(50),nfront,ncb)
               max_needed = AGMG_MUMPS_52(                                   &
     &             cv_slavef, keep48_loc,cv_keep8(21),                  &
     &             cv_keep(50),nfront,ncb)
               if(cand_strat.eq.1) then
                  more_than_needed = 0
               elseif (cand_strat.eq.2) then
                  if(work_type2_thislayer.gt.0.0D0) then
                  relative_weight=cv_ncostw(inode)/work_type2_thislayer
                  else
                     relative_weight = 0.0D0
                  endif
                  fraction=nint(relative_weight *                       &
     &                 dble(total_cand_layer))
                  more_than_needed=min(max(0,cv_slavef-1-min_needed),   &
     &                                 max(0,fraction-min_needed)    )
               elseif (cand_strat.eq.3) then
                  more_than_needed=cv_slavef-1-min_needed
               else
                  if(cv_lp.gt.0)                                        &
     &            write(cv_lp,*)'Unknown cand. strategy in ',subname
                  return
               endif
               total_nmb_cand=min(min_needed+more_than_needed,          &
     &              cv_slavef-1)
               total_nmb_cand=min(total_nmb_cand,max_needed)
            else
               total_nmb_cand=0
            endif
            cv_layer_p2node(layernmb)%t2_cand(j,cv_slavef+1)            &
     &                                 = total_nmb_cand
            if(cv_keep(50).eq.0) then
               flop1=dble(2*npiv)*dble(nfront)-                         &
     &              dble(npiv+nfront)*dble(npiv+1)
               flop1= dble(npiv)*flop1 +                                &
     &          dble(2 * npiv-npiv-1)*dble(npiv)/dble(2)+               &
     &          dble(npiv)*dble(npiv+1)*dble(2*npiv+1)/dble(3)
            else
               flop1=dble(npiv)*                                        &
     &         ( dble(npiv)*dble(npiv)+dble(npiv)-                      &
     &         dble(npiv*npiv+npiv+1) )+                                &
     &         (dble(npiv)*dble(npiv+1)*dble(2*npiv+1))/dble(6)
            endif
            cv_ncostw(inode)=flop1
            if(total_nmb_cand.gt.0) then
               nrow = dble(max(min(dble(ncb)/dble(total_nmb_cand),      &
     &                     dble(kmax)),                                 &
     &                 dble(ncb)/dble(cv_slavef-1)))
            elseif(cv_slavef.gt.1) then
               nrow = dble(max(dble(kmax),                              &
     &                         dble(ncb)/dble(cv_slavef-1)))
            else
               nrow = dble(ncb)
            endif
            if(cv_keep(50).eq.0) then
               flop1 = dble(npiv)*dble(nrow)+                           &
     &                 dble(nrow)*dble(npiv)*dble(2*nfront-npiv-1)
            else
               ncol= nfront
               flop1 = dble(npiv)*dble(nrow)*                           &
     &          (dble(2*ncol)-dble(nrow)-dble(npiv)+dble(1))
               workmaster = dble(npiv*npiv)*dble(npiv)/dble(3)
               if (workmaster.gt.flop1) flop1=workmaster
            endif
            cv_layer_p2node(layernmb)%t2_candcostw(j)=flop1
            if(cv_keep(50).eq.0) then
               cv_ncostm(inode)=dble(npiv)*dble(nfront)
            else
               cv_ncostm(inode)=dble(npiv)*dble(npiv)
            endif
            if(cv_keep(50).eq.0) then
               cv_layer_p2node(layernmb)%t2_candcostm(j)                &
     &                                  =dble(npiv)*dble(nrow)
            else
               cv_layer_p2node(layernmb)%t2_candcostm(j)                &
     &                                  =dble(npiv)*dble(nrow)
            endif
         end do
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_367
      subroutine AGMG_MUMPS_489(layernmb,nmb_thislayer,istat)
      implicit none
      integer,intent(in)::layernmb,nmb_thislayer
      integer,intent(out)::istat
      integer in,inode,j,jj,kmax,npiv,nfront,ncb,ncol,                  &
     &        total_nmb_cand,nmb_type2_thislayer,                       &
     &        total_cand_layer,npropmap,min_needed,                     &
     &        keep48_loc
      DOUBLE PRECISION flop1,work_type2_thislayer,                      &
     &        relative_weight,workmaster,nrow
      intrinsic count,max
      character (len=48):: subname
      integer AGMG_MUMPS_497, AGMG_MUMPS_50
      external AGMG_MUMPS_497, AGMG_MUMPS_50
      istat=-1
      subname='COSTS_LAYER_T2PM'
      if((cv_keep(24).ne.8).AND.(cv_keep(24).ne.10)                     &
     &    .AND.(cv_keep(24).ne.12).AND.(cv_keep(24).ne.14)              &
     &    .AND.(cv_keep(24).ne.16).AND.(cv_keep(24).ne.18)) then
          if(cv_lp.gt.0)                                                &
     &    write(cv_lp,*)'Error in ',subname,'. Wrong keep24'
         return
      endif
      nmb_type2_thislayer=cv_layer_p2node(layernmb)%nmb_t2s
      if (nmb_type2_thislayer.gt.0) then
         total_cand_layer=0
         work_type2_thislayer=0.0D0
         do j=1,nmb_type2_thislayer
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(j)
            work_type2_thislayer=work_type2_thislayer+cv_ncostw(inode)
            npropmap=0
            do jj=1,cv_slavef
               if( AGMG_MUMPS_481(inode,jj))                                 &
     &              npropmap=npropmap+1
            end do
            total_cand_layer=total_cand_layer+npropmap
         end do
         do j=1,nmb_type2_thislayer
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(j)
            nfront=cv_nfsiz(inode)
            npiv=0
            in=inode
            do while(in.gt.0)
               npiv=npiv+1
               in=cv_fils(in)
            end do
            ncb=nfront-npiv
            kmax = AGMG_MUMPS_497(cv_keep8(21),ncb)
            if(kmax.lt.1) then
               kmax = max(kmax,1)
            endif
            if (cv_keep(50) ==  0) then
              keep48_loc=0
            else
              keep48_loc=3
            endif
            if (cv_keep(48).EQ.5) keep48_loc = 5
            min_needed= AGMG_MUMPS_50                                        &
     &          (cv_slavef, keep48_loc,cv_keep8(21),                    &
     &           cv_keep(50),nfront,ncb)
            if(min_needed.lt.1) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname,'.NEG min_needed'
               return
            endif
            if ((cv_keep(24).eq.8).OR.(cv_keep(24).eq.14).OR.           &
     &          (cv_keep(24).eq.18)) then
               npropmap=0
               do jj=1,cv_slavef
                  if( AGMG_MUMPS_481(inode,jj))                              &
     &                 npropmap=npropmap+1
               end do
               total_nmb_cand=max(npropmap-1,min_needed)
            elseif(cv_keep(24).eq.10) then
               if(work_type2_thislayer.gt.0.0D0) then
                  relative_weight=cv_ncostw(inode)/work_type2_thislayer
               else
                  relative_weight = 0.0D0
               endif
               total_nmb_cand=nint(relative_weight *                    &
     &              dble(total_cand_layer))
               total_nmb_cand=max(total_nmb_cand-1,min_needed)
            elseif((cv_keep(24).eq.12).OR.(cv_keep(24).eq.16)) then
               if(layernmb.lt.cv_dist_L0_mixed_strat_bound) then
                  if(cv_mp.gt.0)then
                     write(cv_mp,*)'Strat', cv_keep(24),                &
     &                             ': use 8 on layer',layernmb
                  endif
                  npropmap=0
                  do jj=1,cv_slavef
                     if( AGMG_MUMPS_481(inode,jj))                           &
     &                    npropmap=npropmap+1
                  end do
                  total_nmb_cand=max(npropmap-1,min_needed)
               else
                  if(cv_mp.gt.0)then
                     write(cv_mp,*)'Strat', cv_keep(24),                &
     &                             ': use 10 on layer',layernmb
                  endif
                  if(work_type2_thislayer.gt.0.0D0) then
                  relative_weight=cv_ncostw(inode)/work_type2_thislayer
                  else
                     relative_weight = 0.0D0
                  endif
                  total_nmb_cand=nint(relative_weight *                 &
     &                 dble(total_cand_layer))
                  total_nmb_cand=max(total_nmb_cand-1,min_needed)
               endif
            else
               if(cv_lp.gt.0)                                           &
     &              write(cv_lp,*)'Unknown cand. strategy in ',subname
               return
            endif
            total_nmb_cand=max(total_nmb_cand,1)
            total_nmb_cand=min(total_nmb_cand,cv_slavef-1)
            total_nmb_cand=min(total_nmb_cand,ncb)
            cv_layer_p2node(layernmb)%t2_cand(j,cv_slavef+1)            &
     &                                 = total_nmb_cand
            if(cv_keep(50).eq.0) then
               flop1=dble(2*npiv)*dble(nfront)-                         &
     &              dble(npiv+nfront)*dble(npiv+1)
               flop1= dble(npiv)*flop1 +                                &
     &          dble(2 * npiv-npiv-1)*dble(npiv)/dble(2)+               &
     &          dble(npiv)*dble(npiv+1)*dble(2*npiv+1)/dble(3)
            else
               flop1=dble(npiv)*                                        &
     &         ( dble(npiv)*dble(npiv)+dble(npiv)-                      &
     &         dble(npiv*npiv+npiv+1) )+                                &
     &         (dble(npiv)*dble(npiv+1)*dble(2*npiv+1))/dble(6)
            endif
            cv_ncostw(inode)=flop1
            if(total_nmb_cand.gt.0) then
               nrow = dble(max(min(dble(ncb)/dble(total_nmb_cand),      &
     &                     dble(kmax)),                                 &
     &                 dble(ncb)/dble(cv_slavef-1)))
            elseif(cv_slavef.gt.1) then
               nrow = dble(max(dble(kmax),                              &
     &                         dble(ncb)/dble(cv_slavef-1)))
            else
               nrow = dble(ncb)
            endif
            if(cv_keep(50).eq.0) then
               flop1 = dble(npiv)*dble(nrow)+                           &
     &                 dble(nrow)*dble(npiv)*dble(2*nfront-npiv-1)
            else
               ncol= nfront
               flop1 = dble(npiv)*dble(nrow)*                           &
     &          (dble(2*ncol)-dble(nrow)-dble(npiv)+dble(1))
               workmaster = dble(npiv*npiv)*dble(npiv)/dble(3)
               if (workmaster.gt.flop1) flop1=workmaster
            endif
            cv_layer_p2node(layernmb)%t2_candcostw(j)=flop1
            if(cv_keep(50).eq.0) then
               cv_ncostm(inode)=dble(npiv)*dble(nfront)
            else
               cv_ncostm(inode)=dble(npiv)*dble(npiv)
            endif
            if(cv_keep(50).eq.0) then
               cv_layer_p2node(layernmb)%t2_candcostm(j)                &
     &                                  =dble(npiv)*dble(nrow)
            else
               cv_layer_p2node(layernmb)%t2_candcostm(j)                &
     &                                  =dble(npiv)*dble(nrow)
            endif
         end do
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_489
      subroutine AGMG_MUMPS_527(                                             &
     &     layernmb,thislayer,nmb_thislayer,                            &
     &     istat)
      implicit none
      integer,intent(in)::layernmb,thislayer(cv_maxnodenmb),            &
     &     nmb_thislayer
      integer,intent(out)::istat
      integer i,k1,k2,k3,ierr,inode,in,nfront,npiv,                     &
     &     npiv_son,ison,ifather,                                       &
     &     nmb_type2
      logical doit
      character (len=48):: err_rep,subname
      istat=-1
      subname='NEWSPLITTING'
      if((layernmb.lt.0).or.(layernmb.gt.cv_maxlayer)) return
      if (cv_slavef.eq.1) then
         return
      endif
      do i=1,nmb_thislayer
         ierr=0
         inode=thislayer(i)
         err_rep='GET_SPLIT_INKPART'
         call AGMG_MUMPS_525(inode,                                          &
     &        doit,npiv,nfront,k1,k3,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Error reported by ',                    &
     &           err_rep,' in ',subname
            istat =ierr
            return
         endif
         if(.NOT.doit) then
            cycle
         endif
         if(k3.eq.1) then
            cycle
         endif
         err_rep='GET_MEMSPLIT_INKPART'
         call AGMG_MUMPS_526(inode,                                          &
     &        doit,npiv,nfront,k2,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Error reported by ',                    &
     &           err_rep,' in ',subname
            istat =ierr
            return
         endif
         k1 = max(k1,k2)
         k1 = min(k1,k3)
         if( k1 .eq. 1) cycle
         err_rep='SPLITNODE_INKPART'
         call AGMG_MUMPS_529(inode,nfront,npiv,k1,                           &
     &        ison,ifather,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,                 &
     &           ' in ',subname
            istat = ierr
            return
         endif
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_527
      subroutine AGMG_MUMPS_525(inode,                                       &
     &     doit,npiv,nfront,k1,k3,istat)
      implicit none
      integer,intent(in)::inode
      logical,intent(out)::doit
      integer,intent(out),OPTIONAL::npiv,nfront,istat
      integer,intent(out),OPTIONAL::k1,k3
      integer in,nfr_ifather,npiv_ifather,npiv2,nfront2,npiv_son2
      integer npropmap,j,ncb,kmax,keep48_loc,nslaves_max,               &
     &     nslaves_estim,strat,kk
      DOUBLE PRECISION wk_master,wk_slave,wk_master2,wk_slave2
      integer AGMG_MUMPS_497,                                                &
     &     AGMG_MUMPS_52,                                                    &
     &     AGMG_MUMPS_50
      external AGMG_MUMPS_497
      external AGMG_MUMPS_52
      external AGMG_MUMPS_50
      doit=.FALSE.
      if(present(npiv)) npiv=cv_invalid
      if(present(nfront)) nfront=cv_invalid
      if(present(k1)) k1=1
      if(present(k3)) k3 =1
      if(present(istat)) istat=-1
      doit=.TRUE.
      if (cv_nodetype(inode) .gt. 0) then
         doit=.FALSE.
         istat = 0
         return
      endif
      if ( (cv_frere(inode).eq.0) ) then
         doit=.FALSE.
         istat = 0
         return
      endif
      nfront=cv_nfsiz(inode)
      in=inode
      npiv=0
      do while (in.gt.0)
         in=cv_fils(in)
         npiv=npiv+1
      end do
      npiv_son2 = max(npiv/2,1)
      if(npiv.le.npiv_son2) then
         doit=.FALSE.
         istat = 0
         return
      endif
      if( .not. AGMG_MUMPS_359(nfront,npiv_son2) )then
         doit=.FALSE.
         istat = 0
         return
      endif
      in = inode
      npropmap=0
      do j=1,cv_slavef
         if( AGMG_MUMPS_481(in,j)) then
            npropmap=npropmap+1
         endif
      end do
      ncb = nfront - npiv
      kmax = AGMG_MUMPS_497(cv_keep8(21),ncb)
      if (cv_keep(50) ==  0) then
         keep48_loc=0
      else
         keep48_loc=3
      endif
      if (cv_keep(48).EQ.5) keep48_loc = 5
      if(npropmap .gt. cv_keep(83)) then
         nslaves_max   = AGMG_MUMPS_52(                                      &
     &        cv_slavef, keep48_loc, cv_keep8(21),                      &
     &        cv_keep(50), nfront, ncb )
         nslaves_estim = min(npropmap,nslaves_max)
         nslaves_estim = max(nslaves_estim,1)
      else
         nslaves_max   = AGMG_MUMPS_52(                                      &
     &        cv_slavef, keep48_loc, cv_keep8(21),                      &
     &        cv_keep(50), nfront, ncb )
         nslaves_estim = AGMG_MUMPS_50(                                      &
     &     cv_slavef, keep48_loc,cv_keep8(21),                          &
     &     cv_keep(50), nfront, ncb )
         nslaves_estim = max(nslaves_estim,1)
         nslaves_estim = min(nslaves_estim,nslaves_max)
      endif
      if (cv_keep(50).eq.0) then
         wk_master = (dble(2)/dble(3))*                                 &
     &        dble(npiv)*dble(npiv)*dble(npiv)+                         &
     &        dble(npiv)*dble(npiv)*dble(nfront-npiv)
      else
         wk_master = dble(npiv)*dble(npiv)*dble(npiv)/dble(3)
      end if
      strat = cv_keep(62)
      doit = .TRUE.
      k1 = cv_keep(82)
      k3 = cv_keep(82)
      do kk=1,cv_keep(82)-1
         npiv2 = npiv/kk
         if(npiv2 .eq. 0) then
            k1 = max(1,kk-1)
            exit
         endif
         wk_master2 = wk_master / dble(kk)
         nfront2 = nfront-npiv+npiv2
         if (cv_keep(50).eq.0) then
            wk_slave2  = ( dble(npiv2)*dble(nfront-npiv2) *             &
     &           dble(2*nfront-npiv2) ) / dble(nslaves_estim)
         else
            wk_slave2 =                                                 &
     &           ( dble(npiv2)*dble(nfront2-npiv2)*dble(nfront2) )      &
     &           /   dble(nslaves_estim)
         endif
         if(wk_master2.le.                                              &
     &        (1.0D0 +dble(kk*strat)/dble(100))*wk_slave2) then
            k1 = kk
            exit
         endif
      enddo
      do kk=1,cv_keep(82)-1
         npiv2 = npiv/kk
         if(npiv2 .eq. 0) then
            k3 = max(1,kk-1)
            exit
         endif
         wk_master2 = wk_master / dble(kk)
         nfront2 = nfront
         if (cv_keep(50).eq.0) then
            wk_slave2  = ( dble(npiv2)*dble(nfront-npiv2) *             &
     &           dble(2*nfront-npiv2) ) / dble(nslaves_estim)
         else
            wk_slave2 =                                                 &
     &           ( dble(npiv2)*dble(nfront2-npiv2)*dble(nfront2) )      &
     &           /   dble(nslaves_estim)
         endif
         if(wk_master2.le.wk_slave2) then
            k3 = kk
            exit
         endif
      enddo
      if(k3 .lt. k1) then
         k3 = k1
      endif
      if(present(istat)) istat=0
      return
      end subroutine AGMG_MUMPS_525
      subroutine AGMG_MUMPS_526(inode,                                       &
     &     doit,npiv,nfront,k2,istat)
      implicit none
      integer,intent(in)::inode
      logical,intent(out)::doit
      integer,intent(out),OPTIONAL::npiv,nfront,istat
      integer,intent(out),OPTIONAL::k2
      integer in,nfr_ifather,npiv_ifather,npiv2,nfront2,npiv_son2
      integer npropmap,j,ncb,kmax,keep48_loc,nslaves_max,               &
     &     nslaves_estim,kk
      DOUBLE PRECISION mem_master
      doit=.FALSE.
      if(present(npiv)) npiv=cv_invalid
      if(present(nfront)) nfront=cv_invalid
      if(present(k2)) k2=1
      if(present(istat)) istat=-1
      doit=.TRUE.
      nfront=cv_nfsiz(inode)
      in=inode
      npiv=0
      do while (in.gt.0)
         in=cv_fils(in)
         npiv=npiv+1
      end do
      doit = .TRUE.
      k2 = cv_keep(82)
      do kk=1,cv_keep(82)-1
         npiv2 = npiv/kk
         if(npiv2 .eq. 0) then
            k2 = max(1,kk-1)
            exit
         endif
         if (cv_keep(50).eq.0) then
            mem_master  = dble(npiv2)*dble(nfront)
         else
            mem_master  = dble(npiv2)*dble(npiv2)
         endif
         if(mem_master.le.                                              &
     &        (dble(cv_mem_strat)/dble(100))*cv_stack_peak) then
            k2 = kk
            exit
         endif
      enddo
      if(present(istat)) istat=0
      return
      end subroutine AGMG_MUMPS_526
      subroutine AGMG_MUMPS_529(inode,nfront,npiv,k,                         &
     &     ison,ifather,istat)
      implicit none
      integer, intent(in)::nfront,npiv
      integer, intent(in):: k
      integer inode
      integer,intent(out)::ison,ifather
      integer, intent(out)::istat
      integer i,lev,in,in_son,in_father,in_grandpa,                     &
     &     npiv_son,nfrontk,npivk,d1,f1,e1,dk,fk,next_father
      DOUBLE PRECISION:: ncostm,ncostw,ncostm_ison,ncostw_ison,         &
     &                   ncostm_ifather,ncostw_ifather
      character (len=48):: subname
      istat=-1
      subname='SPLITNODE_INKPART'
      ison=-1
      ifather=-1
      ncostw=cv_ncostw(inode)
      ncostm=cv_ncostm(inode)
      nfrontk = nfront
      npivk = npiv
      npiv_son = npiv/k
      cv_keep(2)=max(cv_keep(2),nfront-npiv_son)
      d1 = inode
      f1 = d1
      e1 = cv_frere(d1)
      do i=1,npiv_son-1
         f1 = cv_fils(f1)
      enddo
      ison = d1
      in_son = f1
      next_father = cv_fils(in_son)
      call AGMG_MUMPS_418(npiv_son,nfrontk,                                  &
     &     ncostw_ison,ncostm_ison)
      cv_ncostw(ison)=ncostw_ison
      cv_ncostm(ison)=ncostm_ison
      if(associated(cv_tcostw)) cv_tcostw(ison) = cv_tcostw(inode)      &
     &     -ncostw +cv_ncostw(ison)
      if(associated(cv_tcostm)) cv_tcostm(ison) = cv_tcostm(inode)      &
     &     -ncostm +cv_ncostm(ison)
      do lev = 1,k-1
         ifather = next_father
         in_father = ifather
         if(lev .eq. k-1) then
            do while (cv_fils(in_father).gt.0)
               in_father=cv_fils(in_father)
            end do
         else
            do i=1,npiv_son-1
               in_father=cv_fils(in_father)
            enddo
         endif
         cv_frere(ison)=-ifather
         next_father = cv_fils(in_father)
         cv_fils(in_father)=-ison
         cv_nfsiz(ison)=nfrontk
         cv_nfsiz(ifather)=nfrontk-npiv_son
         cv_ne(ifather)=1
         cv_keep(61)=cv_keep(61)+1
         call AGMG_MUMPS_418(npiv_son,nfrontk-npiv_son,                      &
     &        ncostw_ifather,ncostm_ifather)
         cv_ncostw(ifather)=ncostw_ifather
         cv_ncostm(ifather)=ncostm_ifather
         if(associated(cv_tcostw))                                      &
     &        cv_tcostw(ifather) = cv_tcostw(ison)+cv_ncostw(ifather)
         if(associated(cv_tcostm))                                      &
     &        cv_tcostm(ifather) = cv_tcostm(ison)+cv_ncostm(ifather)
         cv_total_split=cv_total_split+1
         if(lev .gt. 1) then
            call AGMG_MUMPS_437(inode,ison,ierr)
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &              write(cv_lp,*)'PROPMAP4SPLIT error in ',subname
               istat = ierr
               return
            endif
         endif
         if( AGMG_MUMPS_359(nfrontk-npiv_son,npiv_son) ) then
            cv_nodetype(ifather) = 2
         else
            cv_nodetype(ifather) = 1
         endif
         nfrontk = nfrontk-npiv_son
         npivk = npivk - npiv_son
         ison = ifather
         in_son = in_father
      enddo
      dk = ifather
      fk = in_father
      if( AGMG_MUMPS_359(nfrontk,npivk) ) then
         cv_nodetype(dk) = 2
      else
         cv_nodetype(dk) = 1
      endif
      call AGMG_MUMPS_418(npivk,nfrontk,                                     &
     &     ncostw_ifather,ncostm_ifather)
      cv_ncostw(dk)=ncostw_ifather
      cv_ncostm(dk)=ncostm_ifather
      if(associated(cv_tcostw))                                         &
     &     cv_tcostw(dk) = cv_tcostw(ison)+cv_ncostw(dk)
      if(associated(cv_tcostm))                                         &
     &     cv_tcostm(dk) = cv_tcostm(ison)+cv_ncostm(dk)
      cv_fils(f1) = next_father
      cv_frere(dk) = e1
      in = e1
      do while (in.gt.0)
         in=cv_frere(in)
      end do
      in = -in
      do while(cv_fils(in).gt.0)
         in=cv_fils(in)
      end do
      in_grandpa = in
      if(cv_fils(in_grandpa).eq.-d1) then
         cv_fils(in_grandpa)=-dk
      else
         in=-cv_fils(in_grandpa)
         do while(cv_frere(in) .ne. d1)
            in=cv_frere(in)
         end do
         cv_frere(in) = dk
      end if
      ison = dk
      do lev=1,k
         do while (cv_fils(ison).gt.0)
            ison=cv_fils(ison)
         end do
         ison = -cv_fils(ison)
      enddo
      call AGMG_MUMPS_437(inode,dk,ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'PROPMAP4SPLIT error in ',subname
         istat = ierr
         return
      endif
      cv_nsteps = cv_nsteps + k-1
      istat = 0
      return
      end subroutine AGMG_MUMPS_529
      subroutine AGMG_MUMPS_370(                                             &
     &                              layernmb,thislayer,nmb_thislayer,   &
     &                              istat)
      implicit none
      integer,intent(in)::layernmb,thislayer(cv_maxnodenmb),            &
     &                    nmb_thislayer
      integer,intent(out)::istat
      integer i,ierr,inode,in,nfront,npiv,npiv_son,ison,ifather,        &
     &        nmb_type2
      logical doit
      character (len=48):: err_rep,subname
      istat=-1
      subname='DO_SPLITTING'
      if((layernmb.lt.0).or.(layernmb.gt.cv_maxlayer)) return
      if ((cv_slavef.eq.1).OR.(cv_slavef.lt.nmb_thislayer)) then
         istat=0
         return
      endif
      nmb_type2=0
      do i=1,nmb_thislayer
         inode=thislayer(i)
         nfront=cv_nfsiz(inode)
         in=inode
         npiv=0
         do while (in.gt.0)
            in=cv_fils(in)
            npiv=npiv+1
         end do
         if( AGMG_MUMPS_359(nfront,npiv))                                    &
     &                        nmb_type2=nmb_type2+1
      end do
      do i=1,nmb_thislayer
         ierr=0
         inode=thislayer(i)
         err_rep='SPLIT2HALVES'
         call AGMG_MUMPS_400(inode,                                          &
     &                              doit,npiv,nfront,npiv_son,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat =ierr
            return
         endif
         if(.NOT.doit) cycle
         err_rep='SPLITNODE'
         call AGMG_MUMPS_401(inode,nfront,npiv,npiv_son,                     &
     &                           ison,ifather,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat = ierr
            return
         endif
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_370
      subroutine AGMG_MUMPS_371(istat)
      implicit none
      integer, intent(out)::istat
      integer i,in,inode
      character (len=48):: subname
      istat=-1
      subname='ENCODE_PROCNODE'
      do i=1,cv_nbsa
         inode=cv_ssarbr(i)
         cv_nodetype(inode)=0
         in=cv_fils(inode)
         do while (in>0)
            in=cv_fils(in)
         end do
         in=-in
         do while(in.gt.0)
            call AGMG_MUMPS_406(in)
            in=cv_frere(in)
         enddo
      enddo
      do i=1,cv_n
         if (cv_frere(i).lt.cv_n+1) then
            if(cv_nodetype(i).eq.cv_invalid) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname
               return
            endif
            if (i.eq.cv_keep(38)) then
               cv_nodetype(i)=3
            endif
            cv_procnode(i)=(cv_nodetype(i)-1)*cv_slavef+cv_procnode(i)
            in=cv_fils(i)
            do while (in>0)
               cv_procnode(in)=cv_procnode(i)
               in=cv_fils(in)
            end do
         end if
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_371
      subroutine AGMG_MUMPS_372(ifather,istat)
      implicit none
      integer,intent(in)::ifather
      integer,intent(out)::istat
      integer in,son,ierr,allocok,oldl0end
      logical father_has_sons,fathfound
      character (len=48):: subname
      istat=-1
      subname='FATHSON_REPLACE'
      father_has_sons=.TRUE.
      in=ifather
      do while (in.gt.0)
         in=cv_fils(in)
      end do
      if(in.eq.0) then
         cv_nodelayer(ifather)=1
         father_has_sons=.FALSE.
      end if
      if(cv_layerl0_end-cv_layerl0_start.gt.0) then
         cv_layerl0_start= cv_layerl0_start+1
      elseif(father_has_sons) then
         cv_layerl0_start= cv_layerl0_start+1
      else
         istat=1
         cv_nodelayer(ifather)=0
         return
      endif
      cv_nbsa=cv_nbsa-1
      oldl0end = cv_layerl0_end
      if (father_has_sons) then
         son=-in
         son=-in
   10    continue
         cv_layerl0_end=cv_layerl0_end+1
         if (cv_tcostw(son).GT.mincostw)                                &
     &           layerL0_endforarrangeL0 = layerL0_endforarrangeL0+1
         cv_layerl0_array(cv_layerl0_end)=son
         cv_layerl0_sorted_costw(cv_layerl0_end)=cv_tcostw(son)
         cv_nbsa=cv_nbsa+1
         if((cv_frere(son).gt.0).and.(cv_frere(son).lt.cv_n+1)) then
            son=cv_frere(son)
            goto 10
         end if
      endif
         cv_costw_layer0=cv_costw_layer0 - cv_ncostw(ifather)
         cv_costm_layer0=cv_costm_layer0 - cv_ncostm(ifather)
         cv_costw_upper=cv_costw_upper + cv_ncostw(ifather)
         cv_costm_upper=cv_costm_upper + cv_ncostm(ifather)
         if(cv_layerl0_end.gt.oldl0end) then
            call AGMG_MUMPS_459(cv_layerl0_end-oldl0end,                     &
     &            cv_layerl0_array(oldl0end+1:cv_layerl0_end),          &
     &     cv_layerl0_sorted_costw(oldl0end+1:cv_layerl0_end))
           call AGMG_MUMPS_516(                                              &
     &        cv_layerl0_start,oldl0end,oldl0end-cv_layerl0_start+1,    &
     &        oldl0end+1,cv_layerl0_end,cv_layerl0_end-oldl0end,        &
     &        cv_layerl0_array,                                         &
     &        cv_layerl0_sorted_costw)
         endif
      istat=0
      return
      end subroutine AGMG_MUMPS_372
      subroutine AGMG_MUMPS_374(inode,map_strat,work,mem,                    &
     &                       workload,memused,proc,istat,respect_prop)
!DEC$ NOOPTIMIZE
      implicit none
      integer, intent(in)::inode,map_strat
      DOUBLE PRECISION,intent(in)::work,mem
      DOUBLE PRECISION,dimension(cv_slavef),intent(inout)::workload,    &
     &                                                     memused
      integer,intent(out):: proc,istat
      logical,intent(in),OPTIONAL::respect_prop
      integer i
      logical respect_proportional
      intrinsic huge
      DOUBLE PRECISION dummy
      character (len=48):: subname
      istat=-1
      respect_proportional=.FALSE.
      if(present(respect_prop)) respect_proportional=respect_prop
      subname='FIND_BEST_PROC'
      proc=-1
      if((map_strat.ne.cv_equilib_flops).and.                           &
     &   (map_strat.ne.cv_equilib_mem)) return
      dummy=huge(dummy)
      do i=cv_slavef,1,-1
         if (                                                           &
     &       ((.NOT.respect_proportional)                               &
     &        .OR.                                                      &
     &        (AGMG_MUMPS_481(inode,i).AND.respect_proportional))            &
     &      .AND.                                                       &
     &       (((workload(i).lt.dummy).AND.                              &
     &                         (map_strat.eq.cv_equilib_flops))         &
     &        .OR.                                                      &
     &       ((memused(i).lt.dummy).AND.                                &
     &                         (map_strat.eq.cv_equilib_mem))))then
            if((.not.cv_constr_work).or.                                &
     &         (workload(i)+work.lt.cv_proc_maxwork(i))) then
               if((.not.cv_constr_mem).or.                              &
     &            (memused(i)+mem.lt.cv_proc_maxmem(i))) then
                  proc=i
                  if(map_strat.eq.cv_equilib_flops) then
                     dummy=workload(i)
                  elseif(map_strat.eq.cv_equilib_mem) then
                     dummy=memused(i)
                  endif
               end if
            end if
         end if
      end do
      if (proc.ne.-1) then
         workload(proc)=workload(proc)+work
         memused(proc)=memused(proc)+mem
         istat=0
      end if
      return
      end subroutine AGMG_MUMPS_374
      subroutine AGMG_MUMPS_375(map_strat,istat)
      implicit none
      integer,intent(in)::map_strat
      integer,intent(out)::istat
      DOUBLE PRECISION aux_dbl,current_max,temp_max
      integer i,j,aux_int,inode,in,nfront,npiv,                         &
     &     nslaves_estim,strat,kmax,ncb,npropmap, blsize,               &
     &     nslaves_max, keep48_loc,current_max_node
      DOUBLE PRECISION wk_slave, wk_master
      DOUBLE PRECISION,DIMENSION(:), POINTER ::  costs
      logical doit
      character (len=48):: subname
      integer AGMG_MUMPS_497,                                                &
     &        AGMG_MUMPS_52
      external AGMG_MUMPS_497,                                               &
     &        AGMG_MUMPS_52
      istat=-1
      current_max_node=-9999
      current_max=0.0D0
      subname='FIND_POTSPLIT'
      if(map_strat.eq.cv_equilib_flops) then
         costs=>cv_ncostw
      elseif(map_strat.eq.cv_equilib_mem) then
         costs=>cv_ncostm
      endif
      if((cv_slavef.eq.1).OR.(cv_keep(62).lt.1))then
         cv_potsplit(1:cv_maxcut)=cv_invalid
         istat=0
         return
      end if
      current_max=huge(current_max);
      current_max_node=cv_n+1
      aux_int=1
      do i=1,min(cv_maxcut,cv_n)
         temp_max=0.0D0
         do j=1,cv_n
            if(costs(j).ge.temp_max)then
               if(((costs(j).eq.current_max).and.                       &
     &              (j.lt.current_max_node)).or.                        &
     &              (costs(j).lt.current_max))then
                  temp_max=costs(j)
                  inode=j
               endif
            endif
         end do
         current_max=costs(inode)
         current_max_node=inode
         if((inode.lt.0).OR.(inode.gt.cv_n)) cycle
         doit = .TRUE.
         if (cv_frere(inode).eq.0) cycle
         nfront = cv_nfsiz(inode)
         in = inode
         npropmap=0
         do j=1,cv_slavef
            if( AGMG_MUMPS_481(in,j)) then
               npropmap=npropmap+1
            endif
         end do
         npiv = 0
         do while(in>0)
            in = cv_fils(in)
            npiv = npiv + 1
         end do
         ncb = nfront - npiv
         kmax = AGMG_MUMPS_497(cv_keep8(21),ncb)
         if (cv_keep(50) ==  0) then
           keep48_loc=0
         else
           keep48_loc=3
         endif
         if (cv_keep(48).EQ.5) keep48_loc = 5
         nslaves_max   = AGMG_MUMPS_52(                                      &
     &        cv_slavef, keep48_loc, cv_keep8(21),                      &
     &        cv_keep(50), nfront, ncb )
         nslaves_estim = min(npropmap,nslaves_max)
         nslaves_estim = max(nslaves_estim,1)
         if (cv_keep(50).eq.0) then
            wk_master = dble(2)/dble(3)*                                &
     &                  dble(npiv)*dble(npiv)*dble(npiv)+               &
     &                  dble(npiv)*dble(npiv)*dble(nfront-npiv)
            wk_slave  = dble(npiv)*dble(nfront-npiv) *                  &
     &                  dble(2*nfront-npiv)/dble(nslaves_estim)
         else
            wk_master = dble(npiv)*dble(npiv)*dble(npiv)/dble(3)
            wk_slave  =                                                 &
     &           ( dble(npiv)*dble(nfront-npiv)*dble(nfront) )          &
     &            /   dble(nslaves_estim)
         end if
         strat = cv_keep(62)
         if(wk_master.le.                                               &
     &     ( dble(1)                                                    &
     &      +dble(strat)/dble(100) )                                    &
     &      *wk_slave) doit=.false.
         if ((cv_depth(inode).le.cv_maxdepth).and.doit) then
            cv_potsplit(aux_int)=inode
            aux_int=aux_int+1
         endif
      enddo
      cv_potsplit(aux_int:cv_maxcut)=cv_invalid
      istat=0
      return
      end subroutine AGMG_MUMPS_375
      subroutine AGMG_MUMPS_376(nmb,                                         &
     &                                   thislayer,nmb_thislayer,istat)
      implicit none
      integer, intent(in)::nmb
      integer,intent(out)::thislayer(cv_maxnodenmb),nmb_thislayer,istat
      integer i
      character (len=48):: subname
      istat=-1
      subname='FIND_THISLAYER'
      thislayer=0
      nmb_thislayer=0
      if((nmb.lt.0).or.(nmb.gt.cv_maxlayer)) return
      do i=1,cv_n
         if(cv_nodelayer(i).eq.nmb) then
            nmb_thislayer=nmb_thislayer+1
            if(nmb_thislayer.gt.cv_maxnodenmb) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Problem with nmb_thislayer in ',subname
               return
            endif
            thislayer(nmb_thislayer)=i
         end if
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_376
      subroutine AGMG_MUMPS_377(startlayer,thislayer,                        &
     &                 nmb_thislayer,cont,istat)
      implicit none
      integer,intent(in)::startlayer,nmb_thislayer
      integer,intent(in)::thislayer(cv_maxnodenmb)
      logical,intent(inout)::cont
      integer,intent(out)::istat
      integer :: visited
      integer il,i,current,in,ifather
      logical father_valid,upper_layer_exists
      character (len=48):: subname
      istat=-1
      subname='HIGHER_LAYER'
      if(.NOT.cont) return
      if(startlayer.lt.1) return
      current=startlayer-1
      visited = -current-1
         upper_layer_exists=.FALSE.
         if (current.eq.0) then
          do i=1,cv_n
            if (cv_nodelayer(i).ne.current) then
               if(cv_nodelayer(i).eq.1) then
                  upper_layer_exists=.TRUE.
                  exit
               endif
            endif
          enddo
         endif
         do il=1,nmb_thislayer
            i = thislayer(il)
            if (cv_nodelayer(i).lt.current) cycle
            in=i
            if(cv_frere(in).eq.0) cycle
            cv_nodelayer (in) = visited
               father_valid=.TRUE.
            do while(cv_frere(in).gt.0)
               if (cv_nodelayer(cv_frere(in)).gt.current) then
                  father_valid=.FALSE.
                  in = cv_frere(in)
                  cycle
               endif
               if (cv_nodelayer(cv_frere(in)).eq.visited) exit
               in=cv_frere(in)
               if (cv_nodelayer(in).eq.current)  then
                    cv_nodelayer(in) = visited
               endif
            end do
            if (.not.father_valid .or. cv_frere(in).gt.0) then
                    cycle
            endif
            ifather=-cv_frere(in)
            if(cv_nodelayer(ifather).eq.current+1) then
               cycle
            endif
            in=ifather
            do while (cv_fils(in).gt.0)
               in=cv_fils(in)
            end do
            in=-cv_fils(in)
            if(cv_nodelayer(in).gt.current) then
               father_valid=.FALSE.
            else
               father_valid=.TRUE.
               do while(cv_frere(in).gt.0)
                in=cv_frere(in)
                if(cv_nodelayer(in).gt.current) then
                       father_valid=.FALSE.
                       exit
                endif
                if(cv_nodelayer(in).eq.visited) then
                    exit
                endif
               end do
            endif
            if(father_valid) then
               cv_nodelayer(ifather)=current+1
               upper_layer_exists=.TRUE.
            end if
         end do
         if (upper_layer_exists) then
             current=current+1
             cv_maxlayer=current
             cont=.TRUE.
         else
             cv_maxlayer=current
             cont=.FALSE.
         endif
         do il=1,nmb_thislayer
            i = thislayer(il)
           if (cv_nodelayer(i).eq.visited) cv_nodelayer(i) = -visited-1
         enddo
      istat=0
      return
      end subroutine AGMG_MUMPS_377
      subroutine AGMG_MUMPS_478(n,slavef,                                    &
     &                    frere,fils,nfsiz,ne,keep,KEEP8,icntl,info,    &
     &                    procnode,ssarbr,peak,istat                    &
     &     )
      implicit none
      integer, intent(in)::n,slavef
      integer, intent(in), TARGET:: frere(n),fils(n),nfsiz(n),ne(n),    &
     &  keep(500),icntl(40),info(40),                                   &
     &  procnode(n),ssarbr(n)
      INTEGER*8, intent(in), TARGET:: KEEP8(150)
      integer,intent(out)::istat
      integer i,allocok,rest
      DOUBLE PRECISION peak
      character (len=48):: subname
      intrinsic bit_size,min,max
      istat=-1
      nullify(cv_frere,cv_fils,cv_nfsiz,cv_ne,cv_keep,cv_keep8,         &
     &        cv_icntl,cv_info,cv_procnode,cv_ssarbr)
      nullify(cv_ncostw,cv_tcostw,cv_ncostm,cv_tcostm,                  &
     &        cv_nodelayer,cv_nodetype,cv_depth,cv_potsplit,            &
     &        cv_layerworkload,cv_layermemused,cv_prop_map)
      subname='INITPART1'
      cv_n=n
      cv_slavef=slavef
      cv_stack_peak = peak
      cv_mem_strat = max((300 / cv_slavef),1)
      cv_keep=>keep
      cv_keep8=>KEEP8
      if(cv_keep(82) .lt. 0) then
         write(cv_lp,*)                                                 &
     &        'warning in mumps_static_mapping : keep(82) reset to 0'
         cv_keep(82) = 0
      endif
      if(cv_keep(83) .lt. 0) then
         write(cv_lp,*)                                                 &
     &        'warning in mumps_static_mapping : keep(83) reset to 0'
         cv_keep(83) = 0
      endif
      if(cv_keep(82) .gt. 0) then
         cv_maxcut=-99999
      else
         cv_maxcut =min(slavef,n/2)
      endif
      if(slavef.gt.1) then
         cv_mixed_strat_bound = max(cv_keep(78),1)
         cv_maxdepth = slavef
      else
         cv_maxdepth = 0
         cv_mixed_strat_bound=0
      endif
      cv_bitsize_of_int = bit_size(n)
      if(cv_bitsize_of_int.le.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Problem with bit size in ',subname
         return
      endif
      rest = mod(cv_slavef,cv_bitsize_of_int)
      if (rest.eq.0) then
         cv_size_ind_proc = cv_slavef / cv_bitsize_of_int
      else
         cv_size_ind_proc = cv_slavef / cv_bitsize_of_int + 1
      endif
      allocate(cv_ncostw(n),cv_tcostw(n),cv_ncostm(n),cv_tcostm(n),     &
     &       cv_nodelayer(n),cv_nodetype(n),cv_depth(n),                &
     &       cv_layerworkload(slavef),cv_layermemused(slavef),          &
     &       cv_prop_map(n),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 8*n+2*cv_slavef
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'memory allocation error in ',subname
         return
      end if
      if(cv_keep(82) .eq. 0) then
         allocate(cv_potsplit(cv_maxcut),STAT=allocok)
         if (allocok.gt.0) then
            cv_info(1) = cv_error_memalloc
            cv_info(2) = cv_maxcut
            istat = cv_error_memalloc
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'memory allocation error in ',subname
            return
         end if
      endif
      cv_frere=>frere
      cv_fils=>fils
      cv_nfsiz=>nfsiz
      cv_ne=>ne
      cv_icntl=>icntl
      cv_info=>info
      cv_procnode=>procnode
      cv_ssarbr=>ssarbr
      cv_ssarbr=0
      cv_nodetype=cv_invalid
      cv_nsteps=keep(28)
      if((keep(28).gt.n).OR.(keep(28).lt.0)) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'problem with nsteps in ',subname
         return
      end if
      cv_costw_upper=0.0D0
      cv_costm_upper=0.0D0
      cv_costw_layer0=0.0D0
      cv_costm_layer0=0.0D0
      cv_costw_total=0.0D0
      cv_costm_total=0.0D0
      cv_nodelayer=n+2
      cv_depth=cv_invalid
      if(cv_keep(82) .eq. 0) then
         cv_potsplit=cv_invalid
      endif
      cv_l0wthresh=0.0D0
      cv_splitthresh=0.45D0
      cv_relax=dble(1) + dble(max(0,keep(68)))/dble(100)
      cv_maxlayer=0
      if( cv_keep(82) .gt. 0) then
         cv_maxnsteps= cv_nsteps+1
      else
         cv_maxnsteps=cv_nsteps+cv_maxcut+1
      endif
      cv_layerworkload=dble(0)
      cv_layermemused=dble(0)
      cv_total_amalg=0
      cv_total_split=0
      cv_last_splitting%new_ison=cv_invalid
      cv_last_splitting%new_ifather=cv_invalid
      cv_last_splitting%old_keep2=cv_invalid
      cv_last_splitting%ncostw_oldinode=cv_d_invalid
      cv_last_splitting%ncostm_oldinode=cv_d_invalid
      cv_last_splitting%tcostw_oldinode=cv_d_invalid
      cv_last_splitting%tcostm_oldinode=cv_d_invalid
      do i=1,cv_n
         nullify(cv_prop_map(i)%ind_proc)
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_478
      subroutine AGMG_MUMPS_479(istat)
      implicit none
      integer,intent(out)::istat
      integer i,allocok,inode,in,inoderoot,ierr
      character (len=48):: subname
      type(nodelist),pointer::nodeptr
      istat=-1
      subname='INITPART2'
      if(associated(cv_layerl0_array))deallocate(cv_layerl0_array)
      if(associated(cv_layerl0_sorted_costw))                           &
     &           deallocate(cv_layerl0_sorted_costw)
      deallocate(cv_depth,cv_tcostw,cv_tcostm,STAT=ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      if(cv_maxnsteps.lt.1) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'problem with maxnsteps in ',subname
         return
      end if
      cv_maxnodenmb=cv_maxnsteps
      do i=1,cv_nbsa
         inode=cv_ssarbr(i)
         inoderoot=inode
  300    continue
         in = inode
         do while (in.ne.0)
            inode = in
            do while (in.gt.0)
            in = cv_fils(in)
            end do
            if (in.lt.0) in=-in
         end do
  100    continue
         if (inode.ne.inoderoot) then
            cv_maxnodenmb=cv_maxnodenmb-1
            in = cv_frere(inode)
            inode = abs(in)
            if (in.lt.0) then
               go to 100
            else
               go to 300
            end if
         end if
      end do
      if(cv_keep(82) .gt. 0) then
         cv_maxcut = min((cv_keep(82)-1)*cv_maxnodenmb,cv_n)
         cv_maxnsteps = min(cv_maxnsteps+cv_maxcut,cv_n)
         cv_maxnodenmb = cv_maxnsteps
      endif
      nullify(cv_layer_p2node)
      if(cv_maxnodenmb.lt.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'problem with maxnodenmb in ',subname
         return
      elseif(cv_maxnodenmb.lt.1) then
         cv_maxnodenmb = 1
      end if
      allocate(cv_layer_p2node(cv_maxnodenmb),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = cv_maxnodenmb
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'memory allocation error in ',subname
         return
      end if
      do i=1,cv_maxnodenmb
         nullify(cv_layer_p2node(i)%t2_nodenumbers,                     &
     &           cv_layer_p2node(i)%t2_cand,                            &
     &           cv_layer_p2node(i)%t2_candcostw,                       &
     &           cv_layer_p2node(i)%t2_candcostm)
         cv_layer_p2node(i)%nmb_t2s=0
      enddo
      istat = 0
      end subroutine AGMG_MUMPS_479
      function AGMG_MUMPS_359(nfront,npiv)
      implicit none
      logical::AGMG_MUMPS_359
      integer,intent(in)::nfront,npiv
      AGMG_MUMPS_359=.FALSE.
      if( (nfront - npiv > cv_keep(9))                                  &
     &     .and. ((npiv > cv_keep(4)).or.(.TRUE.))                      &
     &     .and. (cv_icntl(40).eq.0) ) AGMG_MUMPS_359=.TRUE.
      return
      end function AGMG_MUMPS_359
      subroutine AGMG_MUMPS_381(istat)
      implicit none
      integer,intent(out)::istat
      integer i,ierr,inode
      logical accepted,splitting_allowed
      integer,parameter::map_strat=cv_equilib_flops
      character (len=48):: err_rep,subname
      logical use_geist_ng_replace, skiparrangeL0
      INTEGER MINSIZE_L0
      istat=-1
      subname='LAYERL0'
      accepted=.FALSE.
      splitting_allowed=.TRUE.
      splitting_allowed=.FALSE.
      IF (cv_keep(72).EQ.2) THEN
       MINSIZE_L0 = 6*cv_slavef
      ELSE
       MINSIZE_L0 = 3*cv_slavef
      ENDIF
   55 continue
      skiparrangeL0 = .false.
      do while(.not.accepted)
         IF ( (    (layerL0_endforarrangeL0.LT.MINSIZE_L0)              &
     &             .OR. skiparrangeL0                                   &
     &        )                                                         &
     &        .AND.                                                     &
     &           (cv_layerl0_end.LT.cv_maxnsteps/2) ) THEN
          accepted = .false.
         ELSE
          err_rep='ARRANGEL0'
          call AGMG_MUMPS_415(map_strat, layerL0_endforarrangeL0,            &
     &                           cv_layerworkload,cv_layermemused,      &
     &                           cv_procnode,ierr)
          if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat = ierr
            return
          end if
          err_rep='ACCEPT_L0'
          call AGMG_MUMPS_413(map_strat,                                     &
     &                           cv_layerworkload,cv_layermemused,      &
     &                           accepted,ierr)
          if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat = ierr
            return
          end if
         ENDIF
         IF (cv_slavef.GT.16)                                           &
     &        skiparrangeL0 = .NOT.skiparrangeL0
         if (accepted.OR.(cv_costw_total.le.0.0D0)) then
            exit
         elseif(((cv_costw_layer0/cv_costw_total).gt.cv_l0wthresh) .AND.&
     &           (.TRUE.))then
            err_rep='MAX_TCOST_L0'
            inode = cv_layerl0_array(cv_layerl0_start)
            use_geist_ng_replace = .TRUE.
            if(use_geist_ng_replace) then
               err_rep='FATHSON_REPLACE'
               call AGMG_MUMPS_372(inode,ierr)
               if(ierr.eq.1) then
                  accepted=.TRUE.
               elseif(ierr.ne.0) then
                  if(cv_lp.gt.0)                                        &
     &            write(cv_lp,*)                                        &
     &            'Error rep. by ',err_rep,' in ',subname
                  istat = ierr
                  return
               endif
            endif
         else
            accepted=.TRUE.
         end if
      end do
      accepted=.TRUE.
      if (accepted) then
      else
         goto 55
      endif
      err_rep='LIST2LAYER'
      call AGMG_MUMPS_382(ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
         istat = ierr
         return
      end if
      err_rep='MAKE_PROPMAP'
      call AGMG_MUMPS_477(ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
         istat = ierr
         return
      end if
      if ( cv_keep(75).EQ.1 ) then
         call AGMG_MUMPS_415(map_strat, cv_layerl0_end,                      &
     &                     cv_layerworkload,cv_layermemused,            &
     &                     cv_procnode,ierr, respect_prop=.TRUE.)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat = ierr
            return
         end if
      else if (layerL0_endforarrangeL0.LT.cv_layerl0_end) THEN
         call AGMG_MUMPS_415(map_strat, cv_layerl0_end,                      &
     &                     cv_layerworkload,cv_layermemused,            &
     &                     cv_procnode,ierr)
      endif
      call AGMG_MUMPS_386(cv_procnode)
      do i=1,cv_slavef
         cv_proc_workload(i)=cv_layerworkload(i)
         cv_proc_memused(i)=cv_layermemused(i)
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_381
      subroutine AGMG_MUMPS_382(istat)
      implicit none
      integer, intent(out)::istat
      type(nodelist),pointer::dummy
      character (len=48):: subname
      integer i,inode
      istat=-1
      subname='LIST2LAYER'
      cv_dist_L0_mixed_strat_bound=0
      cv_nbsa=0
      do i=cv_layerl0_start,cv_layerl0_end
         inode=cv_layerl0_array(i)
         if(inode.gt.0) then
         cv_dist_L0_mixed_strat_bound=max(cv_dist_L0_mixed_strat_bound  &
     &        ,max(cv_depth(inode)-cv_mixed_strat_bound,0))
         cv_nodelayer(inode)=0
         cv_nbsa=cv_nbsa+1
         cv_ssarbr(cv_nbsa)=inode
         endif
      enddo
      istat=0
      return
      end subroutine AGMG_MUMPS_382
      subroutine AGMG_MUMPS_477(istat)
      implicit none
      integer,intent(out)::istat
      integer i,pctr,pctr2,ierr,procindex(cv_size_ind_proc)
      istat = -1
      pctr=cv_n
      pctr2=cv_mixed_strat_bound
      do i=1,cv_slavef
         call AGMG_MUMPS_482(procindex,i,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)write(cv_lp,*)                                &
     &           'BIT_SET signalled error to',subname
            istat = ierr
            return
         end if
      end do
      do i=1,cv_n
         if(cv_frere(i).eq.0) then
            if(.NOT.associated(cv_prop_map(i)%ind_proc)) then
               call AGMG_MUMPS_434(i,ierr)
               if(ierr.ne.0) then
                  if(cv_lp.gt.0)                                        &
     &                 write(cv_lp,*)'PROPMAP_INIT signalled error to'  &
     &                 ,subname
                  istat = ierr
                  return
               end if
            endif
            cv_prop_map(i)%ind_proc = procindex
            call AGMG_MUMPS_433(i,pctr,ierr)
            if(ierr.ne.0) then
            if(cv_lp.gt.0)write(cv_lp,*)                                &
     &           'PROPMAP signalled error to',subname
               istat = ierr
               return
            endif
            if((cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
               call AGMG_MUMPS_517(i,pctr2,ierr)
               if(ierr.ne.0) then
                  if(cv_lp.gt.0)write(cv_lp,*)                          &
     &           'MOD_PROPMAP signalled error to',subname
                  istat = ierr
                  return
               endif
            endif
         endif
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_477
      subroutine AGMG_MUMPS_387(layernmb,thislayer,                          &
     &   nmb_thislayer,map_strat,istat)
      implicit none
      integer, intent(in)::layernmb,thislayer(cv_maxnodenmb),           &
     &                     nmb_thislayer,map_strat
      integer,intent(out)::istat
      integer i,inode,j,k,ierr,nmb,aux_int,nmb_cand_needed
      DOUBLE PRECISION aux_dbl,aux_flop,aux_mem
      INTEGER candid(cv_slavef)
      integer sorted_nmb(2*nmb_thislayer)
      DOUBLE PRECISION sorted_costw(2*nmb_thislayer),                   &
     &                 sorted_costm(2*nmb_thislayer),                   &
     &                 old_workload(cv_slavef),old_memused(cv_slavef)
      character (len=48):: err_rep,subname
      logical use_propmap
      istat=-1
      subname='MAP_LAYER'
      if((cv_keep(24).eq.8).OR.(cv_keep(24).eq.10)                      &
     &   .OR.(cv_keep(24).eq.12).OR.(cv_keep(24).eq.14)                 &
     &   .OR.(cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
         use_propmap=.TRUE.
      else
         use_propmap=.FALSE.
      endif
      if((layernmb.lt.0).or.(layernmb.gt.cv_maxlayer)) return
      if((map_strat.ne.cv_equilib_flops).and.                           &
     &   (map_strat.ne.cv_equilib_mem)) return
      do i=1,nmb_thislayer
         inode=thislayer(i)
         if (cv_nodetype(inode).eq.3) then
            cv_procnode(inode)=1
            exit
         end if
      end do
      do i=1,cv_slavef
         old_workload(i)=cv_layerworkload(i)
         old_memused(i)=cv_layermemused(i)
      enddo
      nmb=0
      do i=1,nmb_thislayer
         inode=thislayer(i)
         if(cv_nodetype(inode).eq.1) then
            nmb=nmb+1
            sorted_nmb(nmb)=inode
            sorted_costw(nmb)=cv_ncostw(inode)
            sorted_costm(nmb)=cv_ncostm(inode)
         else if(cv_nodetype(inode).eq.2) then
            nmb=nmb+1
            do j=1,cv_layer_p2node(layernmb)%nmb_t2s
               if(cv_layer_p2node(layernmb)%t2_nodenumbers(j).ne.inode) &
     &            then
                  cycle
               else
                  sorted_costw(nmb)=                                    &
     &                 cv_layer_p2node(layernmb)%t2_candcostw(j)
                  sorted_costm(nmb)=                                    &
     &                 cv_layer_p2node(layernmb)%t2_candcostm(j)
               endif
            enddo
            if((sorted_costw(nmb).eq.cv_d_invalid).OR.                  &
     &           (sorted_costm(nmb).eq.cv_d_invalid)) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname
               return
            end if
            if(sorted_costw(nmb).lt.cv_ncostw(inode))then
               sorted_costw(nmb)=cv_ncostw(inode)
               sorted_costm(nmb)=cv_ncostm(inode)
               sorted_nmb(nmb)=inode
            else
               sorted_nmb(nmb)=-inode
            endif
         else if(cv_nodetype(inode).eq.3) then
            cycle
         else
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Unknown node type. Error in ',subname
            return
         end if
      end do
      if (map_strat.eq.cv_equilib_flops) then
         call AGMG_MUMPS_459(nmb,sorted_nmb(1:nmb),                          &
     &                   sorted_costw(1:nmb),sorted_costm(1:nmb))
      elseif(map_strat.eq.cv_equilib_mem) then
         call AGMG_MUMPS_459(nmb,sorted_nmb(1:nmb),                          &
     &                   sorted_costm(1:nmb),sorted_costw(1:nmb))
      endif
      do i=1,nmb
         aux_int=sorted_nmb(i)
         aux_flop=sorted_costw(i)
         aux_mem=sorted_costm(i)
         k=1
         if (aux_int.lt.0) then
            inode=-aux_int
            err_rep='SORTPROCS'
            if(use_propmap) then
               call AGMG_MUMPS_398(map_strat,                                &
     &              cv_proc_workload,cv_proc_memused,                   &
     &              inode=inode,istat=ierr)
            else
               call AGMG_MUMPS_398(map_strat,                                &
     &              cv_proc_workload,cv_proc_memused,                   &
     &              istat=ierr)
            end if
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)                                           &
     &         'Error reported by ',err_rep,' in ',subname
               istat = ierr
               return
            endif
            nmb_cand_needed=cv_invalid
            do j=1,cv_layer_p2node(layernmb)%nmb_t2s
               if(cv_layer_p2node(layernmb)%t2_nodenumbers(j).ne.inode) &
     &            then
                  cycle
               else
                  nmb_cand_needed=                                      &
     &                  cv_layer_p2node(layernmb)%t2_cand(j,cv_slavef+1)
                  exit
               endif
            enddo
            if(nmb_cand_needed.eq.cv_invalid) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname
               return
            endif
            do while((k.le.cv_slavef).and.(nmb_cand_needed.gt.0))
               if(((.not.cv_constr_work).or.                            &
     &            (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.     &
     &             cv_proc_maxwork(cv_proc_sorted(k))))                 &
     &           .AND.((.not.cv_constr_mem).or.                         &
     &            (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.       &
     &             cv_proc_maxmem(cv_proc_sorted(k))))                  &
     &           .AND.                                                  &
     & (cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k)).eq.0))   &
     &            then
                  cv_proc_workload(cv_proc_sorted(k))=                  &
     &                 cv_proc_workload(cv_proc_sorted(k))+aux_flop
                  cv_proc_memused(cv_proc_sorted(k))=                   &
     &                 cv_proc_memused(cv_proc_sorted(k))+aux_mem
                  cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k))&
     &                                             =inode
                  cv_layerworkload(cv_proc_sorted(k))=                  &
     &                 cv_layerworkload(cv_proc_sorted(k))+aux_flop
                  cv_layermemused(cv_proc_sorted(k))=                   &
     &                 cv_layermemused(cv_proc_sorted(k))+aux_mem
                  nmb_cand_needed=nmb_cand_needed-1
                  k=k+1
               else
                  k=k+1
                  if(k.gt.cv_slavef) then
                     if(cv_lp.gt.0)                                     &
     &                    write(cv_lp,*)'Error in ',subname
                     return
                  endif
               end if
            end do
            if(nmb_cand_needed.gt.0) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname
               return
            endif
            aux_flop=cv_ncostw(inode)
            aux_mem=cv_ncostm(inode)
            do while(k.le.cv_slavef)
               if(((.not.cv_constr_work).or.                            &
     &              (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.   &
     &              cv_proc_maxwork(cv_proc_sorted(k))))                &
     &              .AND.((.not.cv_constr_mem).or.                      &
     &              (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.     &
     &              cv_proc_maxmem(cv_proc_sorted(k))))                 &
     &              .AND.                                               &
     & (cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k)).eq.0))   &
     &              then
                  cv_procnode(inode)=cv_proc_sorted(k)
                  cv_proc_workload(cv_proc_sorted(k))=                  &
     &                 cv_proc_workload(cv_proc_sorted(k))+aux_flop
                  cv_proc_memused(cv_proc_sorted(k))=                   &
     &                 cv_proc_memused(cv_proc_sorted(k))+aux_mem
                  cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k))&
     &                 =-inode
                  cv_layerworkload(cv_proc_sorted(k))=                  &
     &                 cv_layerworkload(cv_proc_sorted(k))+aux_flop
                  cv_layermemused(cv_proc_sorted(k))=                   &
     &                 cv_layermemused(cv_proc_sorted(k))+aux_mem
                  exit
               else
                  k=k+1
                  if(k.gt.cv_slavef) then
                     if(cv_lp.gt.0)                                     &
     &                    write(cv_lp,*)'Error in ',subname
                     return
                  endif
               end if
            end do
         else
            inode=aux_int
            err_rep='SORTPROCS'
            if(use_propmap) then
               call AGMG_MUMPS_398(map_strat,                                &
     &                        cv_proc_workload,cv_proc_memused,         &
     &                        inode=inode,istat=ierr)
            else
               call AGMG_MUMPS_398(map_strat,                                &
     &                        cv_proc_workload,cv_proc_memused,         &
     &                        inode,istat=ierr)
            endif
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &          write(cv_lp,*)                                          &
     &          'Error reported by ',err_rep,' in ',subname
               istat = ierr
               return
            endif
            if (cv_nodetype(inode).eq.1) then
               do while(k.le.cv_slavef)
                  if((.not.cv_constr_work).or.                          &
     &               (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.  &
     &                cv_proc_maxwork(cv_proc_sorted(k)))               &
     &            .AND.((.not.cv_constr_mem).or.                        &
     &               (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.    &
     &                cv_proc_maxmem(cv_proc_sorted(k))))) then
                     cv_procnode(inode)=cv_proc_sorted(k)
                     cv_proc_workload(cv_proc_sorted(k))=               &
     &                    cv_proc_workload(cv_proc_sorted(k))+aux_flop
                     cv_proc_memused(cv_proc_sorted(k))=                &
     &                    cv_proc_memused(cv_proc_sorted(k))+aux_mem
                     cv_layerworkload(cv_proc_sorted(k))=               &
     &                    cv_layerworkload(cv_proc_sorted(k))+aux_flop
                     cv_layermemused(cv_proc_sorted(k))=                &
     &                    cv_layermemused(cv_proc_sorted(k))+aux_mem
                     exit
                  else
                     k=k+1
                     if(k.gt.cv_slavef) then
                        if(cv_lp.gt.0)                                  &
     &                  write(cv_lp,*)'Inconsist data in ',subname
                        return
                     endif
                  end if
               end do
            elseif (cv_nodetype(inode).eq.2)then
               do j=1,cv_layer_p2node(layernmb)%nmb_t2s
                  if(cv_layer_p2node(layernmb)%t2_nodenumbers(j).ne.    &
     &               inode) then
                     cycle
                  else
                     exit
                  endif
               enddo
               do while(k.le.cv_slavef)
                  if(((.not.cv_constr_work).or.                         &
     &               (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.  &
     &                cv_proc_maxwork(cv_proc_sorted(k))))              &
     &              .AND.((.not.cv_constr_mem).or.                      &
     &               (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.    &
     &                cv_proc_maxmem(cv_proc_sorted(k))))               &
     &              .AND.                                               &
     & (cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k)).eq.0))   &
     &               then
                     cv_procnode(inode)=cv_proc_sorted(k)
                     cv_proc_workload(cv_proc_sorted(k))=               &
     &                    cv_proc_workload(cv_proc_sorted(k))+aux_flop
                     cv_proc_memused(cv_proc_sorted(k))=                &
     &                     cv_proc_memused(cv_proc_sorted(k))+aux_mem
                cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k))  &
     &                                           =-inode
                     cv_layerworkload(cv_proc_sorted(k))=               &
     &                    cv_layerworkload(cv_proc_sorted(k))+aux_flop
                     cv_layermemused(cv_proc_sorted(k))=                &
     &                    cv_layermemused(cv_proc_sorted(k))+aux_mem
                     exit
                  else
                     k=k+1
                     if(k.gt.cv_slavef) then
                        if(cv_lp.gt.0)                                  &
     &                       write(cv_lp,*)'Error in ',subname
                        return
                     endif
                  end if
               end do
               nmb_cand_needed=cv_invalid
               do j=1,cv_layer_p2node(layernmb)%nmb_t2s
                  if(cv_layer_p2node(layernmb)%t2_nodenumbers(j)        &
     &                 .ne.inode)                                       &
     &                 then
                     cycle
                  else
                     nmb_cand_needed=                                   &
     &                    cv_layer_p2node(layernmb)%                    &
     &                    t2_cand(j,cv_slavef+1)
                     exit
                  endif
               enddo
               if(nmb_cand_needed.eq.cv_invalid) then
                  if(cv_lp.gt.0)                                        &
     &                 write(cv_lp,*)'Error in ',subname
                  return
               endif
               aux_flop=                                                &
     &              cv_layer_p2node(layernmb)%t2_candcostw(j)
               aux_mem=                                                 &
     &              cv_layer_p2node(layernmb)%t2_candcostm(j)
               do while((k.le.cv_slavef).and.(nmb_cand_needed.gt.0))
                  if(((.not.cv_constr_work).or.                         &
     &                 (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.&
     &                 cv_proc_maxwork(cv_proc_sorted(k))))             &
     &                 .AND.((.not.cv_constr_mem).or.                   &
     &                 (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.  &
     &                 cv_proc_maxmem(cv_proc_sorted(k))))              &
     &                 .AND.                                            &
     &                 (cv_layer_p2node(layernmb)%                      &
     &                 t2_cand(j,cv_proc_sorted(k)).eq.0))              &
     &                 then
                     cv_proc_workload(cv_proc_sorted(k))=               &
     &                    cv_proc_workload(cv_proc_sorted(k))+aux_flop
                     cv_proc_memused(cv_proc_sorted(k))=                &
     &                    cv_proc_memused(cv_proc_sorted(k))+aux_mem
                     cv_layer_p2node(layernmb)%                         &
     &                    t2_cand(j,cv_proc_sorted(k))                  &
     &                    =inode
                     cv_layerworkload(cv_proc_sorted(k))=               &
     &                    cv_layerworkload(cv_proc_sorted(k))+aux_flop
                     cv_layermemused(cv_proc_sorted(k))=                &
     &                    cv_layermemused(cv_proc_sorted(k))+aux_mem
                     nmb_cand_needed=nmb_cand_needed-1
                     k=k+1
                  else
                     k=k+1
                     if(k.gt.cv_slavef) then
                        if(cv_lp.gt.0)                                  &
     &                       write(cv_lp,*)'Error in ',subname
                        return
                     endif
                  end if
               end do
               if(nmb_cand_needed.gt.0) then
                  if(cv_lp.gt.0)                                        &
     &                 write(cv_lp,*)'Error in ',subname
                  return
               endif
            end if
         end if
      end do
      do i=1,cv_layer_p2node(layernmb)%nmb_t2s
         nmb_cand_needed=                                               &
     &                  cv_layer_p2node(layernmb)%t2_cand(i,cv_slavef+1)
         candid= cv_layer_p2node(layernmb)%t2_cand(i,1:cv_slavef)
         cv_layer_p2node(layernmb)%t2_cand(i,1:cv_slavef)=-1
         k=0
         do j=1,cv_slavef
            if(candid(j).gt.0) then
               k=k+1
               cv_layer_p2node(layernmb)%t2_cand(i,k)=j-1
            end if
         end do
         if (k.ne.nmb_cand_needed) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Error in ',subname
            return
         endif
      enddo
      do i=1,cv_slavef
         cv_layerworkload(i)=cv_layerworkload(i)-old_workload(i)
         cv_layermemused(i)=cv_layermemused(i)-old_memused(i)
      enddo
      istat=0
      return
      end subroutine AGMG_MUMPS_387
      recursive subroutine AGMG_MUMPS_385(inode,procnmb,                     &
     &                                       procnode)
      integer,intent(in)::inode,procnmb
      integer,intent(inout)::procnode(cv_n)
      integer in
      procnode(inode)=procnmb
      if (cv_fils(inode).eq.0) return
      in=cv_fils(inode)
      do while(in>0)
         procnode(in)=procnmb
         in=cv_fils(in)
      end do
      in=-in
      do while(in>0)
         call AGMG_MUMPS_385(in,procnmb,procnode)
         in=cv_frere(in)
      end do
      return
      end subroutine AGMG_MUMPS_385
      subroutine AGMG_MUMPS_386(procnode)
      implicit none
      integer,intent(inout)::procnode(cv_n)
      integer i,inode,procnmb
      type(nodelist),pointer::dummy
      do i=cv_layerl0_start,cv_layerl0_end
         inode=cv_layerl0_array(i)
         if(inode.gt.0) then
            procnmb=procnode(inode)
            call AGMG_MUMPS_385(inode,procnmb,procnode)
         endif
      enddo
      return
      end subroutine AGMG_MUMPS_386
      subroutine AGMG_MUMPS_431()
      implicit none
      integer candid,inode,index,i,j,layernmb,master,nmbcand,swapper,   &
     &        totalnmb,node_of_master,node_of_candid,node_of_swapper
      DOUBLE PRECISION::mastermem,slavemem,maxmem
      logical swapthem,cand_better_master_arch,cand_better_swapper_arch
      intrinsic maxval,minval
      maxmem=maxval(cv_proc_memused(:))
      totalnmb=0
      do layernmb=cv_maxlayer,1,-1
         do i=1,cv_layer_p2node(layernmb)%nmb_t2s
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(i)
            master=cv_procnode(inode)
            if(ke69 .gt. 1) then
               allowed_nodes = .FALSE.
               call AGMG_MUMPS_476(layernmb,i)
               node_of_master = mem_distribmpi(master-1)
               if (node_of_master .lt. 0 ) then
                if(cv_mp.gt.0) write(cv_mp,*)'node_of_master_not found'
               endif
               node_of_swapper = node_of_master
            endif
            mastermem=cv_proc_memused(master)
            nmbcand=cv_layer_p2node(layernmb)%t2_cand(i,cv_slavef+1)
            swapper=master
            index=0
            do j=1,nmbcand
               candid=cv_layer_p2node(layernmb)%t2_cand(i,j)+1
               slavemem=cv_proc_memused(candid)
               if(ke69 .gt. 1) then
                  node_of_candid = mem_distribmpi(candid-1)
                  if (node_of_candid .lt. 0 ) then
                     if(cv_mp.gt.0) write(cv_mp,*)                      &
     &               'node_of_candid_not found'
                  endif
               endif
               if(ke69 .le. 1) then
                  if((slavemem.lt.mastermem) .and.                      &
     &                 (slavemem.lt.cv_proc_memused(swapper))) then
                     swapper=candid
                     index=j
                  endif
               else
                  cand_better_master_arch = (                           &
     &                    (                                             &
     &                       (slavemem.lt.mastermem) .or.               &
     &                       (.not. allowed_nodes(node_of_master))      &
     &                    )                                             &
     &                    .and. allowed_nodes(node_of_candid)           &
     &                 )
                  cand_better_swapper_arch = (                          &
     &                    (                                             &
     &                       (slavemem.lt.cv_proc_memused(swapper)) .or.&
     &                       (.not. allowed_nodes(node_of_swapper))     &
     &                    )                                             &
     &                    .and. allowed_nodes(node_of_candid)           &
     &                 )
                  if(cand_better_master_arch .and.                      &
     &                 cand_better_swapper_arch  ) then
                     swapper=candid
                     node_of_swapper = node_of_candid
                     index=j
                  endif
               endif
            enddo
            if(swapper.ne.master) then
               swapthem = .FALSE.
               if(0.75D0*mastermem.ge.cv_proc_memused(swapper))         &
     &            swapthem=.TRUE.
               if(mastermem.le.mastermem-cv_ncostm(inode)               &
     &                +cv_layer_p2node(layernmb)%t2_candcostm(i))       &
     &            swapthem=.FALSE.
               if(mastermem.le.cv_proc_memused(swapper)                 &
     &                    +cv_ncostm(inode)                             &
     &                    -cv_layer_p2node(layernmb)%t2_candcostm(i))   &
     &            swapthem=.FALSE.
               if(maxmem.le.mastermem-cv_ncostm(inode)                  &
     &                +cv_layer_p2node(layernmb)%t2_candcostm(i))       &
     &            swapthem=.FALSE.
               if(maxmem.le.cv_proc_memused(swapper)+cv_ncostm(inode)   &
     &                -cv_layer_p2node(layernmb)%t2_candcostm(i))       &
     &            swapthem=.FALSE.
               if(ke69 .gt. 1) then
                  if (.not. allowed_nodes(node_of_master)) then
                     swapthem=.TRUE.
                  endif
               endif
               if(.NOT.swapthem) cycle
               cv_proc_workload(master)=cv_proc_workload(master)        &
     &                -cv_ncostw(inode)                                 &
     &                +cv_layer_p2node(layernmb)%t2_candcostw(i)
               cv_proc_memused(master)=cv_proc_memused(master)          &
     &                -cv_ncostm(inode)                                 &
     &                +cv_layer_p2node(layernmb)%t2_candcostm(i)
               cv_proc_workload(swapper)=cv_proc_workload(swapper)      &
     &                +cv_ncostw(inode)                                 &
     &                -cv_layer_p2node(layernmb)%t2_candcostw(i)
               cv_proc_memused(swapper)=cv_proc_memused(swapper)        &
     &                +cv_ncostm(inode)                                 &
     &                -cv_layer_p2node(layernmb)%t2_candcostm(i)
               cv_layer_p2node(layernmb)%t2_cand(i,index)=master-1
               cv_procnode(inode)=swapper
               maxmem=maxval(cv_proc_memused(:))
               totalnmb = totalnmb+1
            endif
         enddo
      enddo
      end subroutine AGMG_MUMPS_431
      subroutine AGMG_MUMPS_391(maxwork,maxmem,istat)
      implicit none
      DOUBLE PRECISION,intent(in),OPTIONAL::maxwork(cv_slavef),         &
     &                                      maxmem(cv_slavef)
      integer,intent(out)::istat
      integer i,allocok
      intrinsic huge
      DOUBLE PRECISION dummy
      character (len=48):: subname
      istat=-1
      subname='PROCINIT'
      if(present(maxwork)) then
         cv_constr_work=.TRUE.
      else
         cv_constr_work=.FALSE.
      end if
      if(present(maxmem)) then
         cv_constr_mem=.TRUE.
      else
         cv_constr_mem=.FALSE.
      end if
      allocate(cv_proc_workload(cv_slavef),                             &
     &         cv_proc_maxwork(cv_slavef),                              &
     &         cv_proc_memused(cv_slavef),                              &
     &         cv_proc_maxmem(cv_slavef),                               &
     &         cv_proc_sorted(cv_slavef),                               &
     &         STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 2*cv_slavef
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'memory allocation error in ',subname
         return
      end if
      allocate(work_per_proc(cv_slavef),id_son(cv_slavef),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 2*cv_slavef
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'memory allocation error in ',subname
         return
      end if
      do i=1,cv_slavef
         cv_proc_workload(i)=dble(0)
         if(cv_constr_work) then
            cv_proc_maxwork(i)=maxwork(i)
         else
            cv_proc_maxwork(i)=(huge(dummy))
         endif
         cv_proc_memused(i)=dble(0)
         if(cv_constr_mem) then
            cv_proc_maxmem(i)=maxmem(i)
         else
            cv_proc_maxmem(i)=(huge(dummy))
         endif
      end do
      cv_proc_sorted=(/(i,i=1,cv_slavef)/)
      istat=0
      return
      end subroutine AGMG_MUMPS_391
      recursive subroutine AGMG_MUMPS_517                                    &
     &                    (inode,ctr,istat)
      implicit none
      integer, intent(in)::inode,ctr
      integer, intent(inout)::istat
      integer::j,k,in,in1,ierr,son,nmb_procs_inode,nmb_sons_inode,      &
     &           procs4son(cv_size_ind_proc),current,i
      character (len=48):: subname
      DOUBLE PRECISION :: relative_weight,costs_sons
      DOUBLE PRECISION :: loc_relax
      INTEGER :: depth
      logical force_cand
      DOUBLE PRECISION Y
      intrinsic random_number
      integer nmb_propmap,nmb_propmap_strict,share2,procsrest,current2
      integer k69onid
      integer procs_inode(slavef)
      if (ctr.le.0) then
         istat = 0
         return
      endif
      procs_inode=-1
      istat= -1
      if(cv_frere(inode).eq.cv_n+1) return
      subname='MOD_PROPMAP'
      if(.NOT.associated(cv_prop_map(inode)%ind_proc)) return
      nmb_procs_inode = 0
      do j=1,cv_slavef
         if( AGMG_MUMPS_481(inode,j))then
            nmb_procs_inode = nmb_procs_inode + 1
         endif
      end do
      i=0
      do j=1,cv_slavef
            if(ke69 .gt.1) then
               call AGMG_MUMPS_493(j-1,                                      &
     &              k69onid,ierr)
            else
               k69onid = j
            endif
            if(AGMG_MUMPS_481(inode,k69onid))then
               i = i + 1
               procs_inode(i)=k69onid
            endif
      end do
      if(i.ne.nmb_procs_inode)then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      endif
      if(nmb_procs_inode.eq.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      end if
      if ((cv_nodelayer(inode).eq.0).AND.                               &
     &         (cv_frere(inode).ne.cv_n+1)) then
         istat = 0
         return
      endif
      nmb_sons_inode = 0
      costs_sons = dble(0)
      force_cand=(mod(cv_keep(24),2).eq.0)
      in = inode
      do while (cv_fils(in).gt.0)
         in=cv_fils(in)
      end do
      if (cv_fils(in).eq.0) then
         istat = 0
         return
      endif
      in = -cv_fils(in)
      son=in
      do while(in.gt.0)
         nmb_sons_inode = nmb_sons_inode + 1
         if(cv_tcostw(in).le.0.0D0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Subtree costs for ',in,                 &
     &                         ' should be positive in ',subname
            return
         endif
         costs_sons = costs_sons + cv_tcostw(in)
         in=cv_frere(in)
      enddo
      if(costs_sons.le.0D0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      endif
      depth= max(cv_mixed_strat_bound - ctr,0)
      if ((cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
         if(depth.ge.cv_mixed_strat_bound) then
            loc_relax = dble(1)
         else
            loc_relax =  dble(1) +                                      &
     &            max(dble(cv_keep(77))/dble(100), dble(0))
         endif
      else
         loc_relax = dble(1)
      endif
      in=son
      current = 1
      do while(in.gt.0)
         if( (nmb_sons_inode.ge.nmb_procs_inode).AND.                   &
     &       (nmb_procs_inode.LT.4) ) then
            procs4son = cv_prop_map(inode)%ind_proc
         else
            do k=1,cv_size_ind_proc
               do j=0,cv_bitsize_of_int-1
                  procs4son(k)=ibclr(procs4son(k),j)
               end do
            end do
            nmb_propmap_strict=0
            do k=1,cv_slavef
               if( AGMG_MUMPS_481(in,k)) then
                  nmb_propmap_strict=nmb_propmap_strict+1
                  call AGMG_MUMPS_482(procs4son,k,ierr)
               end if
            end do
            if(costs_sons.gt.0.0D0) then
               relative_weight=cv_tcostw(in)/costs_sons
            else
               relative_weight=0.0D0
            endif
            current = nmb_propmap_strict
            share2=                                                     &
     &           max(0,nint(relative_weight*(loc_relax-dble(1))*        &
     &           dble(nmb_procs_inode)))
            procsrest=nmb_procs_inode - nmb_propmap_strict
            share2=min(share2,procsrest)
            CALL RANDOM_NUMBER(Y)
            current2=int(dble(Y)*dble(procsrest))
            k=1
            i=1
            do while((share2.gt.0).and.(i.le.2))
               do j=1,nmb_procs_inode
                  if(share2.le.0) exit
                  k69onid = procs_inode(j)
                  if(( AGMG_MUMPS_481(inode,k69onid)).AND.                   &
     &                 (.NOT.AGMG_MUMPS_480(procs4son,k69onid))) then
                     if(k.ge.current2)then
                        call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                        if(ierr.ne.0) then
                           if(cv_lp.gt.0)write(cv_lp,*)                 &
     &                          'BIT_SET signalled error to',subname
                           istat = ierr
                           return
                        end if
                        share2 = share2 - 1
                     endif
                     k=k+1
                  end if
               enddo
               i=i+1
            enddo
            if(share2.ne.0) then
               if(cv_lp.gt.0) write(cv_lp,*)                            &
     &           'Error reported in ',subname
               return
            end if
         end if
         ierr=0
         in1=in
         cv_prop_map(in1)%ind_proc=procs4son
         call AGMG_MUMPS_517(in1,ctr-1,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0) write(cv_lp,*)                               &
     &           'Error reported in ',subname
            istat=ierr
            return
         endif
         in=cv_frere(in)
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_517
      recursive subroutine AGMG_MUMPS_433(inode,ctr,istat)
      implicit none
      integer, intent(in)::inode,ctr
      integer, intent(inout)::istat
      integer::j,k,in,in1,ierr,son,nmb_procs_inode,nmb_sons_inode,      &
     &           share,procs4son(cv_size_ind_proc),current,offset,      &
     &           in_tmp,nfront,npiv,ncb,                                &
     &           keep48_loc,min_cand_needed
      character (len=48):: subname
      DOUBLE PRECISION :: relative_weight,costs_sons, shtemp
      DOUBLE PRECISION :: costs_sons_real
      DOUBLE PRECISION :: PartofaProc
      LOGICAL          :: SkipSmallNodes
      PARAMETER (PartofaProc=0.01D0)
      DOUBLE PRECISION :: loc_relax
      INTEGER :: depth
      logical force_cand
      integer AGMG_MUMPS_497, AGMG_MUMPS_50
      external AGMG_MUMPS_497, AGMG_MUMPS_50
      DOUBLE PRECISION Y
      intrinsic random_number
      integer nmb_propmap_strict,share2,procsrest,current2
      integer k69onid,nb_free_procs,local_son_indice,nb_procs_for_sons, &
     &     ptr_upper_ro_procs
      logical upper_round_off,are_sons_treated
      DOUBLE PRECISION tmp_cost
      if (ctr.le.0) then
         istat = 0
         return
      endif
      istat= -1
      if(cv_frere(inode).eq.cv_n+1) return
      subname='PROPMAP'
      nmb_procs_inode = 0
      do j=1,cv_slavef
         if( AGMG_MUMPS_481(inode,j))                                        &
     &        nmb_procs_inode = nmb_procs_inode + 1
      end do
      if(nmb_procs_inode.eq.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      end if
      if ((cv_nodelayer(inode).eq.0).AND.                               &
     &         (cv_frere(inode).ne.cv_n+1)) then
         istat = 0
         return
      endif
      ptr_upper_ro_procs=1
      work_per_proc(1:cv_slavef)=0.0D0
      id_son(1:cv_slavef)=0
      nmb_sons_inode = 0
      costs_sons = dble(0)
      force_cand=(mod(cv_keep(24),2).eq.0)
      min_cand_needed=0
      in = inode
      do while (cv_fils(in).gt.0)
         in=cv_fils(in)
      end do
      if (cv_fils(in).eq.0) then
         istat = 0
         return
      endif
      in = -cv_fils(in)
      son=in
      do while(in.gt.0)
         nmb_sons_inode = nmb_sons_inode + 1
         if(cv_tcostw(in).le.0.0D0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Subtree costs for ',in,                 &
     &                         ' should be positive in ',subname
            return
         endif
         costs_sons = costs_sons + cv_tcostw(in)
         in=cv_frere(in)
      enddo
      costs_sons_real = costs_sons
      SkipSmallNodes = .true.
      IF (costs_sons_real.gt.0.0D0) then
       in = son
       do while (in.gt.0)
          relative_weight=cv_tcostw(in)/costs_sons_real
          shtemp = relative_weight*dble(nmb_procs_inode)
          IF (shtemp.lt.PartofaProc) THEN
            costs_sons = costs_sons - cv_tcostw(in)
          ENDIF
          in=cv_frere(in)
       enddo
       IF (costs_sons.LT. PartofaProc*costs_sons_real) THEN
         costs_sons = costs_sons_real
         SkipSmallNodes = .false.
       ENDIF
      ENDIF
      if(costs_sons.le.0.0D0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      endif
      if(cv_relax.le.0.0D0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname,'. Wrong cv_relax'
         return
      endif
      depth= max(cv_n - ctr,0)
      if(cv_keep(24).eq.8) then
         loc_relax = cv_relax
      elseif ((cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
         loc_relax = cv_relax
      elseif (cv_keep(24).eq.10) then
         loc_relax = cv_relax
      elseif ((cv_keep(24).eq.12).OR.(cv_keep(24).eq.14)) then
         if(depth.ge.cv_mixed_strat_bound) then
            loc_relax = cv_relax
         else
            loc_relax =  cv_relax +                                     &
     &          max(dble(cv_keep(77))/dble(100), dble(0))
         endif
      endif
      in=son
      current = 1
      local_son_indice=1
      nb_procs_for_sons=0
      upper_round_off=.FALSE.
      are_sons_treated=.TRUE.
      do while(in.gt.0)
         if( (nmb_sons_inode.ge.nmb_procs_inode).AND.                   &
     &       (nmb_procs_inode.LT.4) ) then
            procs4son = cv_prop_map(inode)%ind_proc
            are_sons_treated=.FALSE.
            nb_procs_for_sons=nmb_procs_inode
            nmb_propmap_strict=nmb_procs_inode
         elseif(nmb_procs_inode .LE. cv_keep(83)) then
            procs4son = cv_prop_map(inode)%ind_proc
            are_sons_treated=.FALSE.
            nb_procs_for_sons=nmb_procs_inode
            nmb_propmap_strict=nmb_procs_inode
         else
            do k=1,cv_size_ind_proc
               do j=0,cv_bitsize_of_int-1
                  procs4son(k)=ibclr(procs4son(k),j)
               end do
            end do
            if(costs_sons.gt.0.0D0) then
               relative_weight=cv_tcostw(in)/costs_sons
            else
               relative_weight=dble(0)
            endif
            shtemp = relative_weight*dble(nmb_procs_inode)
            IF ( (shtemp.LT.PartofaProc)                                &
     &             .AND. ( SkipSmallNodes ) )  THEN
             share = 1
             do j=current,cv_slavef
              if(ke69 .gt.1) then
               call AGMG_MUMPS_493(j-1,k69onid,ierr)
              else
               k69onid = j
              endif
              if( AGMG_MUMPS_481(inode,k69onid)) then
                  call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                  if(ierr.ne.0) then
                     if(cv_lp.gt.0)write(cv_lp,*)                       &
     &               'BIT_SET signalled error to',subname
                     istat = ierr
                     return
                  end if
                  share = share -1
                  exit
              endif
             enddo
             if (share.gt.0) then
               do j=1,current-1
                if(ke69 .gt.1) then
                 call AGMG_MUMPS_493(j-1,k69onid,ierr)
                else
                 k69onid = j
                endif
                if( AGMG_MUMPS_481(inode,k69onid)) then
                    call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                    if(ierr.ne.0) then
                       if(cv_lp.gt.0)write(cv_lp,*)                     &
     &               'BIT_SET signalled error to',subname
                       istat = ierr
                       return
                    end if
                    share = share -1
                    exit
                endif
               enddo
              endif
              if(share.ne.0) then
                 if(cv_lp.gt.0) write(cv_lp,*)                          &
     &              'Error reported in ',subname
                 return
              end if
              if(.NOT.associated(cv_prop_map(in)%ind_proc)) then
                 call AGMG_MUMPS_434(in,ierr)
              if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &              write(cv_lp,*)'PROPMAP_INIT signalled error to'     &
     &              ,subname
               istat = ierr
               return
              end if
             endif
             current  = j
             cv_prop_map(in)%ind_proc = procs4son
             in = cv_frere(in)
             cycle
            ENDIF
            share  = max(1,nint(shtemp))
            if (dble(share).ge.shtemp) then
               upper_round_off=.TRUE.
            else
               upper_round_off = .FALSE.
            endif
            share=min(share,nmb_procs_inode)
            nmb_propmap_strict=share
            nb_procs_for_sons=nb_procs_for_sons+nmb_propmap_strict
            offset=1
            do j=current,cv_slavef
               if(ke69 .gt.1) then
                  call AGMG_MUMPS_493(j-1,k69onid,ierr)
               else
                  k69onid = j
               endif
               if( AGMG_MUMPS_481(inode,k69onid)) then
                  call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                  if(ierr.ne.0) then
                     if(cv_lp.gt.0)write(cv_lp,*)                       &
     &               'BIT_SET signalled error to',subname
                     istat = ierr
                     return
                  end if
                  share = share-1
                  if(share.le.0) then
                     current = j + offset
                     if(current.gt.cv_slavef) current = 1
                     exit
                  end if
               end if
            end do
            if(share.gt.0) then
               do j=1,current-1
                  if(ke69 .gt.1) then
                     call AGMG_MUMPS_493(j-1,k69onid,ierr)
                  else
                     k69onid = j
                  endif
                  if( AGMG_MUMPS_481(inode,k69onid)) then
                     call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                     if(ierr.ne.0) then
                        if(cv_lp.gt.0)write(cv_lp,*)                    &
     &                  'BIT_SET signalled error to',subname
                        istat = ierr
                        return
                     end if
                     share = share-1
                     if(share.le.0) then
                        current = j + offset
                        if(current.gt.cv_slavef) current = 1
                        exit
                     end if
                  end if
               end do
            endif
            if(share.ne.0) then
               if(cv_lp.gt.0) write(cv_lp,*)                            &
     &              'Error reported in ',subname
               return
            end if
            if(.not.upper_round_off)then
               if(local_son_indice.lt.cv_slavef)then
                  id_son(local_son_indice)=in
                  work_per_proc(local_son_indice)=cv_tcostw(in)/        &
     &                 dble(nmb_propmap_strict)
                  local_son_indice=local_son_indice+1
                  if(local_son_indice.eq.cv_slavef)then
                     CALL AGMG_MUMPS_459(cv_slavef,id_son,                   &
     &                    work_per_proc)
                  endif
               else
                  current2=cv_slavef
                  tmp_cost=cv_tcostw(in)/dble(nmb_propmap_strict)
                  do while(current2.ge.1)
                     if(tmp_cost.lt.work_per_proc(current2))exit
                     current2=current2-1
                  enddo
                  if(current2.ne.cv_slavef)then
                     if(current2.eq.0)then
                        current2=1
                     endif
                     do j=cv_slavef-1,current2,-1
                        id_son(j+1)=id_son(j)
                        work_per_proc(j+1)=work_per_proc(j)
                     enddo
                     id_son(current2)=in
                     work_per_proc(current2)=tmp_cost
                  endif
               endif
            endif
            upper_round_off=.FALSE.
         endif
         if(.NOT.associated(cv_prop_map(in)%ind_proc)) then
            call AGMG_MUMPS_434(in,ierr)
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &              write(cv_lp,*)'PROPMAP_INIT signalled error to'     &
     &              ,subname
               istat = ierr
               return
            end if
         endif
         cv_prop_map(in)%ind_proc = procs4son
         in=cv_frere(in)
      end do
      if(are_sons_treated)then
         if(nb_procs_for_sons.ne.nmb_procs_inode)then
            do j=1,nmb_procs_inode-nb_procs_for_sons
               procs4son=cv_prop_map(id_son(j))%ind_proc
               do while(current.le.cv_slavef)
                  if(ke69 .gt.1) then
                     call AGMG_MUMPS_493(current-1,k69onid,ierr)
                  else
                     k69onid = current
                  endif
                  if(.NOT.AGMG_MUMPS_481(inode,k69onid)) then
                     current=current+1
                  else
                     exit
                  endif
               enddo
               call AGMG_MUMPS_482(procs4son,k69onid,ierr)
               cv_prop_map(id_son(j))%ind_proc=procs4son
            enddo
            ptr_upper_ro_procs=min(j,nmb_procs_inode-nb_procs_for_sons)
         endif
      endif
      in=son
      current = 1
      do while(in.gt.0)
         if( (nmb_sons_inode.ge.nmb_procs_inode).AND.                   &
     &       (nmb_procs_inode.LT.4) ) then
            procs4son = cv_prop_map(inode)%ind_proc
         elseif(nmb_procs_inode .LE. cv_keep(83)) then
            procs4son = cv_prop_map(inode)%ind_proc
         else
            procs4son = cv_prop_map(in)%ind_proc
            in_tmp=in
            nfront=cv_nfsiz(in_tmp)
            npiv=0
            in_tmp=in_tmp
            do while(in_tmp.gt.0)
               npiv=npiv+1
               in_tmp=cv_fils(in_tmp)
            end do
            ncb=nfront-npiv
            if (force_cand) then
               if (cv_keep(50) ==  0) then
                  keep48_loc=0
               else
                  keep48_loc=3
               endif
               if (cv_keep(48).EQ.5) keep48_loc = 5
               min_cand_needed=                                         &
     &              AGMG_MUMPS_50                                            &
     &              (cv_slavef, keep48_loc,cv_keep8(21),                &
     &              cv_keep(50),                                        &
     &              nfront,ncb)
               min_cand_needed=min(cv_slavef,min_cand_needed+1)
            else
               min_cand_needed = 0
            endif
            min_cand_needed = max(min_cand_needed, cv_keep(91))
            if(costs_sons.gt.0.0D0) then
               relative_weight=cv_tcostw(in)/costs_sons
            else
               relative_weight=dble(0)
            endif
            nmb_propmap_strict=0
            do k=1,cv_slavef
               if( AGMG_MUMPS_480(procs4son,k)) then
                  nmb_propmap_strict=nmb_propmap_strict+1
               end if
            end do
            offset=1
            share2=                                                     &
     &          max(0,nint(relative_weight*(loc_relax-dble(1))*         &
     &                                   dble(nmb_procs_inode)))
            share2 = max(share2, min_cand_needed -nmb_propmap_strict,   &
     &                   (cv_keep(83)/2) - nmb_propmap_strict)
            procsrest=nmb_procs_inode - nmb_propmap_strict
            share2=min(share2,procsrest)
            share2 = 0
            CALL RANDOM_NUMBER(Y)
            current2     =int(dble(Y)*dble(procsrest))
            nb_free_procs=1
            do j=1,cv_slavef
               if(share2.le.0) exit
               if(ke69 .gt.1) then
                     call AGMG_MUMPS_493(j-1,k69onid,ierr)
                  else
                     k69onid = j
                  endif
               if(( AGMG_MUMPS_481(inode,k69onid)).AND.                      &
     &           (.NOT.AGMG_MUMPS_480(procs4son,k69onid))) then
                  if(nb_free_procs.ge.current2)then
                     call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                     if(ierr.ne.0) then
                        if(cv_lp.gt.0)write(cv_lp,*)                    &
     &                       'BIT_SET signalled error to',subname
                        istat = ierr
                        return
                     end if
                     share2 = share2 - 1
                  endif
                  nb_free_procs=nb_free_procs+1
               end if
            end do
            if(share2.gt.0) then
               do j=1,cv_slavef
                  if(share2.le.0) exit
                  if(ke69 .gt.1) then
                     call AGMG_MUMPS_493(j-1,k69onid,ierr)
                  else
                     k69onid = j
                  endif
                  if(( AGMG_MUMPS_481(inode,k69onid)).AND.                   &
     &              (.NOT.AGMG_MUMPS_480(procs4son,k69onid))) then
                        call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                        if(ierr.ne.0) then
                           if(cv_lp.gt.0)write(cv_lp,*)                 &
     &                          'BIT_SET signalled error to',subname
                           istat = ierr
                           return
                        end if
                        share2 = share2 - 1
                  end if
               end do
            endif
            if(share2.ne.0) then
               if(cv_lp.gt.0) write(cv_lp,*)                            &
     &              'Error reported in ',subname
               return
            end if
         endif
         ierr=0
         in1=in
         cv_prop_map(in1)%ind_proc = procs4son
         call AGMG_MUMPS_433(in1,ctr-1,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0) write(cv_lp,*)                               &
     &         'Error reported in ',subname
            istat=ierr
            return
         endif
         in=cv_frere(in)
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_433
      subroutine AGMG_MUMPS_434(inode,istat)
      implicit none
      integer, intent(in)::inode
      integer, intent(out)::istat
      integer j,k,allocok
      character (len=48):: subname
      istat = -1
      if(cv_frere(inode).eq.cv_n+1) return
      subname='PROPMAP_INIT'
      if(.not.associated(                                               &
     &     cv_prop_map(inode)%ind_proc)) then
         allocate(cv_prop_map(inode)%ind_proc                           &
     &        (cv_size_ind_proc),STAT=allocok)
         if (allocok.gt.0) then
            cv_info(1) = cv_error_memalloc
            cv_info(2) = cv_size_ind_proc
            istat = cv_error_memalloc
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)                                         &
     &           'memory allocation error in ',subname
            return
         end if
      end if
      do k=1,cv_size_ind_proc
         do j=0,cv_bitsize_of_int-1
            cv_prop_map(inode)%ind_proc(k)=                             &
     &         ibclr(cv_prop_map(inode)%ind_proc(k),j)
         end do
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_434
      subroutine AGMG_MUMPS_435(inode,istat)
      integer,intent(in)::inode
      integer,intent(out)::istat
      integer ierr
      character (len=48):: subname
      subname='PROPMAP_TERM'
      istat =-1
      if(associated(cv_prop_map(inode)%ind_proc)) then
        deallocate(cv_prop_map(inode)%ind_proc, STAT=ierr)
        if(ierr.ne.0) then
           if(cv_lp.gt.0)                                               &
     &         write(cv_lp,*)'Memory deallocation error in ', subname
               istat = cv_error_memdeloc
               return
        endif
        nullify(cv_prop_map(inode)%ind_proc)
      end if
      istat =0
      return
      end subroutine AGMG_MUMPS_435
      subroutine AGMG_MUMPS_437(inode,ifather,istat)
      implicit none
      integer,intent(in)::inode,ifather
      integer,intent(out)::istat
      character (len=48):: subname
      istat= -1
      subname='PROPMAP4SPLIT'
      if((cv_frere(inode).eq.cv_n+1).OR.(cv_frere(ifather).eq.cv_n+1)   &
     &     .OR.(.NOT.associated(cv_prop_map(inode)%ind_proc))) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'tototo signalled error to'                 &
     &        ,subname
         return
      endif
      if(.NOT.associated(cv_prop_map(ifather)%ind_proc)) then
         call AGMG_MUMPS_434(ifather,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'PROPMAP_INIT signalled error to '       &
     &           ,subname
            istat = ierr
            return
         end if
      endif
      cv_prop_map(ifather)%ind_proc =                                   &
     &                    cv_prop_map(inode)%ind_proc
      istat=0
      return
      end subroutine AGMG_MUMPS_437
      subroutine AGMG_MUMPS_394(istat)
      implicit none
      integer,intent(out)::istat
      integer i,ierr,allocok
      type(nodelist),pointer::nodeptr
      character (len=48):: subname
      istat=-1
      subname='ROOTLIST'
      allocate(cv_layerl0_array(cv_maxnsteps),                          &
     &         cv_layerl0_sorted_costw(cv_maxnsteps),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 12*cv_maxnsteps
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)                                            &
     &        'memory allocation error in ',subname
         return
      end if
      do i=1,cv_maxnsteps
         cv_layerl0_sorted_costw(i)=dble(0)
         cv_layerl0_array(i)=0
      end do
      cv_layerl0_start        = 0
      cv_layerl0_end          = 0
      layerL0_endforarrangeL0 = 0
      if ((.NOT.associated(cv_tcostw)).OR.(.NOT.associated(cv_tcostm))) &
     &   then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:tcost must be allocated in ',subname
         return
      end if
      cv_nbsa=0
      do i=1,cv_n
         if (cv_frere(i).eq.0) then
            cv_layerl0_start=1
            cv_layerl0_end=cv_layerl0_end+1
            IF (cv_tcostw(i).GT.mincostw)                               &
     &           layerL0_endforarrangeL0 = layerL0_endforarrangeL0+1
            cv_layerl0_array(cv_layerl0_end)=i
            cv_layerl0_sorted_costw(cv_layerl0_end)=cv_tcostw(i)
            cv_costw_layer0=cv_costw_layer0 + cv_tcostw(i)
            cv_costm_layer0=cv_costm_layer0 + cv_tcostm(i)
            cv_nbsa=cv_nbsa+1
         end if
      end do
      if(cv_nbsa.eq.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:no root nodes in ',subname
         return
      end if
      call AGMG_MUMPS_459(cv_layerl0_end-cv_layerl0_start+1,                 &
     &            cv_layerl0_array(cv_layerl0_start:cv_layerl0_end),    &
     &     cv_layerl0_sorted_costw(cv_layerl0_start:cv_layerl0_end))
      cv_costw_total=cv_costw_layer0
      cv_costm_total=cv_costm_layer0
      istat=0
      return
      end subroutine AGMG_MUMPS_394
      subroutine AGMG_MUMPS_396(istat)
      implicit none
      integer,intent(out)::istat
      integer i,nfront
      character (len=48):: subname
      subname='SELECT_TYPE3'
      CALL AGMG_MUMPS_712(cv_n, slavef, cv_mp, cv_icntl(13),                 &
     &     cv_keep(1), cv_frere, cv_nfsiz, istat)
      IF (istat .NE. 0) THEN
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)                                         &
     &           'Error: Can''t select type 3 node in ',subname
      ELSE IF (cv_keep(38) .ne. 0) then
        IF(cv_nodelayer(cv_keep(38)).eq.0) then
          cv_keep(38)=0
        ELSE
          cv_nodetype(cv_keep(38))=3
        ENDIF
      ENDIF
      RETURN
      end subroutine AGMG_MUMPS_396
      subroutine AGMG_MUMPS_397(istat)
      integer,intent(out)::istat
      integer i,dummy,layernmb,allocok
      character (len=48):: subname
      istat=-1
      subname='SETUP_CAND'
      cv_nb_niv2=0
      do i=1,cv_n
         if(cv_nodetype(i).eq.2) cv_nb_niv2=cv_nb_niv2+1
      end do
      cv_keep(56)=cv_nb_niv2
      nullify(cv_par2_nodes,cv_cand)
      allocate(cv_par2_nodes(cv_nb_niv2),                               &
     &         cv_cand(cv_nb_niv2,cv_slavef+1),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = cv_nb_niv2*(cv_slavef+2)
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)                                            &
     &        'memory allocation error in ',subname
         return
      end if
      cv_par2_nodes=0
      cv_cand(:,:)=0
      dummy=1
      do layernmb=1,cv_maxlayer
         do i=1,cv_layer_p2node(layernmb)%nmb_t2s
            cv_par2_nodes(dummy)=                                       &
     &                  cv_layer_p2node(layernmb)%t2_nodenumbers(i)
            cv_cand(dummy,:)=cv_layer_p2node(layernmb)%t2_cand(i,:)
            dummy=dummy+1
         enddo
      enddo
      if(dummy.ne.cv_nb_niv2+1) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname,                        &
     &        ' : dummy =',dummy,'nbniv2 =',cv_nb_niv2
         return
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_397
      subroutine AGMG_MUMPS_398(map_strat,workload,memused,                  &
     &                       inode,istat)
      implicit none
      integer,intent(in)::map_strat
      DOUBLE PRECISION,dimension(cv_slavef),intent(in)::workload,       &
     &                                                   memused
      integer, optional::inode,istat
      integer i,j,aux_int,nmb_procs,proc,pos
      character (len=48):: subname
      logical enforce_prefsort
      logical use_propmap
      logical,SAVE::init1 = .FALSE.
      logical,SAVE::init2 = .FALSE.
      logical,SAVE::init3 = .FALSE.
      subname='SORTPROCS'
      enforce_prefsort=.TRUE.
      use_propmap=present(inode)
      if(present(istat))istat=-1
      if((map_strat.ne.cv_equilib_flops).and.                           &
     &   (map_strat.ne.cv_equilib_mem)) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'error in ',subname
         return
      endif
      cv_proc_sorted=(/(i,i=1,cv_slavef)/)
      if (.not.present(inode)) then
         if(.NOT.init1) then
            init1=.TRUE.
         end if
         do i=1,cv_slavef-1
            do j=i+1,cv_slavef
               if(((workload(cv_proc_sorted(j)).lt.                     &
     &                     workload(cv_proc_sorted(i))).AND.            &
     &             (map_strat.eq.cv_equilib_flops))                     &
     &            .OR.                                                  &
     &            ((memused(cv_proc_sorted(j)).lt.                      &
     &                     memused(cv_proc_sorted(i))).AND.             &
     &             (map_strat.eq.cv_equilib_mem)))then
                  aux_int=cv_proc_sorted(j)
                  cv_proc_sorted(j)=cv_proc_sorted(i)
                  cv_proc_sorted(i)=aux_int
               end if
            end do
         end do
      else if(present(inode)) then
         if (use_propmap) then
            if(.NOT.init2) then
               init2=.TRUE.
            end if
            nmb_procs=0
            do pos=1,cv_slavef
               if( AGMG_MUMPS_481(inode,pos)) then
                  if (pos.le.nmb_procs) then
                     exit
                  else
                     nmb_procs=nmb_procs+1
                     aux_int=cv_proc_sorted(pos)
                     cv_proc_sorted(pos)=                               &
     &                    cv_proc_sorted(nmb_procs)
                     cv_proc_sorted(nmb_procs)=aux_int
                     cycle
                  end if
               end if
            end do
         end if
         do i=1,nmb_procs-1
            do j=i+1,nmb_procs
               if(((workload(cv_proc_sorted(j)).lt.                     &
     &                     workload(cv_proc_sorted(i))).AND.            &
     &             (map_strat.eq.cv_equilib_flops))                     &
     &            .OR.                                                  &
     &            ((memused(cv_proc_sorted(j)).lt.                      &
     &                     memused(cv_proc_sorted(i))).AND.             &
     &             (map_strat.eq.cv_equilib_mem)))then
                  aux_int=cv_proc_sorted(j)
                  cv_proc_sorted(j)=cv_proc_sorted(i)
                  cv_proc_sorted(i)=aux_int
               end if
            end do
         end do
         do i=nmb_procs+1,cv_slavef-1
            do j=i+1,cv_slavef
               if(((workload(cv_proc_sorted(j)).lt.                     &
     &                     workload(cv_proc_sorted(i))).AND.            &
     &             (map_strat.eq.cv_equilib_flops))                     &
     &            .OR.                                                  &
     &            ((memused(cv_proc_sorted(j)).lt.                      &
     &                     memused(cv_proc_sorted(i))).AND.             &
     &             (map_strat.eq.cv_equilib_mem)))then
                  aux_int=cv_proc_sorted(j)
                  cv_proc_sorted(j)=cv_proc_sorted(i)
                  cv_proc_sorted(i)=aux_int
               end if
            end do
         end do
         if(.NOT.enforce_prefsort) then
            if(((2.0D0*workload(cv_proc_sorted(nmb_procs+1)).lt.        &
     &           workload(cv_proc_sorted(1))).AND.                      &
     &          (map_strat.eq.cv_equilib_flops))                        &
     &         .OR.                                                     &
     &         ((2.0D0*memused(cv_proc_sorted(nmb_procs+1)).lt.         &
     &           memused(cv_proc_sorted(1))).AND.                       &
     &          (map_strat.eq.cv_equilib_mem)))then
               do i=1,cv_slavef-1
                  do j=i+1,cv_slavef
                     if(((workload(cv_proc_sorted(j)).lt.               &
     &                    workload(cv_proc_sorted(i))).AND.             &
     &                    (map_strat.eq.cv_equilib_flops))              &
     &                    .OR.                                          &
     &                    ((memused(cv_proc_sorted(j)).lt.              &
     &                    memused(cv_proc_sorted(i))).AND.              &
     &                    (map_strat.eq.cv_equilib_mem)))then
                        aux_int=cv_proc_sorted(j)
                        cv_proc_sorted(j)=cv_proc_sorted(i)
                        cv_proc_sorted(i)=aux_int
                     end if
                  end do
               end do
            endif
         end if
      endif
      if(present(istat))istat=0
      return
      end subroutine AGMG_MUMPS_398
      subroutine AGMG_MUMPS_400(inode,                                       &
     &                                 doit,npiv,nfront,npiv_son,istat)
      implicit none
      integer,intent(in)::inode
      logical,intent(out)::doit
      integer,intent(out),OPTIONAL::npiv,nfront,npiv_son,istat
      integer in,nfr_ifather,npiv_ifather,npiv2,nfront2,npiv_son2
      doit=.FALSE.
      if(present(npiv)) npiv=cv_invalid
      if(present(nfront)) nfront=cv_invalid
      if(present(npiv_son)) npiv_son=cv_invalid
      if(present(istat)) istat=-1
      if(cv_frere(inode).eq.cv_n+1) return
      doit=.TRUE.
      if ((cv_frere(inode).eq.0).AND.                                   &
     &     ((cv_keep(60).ne.0).OR.(cv_keep(53).le.0))) doit=.FALSE.
      if(cv_nodetype(inode).eq.3) doit=.FALSE.
      nfront2=cv_nfsiz(inode)
      in=inode
      npiv2=0
      do while (in.gt.0)
         in=cv_fils(in)
         npiv2=npiv2+1
      end do
      if(in.eq.0) doit=.FALSE.
      if(cv_total_split.ge.cv_maxcut) doit=.FALSE.
      if(.NOT.any(cv_potsplit.eq.inode)) doit=.FALSE.
      npiv_son2 = max(npiv2/2,1)
      if(npiv2.le.npiv_son2) doit=.FALSE.
      if(.NOT.( AGMG_MUMPS_359(nfront2,npiv_son2)))                          &
     &   doit=.FALSE.
      nfr_ifather=nfront2-npiv_son2
      npiv_ifather=npiv2-npiv_son2
      if(.NOT.                                                          &
     &     ( AGMG_MUMPS_359(nfr_ifather,npiv_ifather)))then
         doit=.FALSE.
      endif
      if ((dble(nfront2)-dble(npiv2)/dble(2)).le.dble(cv_keep(9)))      &
     &   doit=.FALSE.
      if(cv_nsteps.ge.cv_maxnsteps) doit=.FALSE.
      if(.NOT.doit) then
         if(present(npiv)) npiv=cv_invalid
         if(present(nfront)) nfront=cv_invalid
         if(present(npiv_son)) npiv_son=cv_invalid
      else
         if(present(npiv)) npiv=npiv2
         if(present(nfront)) nfront=nfront2
         if(present(npiv_son)) npiv_son=npiv_son2
      endif
      if(present(istat)) istat=0
      end subroutine AGMG_MUMPS_400
      subroutine AGMG_MUMPS_401(inode,nfront,npiv,npiv_son,                  &
     &                              ison,ifather,istat)
      implicit none
      integer, intent(in)::inode,nfront,npiv,npiv_son
      integer,intent(out)::ison,ifather
      integer, intent(out)::istat
      integer i,in,in_son,in_father,in_grandpa
      DOUBLE PRECISION:: ncostm,ncostw,ncostm_ison,ncostw_ison,         &
     &                   ncostm_ifather,ncostw_ifather
      character (len=48):: subname
      istat=-1
      subname='SPLITNODE'
      ison=-1
      ifather=-1
      ncostw=cv_ncostw(inode)
      ncostm=cv_ncostm(inode)
      if((npiv_son.ge.npiv).OR.(npiv.ge.nfront)) return
      ison=inode
      in_son = ison
      do i = 1,npiv_son-1
         in_son = cv_fils(in_son)
         if (in_son.le.0) return
      end do
      ifather = cv_fils(in_son)
      if(ifather.le.0) return
      in_father=ifather
      do while (cv_fils(in_father).gt.0)
         in_father=cv_fils(in_father)
      end do
      cv_nsteps = cv_nsteps + 1
      cv_frere(ifather)=cv_frere(inode)
      cv_frere(ison)=-ifather
      cv_fils(in_son)=cv_fils(in_father)
      cv_fils(in_father)=-ison
      in=cv_frere(ifather)
      do while (in.gt.0)
         in=cv_frere(in)
      end do
      in=-in
      do while(cv_fils(in).gt.0)
         in=cv_fils(in)
      end do
      in_grandpa = in
      if(cv_fils(in_grandpa).eq.-inode) then
         cv_fils(in_grandpa)=-ifather
      else
         in=-cv_fils(in_grandpa)
         do while(cv_frere(in).gt.0)
            if(cv_frere(in).eq.inode) then
               cv_frere(in)=ifather
            else
               in=cv_frere(in)
            end if
         end do
      end if
      cv_last_splitting%new_ison=ison
      cv_last_splitting%new_ifather=ifather
      cv_last_splitting%old_keep2=cv_keep(2)
      cv_last_splitting%ncostw_oldinode=ncostw
      cv_last_splitting%ncostm_oldinode=ncostm
      if(associated(cv_tcostw))                                         &
     &   cv_last_splitting%tcostw_oldinode=cv_tcostw(inode)
      if(associated(cv_tcostm))                                         &
     &   cv_last_splitting%tcostm_oldinode=cv_tcostm(inode)
      cv_nfsiz(ison)=nfront
      cv_nfsiz(ifather)=nfront-npiv_son
      cv_keep(2)=max(cv_keep(2),nfront-npiv_son)
      cv_ne(ifather)=1
      cv_keep(61)=cv_keep(61)+1
      call AGMG_MUMPS_418(npiv_son,nfront,                                   &
     &                            ncostw_ison,ncostm_ison)
      call AGMG_MUMPS_418(npiv-npiv_son,nfront-npiv_son,                     &
     &                            ncostw_ifather,ncostm_ifather)
      cv_ncostw(ison)=ncostw_ison
      cv_ncostm(ison)=ncostm_ison
      cv_ncostw(ifather)=ncostw_ifather
      cv_ncostm(ifather)=ncostm_ifather
      if(associated(cv_tcostw)) cv_tcostw(ison) = cv_tcostw(inode)      &
     &                 -ncostw +cv_ncostw(ison)
      if(associated(cv_tcostm)) cv_tcostm(ison) = cv_tcostm(inode)      &
     &                 -ncostm +cv_ncostm(ison)
      if(associated(cv_tcostw))                                         &
     &   cv_tcostw(ifather) = cv_tcostw(ison)+cv_ncostw(ifather)
      if(associated(cv_tcostm))                                         &
     &   cv_tcostm(ifather) = cv_tcostm(ison)+cv_ncostm(ifather)
      cv_total_split=cv_total_split+1
      call AGMG_MUMPS_437(inode,ifather,ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'PROPMAP4SPLIT error in ',subname
         istat = ierr
         return
      end if
      istat = 0
      return
      end subroutine AGMG_MUMPS_401
      subroutine AGMG_MUMPS_402(ne,nfsiz,frere,fils,keep,KEEP8,              &
     &                                info,procnode,ssarbr,nbsa)
      implicit none
      integer,dimension(cv_n),intent(inout)::ne,nfsiz,frere,fils,       &
     &                                     procnode,ssarbr
      integer, intent(inout):: keep(500),info(40),nbsa
      INTEGER*8 KEEP8(150)
      ne=cv_ne
      nfsiz=cv_nfsiz
      frere=cv_frere
      fils=cv_fils
      keep(2) =cv_keep(2)
      keep(20)=cv_keep(20)
      keep(28)=cv_nsteps
      keep(38)=cv_keep(38)
      keep(56)=cv_keep(56)
      keep(61)=cv_keep(61)
      info(5)=cv_info(5)
      info(6)=cv_nsteps
      procnode=cv_procnode
      ssarbr=cv_ssarbr
      nbsa=cv_nbsa
      end subroutine AGMG_MUMPS_402
      subroutine AGMG_MUMPS_403(istat)
      implicit none
      integer,intent(out)::istat
      integer i,ierr,layernmb
      character (len=48):: subname
      istat=-1
      subname='TERMGLOB'
      if(cv_keep(82) .eq. 0) then
         deallocate(cv_potsplit,STAT=ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Memory deallocation error in ',subname
            istat = cv_error_memdeloc
            return
         end if
      endif
      nullify(cv_frere,cv_fils,cv_nfsiz,cv_ne,cv_keep,cv_keep8,         &
     &        cv_icntl,cv_info,cv_procnode,cv_ssarbr)
      deallocate(cv_proc_workload,cv_proc_maxwork,cv_proc_memused,      &
     &    cv_proc_maxmem,cv_nodetype,                                   &
     &    cv_nodelayer,cv_proc_sorted,                                  &
     &    cv_ncostw,cv_ncostm,                                          &
     &    cv_layerworkload,cv_layermemused,                             &
     &    STAT=ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      deallocate(work_per_proc,id_son,STAT=ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      do layernmb=1,cv_maxlayer
         if(cv_layer_p2node(layernmb)%nmb_t2s.gt.0) then
            deallocate(cv_layer_p2node(layernmb)%t2_nodenumbers,        &
     &                 cv_layer_p2node(layernmb)%t2_cand,               &
     &                 cv_layer_p2node(layernmb)%t2_candcostw,          &
     &                 cv_layer_p2node(layernmb)%t2_candcostm,          &
     &                 STAT=ierr)
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Memory deallocation error in ',           &
     &                        subname
               istat = cv_error_memdeloc
               return
            end if
         endif
      enddo
      if(associated(cv_layer_p2node)) then
         deallocate(cv_layer_p2node,STAT=ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Memory deallocation error in ',subname
            istat = cv_error_memdeloc
            return
         end if
      end if
      do i=1,cv_n
         call AGMG_MUMPS_435(i,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'PROPMAP_TERM signalled error in ',      &
     &           subname
            istat = ierr
            return
         end if
      end do
      if(associated(cv_prop_map))deallocate(cv_prop_map,STAT=ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      istat=0
      return
      end subroutine AGMG_MUMPS_403
      recursive subroutine AGMG_MUMPS_404(pos,istat)
      implicit none
      integer,intent(in)::pos
      integer, intent(out)::istat
      integer i,nfront,npiv,nextpos,ierr
      DOUBLE PRECISION costw,costm
      character (len=48):: subname
      istat=-1
      subname='TREECOSTS'
      if ((.NOT.associated(cv_tcostw)).OR.(.NOT.associated(cv_tcostm))) &
     &   then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:tcost must be allocated in ',subname
         return
      end if
      nfront=cv_nfsiz(pos)
      npiv=1
      nextpos=cv_fils(pos)
      do
         if(nextpos.le.0) then
            exit
         else
            npiv=npiv+1
            nextpos=cv_fils(nextpos)
         end if
      end do
      call AGMG_MUMPS_418(npiv,nfront,costw,costm)
      cv_ncostw(pos)=costw
      cv_ncostm(pos)=costm
      if (cv_ne(pos).ne.0) then
         nextpos=cv_fils(pos)
         do while(nextpos.gt.0)
            nextpos=cv_fils(nextpos)
         end do
         nextpos=-nextpos
         do i=1,cv_ne(pos)
            cv_depth(nextpos)=cv_depth(pos)+1
            call AGMG_MUMPS_404(nextpos,ierr)
            if (ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Failure in recursive call to ',subname
               return
            end if
            costw=costw+cv_tcostw(nextpos)
            costm=costm+cv_tcostm(nextpos)
            nextpos=cv_frere(nextpos)
         end do
      endif
      cv_tcostw(pos) = costw
      cv_tcostm(pos) = costm
      istat = 0
      end subroutine AGMG_MUMPS_404
      recursive subroutine AGMG_MUMPS_406(inode)
      implicit none
      integer, intent(in)::inode
      integer in
      cv_nodetype(inode)=-1
      in=cv_fils(inode)
      do while (in>0)
         in=cv_fils(in)
      end do
      in=-in
      do while(in.gt.0)
         call AGMG_MUMPS_406(in)
         in=cv_frere(in)
      enddo
      end subroutine AGMG_MUMPS_406
      subroutine AGMG_MUMPS_408(workload,memused,                            &
     &                                    maxwork,minwork,maxmem,minmem)
      implicit none
      DOUBLE PRECISION,dimension(cv_slavef),intent(in)::workload,       &
     &                                                  memused
      DOUBLE PRECISION,intent(out)::maxwork,minwork,maxmem,minmem
      intrinsic maxval,minval
      maxwork=maxval(workload)
      minwork=minval(workload, mask= workload > dble(0))
      maxmem=maxval(memused)
      minmem=minval(memused, mask= memused > dble(0))
      end subroutine AGMG_MUMPS_408
      subroutine AGMG_MUMPS_476(layernumber,nodenumber)
      implicit none
      integer layernumber,nodenumber
      integer i
      integer inode
      integer current_max,current_proc
      current_max = 0
      score = 0
      allowed_nodes = .FALSE.
      inode=cv_layer_p2node(layernumber)%t2_nodenumbers(nodenumber)
      do i=1,cv_layer_p2node(layernumber)%t2_cand(nodenumber,           &
     &     cv_slavef+1)
         current_proc=cv_layer_p2node(layernumber)%t2_cand(nodenumber,i)
         if ( current_proc .ge. 0) then
            score(mem_distribmpi(current_proc)) =                       &
     &           score(mem_distribmpi(current_proc)) + 1
         endif
      enddo
      current_proc = cv_procnode(inode) - 1
      score(mem_distribmpi(current_proc)) =                             &
     &     score(mem_distribmpi(current_proc)) + 1
      do i=0,nb_arch_nodes - 1
         if ( score(i) .gt. current_max ) then
            current_max = score(i)
            allowed_nodes = .FALSE.
            allowed_nodes(i) = .TRUE.
         else
            if(score(i) .eq. current_max) then
               allowed_nodes(i) = .TRUE.
            endif
         endif
      enddo
      return
      end subroutine AGMG_MUMPS_476
      end subroutine AGMG_MUMPS_369
      subroutine AGMG_MUMPS_393(par2_nodes,cand,istat)
      integer, intent(out)::par2_nodes(cv_nb_niv2),                     &
     &                      cand(cv_slavef+1,cv_nb_niv2),istat
      character (len=48):: subname
      integer iloop
      istat=-1
      subname='MUMPS_393'
      par2_nodes=cv_par2_nodes
      do iloop=1, cv_slavef+1
        cand(iloop,:)=cv_cand(:,iloop)
      enddo
      deallocate(cv_par2_nodes,cv_cand,STAT=istat)
      if(istat.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      istat = 0
      return
      end subroutine AGMG_MUMPS_393
      subroutine AGMG_MUMPS_427(                                             &
     &     total_comm,working_comm,keep69,par,                          &
     &     nbslaves,mem_distrib,informerr)
      implicit none
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
      integer nbslaves
      integer, dimension(0:nbslaves-1) :: mem_distrib
      integer total_comm,working_comm,keep69,par
      integer, dimension(40) ::informerr
      integer myrank
      integer host,i,ierr
      integer,dimension(:),allocatable :: buffer_memdistrib
      ierr = 0
      myrank = -1
      host = -1
      ke69 = keep69
      cv_slavef = nbslaves
      if (ke69 .eq. 1) then
         return
      endif
      if ( allocated(mem_distribtmp) ) deallocate(mem_distribtmp )
      allocate( mem_distribtmp( 0:cv_slavef-1 ),                        &
     &          buffer_memdistrib( 0:cv_slavef-1 ), stat=ierr )
      if ( ierr .gt. 0 ) then
         if(cv_mp.gt.0) write(cv_mp,*) 'pb allocation mem_dist'
         informerr(1) = -13
         informerr(2) = cv_slavef
         return
      end if
      mem_distribtmp = -1
      call AGMG_SETIERR2( host, ierr )
      if ((par .eq. 1) .or. (host .ne. 0)) then
         call AGMG_SETIERR2( myrank, ierr )
         call AGMG_MUMPS_430(ierr,myrank,                                    &
     &        working_comm,mem_distrib(0))
         if ( ierr .ne. 0 ) then
            if(cv_mp.gt.0)                                              &
     &      write(cv_mp,*) 'pb in mumps_init_arch_parameters'
            informerr(1) = -13
            informerr(2) = cv_slavef
            return
         end if
         mem_distribtmp = mem_distrib
         call AGMG_MUMPS_429(ierr)
         if ( ierr .ne. 0 ) then
            if(cv_mp.gt.0) write(cv_mp,*)                               &
     &'pb in mumps_init_arch_parameters'
            informerr(1) = -13
            informerr(2) = cv_slavef
            return
         endif
      endif
      if(ke69 .le. 0) then
         deallocate(mem_distribtmp)
         deallocate(buffer_memdistrib)
         return
      endif
      call AGMG_COPY(mem_distribtmp(0),buffer_memdistrib(0),                     cv_slavef,AGMG_MPF_INTEGER,ierr)
      mem_distribtmp = buffer_memdistrib
      deallocate (buffer_memdistrib)
      call AGMG_MUMPS_492()
        if((cv_slavef/nb_arch_nodes) .le. 4) then
          do i = 0, cv_slavef-1
            if ( mem_distrib(i) .NE. 1 ) then
              mem_distrib(i)=max(ke69/2,2)
            endif
          enddo
        endif
        if((nb_arch_nodes .eq. 1) .or.                                  &
     &     (nb_arch_nodes .eq. cv_slavef)) then
         ke69 = 1
         keep69 = 1
         deallocate(mem_distribtmp)
         return
      endif
      if (host .eq. 0) then
         if ( allocated(mem_distribmpi) ) deallocate(mem_distribmpi )
         allocate( mem_distribmpi( 0:cv_slavef-1 ), stat=ierr )
         if ( ierr .gt. 0 ) then
            if(cv_mp.gt.0) write(cv_mp,*) 'pb allocation mem_dist'
            informerr(1) = -13
            informerr(2) = cv_slavef
            return
         endif
         call AGMG_MUMPS_495(ierr)
         if(ierr .ne. 0 ) then
            return
         endif
         mem_distribmpi = mem_distribtmp
         call AGMG_MUMPS_428(ierr)
         if ( ierr .ne. 0 ) then
            if(cv_mp.gt.0)                                              &
     &      write(cv_mp,*) 'pb in mumps_init_arch_parameters'
            informerr(1) = -13
            informerr(2) = cv_slavef
            return
         endif
      else
         deallocate(mem_distribtmp)
      endif
      return
      end subroutine AGMG_MUMPS_427
      subroutine AGMG_MUMPS_492()
      implicit none
      integer i
      nb_arch_nodes = 0
      do i=0,cv_slavef-1
         if(mem_distribtmp(i) .eq. i) then
            nb_arch_nodes = nb_arch_nodes + 1
         endif
      enddo
      return
      end subroutine AGMG_MUMPS_492
      subroutine AGMG_MUMPS_428(ierr)
      implicit none
      external AGMG_MUMPS_463
      integer i,precnode,nodecount
      integer sizesmp
      integer ierr
      ierr = 0
      sizesmp = 0
      if ( allocated(table_of_process) )                                &
     &     deallocate(table_of_process  )
      allocate( table_of_process(0:cv_slavef-1), stat=ierr )
      if ( ierr .gt. 0 ) then
         if(cv_mp.gt.0) write(cv_mp,*)                                  &
     &   'pb allocation in MUMPS_428'
         return
      end if
      do i=0,cv_slavef - 1
         table_of_process(i) = i
      enddo
      call AGMG_MUMPS_463(cv_slavef,mem_distribtmp(0),                       &
     &     table_of_process(0))
      precnode = 0
      nodecount = 0
      do i=0,cv_slavef-1
         if(mem_distribtmp(i) .eq. precnode) then
            sizesmp = sizesmp + 1
            mem_distribtmp(i) = nodecount
            mem_distribmpi(table_of_process(i)) = nodecount
         else
            score(nodecount) = sizesmp
            sizesmp = 1
            nodecount = nodecount + 1
            precnode = mem_distribtmp(i)
            mem_distribtmp(i) = nodecount
            mem_distribmpi(table_of_process(i)) = nodecount
         endif
      enddo
      score(nodecount) = sizesmp
      do i=0,cv_slavef-1
         mem_distribtmp(i) = score(mem_distribtmp(i))
      enddo
      CALL AGMG_MUMPS_466(cv_slavef,mem_distribtmp(0),                       &
     &     table_of_process(0))
      ierr = 0
      return
      end subroutine AGMG_MUMPS_428
      subroutine AGMG_MUMPS_429(ierr)
      implicit none
      integer i,j,ierr
      integer idmaster
      idmaster = -1
      ierr = 0
      do i=0,cv_slavef-1
         if (mem_distribtmp(i) .eq. 1) then
            idmaster = i
            do j=i,cv_slavef-1
               if (mem_distribtmp(j) .eq. 1) then
                  mem_distribtmp(j) = idmaster
               else
                  mem_distribtmp(j) = 0
               endif
            enddo
            return
         else
            mem_distribtmp(i) = 0
         endif
      enddo
      if(cv_mp.gt.0) write(cv_mp,*)'problem in MUMPS_429:               &
     &     cannot find a master'
      ierr = 1
      return
      end subroutine AGMG_MUMPS_429
      subroutine AGMG_MUMPS_430(ierr,myrank,working_comm,                    &
     &     mem_distrib)
      implicit none
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
      integer ierr,resultlen,myrank,i,working_comm
      integer , dimension(0:cv_slavef-1) :: mem_distrib
      integer allocok
      character(len=AGMG_MPF_MAX_PROCESSOR_NAME) name
      integer, dimension(:),allocatable :: namercv
      integer, dimension(:),allocatable :: myname
      integer lenrcv
      external AGMG_MUMPS_438
      logical AGMG_MUMPS_438
      ierr = 0
      call AGMG_MPF_get_processor_name(name,resultlen,ierr)
      allocate(myname(resultlen),stat=allocok)
      if ( allocok .gt. 0 ) then
        if(cv_mp.gt.0) write(cv_mp,*)                                   &
     &  'pb allocation in compute_dist for myname'
        ierr = 1
        return
      end if
      do i=1, resultlen
         myname(i) = ichar(name(i:i))
      enddo
      do i=0, cv_slavef-1
         if(myrank .eq. i) then
            lenrcv = resultlen
         else
            lenrcv = 0
         endif
         call AGMG_SETIERR(ierr)
         allocate(namercv(lenrcv),stat=allocok)
         if ( allocok .gt. 0 ) then
            if(cv_mp.gt.0) write(cv_mp,*)                               &
     &      'pb allocation in compute_dist for namercv'
            ierr = 1
            return
         end if
         if(myrank .eq. i) then
            namercv = myname
         endif
         call AGMG_SETIERR(ierr)
         if( AGMG_MUMPS_438(myname,namercv,                                  &
     &        resultlen,lenrcv)) then
            mem_distrib(i)=1
         else
            mem_distrib(i)=ke69
         endif
         deallocate(namercv)
      enddo
      deallocate(myname)
      ierr = 0
      return
      end subroutine AGMG_MUMPS_430
      subroutine AGMG_MUMPS_493(current_proc,idarch,ierr)
      implicit none
      integer current_proc
      integer idarch,ierr
      ierr = 0
      if (current_proc .ge. cv_slavef) then
         ierr = -1
         return
      endif
      if (current_proc .lt. 0) then
         idarch = 1
         return
      else
         idarch = table_of_process(current_proc) + 1
      endif
      return
      end subroutine AGMG_MUMPS_493
      subroutine AGMG_MUMPS_494()
      if (allocated(table_of_process)) deallocate(table_of_process)
      if (allocated(allowed_nodes)) deallocate(allowed_nodes)
      if (allocated(score)) deallocate(score)
      if (allocated(mem_distribtmp)) deallocate(mem_distribtmp)
      if (allocated(mem_distribmpi)) deallocate(mem_distribmpi)
      return
      end subroutine AGMG_MUMPS_494
      subroutine AGMG_MUMPS_495(ierr)
      integer ierr
      ierr = 0
      if (allocated(allowed_nodes)) deallocate(allowed_nodes)
      allocate( allowed_nodes(0:nb_arch_nodes-1),stat=ierr)
      if ( ierr .gt. 0 ) then
         if(cv_mp.gt.0) write(cv_mp,*)                                  &
     &   'pb allocation MUMPS_495'
         ierr = -13
         return
      end if
      allowed_nodes = .FALSE.
       if (allocated(score)) deallocate(score)
      allocate( score(0:nb_arch_nodes-1),stat=ierr)
      if ( ierr .gt. 0 ) then
         if(cv_mp.gt.0) write(cv_mp,*)                                  &
     &   'pb allocation MUMPS_495'
         ierr = -13
         return
      end if
      score = 0
      ierr = 0
      return
      end subroutine AGMG_MUMPS_495
      subroutine AGMG_MUMPS_496(idproc,thenode)
      implicit none
      integer idproc,thenode
      thenode = mem_distribmpi(idproc)
      return
      end subroutine AGMG_MUMPS_496
      SUBROUTINE AGMG_MUMPS_516(start1st,end1st,dim1,                        &
     &                             start2nd,end2nd,dim2,                &
     &                             indx,                                &
     &                             val)
      implicit none
      integer, intent(in):: start1st,end1st,dim1,start2nd,end2nd,dim2
      integer, intent(inout):: indx(end2nd)
      DOUBLE PRECISION, intent(inout):: val(end2nd)
      integer::index(dim1+dim2)
      DOUBLE PRECISION ::dummy1(dim1+dim2)
      integer:: a,b,c
      a=start1st
      b=start2nd
      c=1
      do while((a.LT.end1st+1).AND.(b.LT.end2nd+1))
         if(val(a).GT.val(b))then
            index(c)=indx(a)
            dummy1(c)=val(a)
            a=a+1
            c=c+1
         else
            index(c)=indx(b)
            dummy1(c)=val(b)
            b=b+1
            c=c+1
         endif
      end do
      if(a.LT.end1st+1) then
         do while(a.LT.end1st+1)
            index(c)=indx(a)
            dummy1(c)=val(a)
            a=a+1
            c=c+1
         enddo
      elseif(b.LT.end2nd+1) then
         do while(b.LT.end2nd+1)
            index(c)=indx(b)
            dummy1(c)=val(b)
            b=b+1
            c=c+1
         enddo
      endif
      indx(start1st:end1st)=index(1:dim1)
      val(start1st:end1st)=dummy1(1:dim1)
      indx(start2nd:end2nd)=index(dim1+1:dim1+dim2)
      val(start2nd:end2nd)=dummy1(dim1+1:dim1+dim2)
      end SUBROUTINE AGMG_MUMPS_516
      SUBROUTINE AGMG_MUMPS_459(dim,indx,val1,val2)
      implicit none
      integer, intent(in):: dim
      integer, intent(inout):: indx(dim)
      DOUBLE PRECISION, intent(inout):: val1(dim)
      DOUBLE PRECISION, intent(inout),optional:: val2(dim)
      integer::index(dim),dummy1(dim)
      DOUBLE PRECISION ::dummy2(dim)
      integer, parameter :: ss = 35
      integer:: a,b,c,i,j,k,l,r,s,stackl(ss),stackr(ss)
      DOUBLE PRECISION :: w,x
      do i=1,dim
         index(i)=i
      enddo
      s = 1
      stackl(1) = 1
      stackr(1) = dim
 5511 CONTINUE
      l = stackl(s)
      r = stackr(s)
      k = (l+r) / 2
      if(l.LT.k) then
         if(s.GE.ss) stop 'maxsize of stack reached'
         s = s + 1
         stackl(s) = l
         stackr(s) = k
         goto 5511
      endif
 5512 CONTINUE
      l = stackl(s)
      r = stackr(s)
      k = (l+r) / 2
      if(k+1.LT.r) then
         if(s.GE.ss) stop 'maxsize of stack reached'
         s = s + 1
         stackl(s) = k+1
         stackr(s) = r
         goto 5511
      endif
 5513 CONTINUE
      l = stackl(s)
      r = stackr(s)
      k = (l+r) / 2
      a=l
      b=k+1
      c=1
      do while((a.LT.k+1).AND.(b.LT.r+1))
         if(val1(index(a)).GT.val1(index(b)))then
            dummy1(c)=index(a)
            a=a+1
            c=c+1
         else
            dummy1(c)=index(b)
            b=b+1
            c=c+1
         endif
      end do
      if(a.LT.k+1) then
         dummy1(c:r-l+1)=index(a:k)
      elseif(b.LT.r+1) then
         dummy1(c:r-l+1)=index(b:r)
      endif
      index(l:r)=dummy1(1:r-l+1)
      if(s.GT.1) then
         s = s - 1
         if(l.EQ.stackl(s)) goto 5512
         if(r.EQ.stackr(s)) goto 5513
      endif
      do i=1,dim
         dummy1(i)=indx(index(i))
      enddo
      indx=dummy1
      do i=1,dim
         dummy2(i)=val1(index(i))
      enddo
      val1=dummy2
      if(present(val2)) then
         do i=1,dim
            dummy2(i)=val2(index(i))
         enddo
         val2=dummy2
      endif
      return
      end subroutine AGMG_MUMPS_459
      END MODULE AGMG_MUMPS_STATIC_MAPPING
      SUBROUTINE AGMG_MUMPS_712(N, SLAVEF, MP,                               &
     &                   ICNTL13, KEEP, FRERE, ND, ISTAT)
      IMPLICIT NONE
      INTEGER, intent(in) :: N, SLAVEF, ICNTL13, MP
      INTEGER KEEP(150)
      INTEGER FRERE(N), ND(N)
      INTEGER, intent(out) :: ISTAT
      INTEGER IROOTTREE, SIZEROOT, NFRONT, I
      ISTAT = 0
      IF (KEEP(60).EQ.2 .or. KEEP(60).EQ.3 ) THEN
      ELSE
        IF((SLAVEF.EQ.1).OR.(ICNTL13.GT.0).OR.                          &
     &     (KEEP(60).NE.0)) THEN
          KEEP(38) = 0
        ELSE
         IROOTTREE=-1
         SIZEROOT=-1
         DO I=1,N
            IF (FRERE(I).EQ.0) THEN
               NFRONT = ND(I)
               IF (NFRONT .GT.SIZEROOT) THEN
                  IROOTTREE = I
                  SIZEROOT  = NFRONT
               END IF
            END IF
         END DO
         IF ((IROOTTREE.EQ.-1).OR.(SIZEROOT.EQ.-1)) THEN
            ISTAT = -1
            RETURN
         ENDIF
         IF (SIZEROOT.LE.SLAVEF) THEN
            KEEP(38) = 0
         ELSE IF((SIZEROOT.GT.KEEP(37))                                 &
     &           .AND. (KEEP(53).EQ.0)                                  &
     &           ) THEN
            IF (MP.GT.0) WRITE(MP,*) 'A root of estimated size ',       &
     &           SIZEROOT,' has been selected for Scalapack.'
            KEEP(38) = IROOTTREE
         ELSE
            KEEP(38) = 0
             IF (MP.GT.0) WRITE(MP,*)                                   &
     &          ' WARNING: Largest root node of size ', SIZEROOT,       &
     &          ' not selected for parallel execution'
         END IF
         IF ((KEEP(38).EQ.0).AND.(KEEP(53).NE.0)) THEN
            KEEP(20) = IROOTTREE
         ELSE IF (KEEP(60).EQ.0) THEN
            KEEP(20) = 0
         ENDIF
       ENDIF
      ENDIF
      RETURN
      END SUBROUTINE AGMG_MUMPS_712
      ! try to merge 32 & 33 -- worked
      MODULE AGMG_MUMPS_OOC_COMM
      IMPLICIT NONE
      INTEGER,PARAMETER::FCT=0
      INTEGER,PUBLIC,PARAMETER::TYPEF_L=1, TYPEF_U=2, TYPEF_CB=3
      INTEGER OOC_NB_FILE_TYPE,OOC_FCT_TYPE
      INTEGER, DIMENSION(:,:),POINTER :: OOC_INODE_SEQUENCE
      INTEGER*8, DIMENSION(:,:),POINTER :: OOC_VADDR
      INTEGER,DIMENSION(:),POINTER:: KEEP_OOC
      INTEGER ICNTL1
      INTEGER*8, DIMENSION(:),ALLOCATABLE :: AddVirtLibre
      LOGICAL,SAVE :: STRAT_IO_ASYNC,WITH_BUF,SOLVE,IS_INIT_CALLED
      INTEGER, DIMENSION(:),POINTER :: STEP_OOC,PROCNODE_OOC
      INTEGER, SAVE :: MYID_OOC,SLAVEF_OOC,LOW_LEVEL_STRAT_IO
      INTEGER(8), SAVE :: HBUF_SIZE, DIM_BUF_IO
      INTEGER,PARAMETER::ERR_STR_OOC_MAX_LEN=512
      CHARACTER*1 ERR_STR_OOC(ERR_STR_OOC_MAX_LEN)
      INTEGER DIM_ERR_STR_OOC
      TYPE IO_BLOCK
        INTEGER :: INODE
        LOGICAL :: MASTER
        INTEGER :: Typenode
        INTEGER :: NROW, NCOL, NFS
        LOGICAL :: Last
        INTEGER :: LastPiv
        INTEGER :: LastPanelWritten_L
        INTEGER :: LastPanelWritten_U
        INTEGER,POINTER,DIMENSION(:) :: INDICES
      END TYPE
      PUBLIC IO_BLOCK
      INTEGER,PUBLIC,PARAMETER::STRAT_WRITE_MAX=1,STRAT_TRY_WRITE=2
      END MODULE AGMG_MUMPS_OOC_COMM
      ! now merge also 34 with 32 & 33
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      MODULE DAGMG_MUMPS_OOC_BUFF
      USE AGMG_MUMPS_OOC_COMM
      IMPLICIT NONE
      PUBLIC
      INTEGER FIRST_HBUF,SECOND_HBUF
      PARAMETER (FIRST_HBUF=0, SECOND_HBUF=1)
      INTEGER,SAVE :: OOC_FCT_TYPE_LOC
      INTEGER TYPEF_L_LOC,TYPEF_U_LOC
      PARAMETER ( TYPEF_L_LOC=1, TYPEF_U_LOC=2 )
      INTEGER IO_STRAT
      DOUBLE PRECISION, DIMENSION(:),ALLOCATABLE :: BUF_IO
      LOGICAL,SAVE :: PANEL_FLAG
      INTEGER,SAVE :: EARLIEST_WRITE_MIN_SIZE
      INTEGER(8),SAVE,DIMENSION(:), ALLOCATABLE ::                      &
     &  I_SHIFT_FIRST_HBUF, I_SHIFT_SECOND_HBUF,                        &
     &  I_SHIFT_CUR_HBUF, I_REL_POS_CUR_HBUF
      INTEGER, SAVE, DIMENSION(:), ALLOCATABLE ::                       &
     &  LAST_IOREQUEST, CUR_HBUF
      INTEGER, DIMENSION(:),ALLOCATABLE :: I_CUR_HBUF_NEXTPOS
      INTEGER,SAVE ::  I_CUR_HBUF_FSTPOS,                               &
     &  I_SUB_HBUF_FSTPOS
      INTEGER(8) :: BufferEmpty
      PARAMETER (BufferEmpty=-1_8)
      INTEGER*8, DIMENSION(:),ALLOCATABLE :: NextAddVirtBuffer
      INTEGER*8, DIMENSION(:),ALLOCATABLE :: FIRST_VADDR_IN_BUF
      CONTAINS
      SUBROUTINE DAGMG_MUMPS_689(TYPEF_ARG)
      IMPLICIT NONE
      INTEGER TYPEF_ARG
      SELECT CASE(CUR_HBUF(TYPEF_ARG))
         CASE (FIRST_HBUF)
            CUR_HBUF(TYPEF_ARG) = SECOND_HBUF
            I_SHIFT_CUR_HBUF(TYPEF_ARG) =                               &
     &           I_SHIFT_SECOND_HBUF(TYPEF_ARG)
         CASE (SECOND_HBUF)
            CUR_HBUF(TYPEF_ARG) = FIRST_HBUF
            I_SHIFT_CUR_HBUF(TYPEF_ARG) =                               &
     &           I_SHIFT_FIRST_HBUF(TYPEF_ARG)
      END SELECT
      IF(.NOT.PANEL_FLAG)THEN
         I_SUB_HBUF_FSTPOS =I_CUR_HBUF_FSTPOS
         I_CUR_HBUF_FSTPOS =I_CUR_HBUF_NEXTPOS(TYPEF_ARG)
      ENDIF
      I_REL_POS_CUR_HBUF(TYPEF_ARG) = 1_8
      RETURN
      END SUBROUTINE DAGMG_MUMPS_689
      SUBROUTINE DAGMG_MUMPS_707(TYPEF_ARG,IERR)
      IMPLICIT NONE
      INTEGER TYPEF_ARG
      INTEGER NEW_IOREQUEST
      INTEGER IERR
      IERR=0
      CALL DAGMG_MUMPS_696(TYPEF_ARG,NEW_IOREQUEST,                          &
     &     IERR)
      IF(IERR.LT.0)THEN
         RETURN
      ENDIF
      IERR=0
      CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
      IF(IERR.LT.0)THEN
         IF (ICNTL1>0)                                                  &
     &   WRITE(ICNTL1,*) MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         RETURN
      ENDIF
      LAST_IOREQUEST(TYPEF_ARG) = NEW_IOREQUEST
      CALL DAGMG_MUMPS_689(TYPEF_ARG)
      IF(PANEL_FLAG)THEN
         NextAddVirtBuffer(TYPEF_ARG)=BufferEmpty
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_707
      SUBROUTINE DAGMG_MUMPS_675(IERR)
      IMPLICIT NONE
      INTEGER, intent(out) :: IERR
      INTEGER TYPEF_LAST
      INTEGER TYPEF_LOC
      IERR = 0
      TYPEF_LAST = OOC_FCT_TYPE_LOC
      DO TYPEF_LOC = 1, OOC_FCT_TYPE_LOC
         IERR=0
         CALL  DAGMG_MUMPS_707(TYPEF_LOC,IERR)
         IF(IERR.LT.0)THEN
            RETURN
         ENDIF
         IERR=0
         CALL DAGMG_MUMPS_707(TYPEF_LOC,IERR)
         IF(IERR.LT.0)THEN
            RETURN
         ENDIF
      ENDDO
      RETURN
      END SUBROUTINE DAGMG_MUMPS_675
      SUBROUTINE DAGMG_MUMPS_696(TYPEF_ARG,IOREQUEST,                        &
     &     IERR)
      IMPLICIT NONE
      INTEGER IOREQUEST,IERR
      INTEGER TYPEF_ARG
      INTEGER TOTAL_SIZE,FIRST_INODE
      INTEGER(8) :: FROM_BUFIO_POS, SIZE
      INTEGER TYPE
      INTEGER ADDR_INT1,ADDR_INT2
      INTEGER*8 TMP_VADDR
      INTEGER SIZE_INT1,SIZE_INT2
      IERR=0
      IF (I_REL_POS_CUR_HBUF(TYPEF_ARG) == 1_8) THEN
        IOREQUEST=-1
        RETURN
      END IF
      IF(PANEL_FLAG)THEN
         TYPE=TYPEF_ARG-1
         FIRST_INODE=-9999
         TMP_VADDR=FIRST_VADDR_IN_BUF(TYPEF_ARG)
      ELSE
         TYPE=FCT
         FIRST_INODE =                                                  &
     &        OOC_INODE_SEQUENCE(I_CUR_HBUF_FSTPOS,TYPEF_ARG)
         TMP_VADDR=OOC_VADDR(STEP_OOC(FIRST_INODE),TYPEF_ARG)
      ENDIF
      FROM_BUFIO_POS=I_SHIFT_CUR_HBUF(TYPEF_ARG)+1_8
      SIZE = I_REL_POS_CUR_HBUF(TYPEF_ARG)-1_8
      CALL AGMG_MUMPS_677(ADDR_INT1,ADDR_INT2,                               &
     &     TMP_VADDR)
      CALL AGMG_MUMPS_677(SIZE_INT1,SIZE_INT2,                               &
     &     SIZE)
      CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_WRITE_OOC_C')
      IF(IERR.LT.0)THEN
         IF (ICNTL1>0)                                                  &
     &   WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         RETURN
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_696
      SUBROUTINE DAGMG_MUMPS_669(I1,I2,IERR)
      IMPLICIT NONE
      INTEGER I1,I2,IERR
      INTEGER allocok
      IERR=0
      PANEL_FLAG=.FALSE.
      IF(allocated(I_SHIFT_FIRST_HBUF))THEN
         DEALLOCATE(I_SHIFT_FIRST_HBUF)
      ENDIF
      IF(allocated(I_SHIFT_SECOND_HBUF))THEN
         DEALLOCATE(I_SHIFT_SECOND_HBUF)
      ENDIF
      IF(allocated(I_SHIFT_CUR_HBUF))THEN
         DEALLOCATE(I_SHIFT_CUR_HBUF)
      ENDIF
      IF(allocated(I_REL_POS_CUR_HBUF))THEN
         DEALLOCATE(I_REL_POS_CUR_HBUF)
      ENDIF
      IF(allocated(LAST_IOREQUEST))THEN
         DEALLOCATE(LAST_IOREQUEST)
      ENDIF
      IF(allocated(CUR_HBUF))THEN
         DEALLOCATE(CUR_HBUF)
      ENDIF
      DIM_BUF_IO = int(KEEP_OOC(100),8)
      ALLOCATE(I_SHIFT_FIRST_HBUF(OOC_NB_FILE_TYPE),                    &
     &     stat=allocok)
      IF (allocok > 0) THEN
         IF (ICNTL1>0)                                                  &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         I1 = -13
         I2 = OOC_NB_FILE_TYPE
         IERR=-1
         RETURN
      ENDIF
      ALLOCATE(I_SHIFT_SECOND_HBUF(OOC_NB_FILE_TYPE),                   &
     &     stat=allocok)
      IF (allocok > 0) THEN
         IF (ICNTL1>0)                                                  &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         I1 = -13
         I2 = OOC_NB_FILE_TYPE
         IERR=-1
         RETURN
      ENDIF
      ALLOCATE(I_SHIFT_CUR_HBUF(OOC_NB_FILE_TYPE),                      &
     &     stat=allocok)
      IF (allocok > 0) THEN
         IF (ICNTL1>0)                                                  &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         I1 = -13
         I2 = OOC_NB_FILE_TYPE
         IERR=-1
         RETURN
      ENDIF
      ALLOCATE(I_REL_POS_CUR_HBUF(OOC_NB_FILE_TYPE),                    &
     &     stat=allocok)
      IF (allocok > 0) THEN
         IF (ICNTL1>0)                                                  &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         I1 = -13
         I2 = OOC_NB_FILE_TYPE
         IERR=-1
         RETURN
      ENDIF
      ALLOCATE(LAST_IOREQUEST(OOC_NB_FILE_TYPE),                        &
     &     stat=allocok)
      IF (allocok > 0) THEN
         IF (ICNTL1>0)                                                  &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         I1 = -13
         I2 = OOC_NB_FILE_TYPE
         IERR=-1
         RETURN
      ENDIF
      ALLOCATE(CUR_HBUF(OOC_NB_FILE_TYPE),                              &
     &     stat=allocok)
      IF (allocok > 0) THEN
         IF (ICNTL1>0)                                                  &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         I1 = -13
         I2 = OOC_NB_FILE_TYPE
         IERR=-1
         RETURN
      ENDIF
      OOC_FCT_TYPE_LOC=OOC_NB_FILE_TYPE
      ALLOCATE(BUF_IO(DIM_BUF_IO), stat=allocok)
      IF (allocok > 0) THEN
         IF (ICNTL1>0)                                                  &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         I1 = -13
         CALL AGMG_MUMPS_731(DIM_BUF_IO, I2)
         RETURN
      ENDIF
      PANEL_FLAG=(KEEP_OOC(201).EQ.1)
      IF (PANEL_FLAG) THEN
         IERR=0
         KEEP_OOC(228)=0
         IF(allocated(AddVirtLibre))THEN
            DEALLOCATE(AddVirtLibre)
         ENDIF
         ALLOCATE(AddVirtLibre(OOC_NB_FILE_TYPE), stat=allocok)
         IF (allocok > 0) THEN
            IF (ICNTL1>0)                                               &
     &      WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC_BUF_PANEL'
            IERR=-1
            I1=-13
            I2=OOC_NB_FILE_TYPE
            RETURN
         ENDIF
         AddVirtLibre(1:OOC_NB_FILE_TYPE)=0_8
         IF(allocated(NextAddVirtBuffer))THEN
            DEALLOCATE(NextAddVirtBuffer)
         ENDIF
         ALLOCATE(NextAddVirtBuffer(OOC_NB_FILE_TYPE), stat=allocok)
         IF (allocok > 0) THEN
            IF (ICNTL1>0)                                               &
     &      WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC_BUF_PANEL'
            IERR=-1
            I1=-13
            I2=OOC_NB_FILE_TYPE
            RETURN
         ENDIF
         NextAddVirtBuffer (1:OOC_NB_FILE_TYPE)  = BufferEmpty
         IF(allocated(FIRST_VADDR_IN_BUF))THEN
            DEALLOCATE(FIRST_VADDR_IN_BUF)
         ENDIF
         ALLOCATE(FIRST_VADDR_IN_BUF(OOC_NB_FILE_TYPE), stat=allocok)
         IF (allocok > 0) THEN
            IF (ICNTL1>0)                                               &
     &      WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC_BUF_PANEL'
            IERR=-1
            I1=-13
            I2=OOC_NB_FILE_TYPE
            RETURN
         ENDIF
         CALL DAGMG_MUMPS_686()
      ELSE
         CALL DAGMG_MUMPS_685()
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_669
      SUBROUTINE DAGMG_MUMPS_659()
      IMPLICIT NONE
      IF(allocated(BUF_IO))THEN
         DEALLOCATE(BUF_IO)
      ENDIF
      IF(allocated(I_SHIFT_FIRST_HBUF))THEN
         DEALLOCATE(I_SHIFT_FIRST_HBUF)
      ENDIF
      IF(allocated(I_SHIFT_SECOND_HBUF))THEN
         DEALLOCATE(I_SHIFT_SECOND_HBUF)
      ENDIF
      IF(allocated(I_SHIFT_CUR_HBUF))THEN
         DEALLOCATE(I_SHIFT_CUR_HBUF)
      ENDIF
      IF(allocated(I_REL_POS_CUR_HBUF))THEN
         DEALLOCATE(I_REL_POS_CUR_HBUF)
      ENDIF
      IF(allocated(LAST_IOREQUEST))THEN
         DEALLOCATE(LAST_IOREQUEST)
      ENDIF
      IF(allocated(CUR_HBUF))THEN
         DEALLOCATE(CUR_HBUF)
      ENDIF
      IF(PANEL_FLAG)THEN
         IF(allocated(NextAddVirtBuffer))THEN
            DEALLOCATE(NextAddVirtBuffer)
         ENDIF
         IF(allocated(AddVirtLibre))THEN
            DEALLOCATE(AddVirtLibre)
         ENDIF
         IF(allocated(FIRST_VADDR_IN_BUF))THEN
            DEALLOCATE(FIRST_VADDR_IN_BUF)
         ENDIF
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_659
      SUBROUTINE DAGMG_MUMPS_685()
      IMPLICIT NONE
      OOC_FCT_TYPE_LOC=1
      HBUF_SIZE = DIM_BUF_IO / int(2,kind=kind(DIM_BUF_IO))
      EARLIEST_WRITE_MIN_SIZE = 0
      I_SHIFT_FIRST_HBUF(OOC_FCT_TYPE_LOC) = 0_8
      I_SHIFT_SECOND_HBUF(OOC_FCT_TYPE_LOC) = HBUF_SIZE
      LAST_IOREQUEST(OOC_FCT_TYPE_LOC) = -1
      I_CUR_HBUF_NEXTPOS = 1
      I_CUR_HBUF_FSTPOS = 1
      I_SUB_HBUF_FSTPOS = 1
      CUR_HBUF(OOC_FCT_TYPE_LOC) = SECOND_HBUF
      CALL DAGMG_MUMPS_689(OOC_FCT_TYPE_LOC)
      END SUBROUTINE DAGMG_MUMPS_685
      SUBROUTINE DAGMG_MUMPS_678(BLOCK,SIZE_OF_BLOCK,                        &
     &     IERR)
      IMPLICIT NONE
      INTEGER(8) :: SIZE_OF_BLOCK
      DOUBLE PRECISION BLOCK(SIZE_OF_BLOCK)
      INTEGER SIZE
      INTEGER, intent(out) :: IERR
      INTEGER(8) :: I
      IERR=0
      IF (I_REL_POS_CUR_HBUF(OOC_FCT_TYPE_LOC) +                        &
     &    SIZE_OF_BLOCK <= HBUF_SIZE + 1_8) THEN
      ELSE
        CALL DAGMG_MUMPS_707(OOC_FCT_TYPE_LOC,IERR)
        IF(IERR.LT.0)THEN
           RETURN
        ENDIF
      END IF
      DO I = 1_8, SIZE_OF_BLOCK
        BUF_IO(I_SHIFT_CUR_HBUF(OOC_FCT_TYPE_LOC) +                     &
     &        I_REL_POS_CUR_HBUF(OOC_FCT_TYPE_LOC) + I - 1_8) =         &
     &    BLOCK(I)
      END DO
      I_REL_POS_CUR_HBUF(OOC_FCT_TYPE_LOC) =                            &
     &     I_REL_POS_CUR_HBUF(OOC_FCT_TYPE_LOC) + SIZE_OF_BLOCK
      RETURN
      END SUBROUTINE DAGMG_MUMPS_678
      SUBROUTINE DAGMG_MUMPS_686()
      IMPLICIT NONE
      INTEGER(8) :: DIM_BUF_IO_L_OR_U
      INTEGER TYPEF, TYPEF_LAST
      INTEGER NB_DOUBLE_BUFFERS
      IF (KEEP_OOC(50) .EQ.0) THEN
        NB_DOUBLE_BUFFERS = 2
        TYPEF_LAST = 2
      ELSE
        NB_DOUBLE_BUFFERS = 1
        TYPEF_LAST = 1
      ENDIF
      DIM_BUF_IO_L_OR_U = DIM_BUF_IO /                                  &
     & int(NB_DOUBLE_BUFFERS,KIND=kind(DIM_BUF_IO_L_OR_U))
      IF(.NOT.STRAT_IO_ASYNC)THEN
         HBUF_SIZE = DIM_BUF_IO_L_OR_U
      ELSE
         HBUF_SIZE = DIM_BUF_IO_L_OR_U / 2_8
      ENDIF
      DO TYPEF = 1, TYPEF_LAST
        LAST_IOREQUEST(TYPEF) = -1
        IF (TYPEF == 1 ) THEN
          I_SHIFT_FIRST_HBUF(TYPEF) = 0_8
        ELSE
          I_SHIFT_FIRST_HBUF(TYPEF) = DIM_BUF_IO_L_OR_U
        ENDIF
        IF(.NOT.STRAT_IO_ASYNC)THEN
           I_SHIFT_SECOND_HBUF(TYPEF) = I_SHIFT_FIRST_HBUF(TYPEF)
        ELSE
           I_SHIFT_SECOND_HBUF(TYPEF) = I_SHIFT_FIRST_HBUF(TYPEF) +     &
     &          HBUF_SIZE
        ENDIF
        CUR_HBUF(TYPEF) = SECOND_HBUF
        CALL DAGMG_MUMPS_689(TYPEF)
      ENDDO
      I_CUR_HBUF_NEXTPOS = 1
      RETURN
      END SUBROUTINE DAGMG_MUMPS_686
      SUBROUTINE DAGMG_MUMPS_706(TYPEF,IERR)
      IMPLICIT NONE
      INTEGER, INTENT(in)  :: TYPEF
      INTEGER, INTENT(out) :: IERR
      INTEGER IFLAG
      INTEGER NEW_IOREQUEST
      IERR=0
      CALL AGMG_NOCALL('MUMPS_TEST_REQUEST_C')
      IF (IFLAG.EQ.1) THEN
         IERR = 0
         CALL DAGMG_MUMPS_696(TYPEF,                                         &
     &        NEW_IOREQUEST,                                            &
     &        IERR)
         IF(IERR.LT.0)THEN
            RETURN
         ENDIF
         LAST_IOREQUEST(TYPEF) = NEW_IOREQUEST
         CALL DAGMG_MUMPS_689(TYPEF)
         NextAddVirtBuffer(TYPEF)=BufferEmpty
         RETURN
      ELSE IF(IFLAG.LT.0)THEN
         WRITE(*,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         RETURN
      ELSE
         IERR = 1
         RETURN
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_706
      SUBROUTINE DAGMG_MUMPS_709(TYPEF,VADDR)
      IMPLICIT NONE
      INTEGER*8, INTENT(in) :: VADDR
      INTEGER, INTENT(in) :: TYPEF
      IF(I_REL_POS_CUR_HBUF(TYPEF).EQ.1_8)THEN
         FIRST_VADDR_IN_BUF(TYPEF)=VADDR
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_709
      SUBROUTINE DAGMG_MUMPS_653( STRAT, TYPEF, MonBloc,                     &
     &     AFAC, LAFAC,                                                 &
     &     AddVirtCour, IPIVBEG, IPIVEND, LPANELeff,                    &
     &     IERR)
      IMPLICIT NONE
      INTEGER,          INTENT(IN) :: TYPEF, IPIVBEG, IPIVEND, STRAT
      INTEGER(8),       INTENT(IN) :: LAFAC
      DOUBLE PRECISION, INTENT(IN) :: AFAC(LAFAC)
      INTEGER*8,        INTENT(IN) :: AddVirtCour
      TYPE(IO_BLOCK),   INTENT(IN) :: MonBloc
      INTEGER,          INTENT(OUT):: LPANELeff
      INTEGER,          INTENT(OUT):: IERR
      INTEGER   :: II, NBPIVeff
      INTEGER(8) :: IPOS, IDIAG, IDEST
      INTEGER(8) :: DeltaIPOS
      INTEGER   :: StrideIPOS
      IERR=0
      IF (STRAT.NE.STRAT_WRITE_MAX.AND.STRAT.NE.STRAT_TRY_WRITE) THEN
         write(6,*) ' DMUMPS_653: STRAT Not implemented '
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      NBPIVeff = IPIVEND - IPIVBEG + 1
      IF (MonBloc%MASTER .AND. MonBloc%TYPENODE .NE. 3) THEN
        IF (TYPEF.EQ.TYPEF_L) THEN
          LPANELeff = (MonBloc%NROW-IPIVBEG+1)*NBPIVeff
        ELSE
          LPANELeff = (MonBloc%NCOL-IPIVBEG+1)*NBPIVeff
        ENDIF
      ELSE
          LPANELeff = MonBloc%NROW*NBPIVeff
      ENDIF
      IF ( ( I_REL_POS_CUR_HBUF(TYPEF) + int(LPANELeff - 1,8)           &
     &     >                                                            &
     &     HBUF_SIZE )                                                  &
     &     .OR.                                                         &
     &     ( (AddVirtCour.NE.NextAddVirtBuffer(TYPEF)) .AND.            &
     &     (NextAddVirtBuffer(TYPEF).NE.BufferEmpty) )                  &
     &     ) THEN
         IF (STRAT.EQ.STRAT_WRITE_MAX) THEN
            CALL DAGMG_MUMPS_707(TYPEF,IERR)
         ELSE IF (STRAT.EQ.STRAT_TRY_WRITE) THEN
            CALL DAGMG_MUMPS_706(TYPEF,IERR)
            IF (IERR.EQ.1) RETURN
         ELSE
            write(6,*) 'DMUMPS_653: STRAT Not implemented'
         ENDIF
      ENDIF
      IF (IERR < 0 ) THEN
        RETURN
      ENDIF
      IF (NextAddVirtBuffer(TYPEF).EQ. BufferEmpty) THEN
         CALL DAGMG_MUMPS_709(TYPEF,AddVirtCour)
         NextAddVirtBuffer(TYPEF) = AddVirtCour
      ENDIF
      IF (MonBloc%MASTER .AND. MonBloc%TYPENODE .NE. 3) THEN
         IDIAG =  int(IPIVBEG-1,8)*int(MonBloc%NCOL,8) + int(IPIVBEG,8)
         IPOS   = IDIAG
         IDEST = I_SHIFT_CUR_HBUF(TYPEF) +                              &
     &        I_REL_POS_CUR_HBUF(TYPEF)
         IF (TYPEF.EQ.TYPEF_L) THEN
            DO II = IPIVBEG, IPIVEND
               CALL DCOPY(MonBloc%NROW-IPIVBEG+1,                       &
     &              AFAC(IPOS), MonBloc%NCOL,                           &
     &              BUF_IO(IDEST), 1)
               IDEST = IDEST + int(MonBloc%NROW-IPIVBEG+1,8)
               IPOS  = IPOS  + 1_8
            ENDDO
         ELSE
            DO II = IPIVBEG, IPIVEND
               CALL DCOPY(MonBloc%NCOL-IPIVBEG+1,                       &
     &              AFAC(IPOS), 1,                                      &
     &              BUF_IO(IDEST), 1)
               IDEST = IDEST + int(MonBloc%NCOL-IPIVBEG+1,8)
               IPOS  = IPOS  + int(MonBloc%NCOL,8)
            ENDDO
         ENDIF
      ELSE
         IDEST = I_SHIFT_CUR_HBUF(TYPEF) +                              &
     &        I_REL_POS_CUR_HBUF(TYPEF)
         IF (MonBloc%Typenode.EQ.3) THEN
           DeltaIPOS  = int(MonBloc%NROW,8)
           StrideIPOS = 1
         ELSE
           DeltaIPOS  = 1_8
           StrideIPOS = MonBloc%NCOL
         ENDIF
         IPOS  = 1_8 + int(IPIVBEG - 1,8) * DeltaIPOS
         DO II = IPIVBEG, IPIVEND
            CALL DCOPY(MonBloc%NROW,                                    &
     &           AFAC(IPOS), StrideIPOS,                                &
     &           BUF_IO(IDEST), 1)
            IDEST = IDEST+int(MonBloc%NROW,8)
            IPOS  = IPOS + DeltaIPOS
         ENDDO
      ENDIF
      I_REL_POS_CUR_HBUF(TYPEF) =                                       &
     &     I_REL_POS_CUR_HBUF(TYPEF) + int(LPANELeff,8)
      NextAddVirtBuffer(TYPEF) = NextAddVirtBuffer(TYPEF)               &
     &                         + int(LPANELeff,8)
      RETURN
      END SUBROUTINE DAGMG_MUMPS_653
      END MODULE DAGMG_MUMPS_OOC_BUFF
      ! added 35 now to mix
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
        MODULE DAGMG_MUMPS_COMM_BUFF
        PRIVATE
        PUBLIC :: DAGMG_MUMPS_61, DAGMG_MUMPS_528,                                &
     &   DAGMG_MUMPS_53 ,       DAGMG_MUMPS_57 ,                                  &
     &   DAGMG_MUMPS_55, DAGMG_MUMPS_59,                                          &
     &   DAGMG_MUMPS_54,DAGMG_MUMPS_58,                                           &
     &   DAGMG_MUMPS_66,     DAGMG_MUMPS_78,                                      &
     &   DAGMG_MUMPS_62,       DAGMG_MUMPS_68,                                    &
     &   DAGMG_MUMPS_71, DAGMG_MUMPS_70,                                          &
     &   DAGMG_MUMPS_67,                                                     &
     &   DAGMG_MUMPS_65,                                                     &
     &   DAGMG_MUMPS_64,                                                     &
     &   DAGMG_MUMPS_72,                                                     &
     &   DAGMG_MUMPS_648,                                                    &
     &   DAGMG_MUMPS_76,                                                     &
     &   DAGMG_MUMPS_73, DAGMG_MUMPS_74,                                          &
     &   DAGMG_MUMPS_63,DAGMG_MUMPS_77,                                           &
     &   DAGMG_MUMPS_60,                                                     &
     &   DAGMG_MUMPS_524, DAGMG_MUMPS_469,                                        &
     &   DAGMG_MUMPS_460, DAGMG_MUMPS_502,                                        &
     &   DAGMG_MUMPS_519                                                     &
     &   ,DAGMG_MUMPS_620                                                    &
     &   ,DAGMG_MUMPS_617
        INTEGER NEXT, REQ, CONTENT, OVHSIZE
        PARAMETER( NEXT = 0, REQ = 1, CONTENT = 2, OVHSIZE = 2 )
        INTEGER, SAVE :: SIZEofINT, SIZEofREAL, BUF_MYID
        TYPE DAGMG_MUMPS_COMM_BUFFER_TYPE
          INTEGER LBUF, HEAD, TAIL,LBUF_INT, ILASTMSG
          INTEGER, DIMENSION(:),POINTER :: CONTENT
        END TYPE DAGMG_MUMPS_COMM_BUFFER_TYPE
        TYPE ( DAGMG_MUMPS_COMM_BUFFER_TYPE ), SAVE :: BUF_CB
        TYPE ( DAGMG_MUMPS_COMM_BUFFER_TYPE ), SAVE :: BUF_SMALL
        TYPE ( DAGMG_MUMPS_COMM_BUFFER_TYPE ), SAVE :: BUF_LOAD
        INTEGER, SAVE :: SIZE_RBUF_BYTES
        INTEGER BUF_LMAX_ARRAY
        DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: BUF_MAX_ARRAY
        PUBLIC :: BUF_LMAX_ARRAY, BUF_MAX_ARRAY
      CONTAINS
        SUBROUTINE DAGMG_MUMPS_528( MYID )
        IMPLICIT NONE
        INTEGER MYID
        BUF_MYID  = MYID
        RETURN
        END SUBROUTINE DAGMG_MUMPS_528
        SUBROUTINE DAGMG_MUMPS_61( IntSize, RealSize )
        IMPLICIT NONE
        INTEGER IntSize, RealSize
        SIZEofINT = IntSize
        SIZEofREAL = RealSize
        NULLIFY(BUF_CB  %CONTENT)
        NULLIFY(BUF_SMALL%CONTENT)
        NULLIFY(BUF_LOAD%CONTENT)
        BUF_CB%LBUF     = 0
        BUF_CB%LBUF_INT = 0
        BUF_CB%HEAD     = 1
        BUF_CB%TAIL     = 1
        BUF_CB%ILASTMSG = 1
        BUF_SMALL%LBUF     = 0
        BUF_SMALL%LBUF_INT = 0
        BUF_SMALL%HEAD     = 1
        BUF_SMALL%TAIL     = 1
        BUF_SMALL%ILASTMSG = 1
        BUF_LOAD%LBUF     = 0
        BUF_LOAD%LBUF_INT = 0
        BUF_LOAD%HEAD     = 1
        BUF_LOAD%TAIL     = 1
        BUF_LOAD%ILASTMSG = 1
        RETURN
        END SUBROUTINE DAGMG_MUMPS_61
        SUBROUTINE DAGMG_MUMPS_53( SIZE, IERR )
        IMPLICIT NONE
        INTEGER SIZE, IERR
        CALL DAGMG_MUMPS_2( BUF_CB, SIZE, IERR )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_53
        SUBROUTINE DAGMG_MUMPS_55( SIZE, IERR )
        IMPLICIT NONE
        INTEGER SIZE, IERR
        CALL DAGMG_MUMPS_2( BUF_SMALL, SIZE, IERR )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_55
        SUBROUTINE DAGMG_MUMPS_54( SIZE, IERR )
        IMPLICIT NONE
        INTEGER SIZE, IERR
        INTEGER stat
        CALL DAGMG_MUMPS_2( BUF_LOAD, SIZE, IERR )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_54
        SUBROUTINE DAGMG_MUMPS_58( IERR )
        IMPLICIT NONE
        INTEGER IERR
        CALL DAGMG_MUMPS_3( BUF_LOAD, IERR )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_58
        SUBROUTINE DAGMG_MUMPS_620()
        IMPLICIT NONE
        IF (allocated( BUF_MAX_ARRAY)) DEALLOCATE( BUF_MAX_ARRAY )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_620
        SUBROUTINE DAGMG_MUMPS_617(NFS4FATHER,IERR)
        IMPLICIT NONE
        INTEGER IERR, NFS4FATHER
        IERR = 0
        IF (allocated( BUF_MAX_ARRAY)) THEN
          IF (BUF_LMAX_ARRAY .GE. NFS4FATHER) RETURN
          DEALLOCATE( BUF_MAX_ARRAY )
        ENDIF
        ALLOCATE(BUF_MAX_ARRAY(NFS4FATHER),stat=IERR)
        BUF_LMAX_ARRAY=NFS4FATHER
        RETURN
        END SUBROUTINE DAGMG_MUMPS_617
        SUBROUTINE DAGMG_MUMPS_57( IERR )
        IMPLICIT NONE
        INTEGER IERR
        CALL DAGMG_MUMPS_3( BUF_CB, IERR )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_57
        SUBROUTINE DAGMG_MUMPS_59( IERR )
        IMPLICIT NONE
        INTEGER IERR
        CALL DAGMG_MUMPS_3( BUF_SMALL, IERR )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_59
        SUBROUTINE DAGMG_MUMPS_2( BUF, SIZE, IERR )
        IMPLICIT NONE
        TYPE ( DAGMG_MUMPS_COMM_BUFFER_TYPE ) :: BUF
        INTEGER SIZE, IERR
        IERR         = 0
        BUF%LBUF     = SIZE
        BUF%LBUF_INT = ( SIZE + SIZEofINT - 1 ) / SIZEofINT
        IF ( associated ( BUF%CONTENT ) ) DEALLOCATE( BUF%CONTENT )
        ALLOCATE( BUF%CONTENT( BUF%LBUF_INT ), stat = IERR )
        IF (IERR .NE. 0) THEN
          NULLIFY( BUF%CONTENT )
          IERR         = -1
          BUF%LBUF     =  0
          BUF%LBUF_INT =  0
        END IF
        BUF%HEAD     = 1
        BUF%TAIL     = 1
        BUF%ILASTMSG = 1
        RETURN
        END SUBROUTINE DAGMG_MUMPS_2
        SUBROUTINE DAGMG_MUMPS_3( BUF, IERR )
        IMPLICIT NONE
        TYPE ( DAGMG_MUMPS_COMM_BUFFER_TYPE ) :: BUF
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
        INTEGER IERR
        INTEGER STATUS( AGMG_MPF_STATUS_SIZE )
        LOGICAL FLAG
        IF ( .NOT. associated ( BUF%CONTENT ) ) THEN
          BUF%HEAD     = 1
          BUF%LBUF     = 0
          BUF%LBUF_INT = 0
          BUF%TAIL     = 1
          BUF%ILASTMSG = 1
          RETURN
        END IF
        DO WHILE ( BUF%HEAD.NE.0 .AND. BUF%HEAD .NE. BUF%TAIL )
          CALL AGMG_MPF_TEST(BUF%CONTENT( BUF%HEAD + REQ ), FLAG,            &
     &                  STATUS, IERR)
          IF ( .not. FLAG ) THEN
            WRITE(*,*) '** Warning: trying to cancel a request.'
            WRITE(*,*) '** This might be problematic on SGI'
            CALL AGMG_SETIERR( IERR )
            CALL AGMG_SETIERR( IERR )
          END IF
          BUF%HEAD = BUF%CONTENT( BUF%HEAD + NEXT )
        END DO
        DEALLOCATE( BUF%CONTENT )
        NULLIFY( BUF%CONTENT )
        BUF%LBUF     = 0
        BUF%LBUF_INT = 0
        BUF%HEAD     = 1
        BUF%TAIL     = 1
        BUF%ILASTMSG = 1
        RETURN
        END SUBROUTINE DAGMG_MUMPS_3
        SUBROUTINE DAGMG_MUMPS_66( NBROWS_ALREADY_SENT,                      &
     &                                INODE, FPERE, NFRONT, LCONT,      &
     &                                NASS, NPIV,                       &
     &                                IWROW, IWCOL, A, COMPRESSCB,      &
     &                                DEST, TAG, COMM, IERR )
        IMPLICIT NONE
        INTEGER DEST, TAG, COMM, IERR
        INTEGER NBROWS_ALREADY_SENT
        INTEGER INODE, FPERE, NFRONT, LCONT, NASS, NPIV
        INTEGER IWROW( LCONT ), IWCOL( LCONT )
        DOUBLE PRECISION A( * )
        LOGICAL COMPRESSCB
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
        INTEGER NBROWS_PACKET
        INTEGER POSITION, IREQ, IPOS, I, J1
        INTEGER SIZE1, SIZE2, SIZE_PACK, SIZE_AV, SIZE_AV_REALS
        INTEGER IZERO, IONE
        INTEGER SIZECB
        INTEGER LCONT_SENT
        INTEGER DEST2(1)
        PARAMETER( IZERO = 0, IONE = 1 )
        LOGICAL RECV_BUF_SMALLER_THAN_SEND
        DEST2(1) = DEST
        IERR = 0
        IF (COMPRESSCB) THEN
          SIZECB = (LCONT*(LCONT+1))/2
        ELSE
          SIZECB = LCONT*LCONT
        ENDIF
      IF (NBROWS_ALREADY_SENT == 0) THEN
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
      ELSE
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
      ENDIF
      CALL DAGMG_MUMPS_79( BUF_CB, SIZE_AV )
      IF ( SIZE_AV .LT. SIZE_RBUF_BYTES ) THEN
        RECV_BUF_SMALLER_THAN_SEND = .FALSE.
      ELSE
        SIZE_AV = SIZE_RBUF_BYTES
        RECV_BUF_SMALLER_THAN_SEND = .TRUE.
      ENDIF
      SIZE_AV_REALS = ( SIZE_AV - SIZE1 ) / SIZEofREAL
      IF (SIZE_AV_REALS < 0 ) THEN
        NBROWS_PACKET = 0
      ELSE
        IF (COMPRESSCB) THEN
          NBROWS_PACKET = int( ( sqrt(                                  &
     &                     (2.0D0*dble(NBROWS_ALREADY_SENT)-1.0D0)*     &
     &                     (2.0D0*dble(NBROWS_ALREADY_SENT)-1.0D0)      &
     &                     + 8.0D0 * dble(SIZE_AV_REALS))               &
     &                  - 2.0D0 * dble(NBROWS_ALREADY_SENT) + 1.0D0 )   &
     &                  /2.0D0 )
        ELSE
          NBROWS_PACKET = SIZE_AV_REALS / LCONT
        ENDIF
      ENDIF
   10 CONTINUE
      NBROWS_PACKET = max(0,                                            &
     &          min(NBROWS_PACKET, LCONT - NBROWS_ALREADY_SENT))
      IF (NBROWS_PACKET .EQ. 0 .AND. LCONT .NE. 0) THEN
        IF (RECV_BUF_SMALLER_THAN_SEND) THEN
          IERR = -3
          GOTO 100
        ELSE
          IERR = -1
          GOTO 100
        ENDIF
      ENDIF
      IF (COMPRESSCB) THEN
        SIZECB = (NBROWS_ALREADY_SENT*NBROWS_PACKET)+(NBROWS_PACKET     &
     &           *(NBROWS_PACKET+1))/2
      ELSE
        SIZECB = NBROWS_PACKET * LCONT
      ENDIF
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE_PACK = SIZE1 + SIZE2
       IF (SIZE_PACK .GT. SIZE_AV ) THEN
          NBROWS_PACKET = NBROWS_PACKET - 1
          IF (NBROWS_PACKET > 0) THEN
             GOTO 10
          ELSE
             IF (RECV_BUF_SMALLER_THAN_SEND) THEN
               IERR=-3
               GOTO 100
             ELSE
               IERR = -1
               GOTO 100
             ENDIF
          ENDIF
       ENDIF
       IF (NBROWS_PACKET + NBROWS_ALREADY_SENT.NE.LCONT .AND.           &
     &   SIZE_PACK  .LT. SIZE_RBUF_BYTES / 4                            &
     &  .AND.                                                           &
     &   .NOT. RECV_BUF_SMALLER_THAN_SEND)                              &
     &   THEN
           IERR = -1
           GOTO 100
       ENDIF
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE_PACK, IERR,             &
     &                 IONE , DEST2                                     &
     &               )
        IF (IERR .EQ. -1 .OR. IERR .EQ. -2) THEN
          NBROWS_PACKET = NBROWS_PACKET - 1
          IF ( NBROWS_PACKET > 0 )  GOTO 10
        ENDIF
        IF ( IERR .LT. 0 ) GOTO 100
          IF (SIZE_PACK.GT.SIZE_RBUF_BYTES ) THEN
             IERR = -3
             RETURN
          ENDIF
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IF (COMPRESSCB) THEN
          LCONT_SENT=-LCONT
        ELSE
          LCONT_SENT=LCONT
        ENDIF
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
      IF (NBROWS_ALREADY_SENT == 0) THEN
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
       ENDIF
        IF ( LCONT .NE. 0 ) THEN
          J1 = 1 + NBROWS_ALREADY_SENT * NFRONT
          IF (COMPRESSCB) THEN
           DO I = NBROWS_ALREADY_SENT+1,                                &
     &            NBROWS_ALREADY_SENT+NBROWS_PACKET
            CALL AGMG_NOCALL('MPI_PACK')
             J1 = J1 + NFRONT
           END DO
          ELSE
           DO I = NBROWS_ALREADY_SENT+1,                                &
     &            NBROWS_ALREADY_SENT+NBROWS_PACKET
            CALL AGMG_NOCALL('MPI_PACK')
             J1 = J1 + NFRONT
           END DO
          ENDIF
        END IF
        CALL AGMG_NOCALL('MPI_ISEND')
        IF ( SIZE_PACK .LT. POSITION ) THEN
          WRITE(*,*) 'Try_send_cb: SIZE, POSITION = ', SIZE_PACK,       &
     &               POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE_PACK .NE. POSITION )                                  &
     &  CALL DAGMG_MUMPS_1( BUF_CB, POSITION )
        NBROWS_ALREADY_SENT = NBROWS_ALREADY_SENT + NBROWS_PACKET
        IF (NBROWS_ALREADY_SENT .NE. LCONT ) THEN
          IERR = -1
          RETURN
        ENDIF
  100   CONTINUE
        RETURN
        END SUBROUTINE DAGMG_MUMPS_66
        SUBROUTINE DAGMG_MUMPS_72( NRHS, INODE, IFATH,                       &
     &             EFF_CB_SIZE, LD_CB, LD_PIV, NPIV, CB, SOL,           &
     &             DEST, COMM, IERR )
        IMPLICIT NONE
        INTEGER NRHS, INODE, IFATH, EFF_CB_SIZE, LD_CB, LD_PIV, NPIV
        INTEGER DEST, COMM, IERR
        DOUBLE PRECISION CB( LD_CB*(NRHS-1)+EFF_CB_SIZE )
        DOUBLE PRECISION SOL( max(1, LD_PIV*(NRHS-1)+NPIV) )
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER SIZE, SIZE1, SIZE2, K
        INTEGER POSITION, IREQ, IPOS
        INTEGER IONE
        INTEGER DEST2(1)
        PARAMETER ( IONE=1 )
        DEST2(1) = DEST
        IERR = 0
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE = SIZE1 + SIZE2
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE, IERR,                  &
     &                 IONE , DEST2                                     &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        DO K = 1, NRHS
               CALL AGMG_NOCALL('MPI_PACK')
        END DO
        IF ( NPIV .GT. 0 ) THEN
          DO K=1, NRHS
          CALL AGMG_NOCALL('MPI_PACK')
          ENDDO
        END IF
        CALL AGMG_NOCALL('MPI_ISEND')
        IF ( SIZE .LT. POSITION ) THEN
          WRITE(*,*) 'Try_send_master2slave: SIZE, POSITION = ',        &
     &               SIZE, POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE .NE. POSITION ) CALL DAGMG_MUMPS_1( BUF_CB, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_72
        SUBROUTINE DAGMG_MUMPS_78( NRHS, NODE1, NODE2, NCB, LDW,             &
     &             LONG,                                                &
     &             IW, W,                                               &
     &             DEST, TAG, COMM, IERR )
        IMPLICIT NONE
        INTEGER LDW, DEST, TAG, COMM, IERR
        INTEGER NRHS, NODE1, NODE2, NCB, LONG
        INTEGER IW( max( 1, LONG ) )
        DOUBLE PRECISION W( max( 1, LDW * NRHS ) )
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
        INTEGER POSITION, IREQ, IPOS
        INTEGER SIZE1, SIZE2, SIZE, K
        INTEGER IONE
        INTEGER DEST2(1)
        PARAMETER ( IONE=1 )
        DEST2(1)=DEST
        IERR = 0
        IF ( NODE2 .EQ. 0 ) THEN
         CALL AGMG_NOCALL('MPI_PACK_SIZE')
        ELSE
         CALL AGMG_NOCALL('MPI_PACK_SIZE')
        END IF
        SIZE2 = 0
        IF ( LONG .GT. 0 ) THEN
          CALL AGMG_NOCALL('MPI_PACK_SIZE')
        END IF
        SIZE = SIZE1 + SIZE2
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE, IERR,                  &
     &                 IONE , DEST2                                     &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        IF ( NODE2 .NE. 0 ) THEN
          CALL AGMG_NOCALL('MPI_PACK')
          CALL AGMG_NOCALL('MPI_PACK')
        END IF
        CALL AGMG_NOCALL('MPI_PACK')
        IF ( LONG .GT. 0 ) THEN
          CALL AGMG_NOCALL('MPI_PACK')
          DO K=1, NRHS
          CALL AGMG_NOCALL('MPI_PACK')
          END DO
        END IF
        CALL AGMG_NOCALL('MPI_ISEND')
        IF ( SIZE .NE. POSITION ) CALL DAGMG_MUMPS_1( BUF_CB, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_78
        SUBROUTINE DAGMG_MUMPS_62( I, DEST, TAG, COMM, IERR )
        IMPLICIT NONE
        INTEGER I
        INTEGER DEST, TAG, COMM, IERR
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
        INTEGER IPOS, IREQ, MSG_SIZE, POSITION
        INTEGER IONE
        INTEGER DEST2(1)
        PARAMETER ( IONE=1 )
        DEST2(1)=DEST
        IERR = 0
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        CALL DAGMG_MUMPS_4( BUF_SMALL, IPOS, IREQ, MSG_SIZE, IERR,           &
     &                 IONE , DEST2                                     &
     &               )
        IF ( IERR .LT. 0 ) THEN
         write(6,*) ' Internal error in DMUMPS_62',                     &
     &       ' Buf size (bytes)= ',BUF_SMALL%LBUF
         RETURN
        ENDIF
        POSITION=0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_ISEND')
        RETURN
        END SUBROUTINE DAGMG_MUMPS_62
        SUBROUTINE DAGMG_MUMPS_469(FLAG)
        LOGICAL FLAG
        LOGICAL FLAG1, FLAG2, FLAG3
        CALL DAGMG_MUMPS_468( BUF_SMALL, FLAG1 )
        CALL DAGMG_MUMPS_468( BUF_CB, FLAG2 )
        CALL DAGMG_MUMPS_468( BUF_LOAD, FLAG3 )
        FLAG = FLAG1 .AND. FLAG2 .AND. FLAG3
        RETURN
        END SUBROUTINE DAGMG_MUMPS_469
        SUBROUTINE DAGMG_MUMPS_468( B, FLAG )
        TYPE ( DAGMG_MUMPS_COMM_BUFFER_TYPE ) :: B
        LOGICAL :: FLAG
        INTEGER SIZE_AVAIL
        CALL DAGMG_MUMPS_79(B, SIZE_AVAIL)
        FLAG = ( B%HEAD == B%TAIL )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_468
        SUBROUTINE DAGMG_MUMPS_79( B, SIZE_AV )
        IMPLICIT NONE
        TYPE ( DAGMG_MUMPS_COMM_BUFFER_TYPE ) :: B
        INTEGER SIZE_AV
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
        INTEGER IERR
        INTEGER STATUS( AGMG_MPF_STATUS_SIZE )
        LOGICAL FLAG
        IF ( B%HEAD .NE. B%TAIL ) THEN
   10     CONTINUE
          CALL AGMG_MPF_TEST( B%CONTENT( B%HEAD + REQ ), FLAG, STATUS, IERR )
          IF ( FLAG ) THEN
            B%HEAD = B%CONTENT( B%HEAD + NEXT )
            IF ( B%HEAD .EQ. 0 ) B%HEAD = B%TAIL
            IF ( B%HEAD .NE. B%TAIL ) GOTO 10
          END IF
        END IF
        IF ( B%HEAD .EQ. B%TAIL ) THEN
          B%HEAD = 1
          B%TAIL = 1
          B%ILASTMSG = 1
        END IF
        IF ( B%HEAD .LE. B%TAIL ) THEN
           SIZE_AV = max( B%LBUF_INT - B%TAIL, B%HEAD - 2 )
        ELSE
           SIZE_AV = B%HEAD - B%TAIL - 1
        END IF
        SIZE_AV = min(SIZE_AV - OVHSIZE, SIZE_AV)
        SIZE_AV = SIZE_AV * SIZEofINT
        RETURN
        END SUBROUTINE DAGMG_MUMPS_79
        SUBROUTINE DAGMG_MUMPS_4( B, IPOS, IREQ, MSG_SIZE, IERR,             &
     &    NDEST , PDEST                                                 &
     &         )
        IMPLICIT NONE
        TYPE ( DAGMG_MUMPS_COMM_BUFFER_TYPE ) :: B
        INTEGER, INTENT(IN)        :: MSG_SIZE
        INTEGER, INTENT(OUT)       :: IPOS, IREQ, IERR
        INTEGER NDEST
        INTEGER, INTENT(IN)        :: PDEST(max(1,NDEST))
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
        INTEGER MSG_SIZE_INT
        INTEGER IBUF
        LOGICAL FLAG
        INTEGER STATUS( AGMG_MPF_STATUS_SIZE )
        IERR = 0
        IF ( B%HEAD .NE. B%TAIL ) THEN
   10     CONTINUE
          CALL AGMG_MPF_TEST( B%CONTENT( B%HEAD + REQ ), FLAG, STATUS, IERR )
          IF ( FLAG ) THEN
            B%HEAD = B%CONTENT( B%HEAD + NEXT )
            IF ( B%HEAD .EQ. 0 ) B%HEAD = B%TAIL
            IF ( B%HEAD .NE. B%TAIL ) GOTO 10
          END IF
        END IF
        IF ( B%HEAD .EQ. B%TAIL ) THEN
          B%HEAD = 1
          B%TAIL = 1
          B%ILASTMSG = 1
        END iF
        MSG_SIZE_INT = ( MSG_SIZE + ( SIZEofINT - 1 ) ) / SIZEofINT
        MSG_SIZE_INT = MSG_SIZE_INT + OVHSIZE
        FLAG = (     ( B%HEAD .LE. B%TAIL )                             &
     &               .AND. (                                            &
     &                 ( MSG_SIZE_INT .LE. B%LBUF_INT - B%TAIL )        &
     &                 .OR. ( MSG_SIZE_INT .LE. B%HEAD - 2 ) ) )        &
     &         .OR.                                                     &
     &               ( ( B%HEAD .GT. B%TAIL )                           &
     &               .AND. ( MSG_SIZE_INT .LE. B%HEAD - B%TAIL - 1 ) )
        IF ( .NOT. FLAG                                                 &
     &    ) THEN
          IERR = -1
          IF ( MSG_SIZE_INT .GT. B%LBUF_INT - 1 ) then
            IERR = -2
          ENDIF
          IPOS = -1
          IREQ = -1
          RETURN
        END IF
        IF ( B%HEAD .LE. B%TAIL ) THEN
          IF ( MSG_SIZE_INT .LE. B%LBUF_INT - B%TAIL + 1 ) THEN
            IBUF = B%TAIL
          ELSE IF ( MSG_SIZE_INT .LE. B%HEAD - 1 ) THEN
            IBUF = 1
          END IF
        ELSE
          IBUF = B%TAIL
        END IF
        B%CONTENT( B%ILASTMSG + NEXT ) = IBUF
        B%ILASTMSG = IBUF
        B%TAIL = IBUF + MSG_SIZE_INT
        B%CONTENT( IBUF + NEXT ) = 0
        IPOS = IBUF + CONTENT
        IREQ = IBUF + REQ
        RETURN
        END SUBROUTINE DAGMG_MUMPS_4
        SUBROUTINE DAGMG_MUMPS_1( BUF, SIZE )
        IMPLICIT NONE
        TYPE ( DAGMG_MUMPS_COMM_BUFFER_TYPE ) :: BUF
        INTEGER SIZE
        INTEGER SIZE_INT
        SIZE_INT = ( SIZE + SIZEofINT - 1 ) / SIZEofINT
        SIZE_INT = SIZE_INT + OVHSIZE
        BUF%TAIL = BUF%ILASTMSG + SIZE_INT
        RETURN
        END SUBROUTINE DAGMG_MUMPS_1
      SUBROUTINE DAGMG_MUMPS_68(                                             &
     &             INODE, NBPROCFILS, NLIG, ILIG, NCOL, ICOL,           &
     &             NASS, NSLAVES, LIST_SLAVES,                          &
     &             DEST, NFRONT, COMM, IERR )
      IMPLICIT NONE
        INTEGER COMM, IERR, NFRONT
        INTEGER INODE
        INTEGER NLIG, NCOL, NASS, NSLAVES
        INTEGER NBPROCFILS, DEST
        INTEGER ILIG( NLIG )
        INTEGER ICOL( NCOL )
        INTEGER LIST_SLAVES( NSLAVES )
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER SIZE, POSITION, IPOS, IREQ
        INTEGER IONE
        INTEGER DEST2(1)
        PARAMETER ( IONE=1 )
        DEST2(1) = DEST
        IERR = 0
        SIZE = ( 6 + NLIG + NCOL + NSLAVES + 1 ) * SIZEofINT
        IF (SIZE.GT.SIZE_RBUF_BYTES ) THEN
         IERR = -2
         RETURN
        END IF
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE, IERR,                  &
     &                 IONE , DEST2                                     &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        POSITION = IPOS
        BUF_CB%CONTENT( POSITION ) = INODE
        POSITION = POSITION + 1
        BUF_CB%CONTENT( POSITION ) = NBPROCFILS
        POSITION = POSITION + 1
        BUF_CB%CONTENT( POSITION ) = NLIG
        POSITION = POSITION + 1
        BUF_CB%CONTENT( POSITION ) = NCOL
        POSITION = POSITION + 1
        BUF_CB%CONTENT( POSITION ) = NASS
        POSITION = POSITION + 1
        BUF_CB%CONTENT( POSITION ) = NFRONT
        POSITION = POSITION + 1
        BUF_CB%CONTENT( POSITION ) = NSLAVES
        POSITION = POSITION + 1
        IF (NSLAVES.GT.0) THEN
         BUF_CB%CONTENT( POSITION: POSITION + NSLAVES - 1 ) =           &
     &   LIST_SLAVES( 1: NSLAVES )
         POSITION = POSITION + NSLAVES
        ENDIF
        BUF_CB%CONTENT( POSITION:POSITION + NLIG - 1 ) = ILIG
        POSITION = POSITION + NLIG
        BUF_CB%CONTENT( POSITION:POSITION + NCOL - 1 ) = ICOL
        POSITION = POSITION + NCOL
        POSITION = POSITION - IPOS
        IF ( POSITION * SIZEofINT .NE. SIZE ) THEN
          WRITE(*,*) 'Error in DMUMPS_68 :',                            &
     &               ' wrong estimated size'
          CALL AGMG_MUMPS_ABORT()
        END IF
        CALL AGMG_NOCALL('MPI_ISEND')
        RETURN
        END SUBROUTINE DAGMG_MUMPS_68
        SUBROUTINE DAGMG_MUMPS_70( NBROWS_ALREADY_SENT,                      &
     &  IPERE, ISON, NROW,                                              &
     &  IROW, NCOL, ICOL, VAL, LDA, NELIM, TYPE_SON,                    &
     &  NSLAVES, SLAVES, DEST, COMM, IERR,                              &
     &  SLAVEF, KEEP,KEEP8, INIV2, TAB_POS_IN_PERE )
        IMPLICIT NONE
        INTEGER NBROWS_ALREADY_SENT
        INTEGER LDA, NELIM, TYPE_SON
        INTEGER IPERE, ISON, NROW, NCOL, NSLAVES
        INTEGER IROW( NROW )
        INTEGER ICOL( NCOL )
        INTEGER SLAVES( NSLAVES )
        DOUBLE PRECISION VAL(LDA, *)
        INTEGER IPOS, IREQ, DEST, COMM, IERR
        INTEGER SLAVEF, KEEP(500), INIV2
        INTEGER*8 KEEP8(150)
        INTEGER TAB_POS_IN_PERE(SLAVEF+2,max(1,KEEP(56)))
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER SIZE1, SIZE2, SIZE3, SIZE_PACK, POSITION, I
        INTEGER NBROWS_PACKET, NCOL_SEND
        INTEGER SIZE_AV
        LOGICAL RECV_BUF_SMALLER_THAN_SEND
        INTEGER IONE
        INTEGER DEST2(1)
        PARAMETER ( IONE=1 )
        DEST2(1) = DEST
        IERR = 0
        IF ( NELIM .NE. NROW ) THEN
          WRITE(*,*) 'Error in TRY_SEND_MAITRE2:',NELIM, NROW
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF (NBROWS_ALREADY_SENT .EQ. 0) THEN
          CALL AGMG_NOCALL('MPI_PACK_SIZE')
          IF ( ( KEEP(48).NE. 0 ).AND.(TYPE_SON .eq. 2 )) THEN
          CALL AGMG_NOCALL('MPI_PACK_SIZE')
          ELSE
            SIZE3 = 0
          ENDIF
          SIZE1=SIZE1+SIZE3
        ELSE
          CALL AGMG_NOCALL('MPI_PACK_SIZE')
        ENDIF
        IF ( KEEP(50).ne.0  .AND. TYPE_SON .eq. 2 ) THEN
          NCOL_SEND = NROW
        ELSE
          NCOL_SEND = NCOL
        ENDIF
        CALL DAGMG_MUMPS_79( BUF_CB, SIZE_AV )
        IF (SIZE_AV .LT. SIZE_RBUF_BYTES) THEN
          RECV_BUF_SMALLER_THAN_SEND = .FALSE.
        ELSE
          RECV_BUF_SMALLER_THAN_SEND = .TRUE.
        ENDIF
        SIZE_AV = min(SIZE_AV, SIZE_RBUF_BYTES)
        IF (NROW .GT. 0 ) THEN
         NBROWS_PACKET = (SIZE_AV - SIZE1) / NCOL_SEND / SIZEofREAL
         NBROWS_PACKET = min(NBROWS_PACKET, NROW - NBROWS_ALREADY_SENT)
         NBROWS_PACKET = max(NBROWS_PACKET, 0)
        ELSE
          NBROWS_PACKET =0
        ENDIF
        IF (NBROWS_PACKET .EQ. 0 .AND. NROW .NE. 0) THEN
          IF (RECV_BUF_SMALLER_THAN_SEND) THEN
              IERR=-3
              GOTO 100
          ELSE
              IERR=-1
              GOTO 100
          ENDIF
        ENDIF
   10   CONTINUE
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE_PACK = SIZE1 + SIZE2
        IF (SIZE_PACK .GT. SIZE_AV) THEN
          NBROWS_PACKET = NBROWS_PACKET - 1
          IF ( NBROWS_PACKET .GT. 0 ) THEN
            GOTO 10
          ELSE
            IF (RECV_BUF_SMALLER_THAN_SEND) THEN
                IERR = -3
                GOTO 100
            ELSE
                IERR = -1
                GOTO 100
            ENDIF
          ENDIF
        ENDIF
       IF (NBROWS_PACKET + NBROWS_ALREADY_SENT.NE.NROW .AND.            &
     &   SIZE_PACK - SIZE1  .LT. ( SIZE_RBUF_BYTES - SIZE1 ) / 2        &
     &  .AND.                                                           &
     &   .NOT. RECV_BUF_SMALLER_THAN_SEND)                              &
     &   THEN
           IERR = -1
           GOTO 100
       ENDIF
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE_PACK, IERR,             &
     &                 IONE , DEST2                                     &
     &               )
        IF ( IERR .LT. 0 ) THEN
          GOTO 100
        ENDIF
        IF (SIZE_PACK .GT. SIZE_RBUF_BYTES) THEN
          IERR=-3
             GOTO 100
        ENDIF
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IF (NBROWS_ALREADY_SENT .EQ. 0) THEN
          IF (NSLAVES.GT.0) THEN
            CALL AGMG_NOCALL('MPI_PACK')
          ENDIF
          CALL AGMG_NOCALL('MPI_PACK')
          CALL AGMG_NOCALL('MPI_PACK')
          IF ( ( KEEP(48).NE. 0 ).AND.(TYPE_SON .eq. 2 ) ) THEN
            CALL AGMG_NOCALL('MPI_PACK')
          ENDIF
        ENDIF
        IF (NBROWS_PACKET.GE.1) THEN
          DO I=NBROWS_ALREADY_SENT+1,                                   &
     &                   NBROWS_ALREADY_SENT+NBROWS_PACKET
            CALL AGMG_NOCALL('MPI_PACK')
          ENDDO
        ENDIF
        CALL AGMG_NOCALL('MPI_ISEND')
        IF ( SIZE_PACK .LT. POSITION ) THEN
          write(*,*) 'Try_send_maitre2, SIZE,POSITION=',                &
     &                SIZE_PACK,POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE_PACK .NE. POSITION )                                  &
     &    CALL DAGMG_MUMPS_1( BUF_CB, POSITION )
        NBROWS_ALREADY_SENT = NBROWS_ALREADY_SENT + NBROWS_PACKET
        IF ( NBROWS_ALREADY_SENT .NE. NROW ) THEN
          IERR = -1
        ENDIF
  100   CONTINUE
        RETURN
        END SUBROUTINE DAGMG_MUMPS_70
        SUBROUTINE DAGMG_MUMPS_67(NBROWS_ALREADY_SENT,                       &
     &  DESC_IN_LU,                                                     &
     &  IPERE, NFRONT_PERE, NASS_PERE, NFS4FATHER,                      &
     &  NSLAVES_PERE,                                                   &
     &  ISON, NBROW, LMAP, MAPROW, PERM, IW_CBSON, A_CBSON,             &
     &  ISLAVE, PDEST, PDEST_MASTER, COMM, IERR,                        &
     & KEEP,KEEP8, STEP, N, SLAVEF,                                     &
     & ISTEP_TO_INIV2, TAB_POS_IN_PERE,                                 &
     & COMPRESSCB )
        IMPLICIT NONE
        INTEGER NBROWS_ALREADY_SENT
        INTEGER IPERE, ISON, NBROW
        INTEGER PDEST, ISLAVE, COMM, IERR
        INTEGER PDEST_MASTER, NASS_PERE, NSLAVES_PERE,                  &
     &       NFRONT_PERE, LMAP
        INTEGER MAPROW( LMAP ), PERM( max(1, NBROW ))
        INTEGER IW_CBSON( * )
        DOUBLE PRECISION A_CBSON( * )
        LOGICAL DESC_IN_LU, COMPRESSCB
       INTEGER   KEEP(500), N , SLAVEF
       INTEGER*8 KEEP8(150)
       INTEGER   STEP(N),                                               &
     &          ISTEP_TO_INIV2(KEEP(71)),                               &
     &          TAB_POS_IN_PERE(SLAVEF+2,max(1,KEEP(56)))
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
      INTEGER NFS4FATHER,SIZE3,PS1,NCA,OFFSET,LROW1
      INTEGER(8) :: ASIZE
      LOGICAL COMPUTE_MAX
      DOUBLE PRECISION TMP
      INTEGER NBROWS_PACKET
      INTEGER MAX_ROW_LENGTH
      INTEGER LROW, NELIM
      INTEGER(8) :: SIZFR, ITMP8
      INTEGER NPIV, NFRONT, HS
      INTEGER SIZE_PACK, SIZE1, SIZE2, POSITION,I
      INTEGER SIZE_INTEGERS, B, SIZE_REALS, TMPSIZE, ONEorTWO, SIZE_AV
      INTEGER NBINT, L
      INTEGER(8) :: APOS, SHIFTCB_SON, LDA_SON8
      INTEGER IPOS_IN_SLAVE
      INTEGER STATE_SON
      INTEGER INDICE_PERE, NROW, IPOS, IREQ, NOSLA
      INTEGER IONE, J, THIS_ROW_LENGTH
      INTEGER SIZE_DESC_BANDE, DESC_BANDE_BYTES
      LOGICAL RECV_BUF_SMALLER_THAN_SEND
      LOGICAL NOT_ENOUGH_SPACE
      INTEGER PDEST2(1)
      PARAMETER ( IONE=1 )
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER XXI, XXR, XXS, XXN, XXP
      PARAMETER(XXI=0,XXR=1,XXS=3,XXN=4,XXP=5)
      INTEGER XXNDIAG2W
      PARAMETER(XXNDIAG2W=6)
      INTEGER XSIZE_IC, XSIZE_OOC_SYM, XSIZE_OOC_UNSYM
      INTEGER XSIZE_OOC_NOPANEL
      PARAMETER (XSIZE_IC=6,XSIZE_OOC_SYM=7,XSIZE_OOC_UNSYM=7,          &
     &           XSIZE_OOC_NOPANEL=6)
      INTEGER IXSZ
      PARAMETER(IXSZ= 222)
      INTEGER S_CB1COMP
      PARAMETER (S_CB1COMP=314)
      INTEGER S_ACTIVE, S_ALL, S_NOLCBCONTIG,                           &
     &        S_NOLCBNOCONTIG, S_NOLCLEANED,                            &
     &        S_NOLCBNOCONTIG38, S_NOLCBCONTIG38,                       &
     &        S_NOLCLEANED38, C_FINI
      PARAMETER(S_ACTIVE=400, S_ALL=401, S_NOLCBCONTIG=402,             &
     &          S_NOLCBNOCONTIG=403, S_NOLCLEANED=404,                  &
     &          S_NOLCBNOCONTIG38=405, S_NOLCBCONTIG38=406,             &
     &          S_NOLCLEANED38=407,C_FINI=1)
      INTEGER S_FREE, S_NOTFREE
      PARAMETER(S_FREE=54321,S_NOTFREE=-123456)
      INTEGER TOP_OF_STACK
      PARAMETER(TOP_OF_STACK=-999999)
      INTEGER XTRA_SLAVES_SYM, XTRA_SLAVES_UNSYM
      PARAMETER(XTRA_SLAVES_SYM=3, XTRA_SLAVES_UNSYM=1)
         INTEGER S_ROOT2SON_CALLED, S_REC_CONTSTATIC,                   &
     &  S_ROOTBAND_INIT
         PARAMETER(S_ROOT2SON_CALLED=-341,S_REC_CONTSTATIC=1,           &
     &             S_ROOTBAND_INIT=0)
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO = 0.0D0)
      IF (NBROWS_ALREADY_SENT == 0) THEN
        IF (KEEP(219).NE.0) THEN
          CALL DAGMG_MUMPS_617(NFS4FATHER,IERR)
          IF (IERR .NE. 0) THEN
            IERR         = -4
            RETURN
          ENDIF
        ENDIF
      ENDIF
      PDEST2(1) = PDEST
      IERR   = 0
      LROW   = IW_CBSON( 1 + KEEP(IXSZ))
      NELIM  = IW_CBSON( 2 + KEEP(IXSZ))
      NPIV   = IW_CBSON( 4 + KEEP(IXSZ))
      IF ( NPIV .LT. 0 ) THEN
          NPIV = 0
      END IF
      NROW   = IW_CBSON( 3 + KEEP(IXSZ))
      NFRONT = LROW + NPIV
      HS     = 6 + IW_CBSON( 6 + KEEP(IXSZ)) + KEEP(IXSZ)
      CALL AGMG_MUMPS_729( SIZFR, IW_CBSON( 1 + XXR ) )
      STATE_SON = IW_CBSON(1+XXS)
      IF (STATE_SON .EQ. S_NOLCBCONTIG) THEN
               LDA_SON8    = int(LROW,8)
               SHIFTCB_SON = int(NPIV,8)*int(NROW,8)
      ELSE IF (STATE_SON .EQ. S_NOLCLEANED) THEN
               LDA_SON8    = int(LROW,8)
               SHIFTCB_SON = 0_8
      ELSE
               LDA_SON8     = int(NFRONT,8)
               SHIFTCB_SON = int(NPIV,8)
      ENDIF
      CALL DAGMG_MUMPS_79( BUF_CB, SIZE_AV )
      IF (PDEST .EQ. PDEST_MASTER) THEN
        SIZE_DESC_BANDE=0
      ELSE
        SIZE_DESC_BANDE=(7+SLAVEF+KEEP(127)*2)
        SIZE_DESC_BANDE=SIZE_DESC_BANDE+int(dble(KEEP(12))*             &
     &                  dble(SIZE_DESC_BANDE)/100.0D0)
        SIZE_DESC_BANDE=max(SIZE_DESC_BANDE,                            &
     &     7+NSLAVES_PERE+NFRONT_PERE+NFRONT_PERE-NASS_PERE)
      ENDIF
      DESC_BANDE_BYTES=SIZE_DESC_BANDE*SIZEofINT
      IF ( SIZE_AV .LT. SIZE_RBUF_BYTES-DESC_BANDE_BYTES ) THEN
        RECV_BUF_SMALLER_THAN_SEND = .FALSE.
      ELSE
        RECV_BUF_SMALLER_THAN_SEND = .TRUE.
        SIZE_AV = SIZE_RBUF_BYTES-DESC_BANDE_BYTES
      ENDIF
      SIZE1=0
      IF (NBROWS_ALREADY_SENT==0) THEN
          IF (KEEP(219).NE.0) THEN
            COMPUTE_MAX = KEEP(50).EQ.2 .AND.                           &
     &           PDEST.EQ.PDEST_MASTER
            IF(COMPUTE_MAX) THEN
               CALL AGMG_NOCALL('MPI_PACK_SIZE')
               IF(NFS4FATHER .GT. 0) THEN
                CALL AGMG_NOCALL('MPI_PACK_SIZE')
               ENDIF
               SIZE1 = SIZE1+PS1
            ENDIF
          ENDIF
      ENDIF
      IF (KEEP(50) .EQ. 0) THEN
        ONEorTWO = 1
      ELSE
        ONEorTWO = 2
      ENDIF
      IF (PDEST .EQ.PDEST_MASTER) THEN
        L = 0
      ELSE IF (KEEP(50) .EQ. 0) THEN
        L = LROW
      ELSE
        L = LROW + PERM(1) - LMAP + NBROWS_ALREADY_SENT - 1
        ONEorTWO=ONEorTWO+1
      ENDIF
      NBINT = 6 + L
      CALL AGMG_NOCALL('MPI_PACK_SIZE')
      SIZE1 = SIZE1 + TMPSIZE
      SIZE_AV = SIZE_AV - SIZE1
      NOT_ENOUGH_SPACE=.FALSE.
      IF (SIZE_AV .LT.0 ) THEN
        NBROWS_PACKET = 0
        NOT_ENOUGH_SPACE=.TRUE.
      ELSE
        IF ( KEEP(50) .EQ. 0 ) THEN
          NBROWS_PACKET =                                               &
     &       SIZE_AV / ( ONEorTWO*SIZEofINT+LROW*SIZEofREAL)
        ELSE
          B = 2 * ONEorTWO +                                            &
     &      ( 1 + 2 *  LROW + 2 * PERM(1) + 2 * NBROWS_ALREADY_SENT )   &
     &      * SIZEofREAL / SIZEofINT
          NBROWS_PACKET=int((dble(-B)+sqrt((dble(B)*dble(B))+           &
     &        dble(4)*dble(2*SIZE_AV)/dble(SIZEofINT) *                 &
     &        dble(SIZEofREAL/SIZEofINT)))*                             &
     &        dble(SIZEofINT) / dble(2) / dble(SIZEofREAL))
        ENDIF
      ENDIF
   10 CONTINUE
      NBROWS_PACKET = max( 0,                                           &
     &           min( NBROWS_PACKET, NBROW - NBROWS_ALREADY_SENT))
      NOT_ENOUGH_SPACE = NOT_ENOUGH_SPACE .OR.                          &
     &                   (NBROWS_PACKET .EQ.0.AND. NBROW.NE.0)
      IF (NOT_ENOUGH_SPACE) THEN
        IF (RECV_BUF_SMALLER_THAN_SEND) THEN
          IERR = -3
          GOTO 100
        ELSE
          IERR = -1
          GOTO 100
        ENDIF
      ENDIF
      IF (KEEP(50).EQ.0) THEN
        MAX_ROW_LENGTH = -99999
        SIZE_REALS = NBROWS_PACKET * LROW
      ELSE
        SIZE_REALS = (  LROW + PERM(1) + NBROWS_ALREADY_SENT ) *        &
     &  NBROWS_PACKET + ( NBROWS_PACKET * ( NBROWS_PACKET + 1) ) / 2
        MAX_ROW_LENGTH = LROW+PERM(1)-LMAP+NBROWS_ALREADY_SENT          &
     &                 + NBROWS_PACKET-1
      ENDIF
      SIZE_INTEGERS = ONEorTWO* NBROWS_PACKET
      CALL AGMG_NOCALL('MPI_PACK_SIZE')
      CALL AGMG_NOCALL('MPI_PACK_SIZE')
      IF (SIZE2 + SIZE3 .GT. SIZE_AV ) THEN
         NBROWS_PACKET = NBROWS_PACKET -1
         IF (NBROWS_PACKET .GT. 0 ) THEN
           GOTO 10
         ELSE
           IF (RECV_BUF_SMALLER_THAN_SEND) THEN
             IERR = -3
             GOTO 100
           ELSE
             IERR = -1
             GOTO 100
           ENDIF
         ENDIF
      ENDIF
        SIZE_PACK = SIZE1 + SIZE2 + SIZE3
        IF (NBROWS_PACKET + NBROWS_ALREADY_SENT.NE.NBROW .AND.          &
     &    SIZE_PACK  .LT. SIZE_RBUF_BYTES / 4 .AND.                     &
     &    .NOT. RECV_BUF_SMALLER_THAN_SEND)                             &
     &    THEN
            IERR = -1
            GOTO 100
        ENDIF
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE_PACK, IERR,             &
     &                 IONE , PDEST2                                    &
     &               )
        IF (IERR .EQ. -1 .OR. IERR.EQ. -2) THEN
          NBROWS_PACKET = NBROWS_PACKET - 1
          IF (NBROWS_PACKET > 0 ) GOTO 10
        ENDIF
        IF ( IERR .LT. 0 ) GOTO 100
          IF (SIZE_PACK.GT.SIZE_RBUF_BYTES ) THEN
             IERR = -3
             GOTO 100
          ENDIF
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IF (KEEP(50)==0) THEN
        CALL AGMG_NOCALL('MPI_PACK')
        ELSE
        CALL AGMG_NOCALL('MPI_PACK')
        ENDIF
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IF ( PDEST .NE. PDEST_MASTER ) THEN
          IF (KEEP(50)==0) THEN
          CALL AGMG_NOCALL('MPI_PACK')
          ELSE
           IF (MAX_ROW_LENGTH > 0) THEN
           CALL AGMG_NOCALL('MPI_PACK')
           ENDIF
          ENDIF
        END IF
        DO J=NBROWS_ALREADY_SENT+1,NBROWS_ALREADY_SENT+NBROWS_PACKET
           I = PERM(J)
           INDICE_PERE=MAPROW(I)
           CALL AGMG_MUMPS_47(                                               &
     &          KEEP,KEEP8, IPERE, STEP, N, SLAVEF,                     &
     &          ISTEP_TO_INIV2, TAB_POS_IN_PERE,                        &
     &          NASS_PERE,                                              &
     &          NFRONT_PERE - NASS_PERE,                                &
     &          NSLAVES_PERE,                                           &
     &          INDICE_PERE,                                            &
     &          NOSLA,                                                  &
     &          IPOS_IN_SLAVE )
           INDICE_PERE = IPOS_IN_SLAVE
           CALL AGMG_NOCALL('MPI_PACK')
        ENDDO
        DO J=NBROWS_ALREADY_SENT+1,NBROWS_ALREADY_SENT+NBROWS_PACKET
           I = PERM(J)
           INDICE_PERE=MAPROW(I)
           CALL AGMG_MUMPS_47(                                               &
     &          KEEP,KEEP8, IPERE, STEP, N, SLAVEF,                     &
     &          ISTEP_TO_INIV2, TAB_POS_IN_PERE,                        &
     &          NASS_PERE,                                              &
     &          NFRONT_PERE - NASS_PERE,                                &
     &          NSLAVES_PERE,                                           &
     &          INDICE_PERE,                                            &
     &          NOSLA,                                                  &
     &          IPOS_IN_SLAVE )
          IF (KEEP(50).ne.0) THEN
            THIS_ROW_LENGTH = LROW + I - LMAP
            CALL AGMG_NOCALL('MPI_PACK')
         ELSE
            THIS_ROW_LENGTH = LROW
         ENDIF
         IF (DESC_IN_LU) THEN
            IF ( COMPRESSCB ) THEN
             IF (NELIM.EQ.0) THEN
               ITMP8 = int(I,8)
             ELSE
               ITMP8 = int(NELIM+I,8)
             ENDIF
             APOS = ITMP8 * (ITMP8-1_8) / 2_8 + 1_8
            ELSE
             APOS = int(I+NELIM-1, 8) * int(LROW,8) + 1_8
            ENDIF
         ELSE
            IF ( COMPRESSCB ) THEN
             IF ( LROW .EQ. NROW )  THEN
               ITMP8 = int(I,8)
               APOS  = ITMP8 * (ITMP8-1_8)/2_8 + 1_8
             ELSE
               ITMP8 = int(I + LROW - NROW,8)
               APOS  = ITMP8 * (ITMP8-1_8)/2_8 + 1_8 -                  &
     &                 int(LROW - NROW, 8) * int(LROW-NROW+1,8) / 2_8
             ENDIF
            ELSE
             APOS = int( I - 1, 8 ) * LDA_SON8 + SHIFTCB_SON + 1_8
            ENDIF
         ENDIF
         CALL AGMG_NOCALL('MPI_PACK')
        ENDDO
      IF (NBROWS_ALREADY_SENT == 0) THEN
      IF (KEEP(219).NE.0) THEN
        IF(COMPUTE_MAX) THEN
           CALL AGMG_NOCALL('MPI_PACK')
           IF(NFS4FATHER .GT. 0) THEN
              BUF_MAX_ARRAY(1:NFS4FATHER) = ZERO
              IF(MAPROW(NROW) .GT. NASS_PERE) THEN
                 DO PS1=1,NROW
                    IF(MAPROW(PS1).GT.NASS_PERE) EXIT
                 ENDDO
                 IF (DESC_IN_LU) THEN
                   IF (COMPRESSCB) THEN
                    APOS = int(NELIM+PS1,8) * int(NELIM+PS1-1,8) /      &
     &                     2_8 + 1_8
                    NCA  = -44444
                    ASIZE  = int(NROW,8) * int(NROW+1,8)/2_8 -          &
     &                       int(NELIM+PS1,8) * int(NELIM+PS1-1,8)/2_8
                    LROW1  = PS1 + NELIM
                   ELSE
                    APOS = int(PS1+NELIM-1,8) * int(LROW,8) + 1_8
                    NCA = LROW
                    ASIZE = int(NCA,8) * int(NROW-PS1+1,8)
                    LROW1 = LROW
                   ENDIF
                 ELSE
                    IF (COMPRESSCB) THEN
                      IF (NPIV.NE.0) THEN
         WRITE(*,*) "Error in PARPIV/DAGMG_MUMPS_67"
                        CALL AGMG_MUMPS_ABORT()
                      ENDIF
                      LROW1=LROW-NROW+PS1
                      ITMP8 = int(PS1 + LROW - NROW,8)
                      APOS = ITMP8 * (ITMP8 - 1_8) / 2_8 + 1_8 -        &
     &                       int(LROW-NROW,8)*int(LROW-NROW+1,8)/2_8
                      ASIZE = int(LROW,8)*int(LROW+1,8)/2_8 -           &
     &                       ITMP8*(ITMP8-1_8)/2_8
                      NCA   = -555555
                    ELSE
                      APOS = int(PS1-1,8) * LDA_SON8 + 1_8 + SHIFTCB_SON
                      NCA = int(LDA_SON8)
                      ASIZE = SIZFR - (SHIFTCB_SON -                    &
     &                                 int(PS1-1,8) * LDA_SON8)
                      LROW1=-666666
                    ENDIF
                 ENDIF
                 IF ( NROW-PS1+1 .NE. 0 ) THEN
                   CALL DAGMG_MUMPS_618(                                     &
     &                A_CBSON(APOS),ASIZE,NCA,NROW-PS1+1,               &
     &                BUF_MAX_ARRAY,NFS4FATHER,COMPRESSCB,LROW1)
                 ENDIF
              ENDIF
              CALL AGMG_NOCALL('MPI_PACK')
           ENDIF
        ENDIF
      ENDIF
      ENDIF
        CALL AGMG_NOCALL('MPI_ISEND')
        IF ( SIZE_PACK.LT. POSITION ) THEN
          WRITE(*,*) ' contniv2: SIZE, POSITION =',SIZE_PACK, POSITION
          WRITE(*,*) ' NBROW, LROW = ', NBROW, LROW
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE_PACK .NE. POSITION )                                  &
     &  CALL DAGMG_MUMPS_1( BUF_CB, POSITION )
        NBROWS_ALREADY_SENT=NBROWS_ALREADY_SENT + NBROWS_PACKET
        IF (NBROWS_ALREADY_SENT .NE. NBROW ) THEN
           IERR = -1
        ENDIF
  100   CONTINUE
        RETURN
        END SUBROUTINE DAGMG_MUMPS_67
        SUBROUTINE DAGMG_MUMPS_71(                                           &
     &                INODE, NFRONT, NASS1, NFS4FATHER,                 &
     &                ISON, MYID, NSLAVES, SLAVES_PERE,                 &
     &                TROW, NCBSON,                                     &
     &                COMM, IERR,                                       &
     &                DEST, NDEST, SLAVEF,                              &
     &                KEEP,KEEP8, STEP, N,                              &
     &                ISTEP_TO_INIV2, TAB_POS_IN_PERE                   &
     &                                  )
        IMPLICIT NONE
      INTEGER INODE, NFRONT, NASS1, NCBSON, NSLAVES,                    &
     &          NDEST
      INTEGER SLAVEF, MYID, ISON
      INTEGER TROW( NCBSON )
      INTEGER DEST( NDEST )
      INTEGER SLAVES_PERE( NSLAVES )
      INTEGER COMM, IERR
      INTEGER KEEP(500), N
      INTEGER*8 KEEP8(150)
      INTEGER STEP(N),                                                  &
     &        ISTEP_TO_INIV2(KEEP(71)),                                 &
     &        TAB_POS_IN_PERE(SLAVEF+2,max(1,KEEP(56)))
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER SIZE_AV, IDEST, NSEND, SIZE, NFS4FATHER
        INTEGER TROW_SIZE, POSITION, INDX, INIV2
        INTEGER IPOS, IREQ
        INTEGER IONE
        PARAMETER ( IONE=1 )
        INTEGER NASS_SON
        NASS_SON = -99998
        IERR = 0
        IF ( NDEST .eq. 1 ) THEN
          IF ( DEST(1).EQ.MYID )  GOTO 500
          SIZE = SIZEofINT * ( 7 + NSLAVES + NCBSON )
          IF (( NSLAVES.GT.0 ) .AND. (KEEP(48).NE.0) ) THEN
             SIZE = SIZE + SIZEofINT * ( NSLAVES + 1 )
          ENDIF
          CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE, IERR,                &
     &                 IONE, DEST                                       &
     &                 )
          IF (IERR .LT. 0 ) THEN
             RETURN
          ENDIF
          IF (SIZE.GT.SIZE_RBUF_BYTES ) THEN
             IERR = -3
             RETURN
          END IF
              POSITION = IPOS
              BUF_CB%CONTENT( POSITION ) = INODE
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = ISON
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = NSLAVES
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = NFRONT
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = NASS1
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = NCBSON
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = NFS4FATHER
              POSITION = POSITION + 1
              IF (( NSLAVES.GT.0 ) .AND. (KEEP(48).NE.0) ) THEN
                INIV2 = ISTEP_TO_INIV2 ( STEP(INODE) )
                BUF_CB%CONTENT( POSITION: POSITION + NSLAVES )          &
     &          =  TAB_POS_IN_PERE(1:NSLAVES+1,INIV2)
                POSITION = POSITION + NSLAVES + 1
              ENDIF
              IF ( NSLAVES .NE. 0 ) THEN
                BUF_CB%CONTENT( POSITION: POSITION + NSLAVES - 1 )      &
     &          = SLAVES_PERE( 1: NSLAVES )
                POSITION = POSITION + NSLAVES
              END IF
              BUF_CB%CONTENT( POSITION:POSITION+NCBSON-1 ) =            &
     &        TROW( 1: NCBSON )
              POSITION = POSITION + NCBSON
              POSITION = POSITION - IPOS
              IF ( POSITION * SIZEofINT .NE. SIZE ) THEN
                WRITE(*,*) 'Error in DMUMPS_71 :',                      &
     &                     ' wrong estimated size'
                CALL AGMG_MUMPS_ABORT()
              END IF
              CALL AGMG_NOCALL('MPI_ISEND')
        ELSE
          NSEND = 0
          DO IDEST = 1, NDEST
            IF ( DEST( IDEST ) .ne. MYID ) NSEND = NSEND + 1
          END DO
          SIZE = SIZEofINT *                                            &
     &         ( ( OVHSIZE + 7 + NSLAVES )* NSEND + NCBSON )
          IF (( NSLAVES.GT.0 ) .AND. (KEEP(48).NE.0) ) THEN
           SIZE = SIZE + SIZEofINT * NSEND*( NSLAVES + 1 )
          ENDIF
          CALL DAGMG_MUMPS_79( BUF_CB, SIZE_AV )
          IF ( SIZE_AV .LT. SIZE ) THEN
            IERR = -1
            RETURN
          END IF
          DO IDEST= 1, NDEST
            CALL AGMG_MUMPS_49(                                              &
     &                KEEP,KEEP8, ISON, STEP, N, SLAVEF,                &
     &                ISTEP_TO_INIV2, TAB_POS_IN_PERE,                  &
     &                IDEST, NCBSON,                                    &
     &                NDEST,                                            &
     &                TROW_SIZE, INDX  )
            SIZE = SIZEofINT * ( NSLAVES + TROW_SIZE + 7 )
            IF (( NSLAVES.GT.0 ) .AND. (KEEP(48).NE.0) ) THEN
             SIZE = SIZE + SIZEofINT * ( NSLAVES + 1 )
            ENDIF
            IF ( MYID .NE. DEST( IDEST ) ) THEN
              CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE, IERR,            &
     &                       IONE, DEST(IDEST)                          &
     &                     )
              IF ( IERR .LT. 0 )  THEN
                 WRITE(*,*) 'Problem in DMUMPS_4: IERR<0'
                 CALL AGMG_MUMPS_ABORT()
              END IF
              IF (SIZE.GT.SIZE_RBUF_BYTES) THEN
                 IERR = -3
                 RETURN
              ENDIF
              POSITION = IPOS
              BUF_CB%CONTENT( POSITION ) = INODE
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = ISON
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = NSLAVES
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = NFRONT
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = NASS1
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = TROW_SIZE
              POSITION = POSITION + 1
              BUF_CB%CONTENT( POSITION ) = NFS4FATHER
              POSITION = POSITION + 1
              IF (( NSLAVES.GT.0 ) .AND. (KEEP(48).NE.0) ) THEN
                INIV2 = ISTEP_TO_INIV2 ( STEP(INODE) )
                BUF_CB%CONTENT( POSITION: POSITION + NSLAVES )          &
     &          =  TAB_POS_IN_PERE(1:NSLAVES+1,INIV2)
                POSITION = POSITION + NSLAVES + 1
              ENDIF
              IF ( NSLAVES .NE. 0 ) THEN
                BUF_CB%CONTENT( POSITION: POSITION + NSLAVES - 1 )      &
     &          = SLAVES_PERE( 1: NSLAVES )
                POSITION = POSITION + NSLAVES
              END IF
              BUF_CB%CONTENT( POSITION:POSITION+TROW_SIZE-1 ) =         &
     &        TROW( INDX: INDX + TROW_SIZE - 1 )
              POSITION = POSITION + TROW_SIZE
              POSITION = POSITION - IPOS
              IF ( POSITION * SIZEofINT .NE. SIZE ) THEN
               WRITE(*,*) ' ERROR 1 in TRY_SEND_MAPLIG:',               &
     &          'Wrong estimated size'
               CALL AGMG_MUMPS_ABORT()
              END IF
              CALL AGMG_NOCALL('MPI_ISEND')
            END IF
          END DO
        END IF
  500   CONTINUE
        RETURN
        END SUBROUTINE DAGMG_MUMPS_71
        SUBROUTINE DAGMG_MUMPS_65( INODE, NFRONT,                            &
     &             NCOL, NPIV, FPERE, LASTBL, IPIV, VAL,                &
     &             PDEST, NDEST, KEEP50, NB_BLOC_FAC, COMM, IERR )
        IMPLICIT NONE
        INTEGER INODE, NCOL, NPIV, FPERE, NFRONT, NDEST
        INTEGER IPIV( NPIV )
        DOUBLE PRECISION VAL( NFRONT, * )
        INTEGER PDEST( NDEST )
        INTEGER   KEEP50, NB_BLOC_FAC, COMM, IERR
        LOGICAL LASTBL
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER POSITION, IREQ, IPOS, SIZE1, SIZE2, SIZE,               &
     &          IDEST, IPOSMSG, I
        INTEGER NPIVSENT
        INTEGER SSS, SS2
        IERR = 0
        IF ( LASTBL ) THEN
          IF ( KEEP50 .eq. 0 ) THEN
            CALL AGMG_NOCALL('MPI_PACK_SIZE')
          ELSE
            CALL AGMG_NOCALL('MPI_PACK_SIZE')
          END IF
        ELSE
          IF ( KEEP50 .eq. 0 ) THEN
          CALL AGMG_NOCALL('MPI_PACK_SIZE')
          ELSE
            CALL AGMG_NOCALL('MPI_PACK_SIZE')
          END IF
        END IF
        SIZE2 = 0
        IF (NPIV.GT.0)                                                  &
     &    CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE = SIZE1 + SIZE2
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE, IERR,                  &
     &                 NDEST , PDEST                                    &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        IF (SIZE.GT.SIZE_RBUF_BYTES) THEN
          SSS = 0
          IF ( LASTBL ) THEN
           IF ( KEEP50 .eq. 0 ) THEN
            CALL AGMG_NOCALL('MPI_PACK_SIZE')
           ELSE
            CALL AGMG_NOCALL('MPI_PACK_SIZE')
           END IF
          ELSE
           IF ( KEEP50 .eq. 0 ) THEN
            CALL AGMG_NOCALL('MPI_PACK_SIZE')
           ELSE
            CALL AGMG_NOCALL('MPI_PACK_SIZE')
           END IF
          END IF
          IF (NPIV.GT.0)                                                &
     &    CALL AGMG_NOCALL('MPI_PACK_SIZE')
          SSS = SSS + SS2
          IF (SSS.GT.SIZE_RBUF_BYTES) THEN
           IERR = -2
           RETURN
          ENDIF
        ENDIF
        BUF_CB%ILASTMSG = BUF_CB%ILASTMSG + ( NDEST - 1 ) * OVHSIZE
        IPOS = IPOS - OVHSIZE
        DO IDEST = 1, NDEST - 1
          BUF_CB%CONTENT( IPOS + ( IDEST - 1 ) * OVHSIZE ) =            &
     &    IPOS + IDEST * OVHSIZE
        END DO
        BUF_CB%CONTENT( IPOS + ( NDEST - 1 ) * OVHSIZE ) = 0
        IPOSMSG = IPOS + OVHSIZE * NDEST
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        NPIVSENT = NPIV
        IF (LASTBL) NPIVSENT = -NPIV
        CALL AGMG_NOCALL('MPI_PACK')
        IF ( LASTBL .or. KEEP50.ne.0 ) THEN
          CALL AGMG_NOCALL('MPI_PACK')
        END IF
        IF ( LASTBL .AND. KEEP50 .NE. 0 ) THEN
            CALL AGMG_NOCALL('MPI_PACK')
            CALL AGMG_NOCALL('MPI_PACK')
        END IF
        CALL AGMG_NOCALL('MPI_PACK')
        IF ( NPIV.GT.0) THEN
          CALL AGMG_NOCALL('MPI_PACK')
          DO I = 1, NPIV
            CALL AGMG_NOCALL('MPI_PACK')
          END DO
        ENDIF
        DO IDEST = 1, NDEST
        IF ( KEEP50.eq.0) THEN
        CALL AGMG_NOCALL('MPI_ISEND')
        ELSE
        CALL AGMG_NOCALL('MPI_ISEND')
        END IF
        END DO
        SIZE = SIZE - ( NDEST - 1 ) * OVHSIZE * SIZEofINT
        IF ( SIZE .LT. POSITION ) THEN
          WRITE(*,*) ' Error sending blocfacto : size < position'
          WRITE(*,*) ' Size,position=',SIZE,POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE .NE. POSITION ) CALL DAGMG_MUMPS_1( BUF_CB, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_65
        SUBROUTINE DAGMG_MUMPS_64( INODE,                                    &
     &             NPIV, FPERE, IPOSK, JPOSK, UIP21K, NCOLU,            &
     &             NDEST, PDEST, COMM, IERR )
        IMPLICIT NONE
        INTEGER INODE, NCOLU, IPOSK, JPOSK, NPIV, NDEST, FPERE
        DOUBLE PRECISION UIP21K( NPIV, NCOLU )
        INTEGER PDEST( NDEST )
        INTEGER   COMM, IERR
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER POSITION, IREQ, IPOS, SIZE1, SIZE2, SIZE,               &
     &          IDEST, IPOSMSG, SSS, SS2
        IERR = 0
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE = SIZE1 + SIZE2
        IF (SIZE.GT.SIZE_RBUF_BYTES) THEN
         CALL AGMG_NOCALL('MPI_PACK_SIZE')
         CALL AGMG_NOCALL('MPI_PACK_SIZE')
         SSS = SSS+SS2
         IF (SSS.GT.SIZE_RBUF_BYTES) THEN
           IERR = -2
           RETURN
         ENDIF
        END IF
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE, IERR,                  &
     &                 NDEST, PDEST                                     &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        BUF_CB%ILASTMSG = BUF_CB%ILASTMSG + ( NDEST - 1 ) * OVHSIZE
        IPOS = IPOS - OVHSIZE
        DO IDEST = 1, NDEST - 1
          BUF_CB%CONTENT( IPOS + ( IDEST - 1 ) * OVHSIZE ) =            &
     &    IPOS + IDEST * OVHSIZE
        END DO
        BUF_CB%CONTENT( IPOS + ( NDEST - 1 ) * OVHSIZE ) = 0
        IPOSMSG = IPOS + OVHSIZE * NDEST
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        DO IDEST = 1, NDEST
        CALL AGMG_NOCALL('MPI_ISEND')
        END DO
        SIZE = SIZE - ( NDEST - 1 ) * OVHSIZE * SIZEofINT
        IF ( SIZE .LT. POSITION ) THEN
          WRITE(*,*) ' Error sending blfac slave : size < position'
          WRITE(*,*) ' Size,position=',SIZE,POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE .NE. POSITION ) CALL DAGMG_MUMPS_1( BUF_CB, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_64
        SUBROUTINE DAGMG_MUMPS_648( N, ISON,                                 &
     &             NBCOL_SON, NBROW_SON, INDCOL_SON, INDROW_SON,        &
     &             LD_SON, VAL_SON, TAG, SUBSET_ROW, SUBSET_COL,        &
     &             NSUBSET_ROW, NSUBSET_COL,                            &
     &             NPROW, NPCOL, MBLOCK, RG2L_ROW, RG2L_COL,            &
     &             NBLOCK, PDEST, COMM, IERR ,                          &
     &             TAB, TABSIZE, TRANSP, SIZE_PACK,                     &
     &             N_ALREADY_SENT )
        IMPLICIT NONE
        INTEGER N, ISON, NBCOL_SON, NBROW_SON, NSUBSET_ROW, NSUBSET_COL
        INTEGER NPROW, NPCOL, MBLOCK, NBLOCK, LD_SON
        INTEGER PDEST, TAG, COMM, IERR
        INTEGER INDCOL_SON( NBCOL_SON ), INDROW_SON( NBROW_SON )
        INTEGER SUBSET_ROW( * ), SUBSET_COL( * )
        INTEGER RG2L_ROW( N ), RG2L_COL( N )
        INTEGER(8), INTENT(IN) :: TABSIZE
        INTEGER SIZE_PACK
        DOUBLE PRECISION VAL_SON( LD_SON, * ), TAB(*)
        LOGICAL TRANSP
        INTEGER N_ALREADY_SENT
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
        INTEGER NSUBSET_OTHER, NSUBSET_TOT
        INTEGER SIZE1, SIZE2, SIZE_AV, POSITION
        INTEGER IREQ, IPOS, ITAB
        INTEGER ISUB, JSUB, I, J
        INTEGER ILOC_ROOT, JLOC_ROOT
        INTEGER IPOS_ROOT, JPOS_ROOT
        INTEGER IONE
        LOGICAL RECV_BUF_SMALLER_THAN_SEND
        INTEGER PDEST2(1)
        PARAMETER ( IONE=1 )
        INTEGER N_PACKET
        PDEST2(1) = PDEST
        IERR = 0
        IF ( NSUBSET_ROW * NSUBSET_COL .NE. 0 ) THEN
          CALL DAGMG_MUMPS_79( BUF_CB, SIZE_AV )
          IF (SIZE_AV .LT. SIZE_RBUF_BYTES) THEN
            RECV_BUF_SMALLER_THAN_SEND = .FALSE.
          ELSE
            RECV_BUF_SMALLER_THAN_SEND = .TRUE.
            SIZE_AV = SIZE_RBUF_BYTES
          ENDIF
          SIZE_AV = min(SIZE_AV, SIZE_RBUF_BYTES)
          CALL AGMG_NOCALL('MPI_PACK_SIZE')
          IF (.NOT.TRANSP) THEN
            NSUBSET_TOT  = NSUBSET_ROW
            NSUBSET_OTHER=NSUBSET_COL
          ELSE
            NSUBSET_TOT  = NSUBSET_COL
            NSUBSET_OTHER=NSUBSET_ROW
          ENDIF
          N_PACKET =                                                    &
     &    (SIZE_AV - SIZE1 ) / ( SIZEofINT + NSUBSET_OTHER * SIZEofREAL)
   10     CONTINUE
          N_PACKET = min( N_PACKET,                                     &
     &                    NSUBSET_TOT-N_ALREADY_SENT )
          IF (N_PACKET .LE. 0) THEN
            IF (RECV_BUF_SMALLER_THAN_SEND) THEN
              IERR=-3
              GOTO 100
            ELSE
              IERR = -1
              GOTO 100
            ENDIF
          ENDIF
          CALL AGMG_NOCALL('MPI_PACK_SIZE')
          CALL AGMG_NOCALL('MPI_PACK_SIZE')
          SIZE_PACK = SIZE1 + SIZE2
          IF (SIZE_PACK .GT. SIZE_AV) THEN
            N_PACKET = N_PACKET - 1
            IF ( N_PACKET > 0 ) THEN
              GOTO 10
            ELSE
              IF (RECV_BUF_SMALLER_THAN_SEND) THEN
                IERR = -3
                GOTO 100
              ELSE
                IERR = -1
                GOTO 100
              ENDIF
            ENDIF
          ENDIF
          IF (N_PACKET + N_ALREADY_SENT .NE. NSUBSET_TOT .AND.          &
     &      SIZE_PACK .LT. SIZE_RBUF_BYTES / 4                          &
     &      .AND. .NOT. RECV_BUF_SMALLER_THAN_SEND)                     &
     &      THEN
            IERR = -1
            GOTO 100
          ENDIF
        ELSE
          N_PACKET = 0
          CALL AGMG_NOCALL('MPI_PACK_SIZE')
        END IF
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE_PACK, IERR,             &
     &                 IONE, PDEST2                                     &
     &               )
        IF ( IERR .LT. 0 ) GOTO 100
        IF ( SIZE_PACK.GT.SIZE_RBUF_BYTES ) THEN
             IERR = -3
             GOTO 100
        ENDIF
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        IF ( .NOT. TRANSP ) THEN
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        ELSE
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        END IF
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IF ( NSUBSET_ROW * NSUBSET_COL .NE. 0 ) THEN
          IF ( .NOT. TRANSP ) THEN
            DO ISUB = N_ALREADY_SENT+1, N_ALREADY_SENT+N_PACKET
              I         = SUBSET_ROW( ISUB )
              IPOS_ROOT = RG2L_ROW( INDROW_SON( I ) )
              ILOC_ROOT = MBLOCK                                        &
     &                 * ( ( IPOS_ROOT - 1 ) / ( MBLOCK * NPROW ) )     &
     &                 + mod( IPOS_ROOT - 1, MBLOCK ) + 1
              CALL AGMG_NOCALL('MPI_PACK')
            END DO
            DO JSUB = 1, NSUBSET_COL
              J         = SUBSET_COL( JSUB )
              JPOS_ROOT = RG2L_COL( INDCOL_SON( J ) )
              JLOC_ROOT = NBLOCK                                        &
     &                  * ( ( JPOS_ROOT - 1 ) / ( NBLOCK * NPCOL ) )    &
     &                  + mod( JPOS_ROOT - 1, NBLOCK ) + 1
              CALL AGMG_NOCALL('MPI_PACK')
            END DO
          ELSE
            DO JSUB = N_ALREADY_SENT+1, N_ALREADY_SENT+N_PACKET
              J         = SUBSET_COL( JSUB )
              IPOS_ROOT = RG2L_COL( INDCOL_SON( J ) )
              ILOC_ROOT = MBLOCK                                        &
     &                 * ( ( IPOS_ROOT - 1 ) / ( MBLOCK * NPROW ) )     &
     &                 + mod( IPOS_ROOT - 1, MBLOCK ) + 1
              CALL AGMG_NOCALL('MPI_PACK')
            END DO
            DO ISUB = 1, NSUBSET_ROW
              I         = SUBSET_ROW( ISUB )
              JPOS_ROOT = RG2L_ROW( INDROW_SON( I ) )
              JLOC_ROOT = NBLOCK                                        &
     &                  * ( ( JPOS_ROOT - 1 ) / ( NBLOCK * NPCOL ) )    &
     &                  + mod( JPOS_ROOT - 1, NBLOCK ) + 1
              CALL AGMG_NOCALL('MPI_PACK')
            END DO
          END IF
          IF ( TABSIZE.GE.int(N_PACKET,8)*int(NSUBSET_OTHER,8) ) THEN
            IF ( .NOT. TRANSP ) THEN
              ITAB = 1
              DO ISUB = N_ALREADY_SENT+1,                               &
     &                  N_ALREADY_SENT+N_PACKET
                I         = SUBSET_ROW( ISUB )
                DO JSUB = 1, NSUBSET_COL
                  J              = SUBSET_COL( JSUB )
                  TAB( ITAB )    = VAL_SON(J,I)
                  ITAB           = ITAB + 1
                END DO
              END DO
              CALL AGMG_NOCALL('MPI_PACK')
            ELSE
              ITAB = 1
              DO JSUB = N_ALREADY_SENT+1, N_ALREADY_SENT+N_PACKET
                J = SUBSET_COL( JSUB )
                DO ISUB = 1, NSUBSET_ROW
                  I         = SUBSET_ROW( ISUB )
                  TAB( ITAB ) = VAL_SON( J, I )
                  ITAB = ITAB + 1
                END DO
              END DO
              CALL AGMG_NOCALL('MPI_PACK')
            END IF
          ELSE
            IF ( .NOT. TRANSP ) THEN
              DO ISUB = N_ALREADY_SENT+1, N_ALREADY_SENT+N_PACKET
                I         = SUBSET_ROW( ISUB )
                DO JSUB = 1, NSUBSET_COL
                  J         = SUBSET_COL( JSUB )
                  CALL AGMG_NOCALL('MPI_PACK')
                END DO
              END DO
            ELSE
              DO JSUB = N_ALREADY_SENT+1, N_ALREADY_SENT+N_PACKET
                J = SUBSET_COL( JSUB )
                DO ISUB = 1, NSUBSET_ROW
                  I         = SUBSET_ROW( ISUB )
                  CALL AGMG_NOCALL('MPI_PACK')
                END DO
              END DO
            END IF
          ENDIF
        END IF
        CALL AGMG_NOCALL('MPI_ISEND')
        IF ( SIZE_PACK .LT. POSITION ) THEN
          WRITE(*,*) ' Error sending contribution to root:Size<positn'
          WRITE(*,*) ' Size,position=',SIZE_PACK,POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE_PACK .NE. POSITION )                                  &
     &  CALL DAGMG_MUMPS_1( BUF_CB, POSITION )
        N_ALREADY_SENT = N_ALREADY_SENT + N_PACKET
        IF (NSUBSET_ROW * NSUBSET_COL .NE. 0) THEN
          IF ( N_ALREADY_SENT.NE.NSUBSET_TOT ) IERR = -1
        ENDIF
  100   CONTINUE
        RETURN
        END SUBROUTINE DAGMG_MUMPS_648
        SUBROUTINE DAGMG_MUMPS_76( ISON, NELIM,                              &
     &             NELIM_ROW, NELIM_COL, NSLAVES, SLAVES,               &
     &             DEST, COMM, IERR )
        INTEGER ISON, NELIM
        INTEGER NSLAVES, DEST, COMM, IERR
        INTEGER NELIM_ROW( NELIM ), NELIM_COL( NELIM )
        INTEGER SLAVES( NSLAVES )
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER SIZE, POSITION, IPOS, IREQ
        INTEGER IONE
        INTEGER DEST2(1)
        PARAMETER ( IONE=1 )
        DEST2(1) = DEST
        IERR = 0
        SIZE = ( 3 + NSLAVES + 2 * NELIM ) * SIZEofINT
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE, IERR,                  &
     &                 IONE, DEST2                                      &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
          IF (SIZE.GT.SIZE_RBUF_BYTES) THEN
             IERR = -3
             RETURN
          ENDIF
        POSITION = IPOS
        BUF_CB%CONTENT( POSITION ) = ISON
        POSITION = POSITION + 1
        BUF_CB%CONTENT( POSITION ) = NELIM
        POSITION = POSITION + 1
        BUF_CB%CONTENT( POSITION ) = NSLAVES
        POSITION = POSITION + 1
        BUF_CB%CONTENT( POSITION: POSITION + NELIM - 1 ) = NELIM_ROW
        POSITION = POSITION + NELIM
        BUF_CB%CONTENT( POSITION: POSITION + NELIM - 1 ) = NELIM_COL
        POSITION = POSITION + NELIM
        BUF_CB%CONTENT( POSITION: POSITION + NSLAVES - 1 ) = SLAVES
        POSITION = POSITION + NSLAVES
        POSITION = POSITION - IPOS
        IF ( POSITION * SIZEofINT .NE. SIZE ) THEN
          WRITE(*,*) 'Error in DMUMPS_BUF_SEND_ROOT_NELIM_INDICES:',    &
     &               'wrong estimated size'
           CALL AGMG_MUMPS_ABORT()
        END IF
        CALL AGMG_NOCALL('MPI_ISEND')
        RETURN
        END SUBROUTINE DAGMG_MUMPS_76
        SUBROUTINE DAGMG_MUMPS_74( ISON, NELIM_ROOT,                         &
     &             DEST, COMM, IERR )
        IMPLICIT NONE
        INTEGER ISON, NELIM_ROOT, DEST, COMM, IERR
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER IPOS, IREQ, SIZE
        INTEGER IONE
        INTEGER DEST2(1)
        PARAMETER ( IONE=1 )
        DEST2(1)=DEST
        IERR = 0
        SIZE = 2 * SIZEofINT
   10   CONTINUE
        CALL DAGMG_MUMPS_4( BUF_SMALL, IPOS, IREQ, SIZE, IERR,               &
     &                 IONE, DEST2                                      &
     &               )
        IF ( IERR .LT. 0 ) THEN
          WRITE(*,*) 'Internal error 1 with small buffers '
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        BUF_SMALL%CONTENT( IPOS )     = ISON
        BUF_SMALL%CONTENT( IPOS + 1 ) = NELIM_ROOT
        CALL AGMG_NOCALL('MPI_ISEND')
        RETURN
        END SUBROUTINE DAGMG_MUMPS_74
        SUBROUTINE DAGMG_MUMPS_73                                            &
     &  ( TOT_ROOT_SIZE, TOT_CONT2RECV, DEST, COMM, IERR )
        IMPLICIT NONE
        INTEGER TOT_ROOT_SIZE, TOT_CONT2RECV, DEST, COMM, IERR
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER SIZE, IPOS, IREQ
        INTEGER IONE
        INTEGER DEST2(1)
        PARAMETER ( IONE=1 )
        IERR = 0
        DEST2(1) = DEST
        SIZE = 2 * SIZEofINT
        CALL DAGMG_MUMPS_4( BUF_SMALL, IPOS, IREQ, SIZE, IERR,               &
     &                 IONE, DEST2                                      &
     &               )
        IF ( IERR .LT. 0 ) THEN
          WRITE(*,*) 'Internal error 2 with small buffers '
           CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        BUF_SMALL%CONTENT( IPOS     ) = TOT_ROOT_SIZE
        BUF_SMALL%CONTENT( IPOS + 1 ) = TOT_CONT2RECV
        CALL AGMG_NOCALL('MPI_ISEND')
        RETURN
        END SUBROUTINE DAGMG_MUMPS_73
        SUBROUTINE DAGMG_MUMPS_63                                            &
     &             ( NRHS, INODE, W, LW, LD_W, DEST,MSGTAG,COMM,IERR )
        IMPLICIT NONE
        INTEGER NRHS, INODE,LW,COMM,IERR,DEST,MSGTAG, LD_W
        DOUBLE PRECISION W(LD_W, *)
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
        INTEGER SIZE, SIZE1, SIZE2
        INTEGER POSITION, IREQ, IPOS
        INTEGER IONE, K
        INTEGER DEST2(1)
        PARAMETER ( IONE=1 )
        IERR = 0
        DEST2(1) = DEST
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE = SIZE1 + SIZE2
        CALL DAGMG_MUMPS_4( BUF_CB, IPOS, IREQ, SIZE, IERR,                  &
     &                 IONE, DEST2                                      &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        DO K=1, NRHS
        CALL AGMG_NOCALL('MPI_PACK')
        END DO
        CALL AGMG_NOCALL('MPI_ISEND')
        IF ( SIZE .LT. POSITION ) THEN
          WRITE(*,*) 'Try_update: SIZE, POSITION = ',                   &
     &               SIZE, POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE .NE. POSITION ) CALL DAGMG_MUMPS_1( BUF_CB, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_63
        SUBROUTINE DAGMG_MUMPS_77                                            &
     &             ( BDC_SBTR,BDC_MEM,BDC_MD, COMM, NPROCS, LOAD,       &
     &               MEM,SBTR_CUR,                                      &
     &               LU_USAGE,                                          &
     &               FUTURE_NIV2,                                       &
     &               MYID, IERR)
        IMPLICIT NONE
        INTEGER COMM, NPROCS, MYID, IERR
        INTEGER FUTURE_NIV2(NPROCS)
        DOUBLE PRECISION LU_USAGE
        DOUBLE PRECISION LOAD
        DOUBLE PRECISION MEM,SBTR_CUR
        LOGICAL BDC_MEM,BDC_SBTR,BDC_MD
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER POSITION, IREQ, IPOS, SIZE1, SIZE2, SIZE
        INTEGER I, NDEST, IDEST, IPOSMSG, WHAT, NREALS
        INTEGER IZERO
        INTEGER MYID2(1)
        PARAMETER ( IZERO=0 )
        IERR = 0
        MYID2(1) = MYID
        NDEST = NPROCS - 1
        NDEST = 0
        DO I = 1, NPROCS
          IF ( I .NE. MYID + 1 .AND. FUTURE_NIV2(I).NE.0) THEN
            NDEST = NDEST + 1
          ENDIF
        ENDDO
        IF ( NDEST .eq. 0 ) THEN
           RETURN
        ENDIF
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        NREALS = 1
        IF (BDC_MEM) THEN
          NREALS = 2
        ENDIf
        IF (BDC_SBTR)THEN
          NREALS = 3
        ENDIF
        IF(BDC_MD)THEN
           NREALS=NREALS+1
        ENDIF
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE = SIZE1 + SIZE2
        CALL DAGMG_MUMPS_4( BUF_LOAD, IPOS, IREQ, SIZE, IERR,                &
     &                  IZERO, MYID2                                    &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        BUF_LOAD%ILASTMSG = BUF_LOAD%ILASTMSG + ( NDEST - 1 ) * OVHSIZE
        IPOS = IPOS - OVHSIZE
        DO IDEST = 1, NDEST - 1
          BUF_LOAD%CONTENT( IPOS + ( IDEST - 1 ) * OVHSIZE ) =          &
     &    IPOS + IDEST * OVHSIZE
        END DO
        BUF_LOAD%CONTENT( IPOS + ( NDEST - 1 ) * OVHSIZE ) = 0
        IPOSMSG = IPOS + OVHSIZE * NDEST
        WHAT = 0
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IF (BDC_MEM) THEN
          CALL AGMG_NOCALL('MPI_PACK')
        END IF
        IF (BDC_SBTR) THEN
          CALL AGMG_NOCALL('MPI_PACK')
        END IF
        IF(BDC_MD)THEN
           CALL AGMG_NOCALL('MPI_PACK')
        ENDIF
        IDEST = 0
        DO I = 0, NPROCS - 1
        IF ( I .NE. MYID .AND. FUTURE_NIV2(I+1) .NE. 0) THEN
            IDEST = IDEST + 1
            CALL AGMG_NOCALL('MPI_ISEND')
          END IF
        END DO
        SIZE = SIZE - ( NDEST - 1 ) * OVHSIZE * SIZEofINT
        IF ( SIZE .LT. POSITION ) THEN
          WRITE(*,*) ' Error in DMUMPS_77'
          WRITE(*,*) ' Size,position=',SIZE,POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE .NE. POSITION )                                       &
     &  CALL DAGMG_MUMPS_1( BUF_LOAD, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_77
        SUBROUTINE DAGMG_MUMPS_460                                           &
     &             ( WHAT, COMM, NPROCS,                                &
     &               FUTURE_NIV2,                                       &
     &               LOAD,UPD_LOAD,                                     &
     &               MYID, IERR)
        IMPLICIT NONE
        INTEGER COMM, NPROCS, MYID, IERR, WHAT
        DOUBLE PRECISION LOAD,UPD_LOAD
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER POSITION, IREQ, IPOS, SIZE1, SIZE2, SIZE
        INTEGER I, NDEST, IDEST, IPOSMSG, NREALS
        INTEGER IZERO
        INTEGER MYID2(1)
        INTEGER FUTURE_NIV2(NPROCS)
        PARAMETER ( IZERO=0 )
        IERR = 0
        IF (WHAT .NE. 2 .AND. WHAT .NE. 3 .AND.                         &
     &       WHAT.NE.6.AND. WHAT.NE.8 .AND.WHAT.NE.9.AND.               &
     &       WHAT.NE.17) THEN
          WRITE(*,*)                                                    &
     &  "Internal error 1 in DAGMG_MUMPS_460",WHAT
        END IF
        MYID2(1) = MYID
        NDEST = NPROCS - 1
        NDEST = 0
        DO I = 1, NPROCS
          IF ( I .NE. MYID + 1 .AND. FUTURE_NIV2(I).NE.0) THEN
            NDEST = NDEST + 1
          ENDIF
        ENDDO
        IF ( NDEST .eq. 0 ) THEN
           RETURN
        ENDIF
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        IF((WHAT.NE.17).AND.(WHAT.NE.10))THEN
           NREALS = 1
        ELSE
           NREALS = 2
        ENDIF
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE = SIZE1 + SIZE2
        CALL DAGMG_MUMPS_4( BUF_LOAD, IPOS, IREQ, SIZE, IERR,                &
     &                  IZERO, MYID2                                    &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        BUF_LOAD%ILASTMSG = BUF_LOAD%ILASTMSG + ( NDEST - 1 ) * OVHSIZE
        IPOS = IPOS - OVHSIZE
        DO IDEST = 1, NDEST - 1
          BUF_LOAD%CONTENT( IPOS + ( IDEST - 1 ) * OVHSIZE ) =          &
     &    IPOS + IDEST * OVHSIZE
        END DO
        BUF_LOAD%CONTENT( IPOS + ( NDEST - 1 ) * OVHSIZE ) = 0
        IPOSMSG = IPOS + OVHSIZE * NDEST
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IF((WHAT.EQ.17).OR.(WHAT.EQ.10))THEN
           CALL AGMG_NOCALL('MPI_PACK')
        ENDIF
        IDEST = 0
        DO I = 0, NPROCS - 1
          IF ( I .NE. MYID .AND. FUTURE_NIV2(I+1) .NE. 0) THEN
            IDEST = IDEST + 1
            CALL AGMG_NOCALL('MPI_ISEND')
          END IF
        END DO
        SIZE = SIZE - ( NDEST - 1 ) * OVHSIZE * SIZEofINT
        IF ( SIZE .LT. POSITION ) THEN
          WRITE(*,*) ' Error in DMUMPS_460'
          WRITE(*,*) ' Size,position=',SIZE,POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE .NE. POSITION )                                       &
     &  CALL DAGMG_MUMPS_1( BUF_LOAD, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_460
        SUBROUTINE DAGMG_MUMPS_519                                           &
     &             ( WHAT, COMM, NPROCS,                                &
     &               FATHER_NODE,INODE,NCB,K81,                         &
     &               MYID,REMOTE, IERR)
        IMPLICIT NONE
        INTEGER COMM, NPROCS, MYID, IERR, WHAT,REMOTE
        INTEGER FATHER_NODE,INODE
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER POSITION, IREQ, IPOS, SIZE
        INTEGER I, NDEST, IDEST, IPOSMSG
        INTEGER IZERO,NCB,K81
        INTEGER MYID2(1)
        PARAMETER ( IZERO=0 )
        MYID2(1) = MYID
        NDEST = 1
        IF ( NDEST .eq. 0 ) THEN
           RETURN
        ENDIF
        IF((K81.EQ.2).OR.(K81.EQ.3))THEN
           CALL AGMG_NOCALL('MPI_PACK_SIZE')
        ELSE
           CALL AGMG_NOCALL('MPI_PACK_SIZE')
        ENDIF
        CALL DAGMG_MUMPS_4( BUF_LOAD, IPOS, IREQ, SIZE, IERR,                &
     &                  IZERO, MYID2                                    &
     &               )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        BUF_LOAD%ILASTMSG = BUF_LOAD%ILASTMSG + ( NDEST - 1 ) * OVHSIZE
        IPOS = IPOS - OVHSIZE
        DO IDEST = 1, NDEST - 1
          BUF_LOAD%CONTENT( IPOS + ( IDEST - 1 ) * OVHSIZE ) =          &
     &    IPOS + IDEST * OVHSIZE
        END DO
        BUF_LOAD%CONTENT( IPOS + ( NDEST - 1 ) * OVHSIZE ) = 0
        IPOSMSG = IPOS + OVHSIZE * NDEST
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IF((K81.EQ.2).OR.(K81.EQ.3))THEN
           CALL AGMG_NOCALL('MPI_PACK')
           CALL AGMG_NOCALL('MPI_PACK')
        ENDIF
        IDEST = 1
        CALL AGMG_NOCALL('MPI_ISEND')
        SIZE = SIZE - ( NDEST - 1 ) * OVHSIZE * SIZEofINT
        IF ( SIZE .LT. POSITION ) THEN
          WRITE(*,*) ' Error in DMUMPS_519'
          WRITE(*,*) ' Size,position=',SIZE,POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE .NE. POSITION )                                       &
     &  CALL DAGMG_MUMPS_1( BUF_LOAD, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_519
        SUBROUTINE DAGMG_MUMPS_502( COMM, MYID, NPROCS,                      &
     &  MAX_SURF_MASTER,IERR)
        IMPLICIT NONE
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        INTEGER IPOS, IREQ, IDEST, IPOSMSG, POSITION, I
        INTEGER COMM, MYID, IERR, NPROCS
        DOUBLE PRECISION MAX_SURF_MASTER
        INTEGER IZERO
        INTEGER MYID2(1)
        PARAMETER ( IZERO=0 )
        INTEGER NDEST, NINTS, NREALS, SIZE, SIZE1, SIZE2
        INTEGER WHAT
        IERR = 0
        MYID2(1) = MYID
        NDEST = NPROCS - 1
        NINTS = 1 + ( NDEST-1 ) * OVHSIZE
        NREALS = 1
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE=SIZE1+SIZE2
        CALL DAGMG_MUMPS_4( BUF_LOAD, IPOS, IREQ, SIZE, IERR,                &
     &       IZERO, MYID2 )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        BUF_LOAD%ILASTMSG = BUF_LOAD%ILASTMSG + ( NDEST - 1 ) * OVHSIZE
        IPOS = IPOS - OVHSIZE
        DO IDEST = 1, NDEST - 1
          BUF_LOAD%CONTENT( IPOS + ( IDEST - 1 ) * OVHSIZE ) =          &
     &    IPOS + IDEST * OVHSIZE
        END DO
        BUF_LOAD%CONTENT( IPOS + ( NDEST - 1 ) * OVHSIZE ) = 0
        IPOSMSG = IPOS + OVHSIZE * NDEST
        POSITION = 0
        WHAT = 4
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IDEST = 0
        DO I = 0, NPROCS - 1
           IF ( I .ne. MYID ) THEN
              IDEST = IDEST + 1
              CALL AGMG_NOCALL('MPI_ISEND')
           END IF
        END DO
        SIZE = SIZE - ( NDEST - 1 ) * OVHSIZE * SIZEofINT
        IF ( SIZE .LT. POSITION ) THEN
          WRITE(*,*) ' Error in DMUMPS_524'
          WRITE(*,*) ' Size,position=',SIZE,POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE .NE. POSITION )                                       &
     &  CALL DAGMG_MUMPS_1( BUF_LOAD, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_502
        SUBROUTINE DAGMG_MUMPS_524( BDC_MEM,                                 &
     &      COMM, MYID, NPROCS,                                         &
     &      FUTURE_NIV2,                                                &
     &      NSLAVES,                                                    &
     &      LIST_SLAVES,INODE,                                          &
     &      MEM_INCREMENT, FLOPS_INCREMENT,CB_BAND, WHAT,               &
     &      IERR )
        IMPLICIT NONE
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
        LOGICAL BDC_MEM
        INTEGER COMM, MYID, NPROCS, NSLAVES, IERR
        INTEGER FUTURE_NIV2(NPROCS)
        INTEGER LIST_SLAVES(NSLAVES),INODE
        DOUBLE PRECISION MEM_INCREMENT(NSLAVES)
        DOUBLE PRECISION FLOPS_INCREMENT(NSLAVES)
        DOUBLE PRECISION CB_BAND(NSLAVES)
        INTEGER NDEST, NINTS, NREALS, SIZE1, SIZE2, SIZE
        INTEGER IPOS, IPOSMSG, IREQ, POSITION
        INTEGER I, IDEST, WHAT
        INTEGER IZERO
        INTEGER MYID2(1)
        PARAMETER ( IZERO=0 )
        MYID2(1)=MYID
        IERR = 0
        NDEST = 0
        DO I = 1, NPROCS
          IF ( I .NE. MYID + 1 .AND. FUTURE_NIV2(I).NE.0) THEN
            NDEST = NDEST + 1
          ENDIF
        ENDDO
        IF ( NDEST == 0 ) THEN
           RETURN
        ENDIF
        NINTS = 2 +  NSLAVES + ( NDEST - 1 ) * OVHSIZE + 1
        NREALS = NSLAVES
        IF (BDC_MEM) NREALS = NREALS + NSLAVES
        IF(WHAT.EQ.19) THEN
           NREALS = NREALS + NSLAVES
        ENDIF
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        CALL AGMG_NOCALL('MPI_PACK_SIZE')
        SIZE = SIZE1+SIZE2
        CALL DAGMG_MUMPS_4( BUF_LOAD, IPOS, IREQ, SIZE, IERR,                &
     &       IZERO, MYID2 )
        IF ( IERR .LT. 0 ) THEN
           RETURN
        ENDIF
        BUF_LOAD%ILASTMSG = BUF_LOAD%ILASTMSG + ( NDEST - 1 ) * OVHSIZE
        IPOS = IPOS - OVHSIZE
        DO IDEST = 1, NDEST - 1
          BUF_LOAD%CONTENT( IPOS + ( IDEST - 1 ) * OVHSIZE ) =          &
     &    IPOS + IDEST * OVHSIZE
        END DO
        BUF_LOAD%CONTENT( IPOS + ( NDEST - 1 ) * OVHSIZE ) = 0
        IPOSMSG = IPOS + OVHSIZE * NDEST
        POSITION = 0
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        CALL AGMG_NOCALL('MPI_PACK')
        IF (BDC_MEM) THEN
          CALL AGMG_NOCALL('MPI_PACK')
        END IF
        IF(WHAT.EQ.19)THEN
           CALL AGMG_NOCALL('MPI_PACK')
        ENDIF
        IDEST = 0
        DO I = 0, NPROCS - 1
        IF ( I .NE. MYID .AND. FUTURE_NIV2(I+1) .NE. 0) THEN
            IDEST = IDEST + 1
            CALL AGMG_NOCALL('MPI_ISEND')
          END IF
        END DO
        SIZE = SIZE - ( NDEST - 1 ) * OVHSIZE * SIZEofINT
        IF ( SIZE .LT. POSITION ) THEN
          WRITE(*,*) ' Error in DMUMPS_524'
          WRITE(*,*) ' Size,position=',SIZE,POSITION
          CALL AGMG_MUMPS_ABORT()
        END IF
        IF ( SIZE .NE. POSITION )                                       &
     &  CALL DAGMG_MUMPS_1( BUF_LOAD, POSITION )
        RETURN
        END SUBROUTINE DAGMG_MUMPS_524
        SUBROUTINE DAGMG_MUMPS_60                                            &
     &             ( DAGMG_MUMPS_LBUFR_BYTES)
        IMPLICIT NONE
        INTEGER DAGMG_MUMPS_LBUFR_BYTES
        SIZE_RBUF_BYTES = DAGMG_MUMPS_LBUFR_BYTES
        RETURN
      END SUBROUTINE DAGMG_MUMPS_60
      END MODULE DAGMG_MUMPS_COMM_BUFF
      ! inserted 36 here
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      MODULE DAGMG_MUMPS_OOC_MAIN
      USE AGMG_MUMPS_OOC_COMM
      IMPLICIT NONE
      INTEGER NOT_IN_MEM,BEING_READ,NOT_USED,PERMUTED,USED,             &
     &     USED_NOT_PERMUTED,ALREADY_USED
      PARAMETER (NOT_IN_MEM=0,BEING_READ=-1,NOT_USED=-2,                &
     &     PERMUTED=-3,USED=-4,USED_NOT_PERMUTED=-5,ALREADY_USED=-6)
      INTEGER OOC_NODE_NOT_IN_MEM,OOC_NODE_PERMUTED,                    &
     &     OOC_NODE_NOT_PERMUTED
      PARAMETER (OOC_NODE_NOT_IN_MEM=-20,                               &
     &     OOC_NODE_PERMUTED=-21,OOC_NODE_NOT_PERMUTED=-22)
      INTEGER(8), DIMENSION(:,:),POINTER :: SIZE_OF_BLOCK
      INTEGER, DIMENSION(:),POINTER :: TOTAL_NB_OOC_NODES
      INTEGER :: OOC_SOLVE_TYPE_FCT
      INTEGER, DIMENSION(:),ALLOCATABLE :: IO_REQ
      INTEGER(8), DIMENSION(:), ALLOCATABLE:: LRLUS_SOLVE
      INTEGER(8), DIMENSION(:), ALLOCATABLE:: SIZE_SOLVE_Z,             &
     & LRLU_SOLVE_T, POSFAC_SOLVE, IDEB_SOLVE_Z, LRLU_SOLVE_B
      INTEGER, DIMENSION(:),ALLOCATABLE :: PDEB_SOLVE_Z
      INTEGER (8),SAVE :: FACT_AREA_SIZE,                               &
     &     SIZE_ZONE_SOLVE,SIZE_SOLVE_EMM,TMP_SIZE_FACT,                &
     &     MAX_SIZE_FACTOR_OOC
      INTEGER(8), SAVE :: MIN_SIZE_READ
      INTEGER, SAVE :: TMP_NB_NODES, MAX_NB_NODES_FOR_ZONE,MAX_NB_REQ,  &
     &     CURRENT_SOLVE_READ_ZONE,                                     &
     &     CUR_POS_SEQUENCE,NB_Z,SOLVE_STEP,                            &
     &     NB_ZONE_REQ,MTYPE_OOC,NB_ACT                                 &
     &     ,NB_CALLED,REQ_ACT,NB_CALL
      INTEGER*8, SAVE :: OOC_VADDR_PTR
      INTEGER*8, SAVE :: SIZE_ZONE_REQ
      DOUBLE PRECISION,SAVE :: MAX_OOC_FILE_SIZE
      INTEGER(8), DIMENSION(:), ALLOCATABLE :: SIZE_OF_READ, READ_DEST
      INTEGER,DIMENSION(:),ALLOCATABLE :: FIRST_POS_IN_READ,            &
     &     READ_MNG,REQ_TO_ZONE,POS_HOLE_T,                             &
     &     POS_HOLE_B,REQ_ID,OOC_STATE_NODE
      INTEGER DAGMG_MUMPS_ELEMENTARY_DATA_SIZE,N_OOC
      INTEGER, DIMENSION(:), ALLOCATABLE :: POS_IN_MEM, INODE_TO_POS
      INTEGER, DIMENSION(:), ALLOCATABLE :: CURRENT_POS_T,CURRENT_POS_B
      LOGICAL IS_ROOT_SPECIAL
      INTEGER SPECIAL_ROOT_NODE
      PUBLIC :: DAGMG_MUMPS_575,DAGMG_MUMPS_576,                                  &
     &     DAGMG_MUMPS_577,                                                  &
     &     DAGMG_MUMPS_578,                                                  &
     &     DAGMG_MUMPS_579,                                                  &
     &     DAGMG_MUMPS_581,DAGMG_MUMPS_582,                                       &
     &     DAGMG_MUMPS_583,DAGMG_MUMPS_584,                                       &
     &     DAGMG_MUMPS_585,DAGMG_MUMPS_586
         INTEGER, PARAMETER, PUBLIC :: TYPEF_BOTH_LU = -99976
         PUBLIC DAGMG_MUMPS_688,                                             &
     &        DAGMG_MUMPS_690
         PRIVATE DAGMG_MUMPS_695,                                            &
     &        DAGMG_MUMPS_697
      CONTAINS
      SUBROUTINE DAGMG_MUMPS_711( STRAT_IO_ARG,                              &
     & STRAT_IO_ASYNC_ARG, WITH_BUF_ARG, LOW_LEVEL_STRAT_IO_ARG )
      IMPLICIT NONE
      INTEGER, intent(out) :: LOW_LEVEL_STRAT_IO_ARG
      LOGICAL, intent(out) :: STRAT_IO_ASYNC_ARG, WITH_BUF_ARG
      INTEGER, intent(in)  :: STRAT_IO_ARG
      INTEGER TMP
      CALL AGMG_NOCALL('MUMPS_OOC_IS_ASYNC_AVAIL')
      STRAT_IO_ASYNC_ARG=.FALSE.
      WITH_BUF_ARG=.FALSE.
      IF(TMP.EQ.1)THEN
         IF((STRAT_IO_ARG.EQ.1).OR.(STRAT_IO_ARG.EQ.2))THEN
            STRAT_IO_ASYNC=.TRUE.
            WITH_BUF=.FALSE.
         ELSEIF((STRAT_IO_ARG.EQ.4).OR.(STRAT_IO_ARG.EQ.5))THEN
            STRAT_IO_ASYNC_ARG=.TRUE.
            WITH_BUF_ARG=.TRUE.
         ELSEIF(STRAT_IO_ARG.EQ.3)THEN
            STRAT_IO_ASYNC_ARG=.FALSE.
            WITH_BUF_ARG=.TRUE.
         ENDIF
         LOW_LEVEL_STRAT_IO_ARG=mod(STRAT_IO_ARG,3)
      ELSE
         LOW_LEVEL_STRAT_IO_ARG=0
         IF(STRAT_IO_ARG.GE.3)THEN
            WITH_BUF_ARG=.TRUE.
         ENDIF
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_711
      FUNCTION DAGMG_MUMPS_579(INODE,ZONE)
      IMPLICIT NONE
      INTEGER INODE,ZONE
      LOGICAL DAGMG_MUMPS_579
      DAGMG_MUMPS_579=(LRLUS_SOLVE(ZONE).GE.                                 &
     &     SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE))
      RETURN
      END FUNCTION DAGMG_MUMPS_579
      SUBROUTINE DAGMG_MUMPS_590(LA)
      IMPLICIT NONE
      INTEGER(8) :: LA
      FACT_AREA_SIZE=LA
      END SUBROUTINE DAGMG_MUMPS_590
      SUBROUTINE DAGMG_MUMPS_575(id, MAXS)
      USE DAGMG_MUMPS_STRUC_DF
      USE DAGMG_MUMPS_OOC_BUFF
      IMPLICIT NONE
      INTEGER TMPDIR_MAX_LENGTH, PREFIX_MAX_LENGTH
      PARAMETER (TMPDIR_MAX_LENGTH=255, PREFIX_MAX_LENGTH=63)
      INTEGER(8), intent(in) :: MAXS
      TYPE(DAGMG_MUMPS_STRUC), TARGET :: id
      INTEGER IERR
      INTEGER allocok
      INTEGER ASYNC
      CHARACTER*1 TMP_DIR(TMPDIR_MAX_LENGTH),                           &
     &            TMP_PREFIX(PREFIX_MAX_LENGTH)
      INTEGER DIM_DIR,DIM_PREFIX
      INTEGER, DIMENSION(:), ALLOCATABLE :: FILE_FLAG_TAB
      INTEGER TMP
      INTEGER K211_LOC
      ICNTL1=id%ICNTL(1)
      MAX_SIZE_FACTOR_OOC=0_8
      OOC_FCT_TYPE=1
      N_OOC=id%N
      IS_INIT_CALLED=.TRUE.
      ASYNC=0
      SOLVE=.FALSE.
      IERR=0
      IF(allocated(IO_REQ))THEN
         DEALLOCATE(IO_REQ)
      ENDIF
      IF(associated(KEEP_OOC))THEN
         NULLIFY(KEEP_OOC)
      ENDIF
      IF(associated(STEP_OOC))THEN
         NULLIFY(STEP_OOC)
      ENDIF
      IF(associated(PROCNODE_OOC))THEN
         NULLIFY(PROCNODE_OOC)
      ENDIF
      IF(associated(OOC_INODE_SEQUENCE))THEN
         NULLIFY(OOC_INODE_SEQUENCE)
      ENDIF
      IF(associated(TOTAL_NB_OOC_NODES))THEN
         NULLIFY(TOTAL_NB_OOC_NODES)
      ENDIF
      IF(associated(SIZE_OF_BLOCK))THEN
         NULLIFY(SIZE_OF_BLOCK)
      ENDIF
      IF(associated(OOC_VADDR))THEN
         NULLIFY(OOC_VADDR)
      ENDIF
      IF(allocated(I_CUR_HBUF_NEXTPOS))THEN
         DEALLOCATE(I_CUR_HBUF_NEXTPOS)
      ENDIF
      CALL DAGMG_MUMPS_588(id,IERR)
      IF(IERR.LT.0)THEN
         IF (ICNTL1 > 0)                                                &
     &   WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         id%INFO(1) = IERR
         id%INFO(2) = 0
         RETURN
      ENDIF
      STEP_OOC=>id%STEP
      PROCNODE_OOC=>id%PROCNODE_STEPS
      MYID_OOC=id%MYID
      SLAVEF_OOC=id%NSLAVES
      KEEP_OOC => id%KEEP
      SIZE_OF_BLOCK=>id%OOC_SIZE_OF_BLOCK
      OOC_VADDR=>id%OOC_VADDR
      IF(id%KEEP(107).GT.0)THEN
         SIZE_SOLVE_EMM=max(id%KEEP8(19),int(dble(MAXS)*                &
     &        0.9d0*0.2d0,8))
         SIZE_ZONE_SOLVE=max(SIZE_SOLVE_EMM,                            &
     &        int((dble(MAXS)*0.9d0-                                    &
     &        dble(SIZE_SOLVE_EMM))/dble(id%KEEP(107)),8))
         IF(SIZE_ZONE_SOLVE.EQ.SIZE_SOLVE_EMM)THEN
            SIZE_SOLVE_EMM=id%KEEP8(19)
            SIZE_ZONE_SOLVE=int((dble(MAXS)*0.9d0-                      &
     &           dble(SIZE_SOLVE_EMM))/dble(id%KEEP(107)),8)
         ENDIF
      ELSE
         SIZE_ZONE_SOLVE=int(dble(MAXS)*0.9d0,8)
         SIZE_SOLVE_EMM=SIZE_ZONE_SOLVE
      ENDIF
      DAGMG_MUMPS_ELEMENTARY_DATA_SIZE = id%KEEP(35)
      SIZE_OF_BLOCK=0_8
      ALLOCATE(id%OOC_NB_FILES(OOC_NB_FILE_TYPE), stat=allocok)
      IF (allocok .GT. 0) THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         id%INFO(1) = -13
         id%INFO(2) = OOC_NB_FILE_TYPE
         RETURN
      ENDIF
      id%OOC_NB_FILES=0
      OOC_VADDR_PTR=0_8
      CALL DAGMG_MUMPS_711( id%KEEP(99), STRAT_IO_ASYNC,                     &
     &                                WITH_BUF, LOW_LEVEL_STRAT_IO )
      TMP_SIZE_FACT=0_8
      TMP_NB_NODES=0
      MAX_NB_NODES_FOR_ZONE=0
      OOC_INODE_SEQUENCE=>id%OOC_INODE_SEQUENCE
      ALLOCATE(I_CUR_HBUF_NEXTPOS(OOC_NB_FILE_TYPE),                    &
     &     stat=allocok)
      IF (allocok .GT. 0) THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         id%INFO(1) = -13
         id%INFO(2) = OOC_NB_FILE_TYPE
         RETURN
      ENDIF
      I_CUR_HBUF_NEXTPOS = 1
      IF(WITH_BUF)THEN
         CALL DAGMG_MUMPS_669(id%INFO(1),id%INFO(2),IERR)
         IF(IERR.LT.0)THEN
            RETURN
         ENDIF
      ENDIF
      IF(STRAT_IO_ASYNC)THEN
         ASYNC=1
      ENDIF
      DIM_ERR_STR_OOC = ERR_STR_OOC_MAX_LEN
      CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_INIT_ERR_STR')
      DIM_DIR=len(trim(id%OOC_TMPDIR))
      DIM_PREFIX=len(trim(id%OOC_PREFIX))
      CALL DAGMG_MUMPS_589(TMP_DIR(1),                                       &
     &     id%OOC_TMPDIR, TMPDIR_MAX_LENGTH, DIM_DIR )
      CALL DAGMG_MUMPS_589(TMP_PREFIX(1),                                    &
     &     id%OOC_PREFIX, PREFIX_MAX_LENGTH, DIM_PREFIX)
      CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_INIT_PREFIX')
      CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_INIT_TMPDIR')
      ALLOCATE(FILE_FLAG_TAB(OOC_NB_FILE_TYPE),                         &
     &     stat=allocok)
      IF (allocok .GT. 0) THEN
         IF (ICNTL1 .GT. 0)                                             &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         id%INFO(1) = -13
         id%INFO(2) = OOC_NB_FILE_TYPE
         RETURN
      ENDIF
      FILE_FLAG_TAB(1:OOC_NB_FILE_TYPE)=0
      IERR=0
      TMP=int(id%KEEP8(11)/1000000_8,kind=4)+1
      IF((id%KEEP(201).EQ.1).AND.(id%KEEP(50).EQ.0))THEN
         TMP=MAX(1,TMP/2)
      ENDIF
      CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_INIT_OOC_C')
      IF(IERR.LT.0)THEN
         IF (ICNTL1 .GT. 0 ) THEN
           WRITE(ICNTL1,*)MYID_OOC,': PB in MUMPS_LOW_LEVEL_INIT_OOC_C'
           WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         ENDIF
         id%INFO(1) = IERR
         id%INFO(2) = 0
         RETURN
      ENDIF
      CALL AGMG_NOCALL('MUMPS_GET_MAX_FILE_SIZE_C')
      DEALLOCATE(FILE_FLAG_TAB)
      RETURN
      END SUBROUTINE DAGMG_MUMPS_575
      SUBROUTINE DAGMG_MUMPS_576(INODE,PTRFAC,KEEP,KEEP8,                    &
     &           A,LA,SIZE,IERR)
      USE DAGMG_MUMPS_OOC_BUFF
      IMPLICIT NONE
      INTEGER INODE,KEEP(500)
      INTEGER(8) :: LA
      INTEGER*8 KEEP8(150)
      INTEGER(8) :: PTRFAC(KEEP(28)), SIZE
      DOUBLE PRECISION A(LA)
      INTEGER IERR,I,FLAG,NODE,ASYNC,REQUEST
      LOGICAL IO_C
      INTEGER ADDR_INT1,ADDR_INT2
      INTEGER TYPE
      INTEGER SIZE_INT1,SIZE_INT2
      TYPE=FCT
      IF(STRAT_IO_ASYNC)THEN
         ASYNC=1
      ELSE
         ASYNC=0
      ENDIF
      IERR=0
      IO_C=.TRUE.
      SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)=SIZE
      MAX_SIZE_FACTOR_OOC=max(MAX_SIZE_FACTOR_OOC,SIZE)
      OOC_VADDR(STEP_OOC(INODE),OOC_FCT_TYPE)=OOC_VADDR_PTR
      OOC_VADDR_PTR=OOC_VADDR_PTR+SIZE
      TMP_SIZE_FACT=TMP_SIZE_FACT+SIZE
      TMP_NB_NODES=TMP_NB_NODES+1
      IF(TMP_SIZE_FACT.GT.SIZE_ZONE_SOLVE)THEN
         MAX_NB_NODES_FOR_ZONE=max(MAX_NB_NODES_FOR_ZONE,TMP_NB_NODES)
         TMP_SIZE_FACT=0_8
         TMP_NB_NODES=0
      ENDIF
      IF (.NOT. WITH_BUF) THEN
         CALL AGMG_MUMPS_677(ADDR_INT1,ADDR_INT2,                            &
     &        OOC_VADDR(STEP_OOC(INODE),OOC_FCT_TYPE))
         CALL AGMG_MUMPS_677(SIZE_INT1,SIZE_INT2,                            &
     &        SIZE)
         CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_WRITE_OOC_C')
        IF(IERR.LT.0)THEN
           IF (ICNTL1.GT.0)                                             &
     &     WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
           RETURN
        ENDIF
        IF(I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE).GT.KEEP_OOC(28))THEN
             WRITE(*,*)MYID_OOC,': Internal error (37) in OOC '
           CALL AGMG_MUMPS_ABORT()
        ENDIF
        OOC_INODE_SEQUENCE(I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE),            &
     &       OOC_FCT_TYPE)=INODE
        I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE)=                               &
     &       I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE)+1
      ELSE
         IF(SIZE.LE.HBUF_SIZE)THEN
            CALL DAGMG_MUMPS_678                                             &
     &           (A(PTRFAC(STEP_OOC(INODE))),SIZE,IERR)
            OOC_INODE_SEQUENCE(I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE),        &
     &           OOC_FCT_TYPE) = INODE
            I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE) =                          &
     &           I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE) + 1
            PTRFAC(STEP_OOC(INODE))=-777777_8
            RETURN
         ELSE
            CALL DAGMG_MUMPS_707(OOC_FCT_TYPE,IERR)
            IF(IERR.LT.0)THEN
               RETURN
            ENDIF
            CALL DAGMG_MUMPS_707(OOC_FCT_TYPE,IERR)
            IF(IERR.LT.0)THEN
               RETURN
            ENDIF
            CALL AGMG_MUMPS_677(ADDR_INT1,ADDR_INT2,                         &
     &           OOC_VADDR(STEP_OOC(INODE),OOC_FCT_TYPE))
            CALL AGMG_MUMPS_677(SIZE_INT1,SIZE_INT2,                         &
     &           SIZE)
            CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_WRITE_OOC_C')
            IF(IERR.LT.0)THEN
               IF (ICNTL1.GT.0)                                         &
     &         WRITE(*,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
               RETURN
            ENDIF
            IF(I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE).GT.KEEP_OOC(28))THEN
             WRITE(*,*)MYID_OOC,': Internal error (38) in OOC '
               CALL AGMG_MUMPS_ABORT()
            ENDIF
            OOC_INODE_SEQUENCE(I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE),        &
     &           OOC_FCT_TYPE)=INODE
            I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE)=                           &
     &           I_CUR_HBUF_NEXTPOS(OOC_FCT_TYPE)+1
            CALL DAGMG_MUMPS_689(OOC_FCT_TYPE)
         ENDIF
      END IF
      NODE=-9999
      PTRFAC(STEP_OOC(INODE))=-777777_8
      IF(STRAT_IO_ASYNC)THEN
         IERR=0
         CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
         IF(IERR.LT.0)THEN
            IF (ICNTL1 .GT. 0)                                          &
     &      WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
            RETURN
         ENDIF
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_576
      SUBROUTINE DAGMG_MUMPS_577(DEST,INODE,IERR                             &
     &  )
      IMPLICIT NONE
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
      INTEGER IERR,INODE,STATUS
      DOUBLE PRECISION DEST
      INTEGER FLAG,NODE,ASYNC,REQUEST
      LOGICAL IO_C
      INTEGER ADDR_INT1,ADDR_INT2
      INTEGER TYPE
      INTEGER SIZE_INT1,SIZE_INT2
      TYPE=OOC_SOLVE_TYPE_FCT
      IF(SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)                    &
     &     .EQ.0_8)THEN
         GOTO 555
      ENDIF
      IF(STRAT_IO_ASYNC)THEN
        ASYNC=1
      ELSE
        ASYNC=0
      ENDIF
      IERR=0
      IO_C=.TRUE.
      OOC_STATE_NODE(STEP_OOC(INODE))=NOT_USED
      CALL AGMG_MUMPS_677(ADDR_INT1,ADDR_INT2,                               &
     &     OOC_VADDR(STEP_OOC(INODE),OOC_FCT_TYPE))
      CALL AGMG_MUMPS_677(SIZE_INT1,SIZE_INT2,                               &
     &     SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE))
      CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_DIRECT_READ')
      IF(IERR.LT.0)THEN
         IF (ICNTL1.GT.0) THEN
           WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
           WRITE(ICNTL1,*)MYID_OOC,                                     &
     &     ': Problem in MUMPS_LOW_LEVEL_DIRECT_READ'
         ENDIF
         RETURN
      ENDIF
  555 CONTINUE
      IF(.NOT.DAGMG_MUMPS_727())THEN
         IF(OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE,OOC_FCT_TYPE).EQ.       &
     &        INODE)THEN
            IF(SOLVE_STEP.EQ.0)THEN
               CUR_POS_SEQUENCE=CUR_POS_SEQUENCE+1
            ELSEIF(SOLVE_STEP.EQ.1)THEN
               CUR_POS_SEQUENCE=CUR_POS_SEQUENCE-1
            ENDIF
            CALL DAGMG_MUMPS_728()
         ENDIF
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_577
      SUBROUTINE DAGMG_MUMPS_591(IERR)
      USE DAGMG_MUMPS_OOC_BUFF
      IMPLICIT NONE
      INTEGER, intent(out):: IERR
      IERR=0
      IF (WITH_BUF) THEN
        CALL DAGMG_MUMPS_675(IERR)
        IF(IERR.LT.0)THEN
           RETURN
        ENDIF
      END IF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_591
      SUBROUTINE DAGMG_MUMPS_592(id,IERR)
      USE DAGMG_MUMPS_OOC_BUFF
      USE DAGMG_MUMPS_STRUC_DF
      IMPLICIT NONE
      TYPE(DAGMG_MUMPS_STRUC), TARGET :: id
      INTEGER, intent(out) :: IERR
      INTEGER I,SOLVE_OR_FACTO
      IERR=0
      IF(WITH_BUF)THEN
         CALL DAGMG_MUMPS_659()
      ENDIF
      IF(associated(KEEP_OOC))THEN
         NULLIFY(KEEP_OOC)
      ENDIF
      IF(associated(STEP_OOC))THEN
         NULLIFY(STEP_OOC)
      ENDIF
      IF(associated(PROCNODE_OOC))THEN
         NULLIFY(PROCNODE_OOC)
      ENDIF
      IF(associated(OOC_INODE_SEQUENCE))THEN
         NULLIFY(OOC_INODE_SEQUENCE)
      ENDIF
      IF(associated(TOTAL_NB_OOC_NODES))THEN
         NULLIFY(TOTAL_NB_OOC_NODES)
      ENDIF
      IF(associated(SIZE_OF_BLOCK))THEN
         NULLIFY(SIZE_OF_BLOCK)
      ENDIF
      IF(associated(OOC_VADDR))THEN
         NULLIFY(OOC_VADDR)
      ENDIF
      CALL AGMG_NOCALL('MUMPS_OOC_END_WRITE_C')
      IF(IERR.LT.0)THEN
         IF (ICNTL1 .GT. 0)                                             &
     &   WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         GOTO 500
      ENDIF
      id%OOC_MAX_NB_NODES_FOR_ZONE=max(MAX_NB_NODES_FOR_ZONE,           &
     &     TMP_NB_NODES)
      IF(allocated(I_CUR_HBUF_NEXTPOS))THEN
         DO I=1,OOC_NB_FILE_TYPE
            id%OOC_TOTAL_NB_NODES(I)=I_CUR_HBUF_NEXTPOS(I)-1
         ENDDO
         DEALLOCATE(I_CUR_HBUF_NEXTPOS)
      ENDIF
      id%KEEP8(20)=MAX_SIZE_FACTOR_OOC
      CALL DAGMG_MUMPS_613(id,IERR)
      IF(IERR.LT.0)THEN
         GOTO 500
      ENDIF
  500 CONTINUE
      SOLVE_OR_FACTO=0
      CALL AGMG_NOCALL('MUMPS_CLEAN_IO_DATA_C')
      IF(IERR.LT.0)THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         RETURN
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_592
      SUBROUTINE DAGMG_MUMPS_588(id,IERR)
      USE DAGMG_MUMPS_STRUC_DF
      IMPLICIT NONE
      TYPE(DAGMG_MUMPS_STRUC), TARGET :: id
      INTEGER IERR
      INTEGER I,J,I1,K
      CHARACTER*1 TMP_NAME(350)
      IERR=0
      K=1
      IF(associated(id%OOC_FILE_NAMES).AND.                             &
     &     associated(id%OOC_FILE_NAME_LENGTH))THEN
         DO I1=1,OOC_NB_FILE_TYPE
            DO I=1,id%OOC_NB_FILES(I1)
               DO J=1,id%OOC_FILE_NAME_LENGTH(K)
                  TMP_NAME(J)=id%OOC_FILE_NAMES(K,J)
               ENDDO
               CALL AGMG_NOCALL('MUMPS_OOC_REMOVE_FILE_C')
               IF(IERR.LT.0)THEN
                  IF (ICNTL1.GT.0)THEN
                     WRITE(ICNTL1,*)MYID_OOC,': ',                      &
     &                    ERR_STR_OOC(1:DIM_ERR_STR_OOC)
                     RETURN
                  ENDIF
               ENDIF
               K=K+1
            ENDDO
         ENDDO
      ENDIF
      IF(associated(id%OOC_FILE_NAMES))THEN
         DEALLOCATE(id%OOC_FILE_NAMES)
         NULLIFY(id%OOC_FILE_NAMES)
      ENDIF
      IF(associated(id%OOC_FILE_NAME_LENGTH))THEN
         DEALLOCATE(id%OOC_FILE_NAME_LENGTH)
         NULLIFY(id%OOC_FILE_NAME_LENGTH)
      ENDIF
      IF(associated(id%OOC_NB_FILES))THEN
         DEALLOCATE(id%OOC_NB_FILES)
         NULLIFY(id%OOC_NB_FILES)
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_588
      SUBROUTINE DAGMG_MUMPS_587(id,IERR)
      USE DAGMG_MUMPS_STRUC_DF
      IMPLICIT NONE
      TYPE(DAGMG_MUMPS_STRUC), TARGET :: id
      INTEGER IERR
      IERR=0
      CALL DAGMG_MUMPS_588(id,IERR)
      IF(associated(id%OOC_TOTAL_NB_NODES))THEN
         DEALLOCATE(id%OOC_TOTAL_NB_NODES)
         NULLIFY(id%OOC_TOTAL_NB_NODES)
      ENDIF
      IF(associated(id%OOC_INODE_SEQUENCE))THEN
         DEALLOCATE(id%OOC_INODE_SEQUENCE)
         NULLIFY(id%OOC_INODE_SEQUENCE)
      ENDIF
      IF(associated(id%OOC_SIZE_OF_BLOCK))THEN
         DEALLOCATE(id%OOC_SIZE_OF_BLOCK)
         NULLIFY(id%OOC_SIZE_OF_BLOCK)
      ENDIF
      IF(associated(id%OOC_VADDR))THEN
         DEALLOCATE(id%OOC_VADDR)
         NULLIFY(id%OOC_VADDR)
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_587
      SUBROUTINE DAGMG_MUMPS_586(id)
      USE DAGMG_MUMPS_STRUC_DF
      IMPLICIT NONE
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
      TYPE(DAGMG_MUMPS_STRUC), TARGET :: id
      INTEGER TMP,TMP_SIZE,I,J
      INTEGER(8) :: TMP_SIZE8
      INTEGER ZONE,INODE,SIZE,allocok,IERR,ASYNC
      EXTERNAL AGMG_MUMPS_275
      INTEGER AGMG_MUMPS_275
      INTEGER MASTER_ROOT
      IERR=0
      ICNTL1=id%ICNTL(1)
      SOLVE=.TRUE.
      N_OOC=id%N
      IF(allocated(LRLUS_SOLVE))THEN
         DEALLOCATE(LRLUS_SOLVE)
      ENDIF
      IF(allocated(LRLU_SOLVE_T))THEN
         DEALLOCATE(LRLU_SOLVE_T)
      ENDIF
      IF(allocated(LRLU_SOLVE_B))THEN
         DEALLOCATE(LRLU_SOLVE_B)
      ENDIF
      IF(allocated(POSFAC_SOLVE))THEN
         DEALLOCATE(POSFAC_SOLVE)
      ENDIF
      IF(allocated(IDEB_SOLVE_Z))THEN
         DEALLOCATE(IDEB_SOLVE_Z)
      ENDIF
      IF(allocated(PDEB_SOLVE_Z))THEN
         DEALLOCATE(PDEB_SOLVE_Z)
      ENDIF
      IF(allocated(SIZE_SOLVE_Z))THEN
         DEALLOCATE(SIZE_SOLVE_Z)
      ENDIF
      IF(allocated(CURRENT_POS_T))THEN
         DEALLOCATE(CURRENT_POS_T)
      ENDIF
      IF(allocated(CURRENT_POS_B))THEN
         DEALLOCATE(CURRENT_POS_B)
      ENDIF
      IF(allocated(POS_HOLE_T))THEN
         DEALLOCATE(POS_HOLE_T)
      ENDIF
      IF(allocated(POS_HOLE_B))THEN
         DEALLOCATE(POS_HOLE_B)
      ENDIF
      IF(allocated(OOC_STATE_NODE))THEN
         DEALLOCATE(OOC_STATE_NODE)
      ENDIF
      IF(allocated(POS_IN_MEM))THEN
         DEALLOCATE(POS_IN_MEM)
      ENDIF
      IF(allocated(INODE_TO_POS))THEN
         DEALLOCATE(INODE_TO_POS)
      ENDIF
      IF(allocated(SIZE_OF_READ))THEN
         DEALLOCATE(SIZE_OF_READ)
      ENDIF
      IF(allocated(FIRST_POS_IN_READ))THEN
         DEALLOCATE(FIRST_POS_IN_READ)
      ENDIF
      IF(allocated(READ_DEST))THEN
         DEALLOCATE(READ_DEST)
      ENDIF
      IF(allocated(READ_MNG))THEN
         DEALLOCATE(READ_MNG)
      ENDIF
      IF(allocated(REQ_TO_ZONE))THEN
         DEALLOCATE(REQ_TO_ZONE)
      ENDIF
      IF(allocated(REQ_ID))THEN
         DEALLOCATE(REQ_ID)
      ENDIF
      IF(allocated(IO_REQ))THEN
         DEALLOCATE(IO_REQ)
      ENDIF
      IF(associated(KEEP_OOC))THEN
         NULLIFY(KEEP_OOC)
      ENDIF
      IF(associated(STEP_OOC))THEN
         NULLIFY(STEP_OOC)
      ENDIF
      IF(associated(PROCNODE_OOC))THEN
         NULLIFY(PROCNODE_OOC)
      ENDIF
      IF(associated(TOTAL_NB_OOC_NODES))THEN
         NULLIFY(TOTAL_NB_OOC_NODES)
      ENDIF
      IF(associated(SIZE_OF_BLOCK))THEN
         NULLIFY(SIZE_OF_BLOCK)
      ENDIF
      IF(associated(OOC_INODE_SEQUENCE))THEN
         NULLIFY(OOC_VADDR)
      ENDIF
      DIM_ERR_STR_OOC = ERR_STR_OOC_MAX_LEN
      CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_INIT_ERR_STR')
      CALL DAGMG_MUMPS_614(id)
      IF(id%INFO(1).LT.0)THEN
         RETURN
      ENDIF
      STEP_OOC=>id%STEP
      PROCNODE_OOC=>id%PROCNODE_STEPS
      SLAVEF_OOC=id%NSLAVES
      MYID_OOC=id%MYID
      KEEP_OOC => id%KEEP
      SIZE_OF_BLOCK=>id%OOC_SIZE_OF_BLOCK
      OOC_INODE_SEQUENCE=>id%OOC_INODE_SEQUENCE
      OOC_VADDR=>id%OOC_VADDR
      ALLOCATE(IO_REQ(id%KEEP(28)),                                     &
     &     stat=allocok)
      IF (allocok .GT. 0) THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC'
         id%INFO(1) = -13
         id%INFO(2) = id%KEEP(28)
         RETURN
      ENDIF
      DAGMG_MUMPS_ELEMENTARY_DATA_SIZE = id%KEEP(35)
      MAX_NB_NODES_FOR_ZONE=id%OOC_MAX_NB_NODES_FOR_ZONE
      TOTAL_NB_OOC_NODES=>id%OOC_TOTAL_NB_NODES
      CALL DAGMG_MUMPS_711( id%KEEP(204), STRAT_IO_ASYNC,                    &
     & WITH_BUF, LOW_LEVEL_STRAT_IO)
      IF(id%KEEP(107).GT.0)THEN
         SIZE_SOLVE_EMM=max(id%KEEP8(20),                               &
     &        FACT_AREA_SIZE / 5_8)
         SIZE_ZONE_SOLVE=max(SIZE_SOLVE_EMM,                            &
     &        int((dble(FACT_AREA_SIZE)-                                &
     &        dble(SIZE_SOLVE_EMM))/dble(id%KEEP(107)),8))
         SIZE_ZONE_SOLVE=max(SIZE_ZONE_SOLVE,0_8)
         IF(SIZE_ZONE_SOLVE.EQ.SIZE_SOLVE_EMM)THEN
            SIZE_SOLVE_EMM=id%KEEP8(20)
            SIZE_ZONE_SOLVE=int((dble(FACT_AREA_SIZE)-                  &
     &           dble(SIZE_SOLVE_EMM))/dble(id%KEEP(107)),8)
            SIZE_ZONE_SOLVE=max(SIZE_ZONE_SOLVE,0_8)
         ENDIF
      ELSE
         SIZE_ZONE_SOLVE=FACT_AREA_SIZE
         SIZE_SOLVE_EMM=SIZE_ZONE_SOLVE
      ENDIF
      IF(SIZE_SOLVE_EMM.LT.id%KEEP8(20))THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*)MYID_OOC,': More space needed for               &
     & solution step in DAGMG_MUMPS_586'
         id%INFO(1) = -11
         CALL AGMG_MUMPS_731(id%KEEP8(20), id%INFO(2))
      ENDIF
      TMP=MAX_NB_NODES_FOR_ZONE
      CALL AGMG_COPY(TMP,MAX_NB_NODES_FOR_ZONE,1,                                AGMG_MPF_INTEGER, IERR)
      NB_Z=KEEP_OOC(107)+1
      ALLOCATE(POS_IN_MEM(MAX_NB_NODES_FOR_ZONE*NB_Z),                  &
     &     INODE_TO_POS(KEEP_OOC(28)),                                  &
     &     stat=allocok)
      IF (allocok .GT. 0) THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_586'
         id%INFO(1) = -13
         id%INFO(2) = id%KEEP(28)+(MAX_NB_NODES_FOR_ZONE*NB_Z)
         RETURN
      ENDIF
      ALLOCATE(OOC_STATE_NODE(KEEP_OOC(28)),stat=allocok)
      IF (allocok .GT. 0) THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_586'
         id%INFO(1) = -13
         id%INFO(2) = id%KEEP(28)
         RETURN
      ENDIF
      OOC_STATE_NODE(1:KEEP_OOC(28))=0
      INODE_TO_POS=0
      POS_IN_MEM=0
      ALLOCATE(LRLUS_SOLVE(NB_Z), LRLU_SOLVE_T(NB_Z),LRLU_SOLVE_B(NB_Z),&
     &     POSFAC_SOLVE(NB_Z),IDEB_SOLVE_Z(NB_Z),                       &
     &     PDEB_SOLVE_Z(NB_Z),SIZE_SOLVE_Z(NB_Z),                       &
     &     CURRENT_POS_T(NB_Z),CURRENT_POS_B(NB_Z),                     &
     &     POS_HOLE_T(NB_Z),POS_HOLE_B(NB_Z),                           &
     &     stat=allocok)
      IF (allocok .GT. 0) THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC_SOLVE'
         id%INFO(1) = -13
         id%INFO(2) = 9*(NB_Z+1)
         RETURN
      ENDIF
      IERR=0
      CALL AGMG_NOCALL('MUMPS_GET_MAX_NB_REQ_C')
      ALLOCATE(SIZE_OF_READ(MAX_NB_REQ),FIRST_POS_IN_READ(MAX_NB_REQ),  &
     &     READ_DEST(MAX_NB_REQ),READ_MNG(MAX_NB_REQ),                  &
     &     REQ_TO_ZONE(MAX_NB_REQ),REQ_ID(MAX_NB_REQ),stat=allocok)
      SIZE_OF_READ=-9999_8
      FIRST_POS_IN_READ=-9999
      READ_DEST=-9999_8
      READ_MNG=-9999
      REQ_TO_ZONE=-9999
      REQ_ID=-9999
      IF (allocok .GT. 0) THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_INIT_OOC_SOLVE'
         id%INFO(1) = -13
         id%INFO(2) = 6*(NB_Z+1)
         RETURN
      ENDIF
      MIN_SIZE_READ=min(max((1024_8*1024_8)/int(id%KEEP(35),8),         &
     &                       SIZE_ZONE_SOLVE/3_8),                      &
     &                  SIZE_ZONE_SOLVE)
      TMP_SIZE8=1_8
      J=1
      DO I=1,NB_Z-1
         IDEB_SOLVE_Z(I)=TMP_SIZE8
         POSFAC_SOLVE(I)=TMP_SIZE8
         LRLUS_SOLVE(I)=SIZE_ZONE_SOLVE
         LRLU_SOLVE_T(I)=SIZE_ZONE_SOLVE
         LRLU_SOLVE_B(I)=0_8
         SIZE_SOLVE_Z(I)=SIZE_ZONE_SOLVE
         CURRENT_POS_T(I)=J
         CURRENT_POS_B(I)=J
         PDEB_SOLVE_Z(I)=J
         POS_HOLE_T(I)=J
         POS_HOLE_B(I)=J
         J=J+MAX_NB_NODES_FOR_ZONE
         TMP_SIZE8=TMP_SIZE8+SIZE_ZONE_SOLVE
      ENDDO
      IDEB_SOLVE_Z(NB_Z)=TMP_SIZE8
      PDEB_SOLVE_Z(NB_Z)=J
      POSFAC_SOLVE(NB_Z)=TMP_SIZE8
      LRLUS_SOLVE(NB_Z)=SIZE_SOLVE_EMM
      LRLU_SOLVE_T(NB_Z)=SIZE_SOLVE_EMM
      LRLU_SOLVE_B(NB_Z)=0_8
      SIZE_SOLVE_Z(NB_Z)=SIZE_SOLVE_EMM
      CURRENT_POS_T(NB_Z)=J
      CURRENT_POS_B(NB_Z)=J
      POS_HOLE_T(NB_Z)=J
      POS_HOLE_B(NB_Z)=J
      IO_REQ=-77777
      REQ_ACT=0
      OOC_STATE_NODE(1:KEEP_OOC(28))=NOT_IN_MEM
      IF(KEEP_OOC(38).NE.0)THEN
         MASTER_ROOT=AGMG_MUMPS_275(STEP_OOC( KEEP_OOC(38)),                 &
     &        PROCNODE_OOC, SLAVEF_OOC )
         SPECIAL_ROOT_NODE=KEEP_OOC(38)
      ELSEIF(KEEP_OOC(20).NE.0)THEN
         MASTER_ROOT=AGMG_MUMPS_275(STEP_OOC( KEEP_OOC(20)),                 &
     &        PROCNODE_OOC, SLAVEF_OOC )
         SPECIAL_ROOT_NODE=KEEP_OOC(20)
      ELSE
         MASTER_ROOT=-111111
         SPECIAL_ROOT_NODE=-2222222
      ENDIF
      IF ( KEEP_OOC(60).EQ.0 .AND.                                      &
     &     (                                                            &
     &     (KEEP_OOC(38).NE.0 .AND.  id%root%yes)                       &
     &     .OR.                                                         &
     &     (KEEP_OOC(20).NE.0 .AND. MYID_OOC.EQ.MASTER_ROOT))           &
     &     )                                                            &
     &     THEN
        IS_ROOT_SPECIAL = .TRUE.
      ELSE
        IS_ROOT_SPECIAL = .FALSE.
      ENDIF
      NB_ZONE_REQ=0
      SIZE_ZONE_REQ=0_8
      CURRENT_SOLVE_READ_ZONE=0
      NB_CALLED=0
      NB_CALL=0
      SOLVE_STEP=-9999
      RETURN
      END SUBROUTINE DAGMG_MUMPS_586
      SUBROUTINE DAGMG_MUMPS_585(A,LA,PTRFAC,NSTEPS,IERR)
      IMPLICIT NONE
      INTEGER NSTEPS,IERR
      INTEGER(8) :: LA
      DOUBLE PRECISION A(LA)
      INTEGER(8) :: PTRFAC(NSTEPS)
      INTEGER I
      IERR=0
      IF(NB_Z.GT.1)THEN
         IF(STRAT_IO_ASYNC)THEN
            DO I=1,NB_Z-1
               CALL DAGMG_MUMPS_594(A,LA,PTRFAC,NSTEPS,IERR)
               IF(IERR.LT.0)THEN
                  RETURN
               ENDIF
            ENDDO
         ELSE
            CALL DAGMG_MUMPS_594(A,LA,PTRFAC,NSTEPS,IERR)
            IF(IERR.LT.0)THEN
               RETURN
            ENDIF
         ENDIF
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_585
      SUBROUTINE DAGMG_MUMPS_594(A,LA,PTRFAC,NSTEPS,IERR)
      IMPLICIT NONE
      INTEGER NSTEPS,IERR
      INTEGER(8) :: LA
      DOUBLE PRECISION A(LA)
      INTEGER(8) :: PTRFAC(NSTEPS)
      INTEGER ZONE,SIZE,DEST,FLAG
      CALL DAGMG_MUMPS_603(ZONE)
      IERR=0
      CALL DAGMG_MUMPS_611(ZONE,A,LA,PTRFAC,NSTEPS,IERR)
      RETURN
      END SUBROUTINE DAGMG_MUMPS_594
      SUBROUTINE DAGMG_MUMPS_595(DEST,INDICE,SIZE,                           &
     &     ZONE,PTRFAC,NSTEPS,POS_SEQ,NB_NODES,FLAG,IERR)
      IMPLICIT NONE
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
      INTEGER ZONE,NSTEPS,FLAG,POS_SEQ,NB_NODES
      DOUBLE PRECISION DEST
      INTEGER (8) :: INDICE, SIZE, PTRFAC(NSTEPS)
      INTEGER REQUEST,INODE,IERR
      INTEGER ADDR_INT1,ADDR_INT2
      INTEGER TYPE
      INTEGER SIZE_INT1,SIZE_INT2
      TYPE=OOC_SOLVE_TYPE_FCT
      IERR=0
      INODE=OOC_INODE_SEQUENCE(POS_SEQ,OOC_FCT_TYPE)
      CALL AGMG_MUMPS_677(ADDR_INT1,ADDR_INT2,                               &
     &     OOC_VADDR(STEP_OOC(INODE),OOC_FCT_TYPE))
      CALL AGMG_MUMPS_677(SIZE_INT1,SIZE_INT2,                               &
     &     SIZE)
      CALL AGMG_NOCALL('MUMPS_LOW_LEVEL_READ_OOC_C')
      IF(IERR.LT.0)THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         RETURN
      ENDIF
      IF(STRAT_IO_ASYNC)THEN
         CALL DAGMG_MUMPS_597(INODE,SIZE,INDICE,ZONE,                        &
     &        REQUEST,POS_SEQ,NB_NODES,FLAG,PTRFAC,NSTEPS,IERR)
         IF(IERR.LT.0)THEN
            RETURN
         ENDIF
      ELSE
         CALL DAGMG_MUMPS_597(INODE,SIZE,INDICE,ZONE,                        &
     &        REQUEST,POS_SEQ,NB_NODES,FLAG,PTRFAC,NSTEPS,IERR)
         IF(IERR.LT.0)THEN
            RETURN
         ENDIF
         CALL DAGMG_MUMPS_596(IO_REQ(STEP_OOC(INODE)),                       &
     &        PTRFAC,NSTEPS)
         REQ_ACT=REQ_ACT-1
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_595
      SUBROUTINE DAGMG_MUMPS_596(REQUEST,PTRFAC,                             &
     &     NSTEPS)
      IMPLICIT NONE
      INTEGER NSTEPS,REQUEST
      INTEGER (8) :: PTRFAC(NSTEPS)
      INTEGER (8) :: LAST, POS_IN_S, J
      INTEGER ZONE
      INTEGER POS,POS_REQ,I,TMP_NODE,POS_IN_MANAGE
      INTEGER (8) SIZE
      INTEGER STATUS
      LOGICAL DONT_USE
      EXTERNAL AGMG_MUMPS_330,AGMG_MUMPS_275
      INTEGER AGMG_MUMPS_330,AGMG_MUMPS_275
      POS_REQ=mod(REQUEST,MAX_NB_REQ)+1
      SIZE=SIZE_OF_READ(POS_REQ)
      I=FIRST_POS_IN_READ(POS_REQ)
      POS_IN_S=READ_DEST(POS_REQ)
      POS_IN_MANAGE=READ_MNG(POS_REQ)
      ZONE=REQ_TO_ZONE(POS_REQ)
      DONT_USE=.FALSE.
      J=0_8
      DO WHILE((J.LT.SIZE).AND.(I.LE.TOTAL_NB_OOC_NODES(OOC_FCT_TYPE)))
         TMP_NODE=OOC_INODE_SEQUENCE(I,OOC_FCT_TYPE)
         LAST=SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)
         IF(LAST.EQ.0_8)THEN
            I=I+1
            CYCLE
         ENDIF
         IF((INODE_TO_POS(STEP_OOC(TMP_NODE)).NE.0).AND.                &
     &        (INODE_TO_POS(STEP_OOC(TMP_NODE)).LT.                     &
     &        -((N_OOC+1)*NB_Z)))THEN
            DONT_USE=                                                   &
     &           (((MTYPE_OOC.EQ.1).AND.(KEEP_OOC(50).EQ.0).AND.        &
     &           (SOLVE_STEP.EQ.1).AND.                                 &
     &           ((AGMG_MUMPS_330(STEP_OOC(TMP_NODE),PROCNODE_OOC,           &
     &           SLAVEF_OOC).EQ.2).AND.(AGMG_MUMPS_275(                      &
     &           STEP_OOC(TMP_NODE),PROCNODE_OOC,SLAVEF_OOC).NE.        &
     &           MYID_OOC)))                                            &
     &           .OR.                                                   &
     &           ((MTYPE_OOC.NE.1).AND.(KEEP_OOC(50).EQ.0).AND.         &
     &           (SOLVE_STEP.EQ.0).AND.                                 &
     &           ((AGMG_MUMPS_330(STEP_OOC(TMP_NODE),PROCNODE_OOC,           &
     &           SLAVEF_OOC).EQ.2).AND.(AGMG_MUMPS_275(                      &
     &           STEP_OOC(TMP_NODE),PROCNODE_OOC,SLAVEF_OOC).NE.        &
     &           MYID_OOC)))).OR.                                       &
     &           (OOC_STATE_NODE(STEP_OOC(TMP_NODE)).EQ.ALREADY_USED)
            IF(DONT_USE)THEN
               PTRFAC(STEP_OOC(TMP_NODE))=-POS_IN_S
            ELSE
               PTRFAC(STEP_OOC(TMP_NODE))=POS_IN_S
            ENDIF
            IF(abs(PTRFAC(STEP_OOC(TMP_NODE))).LT.                      &
     &           IDEB_SOLVE_Z(ZONE))THEN
               WRITE(*,*)MYID_OOC,': Inernal error (42) in OOC ',       &
     &              PTRFAC(STEP_OOC(TMP_NODE)),IDEB_SOLVE_Z(ZONE)
               CALL AGMG_MUMPS_ABORT()
            ENDIF
            IF(abs(PTRFAC(STEP_OOC(TMP_NODE))).GT.                      &
     &           (IDEB_SOLVE_Z(ZONE)+SIZE_SOLVE_Z(ZONE)-1_8))THEN
               WRITE(*,*)MYID_OOC,': Inernal error (43) in OOC '
               CALL AGMG_MUMPS_ABORT()
            ENDIF
            IF(DONT_USE)THEN
               POS_IN_MEM(POS_IN_MANAGE)=-TMP_NODE
               INODE_TO_POS(STEP_OOC(TMP_NODE))=-POS_IN_MANAGE
               IF(OOC_STATE_NODE(STEP_OOC(TMP_NODE)).NE.                &
     &              ALREADY_USED)THEN
                  OOC_STATE_NODE(STEP_OOC(TMP_NODE))=USED_NOT_PERMUTED
               ENDIF
               LRLUS_SOLVE(ZONE)=LRLUS_SOLVE(ZONE)+LAST
            ELSE
               POS_IN_MEM(POS_IN_MANAGE)=TMP_NODE
               INODE_TO_POS(STEP_OOC(TMP_NODE))=POS_IN_MANAGE
               OOC_STATE_NODE(STEP_OOC(TMP_NODE))=NOT_USED
            ENDIF
            IO_REQ(STEP_OOC(TMP_NODE))=-7777
         ELSE
            POS_IN_MEM(POS_IN_MANAGE)=0
         ENDIF
         POS_IN_S=POS_IN_S+LAST
         POS_IN_MANAGE=POS_IN_MANAGE+1
         J=J+LAST
         I=I+1
      ENDDO
      SIZE_OF_READ(POS_REQ)=-9999_8
      FIRST_POS_IN_READ(POS_REQ)=-9999
      READ_DEST(POS_REQ)=-9999_8
      READ_MNG(POS_REQ)=-9999
      REQ_TO_ZONE(POS_REQ)=-9999
      REQ_ID(POS_REQ)=-9999
      RETURN
      END SUBROUTINE DAGMG_MUMPS_596
      SUBROUTINE DAGMG_MUMPS_597(INODE,SIZE,DEST,ZONE,                       &
     &     REQUEST,POS_SEQ,NB_NODES,FLAG,PTRFAC,NSTEPS,IERR)
      IMPLICIT NONE
      INTEGER INODE,ZONE,REQUEST,FLAG,POS_SEQ,NB_NODES,NSTEPS
      INTEGER(8) :: SIZE
      INTEGER(8) :: PTRFAC(NSTEPS)
      INTEGER(8) :: DEST, LOCAL_DEST, J8
      INTEGER I,TMP_NUM_FILE,TMP_NODE,LOC_I,POS_REQ,NB
      INTEGER(8)::LAST
      INTEGER, intent(out) :: IERR
      IERR=0
      IF(CUR_POS_SEQUENCE.GT.TOTAL_NB_OOC_NODES(OOC_FCT_TYPE))THEN
         RETURN
      ENDIF
      NB=0
      LOCAL_DEST=DEST
      I=POS_SEQ
      POS_REQ=mod(REQUEST,MAX_NB_REQ)+1
      IF(REQ_ID(POS_REQ).NE.-9999)THEN
         CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
         IF(IERR.LT.0)THEN
            IF (ICNTL1.GT.0)                                            &
     &      WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
            RETURN
         ENDIF
         CALL DAGMG_MUMPS_596(REQUEST,PTRFAC,NSTEPS)
         REQ_ACT=REQ_ACT-1
      ENDIF
      SIZE_OF_READ(POS_REQ)=SIZE
      FIRST_POS_IN_READ(POS_REQ)=I
      READ_DEST(POS_REQ)=DEST
      IF(FLAG.EQ.0)THEN
         READ_MNG(POS_REQ)=CURRENT_POS_B(ZONE)-NB_NODES+1
      ELSEIF(FLAG.EQ.1)THEN
         READ_MNG(POS_REQ)=CURRENT_POS_T(ZONE)
      ENDIF
      REQ_TO_ZONE(POS_REQ)=ZONE
      REQ_ID(POS_REQ)=REQUEST
      J8=0_8
      IF(FLAG.EQ.0)THEN
         LOC_I=CURRENT_POS_B(ZONE)-NB_NODES+1
      ENDIF
      DO WHILE((J8.LT.SIZE).AND.(I.LE.TOTAL_NB_OOC_NODES(OOC_FCT_TYPE)))
         TMP_NODE=OOC_INODE_SEQUENCE(I,OOC_FCT_TYPE)
         LAST=SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)
         IF(LAST.EQ.0_8)THEN
            INODE_TO_POS(STEP_OOC(TMP_NODE))=1
            OOC_STATE_NODE(STEP_OOC(TMP_NODE))=NOT_USED
            I=I+1
            CYCLE
         ENDIF
         IF((IO_REQ(STEP_OOC(TMP_NODE)).GE.0).OR.                       &
     &        (INODE_TO_POS(STEP_OOC(TMP_NODE)).NE.0))THEN
            IF(FLAG.EQ.1)THEN
               POS_IN_MEM(CURRENT_POS_T(ZONE))=0
            ELSEIF(FLAG.EQ.0)THEN
               POS_IN_MEM(CURRENT_POS_B(ZONE))=0
            ENDIF
         ELSE
            IO_REQ(STEP_OOC(TMP_NODE))=REQUEST
            LRLUS_SOLVE(ZONE)=LRLUS_SOLVE(ZONE)-LAST
            IF(FLAG.EQ.1)THEN
               IF(POSFAC_SOLVE(ZONE).EQ.IDEB_SOLVE_Z(ZONE))THEN
                  POS_HOLE_B(ZONE)=-9999
                  CURRENT_POS_B(ZONE)=-9999
                  LRLU_SOLVE_B(ZONE)=0_8
               ENDIF
               POSFAC_SOLVE(ZONE)=POSFAC_SOLVE(ZONE)+LAST
               LRLU_SOLVE_T(ZONE)=LRLU_SOLVE_T(ZONE)-LAST
               POS_IN_MEM(CURRENT_POS_T(ZONE))=-TMP_NODE-               &
     &              ((N_OOC+1)*NB_Z)
               INODE_TO_POS(STEP_OOC(TMP_NODE))=-CURRENT_POS_T(ZONE)-   &
     &              ((N_OOC+1)*NB_Z)
               OOC_STATE_NODE(STEP_OOC(TMP_NODE))=BEING_READ
               PTRFAC(STEP_OOC(TMP_NODE))=-LOCAL_DEST
               LOCAL_DEST=LOCAL_DEST+SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),  &
     &              OOC_FCT_TYPE)
            ELSEIF(FLAG.EQ.0)THEN
               LRLU_SOLVE_B(ZONE)=LRLU_SOLVE_B(ZONE)-LAST
               POS_IN_MEM(LOC_I)=-TMP_NODE-((N_OOC+1)*NB_Z)
               IF(LOC_I.EQ.POS_HOLE_T(ZONE))THEN
                  IF(POS_HOLE_T(ZONE).LT.CURRENT_POS_T(ZONE))THEN
                     POS_HOLE_T(ZONE)=POS_HOLE_T(ZONE)+1
                  ENDIF
               ENDIF
               INODE_TO_POS(STEP_OOC(TMP_NODE))=-LOC_I-((N_OOC+1)*NB_Z)
               OOC_STATE_NODE(STEP_OOC(TMP_NODE))=BEING_READ
               PTRFAC(STEP_OOC(TMP_NODE))=-LOCAL_DEST
               LOCAL_DEST=LOCAL_DEST+SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),  &
     &              OOC_FCT_TYPE)
            ELSE
             WRITE(*,*)MYID_OOC,': Internal error (39) in OOC ',        &
     &                 ' Invalid Flag Value in ',                       &
     &                 ' DMUMPS_597',FLAG
               CALL AGMG_MUMPS_ABORT()
            ENDIF
         ENDIF
         IF(POS_IN_MEM(CURRENT_POS_T(ZONE)).NE.0)THEN
            IF(POS_IN_MEM(CURRENT_POS_T(ZONE)).EQ.                      &
     &           POS_IN_MEM(PDEB_SOLVE_Z(ZONE)))THEN
               IF(CURRENT_POS_T(ZONE).NE.PDEB_SOLVE_Z(ZONE))THEN
             WRITE(*,*)MYID_OOC,': Internal error (40) in OOC ',        &
     &                      CURRENT_POS_T(ZONE),                        &
     &                 PDEB_SOLVE_Z(ZONE),                              &
     &                 POS_IN_MEM(CURRENT_POS_T(ZONE)),                 &
     &                 POS_IN_MEM(PDEB_SOLVE_Z(ZONE))
                  CALL AGMG_MUMPS_ABORT()
               ENDIF
            ENDIF
         ENDIF
         J8=J8+LAST
         IF(LRLUS_SOLVE(ZONE).LT.0_8)THEN
             WRITE(*,*)MYID_OOC,': Internal error (41) in OOC ',        &
     &           ' LRLUS_SOLVE must be (1) > 0',                        &
     &           LRLUS_SOLVE(ZONE)
            CALL AGMG_MUMPS_ABORT()
         ENDIF
         I=I+1
         IF(FLAG.EQ.1)THEN
            CURRENT_POS_T(ZONE)=CURRENT_POS_T(ZONE)+1
            IF(CURRENT_POS_T(ZONE).GT.                                  &
     &           MAX_NB_NODES_FOR_ZONE+PDEB_SOLVE_Z(ZONE))THEN
               WRITE(*,*)MYID_OOC,': Internal error (1) in OOC '
               CALL AGMG_MUMPS_ABORT()
            ENDIF
            POS_HOLE_T(ZONE)=CURRENT_POS_T(ZONE)
         ELSEIF(FLAG.EQ.0)THEN
            IF(POS_HOLE_B(ZONE).LT.PDEB_SOLVE_Z(ZONE))THEN
               WRITE(*,*)MYID_OOC,': Internal error (2) in OOC ',       &
     &              POS_HOLE_B(ZONE),LOC_I
               CALL AGMG_MUMPS_ABORT()
            ENDIF
            CURRENT_POS_B(ZONE)=CURRENT_POS_B(ZONE)-1
            POS_HOLE_B(ZONE)=CURRENT_POS_B(ZONE)
            IF(POS_HOLE_B(ZONE).LT.PDEB_SOLVE_Z(ZONE))THEN
               POS_HOLE_B(ZONE)=-9999
               LRLU_SOLVE_B(ZONE)=0_8
            ENDIF
         ELSE
            WRITE(*,*)MYID_OOC,': Internal error (3) in OOC ',          &
     &       ' Invalid Flag Value in ',                                 &
     &       ' DMUMPS_597',FLAG
            CALL AGMG_MUMPS_ABORT()
         ENDIF
         IF(FLAG.EQ.0)THEN
            LOC_I=LOC_I+1
         ENDIF
         NB=NB+1
      ENDDO
      IF(NB.NE.NB_NODES)THEN
         WRITE(*,*)MYID_OOC,': Internal error (4) in OOC ',             &
     &        ' DMUMPS_597 ',NB,NB_NODES
      ENDIF
      IF(SOLVE_STEP.EQ.0)THEN
         CUR_POS_SEQUENCE=I
      ELSE
         CUR_POS_SEQUENCE=POS_SEQ-1
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_597
      SUBROUTINE DAGMG_MUMPS_598(INODE,PTRFAC,NSTEPS,A,                      &
     &     LA,FLAG,IERR)
      IMPLICIT NONE
      INTEGER(8) :: LA
      INTEGER, intent(out):: IERR
      DOUBLE PRECISION A(LA)
      INTEGER INODE,NSTEPS
      INTEGER(8) :: PTRFAC(NSTEPS)
      LOGICAL FLAG
      INTEGER(8) FREE_SIZE, FREE_HOLE_POS
      INTEGER TMP,TMP_NODE,I,ZONE,J, FREE_HOLE_FLAG
      LOGICAL CONTIGUOUS
      INTEGER WHICH
      INTEGER(8) :: DUMMY_SIZE
      DUMMY_SIZE=1_8
      IERR = 0
      WHICH=-1
      IF(INODE_TO_POS(STEP_OOC(INODE)).LE.0)THEN
         WRITE(*,*)MYID_OOC,': Internal error (5) in OOC ',             &
     &            ' Problem in DMUMPS_598',                             &
     &        INODE, STEP_OOC(INODE), INODE_TO_POS(STEP_OOC(INODE))
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF(SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE).EQ.0_8)THEN
         INODE_TO_POS(STEP_OOC(INODE))=0
         OOC_STATE_NODE(STEP_OOC(INODE))=ALREADY_USED
         RETURN
      ENDIF
      CALL DAGMG_MUMPS_600(INODE,ZONE,PTRFAC,NSTEPS)
      TMP=INODE_TO_POS(STEP_OOC(INODE))
      INODE_TO_POS(STEP_OOC(INODE))=-TMP
      POS_IN_MEM(TMP)=-INODE
      PTRFAC(STEP_OOC(INODE))=-PTRFAC(STEP_OOC(INODE))
      IF(OOC_STATE_NODE(STEP_OOC(INODE)).NE.PERMUTED)THEN
         WRITE(*,*)MYID_OOC,': INTERNAL ERROR (53) in OOC',INODE,       &
     &        OOC_STATE_NODE(STEP_OOC(INODE))
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      OOC_STATE_NODE(STEP_OOC(INODE))=USED
      LRLUS_SOLVE(ZONE)=LRLUS_SOLVE(ZONE)+                              &
     &        SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)
      IF(LRLUS_SOLVE(ZONE).LT.0_8)THEN
         WRITE(*,*)MYID_OOC,': Internal error (6) in OOC ',             &
     &        ': LRLUS_SOLVE must be (2) > 0'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF(ZONE.EQ.NB_Z)THEN
         IF(INODE.NE.SPECIAL_ROOT_NODE)THEN
            CALL DAGMG_MUMPS_608(A,FACT_AREA_SIZE,                           &
     &           DUMMY_SIZE,PTRFAC,KEEP_OOC(28),ZONE,IERR)
         ENDIF
      ELSE
         FREE_HOLE_FLAG=0
         IF(SOLVE_STEP.EQ.0)THEN
            IF(TMP.GT.POS_HOLE_B(ZONE))THEN
               WHICH=0
            ELSEIF(TMP.LT.POS_HOLE_T(ZONE))THEN
               WHICH=1
            ENDIF
         ELSEIF(SOLVE_STEP.EQ.1)THEN
            IF(TMP.LT.POS_HOLE_T(ZONE))THEN
               WHICH=1
            ELSEIF(TMP.GT.POS_HOLE_B(ZONE))THEN
               WHICH=0
            ENDIF
         ENDIF
         IF(WHICH.EQ.1)THEN
            J=max(PDEB_SOLVE_Z(ZONE),POS_HOLE_T(ZONE))
            J=min(J,PDEB_SOLVE_Z(ZONE)+MAX_NB_NODES_FOR_ZONE-1)
            FREE_SIZE=0_8
            DO I=J,TMP,-1
               IF((POS_IN_MEM(I).LT.0).AND.(POS_IN_MEM(I).GT.           &
     &              -(N_OOC+1)*NB_Z))THEN
                  TMP_NODE=-POS_IN_MEM(I)
                  FREE_SIZE=FREE_SIZE+SIZE_OF_BLOCK(STEP_OOC(TMP_NODE), &
     &                 OOC_FCT_TYPE)
               ELSEIF(POS_IN_MEM(I).NE.0)THEN
                  GOTO 666
               ENDIF
            ENDDO
            POS_HOLE_T(ZONE)=TMP
  666       CONTINUE
         ELSEIF(WHICH.EQ.0)THEN
            J=max(PDEB_SOLVE_Z(ZONE),POS_HOLE_B(ZONE))
            J=min(J,PDEB_SOLVE_Z(ZONE)+MAX_NB_NODES_FOR_ZONE-1)
            FREE_SIZE=0_8
            DO I=J,TMP
               IF((POS_IN_MEM(I).LT.0).AND.(POS_IN_MEM(I).GT.           &
     &              -(N_OOC+1)*NB_Z))THEN
                  TMP_NODE=-POS_IN_MEM(I)
                  FREE_SIZE=FREE_SIZE+SIZE_OF_BLOCK(STEP_OOC(TMP_NODE), &
     &                 OOC_FCT_TYPE)
               ELSEIF(POS_IN_MEM(I).NE.0)THEN
                  IF(J.EQ.PDEB_SOLVE_Z(ZONE))THEN
                     POS_HOLE_B(ZONE)=-9999
                     LRLU_SOLVE_B(ZONE)=0_8
                     CURRENT_POS_B(ZONE)=-9999
                  ENDIF
                  GOTO 777
               ENDIF
            ENDDO
            POS_HOLE_B(ZONE)=TMP
  777       CONTINUE
         ENDIF
  222 CONTINUE
      IERR=0
      ENDIF
      IF((NB_Z.GT.1).AND.FLAG)THEN
         CALL DAGMG_MUMPS_601(ZONE)
         IF((LRLUS_SOLVE(ZONE).GE.MIN_SIZE_READ).OR.                    &
     &        (LRLUS_SOLVE(ZONE).GE.                                    &
     &        int(0.3D0*dble(SIZE_SOLVE_Z(ZONE)),8)))THEN
            CALL DAGMG_MUMPS_594(A,LA,PTRFAC,NSTEPS,IERR)
            IF(IERR.LT.0)THEN
               RETURN
            ENDIF
         ELSE
            CALL DAGMG_MUMPS_603(ZONE)
         ENDIF
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_598
      FUNCTION DAGMG_MUMPS_726(INODE,PTRFAC,NSTEPS,A,LA,                     &
     &     IERR)
      IMPLICIT NONE
      INTEGER INODE,NSTEPS
      INTEGER(8) :: LA
      INTEGER, INTENT(out)::IERR
      DOUBLE PRECISION A(LA)
      INTEGER (8) :: PTRFAC(NSTEPS)
      INTEGER DAGMG_MUMPS_726
      INTEGER ZONE
      IERR=0
      IF(INODE_TO_POS(STEP_OOC(INODE)).GT.0)THEN
         IF(OOC_STATE_NODE(STEP_OOC(INODE)).EQ.PERMUTED)THEN
            DAGMG_MUMPS_726=OOC_NODE_PERMUTED
         ELSE
            DAGMG_MUMPS_726=OOC_NODE_NOT_PERMUTED
         ENDIF
         IF(.NOT.DAGMG_MUMPS_727())THEN
            IF(OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE,OOC_FCT_TYPE)        &
     &           .EQ.INODE)THEN
               IF(SOLVE_STEP.EQ.0)THEN
                  CUR_POS_SEQUENCE=CUR_POS_SEQUENCE+1
               ELSEIF(SOLVE_STEP.EQ.1)THEN
                  CUR_POS_SEQUENCE=CUR_POS_SEQUENCE-1
               ENDIF
               CALL DAGMG_MUMPS_728()
            ENDIF
         ENDIF
      ELSEIF(INODE_TO_POS(STEP_OOC(INODE)).LT.0)THEN
         IF(INODE_TO_POS(STEP_OOC(INODE)).LT.-((N_OOC+1)*NB_Z))THEN
            CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
            IF(IERR.LT.0)THEN
               IF (ICNTL1.GT.0)                                         &
     &         WRITE(ICNTL1,*)MYID_OOC,': Internal error (7) in OOC ',  &
     &                   ERR_STR_OOC(1:DIM_ERR_STR_OOC)
               RETURN
            ENDIF
            CALL DAGMG_MUMPS_596(IO_REQ(STEP_OOC(INODE)),                    &
     &           PTRFAC,NSTEPS)
            REQ_ACT=REQ_ACT-1
         ELSE
            CALL DAGMG_MUMPS_599(INODE,PTRFAC,NSTEPS)
            IF(.NOT.DAGMG_MUMPS_727())THEN
               IF(OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE,OOC_FCT_TYPE).EQ. &
     &              INODE)THEN
                  IF(SOLVE_STEP.EQ.0)THEN
                     CUR_POS_SEQUENCE=CUR_POS_SEQUENCE+1
                  ELSEIF(SOLVE_STEP.EQ.1)THEN
                     CUR_POS_SEQUENCE=CUR_POS_SEQUENCE-1
                  ENDIF
                  CALL DAGMG_MUMPS_728()
               ENDIF
            ENDIF
         ENDIF
         IF(OOC_STATE_NODE(STEP_OOC(INODE)).EQ.PERMUTED)THEN
            DAGMG_MUMPS_726=OOC_NODE_PERMUTED
         ELSE
            DAGMG_MUMPS_726=OOC_NODE_NOT_PERMUTED
         ENDIF
      ELSE
         DAGMG_MUMPS_726=OOC_NODE_NOT_IN_MEM
      ENDIF
      RETURN
      END FUNCTION DAGMG_MUMPS_726
      SUBROUTINE DAGMG_MUMPS_682(INODE)
      IMPLICIT NONE
      INTEGER INODE
      IF(OOC_STATE_NODE(STEP_OOC(INODE)).NE.NOT_USED)THEN
         WRITE(*,*)MYID_OOC,': INTERNAL ERROR (51) in OOC',INODE,       &
     &        OOC_STATE_NODE(STEP_OOC(INODE))
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      OOC_STATE_NODE(STEP_OOC(INODE))=PERMUTED
      END SUBROUTINE DAGMG_MUMPS_682
      SUBROUTINE DAGMG_MUMPS_599(INODE,PTRFAC,NSTEPS)
      IMPLICIT NONE
      INTEGER INODE,NSTEPS
      INTEGER (8) :: PTRFAC(NSTEPS)
      INTEGER ZONE
      INODE_TO_POS(STEP_OOC(INODE))=-INODE_TO_POS(STEP_OOC(INODE))
      POS_IN_MEM(INODE_TO_POS(STEP_OOC(INODE)))=                        &
     &     -POS_IN_MEM(INODE_TO_POS(STEP_OOC(INODE)))
      PTRFAC(STEP_OOC(INODE))=-PTRFAC(STEP_OOC(INODE))
      IF(OOC_STATE_NODE(STEP_OOC(INODE)).EQ.USED_NOT_PERMUTED)THEN
         OOC_STATE_NODE(STEP_OOC(INODE))=NOT_USED
      ELSEIF(OOC_STATE_NODE(STEP_OOC(INODE)).EQ.USED)THEN
         OOC_STATE_NODE(STEP_OOC(INODE))=PERMUTED
      ELSE
         WRITE(*,*)MYID_OOC,': Internal error (52) in OOC',INODE,       &
     &        OOC_STATE_NODE(STEP_OOC(INODE)),                          &
     &        INODE_TO_POS(STEP_OOC(INODE))
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      CALL DAGMG_MUMPS_610(PTRFAC(STEP_OOC(INODE)),ZONE)
      IF(INODE_TO_POS(STEP_OOC(INODE)).LE.POS_HOLE_B(ZONE))THEN
         IF(INODE_TO_POS(STEP_OOC(INODE)).GT.                           &
     &        PDEB_SOLVE_Z(ZONE))THEN
            POS_HOLE_B(ZONE)=                                           &
     &           INODE_TO_POS(STEP_OOC(INODE))-1
         ELSE
            CURRENT_POS_B(ZONE)=-9999
            POS_HOLE_B(ZONE)=-9999
            LRLU_SOLVE_B(ZONE)=0_8
         ENDIF
      ENDIF
      IF(INODE_TO_POS(STEP_OOC(INODE)).GE.POS_HOLE_T(ZONE))THEN
         IF(INODE_TO_POS(STEP_OOC(INODE)).LT.                           &
     &        CURRENT_POS_T(ZONE)-1)THEN
            POS_HOLE_T(ZONE)=INODE_TO_POS(STEP_OOC(INODE))+1
         ELSE
            POS_HOLE_T(ZONE)=CURRENT_POS_T(ZONE)
         ENDIF
      ENDIF
      CALL DAGMG_MUMPS_609(INODE,PTRFAC,NSTEPS,1)
      END SUBROUTINE DAGMG_MUMPS_599
      SUBROUTINE DAGMG_MUMPS_600(INODE,ZONE,PTRFAC,NSTEPS)
      IMPLICIT NONE
      INTEGER INODE,ZONE,NSTEPS
      INTEGER (8) :: PTRFAC(NSTEPS)
      ZONE=1
      DO WHILE (ZONE.LE.NB_Z)
         IF(PTRFAC(STEP_OOC(INODE)).LT.IDEB_SOLVE_Z(ZONE))THEN
            ZONE=ZONE-1
            EXIT
         ENDIF
         ZONE=ZONE+1
      ENDDO
      IF(ZONE.EQ.NB_Z+1)THEN
         ZONE=ZONE-1
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_600
      SUBROUTINE DAGMG_MUMPS_601(ZONE)
      IMPLICIT NONE
      INTEGER ZONE
      ZONE=mod((CURRENT_SOLVE_READ_ZONE+1),NB_Z-1)+1
      END SUBROUTINE DAGMG_MUMPS_601
      SUBROUTINE DAGMG_MUMPS_603(ZONE)
      IMPLICIT NONE
      INTEGER ZONE
      IF(NB_Z.GT.1)THEN
         CURRENT_SOLVE_READ_ZONE=mod((CURRENT_SOLVE_READ_ZONE+1),NB_Z-1)
         ZONE=CURRENT_SOLVE_READ_ZONE+1
      ELSE
         ZONE=NB_Z
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_603
      SUBROUTINE DAGMG_MUMPS_578(INODE,PTRFAC,                               &
     &     KEEP,KEEP8,                                                  &
     &     A,IERR)
      IMPLICIT NONE
      INTEGER INODE,KEEP(500)
      INTEGER, intent(out)::IERR
      INTEGER*8 KEEP8(150)
      INTEGER(8) :: PTRFAC(KEEP(28))
      DOUBLE PRECISION A(FACT_AREA_SIZE)
      INTEGER(8) :: REQUESTED_SIZE
      INTEGER ZONE,IFLAG
      LOGICAL FLAG
      IERR=0
      IFLAG=0
      IF(SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)                    &
     &     .EQ.0_8)THEN
         INODE_TO_POS(STEP_OOC(INODE))=1
         OOC_STATE_NODE(STEP_OOC(INODE))=NOT_USED
         PTRFAC(STEP_OOC(INODE))=1
         RETURN
      ENDIF
      REQUESTED_SIZE=SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)
      ZONE=NB_Z
      IF(CURRENT_POS_T(ZONE).GT.                                        &
     &     (PDEB_SOLVE_Z(ZONE)+MAX_NB_NODES_FOR_ZONE-1))THEN
         CALL DAGMG_MUMPS_608(A,FACT_AREA_SIZE,                              &
     &        REQUESTED_SIZE,PTRFAC,KEEP(28),ZONE,IERR)
         IF(IERR.LT.0)THEN
            RETURN
         ENDIF
      ENDIF
      IF((LRLU_SOLVE_T(ZONE).GT.SIZE_OF_BLOCK(STEP_OOC(INODE),          &
     &     OOC_FCT_TYPE)).AND.                                          &
     &     (CURRENT_POS_T(ZONE).LE.                                     &
     &     (PDEB_SOLVE_Z(ZONE)+MAX_NB_NODES_FOR_ZONE-1)))THEN
         CALL DAGMG_MUMPS_606(INODE,PTRFAC,                                  &
     &        KEEP,KEEP8,A,ZONE)
      ELSEIF(LRLU_SOLVE_B(ZONE).GT.SIZE_OF_BLOCK(STEP_OOC(INODE),       &
     &        OOC_FCT_TYPE).AND.                                        &
     &        (CURRENT_POS_B(ZONE).GT.0))THEN
         CALL DAGMG_MUMPS_607(INODE,PTRFAC,                                  &
     &        KEEP,KEEP8,A,ZONE)
      ELSE
         IF(DAGMG_MUMPS_579(INODE,ZONE))THEN
            IF(SOLVE_STEP.EQ.0)THEN
               CALL DAGMG_MUMPS_604(A,FACT_AREA_SIZE,                        &
     &              REQUESTED_SIZE,PTRFAC,                              &
     &              KEEP(28),ZONE,IFLAG,IERR)
               IF(IERR.LT.0)THEN
                  RETURN
               ENDIF
               IF(IFLAG.EQ.1)THEN
                  CALL DAGMG_MUMPS_606(INODE,PTRFAC,                         &
     &                 KEEP,KEEP8,A,ZONE)
               ELSEIF(IFLAG.EQ.0)THEN
                  CALL DAGMG_MUMPS_605(A,FACT_AREA_SIZE,                     &
     &                 REQUESTED_SIZE,PTRFAC,                           &
     &                 KEEP(28),ZONE,IFLAG,IERR)
                  IF(IERR.LT.0)THEN
                     RETURN
                  ENDIF
                  IF(IFLAG.EQ.1)THEN
                     CALL DAGMG_MUMPS_607(INODE,PTRFAC,                      &
     &                    KEEP,KEEP8,A,ZONE)
                  ENDIF
               ENDIF
            ELSE
               CALL DAGMG_MUMPS_605(A,FACT_AREA_SIZE,                        &
     &              REQUESTED_SIZE,PTRFAC,                              &
     &              KEEP(28),ZONE,IFLAG,IERR)
               IF(IERR.LT.0)THEN
                  RETURN
               ENDIF
               IF(IFLAG.EQ.1)THEN
                  CALL DAGMG_MUMPS_607(INODE,PTRFAC,                         &
     &                 KEEP,KEEP8,A,ZONE)
               ELSEIF(IFLAG.EQ.0)THEN
                  CALL DAGMG_MUMPS_604(A,FACT_AREA_SIZE,                     &
     &                 REQUESTED_SIZE,PTRFAC,                           &
     &                 KEEP(28),ZONE,IFLAG,IERR)
                  IF(IERR.LT.0)THEN
                     RETURN
                  ENDIF
                  IF(IFLAG.EQ.1)THEN
                     CALL DAGMG_MUMPS_606(INODE,PTRFAC,                      &
     &                    KEEP,KEEP8,A,ZONE)
                  ENDIF
               ENDIF
            ENDIF
            IF(IFLAG.EQ.0)THEN
               CALL DAGMG_MUMPS_608(A,FACT_AREA_SIZE,                        &
     &              REQUESTED_SIZE,PTRFAC,KEEP(28),ZONE,IERR)
               IF(IERR.LT.0)THEN
                  RETURN
               ENDIF
               CALL DAGMG_MUMPS_606(INODE,PTRFAC,                            &
     &              KEEP,KEEP8,A,ZONE)
            ENDIF
         ELSE
            WRITE(*,*)MYID_OOC,': Internal error (8) in OOC ',          &
     &                         ' Not enough space for Solve',INODE,     &
     &           SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE),           &
     &           LRLUS_SOLVE(ZONE)
            CALL AGMG_MUMPS_ABORT()
         ENDIF
      ENDIF
      IF(LRLUS_SOLVE(ZONE).LT.0_8)THEN
         WRITE(*,*)MYID_OOC,': Internal error (9) in OOC ',             &
     &                      ' LRLUS_SOLVE must be (3) > 0'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_578
      SUBROUTINE DAGMG_MUMPS_604(A,LA,REQUESTED_SIZE,PTRFAC,                 &
     &     NSTEPS,ZONE,FLAG,IERR)
      IMPLICIT NONE
      INTEGER NSTEPS,ZONE,FLAG
      INTEGER(8) :: REQUESTED_SIZE, LA
      INTEGER(8) :: PTRFAC(NSTEPS)
      INTEGER(8) :: FREE_SIZE, FREE_HOLE, FREE_HOLE_POS
      DOUBLE PRECISION A(LA)
      INTEGER I,TMP_NODE,FREE_HOLE_FLAG, J
      INTEGER, intent(out)::IERR
      LOGICAL FLAG_HOLE
      IERR=0
      FLAG=0
      IF(LRLU_SOLVE_T(ZONE).EQ.SIZE_SOLVE_Z(ZONE).AND.                  &
     &     (.NOT.(CURRENT_POS_T(ZONE)                                   &
     &     .GT.PDEB_SOLVE_Z(ZONE)+MAX_NB_NODES_FOR_ZONE-1)))THEN
         GOTO 50
      ENDIF
      J=max(POS_HOLE_B(ZONE),PDEB_SOLVE_Z(ZONE))
      J=min(J,PDEB_SOLVE_Z(ZONE)+MAX_NB_NODES_FOR_ZONE-1)
      DO I=POS_HOLE_T(ZONE)-1,J,-1
         IF((POS_IN_MEM(I).LT.0).AND.(POS_IN_MEM(I).GT.                 &
     &        -(N_OOC+1)*NB_Z))THEN
            TMP_NODE=-POS_IN_MEM(I)
         ELSEIF(POS_IN_MEM(I).NE.0)THEN
            EXIT
         ENDIF
      ENDDO
      POS_HOLE_T(ZONE)=I+1
      IF((POS_HOLE_T(ZONE).EQ.PDEB_SOLVE_Z(ZONE)).OR.                   &
     &     (POS_HOLE_T(ZONE).LE.POS_HOLE_B(ZONE)).OR.                   &
     &     (POS_HOLE_T(ZONE).EQ.POS_HOLE_B(ZONE)+1))THEN
         CURRENT_POS_B(ZONE)=-9999
         POS_HOLE_B(ZONE)=-9999
         LRLU_SOLVE_B(ZONE)=0_8
         POS_HOLE_T(ZONE)=PDEB_SOLVE_Z(ZONE)
      ENDIF
      FREE_HOLE=0_8
      FREE_SIZE=0_8
      FREE_HOLE_FLAG=0
      FREE_HOLE_POS=POSFAC_SOLVE(ZONE)
      DO I=CURRENT_POS_T(ZONE)-1,POS_HOLE_T(ZONE),-1
         IF((POS_IN_MEM(I).LT.0).AND.(POS_IN_MEM(I).GT.                 &
     &        -(N_OOC+1)*NB_Z))THEN
            TMP_NODE=-POS_IN_MEM(I)
            IF(FREE_HOLE_FLAG.EQ.1)THEN
               FREE_HOLE=FREE_HOLE_POS-                                 &
     &              (abs(PTRFAC(STEP_OOC(TMP_NODE)))+                   &
     &              SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE))
               FREE_HOLE_FLAG=0
               FREE_SIZE=FREE_SIZE+FREE_HOLE
            ENDIF
            FREE_HOLE_POS=abs(PTRFAC(STEP_OOC(TMP_NODE)))
            PTRFAC(STEP_OOC(TMP_NODE))=-777777_8
            INODE_TO_POS(STEP_OOC(TMP_NODE))=0
            OOC_STATE_NODE(STEP_OOC(TMP_NODE))=ALREADY_USED
            POS_IN_MEM(I)=0
            FREE_SIZE=FREE_SIZE+SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),       &
     &           OOC_FCT_TYPE)
         ELSEIF(POS_IN_MEM(I).EQ.0)THEN
            FREE_HOLE_FLAG=1
         ELSEIF(POS_IN_MEM(I).NE.0)THEN
            WRITE(*,*)MYID_OOC,': Internal error (10) in OOC ',         &
     &            ' DMUMPS_604',                                        &
     &           CURRENT_POS_T(ZONE)-1,POS_HOLE_T(ZONE),I
            CALL AGMG_MUMPS_ABORT()
         ENDIF
      ENDDO
      IF(POS_HOLE_T(ZONE).EQ.PDEB_SOLVE_Z(ZONE))THEN
         IF(FREE_HOLE_FLAG.EQ.0)THEN
            FREE_HOLE_FLAG=1
         ENDIF
      ENDIF
      IF(FREE_HOLE_FLAG.EQ.1)THEN
         IF(POS_HOLE_T(ZONE)-1.GT.PDEB_SOLVE_Z(ZONE))THEN
            I=POS_HOLE_T(ZONE)-1
            TMP_NODE=abs(POS_IN_MEM(I))
            IF(TMP_NODE.GT.(N_OOC+1)*NB_Z)THEN
               TMP_NODE=TMP_NODE-(N_OOC+1)*NB_Z
               CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
               IF(IERR.LT.0)THEN
                  WRITE(*,*)MYID_OOC,': Internal error (11) in OOC ',   &
     &                               ERR_STR_OOC(1:DIM_ERR_STR_OOC)
                  CALL AGMG_MUMPS_ABORT()
                  RETURN
               ENDIF
               REQ_ACT=REQ_ACT-1
               CALL DAGMG_MUMPS_596(                                         &
     &              IO_REQ(STEP_OOC(TMP_NODE)),PTRFAC,NSTEPS)
               FREE_HOLE=FREE_HOLE_POS-                                 &
     &              (abs(PTRFAC(STEP_OOC(TMP_NODE)))+                   &
     &              SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE))
            ELSEIF(TMP_NODE.EQ.0)THEN
               DO J=I,PDEB_SOLVE_Z(ZONE),-1
                  IF(POS_IN_MEM(J).NE.0) EXIT
               ENDDO
               IF(POS_IN_MEM(J).LT.0)THEN
                  WRITE(*,*)MYID_OOC,': Internal error (12) in OOC ',   &
     &                 ' DMUMPS_604'
                  CALL AGMG_MUMPS_ABORT()
               ENDIF
               IF(J.GE.PDEB_SOLVE_Z(ZONE))THEN
                  TMP_NODE=POS_IN_MEM(J)
                  FREE_HOLE=FREE_HOLE_POS-                              &
     &                 (abs(PTRFAC(STEP_OOC(TMP_NODE)))+                &
     &                 SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE))
               ELSE
                  FREE_HOLE=FREE_HOLE_POS-IDEB_SOLVE_Z(ZONE)
               ENDIF
            ELSEIF(TMP_NODE.LT.0)THEN
               WRITE(*,*)MYID_OOC,': Internal error (13) in OOC',       &
     &           ' DMUMPS_604'
               CALL AGMG_MUMPS_ABORT()
            ELSE
               FREE_HOLE=FREE_HOLE_POS-                                 &
     &              (abs(PTRFAC(STEP_OOC(TMP_NODE)))+                   &
     &              SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE))
            ENDIF
         ELSE
            FREE_HOLE=FREE_HOLE_POS-IDEB_SOLVE_Z(ZONE)
         ENDIF
         FREE_SIZE=FREE_SIZE+FREE_HOLE
      ENDIF
      CURRENT_POS_T(ZONE)=POS_HOLE_T(ZONE)
      LRLU_SOLVE_T(ZONE)=LRLU_SOLVE_T(ZONE)+FREE_SIZE
      POSFAC_SOLVE(ZONE)=POSFAC_SOLVE(ZONE)-FREE_SIZE
   50 CONTINUE
      IF(REQUESTED_SIZE.LE.LRLU_SOLVE_T(ZONE))THEN
         FLAG=1
      ELSE
         FLAG=0
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_604
      SUBROUTINE DAGMG_MUMPS_605(A,LA,REQUESTED_SIZE,                        &
     &     PTRFAC,NSTEPS,ZONE,FLAG,IERR)
      IMPLICIT NONE
      INTEGER NSTEPS,ZONE,FLAG
      INTEGER (8) :: REQUESTED_SIZE
      INTEGER (8) :: LA
      INTEGER (8) :: PTRFAC(NSTEPS)
      DOUBLE PRECISION A(LA)
      INTEGER(8) :: FREE_SIZE, FREE_HOLE_POS, FREE_HOLE
      INTEGER I,J,TMP_NODE,FREE_HOLE_FLAG
      INTEGER, intent(out) :: IERR
      IERR=0
      FLAG=0
      IF(LRLU_SOLVE_B(ZONE).EQ.SIZE_SOLVE_Z(ZONE))THEN
         GOTO 50
      ENDIF
      IF(POS_HOLE_B(ZONE).EQ.-9999)THEN
         GOTO 50
      ENDIF
      J=max(PDEB_SOLVE_Z(ZONE),POS_HOLE_T(ZONE))
      J=min(J,PDEB_SOLVE_Z(ZONE)+MAX_NB_NODES_FOR_ZONE-1)
      DO I=POS_HOLE_B(ZONE)+1,J
         IF((POS_IN_MEM(I).LT.0).AND.(POS_IN_MEM(I).GT.                 &
     &        -(N_OOC+1)*NB_Z))THEN
            TMP_NODE=-POS_IN_MEM(I)
            FREE_SIZE=FREE_SIZE+SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),       &
     &           OOC_FCT_TYPE)
         ELSEIF(POS_IN_MEM(I).NE.0)THEN
            EXIT
         ENDIF
      ENDDO
      POS_HOLE_B(ZONE)=I-1
      IF((POS_HOLE_T(ZONE).EQ.PDEB_SOLVE_Z(ZONE)).OR.                   &
     &     (POS_HOLE_T(ZONE).LE.POS_HOLE_B(ZONE)).OR.                   &
     &     (POS_HOLE_T(ZONE).EQ.POS_HOLE_B(ZONE)+1))THEN
         CURRENT_POS_B(ZONE)=-9999
         POS_HOLE_B(ZONE)=-9999
         LRLU_SOLVE_B(ZONE)=0_8
         POS_HOLE_T(ZONE)=PDEB_SOLVE_Z(ZONE)
      ENDIF
      FREE_HOLE=0_8
      FREE_SIZE=0_8
      FREE_HOLE_FLAG=0
      FREE_HOLE_POS=IDEB_SOLVE_Z(ZONE)
      IF(POS_HOLE_B(ZONE).EQ.-9999)THEN
         GOTO 50
      ENDIF
      DO I=PDEB_SOLVE_Z(ZONE),POS_HOLE_B(ZONE)
         IF((POS_IN_MEM(I).LE.0).AND.(POS_IN_MEM(I).GT.                 &
     &        -(N_OOC+1)*NB_Z))THEN
            TMP_NODE=-POS_IN_MEM(I)
            IF(TMP_NODE.NE.0)THEN
               IF(I.EQ.PDEB_SOLVE_Z(ZONE))THEN
                  IF(abs(PTRFAC(STEP_OOC(TMP_NODE))).NE.                &
     &                 IDEB_SOLVE_Z(ZONE))THEN
                     FREE_SIZE=FREE_SIZE+abs(PTRFAC(STEP_OOC(TMP_NODE)))&
     &                    -IDEB_SOLVE_Z(ZONE)
                  ENDIF
               ENDIF
               IF(FREE_HOLE_FLAG.EQ.1)THEN
                  FREE_HOLE=abs(PTRFAC(STEP_OOC(TMP_NODE)))-            &
     &                 FREE_HOLE_POS
                  FREE_HOLE_FLAG=0
                  FREE_SIZE=FREE_SIZE+FREE_HOLE
               ENDIF
               FREE_HOLE_POS=abs(PTRFAC(STEP_OOC(TMP_NODE)))+           &
     &              SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)
               PTRFAC(STEP_OOC(TMP_NODE))=-777777_8
               INODE_TO_POS(STEP_OOC(TMP_NODE))=0
               OOC_STATE_NODE(STEP_OOC(TMP_NODE))=ALREADY_USED
               FREE_SIZE=FREE_SIZE+SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),    &
     &              OOC_FCT_TYPE)
            ELSE
               FREE_HOLE_FLAG=1
            ENDIF
            POS_IN_MEM(I)=0
         ELSEIF(POS_IN_MEM(I).NE.0)THEN
            WRITE(*,*)MYID_OOC,': Internal error (14) in OOC ',         &
     &            ' DMUMPS_605',                                        &
     &           CURRENT_POS_T(ZONE)-1,POS_HOLE_B(ZONE),I,POS_IN_MEM(I)
            CALL AGMG_MUMPS_ABORT()
         ENDIF
      ENDDO
      IF(FREE_HOLE_FLAG.EQ.1)THEN
         IF(POS_HOLE_B(ZONE)+1.LT.CURRENT_POS_T(ZONE)-1)THEN
            I=POS_HOLE_B(ZONE)+1
            TMP_NODE=abs(POS_IN_MEM(I))
            IF(TMP_NODE.GT.(N_OOC+1)*NB_Z)THEN
               TMP_NODE=TMP_NODE-(N_OOC+1)*NB_Z
               CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
               IF(IERR.LT.0)THEN
                 WRITE(*,*)MYID_OOC,': Internal error (15) in OOC ',    &
     &                               ERR_STR_OOC(1:DIM_ERR_STR_OOC)
                 CALL AGMG_MUMPS_ABORT()
                  RETURN
               ENDIF
               REQ_ACT=REQ_ACT-1
               CALL DAGMG_MUMPS_596(                                         &
     &              IO_REQ(STEP_OOC(TMP_NODE)),PTRFAC,NSTEPS)
               FREE_HOLE=abs(PTRFAC(STEP_OOC(TMP_NODE)))-FREE_HOLE_POS
            ELSEIF(TMP_NODE.EQ.0)THEN
               DO J=I,CURRENT_POS_T(ZONE)-1
                  IF(POS_IN_MEM(J).NE.0) EXIT
               ENDDO
               IF(POS_IN_MEM(J).LT.0)THEN
                  WRITE(*,*)MYID_OOC,': Internal error (16) in OOC ',   &
     &                  ' DMUMPS_605'
                  CALL AGMG_MUMPS_ABORT()
               ENDIF
               IF(J.LE.CURRENT_POS_T(ZONE)-1)THEN
                  TMP_NODE=POS_IN_MEM(J)
                  FREE_HOLE=abs(PTRFAC(STEP_OOC(TMP_NODE)))-            &
     &                 FREE_HOLE_POS
               ELSE
                  FREE_HOLE=POSFAC_SOLVE(ZONE)-FREE_HOLE_POS
               ENDIF
            ELSEIF(TMP_NODE.LT.0)THEN
               WRITE(*,*)MYID_OOC,': Internal error (17) in OOC ',      &
     &           ' DMUMPS_605'
               CALL AGMG_MUMPS_ABORT()
            ELSE
               FREE_HOLE=abs(PTRFAC(STEP_OOC(TMP_NODE)))-               &
     &              FREE_HOLE_POS
            ENDIF
         ELSE
            FREE_HOLE=POSFAC_SOLVE(ZONE)-FREE_HOLE_POS
         ENDIF
         FREE_SIZE=FREE_SIZE+FREE_HOLE
      ENDIF
      LRLU_SOLVE_B(ZONE)=FREE_SIZE
      IF(POS_HOLE_B(ZONE).LT.CURRENT_POS_T(ZONE)-1)THEN
         TMP_NODE=POS_IN_MEM(POS_HOLE_B(ZONE)+1)
         IF(TMP_NODE.LT.-(N_OOC+1)*NB_Z)THEN
            TMP_NODE=abs(TMP_NODE)-(N_OOC+1)*NB_Z
            CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
            IF(IERR.LT.0)THEN
               WRITE(*,*)MYID_OOC,': Internal error (18) in OOC ',      &
     &                            ERR_STR_OOC(1:DIM_ERR_STR_OOC)
               CALL AGMG_MUMPS_ABORT()
               RETURN
            ENDIF
            REQ_ACT=REQ_ACT-1
            CALL DAGMG_MUMPS_596(                                            &
     &           IO_REQ(STEP_OOC(TMP_NODE)),PTRFAC,NSTEPS)
         ENDIF
         LRLU_SOLVE_B(ZONE)=LRLU_SOLVE_B(ZONE)+                         &
     &        (abs(PTRFAC(STEP_OOC(abs(TMP_NODE))))-IDEB_SOLVE_Z(ZONE)- &
     &        LRLU_SOLVE_B(ZONE))
      ENDIF
      CURRENT_POS_B(ZONE)=POS_HOLE_B(ZONE)
   50 CONTINUE
      IF((POS_HOLE_B(ZONE).EQ.-9999).AND.                               &
     &   (LRLU_SOLVE_B(ZONE).NE.0_8))THEN
         WRITE(*,*)MYID_OOC,': Internal error (19) in OOC ',            &
     &             'DMUMPS_605'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF((REQUESTED_SIZE.LE.LRLU_SOLVE_B(ZONE)).AND.                    &
     &     (POS_HOLE_B(ZONE).NE.-9999))THEN
         FLAG=1
      ELSE
         FLAG=0
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_605
      SUBROUTINE DAGMG_MUMPS_606(INODE,PTRFAC,                               &
     &           KEEP,KEEP8, A,ZONE)
      IMPLICIT NONE
      INTEGER INODE,KEEP(500)
      INTEGER*8 KEEP8(150)
      INTEGER(8) :: PTRFAC(KEEP(28))
      DOUBLE PRECISION A(FACT_AREA_SIZE)
      INTEGER REQUESTED_SIZE
      INTEGER ZONE,IFLAG
      LOGICAL FLAG
      LRLU_SOLVE_T(ZONE)=LRLU_SOLVE_T(ZONE)-                            &
     &     SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)
      LRLUS_SOLVE(ZONE)=LRLUS_SOLVE(ZONE)-                              &
     &     SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)
      PTRFAC(STEP_OOC(INODE))=POSFAC_SOLVE(ZONE)
      OOC_STATE_NODE(STEP_OOC(INODE))=NOT_USED
      IF(POSFAC_SOLVE(ZONE).EQ.IDEB_SOLVE_Z(ZONE))THEN
         POS_HOLE_B(ZONE)=-9999
         CURRENT_POS_B(ZONE)=-9999
         LRLU_SOLVE_B(ZONE)=0_8
      ENDIF
      IF(PTRFAC(STEP_OOC(INODE)).LT.IDEB_SOLVE_Z(ZONE))THEN
         WRITE(*,*)MYID_OOC,': Internal error (20) in OOC ',            &
     &                      ' Problem avec debut (2)',INODE,            &
     &              PTRFAC(STEP_OOC(INODE)),IDEB_SOLVE_Z(ZONE),ZONE
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      INODE_TO_POS(STEP_OOC(INODE))=CURRENT_POS_T(ZONE)
      POS_IN_MEM(CURRENT_POS_T(ZONE))=INODE
      IF(CURRENT_POS_T(ZONE).GT.(PDEB_SOLVE_Z(ZONE)+                    &
     &     MAX_NB_NODES_FOR_ZONE-1))THEN
         WRITE(*,*)MYID_OOC,': Internal error (21) in OOC ',            &
     &                      ' Problem with CURRENT_POS_T',              &
     &        CURRENT_POS_T(ZONE),ZONE
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      CURRENT_POS_T(ZONE)=CURRENT_POS_T(ZONE)+1
      POS_HOLE_T(ZONE)=CURRENT_POS_T(ZONE)
      POS_HOLE_T(ZONE)=CURRENT_POS_T(ZONE)
      POSFAC_SOLVE(ZONE)=POSFAC_SOLVE(ZONE)+                            &
     &     SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)
      END SUBROUTINE DAGMG_MUMPS_606
      SUBROUTINE DAGMG_MUMPS_607(INODE,PTRFAC,                               &
     &     KEEP,KEEP8,                                                  &
     &     A,ZONE)
      IMPLICIT NONE
      INTEGER INODE,KEEP(500)
      INTEGER*8 KEEP8(150)
      INTEGER(8) :: PTRFAC(KEEP(28))
      DOUBLE PRECISION A(FACT_AREA_SIZE)
      INTEGER REQUESTED_SIZE
      INTEGER ZONE,IFLAG,TMP_NODE
      LOGICAL FLAG
      IF(POS_HOLE_B(ZONE).EQ.-9999)THEN
         WRITE(*,*)MYID_OOC,': Internal error (22) in OOC ',            &
     &        ' DMUMPS_607'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      LRLUS_SOLVE(ZONE)=LRLUS_SOLVE(ZONE)-                              &
     &     SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)
      LRLU_SOLVE_B(ZONE)=LRLU_SOLVE_B(ZONE)-                            &
     &        SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)
      PTRFAC(STEP_OOC(INODE))=IDEB_SOLVE_Z(ZONE)+                       &
     &     LRLU_SOLVE_B(ZONE)
      OOC_STATE_NODE(STEP_OOC(INODE))=NOT_USED
      IF(PTRFAC(STEP_OOC(INODE)).LT.IDEB_SOLVE_Z(ZONE))THEN
         WRITE(*,*)MYID_OOC,': Internal error (23) in OOC ',            &
     &              PTRFAC(STEP_OOC(INODE)),IDEB_SOLVE_Z(ZONE)
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      INODE_TO_POS(STEP_OOC(INODE))=CURRENT_POS_B(ZONE)
      IF(CURRENT_POS_B(ZONE).EQ.0)THEN
         WRITE(*,*)MYID_OOC,': Internal error (23b) in OOC '
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      POS_IN_MEM(CURRENT_POS_B(ZONE))=INODE
      CURRENT_POS_B(ZONE)=CURRENT_POS_B(ZONE)-1
      POS_HOLE_B(ZONE)=CURRENT_POS_B(ZONE)
      END SUBROUTINE DAGMG_MUMPS_607
      SUBROUTINE DAGMG_MUMPS_608(A,LA,REQUESTED_SIZE,PTRFAC,                 &
     &     NSTEPS,ZONE,IERR)
      IMPLICIT NONE
      INTEGER(8) :: LA, REQUESTED_SIZE
      INTEGER NSTEPS,ZONE
      INTEGER, intent(out) :: IERR
      INTEGER(8) :: PTRFAC(NSTEPS)
      DOUBLE PRECISION A(LA)
      INTEGER (8) :: APOS_FIRST_FREE,                                   &
     &               SIZE_HOLE,                                         &
     &               FREE_HOLE,                                         &
     &               FREE_HOLE_POS
      INTEGER J,I,TMP_NODE, NB_FREE, IPOS_FIRST_FREE
      INTEGER(8) :: K8, AREA_POINTER
      INTEGER FREE_HOLE_FLAG,TMPP
      IERR=0
      IF(LRLU_SOLVE_T(ZONE).EQ.SIZE_SOLVE_Z(ZONE))THEN
         RETURN
      ENDIF
      AREA_POINTER=IDEB_SOLVE_Z(ZONE)
      SIZE_HOLE=0_8
      DO I=PDEB_SOLVE_Z(ZONE),CURRENT_POS_T(ZONE)-1
         IF((POS_IN_MEM(I).LE.0).AND.                                   &
     &        (POS_IN_MEM(I).GT.-((N_OOC+1)*NB_Z))) GOTO 666
         TMP_NODE=abs(POS_IN_MEM(I))
         IF(TMP_NODE.GT.((N_OOC+1)*NB_Z))THEN
            TMP_NODE=TMP_NODE-((N_OOC+1)*NB_Z)
         ENDIF
         AREA_POINTER=AREA_POINTER+                                     &
     &        abs(SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE))
      ENDDO
  666 CONTINUE
      IF((I.EQ.CURRENT_POS_T(ZONE)-1).AND.                              &
     &     (PDEB_SOLVE_Z(ZONE).NE.CURRENT_POS_T(ZONE)-1))THEN
         IF((POS_IN_MEM(I).GT.0).OR.                                    &
     &        (POS_IN_MEM(I).LT.-((N_OOC+1)*NB_Z)))THEN
            WRITE(*,*)MYID_OOC,': Internal error (25) in OOC ',         &
     &                      ': There are no free blocks ',              &
     &         'in DMUMPS_608',PDEB_SOLVE_Z(ZONE),                      &
     &           CURRENT_POS_T(ZONE)
            CALL AGMG_MUMPS_ABORT()
         ENDIF
      ENDIF
      IF(POS_IN_MEM(I).EQ.0)THEN
         APOS_FIRST_FREE=AREA_POINTER
         FREE_HOLE_POS=AREA_POINTER
      ELSE
         TMP_NODE=abs(POS_IN_MEM(I))
         APOS_FIRST_FREE=abs(PTRFAC(STEP_OOC(TMP_NODE)))
      ENDIF
      IF(POS_IN_MEM(PDEB_SOLVE_Z(ZONE)).NE.0)THEN
         IF(POS_IN_MEM(PDEB_SOLVE_Z(ZONE)).LT.-((N_OOC+1)*NB_Z))THEN
            TMP_NODE=abs(POS_IN_MEM(PDEB_SOLVE_Z(ZONE)))-               &
     &           ((N_OOC+1)*NB_Z)
            CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
            IF(IERR.LT.0)THEN
               RETURN
            ENDIF
            REQ_ACT=REQ_ACT-1
            CALL DAGMG_MUMPS_596(                                            &
     &           IO_REQ(STEP_OOC(TMP_NODE)),PTRFAC,NSTEPS)
         ELSE
            TMP_NODE=abs(POS_IN_MEM(PDEB_SOLVE_Z(ZONE)))
         ENDIF
         IF(abs(PTRFAC(STEP_OOC(TMP_NODE))).NE.IDEB_SOLVE_Z(ZONE))THEN
            IF((POS_IN_MEM(I).NE.0).OR.(I.EQ.CURRENT_POS_T(ZONE)))THEN
               SIZE_HOLE=abs(PTRFAC(STEP_OOC(TMP_NODE)))-               &
     &              IDEB_SOLVE_Z(ZONE)
            ENDIF
            APOS_FIRST_FREE=IDEB_SOLVE_Z(ZONE)
            IF(POS_IN_MEM(PDEB_SOLVE_Z(ZONE)).GT.0)THEN
               DO J=PDEB_SOLVE_Z(ZONE),I-1
                  TMP_NODE=POS_IN_MEM(J)
                  IF(TMP_NODE.LE.0)THEN
                     IF(TMP_NODE.LT.-((N_OOC+1)*NB_Z))THEN
                        TMP_NODE=abs(POS_IN_MEM(J))-((N_OOC+1)*NB_Z)
                        CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
                        IF(IERR.LT.0)THEN
                           RETURN
                        ENDIF
                        REQ_ACT=REQ_ACT-1
                        CALL DAGMG_MUMPS_596(                                &
     &                       IO_REQ(STEP_OOC(TMP_NODE)),PTRFAC,NSTEPS)
                        TMP_NODE=POS_IN_MEM(J)
                     ELSE
                    WRITE(*,*)MYID_OOC,': Internal error (26) in OOC ', &
     &                      ' DMUMPS_608',TMP_NODE,                     &
     &                       J,I-1,(N_OOC+1)*NB_Z
                        CALL AGMG_MUMPS_ABORT()
                     ENDIF
                  ENDIF
                  DO K8=1_8,                                            &
     &                  SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)
                     A(APOS_FIRST_FREE+K8-1_8)=                         &
     &                    A(PTRFAC(STEP_OOC(TMP_NODE))+K8-1_8)
                  ENDDO
                  PTRFAC(STEP_OOC(TMP_NODE))=APOS_FIRST_FREE
                  APOS_FIRST_FREE=APOS_FIRST_FREE+                      &
     &                 SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)
               ENDDO
            ENDIF
         ENDIF
      ENDIF
      NB_FREE=0
      FREE_HOLE=0_8
      FREE_HOLE_FLAG=0
      DO J=I,CURRENT_POS_T(ZONE)-1
         TMP_NODE=abs(POS_IN_MEM(J))
         IF(POS_IN_MEM(J).LT.-((N_OOC+1)*NB_Z))THEN
            TMP_NODE=TMP_NODE-((N_OOC+1)*NB_Z)
            CALL AGMG_NOCALL('MUMPS_WAIT_REQUEST')
            IF(IERR.LT.0)THEN
               RETURN
            ENDIF
            REQ_ACT=REQ_ACT-1
            CALL DAGMG_MUMPS_596(                                            &
     &           IO_REQ(STEP_OOC(TMP_NODE)),PTRFAC,NSTEPS)
            TMP_NODE=abs(POS_IN_MEM(J))
         ENDIF
         IF(POS_IN_MEM(J).GT.0)THEN
            DO K8=1_8,SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)
               A(APOS_FIRST_FREE+K8-1_8)=                               &
     &         A(PTRFAC(STEP_OOC(TMP_NODE))+K8-1_8)
            ENDDO
            IF(FREE_HOLE_FLAG.EQ.1)THEN
               FREE_HOLE=abs(PTRFAC(STEP_OOC(TMP_NODE)))-               &
     &              FREE_HOLE_POS
               FREE_HOLE_FLAG=0
               SIZE_HOLE=SIZE_HOLE+FREE_HOLE
            ENDIF
            FREE_HOLE_POS=abs(PTRFAC(STEP_OOC(TMP_NODE)))+              &
     &           SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)
            PTRFAC(STEP_OOC(TMP_NODE))=APOS_FIRST_FREE
            APOS_FIRST_FREE=APOS_FIRST_FREE+                            &
     &           SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)
         ELSEIF(POS_IN_MEM(J).EQ.0)THEN
            FREE_HOLE_FLAG=1
            NB_FREE=NB_FREE+1
         ELSE
            NB_FREE=NB_FREE+1
            IF(FREE_HOLE_FLAG.EQ.1)THEN
               FREE_HOLE=abs(PTRFAC(STEP_OOC(TMP_NODE)))-               &
     &              FREE_HOLE_POS
               FREE_HOLE_FLAG=0
               SIZE_HOLE=SIZE_HOLE+FREE_HOLE
            ENDIF
            FREE_HOLE_POS=abs(PTRFAC(STEP_OOC(TMP_NODE)))+              &
     &           SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)
            SIZE_HOLE=SIZE_HOLE+SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),       &
     &           OOC_FCT_TYPE)
            PTRFAC(STEP_OOC(abs(POS_IN_MEM(J))))=-77777_8
         ENDIF
      ENDDO
      IF(FREE_HOLE_FLAG.EQ.1)THEN
         FREE_HOLE=POSFAC_SOLVE(ZONE)-FREE_HOLE_POS
         FREE_HOLE_FLAG=0
         SIZE_HOLE=SIZE_HOLE+FREE_HOLE
      ENDIF
      IPOS_FIRST_FREE=I
      DO J=I,CURRENT_POS_T(ZONE)-1
         IF(POS_IN_MEM(J).LT.0)THEN
            TMP_NODE=abs(POS_IN_MEM(J))
            INODE_TO_POS(STEP_OOC(TMP_NODE))=0
            POS_IN_MEM(J)=0
            OOC_STATE_NODE(STEP_OOC(TMP_NODE))=ALREADY_USED
          ELSEIF(POS_IN_MEM(J).GT.0)THEN
             TMP_NODE=abs(POS_IN_MEM(J))
             POS_IN_MEM(IPOS_FIRST_FREE)=POS_IN_MEM(J)
             INODE_TO_POS(STEP_OOC(TMP_NODE))=IPOS_FIRST_FREE
             IPOS_FIRST_FREE=IPOS_FIRST_FREE+1
         ENDIF
      ENDDO
  777 CONTINUE
      LRLU_SOLVE_T(ZONE)=LRLU_SOLVE_T(ZONE)+SIZE_HOLE
      POSFAC_SOLVE(ZONE)=POSFAC_SOLVE(ZONE)-SIZE_HOLE
      CURRENT_POS_T(ZONE)=CURRENT_POS_T(ZONE)-NB_FREE
      POS_HOLE_T(ZONE)=CURRENT_POS_T(ZONE)
      LRLU_SOLVE_B(ZONE)=0_8
      POS_HOLE_B(ZONE)=-9999
      CURRENT_POS_B(ZONE)=-9999
      LRLU_SOLVE_B(ZONE)=0_8
      IF(LRLU_SOLVE_T(ZONE).NE.LRLUS_SOLVE(ZONE))THEN
         WRITE(*,*)MYID_OOC,': Internal error (27) in OOC ',            &
     &                 LRLU_SOLVE_T(ZONE),                              &
     &                 LRLUS_SOLVE(ZONE)
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      LRLU_SOLVE_T(ZONE)=LRLUS_SOLVE(ZONE)
      IF(LRLUS_SOLVE(ZONE).LT.0_8)THEN
         WRITE(*,*)MYID_OOC,': Internal error (28) in OOC ',            &
     &                      ' LRLUS_SOLVE must be (4) > 0'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF(POSFAC_SOLVE(ZONE).LT.IDEB_SOLVE_Z(ZONE))THEN
         WRITE(*,*)MYID_OOC,': Internal error (29) in OOC ',            &
     &        POSFAC_SOLVE(ZONE),IDEB_SOLVE_Z(ZONE)
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF(POSFAC_SOLVE(ZONE).NE.(IDEB_SOLVE_Z(ZONE)+SIZE_SOLVE_Z(ZONE)-  &
     &     LRLUS_SOLVE(ZONE)))THEN
         WRITE(*,*)MYID_OOC,': Internal error (30) in OOC ',            &
     &                      ' Problem avec debut POSFAC_SOLVE',         &
     &        POSFAC_SOLVE(ZONE),(SIZE_SOLVE_Z(ZONE)-                   &
     &     LRLUS_SOLVE(ZONE))+IDEB_SOLVE_Z(ZONE),LRLUS_SOLVE(ZONE)
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF(POSFAC_SOLVE(ZONE).GT.                                         &
     &     (IDEB_SOLVE_Z(ZONE)+SIZE_SOLVE_Z(ZONE)-1_8))THEN
         WRITE(*,*)MYID_OOC,': Internal error (31) in OOC ',            &
     &        POSFAC_SOLVE(ZONE),IDEB_SOLVE_Z(ZONE)+                    &
     &        SIZE_SOLVE_Z(ZONE)-1_8
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_608
      SUBROUTINE DAGMG_MUMPS_581()
      IMPLICIT NONE
      IS_INIT_CALLED=.FALSE.
      END SUBROUTINE DAGMG_MUMPS_581
      SUBROUTINE DAGMG_MUMPS_609(INODE,PTRFAC,NSTEPS,FLAG)
      IMPLICIT NONE
      INTEGER INODE,NSTEPS,FLAG
      INTEGER (8) :: PTRFAC(NSTEPS)
      INTEGER ZONE
      IF((FLAG.LT.0).OR.(FLAG.GT.1))THEN
         WRITE(*,*)MYID_OOC,': Internal error (32) in OOC ',            &
     &        ' DMUMPS_609'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      CALL DAGMG_MUMPS_610(PTRFAC(STEP_OOC(INODE)),ZONE)
      IF(LRLUS_SOLVE(ZONE).LT.0_8)THEN
         WRITE(*,*)MYID_OOC,': Internal error (33) in OOC ',            &
     &        ' LRLUS_SOLVE must be (5) ++ > 0'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF(FLAG.EQ.0)THEN
         LRLUS_SOLVE(ZONE)=LRLUS_SOLVE(ZONE)+                           &
     &        SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)
      ELSE
         LRLUS_SOLVE(ZONE)=LRLUS_SOLVE(ZONE)-                           &
     &        SIZE_OF_BLOCK(STEP_OOC(INODE),OOC_FCT_TYPE)
      ENDIF
      IF(LRLUS_SOLVE(ZONE).LT.0_8)THEN
         WRITE(*,*)MYID_OOC,': Internal error (34) in OOC ',            &
     &                      ' LRLUS_SOLVE must be (5) > 0'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_609
      SUBROUTINE DAGMG_MUMPS_610(ADDR,ZONE)
      IMPLICIT NONE
      INTEGER (8) :: ADDR
      INTEGER ZONE
      INTEGER I
      I=1
      DO WHILE (I.LE.NB_Z)
         IF(ADDR.LT.IDEB_SOLVE_Z(I))THEN
            EXIT
         ENDIF
         I=I+1
      ENDDO
      ZONE=I-1
      END SUBROUTINE DAGMG_MUMPS_610
      FUNCTION DAGMG_MUMPS_727()
      IMPLICIT NONE
      LOGICAL DAGMG_MUMPS_727
      DAGMG_MUMPS_727=.FALSE.
      IF(SOLVE_STEP.EQ.0)THEN
         IF(CUR_POS_SEQUENCE.GT.TOTAL_NB_OOC_NODES(OOC_FCT_TYPE))THEN
            DAGMG_MUMPS_727=.TRUE.
         ENDIF
      ELSEIF(SOLVE_STEP.EQ.1)THEN
         IF(CUR_POS_SEQUENCE.LT.1)THEN
            DAGMG_MUMPS_727=.TRUE.
         ENDIF
      ENDIF
      RETURN
      END FUNCTION DAGMG_MUMPS_727
      SUBROUTINE DAGMG_MUMPS_611(ZONE,A,LA,PTRFAC,NSTEPS,IERR)
      IMPLICIT NONE
      INTEGER NSTEPS,ZONE
      INTEGER(8), INTENT(IN) :: LA
      INTEGER, intent(out) :: IERR
      DOUBLE PRECISION A(LA)
      INTEGER(8) :: PTRFAC(NSTEPS)
      INTEGER(8) :: SIZE, DEST
      INTEGER(8) :: NEEDED_SIZE
      INTEGER FLAG,TMP_FLAG,POS_SEQ,TMP_NODE,                           &
     &     NB_NODES
      EXTERNAL AGMG_MUMPS_330,AGMG_MUMPS_275
      INTEGER AGMG_MUMPS_330,AGMG_MUMPS_275
      IERR=0
      TMP_FLAG=0
      FLAG=0
      IF(DAGMG_MUMPS_727())THEN
         RETURN
      ENDIF
      IF(SOLVE_STEP.EQ.0)THEN
         IF(CUR_POS_SEQUENCE.LE.TOTAL_NB_OOC_NODES(OOC_FCT_TYPE))THEN
            TMP_NODE=OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE,               &
     &           OOC_FCT_TYPE)
            DO WHILE(SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE).GT. &
     &           SIZE_SOLVE_Z(ZONE))
               CUR_POS_SEQUENCE=CUR_POS_SEQUENCE+1
               IF(DAGMG_MUMPS_727())THEN
                  RETURN
               ENDIF
               TMP_NODE=OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE,            &
     &              OOC_FCT_TYPE)
            ENDDO
            CALL DAGMG_MUMPS_728()
            NEEDED_SIZE=max(MIN_SIZE_READ,                              &
     &           SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE))
         ELSE
            NEEDED_SIZE=MIN_SIZE_READ
         ENDIF
      ELSEIF(SOLVE_STEP.EQ.1)THEN
         IF(CUR_POS_SEQUENCE.GE.1)THEN
            TMP_NODE=OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE,               &
     &           OOC_FCT_TYPE)
            DO WHILE(SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE).GT. &
     &           SIZE_SOLVE_Z(ZONE))
               CUR_POS_SEQUENCE=CUR_POS_SEQUENCE-1
               IF(DAGMG_MUMPS_727())THEN
                  RETURN
               ENDIF
               TMP_NODE=OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE,            &
     &              OOC_FCT_TYPE)
            ENDDO
            CALL DAGMG_MUMPS_728()
            NEEDED_SIZE=max(MIN_SIZE_READ,                              &
     &           SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE))
         ELSE
            NEEDED_SIZE=MIN_SIZE_READ
         ENDIF
      ENDIF
      IF(LRLUS_SOLVE(ZONE).LT.NEEDED_SIZE)THEN
         RETURN
      ELSEIF((LRLU_SOLVE_T(ZONE).LT.NEEDED_SIZE).AND.                   &
     &        (LRLU_SOLVE_B(ZONE).LT.NEEDED_SIZE).AND.                  &
     &        (dble(LRLUS_SOLVE(ZONE)).LT.0.3d0*                        &
     &         dble(SIZE_SOLVE_Z(ZONE)))) THEN
         RETURN
      ENDIF
      IF((LRLU_SOLVE_T(ZONE).GT.NEEDED_SIZE).AND.(SOLVE_STEP.EQ.0).AND. &
     &     ((CURRENT_POS_T(ZONE)-PDEB_SOLVE_Z(ZONE)+1).LT.              &
     &     MAX_NB_NODES_FOR_ZONE))THEN
         FLAG=1
      ELSE
         IF(SOLVE_STEP.EQ.0)THEN
            CALL DAGMG_MUMPS_604(A,FACT_AREA_SIZE,                           &
     &           NEEDED_SIZE,PTRFAC,NSTEPS,ZONE,TMP_FLAG,IERR)
            IF(IERR.LT.0)THEN
               RETURN
            ENDIF
            FLAG=1
            IF(TMP_FLAG.EQ.0)THEN
               CALL DAGMG_MUMPS_605(A,FACT_AREA_SIZE,                        &
     &              NEEDED_SIZE,PTRFAC,NSTEPS,ZONE,TMP_FLAG,IERR)
               IF(IERR.LT.0)THEN
                  RETURN
               ENDIF
               FLAG=0
            ENDIF
         ELSE
            CALL DAGMG_MUMPS_605(A,FACT_AREA_SIZE,                           &
     &           NEEDED_SIZE,PTRFAC,NSTEPS,ZONE,TMP_FLAG,IERR)
            IF(IERR.LT.0)THEN
               RETURN
            ENDIF
            FLAG=0
            IF(TMP_FLAG.EQ.0)THEN
               CALL DAGMG_MUMPS_604(A,FACT_AREA_SIZE,                        &
     &              NEEDED_SIZE,PTRFAC,NSTEPS,ZONE,TMP_FLAG,IERR)
               IF(IERR.LT.0)THEN
                  RETURN
               ENDIF
               FLAG=1
            ENDIF
         ENDIF
         IF(TMP_FLAG.EQ.0)THEN
            CALL DAGMG_MUMPS_608(A,FACT_AREA_SIZE,                           &
     &           NEEDED_SIZE,PTRFAC,NSTEPS,ZONE,IERR)
            IF(IERR.LT.0)THEN
               RETURN
            ENDIF
            FLAG=1
         ENDIF
      ENDIF
      CALL DAGMG_MUMPS_602(ZONE,SIZE,DEST,POS_SEQ,                           &
     &     NB_NODES,FLAG,PTRFAC,NSTEPS)
      IF(SIZE.EQ.0_8)THEN
         RETURN
      ENDIF
      NB_ZONE_REQ=NB_ZONE_REQ+1
      SIZE_ZONE_REQ=SIZE_ZONE_REQ+SIZE
      REQ_ACT=REQ_ACT+1
      CALL DAGMG_MUMPS_595(A(DEST),DEST,SIZE,ZONE,PTRFAC,NSTEPS,             &
     &     POS_SEQ,NB_NODES,FLAG,IERR)
      IF(IERR.LT.0)THEN
         RETURN
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_611
      SUBROUTINE DAGMG_MUMPS_602(ZONE,SIZE,DEST,POS_SEQ,                     &
     &     NB_NODES,FLAG,PTRFAC,NSTEPS)
      IMPLICIT NONE
      INTEGER(8) :: SIZE, DEST
      INTEGER ZONE,FLAG,POS_SEQ,NSTEPS
      INTEGER(8) :: PTRFAC(NSTEPS), MAX_SIZE, LAST, J8
      INTEGER I,TMP_NUM_FILE,START_NODE,K,TMP,MAX_NB,                   &
     &     NB_NODES
      INTEGER NB_NODES_LOC
      LOGICAL ALREADY
      IF(DAGMG_MUMPS_727())THEN
         SIZE=0_8
         RETURN
      ENDIF
      IF(FLAG.EQ.0)THEN
         MAX_SIZE=LRLU_SOLVE_B(ZONE)
         MAX_NB=max(0,CURRENT_POS_B(ZONE)-PDEB_SOLVE_Z(ZONE)+1)
      ELSEIF(FLAG.EQ.1)THEN
         MAX_SIZE=LRLU_SOLVE_T(ZONE)
         MAX_NB=MAX_NB_NODES_FOR_ZONE
      ELSE
         WRITE(*,*)MYID_OOC,': Internal error (35) in OOC ',            &
     &                      ' Unknown Flag value in ',                  &
     &         ' DMUMPS_602',FLAG
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      CALL DAGMG_MUMPS_728()
      I=CUR_POS_SEQUENCE
      START_NODE=OOC_INODE_SEQUENCE(I,OOC_FCT_TYPE)
      ALREADY=.FALSE.
      NB_NODES=0
      NB_NODES_LOC=0
      IF(ZONE.EQ.NB_Z)THEN
         SIZE=SIZE_OF_BLOCK(STEP_OOC(START_NODE),OOC_FCT_TYPE)
      ELSE
         J8=0_8
         IF(FLAG.EQ.0)THEN
            K=0
         ELSEIF(FLAG.EQ.1)THEN
            K=CURRENT_POS_T(ZONE)-PDEB_SOLVE_Z(ZONE)+1
         ENDIF
         IF(SOLVE_STEP.EQ.0)THEN
            I=CUR_POS_SEQUENCE
            DO WHILE(I.LE.TOTAL_NB_OOC_NODES(OOC_FCT_TYPE))
               IF(SIZE_OF_BLOCK(STEP_OOC(OOC_INODE_SEQUENCE(I,          &
     &              OOC_FCT_TYPE)),                                     &
     &              OOC_FCT_TYPE)                                       &
     &              .NE.0_8)THEN
                  EXIT
               ENDIF
               I=I+1
            ENDDO
            CUR_POS_SEQUENCE=min(I,TOTAL_NB_OOC_NODES(OOC_FCT_TYPE))
            I=CUR_POS_SEQUENCE
            DO WHILE((J8.LE.MAX_SIZE).AND.                              &
     &           (I.LE.TOTAL_NB_OOC_NODES(OOC_FCT_TYPE)).AND.           &
     &           (K.LT.MAX_NB) )
               LAST=SIZE_OF_BLOCK(STEP_OOC(OOC_INODE_SEQUENCE(I,        &
     &              OOC_FCT_TYPE)),                                     &
     &              OOC_FCT_TYPE)
               IF(LAST.EQ.0_8)THEN
                  IF(.NOT.ALREADY)THEN
                     CUR_POS_SEQUENCE=CUR_POS_SEQUENCE+1
                  ENDIF
                  I=I+1
                  NB_NODES_LOC=NB_NODES_LOC+1
                  CYCLE
               ENDIF
               IF((INODE_TO_POS(STEP_OOC(OOC_INODE_SEQUENCE(I,          &
     &              OOC_FCT_TYPE)))                                     &
     &              .NE.0).OR.                                          &
     &              (IO_REQ(STEP_OOC(OOC_INODE_SEQUENCE(I,              &
     &              OOC_FCT_TYPE))).GE.                                 &
     &              0))THEN
                  IF(.NOT.ALREADY)THEN
                     CUR_POS_SEQUENCE=CUR_POS_SEQUENCE+1
                     I=I+1
                     CYCLE
                  ELSE
                     EXIT
                  ENDIF
               ENDIF
               ALREADY=.TRUE.
               J8=J8+LAST
               I=I+1
               K=K+1
               NB_NODES_LOC=NB_NODES_LOC+1
               NB_NODES=NB_NODES+1
            ENDDO
            IF(J8.GT.MAX_SIZE)THEN
               SIZE=J8-LAST
               NB_NODES=NB_NODES-1
               NB_NODES_LOC=NB_NODES_LOC-1
            ELSE
               SIZE=J8
            ENDIF
            DO WHILE (CUR_POS_SEQUENCE+NB_NODES_LOC-1.GE.               &
     &                             CUR_POS_SEQUENCE)
               IF(SIZE_OF_BLOCK(STEP_OOC(                               &
     &              OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE+NB_NODES-1,     &
     &              OOC_FCT_TYPE)),                                     &
     &              OOC_FCT_TYPE)                                       &
     &              .NE.0_8)THEN
                  EXIT
               ENDIF
               NB_NODES_LOC=NB_NODES_LOC-1
            ENDDO
            POS_SEQ=CUR_POS_SEQUENCE
         ELSEIF(SOLVE_STEP.EQ.1)THEN
            DO WHILE(I.GE.1)
               IF(SIZE_OF_BLOCK(STEP_OOC(OOC_INODE_SEQUENCE(I,          &
     &              OOC_FCT_TYPE)),                                     &
     &              OOC_FCT_TYPE)                                       &
     &              .NE.0_8)THEN
                  EXIT
               ENDIF
               I=I-1
            ENDDO
            CUR_POS_SEQUENCE=max(I,1)
            I=CUR_POS_SEQUENCE
            DO WHILE((J8.LE.MAX_SIZE).AND.(I.GE.1).AND.                 &
     &           (K.LT.MAX_NB))
               LAST=SIZE_OF_BLOCK(STEP_OOC(OOC_INODE_SEQUENCE(I,        &
     &              OOC_FCT_TYPE)),                                     &
     &              OOC_FCT_TYPE)
               IF(LAST.EQ.0_8)THEN
                  IF(.NOT.ALREADY)THEN
                     CUR_POS_SEQUENCE=CUR_POS_SEQUENCE-1
                  ENDIF
                  NB_NODES_LOC=NB_NODES_LOC+1
                  I=I-1
                  CYCLE
               ENDIF
               IF((INODE_TO_POS(STEP_OOC(OOC_INODE_SEQUENCE(I,          &
     &              OOC_FCT_TYPE)))                                     &
     &              .NE.0).OR.                                          &
     &              (IO_REQ(STEP_OOC(OOC_INODE_SEQUENCE(I,              &
     &              OOC_FCT_TYPE))).GE.                                 &
     &              0))THEN
                  IF(.NOT.ALREADY)THEN
                     I=I-1
                     CUR_POS_SEQUENCE=CUR_POS_SEQUENCE-1
                     CYCLE
                  ELSE
                     EXIT
                  ENDIF
               ENDIF
               ALREADY=.TRUE.
               J8=J8+LAST
               I=I-1
               K=K+1
               NB_NODES=NB_NODES+1
               NB_NODES_LOC=NB_NODES_LOC+1
            ENDDO
            IF(J8.GT.MAX_SIZE)THEN
               SIZE=J8-LAST
               NB_NODES=NB_NODES-1
               NB_NODES_LOC=NB_NODES_LOC-1
            ELSE
               SIZE=J8
            ENDIF
            I=CUR_POS_SEQUENCE-NB_NODES_LOC+1
            DO WHILE (I.LE.CUR_POS_SEQUENCE)
               IF(SIZE_OF_BLOCK(STEP_OOC(                               &
     &              OOC_INODE_SEQUENCE(I,OOC_FCT_TYPE)),                &
     &              OOC_FCT_TYPE).NE.0_8)THEN
                  EXIT
               ENDIF
               I=I+1
               NB_NODES_LOC=NB_NODES_LOC-1
            ENDDO
            POS_SEQ=CUR_POS_SEQUENCE-NB_NODES_LOC+1
         ENDIF
      ENDIF
      IF(FLAG.EQ.0)THEN
         DEST=IDEB_SOLVE_Z(ZONE)+LRLU_SOLVE_B(ZONE)-SIZE
      ELSE
         DEST=POSFAC_SOLVE(ZONE)
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_602
      SUBROUTINE DAGMG_MUMPS_582(IERR)
      IMPLICIT NONE
      INTEGER SOLVE_OR_FACTO
      INTEGER, intent(out) :: IERR
      IERR=0
      IF(allocated(LRLUS_SOLVE))THEN
         DEALLOCATE(LRLUS_SOLVE)
      ENDIF
      IF(allocated(LRLU_SOLVE_T))THEN
         DEALLOCATE(LRLU_SOLVE_T)
      ENDIF
      IF(allocated(LRLU_SOLVE_B))THEN
         DEALLOCATE(LRLU_SOLVE_B)
      ENDIF
      IF(allocated(POSFAC_SOLVE))THEN
         DEALLOCATE(POSFAC_SOLVE)
      ENDIF
      IF(allocated(IDEB_SOLVE_Z))THEN
         DEALLOCATE(IDEB_SOLVE_Z)
      ENDIF
      IF(allocated(PDEB_SOLVE_Z))THEN
         DEALLOCATE(PDEB_SOLVE_Z)
      ENDIF
      IF(allocated(SIZE_SOLVE_Z))THEN
         DEALLOCATE(SIZE_SOLVE_Z)
      ENDIF
      IF(allocated(CURRENT_POS_T))THEN
         DEALLOCATE(CURRENT_POS_T)
      ENDIF
      IF(allocated(CURRENT_POS_B))THEN
         DEALLOCATE(CURRENT_POS_B)
      ENDIF
      IF(allocated(POS_HOLE_T))THEN
         DEALLOCATE(POS_HOLE_T)
      ENDIF
      IF(allocated(POS_HOLE_B))THEN
         DEALLOCATE(POS_HOLE_B)
      ENDIF
      IF(allocated(OOC_STATE_NODE))THEN
         DEALLOCATE(OOC_STATE_NODE)
      ENDIF
      IF(allocated(POS_IN_MEM))THEN
         DEALLOCATE(POS_IN_MEM)
      ENDIF
      IF(allocated(INODE_TO_POS))THEN
         DEALLOCATE(INODE_TO_POS)
      ENDIF
      IF(allocated(IO_REQ))THEN
         DEALLOCATE(IO_REQ)
      ENDIF
      IF(allocated(SIZE_OF_READ))THEN
         DEALLOCATE(SIZE_OF_READ)
      ENDIF
      IF(allocated(FIRST_POS_IN_READ))THEN
         DEALLOCATE(FIRST_POS_IN_READ)
      ENDIF
      IF(allocated(READ_DEST))THEN
         DEALLOCATE(READ_DEST)
      ENDIF
      IF(allocated(READ_MNG))THEN
         DEALLOCATE(READ_MNG)
      ENDIF
      IF(allocated(REQ_TO_ZONE))THEN
         DEALLOCATE(REQ_TO_ZONE)
      ENDIF
      IF(allocated(REQ_ID))THEN
         DEALLOCATE(REQ_ID)
      ENDIF
      SOLVE_OR_FACTO=1
      CALL AGMG_NOCALL('MUMPS_CLEAN_IO_DATA_C')
      IF(IERR.LT.0)THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         RETURN
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_582
      SUBROUTINE DAGMG_MUMPS_612(PTRFAC,NSTEPS,                              &
     &            A,LA)
      IMPLICIT NONE
      INTEGER, INTENT(in)       :: NSTEPS
      INTEGER(8), INTENT(INOUT) :: PTRFAC(NSTEPS)
      INTEGER(8), INTENT(IN)    :: LA
      DOUBLE PRECISION                   :: A(LA)
      INTEGER    :: I, TMP, ZONE, IPAS, IBEG, IEND
      INTEGER(8) :: SAVE_PTR
      LOGICAL    :: COMPRESS_TO_BE_DONE, SET_POS_SEQUENCE
      INTEGER    :: J, IERR
      INTEGER(8) :: DUMMY_SIZE
      COMPRESS_TO_BE_DONE         = .FALSE.
      DUMMY_SIZE                  = 1_8
      IERR                        = 0
      SET_POS_SEQUENCE            = .TRUE.
      IF(SOLVE_STEP.EQ.0)THEN
        IBEG = 1
        IEND = TOTAL_NB_OOC_NODES(OOC_FCT_TYPE)
        IPAS = 1
      ELSE
        IBEG = TOTAL_NB_OOC_NODES(OOC_FCT_TYPE)
        IEND = 1
        IPAS = -1
      ENDIF
      DO I=IBEG,IEND,IPAS
            J = OOC_INODE_SEQUENCE(I,OOC_FCT_TYPE)
            TMP=INODE_TO_POS(STEP_OOC(J))
            IF(TMP.EQ.0)THEN
               IF (SET_POS_SEQUENCE) THEN
                 SET_POS_SEQUENCE = .FALSE.
                 CUR_POS_SEQUENCE = I
               ENDIF
               IF (KEEP_OOC(237).EQ.0 .AND. KEEP_OOC(235).EQ.0) THEN
                 OOC_STATE_NODE(STEP_OOC(J)) = NOT_IN_MEM
               ENDIF
               CYCLE
            ELSE IF(TMP.LT.0)THEN
               IF(TMP.GT.-(N_OOC+1)*NB_Z)THEN
                  SAVE_PTR=PTRFAC(STEP_OOC(J))
                  PTRFAC(STEP_OOC(J)) = abs(SAVE_PTR)
                  CALL DAGMG_MUMPS_600(J,                                    &
     &                 ZONE,PTRFAC,NSTEPS)
                  PTRFAC(STEP_OOC(J)) = SAVE_PTR
                  IF(ZONE.EQ.NB_Z)THEN
                     IF(J.NE.SPECIAL_ROOT_NODE)THEN
                        WRITE(*,*)MYID_OOC,': Internal error 6 ',       &
     &                       ' Node ', J,                               &
     &                       ' is in status USED in the                 &
     &                        emmergency buffer '
                        CALL AGMG_MUMPS_ABORT()
                     ENDIF
                  ENDIF
                 IF (KEEP_OOC(237).EQ.0 .AND. KEEP_OOC(235).EQ.0) THEN
                    CALL DAGMG_MUMPS_599(J,PTRFAC,NSTEPS)
                 ENDIF
               ENDIF
            ENDIF
      ENDDO
      RETURN
      END SUBROUTINE DAGMG_MUMPS_612
      SUBROUTINE DAGMG_MUMPS_583(PTRFAC,NSTEPS,MTYPE,                        &
     &                                    A,LA,DOPREFETCH,IERR)
      IMPLICIT NONE
      INTEGER NSTEPS,MTYPE
      INTEGER, intent(out)::IERR
      INTEGER(8) :: LA
      DOUBLE PRECISION A(LA)
      INTEGER(8) :: PTRFAC(NSTEPS)
      LOGICAL DOPREFETCH
      IERR = 0
      OOC_FCT_TYPE=1
      OOC_SOLVE_TYPE_FCT=FCT
      IF (KEEP_OOC(201).EQ.1) THEN
       OOC_FCT_TYPE=1
       OOC_SOLVE_TYPE_FCT=TYPEF_L-1
      ENDIF
      IF (KEEP_OOC(201).EQ.1) THEN
       IF((MTYPE.NE.1).AND.(KEEP_OOC(50).EQ.0))THEN
         OOC_SOLVE_TYPE_FCT=TYPEF_U-1
         OOC_FCT_TYPE=2
       ENDIF
      ENDIF
      SOLVE_STEP=0
      CUR_POS_SEQUENCE=1
      MTYPE_OOC=MTYPE
      IF ( KEEP_OOC(201).NE.1                                           &
     &  .OR. KEEP_OOC(50).NE.0                                          &
     &  ) THEN
        CALL DAGMG_MUMPS_612(PTRFAC,NSTEPS,A,LA)
      ELSE
        CALL DAGMG_MUMPS_683(KEEP_OOC(28),                                   &
     &              KEEP_OOC(38), KEEP_OOC(20) )
      ENDIF
      IF (DOPREFETCH) THEN
          CALL DAGMG_MUMPS_585(A,LA,PTRFAC,                                  &
     &                                 KEEP_OOC(28),IERR)
      ELSE
          CUR_POS_SEQUENCE = TOTAL_NB_OOC_NODES(OOC_FCT_TYPE)
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_583
      SUBROUTINE DAGMG_MUMPS_584(PTRFAC,NSTEPS,MTYPE,                        &
     &           I_WORKED_ON_ROOT,IROOT,A,LA,IERR)
      IMPLICIT NONE
      INTEGER NSTEPS
      INTEGER(8) :: LA
      INTEGER(8) :: PTRFAC(NSTEPS)
      INTEGER MTYPE
      INTEGER IROOT
      LOGICAL I_WORKED_ON_ROOT
      INTEGER, intent(out):: IERR
      DOUBLE PRECISION A(LA)
      INTEGER(8) :: DUMMY_SIZE
      INTEGER ZONE
      IERR=0
      OOC_FCT_TYPE=1
      OOC_SOLVE_TYPE_FCT=FCT
      IF (KEEP_OOC(201).EQ.1) THEN
       IF(KEEP_OOC(50).EQ.0)THEN
         OOC_SOLVE_TYPE_FCT=TYPEF_U-1
         OOC_FCT_TYPE=2
       ELSE
         OOC_SOLVE_TYPE_FCT=TYPEF_L-1
         OOC_FCT_TYPE=1
       ENDIF
      ENDIF
      IF (KEEP_OOC(201).EQ.1) THEN
       IF((MTYPE.NE.1).AND.(KEEP_OOC(50).EQ.0))THEN
         OOC_FCT_TYPE=1
         OOC_SOLVE_TYPE_FCT=TYPEF_L-1
       ENDIF
      ENDIF
      SOLVE_STEP=1
      CUR_POS_SEQUENCE=TOTAL_NB_OOC_NODES(OOC_FCT_TYPE)
      MTYPE_OOC=MTYPE
      IF ( KEEP_OOC(201).NE.1                                           &
     &  .OR. KEEP_OOC(50).NE.0                                          &
     &  ) THEN
        CALL DAGMG_MUMPS_612(PTRFAC,NSTEPS,A,LA)
        IF (I_WORKED_ON_ROOT) THEN
          CALL DAGMG_MUMPS_598( IROOT,                                       &
     &         PTRFAC, KEEP_OOC(28), A, LA,.FALSE.,IERR)
          IF (IERR .LT. 0) RETURN
          CALL DAGMG_MUMPS_600(IROOT,                                        &
     &         ZONE,PTRFAC,NSTEPS)
          IF(IROOT.EQ.NB_Z)THEN
             DUMMY_SIZE=1_8
             CALL DAGMG_MUMPS_608(A,LA,                                      &
     &            DUMMY_SIZE,PTRFAC,                                    &
     &            NSTEPS,NB_Z,IERR)
             IF (IERR .LT. 0) THEN
                WRITE(*,*)MYID_OOC,': Internal error in                 &
     &               DAGMG_MUMPS_608',                                       &
     &               IERR
                CALL AGMG_MUMPS_ABORT()
             ENDIF
          ENDIF
        ENDIF
        IF (NB_Z.GT.1) THEN
          CALL DAGMG_MUMPS_594(A,LA,PTRFAC,                                  &
     &                                  KEEP_OOC(28),IERR)
          IF (IERR .LT. 0) RETURN
        ENDIF
      ELSE
        CALL DAGMG_MUMPS_683(KEEP_OOC(28),                                   &
     &              KEEP_OOC(38), KEEP_OOC(20) )
        CALL DAGMG_MUMPS_585(A,LA,PTRFAC,KEEP_OOC(28),IERR)
        IF (IERR .LT. 0 ) RETURN
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_584
      SUBROUTINE DAGMG_MUMPS_613(id,IERR)
      USE DAGMG_MUMPS_STRUC_DF
      IMPLICIT NONE
      TYPE(DAGMG_MUMPS_STRUC), TARGET :: id
      INTEGER, intent(out) :: IERR
      INTEGER I,DIM,J,TMP,SIZE,K,I1
      CHARACTER*1 TMP_NAME(350)
      IERR=0
      SIZE=0
      DO J=1,OOC_NB_FILE_TYPE
         TMP=J-1
         CALL AGMG_NOCALL('MUMPS_OOC_GET_NB_FILES_C')
         id%OOC_NB_FILES(J)=I
         SIZE=SIZE+I
      ENDDO
      IF(associated(id%OOC_FILE_NAMES))THEN
         DEALLOCATE(id%OOC_FILE_NAMES)
         NULLIFY(id%OOC_FILE_NAMES)
      ENDIF
      ALLOCATE(id%OOC_FILE_NAMES(SIZE,350),stat=IERR)
      IF (IERR .GT. 0) THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*) 'PB allocation in DMUMPS_613'
         IERR=-1
         IF(id%INFO(1).GE.0)THEN
            id%INFO(1) = -13
            id%INFO(2) = SIZE*350
            RETURN
         ENDIF
      ENDIF
      IF(associated(id%OOC_FILE_NAME_LENGTH))THEN
         DEALLOCATE(id%OOC_FILE_NAME_LENGTH)
         NULLIFY(id%OOC_FILE_NAME_LENGTH)
      ENDIF
      ALLOCATE(id%OOC_FILE_NAME_LENGTH(SIZE),stat=IERR)
      IF (IERR .GT. 0) THEN
         IERR=-1
         IF(id%INFO(1).GE.0)THEN
            IF (ICNTL1.GT.0)                                            &
     &      WRITE(ICNTL1,*)                                             &
     &      'PB allocation in DMUMPS_613'
            id%INFO(1) = -13
            id%INFO(2) = SIZE
            RETURN
         ENDIF
      ENDIF
      K=1
      DO I1=1,OOC_NB_FILE_TYPE
         TMP=I1-1
         DO I=1,id%OOC_NB_FILES(I1)
            CALL AGMG_NOCALL('MUMPS_OOC_GET_FILE_NAME_C')
            DO J=1,DIM+1
               id%OOC_FILE_NAMES(K,J)=TMP_NAME(J)
            ENDDO
            id%OOC_FILE_NAME_LENGTH(K)=DIM+1
            K=K+1
         ENDDO
      ENDDO
      END SUBROUTINE DAGMG_MUMPS_613
      SUBROUTINE DAGMG_MUMPS_614(id)
      USE DAGMG_MUMPS_STRUC_DF
      IMPLICIT NONE
      TYPE(DAGMG_MUMPS_STRUC), TARGET :: id
      CHARACTER*1 TMP_NAME(350)
      INTEGER I,I1,TMP,J,K,L,DIM,IERR
      INTEGER, DIMENSION(:),ALLOCATABLE :: NB_FILES
      INTEGER K211
      ALLOCATE(NB_FILES(OOC_NB_FILE_TYPE),stat=IERR)
      IF (IERR .GT. 0) THEN
         IERR=-1
         IF(id%INFO(1).GE.0)THEN
            IF (ICNTL1.GT.0)                                            &
     &      WRITE(ICNTL1,*)                                             &
     &      'PB allocation in DMUMPS_614'
            id%INFO(1) = -13
            id%INFO(2) = OOC_NB_FILE_TYPE
            RETURN
         ENDIF
      ENDIF
      IERR=0
      NB_FILES=id%OOC_NB_FILES
      I=id%MYID
      K=id%KEEP(35)
      L=mod(id%KEEP(204),3)
      K211=id%KEEP(211)
      CALL AGMG_NOCALL('MUMPS_OOC_ALLOC_POINTERS_C')
      IF(IERR.LT.0)THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         id%INFO(1)=IERR
         RETURN
      ENDIF
      CALL AGMG_NOCALL('MUMPS_OOC_INIT_VARS_C')
      IF(IERR.LT.0)THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         id%INFO(1)=IERR
         RETURN
      ENDIF
      K=1
      DO I1=1,OOC_NB_FILE_TYPE
         DO I=1,NB_FILES(I1)
            DIM=id%OOC_FILE_NAME_LENGTH(K)
            DO J=1,DIM
               TMP_NAME(J)=id%OOC_FILE_NAMES(K,J)
            ENDDO
            TMP=I1-1
            CALL AGMG_NOCALL('MUMPS_OOC_SET_FILE_NAME_C')
            IF(IERR.LT.0)THEN
               IF (ICNTL1.GT.0)                                         &
     &         WRITE(ICNTL1,*)MYID_OOC,': ',                            &
     &         ERR_STR_OOC(1:DIM_ERR_STR_OOC)
               id%INFO(1)=IERR
               RETURN
            ENDIF
            K=K+1
         ENDDO
      ENDDO
      CALL AGMG_NOCALL('MUMPS_OOC_START_LOW_LEVEL')
      IF(IERR.LT.0)THEN
         IF (ICNTL1.GT.0)                                               &
     &   WRITE(ICNTL1,*)MYID_OOC,': ',ERR_STR_OOC(1:DIM_ERR_STR_OOC)
         id%INFO(1)=IERR
         RETURN
      ENDIF
      DEALLOCATE(NB_FILES)
      RETURN
      END SUBROUTINE DAGMG_MUMPS_614
      SUBROUTINE DAGMG_MUMPS_589(DEST,SRC,NB,NB_EFF)
      IMPLICIT NONE
      INTEGER NB, NB_EFF
      CHARACTER(LEN=NB) SRC
      CHARACTER*1 DEST(NB)
      INTEGER I
      DO I=1,NB_EFF
         DEST(I)=SRC(I:I)
      ENDDO
      END SUBROUTINE DAGMG_MUMPS_589
      SUBROUTINE DAGMG_MUMPS_580(IERR)
      USE DAGMG_MUMPS_OOC_BUFF
      IMPLICIT NONE
      INTEGER, intent(out) :: IERR
      IERR=0
      IF(.NOT.WITH_BUF)THEN
         RETURN
      ENDIF
      CALL DAGMG_MUMPS_707(OOC_FCT_TYPE,IERR)
      IF (IERR < 0) THEN
        RETURN
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_580
      SUBROUTINE DAGMG_MUMPS_681(IERR)
      USE DAGMG_MUMPS_OOC_BUFF
      IMPLICIT NONE
      INTEGER, intent(out) :: IERR
      IERR=0
      IF(.NOT.WITH_BUF)THEN
         RETURN
      ENDIF
      CALL DAGMG_MUMPS_707(TYPEF_L_LOC,IERR)
      IF (IERR < 0) RETURN
      IF (KEEP_OOC(50).EQ.0) THEN
         CALL DAGMG_MUMPS_707(TYPEF_U_LOC,IERR)
         IF (IERR < 0) RETURN
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_681
       SUBROUTINE DAGMG_MUMPS_683(NSTEPS,                                    &
     &     KEEP38, KEEP20)
      IMPLICIT NONE
      INTEGER MASTER
      PARAMETER(MASTER=0)
      INTEGER NSTEPS
      INTEGER I, TMPNODE, TMP, ZONE, J
      INTEGER(8) :: TMP_SIZE8
      INTEGER KEEP38, KEEP20
      INODE_TO_POS = 0
      POS_IN_MEM   = 0
      OOC_STATE_NODE(1:NSTEPS)=0
      TMP_SIZE8=1_8
      J=1
      DO I=1,NB_Z-1
         IDEB_SOLVE_Z(I)=TMP_SIZE8
         PDEB_SOLVE_Z(I)=J
         POSFAC_SOLVE(I)=TMP_SIZE8
         LRLUS_SOLVE(I) =SIZE_ZONE_SOLVE
         LRLU_SOLVE_T(I)=SIZE_ZONE_SOLVE
         LRLU_SOLVE_B(I)=0_8
         SIZE_SOLVE_Z(I)=SIZE_ZONE_SOLVE
         CURRENT_POS_T(I)=J
         CURRENT_POS_B(I)=J
         POS_HOLE_T(I)   =J
         POS_HOLE_B(I)   =J
         J = J + MAX_NB_NODES_FOR_ZONE
         TMP_SIZE8 = TMP_SIZE8 + SIZE_ZONE_SOLVE
      ENDDO
      IDEB_SOLVE_Z(NB_Z)=TMP_SIZE8
      PDEB_SOLVE_Z(NB_Z)=J
      POSFAC_SOLVE(NB_Z)=TMP_SIZE8
      LRLUS_SOLVE(NB_Z) =SIZE_SOLVE_EMM
      LRLU_SOLVE_T(NB_Z)=SIZE_SOLVE_EMM
      LRLU_SOLVE_B(NB_Z)=0_8
      SIZE_SOLVE_Z(NB_Z)=SIZE_SOLVE_EMM
      CURRENT_POS_T(NB_Z)=J
      CURRENT_POS_B(NB_Z)=J
      POS_HOLE_T(NB_Z)   =J
      POS_HOLE_B(NB_Z)   =J
      IO_REQ=-77777
      SIZE_OF_READ=-9999_8
      FIRST_POS_IN_READ=-9999
      READ_DEST=-9999_8
      READ_MNG=-9999
      REQ_TO_ZONE=-9999
      REQ_ID=-9999
      RETURN
      END SUBROUTINE DAGMG_MUMPS_683
      SUBROUTINE DAGMG_MUMPS_688                                             &
     &     ( STRAT, TYPEFile,                                           &
     &     AFAC, LAFAC, MonBloc,                                        &
     &     LNextPiv2beWritten, UNextPiv2beWritten,                      &
     &     IW, LIWFAC,                                                  &
     &     MYID, FILESIZE, IERR , LAST_CALL)
      IMPLICIT NONE
      TYPE(IO_BLOCK), INTENT(INOUT):: MonBloc
      INTEGER(8) :: LAFAC
      INTEGER,        INTENT(IN)   :: STRAT, LIWFAC,                    &
     &     MYID, TYPEFile
      INTEGER,   INTENT(INOUT)        :: IW(0:LIWFAC-1)
      DOUBLE PRECISION, INTENT(IN) :: AFAC(LAFAC)
      INTEGER,   INTENT(INOUT) :: LNextPiv2beWritten,                   &
     &     UNextPiv2beWritten
      INTEGER*8, INTENT(INOUT) :: FILESIZE
      INTEGER,   INTENT(OUT) :: IERR
      LOGICAL,   INTENT(IN)  :: LAST_CALL
      INTEGER(8) :: L_TailleEcrite, U_TailleEcrite
      INTEGER(8) :: L_AddVirtNodeI8
      INTEGER(8) :: U_AddVirtNodeI8
      INTEGER(8) :: TMPSIZE_OF_BLOCK
      INTEGER :: TempFTYPE
      LOGICAL WRITE_L, WRITE_U
      LOGICAL DO_U_FIRST
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER XXI, XXR, XXS, XXN, XXP
      PARAMETER(XXI=0,XXR=1,XXS=3,XXN=4,XXP=5)
      INTEGER XXNDIAG2W
      PARAMETER(XXNDIAG2W=6)
      INTEGER XSIZE_IC, XSIZE_OOC_SYM, XSIZE_OOC_UNSYM
      INTEGER XSIZE_OOC_NOPANEL
      PARAMETER (XSIZE_IC=6,XSIZE_OOC_SYM=7,XSIZE_OOC_UNSYM=7,          &
     &           XSIZE_OOC_NOPANEL=6)
      INTEGER IXSZ
      PARAMETER(IXSZ= 222)
      INTEGER S_CB1COMP
      PARAMETER (S_CB1COMP=314)
      INTEGER S_ACTIVE, S_ALL, S_NOLCBCONTIG,                           &
     &        S_NOLCBNOCONTIG, S_NOLCLEANED,                            &
     &        S_NOLCBNOCONTIG38, S_NOLCBCONTIG38,                       &
     &        S_NOLCLEANED38, C_FINI
      PARAMETER(S_ACTIVE=400, S_ALL=401, S_NOLCBCONTIG=402,             &
     &          S_NOLCBNOCONTIG=403, S_NOLCLEANED=404,                  &
     &          S_NOLCBNOCONTIG38=405, S_NOLCBCONTIG38=406,             &
     &          S_NOLCLEANED38=407,C_FINI=1)
      INTEGER S_FREE, S_NOTFREE
      PARAMETER(S_FREE=54321,S_NOTFREE=-123456)
      INTEGER TOP_OF_STACK
      PARAMETER(TOP_OF_STACK=-999999)
      INTEGER XTRA_SLAVES_SYM, XTRA_SLAVES_UNSYM
      PARAMETER(XTRA_SLAVES_SYM=3, XTRA_SLAVES_UNSYM=1)
         INTEGER S_ROOT2SON_CALLED, S_REC_CONTSTATIC,                   &
     &  S_ROOTBAND_INIT
         PARAMETER(S_ROOT2SON_CALLED=-341,S_REC_CONTSTATIC=1,           &
     &             S_ROOTBAND_INIT=0)
      IERR = 0
      WRITE_L =  (TYPEFile.EQ.TYPEF_BOTH_LU .OR. TYPEFile.EQ.TYPEF_L)
      WRITE_U =  (TYPEFile.EQ.TYPEF_BOTH_LU .OR. TYPEFile.EQ.TYPEF_U)
      DO_U_FIRST = .FALSE.
      IF ( TYPEFile.EQ.TYPEF_BOTH_LU ) THEN
         IF ( LNextPiv2bewritten .GT. UNextPiv2beWritten ) THEN
            DO_U_FIRST = .TRUE.
         END IF
      END IF
      IF (DO_U_FIRST) GOTO 200
  100 IF (WRITE_L) THEN
         TempFTYPE  = TYPEF_L
         IF ((MonBloc%Typenode.EQ.2).AND.(.NOT.MonBloc%MASTER))         &
     &        THEN
           TMPSIZE_OF_BLOCK = SIZE_OF_BLOCK(STEP_OOC(MonBloc%INODE),    &
     &                                      TempFTYPE)
           IF (TMPSIZE_OF_BLOCK .LT. 0_8) THEN
               TMPSIZE_OF_BLOCK = -TMPSIZE_OF_BLOCK - 1_8
           ENDIF
           LNextPiv2beWritten =                                         &
     &     int(                                                         &
     &          TMPSIZE_OF_BLOCK                                        &
     &          / int(MonBloc%NROW,8)                                   &
     &        )                                                         &
     &     + 1
         ENDIF
         CALL DAGMG_MUMPS_695( STRAT,                                        &
     &        TempFTYPE, AFAC, LAFAC, MonBloc,                          &
     &        IERR,                                                     &
     &        LNextPiv2beWritten,                                       &
     &        OOC_VADDR(STEP_OOC(MonBloc%INODE),TEMPFTYPE),             &
     &        SIZE_OF_BLOCK(STEP_OOC(MonBloc%INODE),TempFTYPE),         &
     &        FILESIZE, LAST_CALL )
         IF (IERR .LT. 0) RETURN
         IF (DO_U_FIRST) GOTO 300
      ENDIF
  200 IF (WRITE_U) THEN
         TempFTYPE  = TYPEF_U
         CALL DAGMG_MUMPS_695( STRAT,                                        &
     &        TempFTYPE, AFAC, LAFAC, MonBloc,                          &
     &        IERR,                                                     &
     &        UNextPiv2beWritten,                                       &
     &        OOC_VADDR(STEP_OOC(MonBloc%INODE),TEMPFTYPE),             &
     &        SIZE_OF_BLOCK(STEP_OOC(MonBloc%INODE),TempFTYPE),         &
     &        FILESIZE, LAST_CALL)
         IF (IERR .LT. 0) RETURN
         IF (DO_U_FIRST) GOTO 100
      ENDIF
  300 CONTINUE
      RETURN
      END SUBROUTINE DAGMG_MUMPS_688
      SUBROUTINE DAGMG_MUMPS_695( STRAT, TYPEF,                              &
     &     AFAC, LAFAC, MonBloc,                                        &
     &     IERR,                                                        &
     &     LorU_NextPiv2beWritten,                                      &
     &     LorU_AddVirtNodeI8, LorUSIZE_OF_BLOCK,                       &
     &     FILESIZE, LAST_CALL                                          &
     &     )
      USE DAGMG_MUMPS_OOC_BUFF
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: STRAT
      INTEGER, INTENT(IN) :: TYPEF
      INTEGER(8), INTENT(INOUT) :: FILESIZE
      INTEGER(8), INTENT(IN) :: LAFAC
      DOUBLE PRECISION, INTENT(IN) :: AFAC(LAFAC)
      INTEGER, INTENT(INOUT) :: LorU_NextPiv2beWritten
      INTEGER(8), INTENT(INOUT) :: LorU_AddVirtNodeI8
      INTEGER(8), INTENT(INOUT) :: LorUSIZE_OF_BLOCK
      TYPE(IO_BLOCK), INTENT(INOUT) :: MonBloc
      INTEGER, INTENT(OUT)  :: IERR
      LOGICAL, INTENT(IN)   :: LAST_CALL
      INTEGER NNMAX
      INTEGER(8) :: TOTSIZE, EFFSIZE
      INTEGER(8) :: TailleEcrite
      INTEGER SIZE_PANEL
      INTEGER(8) :: AddVirtCour
      LOGICAL VIRT_ADD_RESERVED_BEF_CALL
      LOGICAL VIRTUAL_ADDRESS_JUST_RESERVED
      LOGICAL HOLE_PROCESSED_BEFORE_CALL
      LOGICAL TMP_ESTIM
      INTEGER ICUR, INODE_CUR, ILAST
      INTEGER(8) :: ADDR_LAST
      IERR = 0
      IF (TYPEF == TYPEF_L ) THEN
         NNMAX = MonBloc%NROW
      ELSE
         NNMAX = MonBloc%NCOL
      ENDIF
      SIZE_PANEL = DAGMG_MUMPS_690(NNMAX)
      IF ( (.NOT.MonBloc%Last) .AND.                                    &
     &     (MonBloc%LastPiv-LorU_NextPiv2beWritten+1.LT.SIZE_PANEL))    &
     &     THEN
        RETURN
      ENDIF
      TMP_ESTIM = .TRUE.
      TOTSIZE = DAGMG_MUMPS_725                                              &
     &          (MonBloc%NFS, NNMAX, SIZE_PANEL, MonBloc, TMP_ESTIM)
      IF (MonBloc%Last) THEN
           TMP_ESTIM=.FALSE.
           EFFSIZE = DAGMG_MUMPS_725                                         &
     &     (MonBloc%LastPiv, NNMAX, SIZE_PANEL, MonBloc, TMP_ESTIM)
      ELSE
            EFFSIZE = -1034039740327_8
      ENDIF
      IF (MonBloc%Typenode.EQ.3.AND. MonBloc%NFS.NE.MonBloc%NCOL) THEN
         WRITE(*,*) 'Internal error in DMUMPS_695 for type3',           &
     &   MonBloc%NFS,MonBloc%NCOL
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF (MonBloc%Typenode.EQ.3.AND. TYPEF.NE.TYPEF_L) THEN
         WRITE(*,*) 'Internal error in DMUMPS_695,TYPEF=',              &
     &   TYPEF, 'for typenode=3'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF (MonBloc%Typenode.EQ.2.AND.                                    &
     &     TYPEF.EQ.TYPEF_U.AND.                                        &
     &     .NOT. MonBloc%MASTER ) THEN
         WRITE(*,*) 'Internal error in DMUMPS_695',                     &
     &   MonBloc%MASTER,MonBloc%TYPENODE, TYPEF
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      HOLE_PROCESSED_BEFORE_CALL  = (LorUSIZE_OF_BLOCK .LT. 0_8)
      IF (HOLE_PROCESSED_BEFORE_CALL.AND.(.NOT.MonBloc%Last)) THEN
          WRITE(6,*) ' Internal error  in DMUMPS_695 ',                 &
     &    ' last is false after earlier calls with last=true'
          CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF (HOLE_PROCESSED_BEFORE_CALL) THEN
        LorUSIZE_OF_BLOCK = - LorUSIZE_OF_BLOCK - 1_8
        TOTSIZE = -99999999_8
      ENDIF
      VIRTUAL_ADDRESS_JUST_RESERVED = .FALSE.
      VIRT_ADD_RESERVED_BEF_CALL =                                      &
     &                    ( LorUSIZE_OF_BLOCK .NE. 0_8 .OR.             &
     &                      HOLE_PROCESSED_BEFORE_CALL )
      IF (MonBloc%Last .AND. .NOT. HOLE_PROCESSED_BEFORE_CALL) THEN
        KEEP_OOC(228) = max(KEEP_OOC(228),                              &
     &        (MonBloc%LastPiv+SIZE_PANEL-1) / SIZE_PANEL)
        IF (VIRT_ADD_RESERVED_BEF_CALL) THEN
            IF (AddVirtLibre(TYPEF).EQ.                                 &
     &           (LorU_AddVirtNodeI8+TOTSIZE) ) THEN
              AddVirtLibre(TYPEF) = LorU_AddVirtNodeI8 + EFFSIZE
            ENDIF
        ELSE
            VIRTUAL_ADDRESS_JUST_RESERVED = .TRUE.
            IF (EFFSIZE .EQ. 0_8) THEN
              LorU_AddVirtNodeI8 = -9999_8
            ELSE
              LorU_AddVirtNodeI8  = AddVirtLibre(TYPEF)
            ENDIF
            AddVirtLibre(TYPEF) = AddVirtLibre(TYPEF) + EFFSIZE
        ENDIF
      ELSE
        IF (.NOT. VIRT_ADD_RESERVED_BEF_CALL                            &
     &    ) THEN
          LorU_AddVirtNodeI8 = AddVirtLibre(TYPEF)
          AddVirtLibre(TYPEF) = AddVirtLibre(TYPEF) + TOTSIZE
        ENDIF
      ENDIF
      AddVirtCour = LorU_AddVirtNodeI8 + LorUSIZE_OF_BLOCK
      CALL DAGMG_MUMPS_697( STRAT, TYPEF, MonBloc,                           &
     &     SIZE_PANEL,                                                  &
     &     AFAC, LAFAC,                                                 &
     &     LorU_NextPiv2beWritten, AddVirtCour,                         &
     &     TailleEcrite,                                                &
     &     IERR )
      IF ( IERR .LT. 0 ) RETURN
      LorUSIZE_OF_BLOCK = LorUSIZE_OF_BLOCK + TailleEcrite
      IF (LorUSIZE_OF_BLOCK.EQ.0_8 ) THEN
        IF ( .NOT. VIRT_ADD_RESERVED_BEF_CALL                           &
     &    .AND. .NOT. VIRTUAL_ADDRESS_JUST_RESERVED )                   &
     &    THEN
          AddVirtLibre(TYPEF) = AddVirtLibre(TYPEF) - TOTSIZE
          LorU_AddVirtNodeI8 = 0_8
        ENDIF
      ELSE IF (.NOT. VIRT_ADD_RESERVED_BEF_CALL ) THEN
          VIRTUAL_ADDRESS_JUST_RESERVED = .TRUE.
      ENDIF
      IF ( VIRTUAL_ADDRESS_JUST_RESERVED) THEN
         OOC_INODE_SEQUENCE(I_CUR_HBUF_NEXTPOS(TYPEF),                  &
     &        TYPEF) = MonBloc%INODE
         I_CUR_HBUF_NEXTPOS(TYPEF) = I_CUR_HBUF_NEXTPOS(TYPEF) + 1
         IF (MonBloc%Last) THEN
           MAX_SIZE_FACTOR_OOC=max(MAX_SIZE_FACTOR_OOC,EFFSIZE)
           TMP_SIZE_FACT=TMP_SIZE_FACT+EFFSIZE
         ELSE
           MAX_SIZE_FACTOR_OOC=max(MAX_SIZE_FACTOR_OOC,TOTSIZE)
           TMP_SIZE_FACT=TMP_SIZE_FACT+TOTSIZE
         ENDIF
         TMP_NB_NODES=TMP_NB_NODES+1
         IF(TMP_SIZE_FACT.GT.SIZE_ZONE_SOLVE)THEN
            MAX_NB_NODES_FOR_ZONE=max(MAX_NB_NODES_FOR_ZONE,            &
     &           TMP_NB_NODES)
            TMP_SIZE_FACT=0_8
            TMP_NB_NODES=0
         ENDIF
      ENDIF
      IF (MonBloc%Last) THEN
        LorUSIZE_OF_BLOCK = - LorUSIZE_OF_BLOCK - 1_8
      ENDIF
      IF (LAST_CALL) THEN
        IF (.NOT.MonBloc%Last) THEN
          WRITE(6,*) ' Internal error in DMUMPS_695 ',                  &
     &               ' LAST and LAST_CALL are incompatible '
         CALL AGMG_MUMPS_ABORT()
        ENDIF
        LorUSIZE_OF_BLOCK = - LorUSIZE_OF_BLOCK - 1_8
        ICUR      = I_CUR_HBUF_NEXTPOS(TYPEF) - 1
        INODE_CUR = OOC_INODE_SEQUENCE(ICUR,TYPEF)
        ADDR_LAST = AddVirtLibre(TYPEF)
        IF (INODE_CUR .NE. MonBloc%INODE) THEN
   10     CONTINUE
          ILAST = ICUR
          IF ( OOC_VADDR(STEP_OOC(INODE_CUR),TYPEF) .NE. -9999_8) THEN
            ADDR_LAST = OOC_VADDR(STEP_OOC(INODE_CUR), TYPEF)
          ENDIF
          ICUR = ICUR - 1
          INODE_CUR = OOC_INODE_SEQUENCE(ICUR,TYPEF)
          IF (INODE_CUR .EQ. MonBloc%INODE) THEN
            LorUSIZE_OF_BLOCK = ADDR_LAST -                             &
     &                          OOC_VADDR(STEP_OOC(INODE_CUR),TYPEF)
          ELSE
            IF (ICUR .LE. 1) THEN
              WRITE(*,*) "Internal error in DAGMG_MUMPS_695"
              WRITE(*,*) "Did not find current node in sequence"
              CALL AGMG_MUMPS_ABORT()
            ENDIF
            GOTO 10
          ENDIF
        ENDIF
        FILESIZE  = FILESIZE + LorUSIZE_OF_BLOCK
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_695
      SUBROUTINE DAGMG_MUMPS_697(                                            &
     &     STRAT, TYPEF, MonBloc,                                       &
     &     SIZE_PANEL,                                                  &
     &     AFAC, LAFAC,                                                 &
     &     NextPiv2beWritten, AddVirtCour,                              &
     &     TailleEcrite, IERR )
      USE DAGMG_MUMPS_OOC_BUFF
      IMPLICIT NONE
      INTEGER,       INTENT(IN) :: STRAT, TYPEF, SIZE_PANEL
      INTEGER(8)                :: LAFAC
      INTEGER*8,     INTENT(IN) ::  AddVirtCour
      DOUBLE PRECISION, INTENT(IN) :: AFAC(LAFAC)
      INTEGER,       INTENT(INOUT) :: NextPiv2beWritten
      TYPE(IO_BLOCK),INTENT(INOUT) :: MonBloc
      INTEGER(8),      INTENT(OUT) :: TailleEcrite
      INTEGER, INTENT(OUT)  :: IERR
      INTEGER   :: I, NBeff, LPANELeff, IEND
      INTEGER*8 :: AddVirtDeb
      IERR = 0
      TailleEcrite = 0_8
      AddVirtDeb   = AddVirtCour
      I = NextPiv2beWritten
      IF ( NextPiv2beWritten .GT. MonBloc%LastPiv ) THEN
        RETURN
      ENDIF
   10 CONTINUE
      NBeff  = min(SIZE_PANEL,MonBloc%LastPiv-I+1 )
      IF ((NBeff.NE.SIZE_PANEL) .AND. (.NOT.MonBloc%Last)) THEN
         GOTO 20
      ENDIF
      IF (TYPEF.EQ.TYPEF_L.AND.MonBloc%MASTER.AND.                      &
     &     KEEP_OOC(50).EQ.2 .AND. MonBloc%Typenode.NE.3) THEN
         IF (MonBloc%INDICES(NBeff+I-1) < 0)                            &
     &        THEN
            NBeff=NBeff+1
         ENDIF
      ENDIF
      IEND   = I + NBeff -1
      CALL DAGMG_MUMPS_653( STRAT, TYPEF, MonBloc,                           &
     &     AFAC, LAFAC,                                                 &
     &     AddVirtDeb, I, IEND, LPANELeff,                              &
     &     IERR)
      IF ( IERR .LT. 0 ) THEN
        RETURN
      ENDIF
      IF ( IERR .EQ. 1 ) THEN
         IERR=0
         GOTO 20
      ENDIF
      IF (TYPEF .EQ. TYPEF_L) THEN
         MonBloc%LastPanelWritten_L = MonBloc%LastPanelWritten_L+1
      ELSE
         MonBloc%LastPanelWritten_U = MonBloc%LastPanelWritten_U+1
      ENDIF
      AddVirtDeb   = AddVirtDeb + int(LPANELeff,8)
      TailleEcrite = TailleEcrite + int(LPANELeff,8)
      I=I+NBeff
      IF ( I .LE. MonBloc%LastPiv ) GOTO 10
   20 CONTINUE
      NextPiv2beWritten = I
      RETURN
      END SUBROUTINE DAGMG_MUMPS_697
      INTEGER(8) FUNCTION DAGMG_MUMPS_725                                    &
     &      (NFSorNPIV, NNMAX, SIZE_PANEL, MonBloc, ESTIM)
      IMPLICIT NONE
      TYPE(IO_BLOCK), INTENT(IN):: MonBloc
      INTEGER, INTENT(IN) :: NFSorNPIV, NNMAX, SIZE_PANEL
      LOGICAL, INTENT(IN) :: ESTIM
      INTEGER :: I, NBeff
      INTEGER(8) :: TOTSIZE
      TOTSIZE = 0_8
      IF (NFSorNPIV.EQ.0) GOTO 100
      IF (.NOT. MonBloc%MASTER .OR. MonBloc%TYPENODE.EQ.3) THEN
        TOTSIZE = int(NFSorNPIV,8) * int(NNMAX,8)
      ELSE
        I = 1
   10   CONTINUE
        NBeff = min(SIZE_PANEL, NFSorNPIV-I+1)
        IF (KEEP_OOC(50).EQ.2) THEN
          IF (ESTIM) THEN
            NBeff = NBeff + 1
          ELSE
             IF (MonBloc%INDICES(I+NBeff-1) < 0) THEN
            NBeff = NBeff + 1
            ENDIF
          ENDIF
        ENDIF
        TOTSIZE = TOTSIZE +                                             &
     &           int(NNMAX-I+1,8) * int(NBeff,8)
        I = I + NBeff
        IF ( I .LE. NFSorNPIV ) GOTO 10
      ENDIF
  100 CONTINUE
      DAGMG_MUMPS_725 = TOTSIZE
      RETURN
      END FUNCTION DAGMG_MUMPS_725
      INTEGER FUNCTION DAGMG_MUMPS_690( NNMAX )
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: NNMAX
      INTEGER DAGMG_MUMPS_748
      DAGMG_MUMPS_690=DAGMG_MUMPS_748(                                            &
     &     HBUF_SIZE, NNMAX, KEEP_OOC(227),KEEP_OOC(50))
      RETURN
      END FUNCTION DAGMG_MUMPS_690
      SUBROUTINE DAGMG_MUMPS_728()
      IMPLICIT NONE
      INTEGER I,TMP_NODE
      IF(.NOT.DAGMG_MUMPS_727())THEN
         IF(SOLVE_STEP.EQ.0)THEN
            I=CUR_POS_SEQUENCE
            TMP_NODE=OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE,               &
     &           OOC_FCT_TYPE)
            DO WHILE ((I.LE.TOTAL_NB_OOC_NODES(OOC_FCT_TYPE)).AND.      &
     &           (SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)        &
     &           .EQ.0_8))
               INODE_TO_POS(STEP_OOC(TMP_NODE))=1
               OOC_STATE_NODE(STEP_OOC(TMP_NODE))=NOT_USED
               I=I+1
               IF(I.LE.TOTAL_NB_OOC_NODES(OOC_FCT_TYPE))THEN
                  TMP_NODE=OOC_INODE_SEQUENCE(I,OOC_FCT_TYPE)
               ENDIF
            ENDDO
            CUR_POS_SEQUENCE=min(I,TOTAL_NB_OOC_NODES(OOC_FCT_TYPE))
         ELSE
            I=CUR_POS_SEQUENCE
            TMP_NODE=OOC_INODE_SEQUENCE(CUR_POS_SEQUENCE,               &
     &           OOC_FCT_TYPE)
            DO WHILE ((I.GE.1).AND.                                     &
     &           (SIZE_OF_BLOCK(STEP_OOC(TMP_NODE),OOC_FCT_TYPE)        &
     &           .EQ.0_8))
               INODE_TO_POS(STEP_OOC(TMP_NODE))=1
               OOC_STATE_NODE(STEP_OOC(TMP_NODE))=NOT_USED
               I=I-1
               IF(I.GE.1)THEN
                  TMP_NODE=OOC_INODE_SEQUENCE(I,OOC_FCT_TYPE)
               ENDIF
            ENDDO
            CUR_POS_SEQUENCE=max(I,1)
         ENDIF
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_728
      END MODULE DAGMG_MUMPS_OOC_MAIN
      ! insert 37 here
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      MODULE DAGMG_MUMPS_LD
      implicit none
      PUBLIC :: DAGMG_MUMPS_188, DAGMG_MUMPS_185,                                 &
     &       DAGMG_MUMPS_189,                                                &
     &       DAGMG_MUMPS_184, DAGMG_MUMPS_190,                                    &
     &       DAGMG_MUMPS_183, DAGMG_MUMPS_187,                                    &
     &       DAGMG_MUMPS_186,                                                &
     &       DAGMG_MUMPS_409,                                                &
     &       DAGMG_MUMPS_384, DAGMG_MUMPS_461,                                    &
     &       DAGMG_MUMPS_467, DAGMG_MUMPS_471,                                    &
     &       DAGMG_MUMPS_472,                                                &
     &       DAGMG_MUMPS_500,                                                &
     &       DAGMG_MUMPS_501,                                                &
     &       DAGMG_MUMPS_520,                                                &
     &       DAGMG_MUMPS_513,                                                &
     &       DAGMG_MUMPS_514, DAGMG_MUMPS_512                                     &
     &       ,DAGMG_MUMPS_533,                                               &
     &       CLEAN_POOL_MEM_INFO,COMPUTE_MAX_MEM,                       &
     &       CHECK_MEM_CONST_FOR_POOL,DAGMG_MUMPS_554,                       &
     &       DAGMG_MUMPS_553,                                                &
     &       DAGMG_MUMPS_555
      DOUBLE PRECISION, DIMENSION(:),                                   &
     &       ALLOCATABLE, SAVE, PRIVATE :: LOAD_FLOPS
      INTEGER, DIMENSION(:), ALLOCATABLE, SAVE, PRIVATE :: BUF_LOAD_RECV
      INTEGER, SAVE, PRIVATE :: LBUF_LOAD_RECV, LBUF_LOAD_RECV_BYTES
      INTEGER, SAVE, PRIVATE :: K50, K69, K35
      INTEGER(8), SAVE, PRIVATE :: MAX_SURF_MASTER
      LOGICAL, SAVE, PRIVATE :: BDC_MEM, BDC_POOL, BDC_SBTR,            &
     &     BDC_POOL_MNG,                                                &
     &     BDC_M2_MEM,BDC_M2_FLOPS,BDC_MD,REMOVE_NODE_FLAG,             &
     &     REMOVE_NODE_FLAG_MEM
      DOUBLE PRECISION, SAVE, PRIVATE :: REMOVE_NODE_COST,              &
     &     REMOVE_NODE_COST_MEM
      INTEGER, SAVE, PRIVATE :: SBTR_WHICH_M
      DOUBLE PRECISION, DIMENSION(:),                                   &
     &       ALLOCATABLE, TARGET, SAVE, PRIVATE :: WLOAD
      INTEGER, DIMENSION(:), ALLOCATABLE, SAVE, PRIVATE :: FUTURE_NIV2
      DOUBLE PRECISION, SAVE, PRIVATE :: DELTA_LOAD, DELTA_MEM
      INTEGER(8), SAVE, PRIVATE :: CHECK_MEM
      INTEGER, DIMENSION(:), ALLOCATABLE, SAVE, TARGET, PRIVATE ::      &
     &          IDWLOAD
      DOUBLE PRECISION, SAVE, PRIVATE :: COST_SUBTREE
      DOUBLE PRECISION, SAVE, PRIVATE :: ALPHA
      DOUBLE PRECISION, SAVE, PRIVATE :: BETA
      INTEGER, SAVE, PRIVATE :: MYID, NPROCS, COMM_LD
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, SAVE,                &
     &           PRIVATE :: POOL_MEM
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, PRIVATE,             &
     &           SAVE :: SBTR_MEM
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE,                      &
     &           PRIVATE, SAVE :: SBTR_CUR
      INTEGER, DIMENSION(:), ALLOCATABLE,                               &
     &           PRIVATE, SAVE :: NB_SON
      DOUBLE PRECISION,                                                 &
     &           PRIVATE, SAVE :: SBTR_CUR_LOCAL
      DOUBLE PRECISION,                                                 &
     &           PRIVATE, SAVE :: PEAK_SBTR_CUR_LOCAL
      DOUBLE PRECISION,                                                 &
     &           PRIVATE, SAVE :: MAX_PEAK_STK
      DOUBLE PRECISION, SAVE,                                           &
     &           PRIVATE :: POOL_LAST_COST_SENT
      DOUBLE PRECISION, SAVE,                                           &
     &           PRIVATE :: MIN_DIFF
      INTEGER, SAVE :: POS_ID,POS_MEM
      INTEGER, DIMENSION(:), ALLOCATABLE, SAVE :: CB_COST_ID
      INTEGER(8), DIMENSION(:), ALLOCATABLE, SAVE                       &
     &           :: CB_COST_MEM
      PUBLIC :: CB_COST_ID, CB_COST_MEM,POS_MEM,POS_ID
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: LU_USAGE
      INTEGER(8), DIMENSION(:), ALLOCATABLE, SAVE,                      &
     &        PRIVATE::MD_MEM, TAB_MAXS
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, SAVE ::MEM_SUBTREE
      INTEGER  :: NB_SUBTREES,NIV1_FLAG
      INTEGER, PRIVATE  :: INDICE_SBTR,INDICE_SBTR_ARRAY
      INTEGER,SAVE :: INSIDE_SUBTREE
      PUBLIC :: NB_SUBTREES,MEM_SUBTREE,INSIDE_SUBTREE,NIV1_FLAG
      DOUBLE PRECISION, SAVE, PRIVATE :: DM_SUMLU,                      &
     &                   DM_THRES_MEM
      DOUBLE PRECISION, DIMENSION(:),                                   &
     &   ALLOCATABLE, SAVE , PRIVATE:: DM_MEM
      INTEGER, SAVE, PRIVATE :: POOL_SIZE,ID_MAX_M2
      DOUBLE PRECISION, SAVE, PRIVATE :: MAX_M2,TMP_M2
      INTEGER, DIMENSION(:),ALLOCATABLE,SAVE, PRIVATE:: POOL_NIV2
      DOUBLE PRECISION, DIMENSION(:),ALLOCATABLE,SAVE,                  &
     &      PRIVATE :: POOL_NIV2_COST, NIV2
      DOUBLE PRECISION, SAVE, PRIVATE  ::      CHK_LD
      INTEGER, DIMENSION(:),POINTER, SAVE, PRIVATE  ::                  &
     &         KEEP_LOAD, PROCNODE_LOAD, STEP_TO_NIV2_LOAD
      INTEGER*8, DIMENSION(:), POINTER, SAVE, PRIVATE:: KEEP8_LOAD
      INTEGER, DIMENSION(:),POINTER, SAVE ::                            &
     &         FILS_LOAD, STEP_LOAD,                                    &
     &         FRERE_LOAD, ND_LOAD,                                     &
     &         NE_LOAD,DAD_LOAD
      INTEGER, DIMENSION(:,:),POINTER, SAVE, PRIVATE :: CAND_LOAD
      INTEGER, DIMENSION(:),POINTER, SAVE,                              &
     &         PRIVATE :: MY_FIRST_LEAF,MY_NB_LEAF, MY_ROOT_SBTR
      INTEGER, DIMENSION(:),ALLOCATABLE,SAVE,                           &
     &         PRIVATE ::SBTR_FIRST_POS_IN_POOL
      DOUBLE PRECISION, DIMENSION(:),ALLOCATABLE,SAVE,                  &
     &         PRIVATE ::SBTR_PEAK_ARRAY,                               &
     &     SBTR_CUR_ARRAY
      INTEGER, SAVE,                                                    &
     &         PRIVATE :: N_LOAD
      DOUBLE PRECISION,DIMENSION(:),POINTER, SAVE :: COST_TRAV
      INTEGER, DIMENSION(:),POINTER, SAVE :: DEPTH_FIRST_LOAD
      PUBLIC :: DEPTH_FIRST_LOAD,COST_TRAV, FILS_LOAD,STEP_LOAD,        &
     &     FRERE_LOAD, ND_LOAD,NE_LOAD,DAD_LOAD
      INTEGER, SAVE     :: ROOT_CURRENT_SUBTREE,CURRENT_BEST,           &
     &     SECOND_CURRENT_BEST
      PUBLIC :: ROOT_CURRENT_SUBTREE,CURRENT_BEST,                      &
     &     SECOND_CURRENT_BEST
      CONTAINS
      SUBROUTINE DAGMG_MUMPS_188( COST_SUBTREE_ARG, K64, K66,                &
     &     MAXS )
      IMPLICIT NONE
      DOUBLE PRECISION COST_SUBTREE_ARG
      INTEGER K64, K66
      INTEGER(8)::MAXS
      DOUBLE PRECISION T64, T66
      T64 = max ( dble(K64), dble(1) )
      T64 = min ( T64, dble(1000)  )
      T66 = max (dble(K66), dble(100))
      MIN_DIFF     =  ( T64 / dble(1000)  )*                            &
     &                  T66 * dble(1000000)
      DM_THRES_MEM = dble(MAXS/1000_8)
      COST_SUBTREE = COST_SUBTREE_ARG
      END SUBROUTINE DAGMG_MUMPS_188
      SUBROUTINE DAGMG_MUMPS_472(                                            &
     &  NCBSON_MAX, SLAVEF,                                             &
     &  KEEP,KEEP8,ICNTL,                                               &
     &  CAND_OF_NODE,                                                   &
     &  MEM_DISTRIB, NCB, NFRONT, NSLAVES_NODE,                         &
     &  TAB_POS, SLAVES_LIST, SIZE_SLAVES_LIST,INODE)
       IMPLICIT NONE
      INTEGER, intent(in) :: KEEP(500),SIZE_SLAVES_LIST
      INTEGER*8 KEEP8(150)
      INTEGER, intent(in) :: ICNTL(40)
      INTEGER, intent(in) :: SLAVEF, NFRONT, NCB
      INTEGER, intent(in) :: CAND_OF_NODE(SLAVEF+1)
      INTEGER, intent(in) :: MEM_DISTRIB(0:SLAVEF-1),INODE
      INTEGER, intent(in) :: NCBSON_MAX
      INTEGER, intent(out):: SLAVES_LIST(SIZE_SLAVES_LIST)
      INTEGER, intent(out):: TAB_POS(SLAVEF+2)
      INTEGER, intent(out):: NSLAVES_NODE
      INTEGER i
      INTEGER LP,MP
      LP=ICNTL(4)
      MP=ICNTL(2)
      IF ( KEEP(48) == 0 .OR. KEEP(48) .EQ. 3 ) THEN
         CALL DAGMG_MUMPS_499(                                               &
     &        SLAVEF,                                                   &
     &        KEEP,KEEP8,                                               &
     &        CAND_OF_NODE,                                             &
     &        MEM_DISTRIB, NCB, NFRONT, NSLAVES_NODE,                   &
     &        TAB_POS, SLAVES_LIST, SIZE_SLAVES_LIST)
      ELSE IF ( KEEP(48) == 4 ) THEN
         CALL DAGMG_MUMPS_504(                                               &
     &        SLAVEF,                                                   &
     &        KEEP,KEEP8,                                               &
     &        CAND_OF_NODE,                                             &
     &        MEM_DISTRIB, NCB, NFRONT, NSLAVES_NODE,                   &
     &        TAB_POS, SLAVES_LIST, SIZE_SLAVES_LIST,MYID)
         DO i=1,NSLAVES_NODE
            IF(TAB_POS(i+1)-TAB_POS(i).LE.0)THEN
               WRITE(*,*)'probleme de partition dans                    &
     &DAGMG_MUMPS_545'
               CALL AGMG_MUMPS_ABORT()
            ENDIF
         ENDDO
      ELSE IF ( KEEP(48) == 5 ) THEN
         CALL DAGMG_MUMPS_518(                                               &
     &        NCBSON_MAX,                                               &
     &        SLAVEF,                                                   &
     &        KEEP,KEEP8,                                               &
     &        CAND_OF_NODE,                                             &
     &        MEM_DISTRIB, NCB, NFRONT, NSLAVES_NODE,                   &
     &        TAB_POS, SLAVES_LIST, SIZE_SLAVES_LIST,MYID,INODE,        &
     &        MP,LP)
         DO i=1,NSLAVES_NODE
            IF(TAB_POS(i+1)-TAB_POS(i).LE.0)THEN
               WRITE(*,*)'problem with partition in                     &
     &DAGMG_MUMPS_518'
               CALL AGMG_MUMPS_ABORT()
            ENDIF
         ENDDO
      ELSE
        WRITE(*,*) "Strategy 6 not implemented"
        CALL AGMG_MUMPS_ABORT()
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_472
      SUBROUTINE DAGMG_MUMPS_499(                                            &
     &  SLAVEF,                                                         &
     &  KEEP,KEEP8,                                                     &
     &  CAND_OF_NODE,                                                   &
     &  MEM_DISTRIB, NCB, NFRONT, NSLAVES_NODE,                         &
     &  TAB_POS, SLAVES_LIST, SIZE_SLAVES_LIST)
      IMPLICIT NONE
      INTEGER, intent(in) :: KEEP(500),SIZE_SLAVES_LIST
      INTEGER*8 KEEP8(150)
      INTEGER, intent(in) :: SLAVEF, NFRONT, NCB
      INTEGER, intent(in) :: CAND_OF_NODE(SLAVEF+1)
      INTEGER, intent(in) :: MEM_DISTRIB(0:SLAVEF-1)
      INTEGER, intent(out):: SLAVES_LIST(SIZE_SLAVES_LIST)
      INTEGER, intent(out):: TAB_POS(SLAVEF+2)
      INTEGER, intent(out):: NSLAVES_NODE
      INTEGER ITEMP, NMB_OF_CAND, NSLAVES_LESS
      DOUBLE PRECISION MSG_SIZE
      LOGICAL FORCE_CAND
      INTEGER  AGMG_MUMPS_12
      EXTERNAL AGMG_MUMPS_12
      IF ( KEEP(48) == 0 .AND. KEEP(50) .NE. 0) THEN
      write(*,*) "Internal error 2 in DAGMG_MUMPS_499."
      CALL AGMG_MUMPS_ABORT()
      END IF
      IF ( KEEP(48) == 3 .AND. KEEP(50) .EQ. 0) THEN
      write(*,*) "Internal error 3 in DAGMG_MUMPS_499."
      CALL AGMG_MUMPS_ABORT()
      END IF
      MSG_SIZE = dble( NFRONT - NCB ) * dble(NCB)
      IF ( KEEP(24) == 0 .OR. KEEP(24) == 1 ) THEN
        FORCE_CAND = .FALSE.
      ELSE
        FORCE_CAND = (mod(KEEP(24),2).eq.0)
      END IF
      IF (FORCE_CAND) THEN
        ITEMP=DAGMG_MUMPS_409                                                &
     &       (MEM_DISTRIB(0:SLAVEF-1),                                  &
     &        CAND_OF_NODE,                                             &
     &        KEEP(69), SLAVEF, MSG_SIZE,                               &
     &        NMB_OF_CAND )
      ELSE
        ITEMP=DAGMG_MUMPS_186(KEEP(69),MEM_DISTRIB,MSG_SIZE)
        NMB_OF_CAND = SLAVEF - 1
      END IF
      NSLAVES_LESS = max(ITEMP,1)
      NSLAVES_NODE = AGMG_MUMPS_12(KEEP8(21), KEEP(48),                      &
     &          KEEP(50),SLAVEF,                                        &
     &          NCB, NFRONT, NSLAVES_LESS, NMB_OF_CAND)
      CALL AGMG_MUMPS_441(                                                   &
     &            KEEP,KEEP8, SLAVEF,                                   &
     &            TAB_POS,                                              &
     &            NSLAVES_NODE, NFRONT, NCB                             &
     &             )
      IF (FORCE_CAND) THEN
        CALL DAGMG_MUMPS_384(MEM_DISTRIB(0),                                 &
     &       CAND_OF_NODE, SLAVEF, NSLAVES_NODE,                        &
     &       SLAVES_LIST)
      ELSE
        CALL DAGMG_MUMPS_189(MEM_DISTRIB(0),                                 &
     &       MSG_SIZE, SLAVES_LIST, NSLAVES_NODE)
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_499
      SUBROUTINE DAGMG_MUMPS_185( id, MEMORY_MD_ARG, MAXS )
      USE DAGMG_MUMPS_COMM_BUFF
      USE DAGMG_MUMPS_STRUC_DF
      IMPLICIT NONE
      TYPE(DAGMG_MUMPS_STRUC), TARGET :: id
      INTEGER(8), intent(in) :: MEMORY_MD_ARG
      INTEGER(8), intent(in) :: MAXS
      INTEGER K34_LOC,K35_LOC
      INTEGER allocok, IERR, i, BUF_LOAD_SIZE
      DOUBLE PRECISION :: MAX_SBTR
      DOUBLE PRECISION ZERO
      DOUBLE PRECISION MEMORY_SENT
      INTEGER,DIMENSION(:),POINTER:: KEEP
      PARAMETER( ZERO=0.0d0 )
      INTEGER WHAT
      INTEGER(8) MEMORY_MD, LA
      STEP_TO_NIV2_LOAD=>id%ISTEP_TO_INIV2
      CAND_LOAD=>id%CANDIDATES
      ND_LOAD=>id%ND_STEPS
      KEEP_LOAD=>id%KEEP
      KEEP     =>id%KEEP
      KEEP8_LOAD=>id%KEEP8
      FILS_LOAD=>id%FILS
      FRERE_LOAD=>id%FRERE_STEPS
      DAD_LOAD=>id%DAD_STEPS
      PROCNODE_LOAD=>id%PROCNODE_STEPS
      STEP_LOAD=>id%STEP
      NE_LOAD=>id%NE_STEPS
      N_LOAD=id%N
      ROOT_CURRENT_SUBTREE=-9999
      MEMORY_MD=MEMORY_MD_ARG
      LA=MAXS
      MAX_SURF_MASTER=id%MAX_SURF_MASTER+                               &
     & (int(id%KEEP(12),8)*int(id%MAX_SURF_MASTER,8)/int(100,8))
      COMM_LD = id%COMM_LOAD
      K69  = KEEP(69)
      IF ( KEEP(47) .le. 0 .OR. KEEP(47) .gt. 4 ) THEN
        write(*,*) "Internal error 1 in DAGMG_MUMPS_185"
        CALL AGMG_MUMPS_ABORT()
      END IF
      CHK_LD=dble(0)
      BDC_MEM      = ( KEEP(47) >= 2 )
      BDC_POOL     = ( KEEP(47) >= 3 )
      BDC_SBTR     = ( KEEP(47) >= 4 )
      BDC_M2_MEM   = ( ( KEEP(80) == 2 .OR. KEEP(80) == 3 )             &
     &             .AND. KEEP(47) == 4 )
      BDC_M2_FLOPS   = ( KEEP(80) == 1                                  &
     &             .AND. KEEP(47) .GE. 1 )
      BDC_MD       = (KEEP(86)==1)
      SBTR_WHICH_M       = KEEP(90)
      REMOVE_NODE_FLAG=.FALSE.
      REMOVE_NODE_FLAG_MEM=.FALSE.
      REMOVE_NODE_COST_MEM=dble(0)
      REMOVE_NODE_COST=dble(0)
      IF (KEEP(80) .LT. 0 .OR. KEEP(80)>3) THEN
        WRITE(*,*) "Unimplemented KEEP(80) Strategy"
        CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF ((KEEP(80) == 2 .OR. KEEP(80)==3 ).AND. KEEP(47).NE.4)         &
     &  THEN
        WRITE(*,*) "Internal error 3 in DAGMG_MUMPS_185"
         CALL AGMG_MUMPS_ABORT()
      END IF
      IF (KEEP(81) == 1 .AND. KEEP(47) < 2) THEN
        WRITE(*,*) "Internal error 2 in DAGMG_MUMPS_185"
        CALL AGMG_MUMPS_ABORT()
      ENDIF
      BDC_POOL_MNG = ((KEEP(81) == 1).AND.(KEEP(47) >= 2))
      IF(KEEP(76).EQ.4)THEN
         DEPTH_FIRST_LOAD=>id%DEPTH_FIRST
      ENDIF
      IF(KEEP(76).EQ.5)THEN
         COST_TRAV=>id%COST_TRAV
      ENDIF
      IF (BDC_M2_MEM.OR.BDC_M2_FLOPS) THEN
         ALLOCATE(NIV2(id%NSLAVES), NB_SON(KEEP(28)),                   &
     &            POOL_NIV2(100),POOL_NIV2_COST(100),                   &
     &            stat=allocok)
         NB_SON=id%NE_STEPS
         NIV2=dble(0)
         IF (allocok > 0) THEN
           WRITE(*,*) 'PB allocation in DMUMPS_185'
           id%INFO(1) = -13
           id%INFO(2) = id%NSLAVES + KEEP(28) + 200
           RETURN
         ENDIF
      ENDIF
      K50      = id%SYM
      CALL AGMG_SETIERR2( MYID, IERR )
      NPROCS = id%NSLAVES
      DM_SUMLU=ZERO
      POOL_SIZE=0
      IF(BDC_MD)THEN
         IF ( allocated(MD_MEM) ) DEALLOCATE(MD_MEM)
         ALLOCATE( MD_MEM( 0: NPROCS - 1 ), stat=allocok )
         IF ( allocok .gt. 0 ) THEN
            WRITE(*,*) 'PB allocation in DMUMPS_185'
            id%INFO(1) = -13
            id%INFO(2) = NPROCS
            RETURN
         END IF
         IF ( allocated(TAB_MAXS) ) DEALLOCATE(TAB_MAXS)
         ALLOCATE( TAB_MAXS( 0: NPROCS - 1 ), stat=allocok )
         IF ( allocok .gt. 0 ) THEN
            WRITE(*,*) 'PB allocation in DMUMPS_185'
            id%INFO(1) = -13
            id%INFO(2) = NPROCS
            RETURN
         END IF
         TAB_MAXS=0_8
         IF ( allocated(LU_USAGE) ) DEALLOCATE(LU_USAGE)
         ALLOCATE( LU_USAGE( 0: NPROCS - 1 ), stat=allocok )
         IF ( allocok .gt. 0 ) THEN
            WRITE(*,*) 'PB allocation in DMUMPS_185'
            id%INFO(1) = -13
            id%INFO(2) = NPROCS
            RETURN
         END IF
         LU_USAGE=dble(0)
         MD_MEM=int(0,8)
      ENDIF
      IF((KEEP(81).EQ.2).OR.(KEEP(81).EQ.3))THEN
         ALLOCATE(CB_COST_MEM(2*2000*id%NSLAVES),                       &
     &            stat=allocok)
         IF (allocok > 0) THEN
           WRITE(*,*) 'PB allocation in DMUMPS_185'
           id%INFO(1) = -13
           id%INFO(2) = id%NSLAVES
           RETURN
         ENDIF
         CB_COST_MEM=int(0,8)
         ALLOCATE(CB_COST_ID(2000*3),                                   &
     &            stat=allocok)
         IF (allocok > 0) THEN
           WRITE(*,*) 'PB allocation in DMUMPS_185'
           id%INFO(1) = -13
           id%INFO(2) = id%NSLAVES
           RETURN
         ENDIF
         CB_COST_ID=0
         POS_MEM=1
         POS_ID=1
      ENDIF
      ALLOCATE(FUTURE_NIV2(NPROCS), stat=allocok)
      IF (allocok > 0 ) THEN
         WRITE(*,*) 'PB allocation in DMUMPS_185'
         id%INFO(1) = -13
         id%INFO(2) = NPROCS
         RETURN
      ENDIF
      DO i = 1, NPROCS
        FUTURE_NIV2(i) = id%FUTURE_NIV2(i)
        IF(BDC_MD)THEN
           IF(FUTURE_NIV2(i).EQ.0)THEN
              MD_MEM(i-1)=999999999_8
           ENDIF
        ENDIF
      ENDDO
      DELTA_MEM=ZERO
      DELTA_LOAD=ZERO
      CHECK_MEM=0_8
      IF(BDC_SBTR.OR.BDC_POOL_MNG)THEN
         NB_SUBTREES=id%NBSA_LOCAL
         IF (allocated(MEM_SUBTREE)) DEALLOCATE(MEM_SUBTREE)
         ALLOCATE(MEM_SUBTREE(id%NBSA_LOCAL),stat=allocok)
         DO i=1,id%NBSA_LOCAL
            MEM_SUBTREE(i)=id%MEM_SUBTREE(i)
         ENDDO
         MY_FIRST_LEAF=>id%MY_FIRST_LEAF
         MY_NB_LEAF=>id%MY_NB_LEAF
         MY_ROOT_SBTR=>id%MY_ROOT_SBTR
         IF (allocated(SBTR_FIRST_POS_IN_POOL))                         &
     &        DEALLOCATE(SBTR_FIRST_POS_IN_POOL)
         ALLOCATE(SBTR_FIRST_POS_IN_POOL(id%NBSA_LOCAL),stat=allocok)
         INSIDE_SUBTREE=0
         PEAK_SBTR_CUR_LOCAL = dble(0)
         SBTR_CUR_LOCAL      = dble(0)
         IF (allocated(SBTR_PEAK_ARRAY)) DEALLOCATE(SBTR_PEAK_ARRAY)
         ALLOCATE(SBTR_PEAK_ARRAY(id%NBSA_LOCAL),stat=allocok)
         SBTR_PEAK_ARRAY=dble(0)
         IF (allocated(SBTR_CUR_ARRAY)) DEALLOCATE(SBTR_CUR_ARRAY)
         ALLOCATE(SBTR_CUR_ARRAY(id%NBSA_LOCAL),stat=allocok)
         SBTR_CUR_ARRAY=dble(0)
         INDICE_SBTR_ARRAY=1
         NIV1_FLAG=0
         INDICE_SBTR=1
      ENDIF
      IF ( allocated(LOAD_FLOPS) ) DEALLOCATE( LOAD_FLOPS )
      ALLOCATE( LOAD_FLOPS( 0: NPROCS - 1 ), stat=allocok )
      IF ( allocok .gt. 0 ) THEN
         WRITE(*,*) 'PB allocation in DMUMPS_185'
         id%INFO(1) = -13
         id%INFO(2) = NPROCS
         RETURN
      END IF
      IF ( allocated(WLOAD) ) DEALLOCATE( WLOAD )
      ALLOCATE( WLOAD( NPROCS ), stat=allocok )
      IF ( allocok .gt. 0 ) THEN
         WRITE(*,*) 'PB allocation in DMUMPS_185'
         id%INFO(1) = -13
         id%INFO(2) = NPROCS
         RETURN
      END IF
      IF ( allocated(IDWLOAD) ) DEALLOCATE( IDWLOAD )
      ALLOCATE( IDWLOAD( NPROCS ), stat=allocok )
      IF ( allocok .gt. 0 ) THEN
         WRITE(*,*) 'PB allocation in DMUMPS_185'
         id%INFO(1) = -13
         id%INFO(2) = NPROCS
         RETURN
      END IF
      IF ( BDC_MEM ) THEN
        IF ( allocated(DM_MEM) ) DEALLOCATE( DM_MEM )
        ALLOCATE( DM_MEM( 0:NPROCS-1 ), stat=allocok )
        IF ( allocok .gt. 0 ) THEN
           WRITE(*,*) 'PB allocation in DMUMPS_185'
           id%INFO(1) = -13
           id%INFO(2) = NPROCS
           RETURN
        END IF
      END IF
      IF ( BDC_POOL ) THEN
        IF ( allocated(POOL_MEM) ) DEALLOCATE(POOL_MEM)
        ALLOCATE( POOL_MEM(0: NPROCS -1), stat=allocok)
        IF ( allocok .gt. 0 ) THEN
           WRITE(*,*) 'PB allocation in DMUMPS_185'
           id%INFO(1) = -13
           id%INFO(2) = NPROCS
           RETURN
        END IF
        POOL_MEM = dble(0)
        POOL_LAST_COST_SENT = dble(0)
      END IF
      IF ( BDC_SBTR ) THEN
        IF ( allocated(SBTR_MEM) ) DEALLOCATE(SBTR_MEM)
        ALLOCATE( SBTR_MEM(0: NPROCS -1), stat=allocok)
        IF ( allocok .gt. 0 ) THEN
           WRITE(*,*) 'PB allocation in DMUMPS_185'
           id%INFO(1) = -13
           id%INFO(2) = NPROCS
           RETURN
        END IF
        IF ( allocated(SBTR_CUR) ) DEALLOCATE(SBTR_CUR)
        ALLOCATE( SBTR_CUR(0: NPROCS -1), stat=allocok)
        IF ( allocok .gt. 0 ) THEN
           WRITE(*,*) 'PB allocation in DMUMPS_185'
           id%INFO(1) = -13
           id%INFO(2) = NPROCS
           RETURN
        END IF
        SBTR_CUR = dble(0)
        SBTR_MEM = dble(0)
      END IF
      CALL AGMG_MUMPS_546(K34_LOC,K35_LOC)
      K35  = K35_LOC
      BUF_LOAD_SIZE = K34_LOC * 2 * ( NPROCS - 1 ) +                    &
     &                NPROCS * ( K35_LOC + K34_LOC )
      IF (BDC_MEM) THEN
        BUF_LOAD_SIZE = BUF_LOAD_SIZE + NPROCS * K35_LOC
      END IF
      IF (BDC_SBTR)THEN
        BUF_LOAD_SIZE = BUF_LOAD_SIZE + NPROCS * K35_LOC
      ENDIF
      LBUF_LOAD_RECV = (BUF_LOAD_SIZE+K34_LOC)/K34_LOC
      LBUF_LOAD_RECV_BYTES = LBUF_LOAD_RECV * K34_LOC
      IF ( allocated(BUF_LOAD_RECV) ) DEALLOCATE(BUF_LOAD_RECV)
      ALLOCATE( BUF_LOAD_RECV( LBUF_LOAD_RECV), stat=allocok)
      IF ( allocok > 0 ) THEN
        WRITE(*,*) 'PB allocation in DMUMPS_185'
        id%INFO(1) = -13
        id%INFO(2) = LBUF_LOAD_RECV
        RETURN
      ENDIF
      BUF_LOAD_SIZE = BUF_LOAD_SIZE * 20
      CALL DAGMG_MUMPS_54( BUF_LOAD_SIZE, IERR )
      IF ( IERR .LT. 0 ) THEN
         id%INFO(1) = -13
         id%INFO(2) = BUF_LOAD_SIZE
         RETURN
      END IF
      DO i = 0, NPROCS - 1
         LOAD_FLOPS( i ) = ZERO
      END DO
      IF ( BDC_MEM ) THEN
        DO i = 0, NPROCS - 1
          DM_MEM( i )=ZERO
        END DO
      ENDIF
      CALL DAGMG_MUMPS_425(KEEP(69))
      IF(BDC_MD)THEN
         MAX_SBTR=0.0D0
         IF(BDC_SBTR)THEN
            DO i=1,id%NBSA_LOCAL
               MAX_SBTR=max(id%MEM_SUBTREE(i),MAX_SBTR)
            ENDDO
         ENDIF
         MD_MEM(MYID)=MEMORY_MD
         WHAT=8
         CALL DAGMG_MUMPS_460( WHAT,                                         &
     &        COMM_LD, NPROCS,                                          &
     &        FUTURE_NIV2,                                              &
     &        dble(MEMORY_MD),dble(0) ,MYID, IERR  )
         WHAT=9
         MEMORY_SENT = dble(LA-MAX_SURF_MASTER)-MAX_SBTR                &
     &      - max( dble(LA) * dble(3) / dble(100),                      &
     &      dble(2) * dble(max(KEEP(5),KEEP(6))) * dble(KEEP(127)))
         IF (KEEP(12) > 25) THEN
           MEMORY_SENT = MEMORY_SENT -                                  &
     &                   dble(KEEP(12))*0.2d0*dble(LA)/100.0d0
         ENDIF
         TAB_MAXS(MYID)=int(MEMORY_SENT,8)
         CALL DAGMG_MUMPS_460( WHAT,                                         &
     &        COMM_LD, NPROCS,                                          &
     &        FUTURE_NIV2,                                              &
     &        MEMORY_SENT,                                              &
     &        dble(0),MYID, IERR  )
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_185
      SUBROUTINE DAGMG_MUMPS_190( CHECK_FLOPS,PROCESS_BANDE,                 &
     &     INC_LOAD, KEEP,KEEP8 )
      USE DAGMG_MUMPS_COMM_BUFF
      IMPLICIT NONE
      DOUBLE PRECISION INC_LOAD
      INTEGER KEEP(500)
      INTEGER*8 KEEP8(150)
      LOGICAL PROCESS_BANDE
      INTEGER CHECK_FLOPS
      INTEGER IERR
      DOUBLE PRECISION ZERO, SEND_MEM, SEND_LOAD,SBTR_TMP
      PARAMETER( ZERO=0.0d0 )
      INTRINSIC max, abs
      IF (INC_LOAD == 0.0D0) THEN
         IF(REMOVE_NODE_FLAG)THEN
            REMOVE_NODE_FLAG=.FALSE.
         ENDIF
         RETURN
      ENDIF
      IF((CHECK_FLOPS.NE.0).AND.                                        &
     &     (CHECK_FLOPS.NE.1).AND.(CHECK_FLOPS.NE.2))THEN
         WRITE(*,*)MYID,': Bad value for CHECK_FLOPS'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF(CHECK_FLOPS.EQ.1)THEN
         CHK_LD=CHK_LD+INC_LOAD
      ELSE
         IF(CHECK_FLOPS.EQ.2)THEN
            RETURN
         ENDIF
      ENDIF
      IF ( PROCESS_BANDE ) THEN
         RETURN
      ENDIF
      LOAD_FLOPS( MYID ) = max( LOAD_FLOPS( MYID ) + INC_LOAD, ZERO)
      IF(BDC_M2_FLOPS.AND.REMOVE_NODE_FLAG)THEN
         IF(INC_LOAD.NE.REMOVE_NODE_COST)THEN
            IF(INC_LOAD.GT.REMOVE_NODE_COST)THEN
               DELTA_LOAD = DELTA_LOAD +                                &
     &              (INC_LOAD-REMOVE_NODE_COST)
               GOTO 888
            ELSE
               DELTA_LOAD = DELTA_LOAD -                                &
     &              (REMOVE_NODE_COST-INC_LOAD)
               GOTO 888
            ENDIF
         ENDIF
         GOTO 333
      ENDIF
      DELTA_LOAD = DELTA_LOAD + INC_LOAD
  888 CONTINUE
      IF ( DELTA_LOAD > MIN_DIFF .OR. DELTA_LOAD < -MIN_DIFF) THEN
         SEND_LOAD = DELTA_LOAD
         IF (BDC_MEM) THEN
           SEND_MEM = DELTA_MEM
         ELSE
           SEND_MEM = ZERO
         END IF
         IF(BDC_SBTR)THEN
           SBTR_TMP=SBTR_CUR(MYID)
         ELSE
           SBTR_TMP=dble(0)
         ENDIF
  111    CONTINUE
         CALL DAGMG_MUMPS_77( BDC_SBTR,BDC_MEM,                              &
     &        BDC_MD,COMM_LD, NPROCS,                                   &
     &        SEND_LOAD,                                                &
     &        SEND_MEM,SBTR_TMP,                                        &
     &        DM_SUMLU,                                                 &
     &        FUTURE_NIV2,                                              &
     &        MYID, IERR )
           IF ( IERR == -1 )THEN
             CALL DAGMG_MUMPS_467(COMM_LD, KEEP)
             GOTO 111
           ELSE IF ( IERR .NE.0 ) THEN
             WRITE(*,*) "Internal Error in DAGMG_MUMPS_190",IERR
             CALL AGMG_MUMPS_ABORT()
           ENDIF
         IF ( IERR .EQ. 0 ) THEN
           DELTA_LOAD = ZERO
           IF (BDC_MEM) DELTA_MEM  = ZERO
         END IF
      ENDIF
  333 CONTINUE
      IF(REMOVE_NODE_FLAG)THEN
         REMOVE_NODE_FLAG=.FALSE.
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_190
      SUBROUTINE DAGMG_MUMPS_471( SSARBR,                                    &
     &           PROCESS_BANDE_ARG, MEM_VALUE, NEW_LU, INC_MEM_ARG,     &
     &           KEEP,KEEP8,LRLU)
      USE DAGMG_MUMPS_COMM_BUFF
      IMPLICIT NONE
      INTEGER(8), INTENT(IN) :: MEM_VALUE, INC_MEM_ARG, NEW_LU,LRLU
      LOGICAL, INTENT(IN) :: PROCESS_BANDE_ARG, SSARBR
      INTEGER IERR, KEEP(500)
      INTEGER*8 KEEP8(150)
      DOUBLE PRECISION ZERO, SEND_MEM, TMP_MEM, SBTR_TMP
      PARAMETER( ZERO=0.0d0 )
      INTRINSIC max, abs
      INTEGER(8) :: INC_MEM
      LOGICAL PROCESS_BANDE
      INTEGER i
      PROCESS_BANDE=PROCESS_BANDE_ARG
      INC_MEM = INC_MEM_ARG
      IF ( PROCESS_BANDE .AND. NEW_LU .NE. 0_8) THEN
        WRITE(*,*) " Internal Error in DAGMG_MUMPS_471."
        WRITE(*,*) " NEW_LU must be zero if called from PROCESS_BANDE"
        CALL AGMG_MUMPS_ABORT()
      ENDIF
      DM_SUMLU = DM_SUMLU + dble(NEW_LU)
      IF(KEEP_LOAD(201).EQ.0)THEN
         CHECK_MEM = CHECK_MEM + INC_MEM
      ELSEIF(KEEP_LOAD(201).EQ.1)THEN
         CHECK_MEM = CHECK_MEM + INC_MEM - NEW_LU
      ELSEIF(KEEP_LOAD(201).EQ.2)THEN
         CHECK_MEM = CHECK_MEM + INC_MEM - NEW_LU
      ENDIF
      IF ( MEM_VALUE .NE. CHECK_MEM ) THEN
         WRITE(*,*)MYID,                                                &
     &   ':Problem with increments in DMUMPS_471',                      &
     &   CHECK_MEM, MEM_VALUE, INC_MEM,NEW_LU
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF (PROCESS_BANDE) THEN
         RETURN
      ENDIF
      IF(BDC_POOL_MNG) THEN
         IF(SBTR_WHICH_M.EQ.0)THEN
            IF (SSARBR) SBTR_CUR_LOCAL = SBTR_CUR_LOCAL+                &
     &                                   dble(INC_MEM-NEW_LU)
         ELSE
            IF (SSARBR) SBTR_CUR_LOCAL = SBTR_CUR_LOCAL+                &
     &                                   dble(INC_MEM)
         ENDIF
      ENDIF
      IF ( .NOT. BDC_MEM ) THEN
         RETURN
      ENDIF
      IF (BDC_SBTR .AND. SSARBR) THEN
         IF((SBTR_WHICH_M.EQ.0).AND.(KEEP(201).NE.0))THEN
            SBTR_CUR(MYID) = SBTR_CUR(MYID)+dble(INC_MEM-NEW_LU)
         ELSE
            SBTR_CUR(MYID) = SBTR_CUR(MYID)+dble(INC_MEM)
         ENDIF
         SBTR_TMP = SBTR_CUR(MYID)
      ELSE
        SBTR_TMP=dble(0)
      ENDIF
      IF ( NEW_LU > 0_8 ) THEN
        INC_MEM = INC_MEM - NEW_LU
      ENDIF
      DM_MEM( MYID ) = DM_MEM(MYID) + dble(INC_MEM)
      MAX_PEAK_STK=max(MAX_PEAK_STK,DM_MEM(MYID))
      IF(BDC_M2_MEM.AND.REMOVE_NODE_FLAG_MEM)THEN
         IF(dble(INC_MEM).NE.REMOVE_NODE_COST_MEM)THEN
            IF(dble(INC_MEM).GT.REMOVE_NODE_COST_MEM)THEN
               DELTA_MEM = DELTA_MEM +                                  &
     &              (dble(INC_MEM)-REMOVE_NODE_COST_MEM)
               GOTO 888
            ELSE
               DELTA_MEM = DELTA_MEM -                                  &
     &              (REMOVE_NODE_COST_MEM-dble(INC_MEM))
               GOTO 888
            ENDIF
         ENDIF
         GOTO 333
      ENDIF
      DELTA_MEM = DELTA_MEM + dble(INC_MEM)
  888 CONTINUE
      IF ((KEEP(48).NE.5).OR.                                           &
     &     ((KEEP(48).EQ.5).AND.(abs(DELTA_MEM)                         &
     &      .GE.0.1d0*dble(LRLU))))THEN
         IF ( abs(DELTA_MEM) > DM_THRES_MEM ) THEN
            SEND_MEM = DELTA_MEM
  111       CONTINUE
            CALL DAGMG_MUMPS_77(                                             &
     &           BDC_SBTR,                                              &
     &           BDC_MEM,BDC_MD, COMM_LD,                               &
     &           NPROCS,                                                &
     &           DELTA_LOAD,                                            &
     &           SEND_MEM,SBTR_TMP,                                     &
     &           DM_SUMLU,                                              &
     &           FUTURE_NIV2,                                           &
     &           MYID,IERR )
            IF ( IERR == -1 )THEN
               CALL DAGMG_MUMPS_467(COMM_LD, KEEP)
               GOTO 111
           ELSE IF ( IERR .NE. 0 ) THEN
             WRITE(*,*) "Internal Error in DAGMG_MUMPS_471",IERR
             CALL AGMG_MUMPS_ABORT()
           ENDIF
           IF ( IERR .EQ. 0 ) THEN
             DELTA_LOAD = ZERO
             DELTA_MEM  = ZERO
           END IF
         ENDIF
      ENDIF
  333 CONTINUE
      IF(REMOVE_NODE_FLAG_MEM)THEN
         REMOVE_NODE_FLAG_MEM=.FALSE.
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_471
      INTEGER FUNCTION DAGMG_MUMPS_186( K69, MEM_DISTRIB,MSG_SIZE )
      IMPLICIT NONE
      INTEGER i, NLESS, K69
      INTEGER, DIMENSION(0:NPROCS-1) :: MEM_DISTRIB
      DOUBLE PRECISION LREF
      DOUBLE PRECISION MSG_SIZE
      NLESS = 0
      DO i=1,NPROCS
         IDWLOAD(i) = i - 1
      ENDDO
      WLOAD(1:NPROCS) = LOAD_FLOPS(0:NPROCS-1)
      IF(BDC_M2_FLOPS)THEN
         DO i=1,NPROCS
            WLOAD(i)=WLOAD(i)+NIV2(i)
         ENDDO
      ENDIF
      IF(K69 .gt. 1) THEN
         CALL DAGMG_MUMPS_426(MEM_DISTRIB,MSG_SIZE,IDWLOAD,NPROCS)
      ENDIF
      LREF = LOAD_FLOPS(MYID)
      DO i=1, NPROCS
         IF (WLOAD(i).LT.LREF) NLESS=NLESS+1
      ENDDO
      DAGMG_MUMPS_186 = NLESS
      RETURN
      END FUNCTION DAGMG_MUMPS_186
      SUBROUTINE DAGMG_MUMPS_189(MEM_DISTRIB,MSG_SIZE,DEST,                  &
     &     NSLAVES)
      IMPLICIT NONE
      INTEGER NSLAVES
      INTEGER DEST(NSLAVES)
      INTEGER, DIMENSION(0:NPROCS - 1) :: MEM_DISTRIB
      INTEGER i,J,NBDEST
      DOUBLE PRECISION MSG_SIZE
      IF ( NSLAVES.eq.NPROCS-1 ) THEN
        J = MYID+1
        DO i=1,NSLAVES
           J=J+1
           IF (J.GT.NPROCS) J=1
           DEST(i) = J - 1
        ENDDO
      ELSE
        DO i=1,NPROCS
           IDWLOAD(i) = i - 1
        ENDDO
        CALL AGMG_MUMPS_558(NPROCS, WLOAD, IDWLOAD)
         NBDEST = 0
         DO i=1, NSLAVES
            J = IDWLOAD(i)
            IF (J.NE.MYID) THEN
               NBDEST = NBDEST+1
               DEST(NBDEST) = J
            ENDIF
         ENDDO
         IF (NBDEST.NE.NSLAVES) THEN
            DEST(NSLAVES) = IDWLOAD(NSLAVES+1)
         ENDIF
         IF(BDC_MD)THEN
            J=NSLAVES+1
            do i=NSLAVES+1,NPROCS
               IF(IDWLOAD(i).NE.MYID)THEN
                  DEST(J)= IDWLOAD(i)
                  J=J+1
               ENDIF
            end do
         ENDIF
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_189
      SUBROUTINE DAGMG_MUMPS_184( IPROC, LOAD_IPROC )
      IMPLICIT NONE
      INTEGER, intent(in) :: IPROC
      DOUBLE PRECISION, intent(out) :: LOAD_IPROC
      LOAD_IPROC = LOAD_FLOPS( IPROC )
      RETURN
      END SUBROUTINE DAGMG_MUMPS_184
      SUBROUTINE DAGMG_MUMPS_183( INFO1, IERR )
      USE DAGMG_MUMPS_COMM_BUFF
      IMPLICIT NONE
      INTEGER, intent(in) :: INFO1
      INTEGER, intent(out) :: IERR
      IERR=0
      DEALLOCATE( LOAD_FLOPS )
      DEALLOCATE( WLOAD )
      DEALLOCATE( IDWLOAD )
      DEALLOCATE(FUTURE_NIV2)
      IF(BDC_MD)THEN
         DEALLOCATE(MD_MEM)
         DEALLOCATE(LU_USAGE)
         DEALLOCATE(TAB_MAXS)
      ENDIF
      IF ( BDC_MEM ) DEALLOCATE( DM_MEM )
      IF ( BDC_POOL) DEALLOCATE( POOL_MEM )
      IF ( BDC_SBTR) THEN
         DEALLOCATE( SBTR_MEM )
         DEALLOCATE( SBTR_CUR )
         DEALLOCATE(SBTR_FIRST_POS_IN_POOL)
         NULLIFY(MY_FIRST_LEAF)
         NULLIFY(MY_NB_LEAF)
         NULLIFY(MY_ROOT_SBTR)
      ENDIF
      IF(KEEP_LOAD(76).EQ.4)THEN
         NULLIFY(DEPTH_FIRST_LOAD)
      ENDIF
      IF(KEEP_LOAD(76).EQ.5)THEN
         NULLIFY(COST_TRAV)
      ENDIF
      IF (BDC_M2_MEM.OR.BDC_M2_FLOPS) THEN
        DEALLOCATE(NB_SON,POOL_NIV2,POOL_NIV2_COST, NIV2)
      END IF
      IF((KEEP_LOAD(81).EQ.2).OR.(KEEP_LOAD(81).EQ.3))THEN
         DEALLOCATE(CB_COST_MEM)
         DEALLOCATE(CB_COST_ID)
      ENDIF
      NULLIFY(ND_LOAD)
      NULLIFY(KEEP_LOAD)
      NULLIFY(KEEP8_LOAD)
      NULLIFY(FILS_LOAD)
      NULLIFY(FRERE_LOAD)
      NULLIFY(PROCNODE_LOAD)
      NULLIFY(STEP_LOAD)
      NULLIFY(NE_LOAD)
      NULLIFY(CAND_LOAD)
      NULLIFY(STEP_TO_NIV2_LOAD)
      NULLIFY(DAD_LOAD)
      IF (BDC_SBTR.OR.BDC_POOL_MNG) THEN
         DEALLOCATE(MEM_SUBTREE)
         DEALLOCATE(SBTR_PEAK_ARRAY)
         DEALLOCATE(SBTR_CUR_ARRAY)
      ENDIF
      CALL DAGMG_MUMPS_58( IERR )
      CALL DAGMG_MUMPS_150( MYID, COMM_LD,                                   &
     &     BUF_LOAD_RECV, LBUF_LOAD_RECV,                               &
     &     LBUF_LOAD_RECV_BYTES )
      DEALLOCATE(BUF_LOAD_RECV)
      END SUBROUTINE DAGMG_MUMPS_183
      SUBROUTINE DAGMG_MUMPS_467(COMM, KEEP)
      IMPLICIT NONE
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      INTEGER ARROWHEAD, ARR_INT, ARR_REAL, ELT_INT, ELT_REAL
      PARAMETER ( ARROWHEAD = 20,                                       &
     &            ARR_INT = 29,                                         &
     &            ARR_REAL = 30,                                        &
     &            ELT_INT = 31,                                         &
     &            ELT_REAL = 32 )
      INTEGER COLLECT_NZ, COLLECT_IRN, COLLECT_JCN
      PARAMETER( COLLECT_NZ  = 35,                                      &
     &           COLLECT_IRN = 36,                                      &
     &           COLLECT_JCN = 37 )
      INTEGER RACINE,                                                   &
     &        NOEUD,                                                    &
     &        TERREUR,                                                  &
     &        MAITRE_DESC_BANDE,                                        &
     &        MAITRE2,                                                  &
     &        BLOC_FACTO,                                               &
     &        CONTRIB_TYPE2,                                            &
     &        MAPLIG,                                                   &
     &        FACTOR
      PARAMETER ( RACINE            = 2,                                &
     &            NOEUD             = 3,                                &
     &            MAITRE_DESC_BANDE = 4,                                &
     &            MAITRE2           = 5,                                &
     &            BLOC_FACTO        = 6,                                &
     &            CONTRIB_TYPE2     = 7,                                &
     &            MAPLIG            = 8,                                &
     &            FACTOR            = 9,                                &
     &            TERREUR           = 99 )
      INTEGER ROOT_NELIM_INDICES,                                       &
     &        ROOT_CONT_STATIC,                                         &
     &        ROOT_NON_ELIM_CB,                                         &
     &        ROOT_2SLAVE,                                              &
     &        ROOT_2SON
       PARAMETER( ROOT_NELIM_INDICES = 15,                              &
     &        ROOT_CONT_STATIC       = 16,                              &
     &        ROOT_NON_ELIM_CB       = 17,                              &
     &        ROOT_2SLAVE            = 18,                              &
     &        ROOT_2SON              = 19 )
      INTEGER RACINE_SOLVE,                                             &
     &        ContVec,                                                  &
     &        Master2Slave,                                             &
     &        GatherSol,                                                &
     &        ScatterRhsI,                                              &
     &        ScatterRhsR
      PARAMETER( RACINE_SOLVE = 10,                                     &
     &           ContVec      = 11,                                     &
     &           Master2Slave = 12,                                     &
     &           GatherSol    = 13,                                     &
     &           ScatterRhsI  = 54,                                     &
     &           ScatterRhsR  = 55)
      INTEGER FEUILLE,                                                  &
     &        BACKSLV_UPDATERHS,                                        &
     &        BACKSLV_MASTER2SLAVE
      PARAMETER( FEUILLE = 21,                                          &
     &           BACKSLV_UPDATERHS = 22,                                &
     &           BACKSLV_MASTER2SLAVE = 23 )
      INTEGER SYMMETRIZE
      PARAMETER ( SYMMETRIZE = 24 )
      INTEGER BLOC_FACTO_SYM,                                           &
     &        BLOC_FACTO_SYM_SLAVE, END_NIV2_LDLT
      PARAMETER ( BLOC_FACTO_SYM = 25,                                  &
     &            BLOC_FACTO_SYM_SLAVE = 26 ,                           &
     &            END_NIV2_LDLT = 33 )
      INTEGER UPDATE_LOAD
      PARAMETER ( UPDATE_LOAD = 27 )
      INTEGER DEFIC_TAG
      PARAMETER(  DEFIC_TAG = 28 )
      INTEGER TAG_SCHUR
      PARAMETER( TAG_SCHUR = 38 )
      INTEGER TAG_DUMMY
      PARAMETER( TAG_DUMMY = 39 )
      INTEGER ZERO_PIV
      PARAMETER( ZERO_PIV = 40 )
      INTEGER IERR, MSGTAG, MSGLEN, MSGSOU,COMM
      INTEGER KEEP(500)
      INTEGER STATUS(AGMG_MPF_STATUS_SIZE)
      LOGICAL FLAG
   10 CONTINUE
      CALL AGMG_MPF_IPROBE( AGMG_MPF_ANY_SOURCE, AGMG_MPF_ANY_TAG, COMM,               &
     &     FLAG, STATUS, IERR )
      IF (FLAG) THEN
        KEEP(65)=KEEP(65)+1
        MSGTAG = STATUS( AGMG_MPF_TAG )
        MSGSOU = STATUS( AGMG_MPF_SOURCE )
        IF ( MSGTAG .NE. UPDATE_LOAD) THEN
          write(*,*) "Internal error 1 in DAGMG_MUMPS_467",                  &
     &    MSGTAG
          CALL AGMG_MUMPS_ABORT()
        ENDIF
        CALL AGMG_NOCALL('MPI_GET_COUNT')
        IF ( MSGLEN > LBUF_LOAD_RECV_BYTES ) THEN
          write(*,*) "Internal error 2 in DAGMG_MUMPS_467",                  &
     &    MSGLEN, LBUF_LOAD_RECV_BYTES
          CALL AGMG_MUMPS_ABORT()
        ENDIF
        CALL AGMG_NOCALL('MPI_RECV')
        CALL DAGMG_MUMPS_187( MSGSOU, BUF_LOAD_RECV,                         &
     &  LBUF_LOAD_RECV, LBUF_LOAD_RECV_BYTES )
        GOTO 10
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_467
      RECURSIVE SUBROUTINE DAGMG_MUMPS_187                                   &
     &   ( MSGSOU, BUFR, LBUFR, LBUFR_BYTES )
      IMPLICIT NONE
      INTEGER MSGSOU, LBUFR, LBUFR_BYTES
      INTEGER BUFR( LBUFR )
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
      INTEGER POSITION, IERR, WHAT, NSLAVES, i
      DOUBLE PRECISION LOAD_RECEIVED
      INTEGER INODE_RECEIVED,K,J,SON,NSLAVES_TEMP,POS_TEMP,             &
     &     NCB_RECEIVED
      DOUBLE PRECISION SURF
      INTEGER, POINTER, DIMENSION (:) :: LIST_SLAVES
      DOUBLE PRECISION, POINTER, DIMENSION (:) :: LOAD_INCR
      LOGICAL FLAG_LEADER
      EXTERNAL AGMG_MUMPS_330
      INTEGER AGMG_MUMPS_330
      POSITION = 0
      CALL AGMG_NOCALL('MPI_UNPACK')
      IF ( WHAT == 0 ) THEN
        CALL AGMG_NOCALL('MPI_UNPACK')
      LOAD_FLOPS( MSGSOU ) = LOAD_FLOPS(MSGSOU) + LOAD_RECEIVED
        IF ( BDC_MEM ) THEN
          CALL AGMG_NOCALL('MPI_UNPACK')
          DM_MEM(MSGSOU)  = DM_MEM(MSGSOU) + LOAD_RECEIVED
          MAX_PEAK_STK=max(MAX_PEAK_STK,DM_MEM(MSGSOU))
        END IF
        IF(BDC_SBTR)THEN
          CALL AGMG_NOCALL('MPI_UNPACK')
          SBTR_CUR(MSGSOU)=LOAD_RECEIVED
        ENDIF
        IF(BDC_MD)THEN
           CALL AGMG_NOCALL('MPI_UNPACK')
           IF(KEEP_LOAD(201).EQ.0)THEN
              LU_USAGE(MSGSOU)=LOAD_RECEIVED
           ENDIF
        ENDIF
      ELSEIF (( WHAT == 1).OR.(WHAT.EQ.19)) THEN
        CALL AGMG_NOCALL('MPI_UNPACK')
        CALL AGMG_NOCALL('MPI_UNPACK')
        LIST_SLAVES => IDWLOAD
        LOAD_INCR => WLOAD
        CALL AGMG_NOCALL('MPI_UNPACK')
        CALL AGMG_NOCALL('MPI_UNPACK')
        DO i = 1, NSLAVES
            LOAD_FLOPS(LIST_SLAVES(i)) =                                &
     &      LOAD_FLOPS(LIST_SLAVES(i)) +                                &
     &      LOAD_INCR(i)
        END DO
        IF ( BDC_MEM ) THEN
          CALL AGMG_NOCALL('MPI_UNPACK')
          DO i = 1, NSLAVES
              DM_MEM(LIST_SLAVES(i)) = DM_MEM(LIST_SLAVES(i)) +         &
     &        LOAD_INCR(i)
              MAX_PEAK_STK=max(MAX_PEAK_STK,DM_MEM(LIST_SLAVES(i)))
          END DO
        END IF
        IF(WHAT.EQ.19)THEN
           CALL AGMG_NOCALL('MPI_UNPACK')
           CALL CLEAN_POOL_MEM_INFO(INODE_RECEIVED)
           CB_COST_ID(POS_ID)=INODE_RECEIVED
           CB_COST_ID(POS_ID+1)=NSLAVES
           CB_COST_ID(POS_ID+2)=POS_MEM
           POS_ID=POS_ID+3
           DO i=1,NSLAVES
              WRITE(*,*)MYID,':',LIST_SLAVES(i),'->',LOAD_INCR(i)
              CB_COST_MEM(POS_MEM)=int(LIST_SLAVES(i),8)
              POS_MEM=POS_MEM+1
              CB_COST_MEM(POS_MEM)=int(LOAD_INCR(i),8)
              POS_MEM=POS_MEM+1
           ENDDO
        ENDIF
        NULLIFY( LIST_SLAVES )
        NULLIFY( LOAD_INCR )
      ELSE IF (WHAT == 2 ) THEN
        IF ( .not. BDC_POOL ) THEN
          WRITE(*,*) "Internal error 2 in DAGMG_MUMPS_187"
          CALL AGMG_MUMPS_ABORT()
        END IF
        CALL AGMG_NOCALL('MPI_UNPACK')
        POOL_MEM(MSGSOU)=LOAD_RECEIVED
      ELSE IF ( WHAT == 3 ) THEN
        IF ( .NOT. BDC_SBTR) THEN
          WRITE(*,*) "Internal error 3 in DAGMG_MUMPS_187"
          CALL AGMG_MUMPS_ABORT()
        ENDIF
        CALL AGMG_NOCALL('MPI_UNPACK')
        SBTR_MEM(MSGSOU)=SBTR_MEM(MSGSOU)+LOAD_RECEIVED
      ELSE IF (WHAT == 4) THEN
        FUTURE_NIV2(MSGSOU+1)=0
        IF(BDC_MD)THEN
           CALL AGMG_NOCALL('MPI_UNPACK')
          MD_MEM(MSGSOU)=999999999_8
          TAB_MAXS(MSGSOU)=TAB_MAXS(MSGSOU)+int(SURF,8)
        ENDIF
        IF(BDC_M2_MEM.OR.BDC_M2_FLOPS)THEN
        ENDIF
      ELSE IF (WHAT == 5) THEN
         IF((.NOT.BDC_M2_MEM).AND.(.NOT.BDC_M2_FLOPS))THEN
            WRITE(*,*) "Internal error 7 in DAGMG_MUMPS_187"
            CALL AGMG_MUMPS_ABORT()
         ENDIF
         CALL AGMG_NOCALL('MPI_UNPACK')
         IF(BDC_M2_MEM) THEN
            CALL PROCESS_NIV2_MEM_MSG(INODE_RECEIVED)
         ELSEIF(BDC_M2_FLOPS) THEN
            CALL PROCESS_NIV2_FLOPS_MSG(INODE_RECEIVED)
         ENDIF
         IF((KEEP_LOAD(81).EQ.2).OR.(KEEP_LOAD(81).EQ.3))THEN
            CALL AGMG_NOCALL('MPI_UNPACK')
               CALL AGMG_NOCALL('MPI_UNPACK')
            IF(AGMG_MUMPS_330(STEP_LOAD(INODE_RECEIVED),                     &
     &           PROCNODE_LOAD,NPROCS).EQ.1)THEN
               CB_COST_ID(POS_ID)=INODE_RECEIVED
               CB_COST_ID(POS_ID+1)=1
               CB_COST_ID(POS_ID+2)=POS_MEM
               POS_ID=POS_ID+3
               CB_COST_MEM(POS_MEM)=int(MSGSOU,8)
               POS_MEM=POS_MEM+1
               CB_COST_MEM(POS_MEM)=int(NCB_RECEIVED,8)*                &
     &              int(NCB_RECEIVED,8)
               POS_MEM=POS_MEM+1
            ENDIF
         ENDIF
      ELSE IF ( WHAT == 6 ) THEN
         IF((.NOT.BDC_M2_MEM).AND.(.NOT.BDC_M2_FLOPS))THEN
            WRITE(*,*) "Internal error 8 in DAGMG_MUMPS_187"
            CALL AGMG_MUMPS_ABORT()
         ENDIF
         CALL AGMG_NOCALL('MPI_UNPACK')
         IF(BDC_M2_MEM) THEN
            NIV2(MSGSOU+1) = LOAD_RECEIVED
         ELSEIF(BDC_M2_FLOPS) THEN
            NIV2(MSGSOU+1) = NIV2(MSGSOU+1) + LOAD_RECEIVED
            IF(NIV2(MSGSOU+1).LT.0.0D0)THEN
               IF(abs(NIV2(MSGSOU+1)).LE.                               &
     &              sqrt(epsilon(LOAD_RECEIVED)))THEN
                  NIV2(MSGSOU+1)=0.0D0
               ELSE
                  WRITE(*,*)'problem with NIV2_FLOPS message',          &
     &                 NIV2(MSGSOU+1),MSGSOU,LOAD_RECEIVED
                  CALL AGMG_MUMPS_ABORT()
               ENDIF
            ENDIF
         ENDIF
      ELSEIF(WHAT == 17)THEN
         CALL AGMG_NOCALL('MPI_UNPACK')
         IF(BDC_M2_MEM) THEN
            NIV2(MSGSOU+1) = LOAD_RECEIVED
            CALL AGMG_NOCALL('MPI_UNPACK')
            IF(BDC_MD)THEN
               DM_MEM(MYID)=DM_MEM(MYID)+LOAD_RECEIVED
            ELSEIF(BDC_POOL)THEN
               POOL_MEM(MSGSOU)=LOAD_RECEIVED
            ENDIF
         ELSEIF(BDC_M2_FLOPS) THEN
            NIV2(MSGSOU+1) = NIV2(MSGSOU+1) + LOAD_RECEIVED
            IF(NIV2(MSGSOU+1).LT.0.0D0)THEN
               WRITE(*,*)'problem with NIV2_FLOPS message',             &
     &              NIV2(MSGSOU+1),MSGSOU,LOAD_RECEIVED
               CALL AGMG_MUMPS_ABORT()
            ENDIF
            CALL AGMG_NOCALL('MPI_UNPACK')
            LOAD_FLOPS( MSGSOU ) = LOAD_FLOPS(MSGSOU) + LOAD_RECEIVED
         ENDIF
      ELSEIF ( WHAT == 7 ) THEN
         IF(.NOT.BDC_MD)THEN
            WRITE(*,*)MYID,': Internal error 4                          &
     &in DAGMG_MUMPS_187'
            CALL AGMG_MUMPS_ABORT()
         ENDIF
        CALL AGMG_NOCALL('MPI_UNPACK')
        CALL AGMG_NOCALL('MPI_UNPACK')
        LIST_SLAVES => IDWLOAD
        LOAD_INCR => WLOAD
        CALL AGMG_NOCALL('MPI_UNPACK')
        CALL AGMG_NOCALL('MPI_UNPACK')
        DO i = 1, NSLAVES
            MD_MEM(LIST_SLAVES(i)) =                                    &
     &      MD_MEM(LIST_SLAVES(i)) +                                    &
     &      int(LOAD_INCR(i),8)
            IF(FUTURE_NIV2(LIST_SLAVES(i)+1).EQ.0)THEN
               MD_MEM(LIST_SLAVES(i))=999999999_8
            ENDIF
        END DO
      ELSEIF ( WHAT == 8 ) THEN
         IF(.NOT.BDC_MD)THEN
            WRITE(*,*)MYID,': Internal error 5                          &
     &in DAGMG_MUMPS_187'
            CALL AGMG_MUMPS_ABORT()
         ENDIF
        CALL AGMG_NOCALL('MPI_UNPACK')
        MD_MEM(MSGSOU)=MD_MEM(MSGSOU)+int(LOAD_RECEIVED,8)
        IF(FUTURE_NIV2(MSGSOU+1).EQ.0)THEN
           MD_MEM(MSGSOU)=999999999_8
        ENDIF
      ELSEIF ( WHAT == 9 ) THEN
         IF(.NOT.BDC_MD)THEN
            WRITE(*,*)MYID,': Internal error 6                          &
     &in DAGMG_MUMPS_187'
            CALL AGMG_MUMPS_ABORT()
         ENDIF
        CALL AGMG_NOCALL('MPI_UNPACK')
        TAB_MAXS(MSGSOU)=int(LOAD_RECEIVED,8)
      ELSE
          WRITE(*,*) "Internal error 1 in DAGMG_MUMPS_187"
          CALL AGMG_MUMPS_ABORT()
      END IF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_187
      integer function DAGMG_MUMPS_409                                       &
     &                 (MEM_DISTRIB,CAND,                               &
     &                  K69,                                            &
     &                  SLAVEF,MSG_SIZE,                                &
     &                  NMB_OF_CAND )
      implicit none
      integer, intent(in) :: K69, SLAVEF
      INTEGER, intent(in) :: CAND(SLAVEF+1)
      INTEGER, DIMENSION(0:NPROCS - 1), intent(in) :: MEM_DISTRIB
      INTEGER, intent(out) :: NMB_OF_CAND
      integer i,nless
      DOUBLE PRECISION lref
      DOUBLE PRECISION MSG_SIZE
      nless = 0
      NMB_OF_CAND=CAND(SLAVEF+1)
      do i=1,NMB_OF_CAND
         WLOAD(i)=LOAD_FLOPS(CAND(i))
         IF(BDC_M2_FLOPS)THEN
            WLOAD(i)=WLOAD(i)+NIV2(CAND(i)+1)
         ENDIF
      end do
      IF(K69 .gt. 1) THEN
         CALL DAGMG_MUMPS_426(MEM_DISTRIB,MSG_SIZE,                          &
     &        CAND,NMB_OF_CAND)
      ENDIF
      lref = LOAD_FLOPS(MYID)
      do i=1, NMB_OF_CAND
         if (WLOAD(i).lt.lref) nless=nless+1
      end do
      DAGMG_MUMPS_409 = nless
      return
      end function DAGMG_MUMPS_409
      subroutine DAGMG_MUMPS_384                                             &
     &           (MEM_DISTRIB,CAND,                                     &
     &            SLAVEF,                                               &
     &            nslaves_inode, DEST)
      implicit none
      integer, intent(in) :: nslaves_inode, SLAVEF
      integer, intent(in) :: CAND(SLAVEF+1)
      integer, dimension(0:NPROCS - 1), intent(in) :: MEM_DISTRIB
      integer, intent(out) :: DEST(CAND(SLAVEF+1))
      integer i,j,NMB_OF_CAND,deficit,nextproc
      DOUBLE PRECISION lref
      external AGMG_MUMPS_558
      NMB_OF_CAND = CAND(SLAVEF+1)
      if(nslaves_inode.ge.NPROCS .or.                                   &
     &   nslaves_inode.gt.NMB_OF_CAND) then
         write(*,*)'Internal error in DMUMPS_384',                      &
     &   nslaves_inode, NPROCS, NMB_OF_CAND
         CALL AGMG_MUMPS_ABORT()
      end if
      if (nslaves_inode.eq.NPROCS-1) then
         j=MYID+1
         do i=1,nslaves_inode
            if(j.ge.NPROCS) j=0
            DEST(i)=j
            j=j+1
         end do
      else
        do i=1,NMB_OF_CAND
               IDWLOAD(i)=i
        end do
        call AGMG_MUMPS_558(NMB_OF_CAND,                                     &
     &       WLOAD(1:NMB_OF_CAND),IDWLOAD(1:NMB_OF_CAND) )
        do i=1,nslaves_inode
           DEST(i)= CAND(IDWLOAD(i))
        end do
        IF(BDC_MD)THEN
           do i=nslaves_inode+1,NMB_OF_CAND
              DEST(i)= CAND(IDWLOAD(i))
           end do
        ENDIF
      end if
      return
      end subroutine DAGMG_MUMPS_384
      SUBROUTINE DAGMG_MUMPS_425(K69)
      IMPLICIT NONE
      INTEGER K69
      IF (K69 .LE. 4) THEN
         ALPHA = 0.0d0
         BETA = 0.0d0
         RETURN
      ENDIF
      IF (K69 .EQ. 5) THEN
         ALPHA = 0.5d0
         BETA = 50000.0d0
         RETURN
      ENDIF
      IF (K69 .EQ. 6) THEN
         ALPHA = 0.5d0
         BETA = 100000.0d0
         RETURN
      ENDIF
      IF (K69 .EQ. 7) THEN
         ALPHA = 0.5d0
         BETA = 150000.0d0
         RETURN
      ENDIF
      IF (K69 .EQ. 8) THEN
         ALPHA = 1.0d0
         BETA = 50000.0d0
         RETURN
      ENDIF
      IF (K69 .EQ. 9) THEN
         ALPHA = 1.0d0
         BETA = 100000.0d0
         RETURN
      ENDIF
      IF (K69 .EQ. 10) THEN
         ALPHA = 1.0d0
         BETA = 150000.0d0
         RETURN
      ENDIF
      IF (K69 .EQ. 11) THEN
         ALPHA = 1.5d0
         BETA = 50000.0d0
         RETURN
      ENDIF
      IF (K69 .EQ. 12) THEN
         ALPHA = 1.5d0
         BETA = 100000.0d0
         RETURN
      ENDIF
      ALPHA = 1.5d0
      BETA = 150000.0d0
      RETURN
      END SUBROUTINE DAGMG_MUMPS_425
      SUBROUTINE DAGMG_MUMPS_426(MEM_DISTRIB,MSG_SIZE,ARRAY_ADM,LEN)
      IMPLICIT NONE
      INTEGER i,J,LEN,COUNTMORE,COUNTLOCLESS
      INTEGER, DIMENSION(0:NPROCS-1) :: MEM_DISTRIB
      DOUBLE PRECISION MSG_SIZE,FORBIGMSG
      INTEGER ARRAY_ADM(LEN)
      DOUBLE PRECISION MY_LOAD
      FORBIGMSG = 1.0d0
      IF (K69 .lt.2) THEN
         RETURN
      ENDIF
      IF(BDC_M2_FLOPS)THEN
         MY_LOAD=LOAD_FLOPS(MYID)+NIV2(MYID+1)
      ELSE
         MY_LOAD=LOAD_FLOPS(MYID)
      ENDIF
      IF((MSG_SIZE * dble(K35) ) .gt. 3200000.0d0) THEN
         FORBIGMSG = 2.0d0
      ENDIF
      IF (K69 .le. 4) THEN
         DO i = 1,LEN
            IF ((MEM_DISTRIB(ARRAY_ADM(i)) .EQ. 1) .AND.                &
     &      WLOAD(i) .LT. MY_LOAD ) THEN
               WLOAD(i) = WLOAD(i)/MY_LOAD
            ELSE
              IF ( MEM_DISTRIB(ARRAY_ADM(i)) .NE. 1 ) THEN
                WLOAD(i) = WLOAD(i) *                                   &
     &              dble(MEM_DISTRIB(ARRAY_ADM(i)))                     &
     &              * FORBIGMSG                                         &
     &              + dble(2)
              ENDIF
            ENDIF
         ENDDO
         RETURN
      ENDIF
      DO i = 1,LEN
         IF ((MEM_DISTRIB(ARRAY_ADM(i)) .EQ. 1) .AND.                   &
     &        WLOAD(i) .LT. MY_LOAD ) THEN
            WLOAD(i) = WLOAD(i) /  MY_LOAD
         ELSE
            IF(MEM_DISTRIB(ARRAY_ADM(i)) .NE. 1) THEN
               WLOAD(i) = (WLOAD(i) +                                   &
     &              ALPHA * MSG_SIZE * dble(K35)  +                     &
     &              BETA) * FORBIGMSG
            ENDIF
         ENDIF
      ENDDO
      RETURN
      END SUBROUTINE DAGMG_MUMPS_426
      SUBROUTINE DAGMG_MUMPS_461(MYID, SLAVEF, COMM,                         &
     &     TAB_POS, NASS, KEEP,KEEP8, LIST_SLAVES, NSLAVES,INODE)
      USE DAGMG_MUMPS_COMM_BUFF
      IMPLICIT NONE
      INTEGER, INTENT (IN) :: MYID, SLAVEF, COMM, NASS, NSLAVES
      INTEGER, INTENT (IN) :: TAB_POS(SLAVEF+2)
      INTEGER, INTENT (IN) :: LIST_SLAVES( NSLAVES )
      INTEGER KEEP(500)
      INTEGER*8 KEEP8(150)
      INTEGER NCB, NFRONT, NBROWS_SLAVE
      INTEGER i, IERR,WHAT,INODE
      DOUBLE PRECISION MEM_INCREMENT ( NSLAVES )
      DOUBLE PRECISION FLOPS_INCREMENT( NSLAVES )
      DOUBLE PRECISION CB_BAND( NSLAVES )
      IF((KEEP(81).NE.2).AND.(KEEP(81).NE.3))THEN
         WHAT=1
      ELSE
         WHAT=19
      ENDIF
      FUTURE_NIV2(MYID+1) = FUTURE_NIV2(MYID+1) - 1
      IF ( FUTURE_NIV2(MYID+1) < 0 ) THEN
        WRITE(*,*) "Internal error in DAGMG_MUMPS_461"
        CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF ( FUTURE_NIV2(MYID + 1) == 0 ) THEN
  112   CONTINUE
        CALL DAGMG_MUMPS_502(COMM,MYID,SLAVEF,                               &
     &       dble(MAX_SURF_MASTER),IERR)
        IF (IERR == -1 ) THEN
          CALL DAGMG_MUMPS_467(COMM_LD, KEEP)
          GOTO 112
        ELSE IF ( IERR .NE. 0 ) THEN
          WRITE(*,*) "Internal Error in DAGMG_MUMPS_461",                    &
     &    IERR
          CALL AGMG_MUMPS_ABORT()
        ENDIF
      TAB_MAXS(MYID) = TAB_MAXS(MYID) + int(MAX_SURF_MASTER,8)
      ENDIF
      IF ( NSLAVES /= TAB_POS(SLAVEF + 2) ) THEN
        write(*,*) "Error 1 in DAGMG_MUMPS_461",                             &
     &             NSLAVES, TAB_POS(SLAVEF+2)
        CALL AGMG_MUMPS_ABORT()
      ENDIF
      NCB = TAB_POS(NSLAVES+1) - 1
      NFRONT = NCB + NASS
      DO i = 1, NSLAVES
         NBROWS_SLAVE = TAB_POS(i+1) - TAB_POS(i)
         IF ( KEEP(50) == 0 ) THEN
            FLOPS_INCREMENT( i ) = (dble(NBROWS_SLAVE)*dble( NASS ))+   &
     &           dble(NBROWS_SLAVE) * dble(NASS) *                      &
     &           dble(2*NFRONT-NASS-1)
         ELSE
            FLOPS_INCREMENT( i ) = dble(NBROWS_SLAVE) * dble(NASS ) *   &
     &           dble( 2 * ( NASS + TAB_POS(i+1) - 1 )                  &
     &           - NBROWS_SLAVE - NASS + 1 )
         ENDIF
         IF ( BDC_MEM ) THEN
            IF ( KEEP(50) == 0 ) THEN
               MEM_INCREMENT( i ) = dble(NBROWS_SLAVE) *                &
     &              dble(NFRONT)
            ELSE
               MEM_INCREMENT( i ) = dble(NBROWS_SLAVE) *                &
     &              dble( NASS + TAB_POS(i+1) - 1 )
            END IF
         ENDIF
         IF((KEEP(81).NE.2).AND.(KEEP(81).NE.3))THEN
            CB_BAND(i)=dble(-999999)
         ELSE
            IF ( KEEP(50) == 0 ) THEN
               CB_BAND( i ) = dble(NBROWS_SLAVE) *                      &
     &              dble(NFRONT-NASS)
            ELSE
               CB_BAND( i ) = dble(NBROWS_SLAVE) *                      &
     &              dble(TAB_POS(i+1)-1)
            END IF
         ENDIF
      END DO
      IF((KEEP(81).EQ.2).OR.(KEEP(81).EQ.3))THEN
         CB_COST_ID(POS_ID)=INODE
         CB_COST_ID(POS_ID+1)=NSLAVES
         CB_COST_ID(POS_ID+2)=POS_MEM
         POS_ID=POS_ID+3
         DO i=1,NSLAVES
            CB_COST_MEM(POS_MEM)=int(LIST_SLAVES(i),8)
            POS_MEM=POS_MEM+1
            CB_COST_MEM(POS_MEM)=int(CB_BAND(i),8)
            POS_MEM=POS_MEM+1
         ENDDO
      ENDIF
  111 CONTINUE
      CALL DAGMG_MUMPS_524(BDC_MEM, COMM, MYID, SLAVEF,                      &
     &     FUTURE_NIV2,                                                 &
     &     NSLAVES, LIST_SLAVES,INODE,                                  &
     &     MEM_INCREMENT,                                               &
     &     FLOPS_INCREMENT,CB_BAND, WHAT,IERR)
        IF ( IERR == -1 ) THEN
          CALL DAGMG_MUMPS_467(COMM_LD, KEEP)
          GOTO 111
        ELSE IF ( IERR .NE. 0 ) THEN
          WRITE(*,*) "Internal Error in DAGMG_MUMPS_461",                    &
     &    IERR
          CALL AGMG_MUMPS_ABORT()
        ENDIF
      IF (FUTURE_NIV2(MYID+1) .NE. 0) THEN
        DO i = 1, NSLAVES
          LOAD_FLOPS(LIST_SLAVES(i)) = LOAD_FLOPS(LIST_SLAVES(i))       &
     &       +  FLOPS_INCREMENT(i)
          IF ( BDC_MEM ) THEN
            DM_MEM(LIST_SLAVES(i)) = DM_MEM(LIST_SLAVES(i))             &
     &       +  MEM_INCREMENT(i)
          END IF
        ENDDO
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_461
      SUBROUTINE DAGMG_MUMPS_500(                                            &
     &     POOL, LPOOL,                                                 &
     &     PROCNODE, KEEP,KEEP8, SLAVEF, COMM, MYID, STEP, N,           &
     &     ND, FILS )
      USE DAGMG_MUMPS_COMM_BUFF
      IMPLICIT NONE
      INTEGER LPOOL, SLAVEF, COMM, MYID
      INTEGER N, KEEP(500)
      INTEGER*8 KEEP8(150)
      INTEGER POOL( LPOOL ), PROCNODE( KEEP(28) ), STEP( N )
      INTEGER ND( KEEP(28) ), FILS( N )
      INTEGER i, INODE, NELIM, NFR, LEVEL, IERR, WHAT
      DOUBLE PRECISION COST
      INTEGER NBINSUBTREE,NBTOP,INSUBTREE
      LOGICAL AGMG_MUMPS_167
      INTEGER AGMG_MUMPS_330
      EXTERNAL AGMG_MUMPS_167, AGMG_MUMPS_330
      NBINSUBTREE = POOL(LPOOL)
      NBTOP       = POOL(LPOOL - 1)
      INSUBTREE   = POOL(LPOOL - 2)
      IF(BDC_MD)THEN
         RETURN
      ENDIF
      IF((KEEP(76).EQ.0).OR.(KEEP(76).EQ.2))THEN
         IF(NBTOP.NE.0)THEN
            DO i = LPOOL-NBTOP-2, min(LPOOL-3,LPOOL-NBTOP-2+3)
               INODE = POOL( i )
               IF (INODE .LE. N .AND. INODE .GE. 1 ) THEN
                  GOTO 20
               END IF
            END DO
            COST=dble(0)
            GOTO 30
         ELSE
            DO i = NBINSUBTREE, max(1,NBINSUBTREE-3), -1
               INODE = POOL( i )
               IF (INODE .LE. N .AND. INODE .GE. 1 ) THEN
                  GOTO 20
               END IF
            END DO
            COST=dble(0)
            GOTO 30
         ENDIF
      ELSE
         IF(KEEP(76).EQ.1)THEN
            IF(INSUBTREE.EQ.1)THEN
               DO i = NBINSUBTREE, max(1,NBINSUBTREE-3), -1
                  INODE = POOL( i )
                  IF (INODE .LE. N .AND. INODE .GE. 1 ) THEN
                     GOTO 20
                  END IF
               END DO
               COST=dble(0)
               GOTO 30
            ELSE
               DO i = LPOOL-NBTOP-2, min(LPOOL-3,LPOOL-NBTOP-2+3)
                  INODE = POOL( i )
                  IF (INODE .LE. N .AND. INODE .GE. 1 ) THEN
                     GOTO 20
                  END IF
               END DO
               COST=dble(0)
               GOTO 30
            ENDIF
         ELSE
            WRITE(*,*)                                                  &
     &      'Internal error: Unknown pool management strategy'
            CALL AGMG_MUMPS_ABORT()
         ENDIF
      ENDIF
   20 CONTINUE
        i = INODE
        NELIM = 0
   10   CONTINUE
        IF ( i > 0 ) THEN
          NELIM = NELIM + 1
          i = FILS(i)
          GOTO 10
        ENDIF
        NFR = ND( STEP(INODE) )
        LEVEL = AGMG_MUMPS_330( STEP(INODE), PROCNODE, SLAVEF )
        IF (LEVEL .EQ. 1) THEN
          COST = dble( NFR ) * dble( NFR )
        ELSE
          IF ( KEEP(50) == 0 ) THEN
            COST = dble( NFR ) * dble( NELIM )
          ELSE
            COST = dble( NELIM ) * dble( NELIM )
          ENDIF
        ENDIF
   30 CONTINUE
      IF ( abs(POOL_LAST_COST_SENT-COST).GT.DM_THRES_MEM ) THEN
        WHAT = 2
  111   CONTINUE
        CALL DAGMG_MUMPS_460( WHAT,                                          &
     &         COMM, SLAVEF,                                            &
     &               FUTURE_NIV2,                                       &
     &         COST, dble(0),MYID, IERR  )
        POOL_LAST_COST_SENT = COST
        POOL_MEM(MYID)=COST
        IF ( IERR == -1 )THEN
          CALL DAGMG_MUMPS_467(COMM_LD, KEEP)
          GOTO 111
        ELSE IF ( IERR .NE. 0 ) THEN
          WRITE(*,*) "Internal Error in DAGMG_MUMPS_500",                    &
     &    IERR
          CALL AGMG_MUMPS_ABORT()
        ENDIF
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_500
      SUBROUTINE DAGMG_MUMPS_501(                                            &
     &     OK,INODE,POOL,LPOOL,MYID,SLAVEF,COMM,KEEP,KEEP8)
      USE DAGMG_MUMPS_COMM_BUFF
      IMPLICIT NONE
      INTEGER LPOOL,MYID,SLAVEF,COMM,INODE
      INTEGER POOL(LPOOL),KEEP(500)
      INTEGER*8 KEEP8(150)
      INTEGER WHAT,IERR
      LOGICAL OK
      DOUBLE PRECISION ZERO,COST
      LOGICAL FLAG
      PARAMETER(ZERO = 0.0d0)
      EXTERNAL AGMG_MUMPS_283,AGMG_MUMPS_170
      LOGICAL AGMG_MUMPS_283,AGMG_MUMPS_170
      IF((INODE.LE.0).OR.(INODE.GT.N_LOAD)) THEN
         RETURN
      ENDIF
      IF(.NOT.AGMG_MUMPS_170(STEP_LOAD(INODE),                               &
     &     PROCNODE_LOAD,NPROCS))THEN
         RETURN
      ENDIF
      IF(AGMG_MUMPS_283(STEP_LOAD(INODE),PROCNODE_LOAD,NPROCS))THEN
         IF(NE_LOAD(STEP_LOAD(INODE)).EQ.0)THEN
            RETURN
         ENDIF
      ENDIF
      FLAG=.FALSE.
      IF(INDICE_SBTR.LE.NB_SUBTREES)THEN
         IF(INODE.EQ.MY_FIRST_LEAF(INDICE_SBTR))THEN
            FLAG=.TRUE.
         ENDIF
      ENDIF
      IF(FLAG)THEN
         SBTR_PEAK_ARRAY(INDICE_SBTR_ARRAY)=MEM_SUBTREE(INDICE_SBTR)
         SBTR_CUR_ARRAY(INDICE_SBTR_ARRAY)=SBTR_CUR(MYID)
         INDICE_SBTR_ARRAY=INDICE_SBTR_ARRAY+1
         WHAT = 3
         IF(dble(MEM_SUBTREE(INDICE_SBTR)).GE.DM_THRES_MEM)THEN
  111       CONTINUE
            CALL DAGMG_MUMPS_460(                                            &
     &           WHAT, COMM, SLAVEF,                                    &
     &           FUTURE_NIV2,                                           &
     &           dble(MEM_SUBTREE(INDICE_SBTR)), dble(0),MYID, IERR  )
            IF ( IERR == -1 )THEN
               CALL DAGMG_MUMPS_467(COMM_LD, KEEP)
               GOTO 111
            ELSE IF ( IERR .NE. 0 ) THEN
               WRITE(*,*)                                               &
     &         "Internal Error 1 in DAGMG_MUMPS_501",                        &
     &         IERR
               CALL AGMG_MUMPS_ABORT()
            ENDIF
         ENDIF
         SBTR_MEM(MYID)=SBTR_MEM(MYID)+                                 &
     &        dble(MEM_SUBTREE(INDICE_SBTR))
         INDICE_SBTR=INDICE_SBTR+1
         IF(INSIDE_SUBTREE.EQ.0)THEN
            INSIDE_SUBTREE=1
         ENDIF
      ELSE
         IF(INODE.EQ.MY_ROOT_SBTR(INDICE_SBTR-1))THEN
            WHAT = 3
            COST=-SBTR_PEAK_ARRAY(INDICE_SBTR_ARRAY-1)
            IF(abs(COST).GE.DM_THRES_MEM)THEN
  112          CONTINUE
               CALL DAGMG_MUMPS_460(                                         &
     &              WHAT, COMM, SLAVEF,                                 &
     &              FUTURE_NIV2,                                        &
     &              COST, dble(0) ,MYID,IERR  )
               IF ( IERR == -1 )THEN
                  CALL DAGMG_MUMPS_467(COMM_LD, KEEP)
                  GOTO 112
               ELSE IF ( IERR .NE. 0 ) THEN
                  WRITE(*,*)                                            &
     &        "Internal Error 3 in DAGMG_MUMPS_501",                         &
     &        IERR
                  CALL AGMG_MUMPS_ABORT()
               ENDIF
            ENDIF
            INDICE_SBTR_ARRAY=INDICE_SBTR_ARRAY-1
            SBTR_MEM(MYID)=SBTR_MEM(MYID)-                              &
     &           SBTR_PEAK_ARRAY(INDICE_SBTR_ARRAY)
            SBTR_CUR(MYID)=SBTR_CUR_ARRAY(INDICE_SBTR_ARRAY)
            IF(INDICE_SBTR_ARRAY.EQ.1)THEN
               SBTR_CUR(MYID)=dble(0)
               INSIDE_SUBTREE=0
            ENDIF
         ENDIF
         ENDIF
  666 CONTINUE
      END SUBROUTINE DAGMG_MUMPS_501
      SUBROUTINE DAGMG_MUMPS_504                                             &
     &      (SLAVEF,KEEP,KEEP8,PROCS,MEM_DISTRIB,NCB,NFRONT,            &
     &       NSLAVES_NODE,TAB_POS,                                      &
     &       SLAVES_LIST,SIZE_SLAVES_LIST,MYID)
      IMPLICIT NONE
      INTEGER, intent(in) :: KEEP(500),SIZE_SLAVES_LIST
      INTEGER*8 KEEP8(150)
      INTEGER, intent(in) :: SLAVEF, NFRONT, NCB,MYID
      INTEGER, intent(in) :: PROCS(SLAVEF+1)
      INTEGER, intent(in) :: MEM_DISTRIB(0:SLAVEF-1)
      INTEGER, intent(out):: SLAVES_LIST(SIZE_SLAVES_LIST)
      INTEGER, intent(out):: TAB_POS(SLAVEF+2)
      INTEGER, intent(out):: NSLAVES_NODE
      INTEGER NUMBER_OF_PROCS,K47, K48, K50
      INTEGER(8) :: K821
      DOUBLE PRECISION DK821
      INTEGER J, NSLAVES, NSLAVES_REAL, ISLAVE
      INTEGER KMIN, KMAX, BLSIZE, NPIV,                                 &
     &        KMINSURF, NSLAVES_max
      DOUBLE PRECISION WK_MASTER, WK_SLAVE
      INTEGER OTHERS,CHOSEN,SMALL_SET,ACC
      DOUBLE PRECISION SOMME,TMP_SUM
      INTEGER AFFECTED
      INTEGER ADDITIONNAL_ROWS,BLOCK_SIZE,i,X,REF,POS
      INTEGER(8)::TOTAL_MEM
      LOGICAL FORCE_CAND
      DOUBLE PRECISION TEMP(SLAVEF),SUB(SLAVEF),PEAK,MAX_TEMP
      INTEGER TEMP_ID(SLAVEF),NB_ROWS(SLAVEF),SUB_ID(SLAVEF),M2(SLAVEF)
      EXTERNAL AGMG_MPF_WTIME
      DOUBLE PRECISION AGMG_MPF_WTIME
      IF (KEEP8(21) .GT. 0_8) THEN
      write(*,*)MYID,                                                   &
     & ": Internal Error 1 in DAGMG_MUMPS_504"
      CALL AGMG_MUMPS_ABORT()
      ENDIF
      K821=abs(KEEP8(21))
      DK821=dble(K821)
      K50=KEEP(50)
      K48=KEEP(48)
      K47=KEEP(47)
      IF ( KEEP(24) == 0 .OR. KEEP(24) == 1 ) THEN
        FORCE_CAND = .FALSE.
      ELSE
        FORCE_CAND = (mod(KEEP(24),2).eq.0)
      END IF
      IF(K48.NE.4)THEN
         WRITE(*,*)'DMUMPS_COMPUTE_PARTI_ACTV_MEM_K821                  &
     &      should be called with KEEP(48) different from 4'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
         KMIN=1
         KMAX=int(K821/int(NFRONT,8))
         IF(FORCE_CAND)THEN
            DO i=1,PROCS(SLAVEF+1)
               WLOAD(i)=DM_MEM(PROCS(i))
               IDWLOAD(i)=PROCS(i)
            ENDDO
            NUMBER_OF_PROCS=PROCS(SLAVEF+1)
            OTHERS=NUMBER_OF_PROCS
         ELSE
            NUMBER_OF_PROCS=SLAVEF
            WLOAD(1:SLAVEF) = DM_MEM(0:NUMBER_OF_PROCS-1)
            DO i=1,NUMBER_OF_PROCS
               IDWLOAD(i) = i - 1
            ENDDO
            OTHERS=NUMBER_OF_PROCS-1
         ENDIF
         NB_ROWS=0
         CALL AGMG_MUMPS_558(NUMBER_OF_PROCS, WLOAD, IDWLOAD)
         TOTAL_MEM=int(NCB,8)*int(NFRONT,8)
         SOMME=dble(0)
         J=1
         PEAK=dble(0)
         DO i=1,NUMBER_OF_PROCS
            IF((IDWLOAD(i).NE.MYID))THEN
               PEAK=max(PEAK,WLOAD(i))
               TEMP_ID(J)=IDWLOAD(i)
               TEMP(J)=WLOAD(i)
                IF(BDC_SBTR)THEN
                   TEMP(J)=TEMP(J)+SBTR_MEM(IDWLOAD(i))-                &
     &                  SBTR_CUR(IDWLOAD(i))
                ENDIF
                IF(BDC_POOL)THEN
                   TEMP(J)=TEMP(J)+POOL_MEM(TEMP_ID(J))
                ENDIF
                IF(BDC_M2_MEM)THEN
                   TEMP(J)=TEMP(J)+NIV2(TEMP_ID(J)+1)
                ENDIF
                J=J+1
            ENDIF
         ENDDO
         NUMBER_OF_PROCS=J-1
         CALL AGMG_MUMPS_558(NUMBER_OF_PROCS, TEMP, TEMP_ID)
         IF(K50.EQ.0)THEN
           PEAK=max(PEAK,                                               &
     &       DM_MEM(MYID)+dble(NFRONT)*dble(NFRONT-NCB))
         ELSE
           PEAK=max(PEAK,                                               &
     &       DM_MEM(MYID)+dble(NFRONT-NCB)*dble(NFRONT-NCB))
         ENDIF
         PEAK=max(PEAK,TEMP(OTHERS))
         SOMME=dble(0)
         DO i=1,NUMBER_OF_PROCS
           SOMME=SOMME+TEMP(OTHERS)-TEMP(i)
         ENDDO
         IF(SOMME.LE.dble(TOTAL_MEM)) THEN
            GOTO 096
         ENDIF
   96    CONTINUE
         SOMME=dble(0)
         DO i=1,OTHERS
            SOMME=SOMME+TEMP(OTHERS)-TEMP(i)
         ENDDO
         IF(dble(TOTAL_MEM).GE.SOMME) THEN
  887       CONTINUE
            AFFECTED=0
            CHOSEN=0
            ACC=0
            DO i=1,OTHERS
               IF(K50.EQ.0)THEN
                  IF((TEMP(OTHERS)-TEMP(i)).GT.DK821)THEN
                     TMP_SUM=DK821
                  ELSE
                     TMP_SUM=TEMP(OTHERS)-TEMP(i)
                  ENDIF
                  X=int(TMP_SUM/dble(NFRONT))
                  IF((ACC+X).GT.NCB) X=NCB-ACC
               ENDIF
               IF(K50.NE.0)THEN
                  IF((TEMP(OTHERS)-TEMP(i)).GT.DK821)THEN
                     TMP_SUM=DK821
                  ELSE
                     TMP_SUM=TEMP(OTHERS)-TEMP(i)
                  ENDIF
                  X=int((-dble(NFRONT-NCB+ACC)                          &
     &                 +sqrt(((dble(NFRONT-NCB+ACC)*                    &
     &                 dble(NFRONT-NCB+ACC))+dble(4)*                   &
     &                 (TMP_SUM))))/                                    &
     &                 dble(2))
                  IF((ACC+X).GT.NCB) X=NCB-ACC
                  IF(X.LE.0) THEN
                     WRITE(*,*)"Internal Error 2 in                     &
     &                    DAGMG_MUMPS_504"
                     CALL AGMG_MUMPS_ABORT()
                  ENDIF
               ENDIF
               NB_ROWS(i)=X
               CHOSEN=CHOSEN+1
               ACC=ACC+X
               IF(NCB-ACC.LT.KMIN) GOTO 111
               IF(NCB.EQ.ACC) GOTO 111
               ENDDO
  111          CONTINUE
               IF((ACC.GT.NCB))THEN
                  X=0
                  DO i=1,OTHERS
                     X=X+NB_ROWS(i)
                  ENDDO
                  WRITE(*,*)'NCB=',NCB,',SOMME=',X
                  WRITE(*,*)MYID,                                       &
     &               ": Internal Error 3 in DAGMG_MUMPS_504"
                  CALL AGMG_MUMPS_ABORT()
               ENDIF
               IF((NCB.NE.ACC))THEN
                  IF(K50.NE.0)THEN
                     IF(CHOSEN.NE.0)THEN
                        ADDITIONNAL_ROWS=NCB-ACC
                        NB_ROWS(CHOSEN)=NB_ROWS(CHOSEN)+ADDITIONNAL_ROWS
                     ELSE
                        TMP_SUM=dble(TOTAL_MEM)/dble(NUMBER_OF_PROCS)
                        CHOSEN=0
                        ACC=0
                        DO i=1,OTHERS
                           X=int((-dble(NFRONT-NCB+ACC)                 &
     &                          +sqrt(((dble(NFRONT-NCB+ACC)*           &
     &                          dble(NFRONT-NCB+ACC))+dble(4)*          &
     &                          (TMP_SUM))))/                           &
     &                          dble(2))
                           IF((ACC+X).GT.NCB) X=NCB-ACC
                           NB_ROWS(i)=X
                           CHOSEN=CHOSEN+1
                           ACC=ACC+X
                           IF(NCB-ACC.LT.KMIN) GOTO 002
                           IF(NCB.EQ.ACC) GOTO 002
                        ENDDO
    2                   CONTINUE
                        IF(ACC.LT.NCB)THEN
                           NB_ROWS(CHOSEN)=NB_ROWS(CHOSEN)+(NCB-ACC)
                        ENDIF
                     ENDIF
                     GOTO 333
                  ENDIF
                  ADDITIONNAL_ROWS=NCB-ACC
                  DO i=CHOSEN,1,-1
                     IF(int(dble(ADDITIONNAL_ROWS)/                     &
     &                    dble(i)).NE.0)THEN
                        GOTO 222
                     ENDIF
                  ENDDO
  222             CONTINUE
                  X=int(dble(ADDITIONNAL_ROWS)/dble(i))
                  DO J=1,i
                     NB_ROWS(J)=NB_ROWS(J)+X
                     ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                  ENDDO
                  IF(ADDITIONNAL_ROWS.NE.0) THEN
                     NB_ROWS(1)=NB_ROWS(1)+ADDITIONNAL_ROWS
                  ENDIF
               ENDIF
  333          CONTINUE
               IF(NB_ROWS(CHOSEN).EQ.0) CHOSEN=CHOSEN-1
               GOTO 889
            ELSE
               DO i=OTHERS,1,-1
                  SOMME=dble(0)
                  DO J=1,i
                     SOMME=SOMME+TEMP(J)
                  ENDDO
                  SOMME=(dble(i)*TEMP(i))-SOMME
                  IF(dble(TOTAL_MEM).GE.SOMME) GOTO 444
               ENDDO
  444          CONTINUE
               REF=i
               DO J=1,i
                  IF(TEMP(J).EQ.TEMP(i)) THEN
                     SMALL_SET=J
                     GOTO 123
                  ENDIF
               ENDDO
  123          CONTINUE
               IF(i.EQ.1)THEN
                  NB_ROWS(i)=NCB
                  CHOSEN=1
                  GOTO 666
               ENDIF
  323          CONTINUE
               AFFECTED=0
               CHOSEN=0
               ACC=0
               DO i=1,SMALL_SET
                  IF(K50.EQ.0)THEN
                     IF((TEMP(SMALL_SET)-TEMP(i)).GT.DK821)THEN
                        TMP_SUM=DK821
                     ELSE
                        TMP_SUM=TEMP(SMALL_SET)-TEMP(i)
                     ENDIF
                     X=int(TMP_SUM/dble(NFRONT))
                     IF((ACC+X).GT.NCB) X=NCB-ACC
                  ENDIF
                  IF(K50.NE.0)THEN
                     IF((TEMP(SMALL_SET)-TEMP(i)).GT.DK821)THEN
                        TMP_SUM=DK821
                     ELSE
                        TMP_SUM=TEMP(SMALL_SET)-TEMP(i)
                     ENDIF
                      X=int((-dble(NFRONT-NCB+ACC)                      &
     &                  +sqrt(((dble(NFRONT-NCB+ACC)*                   &
     &                  dble(NFRONT-NCB+ACC))+dble(4)*                  &
     &                  (TMP_SUM))))/                                   &
     &                  dble(2))
                     IF(X.LT.0)THEN
                        WRITE(*,*)MYID,                                 &
     &             ': Internal error 4 in DMUMPS_504'
                        CALL AGMG_MUMPS_ABORT()
                     ENDIF
                     IF((ACC+X).GT.NCB) X=NCB-ACC
                  ENDIF
                  NB_ROWS(i)=X
                  ACC=ACC+X
                  CHOSEN=CHOSEN+1
                  IF(NCB-ACC.LT.KMIN) GOTO 888
                  IF(NCB.EQ.ACC) GOTO 888
                  IF(ACC.GT.NCB) THEN
                    WRITE(*,*)MYID,                                     &
     &            ': Internal error 5 in DMUMPS_504'
                    CALL AGMG_MUMPS_ABORT()
                  ENDIF
               ENDDO
  888          CONTINUE
               SOMME=dble(0)
               X=NFRONT-NCB
               IF((ACC.GT.NCB))THEN
                  WRITE(*,*)MYID,                                       &
     &           ':Internal error 6 in DMUMPS_504'
                  CALL AGMG_MUMPS_ABORT()
               ENDIF
               IF((ACC.LT.NCB))THEN
                  IF(K50.NE.0)THEN
                     IF(SMALL_SET.LT.OTHERS)THEN
                       SMALL_SET=REF+1
                       REF=SMALL_SET
                       GOTO 323
                     ELSE
                       NB_ROWS(CHOSEN)=NB_ROWS(CHOSEN)+NCB-ACC
                       GOTO 666
                     ENDIF
                 ENDIF
                 ADDITIONNAL_ROWS=NCB-ACC
                 i=CHOSEN+1
                 DO WHILE ((ADDITIONNAL_ROWS.NE.0)                      &
     &                .AND.(i.LE.NUMBER_OF_PROCS))
                    J=1
                    X=int(ADDITIONNAL_ROWS/(i-1))
                    IF((X.EQ.0).AND.(ADDITIONNAL_ROWS.NE.0))THEN
                       DO WHILE ((J.LT.i).AND.(ADDITIONNAL_ROWS.GT.0))
                         NB_ROWS(J)=NB_ROWS(J)+1
                         ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-1
                         J=J+1
                       ENDDO
                       IF(ADDITIONNAL_ROWS.NE.0)THEN
                          WRITE(*,*)MYID,                               &
     &             ':Internal error 7 in DMUMPS_504'
                         CALL AGMG_MUMPS_ABORT()
                      ENDIF
                      GOTO 047
                    ENDIF
                    IF((TEMP(1)+dble((NB_ROWS(1)+X)*NFRONT)).LE.        &
     &                   TEMP(i))THEN
                       DO WHILE ((ADDITIONNAL_ROWS.NE.0)                &
     &                      .AND.(J.LT.i))
                          AFFECTED=X
                          IF((AFFECTED+NB_ROWS(J)).GT.                  &
     &                         KMAX)THEN
                             AFFECTED=KMAX-NB_ROWS(J)
                          ENDIF
                          NB_ROWS(J)=NB_ROWS(J)+AFFECTED
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-            &
     &                         AFFECTED
                          J=J+1
                       ENDDO
                    ELSE
                       DO WHILE ((ADDITIONNAL_ROWS.NE.0)                &
     &                      .AND.(J.LE.i))
                          AFFECTED=int((TEMP(i)-(TEMP(J)+               &
     &                         (dble(NB_ROWS(J))*dble(NFRONT))))        &
     &                         /dble(NFRONT))
                          IF((AFFECTED+NB_ROWS(J)).GT.KMAX)THEN
                             AFFECTED=KMAX-NB_ROWS(J)
                          ENDIF
                          IF(AFFECTED.GT.ADDITIONNAL_ROWS)THEN
                             AFFECTED=ADDITIONNAL_ROWS
                          ENDIF
                          NB_ROWS(J)=NB_ROWS(J)+AFFECTED
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-AFFECTED
                          J=J+1
                       ENDDO
                    ENDIF
                    i=i+1
                 ENDDO
   47            CONTINUE
                 IF((ADDITIONNAL_ROWS.EQ.0).AND.                        &
     &                (i.LT.NUMBER_OF_PROCS))THEN
                    CHOSEN=i-1
                 ELSE
                    CHOSEN=i-2
                 ENDIF
                 IF((CHOSEN.EQ.NUMBER_OF_PROCS-1).AND.                  &
     &                 (ADDITIONNAL_ROWS.NE.0))THEN
                    DO i=1,CHOSEN
                       NB_ROWS(i)=NB_ROWS(i)+1
                       ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-1
                       IF(ADDITIONNAL_ROWS.EQ.0) GOTO 048
                    ENDDO
   48               CONTINUE
                 ENDIF
                 IF((CHOSEN.EQ.NUMBER_OF_PROCS-1).AND.                  &
     &                (ADDITIONNAL_ROWS.NE.0))THEN
                    i=CHOSEN+1
                    DO WHILE ((ADDITIONNAL_ROWS.NE.0)                   &
     &                   .AND.(i.LE.NUMBER_OF_PROCS))
                       J=1
                       DO WHILE ((ADDITIONNAL_ROWS.NE.0)                &
     &                      .AND.(J.LE.i))
                          AFFECTED=int((TEMP(i)-(TEMP(J)+               &
     &                         (dble(NB_ROWS(J))*                       &
     &                         dble(NFRONT))))/dble(NFRONT))
                          IF(AFFECTED.GT.ADDITIONNAL_ROWS)THEN
                             AFFECTED=ADDITIONNAL_ROWS
                          ENDIF
                          NB_ROWS(J)=NB_ROWS(J)+AFFECTED
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-AFFECTED
                          J=J+1
                       ENDDO
                       i=i+1
                    ENDDO
                    CHOSEN=i-2
                 ENDIF
   49            CONTINUE
              ENDIF
  666         CONTINUE
              SOMME=dble(0)
              X=0
              POS=0
              DO i=1,CHOSEN
                 IF(K50.NE.0) THEN
                    IF((TEMP(i)+dble(NB_ROWS(i))                        &
     &                   *dble(X+NB_ROWS(i)+NFRONT-NCB))                &
     &                   .GT.PEAK)THEN
                       SMALL_SET=SMALL_SET+1
                    ENDIF
                 ENDIF
                 IF(K50.EQ.0) THEN
                    IF((TEMP(i)+dble(NB_ROWS(i))*dble(NFRONT))          &
     &                   .GT.PEAK)THEN
                       SMALL_SET=SMALL_SET+1
                    ENDIF
                 ENDIF
                 X=X+NB_ROWS(i)
                 SOMME=SOMME+ dble(NB_ROWS(i))
              ENDDO
           ENDIF
  889      CONTINUE
           J=CHOSEN
           X=0
           DO i=J,1,-1
             IF(NB_ROWS(i).EQ.0)THEN
                IF(X.EQ.1)THEN
                  WRITE(*,*)MYID,                                       &
     &         ':Internal error 12 in DMUMPS_504'
                  CALL AGMG_MUMPS_ABORT()
                ENDIF
                CHOSEN=CHOSEN-1
             ELSE
                  IF(NB_ROWS(i).GT.0)THEN
                    X=1
                  ELSE
                    WRITE(*,*)                                          &
     &            'Internal error 13 in DMUMPS_504'
                    CALL AGMG_MUMPS_ABORT()
                  ENDIF
             ENDIF
          ENDDO
           NSLAVES_NODE=CHOSEN
           TAB_POS(NSLAVES_NODE+1)= NCB+1
           TAB_POS(SLAVEF+2) = CHOSEN
           POS=1
           DO i=1,CHOSEN
              SLAVES_LIST(i)=TEMP_ID(i)
              TAB_POS(i)=POS
              POS=POS+NB_ROWS(i)
              IF(NB_ROWS(i).LE.0)THEN
                WRITE(*,*)                                              &
     &          'Internal error 14 in DMUMPS_504'
                 CALL AGMG_MUMPS_ABORT()
              ENDIF
           ENDDO
           DO i=CHOSEN+1,NUMBER_OF_PROCS
              SLAVES_LIST(i)=TEMP_ID(i)
           ENDDO
           IF(POS.NE.(NCB+1))THEN
              WRITE(*,*)                                                &
     &        'Internal error 15 in DMUMPS_504'
             CALL AGMG_MUMPS_ABORT()
           ENDIF
      END SUBROUTINE DAGMG_MUMPS_504
      SUBROUTINE DAGMG_MUMPS_518                                             &
     &      (NCBSON_MAX,SLAVEF,KEEP,KEEP8,                              &
     &       PROCS,MEM_DISTRIB,NCB,NFRONT,                              &
     &       NSLAVES_NODE,TAB_POS,                                      &
     &       SLAVES_LIST,SIZE_SLAVES_LIST,MYID,INODE,MP,LP)
      IMPLICIT NONE
      INTEGER, intent(in) :: KEEP(500),SIZE_SLAVES_LIST
      INTEGER*8 KEEP8(150)
      INTEGER, intent(in) :: SLAVEF, NFRONT, NCB,MYID
      INTEGER, intent(in) :: NCBSON_MAX
      INTEGER, intent(in) :: PROCS(SLAVEF+1)
      INTEGER, intent(in) :: MEM_DISTRIB(0:SLAVEF-1),INODE
      INTEGER, intent(in) :: MP,LP
      INTEGER, intent(out):: SLAVES_LIST(SIZE_SLAVES_LIST)
      INTEGER, intent(out):: TAB_POS(SLAVEF+2)
      INTEGER, intent(out):: NSLAVES_NODE
      INTEGER NUMBER_OF_PROCS,K47,K48, K50,K83,K69
      INTEGER(8) :: K821
      INTEGER J, NSLAVES, NSLAVES_REAL, ISLAVE
      INTEGER KMIN, KMAX, BLSIZE, NPIV,                                 &
     &        KMINSURF, NSLAVES_max
      DOUBLE PRECISION WK_MASTER, WK_SLAVE
      INTEGER OTHERS,CHOSEN,SMALL_SET,ACC
      DOUBLE PRECISION SOMME,TMP_SUM,DELTA,A,B,C,MASTER_WORK
      INTEGER AFFECTED
      INTEGER ADDITIONNAL_ROWS,BLOCK_SIZE,i,X,REF,POS,NELIM,NB_DIFF
      INTEGER*8 X8
      LOGICAL FORCE_CAND,SMP
      DOUBLE PRECISION BANDE_K821
      INTEGER NB_SAT,NB_ZERO
      DOUBLE PRECISION TEMP(SLAVEF),SUB(SLAVEF),PEAK,TOTAL_COST,        &
     &     MAX_MEM_ALLOW,MAX_FLOPS_ALLOW
      INTEGER TEMP_ID(SLAVEF),NB_ROWS(SLAVEF),M2(SLAVEF),               &
     &     NB_ROWS_SMP(SLAVEF)
      INTEGER NSLAVES_REF,NCB_FILS
      EXTERNAL AGMG_MPF_WTIME,AGMG_MUMPS_442
      INTEGER AGMG_MUMPS_442
      INTEGER POS_MIN_LOAD,SIZE_MY_SMP,WHAT,ADDITIONNAL_ROWS_SPECIAL
      LOGICAL HAVE_TYPE1_SON
      DOUBLE PRECISION MIN_LOAD,MAX_LOAD,TEMP_MAX_LOAD
      DOUBLE PRECISION AGMG_MPF_WTIME
      DOUBLE PRECISION BUF_SIZE,NELIM_MEM_SIZE
      DOUBLE PRECISION MEM_SIZE_STRONG(SLAVEF),MEM_SIZE_WEAK(SLAVEF)
      INTEGER AGMG_MUMPS_330
      EXTERNAL AGMG_MUMPS_330
      K821=abs(KEEP8(21))
      TEMP_MAX_LOAD=dble(0)
      K50=KEEP(50)
      K48=KEEP(48)
      K47=KEEP(47)
      K83=KEEP(83)
      K69=0
      NCB_FILS=NCBSON_MAX
      IF(int(NCB_FILS,8)*int(min(NCB,NCB_FILS),8).GT.K821)THEN
         HAVE_TYPE1_SON=.TRUE.
      ELSE
         HAVE_TYPE1_SON=.FALSE.
      ENDIF
      SMP=(K69.NE.0)
      IF ( KEEP(24) == 0 .OR. KEEP(24) == 1 ) THEN
        FORCE_CAND = .FALSE.
      ELSE
        FORCE_CAND = (mod(KEEP(24),2).eq.0)
      END IF
      NELIM=NFRONT-NCB
         KMAX=int(K821/int(NCB,8))
         IF(FORCE_CAND)THEN
           DO i=1,PROCS(SLAVEF+1)
              WLOAD(i)=LOAD_FLOPS(PROCS(i))
              IDWLOAD(i)=PROCS(i)
              IF (WLOAD(i) < -0.5d0 ) THEN
                 IF((MP.GT.0).AND.(LP.GE.2))THEN
                    WRITE(MP,*)MYID,': Warning: negative load ',        &
     &                   WLOAD(i)
                 ENDIF
              ENDIF
              WLOAD(i)=max(WLOAD(i),0.0d0)
           ENDDO
           NUMBER_OF_PROCS=PROCS(SLAVEF+1)
            OTHERS=NUMBER_OF_PROCS
         ELSE
            NUMBER_OF_PROCS=SLAVEF
            WLOAD(1:SLAVEF) = LOAD_FLOPS(0:NUMBER_OF_PROCS-1)
            DO i=1,NUMBER_OF_PROCS
               IDWLOAD(i) = i - 1
               IF (WLOAD(i) < -0.5d0 ) THEN
                  IF((MP.GT.0).AND.(LP.GE.2))THEN
                     WRITE(MP,*)MYID,': Negative load ',                &
     &                    WLOAD(i)
                  ENDIF
               ENDIF
               WLOAD(i)=max(WLOAD(i),0.0d0)
            ENDDO
            OTHERS=NUMBER_OF_PROCS-1
         ENDIF
         KMAX=int(NCB/OTHERS)
         KMIN=AGMG_MUMPS_442(int(NCB,8)*int(KMAX,8),K50,KMAX,NCB)
         NB_ROWS=0
         CALL AGMG_MUMPS_558(NUMBER_OF_PROCS, WLOAD, IDWLOAD)
         IF(K50.EQ.0)THEN
            TOTAL_COST=dble( NELIM ) * dble ( NCB ) +                   &
     &           dble(NCB) * dble(NELIM)*dble(2*NFRONT-NELIM-1)
         ELSE
            TOTAL_COST=dble(NELIM) * dble ( NCB ) *                     &
     &           dble(NFRONT+1)
         ENDIF
         CALL AGMG_MUMPS_511(NFRONT,NELIM,NELIM,K50,                         &
     &        2,MASTER_WORK)
         SOMME=dble(0)
         J=1
         IF(FORCE_CAND.AND.(NUMBER_OF_PROCS.GT.K83))THEN
            MASTER_WORK=dble(KEEP(88))*MASTER_WORK/dble(100)
         ENDIF
         IF(FORCE_CAND.AND.(NUMBER_OF_PROCS.LE.K83))THEN
            MASTER_WORK=dble(KEEP(87))*MASTER_WORK/dble(100)
         ENDIF
         IF(MASTER_WORK.LT.dble(1))THEN
            MASTER_WORK=dble(1)
         ENDIF
         NSLAVES_REF=int(TOTAL_COST/MASTER_WORK)+1
         IF(FORCE_CAND)THEN
            NSLAVES_REF=min(NSLAVES_REF,NUMBER_OF_PROCS)
         ELSE
            NSLAVES_REF=min(NSLAVES_REF,NUMBER_OF_PROCS-1)
         ENDIF
        DO i=1,NUMBER_OF_PROCS
           IF((IDWLOAD(i).NE.MYID))THEN
              TEMP_ID(J)=IDWLOAD(i)
              TEMP(J)=WLOAD(i)
              IF(BDC_M2_FLOPS)THEN
                 TEMP(J)=TEMP(J)+NIV2(TEMP_ID(J)+1)
              ENDIF
              J=J+1
           ENDIF
        ENDDO
        NUMBER_OF_PROCS=J-1
        CALL AGMG_MUMPS_558(NUMBER_OF_PROCS, TEMP, TEMP_ID)
        SOMME=dble(0)
        TMP_SUM=dble(0)
        DO i=1,OTHERS
            SOMME=SOMME+TEMP(OTHERS)-TEMP(i)
            TMP_SUM=TMP_SUM+TEMP(i)
        ENDDO
         TMP_SUM=(TMP_SUM/dble(OTHERS))+                                &
     &        (TOTAL_COST/dble(OTHERS))
         SIZE_MY_SMP=OTHERS
         MIN_LOAD=TEMP(1)
         POS_MIN_LOAD=1
         IF(.NOT.SMP) MAX_LOAD=TEMP(OTHERS)
         IF(SMP)THEN
            J=1
            DO i=1,OTHERS
               IF(MEM_DISTRIB(TEMP_ID(i)).EQ.1)THEN
                  IF(TEMP(i).LE.TMP_SUM)THEN
                     WLOAD(J)=TEMP(i)
                     IDWLOAD(J)=TEMP_ID(i)
                     J=J+1
                  ELSE
                  ENDIF
               ENDIF
            ENDDO
            MAX_LOAD=WLOAD(J-1)
            SIZE_MY_SMP=J-1
            DO i=1,OTHERS
               IF((MEM_DISTRIB(TEMP_ID(i)).NE.1).OR.                    &
     &              ((MEM_DISTRIB(TEMP_ID(i)).EQ.1).AND.                &
     &              (TEMP(i).GE.TMP_SUM)))THEN
                  WLOAD(J)=TEMP(i)
                  IDWLOAD(J)=TEMP_ID(i)
                  J=J+1
               ENDIF
            ENDDO
            TEMP=WLOAD
            TEMP_ID=IDWLOAD
         ENDIF
        IF(BDC_MD)THEN
           BUF_SIZE=dble(K821)
           IF (KEEP(201).EQ.2) THEN
              A=dble(int((dble(KEEP(100))/dble(2))/dble(NELIM)))
              IF(K50.EQ.0)THEN
                 BUF_SIZE=min(BUF_SIZE,A*dble(NCB))
              ELSE
                 BUF_SIZE=min(BUF_SIZE,A*A)
              ENDIF
           ENDIF
           BUF_SIZE=dble(K821)
           DO i=1,NUMBER_OF_PROCS
              A=dble(MD_MEM(TEMP_ID(i)))/                               &
     &             dble(NELIM)
              A=A*dble(NFRONT)
              IF(K50.EQ.0)THEN
                 B=dble(int(dble(NCB)/dble(NUMBER_OF_PROCS))+1)*        &
     &                dble(NFRONT)
              ELSE
                 WHAT = 5
                 CALL AGMG_MUMPS_503(WHAT, KEEP,KEEP8, NCB,                  &
     &                NFRONT, min(NCB,OTHERS), J, X8)
                 B=dble(X8)+(dble(J)*dble(NELIM))
              ENDIF
              NELIM_MEM_SIZE=A+B
              MEM_SIZE_WEAK(i)=NELIM_MEM_SIZE
            IF((SBTR_WHICH_M.EQ.0).OR.(.NOT.BDC_SBTR))THEN
               IF(BDC_M2_MEM)THEN
                  MEM_SIZE_STRONG(i)=                                   &
     &                 dble(TAB_MAXS(TEMP_ID(i)))-DM_MEM(TEMP_ID(i))-   &
     &                 LU_USAGE(TEMP_ID(i))-NIV2(TEMP_ID(i)+1)
               ELSE
                  MEM_SIZE_STRONG(i)=                                   &
     &                 dble(TAB_MAXS(TEMP_ID(i)))-DM_MEM(TEMP_ID(i))-   &
     &                 LU_USAGE(TEMP_ID(i))
               ENDIF
            ELSE
               IF(BDC_SBTR)THEN
                  IF(BDC_M2_MEM)THEN
                     MEM_SIZE_STRONG(i)=                                &
     &                    dble(TAB_MAXS(TEMP_ID(i)))-DM_MEM(TEMP_ID(i))-&
     &                    LU_USAGE(TEMP_ID(i))-NIV2(TEMP_ID(i)+1)-      &
     &                    (SBTR_MEM(TEMP_ID(i))-SBTR_CUR(TEMP_ID(i)))
                  ELSE
                     MEM_SIZE_STRONG(i)=                                &
     &                    dble(TAB_MAXS(TEMP_ID(i)))-DM_MEM(TEMP_ID(i))-&
     &                    LU_USAGE(TEMP_ID(i))-                         &
     &                    (SBTR_MEM(TEMP_ID(i))-SBTR_CUR(TEMP_ID(i)))
                  ENDIF
               ENDIF
            ENDIF
            IF(min(MEM_SIZE_STRONG(i),MEM_SIZE_WEAK(i)).LT.dble(0))THEN
                IF(MEM_SIZE_STRONG(i).LT.0.0d0)THEN
                   MEM_SIZE_STRONG(i)=dble(0)
                ELSE
                   MEM_SIZE_WEAK(i)=dble(0)
                ENDIF
             ENDIF
          ENDDO
       ELSE
          BUF_SIZE=dble(K821)
          DO i=1,NUMBER_OF_PROCS
            IF((SBTR_WHICH_M.EQ.0).OR.(.NOT.BDC_SBTR))THEN
               IF(BDC_M2_MEM)THEN
                  MEM_SIZE_STRONG(i)=                                   &
     &                 dble(TAB_MAXS(TEMP_ID(i)))-DM_MEM(TEMP_ID(i))-   &
     &                 LU_USAGE(TEMP_ID(i))-NIV2(TEMP_ID(i)+1)
               ELSE
                  MEM_SIZE_STRONG(i)=                                   &
     &                 dble(TAB_MAXS(TEMP_ID(i)))-DM_MEM(TEMP_ID(i))-   &
     &                 LU_USAGE(TEMP_ID(i))
               ENDIF
            ELSE
               IF(BDC_SBTR)THEN
                  IF(BDC_M2_MEM)THEN
                     MEM_SIZE_STRONG(i)=                                &
     &                    dble(TAB_MAXS(TEMP_ID(i)))-DM_MEM(TEMP_ID(i))-&
     &                    LU_USAGE(TEMP_ID(i))-NIV2(TEMP_ID(i)+1)-      &
     &                    (SBTR_MEM(TEMP_ID(i))-SBTR_CUR(TEMP_ID(i)))
                  ELSE
                     MEM_SIZE_STRONG(i)=                                &
     &                    dble(TAB_MAXS(TEMP_ID(i)))-DM_MEM(TEMP_ID(i))-&
     &                    LU_USAGE(TEMP_ID(i))-                         &
     &                    (SBTR_MEM(TEMP_ID(i))-SBTR_CUR(TEMP_ID(i)))
                  ENDIF
               ENDIF
            ENDIF
            MEM_SIZE_STRONG(i)=max(dble(0),MEM_SIZE_STRONG(i))
            MEM_SIZE_WEAK(i)=huge(MEM_SIZE_WEAK(i))
          ENDDO
       ENDIF
       IF((((NUMBER_OF_PROCS.LE.K83).AND.FORCE_CAND).AND.               &
     &      (TOTAL_COST.GE.SOMME)).OR.                                  &
     &      (.NOT.FORCE_CAND).OR.                                       &
     &      (((NUMBER_OF_PROCS+1).GT.K83).AND.FORCE_CAND))THEN
               REF=NSLAVES_REF
               SMALL_SET=NSLAVES_REF
               IF(.NOT.SMP)THEN
                  DO i=NSLAVES_REF,1,-1
                     SOMME=dble(0)
                     DO J=1,i
                        SOMME=SOMME+TEMP(J)
                     ENDDO
                     SOMME=(dble(i)*TEMP(i))-SOMME
                     IF(TOTAL_COST.GE.SOMME) GOTO 444
                  ENDDO
  444             CONTINUE
                  REF=i
                  SMALL_SET=REF
                  MAX_LOAD=TEMP(SMALL_SET)
               ELSE
                  X=min(SIZE_MY_SMP,NSLAVES_REF)
  450             CONTINUE
                  SOMME=dble(0)
                  DO J=1,X
                     SOMME=SOMME+(TEMP(X)-TEMP(J))
                  ENDDO
                  IF(SOMME.GT.TOTAL_COST)THEN
                     X=X-1
                     GOTO 450
                  ELSE
                     IF(X.LT.SIZE_MY_SMP) THEN
                        REF=X
                        SMALL_SET=REF
                        MAX_LOAD=TEMP(SMALL_SET)
                     ELSE
                        X=min(SIZE_MY_SMP,NSLAVES_REF)
                        J=X+1
                        MAX_LOAD=TEMP(X)
                        TMP_SUM=MAX_LOAD
                        DO i=X+1,OTHERS
                           IF(TEMP(i).GT.MAX_LOAD)THEN
                              SOMME=SOMME+(dble(i-1)*(TEMP(i)-MAX_LOAD))
                              TMP_SUM=MAX_LOAD
                              MAX_LOAD=TEMP(i)
                           ELSE
                              SOMME=SOMME+(MAX_LOAD-TEMP(i))
                           ENDIF
                           IF(i.EQ.NSLAVES_REF)THEN
                              SMALL_SET=NSLAVES_REF
                              REF=SMALL_SET
                              GOTO 323
                           ENDIF
                           IF(SOMME.GT.TOTAL_COST)THEN
                              REF=i-1
                              SMALL_SET=i-1
                              MAX_LOAD=TMP_SUM
                              GOTO 323
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDIF
               ENDIF
  323          CONTINUE
               MAX_LOAD=dble(0)
               DO i=1,SMALL_SET
                  MAX_LOAD=max(MAX_LOAD,TEMP(i))
               ENDDO
               TEMP_MAX_LOAD=MAX_LOAD
               NB_ROWS=0
               TMP_SUM=dble(0)
               CHOSEN=0
               ACC=0
               NB_SAT=0
               NB_ZERO=0
               DO i=1,SMALL_SET
                  IF(K50.EQ.0)THEN
                     X=int(BUF_SIZE/dble(NCB+1))-1
                     BANDE_K821=dble(X)*dble(NFRONT)
                  ELSE
                     A=dble(1)
                     B=dble(ACC+2)
                     C=-BUF_SIZE+dble(ACC+NELIM)
                     DELTA=(B*B)-(dble(4)*A*C)
                     X=int((-B+sqrt(DELTA))/(dble(2)*A))
                     IF(X.GT.NCB-ACC) X=NCB-ACC
                     BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                  ENDIF
                  IF(HAVE_TYPE1_SON)THEN
                     IF(K50.EQ.0)THEN
                        X=int((BUF_SIZE-dble(NFRONT))/dble(NFRONT+1))
                        BANDE_K821=dble(X)*dble(NFRONT)
                     ELSE
                        A=dble(1)
                        B=dble(ACC+2+NELIM)
                        C=-BUF_SIZE+dble(ACC+NELIM)
                        DELTA=(B*B)-(dble(4)*A*C)
                        X=int((-B+sqrt(DELTA))/(dble(2)*A))
                        IF(X.GT.NCB-ACC) X=NCB-ACC
                        BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                     ENDIF
                  ENDIF
                  MAX_MEM_ALLOW=BANDE_K821
                  IF(BDC_MD)THEN
                     MAX_MEM_ALLOW=min(                                 &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                    BANDE_K821)
                     MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                  ENDIF
                  IF(K50.EQ.0)THEN
                     KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                     X=int((MAX_LOAD-TEMP(i))/                          &
     &                    (dble(NELIM)*dble(2*NFRONT-NELIM)))
                     IF(X.GE.KMAX)THEN
                        IF(KMAX.GE.KMIN)THEN
                           X=KMAX
                           NB_SAT=NB_SAT+1
                        ELSE
                           X=0
                        ENDIF
                     ELSE
                        IF(X.LT.KMIN)THEN
                           X=0
                        ENDIF
                     ENDIF
                     IF((ACC+X).GT.NCB) X=NCB-ACC
                  ENDIF
                  IF(K50.NE.0)THEN
                        A=dble(1)
                        B=dble(ACC+NELIM)
                        C=dble(-MAX_MEM_ALLOW)
                        DELTA=((B*B)-(dble(4)*A*C))
                        KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                     A=dble(NELIM)
                     B=dble(NELIM)*(dble(NELIM)+dble(2*ACC+1))
                     C=-(MAX_LOAD-TEMP(i))
                     DELTA=(B*B-(dble(4)*A*C))
                     X=int((-B+sqrt(DELTA))/(dble(2)*A))
                     IF(X.LT.0) THEN
                        WRITE(*,*)MYID,                                 &
     &    ': Internal error 1 in DMUMPS_518'
                        CALL AGMG_MUMPS_ABORT()
                     ENDIF
                     IF(X.GE.KMAX)THEN
                        IF(KMAX.GE.KMIN)THEN
                           X=KMAX
                           NB_SAT=NB_SAT+1
                        ELSE
                           X=0
                        ENDIF
                     ELSE
                        IF(X.LT.KMIN)THEN
                           X=0
                        ENDIF
                     ENDIF
                     IF((ACC+X).GT.NCB) X=NCB-ACC
                  ENDIF
                  NB_ROWS(i)=X
                  ACC=ACC+X
                  CHOSEN=CHOSEN+1
                  IF(SMP)THEN
                     IF(MIN_LOAD.GT.TEMP(i))THEN
                        MIN_LOAD=TEMP(i)
                        POS_MIN_LOAD=i
                     ENDIF
                  ENDIF
                  TMP_SUM=MAX_LOAD
                  IF(K50.EQ.0)THEN
                     MAX_LOAD=max(MAX_LOAD,                             &
     &                    (TEMP(i)+(dble(NELIM) *                       &
     &                    dble(NB_ROWS(i)))+                            &
     &                    (dble(NB_ROWS(i))*dble(NELIM)*                &
     &                    dble(2*NFRONT-NELIM-1))))
                  ELSE
                     MAX_LOAD=max(MAX_LOAD,                             &
     &               TEMP(i)+(dble(NELIM) * dble(NB_ROWS(i)))*          &
     &                    dble(2*(NELIM+ACC)-NB_ROWS(i)                 &
     &                    -NELIM+1))
                  ENDIF
                  IF(TMP_SUM.LT.MAX_LOAD)THEN
                  ENDIF
                  IF(NCB-ACC.LT.KMIN) GOTO 888
                  IF(NCB.EQ.ACC) GOTO 888
                  IF(ACC.GT.NCB) THEN
                    WRITE(*,*)MYID,                                     &
     &      ': Internal error 2 in DMUMPS_518'
                    CALL AGMG_MUMPS_ABORT()
                  ENDIF
               ENDDO
  888          CONTINUE
               SOMME=dble(0)
               X=NFRONT-NCB
               IF((ACC.GT.NCB))THEN
                  WRITE(*,*)MYID,                                       &
     &          ': Internal error 3 in DMUMPS_518'
                  CALL AGMG_MUMPS_ABORT()
               ENDIF
               IF((ACC.LT.NCB))THEN
                  IF(K50.NE.0)THEN
                     IF(SMALL_SET.LE.OTHERS)THEN
                       IF((NB_SAT.EQ.SMALL_SET).AND.(SMALL_SET.LT.      &
     &                      NSLAVES_REF))THEN
                          SMALL_SET=REF+1
                          REF=REF+1
                          NB_ROWS=0
                          GOTO 323
                       ENDIF
                       ADDITIONNAL_ROWS_SPECIAL=NCB-ACC
                       DO i=1,SMALL_SET
                          MAX_LOAD=TEMP_MAX_LOAD
                          ADDITIONNAL_ROWS=NCB-ACC
                          SOMME=dble(NELIM)*                            &
     &                         dble(ADDITIONNAL_ROWS)*                  &
     &                         dble(2*NFRONT-ADDITIONNAL_ROWS-NELIM     &
     &                         +1)
                          SOMME=SOMME/dble(SMALL_SET-NB_SAT)
                          NB_ROWS=0
                          NB_ZERO=0
                          ACC=0
                          CHOSEN=0
                          NB_SAT=0
                          IF(SMP)THEN
                             MIN_LOAD=TEMP(1)
                             POS_MIN_LOAD=1
                          ENDIF
                          DO J=1,SMALL_SET
                             A=dble(1)
                             B=dble(ACC+2)
                             C=-BUF_SIZE+dble(ACC+NELIM)
                             DELTA=(B*B)-(dble(4)*A*C)
                             X=int((-B+sqrt(DELTA))/(dble(2)*A))
                             IF(X.GT.NCB-ACC) X=NCB-ACC
                             BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                             IF(HAVE_TYPE1_SON)THEN
                                A=dble(1)
                                B=dble(ACC+2+NELIM)
                                C=-BUF_SIZE+dble(ACC+NELIM)
                                DELTA=(B*B)-(dble(4)*A*C)
                                X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                IF(X.GT.NCB-ACC) X=NCB-ACC
                                BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                             ENDIF
                             MAX_MEM_ALLOW=BANDE_K821
                             IF(BDC_MD)THEN
                                MAX_MEM_ALLOW=min(                      &
     &                        min(MEM_SIZE_WEAK(J),MEM_SIZE_STRONG(J)), &
     &                               BANDE_K821)
                                MAX_MEM_ALLOW=max(dble(0),              &
     &                               MAX_MEM_ALLOW)
                             ENDIF
                             A=dble(1)
                             B=dble(ACC+NELIM)
                             C=dble(-MAX_MEM_ALLOW)
                             DELTA=((B*B)-(dble(4)*A*C))
                             KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                             A=dble(NELIM)
                             B=(dble(NELIM)*dble(NELIM+2*ACC+1))
                             C=-(MAX_LOAD-TEMP(J)+SOMME)
                             DELTA=(B*B-(dble(4)*A*C))
                             X=int((-B+sqrt(DELTA))/(dble(2)*A))
                             X=X+1
                             IF(X.LT.0) THEN
                                WRITE(*,*)MYID,                         &
     &    ': Internal error 4 in DMUMPS_518'
                                CALL AGMG_MUMPS_ABORT()
                             ENDIF
                             IF(X.GE.KMAX)THEN
                                IF(KMAX.GE.KMIN)THEN
                                   X=KMAX
                                   NB_SAT=NB_SAT+1
                                ELSE
                                   NB_ZERO=NB_ZERO+1
                                   X=0
                                ENDIF
                             ELSE
                                IF(X.LT.min(KMIN,KMAX))THEN
                                   NB_ZERO=NB_ZERO+1
                                   X=0
                                ENDIF
                             ENDIF
                             IF((ACC+X).GT.NCB) X=NCB-ACC
                             NB_ROWS(J)=X
                             IF(SMP)THEN
                                IF(MIN_LOAD.GT.TEMP(J))THEN
                                   MIN_LOAD=TEMP(J)
                                   POS_MIN_LOAD=i
                                ENDIF
                             ENDIF
                             CHOSEN=CHOSEN+1
                             ACC=ACC+X
                             TMP_SUM=MAX_LOAD
                             TEMP_MAX_LOAD=max(TEMP_MAX_LOAD,           &
     &                            TEMP(J)+(dble(NELIM) *                &
     &                            dble(NB_ROWS(J)))*                    &
     &                            dble(2*(NELIM+                        &
     &                            ACC)-NB_ROWS(J)                       &
     &                            -NELIM+1))
                             IF(REF.LE.NUMBER_OF_PROCS-1)THEN
                                IF(TEMP_MAX_LOAD.GT.TEMP(REF+1))THEN
                                   IF(SMALL_SET.LT.NSLAVES_REF)THEN
                                      SMALL_SET=REF+1
                                      REF=REF+1
                                      NB_ROWS=0
                                      GOTO 323
                                   ENDIF
                                ENDIF
                             ENDIF
                             IF(NCB.EQ.ACC) GOTO 666
                          ENDDO
                          IF(NB_SAT.EQ.SMALL_SET)THEN
                             IF(SMALL_SET.LT.NSLAVES_REF)THEN
                                SMALL_SET=REF+1
                                REF=REF+1
                                NB_ROWS=0
                                GOTO 323
                             ELSE
                                GOTO 434
                             ENDIF
                          ENDIF
                          IF(NB_ZERO.EQ.SMALL_SET)THEN
                             IF(SMALL_SET.LT.NSLAVES_REF)THEN
                                SMALL_SET=REF+1
                                REF=REF+1
                                NB_ROWS=0
                                GOTO 323
                             ELSE
                                GOTO 434
                             ENDIF
                          ENDIF
                          IF((NB_SAT+NB_ZERO).EQ.SMALL_SET)THEN
                             IF(SMALL_SET.LT.NSLAVES_REF)THEN
                                SMALL_SET=REF+1
                                REF=REF+1
                                NB_ROWS=0
                                GOTO 323
                             ELSE
                                GOTO 434
                             ENDIF
                          ENDIF
                       ENDDO
  434                  CONTINUE
                       ADDITIONNAL_ROWS=NCB-ACC
                       IF(ADDITIONNAL_ROWS.NE.0)THEN
                          IF(ADDITIONNAL_ROWS.LT.KMIN)THEN
                             i=CHOSEN
                             J=ACC
  436                        CONTINUE
                             IF(NB_ROWS(i).NE.0)THEN
                                J=J-NB_ROWS(i)
                                A=dble(1)
                                B=dble(J+2)
                                C=-BUF_SIZE+dble(J+NELIM)
                                DELTA=(B*B)-(dble(4)*A*C)
                                X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                IF(X.GT.NCB-J) X=NCB-J
                                BANDE_K821=dble(X)*dble(NELIM+J+X)
                                IF(HAVE_TYPE1_SON)THEN
                                   A=dble(1)
                                   B=dble(J+2+NELIM)
                                   C=-BUF_SIZE+dble(J+NELIM)
                                   DELTA=(B*B)-(dble(4)*A*C)
                                   X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                   IF(X.GT.NCB-J) X=NCB-J
                                   BANDE_K821=dble(X)*dble(NELIM+J+X)
                                ENDIF
                                MAX_MEM_ALLOW=BANDE_K821
                                IF(BDC_MD)THEN
                                   MAX_MEM_ALLOW=min(                   &
     &                         min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),&
     &                                  BANDE_K821)
                                   MAX_MEM_ALLOW=max(dble(0),           &
     &                                  MAX_MEM_ALLOW)
                                ENDIF
                                A=dble(1)
                                B=dble(J+NELIM)
                                C=dble(-MAX_MEM_ALLOW)
                                DELTA=((B*B)-(dble(4)*A*C))
                                KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                                IF(NB_ROWS(i).NE.KMAX)THEN
                                   IF(NCB-J.LE.KMAX)THEN
                                      NB_ROWS(i)=+NCB-J
                                      ADDITIONNAL_ROWS=0
                                   ENDIF
                                ENDIF
                                TEMP_MAX_LOAD=max(TEMP_MAX_LOAD,        &
     &                               TEMP(i)+                           &
     &                               (dble(NELIM) * dble(NB_ROWS(i)))*  &
     &                               dble(2*(NELIM+                     &
     &                               ACC)-NB_ROWS(i)                    &
     &                               -NELIM+1))
                                IF(REF.LE.NUMBER_OF_PROCS-1)THEN
                                   IF(TEMP_MAX_LOAD.GT.TEMP(REF+1))THEN
                                      IF(SMALL_SET.LT.NSLAVES_REF)THEN
                                         SMALL_SET=REF+1
                                         REF=REF+1
                                         NB_ROWS=0
                                         GOTO 323
                                      ENDIF
                                   ENDIF
                                ENDIF
                             ELSE
                                i=i-1
                                IF(i.NE.0)GOTO 436
                             ENDIF
                             IF(ADDITIONNAL_ROWS.NE.0)THEN
                                i=CHOSEN
                                IF(i.NE.SMALL_SET)THEN
                                   i=i+1
                                   IF(NB_ROWS(i).NE.0)THEN
                                      WRITE(*,*)MYID,                   &
     &    ': Internal error 5 in DMUMPS_518'
                                      CALL AGMG_MUMPS_ABORT()
                                   ENDIF
                                ENDIF
                                NB_ROWS(i)=NB_ROWS(i)+ADDITIONNAL_ROWS
                                ADDITIONNAL_ROWS=0
                             ENDIF
                             CHOSEN=i
                          ENDIF
                       ENDIF
                       i=CHOSEN+1
                       DO WHILE ((ADDITIONNAL_ROWS.NE.0)                &
     &                      .AND.(i.LE.NUMBER_OF_PROCS))
                          IF((TEMP(i).LE.MAX_LOAD))THEN
                             A=dble(1)
                             B=dble(ACC+2)
                             C=-BUF_SIZE+dble(ACC+NELIM)
                             DELTA=(B*B)-(dble(4)*A*C)
                             X=int((-B+sqrt(DELTA))/(dble(2)*A))
                             IF(X.GT.NCB-ACC) X=NCB-ACC
                             BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                             IF(HAVE_TYPE1_SON)THEN
                                A=dble(1)
                                B=dble(ACC+2+NELIM)
                                C=-BUF_SIZE+dble(ACC+NELIM)
                                DELTA=(B*B)-(dble(4)*A*C)
                                X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                IF(X.GT.NCB-ACC) X=NCB-ACC
                                BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                             ENDIF
                             MAX_MEM_ALLOW=BANDE_K821
                             IF(BDC_MD)THEN
                                MAX_MEM_ALLOW=min(                      &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                               BANDE_K821)
                                MAX_MEM_ALLOW=max(dble(0),              &
     &                               MAX_MEM_ALLOW)
                             ENDIF
                             A=dble(1)
                             B=dble(ACC+NELIM)
                             C=dble(-MAX_MEM_ALLOW)
                             DELTA=((B*B)-(dble(4)*A*C))
                             KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                             A=dble(NELIM)
                             B=dble(NELIM)*dble(NELIM+2*ACC+1)
                             C=-(MAX_LOAD-TEMP(i))
                             DELTA=(B*B-(dble(4)*A*C))
                             X=int((-B+sqrt(DELTA))/(dble(2)*A))
                             IF(X.GE.KMAX)THEN
                                IF(KMAX.GE.KMIN)THEN
                                   X=KMAX
                                ELSE
                                   X=0
                                ENDIF
                             ELSE
                                IF(X.LT.KMIN)THEN
                                   X=0
                                ENDIF
                             ENDIF
                             IF((ACC+X).GT.NCB) X=NCB-ACC
                             NB_ROWS(i)=X
                             ACC=ACC+X
                             ADDITIONNAL_ROWS=NCB-ACC
                          ELSE IF((TEMP(i).GT.MAX_LOAD))THEN
                             MAX_LOAD=TEMP(i)
                             NB_SAT=0
                             ACC=0
                             NB_ROWS=0
                             DO J=1,i
                                A=dble(1)
                                B=dble(ACC+2)
                                C=-BUF_SIZE+dble(ACC+NELIM)
                                DELTA=(B*B)-(dble(4)*A*C)
                                X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                IF(X.GT.NCB-ACC) X=NCB-ACC
                                BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                                IF(HAVE_TYPE1_SON)THEN
                                   A=dble(1)
                                   B=dble(ACC+2+NELIM)
                                   C=-BUF_SIZE+dble(ACC+NELIM)
                                   DELTA=(B*B)-(dble(4)*A*C)
                                   X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                   IF(X.GT.NCB-ACC) X=NCB-ACC
                                   BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                                ENDIF
                                MAX_MEM_ALLOW=BANDE_K821
                                IF(BDC_MD)THEN
                                   MAX_MEM_ALLOW=min(                   &
     &                    min(MEM_SIZE_WEAK(J),MEM_SIZE_STRONG(J)),     &
     &                                  BANDE_K821)
                                   MAX_MEM_ALLOW=max(dble(0),           &
     &                                  MAX_MEM_ALLOW)
                                ENDIF
                                A=dble(1)
                                B=dble(ACC+NELIM)
                                C=dble(-MAX_MEM_ALLOW)
                                DELTA=((B*B)-(dble(4)*A*C))
                                KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                                A=dble(NELIM)
                                B=dble(NELIM)*dble(NELIM+2*ACC+1)
                                C=-(MAX_LOAD-TEMP(J))
                                DELTA=(B*B-(dble(4)*A*C))
                                X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                IF(X.LT.0) THEN
                        WRITE(*,*)MYID,                                 &
     &    ': Internal error 6 in DMUMPS_518'
                                   CALL AGMG_MUMPS_ABORT()
                                ENDIF
                                IF(X.GE.KMAX)THEN
                                   IF(KMAX.GE.KMIN)THEN
                                      X=KMAX
                                      NB_SAT=NB_SAT+1
                                   ELSE
                                      X=0
                                   ENDIF
                                ELSE
                                   IF(X.LT.min(KMIN,KMAX))THEN
                                      X=0
                                   ENDIF
                                ENDIF
                                IF((ACC+X).GT.NCB) X=NCB-ACC
                                NB_ROWS(J)=X
                                IF(SMP)THEN
                                   IF(MIN_LOAD.GT.TEMP(J))THEN
                                      MIN_LOAD=TEMP(J)
                                      POS_MIN_LOAD=i
                                   ENDIF
                                ENDIF
                                ACC=ACC+X
                                MAX_LOAD=max(MAX_LOAD,                  &
     &                               TEMP(J)+                           &
     &                               (dble(NELIM)*dble(NB_ROWS(J)))*    &
     &                               dble(2*(NELIM+                     &
     &                               ACC)-NB_ROWS(J)                    &
     &                               -NELIM+1))
                                IF(NCB.EQ.ACC) GOTO 741
                                IF(NCB-ACC.LT.KMIN) GOTO 210
                             ENDDO
  210                        CONTINUE
                          ENDIF
  741                     CONTINUE
                          i=i+1
                          ADDITIONNAL_ROWS=NCB-ACC
                       ENDDO
                       CHOSEN=i-1
                       IF(ADDITIONNAL_ROWS.NE.0)THEN
                          ADDITIONNAL_ROWS=NCB-ACC
                          SOMME=dble(NELIM)*dble(ADDITIONNAL_ROWS)*     &
     &                         dble(2*NFRONT-ADDITIONNAL_ROWS-          &
     &                         NELIM+1)
                          SOMME=SOMME/dble(NUMBER_OF_PROCS)
                          NB_ROWS=0
                          ACC=0
                          CHOSEN=0
                          IF(SMP)THEN
                             MIN_LOAD=TEMP(1)
                             POS_MIN_LOAD=1
                          ENDIF
                          DO i=1,OTHERS
                             A=dble(1)
                             B=dble(ACC+2)
                             C=-BUF_SIZE+dble(ACC+NELIM)
                             DELTA=(B*B)-(dble(4)*A*C)
                             X=int((-B+sqrt(DELTA))/(dble(2)*A))
                             IF(X.GT.NCB-ACC) X=NCB-ACC
                             BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                             IF(HAVE_TYPE1_SON)THEN
                                A=dble(1)
                                B=dble(ACC+2+NELIM)
                                C=-BUF_SIZE+dble(ACC+NELIM)
                                DELTA=(B*B)-(dble(4)*A*C)
                                X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                IF(X.GT.NCB-ACC) X=NCB-ACC
                                BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                             ENDIF
                             MAX_MEM_ALLOW=BANDE_K821
                             IF(BDC_MD)THEN
                                MAX_MEM_ALLOW=min(                      &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                               BANDE_K821)
                                MAX_MEM_ALLOW=max(dble(0),              &
     &                               MAX_MEM_ALLOW)
                             ENDIF
                             A=dble(1)
                             B=dble(ACC+NELIM)
                             C=dble(-MAX_MEM_ALLOW)
                             DELTA=((B*B)-(dble(4)*A*C))
                             KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                             A=dble(NELIM)
                             B=dble(NELIM)*dble(NELIM+2*ACC+1)
                             C=-(MAX_LOAD-TEMP(i)+SOMME)
                             DELTA=(B*B-(dble(4)*A*C))
                             X=int((-B+sqrt(DELTA))/(dble(2)*A))
                             IF(X.LT.0) THEN
                                WRITE(*,*)MYID,                         &
     &    ': Internal error 7 in DMUMPS_518'
                                CALL AGMG_MUMPS_ABORT()
                             ENDIF
                             IF(X.GE.KMAX)THEN
                                IF(KMAX.GE.KMIN)THEN
                                   X=KMAX
                                ELSE
                                   X=0
                                ENDIF
                             ELSE
                                IF(X.LT.min(KMIN,KMAX))THEN
                                   X=min(KMAX,KMIN)
                                ENDIF
                             ENDIF
                             IF((ACC+X).GT.NCB) X=NCB-ACC
                             NB_ROWS(i)=X
                             IF(SMP)THEN
                                IF(MIN_LOAD.GT.TEMP(i))THEN
                                   MIN_LOAD=TEMP(i)
                                   POS_MIN_LOAD=i
                                ENDIF
                             ENDIF
                             CHOSEN=CHOSEN+1
                             ACC=ACC+X
                             IF(NCB.EQ.ACC) GOTO 666
                             IF(NCB-ACC.LT.KMIN) GOTO 488
                          ENDDO
  488                     CONTINUE
                          ADDITIONNAL_ROWS=NCB-ACC
                          SOMME=dble(NELIM)*                            &
     &                         dble(ADDITIONNAL_ROWS)*                  &
     &                         dble(2*NFRONT-ADDITIONNAL_ROWS-          &
     &                         NELIM+1)
                          SOMME=SOMME/dble(NUMBER_OF_PROCS)
                          NB_ROWS=0
                          ACC=0
                          CHOSEN=0
                          IF(SMP)THEN
                             MIN_LOAD=TEMP(1)
                             POS_MIN_LOAD=1
                          ENDIF
                          DO i=1,OTHERS
                             A=dble(1)
                             B=dble(ACC+2)
                             C=-BUF_SIZE+dble(ACC+NELIM)
                             DELTA=(B*B)-(dble(4)*A*C)
                             X=int((-B+sqrt(DELTA))/(dble(2)*A))
                             IF(X.GT.NCB-ACC) X=NCB-ACC
                             BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                             IF(HAVE_TYPE1_SON)THEN
                                A=dble(1)
                                B=dble(ACC+2+NELIM)
                                C=-BUF_SIZE+dble(ACC+NELIM)
                                DELTA=(B*B)-(dble(4)*A*C)
                                X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                IF(X.GT.NCB-ACC) X=NCB-ACC
                                BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                             ENDIF
                             MAX_MEM_ALLOW=BANDE_K821
                             IF(BDC_MD)THEN
                                MAX_MEM_ALLOW=min(BANDE_K821,           &
     &                               MEM_SIZE_STRONG(i))
                                MAX_MEM_ALLOW=max(dble(0),              &
     &                               MAX_MEM_ALLOW)
                             ENDIF
                             A=dble(1)
                             B=dble(ACC+NELIM)
                             C=dble(-MAX_MEM_ALLOW)
                             DELTA=((B*B)-(dble(4)*A*C))
                             KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                             A=dble(NELIM)
                             B=dble(NELIM)*dble(NELIM+2*ACC+1)
                             C=-(MAX_LOAD-TEMP(i)+SOMME)
                             DELTA=(B*B-(dble(4)*A*C))
                             X=int((-B+sqrt(DELTA))/(dble(2)*A))
                             IF(X.LT.0) THEN
                                WRITE(*,*)MYID,                         &
     &    ': Internal error 8 in DMUMPS_518'
                                CALL AGMG_MUMPS_ABORT()
                             ENDIF
                             IF(X.GE.KMAX)THEN
                                X=KMAX
                             ELSE
                                IF(X.LT.KMIN)THEN
                                   X=KMIN
                                ENDIF
                             ENDIF
                             IF((ACC+X).GT.NCB) X=NCB-ACC
                             NB_ROWS(i)=X
                             IF(SMP)THEN
                                IF(MIN_LOAD.GT.TEMP(i))THEN
                                   MIN_LOAD=TEMP(i)
                                   POS_MIN_LOAD=i
                                ENDIF
                             ENDIF
                             CHOSEN=CHOSEN+1
                             ACC=ACC+X
                             IF(NCB.EQ.ACC) GOTO 666
                             IF(NCB-ACC.LT.KMIN) GOTO 477
                          ENDDO
  477                     CONTINUE
                          IF(ACC.NE.NCB)THEN
                             NB_SAT=0
                             ACC=0
                             CHOSEN=0
                             IF(SMP)THEN
                                MIN_LOAD=TEMP(1)
                                POS_MIN_LOAD=1
                             ENDIF
                             DO i=1,OTHERS
                                A=dble(1)
                                B=dble(ACC+2)
                                C=-BUF_SIZE+dble(ACC+NELIM)
                                DELTA=(B*B)-(dble(4)*A*C)
                                X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                IF(X.GT.NCB-ACC) X=NCB-ACC
                                BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                                IF(HAVE_TYPE1_SON)THEN
                                   A=dble(1)
                                   B=dble(ACC+2+NELIM)
                                   C=-BUF_SIZE+dble(ACC+NELIM)
                                   DELTA=(B*B)-(dble(4)*A*C)
                                   X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                   IF(X.GT.NCB-ACC) X=NCB-ACC
                                   BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                                ENDIF
                                MAX_MEM_ALLOW=BANDE_K821
                                IF(BDC_MD)THEN
                                   MAX_MEM_ALLOW=min(BANDE_K821,        &
     &                                  MEM_SIZE_STRONG(i))
                                   MAX_MEM_ALLOW=max(dble(0),           &
     &                                  MAX_MEM_ALLOW)
                                ENDIF
                                A=dble(1)
                                B=dble(ACC+NELIM)
                                C=dble(-MAX_MEM_ALLOW)
                                DELTA=((B*B)-(dble(4)*A*C))
                                KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                                X=KMAX-NB_ROWS(i)
                                IF((ACC+NB_ROWS(i)+X).GT.NCB)           &
     &                               X=NCB-(ACC+NB_ROWS(i))
                                NB_ROWS(i)=NB_ROWS(i)+X
                                IF((dble(NB_ROWS(i))*                   &
     &                               dble(NB_ROWS(i)+ACC)).EQ.          &
     &                               BANDE_K821)THEN
                                   NB_SAT=NB_SAT+1
                                ENDIF
                                ACC=ACC+NB_ROWS(i)
                                IF(SMP)THEN
                                   IF(MIN_LOAD.GT.TEMP(i))THEN
                                      MIN_LOAD=TEMP(i)
                                      POS_MIN_LOAD=i
                                   ENDIF
                                ENDIF
                                CHOSEN=CHOSEN+1
                                IF(NCB.EQ.ACC) GOTO 666
                                IF(NCB-ACC.LT.KMIN) GOTO 834
                             ENDDO
  834                        CONTINUE
                          ENDIF
                          IF(ACC.NE.NCB)THEN
                            ADDITIONNAL_ROWS=NCB-ACC
                            SOMME=dble(NELIM)*                          &
     &                           dble(ADDITIONNAL_ROWS)*                &
     &                           dble(2*NFRONT-ADDITIONNAL_ROWS-        &
     &                           NELIM+1)
                            SOMME=SOMME/dble(NUMBER_OF_PROCS-NB_SAT)
                            ACC=0
                            DO i=1,CHOSEN
                               A=dble(1)
                               B=dble(ACC+2)
                               C=-BUF_SIZE+dble(ACC+NELIM)
                               DELTA=(B*B)-(dble(4)*A*C)
                               X=int((-B+sqrt(DELTA))/(dble(2)*A))
                               IF(X.GT.NCB-ACC) X=NCB-ACC
                               BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                               IF(HAVE_TYPE1_SON)THEN
                                  A=dble(1)
                                  B=dble(ACC+2+NELIM)
                                  C=-BUF_SIZE+dble(ACC+NELIM)
                                  DELTA=(B*B)-(dble(4)*A*C)
                                  X=int((-B+sqrt(DELTA))/(dble(2)*A))
                                  IF(X.GT.NCB-ACC) X=NCB-ACC
                                  BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                               ENDIF
                               IF((dble(NB_ROWS(i))*                    &
     &                              dble(NB_ROWS(i)+ACC)).EQ.           &
     &                              BANDE_K821)THEN
                                  GOTO 102
                               ENDIF
                               A=dble(NELIM)
                               B=dble(NELIM)*                           &
     &                              dble(NELIM+2*(ACC+NB_ROWS(i))+1)
                               C=-(SOMME)
                               DELTA=(B*B-(dble(4)*A*C))
                               X=int((-B+sqrt(DELTA))/(dble(2)*A))
                               A=dble(1)
                               B=dble(ACC+NELIM)
                               C=dble(-BANDE_K821)
                               DELTA=((B*B)-(dble(4)*A*C))
                               KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                               IF(X.LT.0) THEN
                                  WRITE(*,*)MYID,                       &
     &    ': Internal error 9 in DMUMPS_518'
                                  CALL AGMG_MUMPS_ABORT()
                               ENDIF
                               IF((ACC+X+NB_ROWS(i)).GT.NCB)THEN
                                  IF((NCB-ACC).GT.KMAX)THEN
                                     NB_ROWS(i)=KMAX
                                  ELSE
                                     NB_ROWS(i)=NCB-ACC
                                  ENDIF
                               ELSE
                                  IF((NB_ROWS(i)+X).GT.KMAX)THEN
                                     NB_ROWS(i)=KMAX
                                  ELSE
                                     NB_ROWS(i)=NB_ROWS(i)+X
                                  ENDIF
                               ENDIF
  102                          CONTINUE
                               ACC=ACC+NB_ROWS(i)
                               IF(NCB.EQ.ACC) THEN
                                  CHOSEN=i
                                  GOTO 666
                               ENDIF
                               IF(NCB-ACC.LT.KMIN) THEN
                                  CHOSEN=i
                                  GOTO 007
                               ENDIF
                            ENDDO
    7                       CONTINUE
                            DO i=1,CHOSEN
                               NB_ROWS(i)=NB_ROWS(i)+1
                               ACC=ACC+1
                               IF(ACC.EQ.NCB)GOTO 666
                            ENDDO
                            IF(ACC.LT.NCB)THEN
                               IF(SMP)THEN
                                  NB_ROWS(1)=NB_ROWS(1)+NCB-ACC
                               ELSE
                                  NB_ROWS(POS_MIN_LOAD)=                &
     &                                 NB_ROWS(POS_MIN_LOAD)+NCB-ACC
                               ENDIF
                            ENDIF
                         ENDIF
                         GOTO 666
                     ENDIF
                  ENDIF
                  GOTO 666
                 ENDIF
                 ADDITIONNAL_ROWS=NCB-ACC
                 i=CHOSEN+1
                 IF(NB_SAT.EQ.SMALL_SET) GOTO 777
                 DO i=1,SMALL_SET
                    IDWLOAD(i)=i
                    AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                    BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                    IF(HAVE_TYPE1_SON)THEN
                       AFFECTED=int((BUF_SIZE-dble(NFRONT))/            &
     &                      (dble(NFRONT+1)))
                       BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                    ENDIF
                    MAX_MEM_ALLOW=BANDE_K821
                    IF(BDC_MD)THEN
                       MAX_MEM_ALLOW=min(                               &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                      BANDE_K821)
                       MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                    ENDIF
                    WLOAD(i)=MAX_MEM_ALLOW
                 ENDDO
                 CALL AGMG_MUMPS_558(SMALL_SET, WLOAD, IDWLOAD)
                 NB_ZERO=0
                 IF((NB_SAT.EQ.SMALL_SET).AND.                          &
     &                (SMALL_SET.LT.NSLAVES_REF))THEN
                    SMALL_SET=REF+1
                    REF=REF+1
                    NB_ROWS=0
                    GOTO 323
                 ENDIF
                 IF((NB_SAT.EQ.SMALL_SET).AND.                          &
     &                (SMALL_SET.LE.NUMBER_OF_PROCS))GOTO 777
                 AFFECTED=int(ADDITIONNAL_ROWS/(SMALL_SET-NB_SAT))
                 AFFECTED=max(AFFECTED,1)
                 DO i=1,SMALL_SET
                    KMAX=int(WLOAD(i)/dble(NFRONT))
                    IF(NB_ROWS(IDWLOAD(i)).EQ.KMAX)THEN
                       GOTO 912
                    ENDIF
                    IF((NB_ROWS(IDWLOAD(i))+min(AFFECTED,               &
     &                   ADDITIONNAL_ROWS)).GT.KMAX)THEN
                       IF(NB_ROWS(IDWLOAD(i)).GT.KMAX)THEN
                       ENDIF
                       ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-               &
     &                      (KMAX-NB_ROWS(IDWLOAD(i)))
                       NB_ROWS(IDWLOAD(i))=KMAX
                       NB_SAT=NB_SAT+1
                       IF(NB_SAT.EQ.SMALL_SET)THEN
                          IF(SMALL_SET.NE.NSLAVES_REF)THEN
                             SMALL_SET=REF+1
                             REF=REF+1
                             NB_ROWS=0
                             GOTO 323
                          ELSE
                             MAX_LOAD=max(MAX_LOAD,                     &
     &                            (TEMP(IDWLOAD(i))+(dble(NELIM) *      &
     &                            dble(NB_ROWS(IDWLOAD(i))))+           &
     &                            (dble(NB_ROWS(IDWLOAD(i)))*           &
     &                            dble(NELIM))*                         &
     &                            dble(2*NFRONT-NELIM-1)))
                             GOTO 777
                          ENDIF
                       ENDIF
                       AFFECTED=int(ADDITIONNAL_ROWS/(SMALL_SET-NB_SAT))
                       AFFECTED=max(AFFECTED,1)
                    ELSE
                       IF((NB_ROWS(IDWLOAD(i))+min(AFFECTED,            &
     &                      ADDITIONNAL_ROWS)).GE.KMIN)THEN
                          X=min(AFFECTED,ADDITIONNAL_ROWS)
                          NB_ROWS(IDWLOAD(i))=NB_ROWS(IDWLOAD(i))+      &
     &                         X
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                       ELSE
                          X=int((MAX_LOAD-TEMP(IDWLOAD(i)))/            &
     &                         (dble(NELIM)*dble(2*NFRONT-NELIM)))
                          IF(X+AFFECTED.GT.ADDITIONNAL_ROWS)THEN
                             X=ADDITIONNAL_ROWS
                          ELSE
                             X=AFFECTED+X
                          ENDIF
                          IF(X.GE.KMIN)THEN
                             NB_ROWS(IDWLOAD(i))=NB_ROWS(IDWLOAD(i))+   &
     &                            X
                             ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-         &
     &                            X
                          ELSE
                             NB_ZERO=NB_ZERO+1
                          ENDIF
                       ENDIF
                    ENDIF
  912               CONTINUE
                    MAX_LOAD=max(MAX_LOAD,                              &
     &                   (TEMP(IDWLOAD(i))+(dble(NELIM)*                &
     &                   dble(NB_ROWS(IDWLOAD(i))))+                    &
     &                   (dble(NB_ROWS(IDWLOAD(i)))*dble(NELIM))*       &
     &                   dble(2*NFRONT-NELIM-1)))
                    IF(SMALL_SET.LT.NUMBER_OF_PROCS)THEN
                       IF(MAX_LOAD.GT.TEMP(SMALL_SET+1))THEN
                          IF(SMALL_SET.LT.NSLAVES_REF)THEN
                             SMALL_SET=REF+1
                             REF=REF+1
                             NB_ROWS=0
                             GOTO 323
                          ENDIF
                       ENDIF
                    ENDIF
                    IF(SMALL_SET.EQ.NB_SAT)GOTO 777
                    IF(ADDITIONNAL_ROWS.EQ.0)THEN
                       CHOSEN=SMALL_SET
                       GOTO 049
                    ENDIF
                 ENDDO
  777            CONTINUE
                 IF((NB_ZERO.NE.0).AND.(ADDITIONNAL_ROWS.GE.KMIN))THEN
                    J=NB_ZERO
  732               CONTINUE
                    X=int(ADDITIONNAL_ROWS/(J))
                    IF(X.LT.KMIN)THEN
                       J=J-1
                       GOTO 732
                    ENDIF
                    IF(X*J.LT.ADDITIONNAL_ROWS)THEN
                       X=X+1
                    ENDIF
                    DO i=1,SMALL_SET
                       AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                       BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       IF(HAVE_TYPE1_SON)THEN
                          AFFECTED=int((BUF_SIZE-dble(NFRONT))/         &
     &                         dble(NFRONT+1))
                          BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       ENDIF
                       MAX_MEM_ALLOW=BANDE_K821
                       IF(BDC_MD)THEN
                          MAX_MEM_ALLOW=min(                            &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                         dble(BANDE_K821))
                          MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                       ENDIF
                       KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                       IF(NB_ROWS(i).EQ.0)THEN
                          IF(X.GT.ADDITIONNAL_ROWS)THEN
                             X=ADDITIONNAL_ROWS
                          ENDIF
                          IF(X.GT.KMAX)THEN
                             X=KMAX
                          ENDIF
                          IF(X.GT.KMIN)THEN
                             NB_ROWS(i)=X
                             ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                             MAX_LOAD=max(MAX_LOAD,                     &
     &                            (TEMP(i)+(dble(NELIM) *               &
     &                            dble(NB_ROWS(i)))+                    &
     &                            (dble(NB_ROWS(i))*dble(NELIM))*       &
     &                            dble(2*NFRONT-NELIM-1)))
                          ENDIF
                       ENDIF
                    ENDDO
                 ENDIF
                 i=CHOSEN+1
                 DO WHILE ((ADDITIONNAL_ROWS.NE.0)                      &
     &                .AND.(i.LE.NUMBER_OF_PROCS))
                    IF((TEMP(i).LE.MAX_LOAD))THEN
                       AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                       BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       IF(HAVE_TYPE1_SON)THEN
                          AFFECTED=int((BUF_SIZE-dble(NFRONT))/         &
     &                         dble(NFRONT+1))
                          BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       ENDIF
                       MAX_MEM_ALLOW=BANDE_K821
                       IF(BDC_MD)THEN
                          MAX_MEM_ALLOW=min(                            &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                         BANDE_K821)
                          MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                       ENDIF
                       KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                       AFFECTED=int((MAX_LOAD-TEMP(i))/                 &
     &                      (dble(NELIM)*dble(2*NFRONT-NELIM)))
                       IF(AFFECTED.GT.ADDITIONNAL_ROWS)THEN
                          AFFECTED=ADDITIONNAL_ROWS
                       ENDIF
                       IF(NB_ROWS(i).LT.KMAX)THEN
                          IF((AFFECTED+NB_ROWS(i)).GT.KMAX)THEN
                             AFFECTED=KMAX-NB_ROWS(i)
                             NB_SAT=NB_SAT+1
                          ELSE
                             IF((AFFECTED+NB_ROWS(i)).LT.               &
     &                            KMIN)THEN
                                AFFECTED=0
                             ENDIF
                          ENDIF
                          NB_ROWS(i)=NB_ROWS(i)+AFFECTED
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-AFFECTED
                       ENDIF
                    ELSE IF((TEMP(i).GT.MAX_LOAD))THEN
                       IF(NB_SAT.EQ.i-1) GOTO 218
                       X=(ADDITIONNAL_ROWS/(i-1-NB_SAT))
                       ACC=1
                       DO J=1,i-1
                          TMP_SUM=((dble(NELIM) * dble(NB_ROWS(J)+X))   &
     &                         +(dble(NB_ROWS(J)+X)*dble(NELIM))*       &
     &                         dble(2*NFRONT-NELIM-1))
                          IF((TEMP(J)+TMP_SUM).GT.MAX_LOAD)THEN
                             ACC=0
                          ENDIF
                       ENDDO
                       IF(ACC.EQ.1)THEN
                          MAX_LOAD=TEMP(i)
                          J=1
                          DO WHILE ((ADDITIONNAL_ROWS.NE.0)             &
     &                         .AND.(J.LT.i))
                             AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                             BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                             IF(HAVE_TYPE1_SON)THEN
                                AFFECTED=int((BUF_SIZE-dble(NFRONT))/   &
     &                               dble(NFRONT+1))
                                BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                             ENDIF
                             AFFECTED=X
                             MAX_MEM_ALLOW=BANDE_K821
                             IF(BDC_MD)THEN
                                MAX_MEM_ALLOW=min(                      &
     &                    min(MEM_SIZE_WEAK(J),MEM_SIZE_STRONG(J)),     &
     &                               BANDE_K821)
                                MAX_MEM_ALLOW=max(dble(0),              &
     &                               MAX_MEM_ALLOW)
                             ENDIF
                             KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                             IF(AFFECTED.GT.ADDITIONNAL_ROWS)THEN
                                AFFECTED=ADDITIONNAL_ROWS
                             ENDIF
                             IF(NB_ROWS(J).LT.KMAX)THEN
                                IF((AFFECTED+NB_ROWS(J)).GT.KMAX)THEN
                                   AFFECTED=KMAX-NB_ROWS(J)
                                   NB_SAT=NB_SAT+1
                                ELSE
                                   IF((AFFECTED+NB_ROWS(J)).LT.         &
     &                                  KMIN)THEN
                                      AFFECTED=0
                                   ENDIF
                                ENDIF
                                NB_ROWS(J)=NB_ROWS(J)+AFFECTED
                                ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-      &
     &                               AFFECTED
                             ENDIF
                             J=J+1
                          ENDDO
                       ELSE
                          MAX_LOAD=TEMP(i)
                          J=1
                          DO WHILE ((ADDITIONNAL_ROWS.NE.0)             &
     &                         .AND.(J.LT.i))
                             AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                             BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                             IF(HAVE_TYPE1_SON)THEN
                                AFFECTED=int((BUF_SIZE-dble(NFRONT))/   &
     &                               dble(NFRONT+1))
                                BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                             ENDIF
                             TMP_SUM=((dble(NELIM)* dble(NB_ROWS(J)))   &
     &                            +(dble(NB_ROWS(J))*dble(NELIM))*      &
     &                            dble(2*NFRONT-NELIM-1))
                             X=int((MAX_LOAD-(TEMP(J)+TMP_SUM))/        &
     &                            (dble(NELIM)*dble(2*NFRONT-NELIM)))
                             IF(X.LT.0)THEN
                                WRITE(*,*)MYID,                         &
     &    ': Internal error 10 in DMUMPS_518'
                                CALL AGMG_MUMPS_ABORT()
                             ENDIF
                             AFFECTED=X
                             MAX_MEM_ALLOW=BANDE_K821
                             IF(BDC_MD)THEN
                                MAX_MEM_ALLOW=min(                      &
     &                    min(MEM_SIZE_WEAK(J),MEM_SIZE_STRONG(J)),     &
     &                               BANDE_K821)
                                MAX_MEM_ALLOW=max(dble(0),              &
     &                               MAX_MEM_ALLOW)
                             ENDIF
                             KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                             IF(AFFECTED.GT.ADDITIONNAL_ROWS)THEN
                                AFFECTED=ADDITIONNAL_ROWS
                             ENDIF
                             IF(NB_ROWS(J).LT.KMAX)THEN
                                IF((AFFECTED+NB_ROWS(J)).GT.KMAX)THEN
                                   AFFECTED=KMAX-NB_ROWS(J)
                                   NB_SAT=NB_SAT+1
                                ELSE
                                   IF((AFFECTED+NB_ROWS(J)).LT.         &
     &                                  KMIN)THEN
                                      AFFECTED=0
                                   ENDIF
                                ENDIF
                                NB_ROWS(J)=NB_ROWS(J)+AFFECTED
                                ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-      &
     &                               AFFECTED
                             ENDIF
                             J=J+1
                          ENDDO
                       ENDIF
                    ENDIF
  218               CONTINUE
                    i=i+1
                 ENDDO
                 CHOSEN=i-1
                 IF((CHOSEN.EQ.NUMBER_OF_PROCS-1).AND.                  &
     &                 (ADDITIONNAL_ROWS.NE.0))THEN
                    DO i=1,CHOSEN
                       IF(NB_ROWS(i)+1.GE.KMIN)THEN
                          NB_ROWS(i)=NB_ROWS(i)+1
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-1
                       ENDIF
                       MAX_LOAD=max(MAX_LOAD,                           &
     &                      (TEMP(i)+(dble(NELIM) *                     &
     &                      dble(NB_ROWS(i)))+                          &
     &                      (dble(NB_ROWS(i))*dble(NELIM))*             &
     &                      dble(2*NFRONT-NELIM-1)))
                       IF(ADDITIONNAL_ROWS.EQ.0) GOTO 048
                    ENDDO
   48               CONTINUE
                 ENDIF
                 IF((ADDITIONNAL_ROWS.NE.0))THEN
                    IF(CHOSEN.LT.NUMBER_OF_PROCS)THEN
                       i=CHOSEN+1
                    ELSE
                       IF(CHOSEN.NE.NUMBER_OF_PROCS)THEN
                          WRITE(*,*)MYID,                               &
     &    ': Internal error 11 in DMUMPS_518'
                          CALL AGMG_MUMPS_ABORT()
                       ENDIF
                       i=CHOSEN
                    ENDIF
                    DO WHILE ((ADDITIONNAL_ROWS.NE.0)                   &
     &                   .AND.(i.LE.NUMBER_OF_PROCS))
                       IF(TEMP(i).LE.MAX_LOAD)THEN
                          AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                          BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                          IF(HAVE_TYPE1_SON)THEN
                             AFFECTED=int((BUF_SIZE-dble(NFRONT))/      &
     &                            dble(NFRONT+1))
                             BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                          ENDIF
                          MAX_MEM_ALLOW=BANDE_K821
                          IF(BDC_MD)THEN
                             MAX_MEM_ALLOW=min(                         &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                            BANDE_K821)
                             MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                          ENDIF
                          KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                          TMP_SUM=((dble(NELIM) * dble(NB_ROWS(i)))     &
     &                         +(dble(NB_ROWS(i))*dble(NELIM))*         &
     &                         dble(2*NFRONT-NELIM-1))
                          X=int((MAX_LOAD-(TEMP(i)+TMP_SUM))/           &
     &                         (dble(NELIM)*dble(2*NFRONT-NELIM)))
                          AFFECTED=X
                          IF(X.LT.0)THEN
                             WRITE(*,*)MYID,                            &
     &    ': Internal error 12 in DMUMPS_518'
                             CALL AGMG_MUMPS_ABORT()
                          ENDIF
                          IF(AFFECTED.GT.ADDITIONNAL_ROWS)THEN
                             AFFECTED=ADDITIONNAL_ROWS
                          ENDIF
                          IF(NB_ROWS(i).LT.KMAX)THEN
                             IF((AFFECTED+NB_ROWS(i)).GT.KMAX)THEN
                                AFFECTED=KMAX-NB_ROWS(i)
                             ELSE
                                IF((AFFECTED+NB_ROWS(i)).LT.            &
     &                               KMIN)THEN
                                   AFFECTED=0
                                ENDIF
                             ENDIF
                             NB_ROWS(i)=NB_ROWS(i)+AFFECTED
                             ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-AFFECTED
                          ENDIF
                          IF(i.NE.NUMBER_OF_PROCS) GOTO 624
                       ELSE IF((TEMP(i).GT.MAX_LOAD))THEN
                          X=int(ADDITIONNAL_ROWS/i-1)
                          X=max(X,1)
                          IF((MAX_LOAD+((dble(NELIM)*                   &
     &                         dble(X))+(dble(                          &
     &                         X)*dble(NELIM))*dble(                    &
     &                         (2*NFRONT-NELIM-1)))).LE.TEMP(i))THEN
                             AFFECTED=X
                             POS=1
                          ELSE
                             POS=0
                          ENDIF
                          MAX_LOAD=TEMP(i)
                          J=1
                          DO WHILE ((ADDITIONNAL_ROWS.NE.0)             &
     &                         .AND.(J.LT.i))
                             X=int(BUF_SIZE/dble(NCB+1))-1
                             BANDE_K821=dble(X)*dble(NFRONT)
                             MAX_MEM_ALLOW=BANDE_K821
                             IF(HAVE_TYPE1_SON)THEN
                                X=int((BUF_SIZE-dble(NFRONT))/          &
     &                               dble(NFRONT+1))
                                BANDE_K821=dble(X)*dble(NFRONT)
                             ENDIF
                             IF(BDC_MD)THEN
                                MAX_MEM_ALLOW=min(                      &
     &                    min(MEM_SIZE_WEAK(J),MEM_SIZE_STRONG(J)),     &
     &                               BANDE_K821)
                                MAX_MEM_ALLOW=max(dble(0),              &
     &                               MAX_MEM_ALLOW)
                             ENDIF
                             KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                             IF(POS.EQ.0)THEN
                                TMP_SUM=((dble(NELIM) *                 &
     &                               dble(NB_ROWS(J)))                  &
     &                               +(dble(NB_ROWS(J))*dble(NELIM))*   &
     &                               dble(2*NFRONT-NELIM-1))
                                X=int((TEMP(i)-(TEMP(J)+TMP_SUM))/      &
     &                               (dble(NELIM)*dble(2*NFRONT-        &
     &                               NELIM)))
                             ELSE
                                X=int(TMP_SUM)
                             ENDIF
                             IF(X.GT.ADDITIONNAL_ROWS)THEN
                                X=ADDITIONNAL_ROWS
                             ENDIF
                             IF(NB_ROWS(J).LT.KMAX)THEN
                                IF((X+NB_ROWS(J)).GT.KMAX)THEN
                                   X=KMAX-NB_ROWS(J)
                                ELSE
                                   IF((NB_ROWS(J)+X).LT.                &
     &                                  KMIN)THEN
                                     X=0
                                  ENDIF
                               ENDIF
                               NB_ROWS(J)=NB_ROWS(J)+X
                               ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                            ENDIF
                            J=J+1
                         ENDDO
                       ENDIF
  624                  CONTINUE
                       i=i+1
                    ENDDO
                    CHOSEN=i-1
                    IF(ADDITIONNAL_ROWS.NE.0)THEN
                       ACC=0
                       DO i=1,CHOSEN
                          X=int(BUF_SIZE/dble(NCB+1))-1
                          BANDE_K821=dble(X)*dble(NFRONT)
                          IF(HAVE_TYPE1_SON)THEN
                             X=int((BUF_SIZE-dble(NFRONT))/             &
     &                            dble(NFRONT+1))
                             BANDE_K821=dble(X)*dble(NFRONT)
                          ENDIF
                          MAX_MEM_ALLOW=BANDE_K821
                          IF(BDC_MD)THEN
                             MAX_MEM_ALLOW=min(                         &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                            BANDE_K821)
                             MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                          ENDIF
                             KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                          TMP_SUM=((dble(NELIM) * dble(NB_ROWS(i)))     &
     &                    +(dble(NB_ROWS(i))*dble(NELIM))*              &
     &                    dble(2*NFRONT-NELIM-1))
                          X=int((MAX_LOAD-                              &
     &                         (TEMP(i)+TMP_SUM))/                      &
     &                         (dble(NELIM)*dble(2*NFRONT-NELIM)))
                          IF(X.LT.0)THEN
                             WRITE(*,*)MYID,                            &
     &    ': Internal error 13 in DMUMPS_518'
                             CALL AGMG_MUMPS_ABORT()
                          ENDIF
                          IF(X.GT.ADDITIONNAL_ROWS)THEN
                             X=ADDITIONNAL_ROWS
                          ENDIF
                          IF(NB_ROWS(i).LT.KMAX)THEN
                             IF((X+NB_ROWS(i)).GE.KMAX)THEN
                                ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-      &
     &                               (KMAX-NB_ROWS(i))
                                NB_ROWS(i)=KMAX
                             ELSE
                                IF((X+NB_ROWS(i)).GE.                   &
     &                               KMIN)THEN
                                   NB_ROWS(i)=NB_ROWS(i)+X
                                   ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                                   ACC=ACC+1
                                ELSE
                                   ACC=ACC+1
                                ENDIF
                             ENDIF
                          ENDIF
                          IF(ADDITIONNAL_ROWS.EQ.0)GOTO 049
                       ENDDO
                       IF(CHOSEN.LT.NUMBER_OF_PROCS)THEN
                          CHOSEN=CHOSEN+1
                       ENDIF
                       IF(ACC.EQ.0)THEN
                          ACC=1
                       ENDIF
                       X=int(ADDITIONNAL_ROWS/ACC)
                       X=max(X,1)
                       ACC=0
                       DO i=1,CHOSEN
                          J=int(BUF_SIZE/dble(NCB+1))-1
                          BANDE_K821=dble(J)*dble(NFRONT)
                          IF(HAVE_TYPE1_SON)THEN
                             J=int((BUF_SIZE-dble(NFRONT))/             &
     &                            dble(NFRONT+1))
                             BANDE_K821=dble(J)*dble(NFRONT)
                          ENDIF
                          MAX_MEM_ALLOW=BANDE_K821
                          IF(BDC_MD)THEN
                             MAX_MEM_ALLOW=min(                         &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                            BANDE_K821)
                             MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                          ENDIF
                          KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                          TMP_SUM=((dble(NELIM) * dble(NB_ROWS(i)))     &
     &                         +(dble(NB_ROWS(i))*dble(NELIM))*         &
     &                         dble(2*NFRONT-NELIM-1))
                          J=int((MAX_LOAD-                              &
     &                         (TEMP(i)+TMP_SUM))/                      &
     &                         (dble(NELIM)*dble(2*NFRONT-NELIM)))
                          IF(NB_ROWS(i).LT.KMAX)THEN
                             IF((min(X,J)+NB_ROWS(i)).GE.KMAX)THEN
                                IF((KMAX-NB_ROWS(i)).GT.                &
     &                               ADDITIONNAL_ROWS)THEN
                                   NB_ROWS(i)=NB_ROWS(i)+               &
     &                                  ADDITIONNAL_ROWS
                                   ADDITIONNAL_ROWS=0
                                ELSE
                                   ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-   &
     &                                  (KMAX-NB_ROWS(i))
                                   NB_ROWS(i)=KMAX
                                ENDIF
                             ELSE
                                IF((min(X,J)+NB_ROWS(i)).GE.            &
     &                            KMIN)THEN
                                   NB_ROWS(i)=NB_ROWS(i)+min(X,J)
                                   ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-   &
     &                                  min(X,J)
                                   ACC=ACC+1
                                ENDIF
                             ENDIF
                          ENDIF
                          IF(ADDITIONNAL_ROWS.EQ.0)GOTO 049
                       ENDDO
                       IF(ACC.GT.0)THEN
                          DO i=1,CHOSEN
                             X=int(BUF_SIZE/dble(NCB+1))-1
                             BANDE_K821=dble(X)*dble(NFRONT)
                             IF(HAVE_TYPE1_SON)THEN
                                X=int((BUF_SIZE-dble(NFRONT))/          &
     &                               dble(NFRONT+1))
                                BANDE_K821=dble(X)*dble(NFRONT)
                             ENDIF
                             MAX_MEM_ALLOW=BANDE_K821
                             IF(BDC_MD)THEN
                                MAX_MEM_ALLOW=min(                      &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)),     &
     &                               BANDE_K821)
                                MAX_MEM_ALLOW=max(dble(0),              &
     &                               MAX_MEM_ALLOW)
                             ENDIF
                             KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                             IF(KMAX-NB_ROWS(i).LT.                     &
     &                            ADDITIONNAL_ROWS)THEN
                                ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-      &
     &                               (KMAX-NB_ROWS(i))
                                NB_ROWS(i)=KMAX
                             ELSE
                                IF(NB_ROWS(i).EQ.0)THEN
                                   IF(min(KMIN,KMAX).LT.                &
     &                                  ADDITIONNAL_ROWS)THEN
                                      NB_ROWS(i)=min(KMIN,KMAX)
                                      ADDITIONNAL_ROWS=                 &
     &                                     ADDITIONNAL_ROWS-            &
     &                                     min(KMIN,KMAX)
                                   ENDIF
                                ELSE
                                   NB_ROWS(i)=NB_ROWS(i)+               &
     &                                  ADDITIONNAL_ROWS
                                   ADDITIONNAL_ROWS=0
                                ENDIF
                             ENDIF
                             IF(ADDITIONNAL_ROWS.EQ.0)GOTO 049
                          ENDDO
                       ENDIF
                       DO i=1,CHOSEN
                          IDWLOAD(i)=i
                          AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                          BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                          IF(HAVE_TYPE1_SON)THEN
                             AFFECTED=int((BUF_SIZE-dble(NFRONT))/      &
     &                            dble(NFRONT+1))
                             BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                          ENDIF
                          WLOAD(i)=(BANDE_K821-dble(NB_ROWS(i)*NFRONT))
                       ENDDO
                       CALL AGMG_MUMPS_558(NUMBER_OF_PROCS, WLOAD,           &
     &                      IDWLOAD)
                       NB_SAT=0
                       DO i=1,CHOSEN
                          X=int(ADDITIONNAL_ROWS/(CHOSEN-NB_SAT))
                          X=max(X,1)
                          AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                          BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                          IF(HAVE_TYPE1_SON)THEN
                             AFFECTED=int((BUF_SIZE-dble(NFRONT))/      &
     &                            dble(NFRONT+1))
                             BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                          ENDIF
                          IF(BDC_MD)THEN
                             MAX_MEM_ALLOW=min(BANDE_K821,              &
     &                            MEM_SIZE_STRONG(i))
                             MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                          ENDIF
                          KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                          IF(NB_ROWS(IDWLOAD(i)).LT.KMAX)THEN
                             IF((NB_ROWS(IDWLOAD(i))+X).LT.KMAX)THEN
                                NB_ROWS(IDWLOAD(i))=                    &
     &                               NB_ROWS(IDWLOAD(i))+X
                                ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                             ELSE
                                ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-      &
     &                               (KMAX-NB_ROWS(IDWLOAD(i)))
                                NB_ROWS(IDWLOAD(i))=KMAX
                             ENDIF
                          ENDIF
                          IF(NB_ROWS(IDWLOAD(i)).EQ.KMAX)THEN
                             NB_SAT=NB_SAT+1
                          ENDIF
                          IF(ADDITIONNAL_ROWS.EQ.0) GOTO 049
                       ENDDO
                       DO i=1,CHOSEN
                          X=int(BUF_SIZE/dble(NCB+1))-1
                          BANDE_K821=dble(X)*dble(NFRONT)
                          IF(HAVE_TYPE1_SON)THEN
                             X=int((BUF_SIZE-dble(NFRONT))/             &
     &                            dble(NFRONT+1))
                             BANDE_K821=dble(X)*dble(NFRONT)
                          ENDIF
                          MAX_MEM_ALLOW=BANDE_K821
                          IF(BDC_MD)THEN
                             MAX_MEM_ALLOW=min(BANDE_K821,              &
     &                            MEM_SIZE_STRONG(i))
                             MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                          ENDIF
                          KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                          IF(KMAX-NB_ROWS(i).LT.ADDITIONNAL_ROWS)THEN
                             ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-         &
     &                            (KMAX-NB_ROWS(i))
                             NB_ROWS(i)=KMAX
                          ELSE
                             NB_ROWS(i)=NB_ROWS(i)+ADDITIONNAL_ROWS
                             ADDITIONNAL_ROWS=0
                          ENDIF
                          IF(ADDITIONNAL_ROWS.EQ.0)GOTO 049
                       ENDDO
                       X=int(ADDITIONNAL_ROWS/CHOSEN)
                       X=max(X,1)
                       DO i=1,CHOSEN
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                          NB_ROWS(i)=NB_ROWS(i)+X
                          IF(ADDITIONNAL_ROWS.EQ.0)GOTO 049
                       ENDDO
                       NB_ROWS(1)=NB_ROWS(1)+ADDITIONNAL_ROWS
                    ENDIF
                 ENDIF
   49            CONTINUE
              ENDIF
  666         CONTINUE
              SOMME=dble(0)
              X=0
              POS=0
              DO i=1,CHOSEN
                 X=X+NB_ROWS(i)
                 SOMME=SOMME+ dble(NB_ROWS(i))
              ENDDO
              GOTO 890
           ELSE IF((KEEP(83).GE.NUMBER_OF_PROCS).AND.FORCE_CAND)THEN
              MAX_LOAD=dble(0)
              DO i=1,OTHERS
                 MAX_LOAD=max(MAX_LOAD,TEMP(i))
              ENDDO
              ACC=0
              CHOSEN=0
              X=1
              DO i=1,OTHERS
              ENDDO
              DO i=2,OTHERS
                 IF(TEMP(i).EQ.TEMP(1))THEN
                    X=X+1
                 ELSE
                    GOTO 329
                 ENDIF
              ENDDO
  329         CONTINUE
              TMP_SUM=TOTAL_COST/dble(X)
              TEMP_MAX_LOAD=dble(0)
              DO i=1,OTHERS
                 IF(K50.EQ.0)THEN
                    X=int(BUF_SIZE/dble(NCB+1))-1
                    BANDE_K821=dble(X)*dble(NFRONT)
                 ELSE
                    A=dble(1)
                    B=dble(ACC+2)
                    C=-BUF_SIZE+dble(ACC+NELIM)
                    DELTA=(B*B)-(dble(4)*A*C)
                    X=int((-B+sqrt(DELTA))/(dble(2)*A))
                    IF(X.GT.NCB-ACC) X=NCB-ACC
                    BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                 ENDIF
                 IF(HAVE_TYPE1_SON)THEN
                    IF(K50.EQ.0)THEN
                       X=int((BUF_SIZE-dble(NFRONT))/dble(NFRONT+1))
                       BANDE_K821=dble(X)*dble(NFRONT)
                    ELSE
                       A=dble(1)
                       B=dble(ACC+2+NELIM)
                       C=-BUF_SIZE+dble(ACC+NELIM)
                       DELTA=(B*B)-(dble(4)*A*C)
                       X=int((-B+sqrt(DELTA))/(dble(2)*A))
                       IF(X.GT.NCB-ACC) X=NCB-ACC
                       BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                    ENDIF
                 ENDIF
                 MAX_MEM_ALLOW=BANDE_K821
                 IF(BDC_MD)THEN
                    MAX_MEM_ALLOW=min(BANDE_K821,                       &
     &                    min(MEM_SIZE_WEAK(i),MEM_SIZE_STRONG(i)))
                    MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                 ENDIF
                 IF(K50.EQ.0)THEN
                       KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                    IF(TMP_SUM+TEMP(i).GT.MAX_LOAD)THEN
                       SOMME=MAX_LOAD-TEMP(i)
                    ELSE
                       SOMME=TMP_SUM
                    ENDIF
                    X=int(SOMME/                                        &
     &                   (dble(NELIM)*dble(2*NFRONT-NELIM)))
                    IF(X.GT.KMAX)THEN
                       X=KMAX
                    ELSE
                       IF(X.LT.KMIN)THEN
                          X=min(KMIN,KMAX)
                       ENDIF
                    ENDIF
                    IF((ACC+X).GT.NCB) X=NCB-ACC
                 ENDIF
                 IF(K50.NE.0)THEN
                       A=dble(1)
                       B=dble(ACC+NELIM)
                       C=dble(-MAX_MEM_ALLOW)
                       DELTA=((B*B)-(dble(4)*A*C))
                       KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                    A=dble(NELIM)
                    B=dble(NELIM)*dble(NELIM+2*ACC+1)
                    IF(TMP_SUM+TEMP(i).GT.MAX_LOAD)THEN
                       C=-(MAX_LOAD-TEMP(i))
                    ELSE
                       C=-TMP_SUM
                    ENDIF
                    DELTA=(B*B-(dble(4)*A*C))
                    X=int((-B+sqrt(DELTA))/(dble(2)*A))
                    IF(X.LT.0) THEN
                       WRITE(*,*)MYID,                                  &
     &    ': Internal error 14 in DMUMPS_518'
                       CALL AGMG_MUMPS_ABORT()
                    ENDIF
                    IF(X.GE.KMAX)THEN
                       IF(KMAX.GT.KMIN)THEN
                          X=KMAX
                       ELSE
                          X=0
                       ENDIF
                    ELSE
                       IF(X.LE.min(KMIN,KMAX))THEN
                          IF(KMAX.LT.KMIN)THEN
                             X=0
                          ELSE
                             X=min(KMIN,KMAX)
                          ENDIF
                       ENDIF
                    ENDIF
                    IF((ACC+X).GT.NCB) X=NCB-ACC
                 ENDIF
                 TEMP_MAX_LOAD=max(TEMP_MAX_LOAD,TEMP(i))
                 NB_ROWS(i)=X
                 CHOSEN=CHOSEN+1
                 ACC=ACC+X
                 IF(ACC.EQ.NCB) GOTO 541
              ENDDO
  541         CONTINUE
              IF(ACC.LT.NCB)THEN
                 IF(K50.EQ.0)THEN
                    ADDITIONNAL_ROWS=NCB-ACC
                    DO J=1,CHOSEN
                       AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                       BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       IF(HAVE_TYPE1_SON)THEN
                          AFFECTED=int((BUF_SIZE-dble(NFRONT))/         &
     &                         dble(NFRONT+1))
                          BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       ENDIF
                       MAX_MEM_ALLOW=BANDE_K821
                       IF(BDC_MD)THEN
                          MAX_MEM_ALLOW=min(                            &
     &                    min(MEM_SIZE_WEAK(J),MEM_SIZE_STRONG(J)),     &
     &                         dble(BANDE_K821))
                          MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                       ENDIF
                       KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                       IF((NB_ROWS(J)).LT.KMAX)THEN
                          IF(ADDITIONNAL_ROWS.GT.(KMAX-NB_ROWS(J)))THEN
                             ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-         &
     &                            (KMAX-NB_ROWS(J))
                             NB_ROWS(J)=KMAX
                          ELSE
                             NB_ROWS(J)=NB_ROWS(J)+ADDITIONNAL_ROWS
                             ADDITIONNAL_ROWS=0
                          ENDIF
                       ENDIF
                       IF(ADDITIONNAL_ROWS.EQ.0)GOTO 889
                    ENDDO
                    X=int(ADDITIONNAL_ROWS/CHOSEN)
                    X=max(X,1)
                    DO J=1,CHOSEN
                       AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                       BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       IF(HAVE_TYPE1_SON)THEN
                          AFFECTED=int((BUF_SIZE-dble(NFRONT))/         &
     &                         dble(NFRONT+1))
                          BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       ENDIF
                       MAX_MEM_ALLOW=BANDE_K821
                       IF(BDC_MD)THEN
                          MAX_MEM_ALLOW=min(BANDE_K821,                 &
     &                         MEM_SIZE_STRONG(J))
                          MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                       ENDIF
                       KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                       IF((NB_ROWS(J)+X).GT.KMAX)THEN
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-            &
     &                         (KMAX-NB_ROWS(J))
                          NB_ROWS(J)=KMAX
                       ELSE
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                          NB_ROWS(J)=NB_ROWS(J)+X
                       ENDIF
                       IF(ADDITIONNAL_ROWS.EQ.0)GOTO 889
                    ENDDO
                    DO i=1,CHOSEN
                       X=int(BUF_SIZE/dble(NCB+1))-1
                       BANDE_K821=dble(X)*dble(NFRONT)
                       IF(HAVE_TYPE1_SON)THEN
                          X=int((BUF_SIZE-dble(NFRONT))/                &
     &                         dble(NFRONT+1))
                          BANDE_K821=dble(X)*dble(NFRONT)
                       ENDIF
                       MAX_MEM_ALLOW=BANDE_K821
                       IF(BDC_MD)THEN
                          MAX_MEM_ALLOW=min(BANDE_K821,                 &
     &                         MEM_SIZE_STRONG(i))
                          MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                       ENDIF
                       KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                       IF(KMAX-NB_ROWS(i).LT.ADDITIONNAL_ROWS)THEN
                          ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-            &
     &                         (KMAX-NB_ROWS(i))
                          NB_ROWS(i)=KMAX
                       ELSE
                          NB_ROWS(i)=NB_ROWS(i)+ADDITIONNAL_ROWS
                          ADDITIONNAL_ROWS=0
                       ENDIF
                       IF(ADDITIONNAL_ROWS.EQ.0)GOTO 889
                    ENDDO
                    DO i=1,NUMBER_OF_PROCS
                       IDWLOAD(i)=i
                       AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                       BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       IF(HAVE_TYPE1_SON)THEN
                          AFFECTED=int((BUF_SIZE-dble(NFRONT))/         &
     &                         dble(NFRONT+1))
                          BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       ENDIF
                       WLOAD(i)=(BANDE_K821-(dble(NB_ROWS(i))*          &
     &                      dble(NFRONT)))
                    ENDDO
                    CALL AGMG_MUMPS_558(NUMBER_OF_PROCS, WLOAD,              &
     &                   IDWLOAD)
                    NB_SAT=0
                    DO i=1,CHOSEN
                       X=int(ADDITIONNAL_ROWS/(CHOSEN-NB_SAT))
                       X=max(X,1)
                       AFFECTED=int(BUF_SIZE/dble(NCB+1))-1
                       BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       IF(HAVE_TYPE1_SON)THEN
                          AFFECTED=int((BUF_SIZE-dble(NFRONT))/         &
     &                         dble(NFRONT+1))
                          BANDE_K821=dble(AFFECTED)*dble(NFRONT)
                       ENDIF
                       MAX_MEM_ALLOW=BANDE_K821
                       KMAX=int(MAX_MEM_ALLOW/dble(NFRONT))
                       IF(NB_ROWS(IDWLOAD(i)).LT.KMAX)THEN
                          IF((NB_ROWS(IDWLOAD(i))+X).LT.KMAX)THEN
                             NB_ROWS(IDWLOAD(i))=                       &
     &                            NB_ROWS(IDWLOAD(i))+X
                             ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                          ELSE
                             ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-         &
     &                            (KMAX-NB_ROWS(IDWLOAD(i)))
                             NB_ROWS(IDWLOAD(i))=KMAX
                          ENDIF
                       ENDIF
                       IF(NB_ROWS(IDWLOAD(i)).EQ.KMAX)THEN
                          NB_SAT=NB_SAT+1
                       ENDIF
                       IF(ADDITIONNAL_ROWS.EQ.0) GOTO 889
                    ENDDO
                    GOTO 994
                 ELSE
                    ACC=0
                    CHOSEN=0
                    DO i=1,OTHERS
                       A=dble(1)
                       B=dble(ACC+2)
                       C=-BUF_SIZE+dble(ACC+NELIM)
                       DELTA=(B*B)-(dble(4)*A*C)
                       X=int((-B+sqrt(DELTA))/(dble(2)*A))
                       IF(X.GT.NCB-ACC) X=NCB-ACC
                       BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                       IF(HAVE_TYPE1_SON)THEN
                          A=dble(1)
                          B=dble(ACC+2+NELIM)
                          C=-BUF_SIZE+dble(ACC+NELIM)
                          DELTA=(B*B)-(dble(4)*A*C)
                          X=int((-B+sqrt(DELTA))/(dble(2)*A))
                          IF(X.GT.NCB-ACC) X=NCB-ACC
                          BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                       ENDIF
                       MAX_MEM_ALLOW=BANDE_K821
                       IF(BDC_MD)THEN
                          MAX_MEM_ALLOW=min(BANDE_K821,                 &
     &                         MEM_SIZE_STRONG(i))
                          MAX_MEM_ALLOW=max(dble(0),MAX_MEM_ALLOW)
                       ENDIF
                       A=dble(1)
                       B=dble(ACC+NELIM)
                       C=dble(-MAX_MEM_ALLOW)
                       DELTA=((B*B)-(dble(4)*A*C))
                       KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                       X=KMAX-NB_ROWS(i)
                       IF((ACC+NB_ROWS(i)+X).GT.NCB)                    &
     &                            X=NCB-(ACC+NB_ROWS(i))
                       NB_ROWS(i)=NB_ROWS(i)+X
                       ACC=ACC+NB_ROWS(i)
                       CHOSEN=CHOSEN+1
                       IF(NCB.EQ.ACC) GOTO 889
                    ENDDO
                    ADDITIONNAL_ROWS=NCB-ACC
                 ENDIF
  993            CONTINUE
                 ACC=0
                 CHOSEN=0
                 DO i=1,OTHERS
                    A=dble(1)
                    B=dble(ACC+2)
                    C=-BUF_SIZE+dble(ACC+NELIM)
                    DELTA=(B*B)-(dble(4)*A*C)
                    X=int((-B+sqrt(DELTA))/(dble(2)*A))
                    IF(X.GT.NCB-ACC) X=NCB-ACC
                    BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                    IF(HAVE_TYPE1_SON)THEN
                       A=dble(1)
                       B=dble(ACC+2+NELIM)
                       C=-BUF_SIZE+dble(ACC+NELIM)
                       DELTA=(B*B)-(dble(4)*A*C)
                       X=int((-B+sqrt(DELTA))/(dble(2)*A))
                       IF(X.GT.NCB-ACC) X=NCB-ACC
                       BANDE_K821=dble(X)*dble(NELIM+ACC+X)
                    ENDIF
                    MAX_MEM_ALLOW=BANDE_K821
                    A=dble(1)
                    B=dble(ACC+NELIM)
                    C=dble(-MAX_MEM_ALLOW)
                    DELTA=((B*B)-(dble(4)*A*C))
                    KMAX=int((-B+sqrt(DELTA))/(dble(2)*A))
                    X=KMAX-NB_ROWS(i)
                    IF((ACC+NB_ROWS(i)+X).GT.NCB)                       &
     &                   X=NCB-(ACC+NB_ROWS(i))
                    NB_ROWS(i)=NB_ROWS(i)+X
                    ACC=ACC+NB_ROWS(i)
                    CHOSEN=CHOSEN+1
                    IF(NCB.EQ.ACC) GOTO 889
                 ENDDO
                 ADDITIONNAL_ROWS=NCB-ACC
  994            CONTINUE
                 X=int(dble(ADDITIONNAL_ROWS)/dble(OTHERS))
                 IF((X*OTHERS).LT.ADDITIONNAL_ROWS)THEN
                    X=X+1
                 ENDIF
                 DO i=1,OTHERS
                    NB_ROWS(i)=NB_ROWS(i)+X
                    ADDITIONNAL_ROWS=ADDITIONNAL_ROWS-X
                    IF(ADDITIONNAL_ROWS.LT.X)X=ADDITIONNAL_ROWS
                 ENDDO
                 CHOSEN=OTHERS
              ENDIF
           ENDIF
  889      CONTINUE
           MAX_LOAD=TEMP_MAX_LOAD
  890      CONTINUE
           J=CHOSEN
           X=0
              DO i=J,1,-1
                 IF(NB_ROWS(i).EQ.0)THEN
                    CHOSEN=CHOSEN-1
                    ELSE
                       IF(NB_ROWS(i).GT.0)THEN
                          X=1
                       ELSE
                          WRITE(*,*)MYID,                               &
     &    ': Internal error 15 in DMUMPS_518'
                          CALL AGMG_MUMPS_ABORT()
                       ENDIF
                    ENDIF
                 ENDDO
           NSLAVES_NODE=CHOSEN
           TAB_POS(NSLAVES_NODE+1)= NCB+1
           TAB_POS(SLAVEF+2) = CHOSEN
           POS=1
           X=1
           DO i=1,J
              IF(NB_ROWS(i).NE.0)THEN
                 SLAVES_LIST(X)=TEMP_ID(i)
                 TAB_POS(X)=POS
                 POS=POS+NB_ROWS(i)
                 IF(NB_ROWS(i).LE.0)THEN
                    WRITE(*,*)MYID,                                     &
     &    ': Internal error 16 in DMUMPS_518'
                    CALL AGMG_MUMPS_ABORT()
                 ENDIF
                 X=X+1
               ENDIF
           ENDDO
           DO i=CHOSEN+1,NUMBER_OF_PROCS
              SLAVES_LIST(i)=TEMP_ID(i)
           ENDDO
           IF(POS.NE.(NCB+1))THEN
              WRITE(*,*)MYID,                                           &
     &    ': Internal error 17 in DMUMPS_518',                          &
     &             POS,NCB+1
             CALL AGMG_MUMPS_ABORT()
           ENDIF
      END SUBROUTINE DAGMG_MUMPS_518
      SUBROUTINE DAGMG_MUMPS_520                                             &
     &      (INODE,UPPER,SLAVEF,KEEP,KEEP8,                             &
     &       STEP,POOL,LPOOL,PROCNODE,N)
      IMPLICIT NONE
      INTEGER INODE, LPOOL, SLAVEF, N
      INTEGER KEEP(500)
      INTEGER*8 KEEP8(150)
      INTEGER STEP(KEEP(28)), POOL(LPOOL), PROCNODE(KEEP(28))
      LOGICAL UPPER
      INTEGER J
      DOUBLE PRECISION MEM_COST
      INTEGER NBINSUBTREE,i,NBTOP
      EXTERNAL AGMG_MUMPS_167, AGMG_MUMPS_283, DAGMG_MUMPS_508,                        &
     & AGMG_MUMPS_170
      LOGICAL AGMG_MUMPS_167, AGMG_MUMPS_283, DAGMG_MUMPS_508,                         &
     & AGMG_MUMPS_170
      NBINSUBTREE = POOL(LPOOL)
      NBTOP       = POOL(LPOOL - 1)
      IF(KEEP(47).LT.2)THEN
         WRITE(*,*)'DMUMPS_520 must                                     &
     &        be called with K47>=2'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF((INODE.GT.0).AND.(INODE.LE.N))THEN
      MEM_COST=DAGMG_MUMPS_543(INODE)
         IF((DM_MEM(MYID)+dble(MEM_COST)+ PEAK_SBTR_CUR_LOCAL-          &
     &        SBTR_CUR_LOCAL)                                           &
     &        .GT.MAX_PEAK_STK)THEN
            DO i=NBTOP-1,1,-1
               INODE = POOL( LPOOL - 2 - i)
               MEM_COST=DAGMG_MUMPS_543(INODE)
               IF((INODE.LT.0).OR.(INODE.GT.N)) THEN
                  DO J=i+1,NBTOP,-1
                     POOL(J-1)=POOL(J)
                  ENDDO
                  UPPER=.TRUE.
                  RETURN
               ENDIF
               IF((DM_MEM(MYID)+dble(MEM_COST)+PEAK_SBTR_CUR_LOCAL-     &
     &              SBTR_CUR_LOCAL).LE.                                 &
     &              MAX_PEAK_STK) THEN
                  DO J=i+1,NBTOP,-1
                     POOL(J-1)=POOL(J)
                  ENDDO
                  UPPER=.TRUE.
                  RETURN
               ENDIF
            ENDDO
            IF(NBINSUBTREE.NE.0)THEN
               INODE = POOL( NBINSUBTREE )
               IF(.NOT.AGMG_MUMPS_170(STEP(INODE),                           &
     &              PROCNODE,SLAVEF))THEN
                  WRITE(*,*)                                            &
     &        'Internal error 1 in DMUMPS_520'
                  CALL AGMG_MUMPS_ABORT()
               ENDIF
               UPPER=.FALSE.
               RETURN
            ENDIF
            INODE=POOL(LPOOL-2-NBTOP)
            UPPER=.TRUE.
            RETURN
         ENDIF
      ENDIF
      UPPER=.TRUE.
      END SUBROUTINE DAGMG_MUMPS_520
      SUBROUTINE DAGMG_MUMPS_513(WHAT)
      IMPLICIT NONE
      LOGICAL WHAT
      IF(.NOT.BDC_POOL_MNG)THEN
         WRITE(*,*)'DMUMPS_513                                          &
     &        should be called when K81>0 and K47>2'
      ENDIF
      IF(WHAT)THEN
         PEAK_SBTR_CUR_LOCAL=PEAK_SBTR_CUR_LOCAL+                       &
     &        dble(MEM_SUBTREE(INDICE_SBTR))
         IF(.NOT.BDC_SBTR) INDICE_SBTR=INDICE_SBTR+1
      ELSE
         PEAK_SBTR_CUR_LOCAL=dble(0)
         SBTR_CUR_LOCAL=dble(0)
      ENDIF
      END SUBROUTINE DAGMG_MUMPS_513
      DOUBLE PRECISION FUNCTION DAGMG_MUMPS_543( INODE )
      IMPLICIT NONE
      INTEGER INODE,LEVEL,i,NELIM,NFR
      DOUBLE PRECISION COST
      EXTERNAL AGMG_MUMPS_330
      INTEGER AGMG_MUMPS_330
      i = INODE
      NELIM = 0
   10 CONTINUE
      IF ( i > 0 ) THEN
        NELIM = NELIM + 1
        i = FILS_LOAD(i)
        GOTO 10
      ENDIF
      NFR = ND_LOAD( STEP_LOAD(INODE) )
      LEVEL = AGMG_MUMPS_330( STEP_LOAD(INODE), PROCNODE_LOAD, NPROCS )
      IF (LEVEL .EQ. 1) THEN
        COST =  dble(NFR) * dble(NFR)
      ELSE
        IF ( K50 == 0 ) THEN
           COST =  dble(NFR) * dble(NELIM)
        ELSE
           COST = dble(NELIM) * dble(NELIM)
        ENDIF
      ENDIF
      DAGMG_MUMPS_543=COST
      RETURN
      END FUNCTION DAGMG_MUMPS_543
      RECURSIVE SUBROUTINE DAGMG_MUMPS_515(FLAG,COST,COMM)
      USE DAGMG_MUMPS_COMM_BUFF
      IMPLICIT NONE
      INTEGER COMM,WHAT,IERR
      LOGICAL FLAG
      DOUBLE PRECISION COST
      DOUBLE PRECISION TO_BE_SENT
      EXTERNAL AGMG_MUMPS_330
      INTEGER AGMG_MUMPS_330
      IF(FLAG)THEN
         WHAT=17
         IF(BDC_M2_FLOPS)THEN
            TO_BE_SENT=DELTA_LOAD-COST
            DELTA_LOAD=dble(0)
         ELSE IF(BDC_M2_MEM)THEN
            IF(BDC_POOL.AND.(.NOT.BDC_MD))THEN
               TO_BE_SENT=max(TMP_M2,POOL_LAST_COST_SENT)
               POOL_LAST_COST_SENT=TO_BE_SENT
            ELSE IF(BDC_MD)THEN
               DELTA_MEM=DELTA_MEM+TMP_M2
               TO_BE_SENT=DELTA_MEM
            ELSE
               TO_BE_SENT=dble(0)
            ENDIF
         ENDIF
      ELSE
         WHAT=6
         TO_BE_SENT=dble(0)
      ENDIF
  111 CONTINUE
      CALL DAGMG_MUMPS_460( WHAT,                                            &
     &         COMM, NPROCS,                                            &
     &               FUTURE_NIV2,                                       &
     &         COST,                                                    &
     &         TO_BE_SENT,                                              &
     &         MYID, IERR  )
      IF ( IERR == -1 )THEN
         CALL DAGMG_MUMPS_467(COMM_LD, KEEP_LOAD)
         GOTO 111
      ELSE IF ( IERR .NE. 0 ) THEN
         WRITE(*,*) "Internal Error in DAGMG_MUMPS_500",                     &
     &   IERR
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      RETURN
      END SUBROUTINE DAGMG_MUMPS_515
      SUBROUTINE DAGMG_MUMPS_512(INODE,STEP,NSTEPS,PROCNODE,FRERE,           &
     &     NE,COMM,SLAVEF,MYID,KEEP,KEEP8,N)
      USE DAGMG_MUMPS_COMM_BUFF
      IMPLICIT NONE
      INTEGER INODE,NSTEPS,MYID,SLAVEF,COMM,N
      INTEGER KEEP(500)
      INTEGER*8 KEEP8(150)
      INTEGER FRERE(NSTEPS),NE(NSTEPS),STEP(N),PROCNODE(NSTEPS)
      EXTERNAL AGMG_MUMPS_170,AGMG_MUMPS_275
      LOGICAL AGMG_MUMPS_170
      INTEGER i,NCB,NELIM
      INTEGER AGMG_MUMPS_275
      INTEGER FATHER_NODE,FATHER,WHAT,IERR
      EXTERNAL AGMG_MUMPS_330
      INTEGER AGMG_MUMPS_330
      DOUBLE PRECISION ID
      IF((.NOT.BDC_M2_MEM).AND.(.NOT.BDC_M2_FLOPS))THEN
         WRITE(*,*)MYID,': Problem in DMUMPS_512'
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF((INODE.LT.0).OR.(INODE.GT.N)) THEN
         RETURN
      ENDIF
      i=INODE
      NELIM = 0
   10 CONTINUE
      IF ( i > 0 ) THEN
         NELIM = NELIM + 1
         i = FILS_LOAD(i)
         GOTO 10
      ENDIF
      NCB=ND_LOAD(STEP_LOAD(INODE))-NELIM
      WHAT=5
      FATHER_NODE=DAD_LOAD(STEP_LOAD(INODE))
      IF (FATHER_NODE.EQ.0) THEN
         RETURN
      ENDIF
      IF((FRERE(STEP(FATHER_NODE)).EQ.0).AND.                           &
     &     ((FATHER_NODE.EQ.KEEP(38)).OR.                               &
     &     (FATHER_NODE.EQ.KEEP(20))))THEN
         RETURN
      ENDIF
      IF(AGMG_MUMPS_170(STEP(FATHER_NODE),                                   &
     &   PROCNODE,SLAVEF)) THEN
         RETURN
      ENDIF
      FATHER=AGMG_MUMPS_275(STEP(FATHER_NODE),PROCNODE,SLAVEF)
      IF(FATHER.EQ.MYID)THEN
        IF(BDC_M2_MEM)THEN
           CALL PROCESS_NIV2_MEM_MSG(FATHER_NODE)
        ELSEIF(BDC_M2_FLOPS)THEN
           CALL PROCESS_NIV2_FLOPS_MSG(FATHER_NODE)
        ENDIF
        IF((KEEP(81).EQ.2).OR.(KEEP(81).EQ.3))THEN
           IF(AGMG_MUMPS_330(STEP_LOAD(INODE),                               &
     &          PROCNODE_LOAD,NPROCS).EQ.1)THEN
              CB_COST_ID(POS_ID)=INODE
              CB_COST_ID(POS_ID+1)=1
              CB_COST_ID(POS_ID+2)=POS_MEM
              POS_ID=POS_ID+3
              CB_COST_MEM(POS_MEM)=int(MYID,8)
              POS_MEM=POS_MEM+1
              CB_COST_MEM(POS_MEM)=int(NCB,8)*int(NCB,8)
              POS_MEM=POS_MEM+1
           ENDIF
        ENDIF
        GOTO 666
      ENDIF
  111 CONTINUE
      CALL DAGMG_MUMPS_519(WHAT, COMM, NPROCS,                               &
     &     FATHER_NODE,INODE,NCB, KEEP(81),MYID,                        &
     &     FATHER, IERR)
      IF (IERR == -1 ) THEN
        CALL DAGMG_MUMPS_467(COMM, KEEP)
        GOTO 111
      ELSE IF ( IERR .NE. 0 ) THEN
        WRITE(*,*) "Internal Error in DAGMG_MUMPS_512",                      &
     &  IERR
        CALL AGMG_MUMPS_ABORT()
      ENDIF
  666 CONTINUE
      END SUBROUTINE DAGMG_MUMPS_512
      SUBROUTINE DAGMG_MUMPS_514(INODE,NUM_CALL)
      IMPLICIT NONE
      DOUBLE PRECISION MAXI
      INTEGER i,J,IND_MAXI
      INTEGER INODE,NUM_CALL
      DOUBLE PRECISION MAX_ALL_M2
      IF(BDC_M2_MEM)THEN
         IF(((NUM_CALL.EQ.1).AND.(BDC_MD)).OR.                          &
     &       ((NUM_CALL.EQ.2).AND.(.NOT.BDC_MD)))THEN
            RETURN
         ENDIF
      ENDIF
      IF((FRERE_LOAD(STEP_LOAD(INODE)).EQ.0).AND.                       &
     &     ((INODE.EQ.KEEP_LOAD(38)).OR.                                &
     &     (INODE.EQ.KEEP_LOAD(20)))) THEN
         RETURN
      ENDIF
      DO i=POOL_SIZE,1,-1
         IF(POOL_NIV2(i).EQ.INODE) GOTO 666
      ENDDO
         NB_SON(STEP_LOAD(INODE))=-1
      RETURN
  666 CONTINUE
      IF(BDC_M2_MEM)THEN
         IF(POOL_NIV2_COST(i).EQ.MAX_M2)THEN
            TMP_M2=MAX_M2
            MAXI=dble(0)
            IND_MAXI=-9999
            DO J=POOL_SIZE,1,-1
               IF(J.NE.i) THEN
                  IF(POOL_NIV2_COST(J).GT.MAXI)THEN
                     MAXI=POOL_NIV2_COST(J)
                     IND_MAXI=J
                  ENDIF
               ENDIF
            ENDDO
            MAX_M2=MAXI
            J=IND_MAXI
            REMOVE_NODE_FLAG_MEM=.TRUE.
            REMOVE_NODE_COST_MEM=TMP_M2
            CALL DAGMG_MUMPS_515(REMOVE_NODE_FLAG,MAX_M2,COMM_LD)
            NIV2(MYID+1)=MAX_M2
         ENDIF
      ELSEIF(BDC_M2_FLOPS)THEN
         REMOVE_NODE_COST=POOL_NIV2_COST(i)
         REMOVE_NODE_FLAG=.TRUE.
         CALL DAGMG_MUMPS_515(REMOVE_NODE_FLAG,                              &
     &        -POOL_NIV2_COST(i),COMM_LD)
         NIV2(MYID+1)=NIV2(MYID+1)-POOL_NIV2_COST(i)
      ENDIF
      DO J=i+1,POOL_SIZE
         POOL_NIV2(J-1)=POOL_NIV2(J)
         POOL_NIV2_COST(J-1)=POOL_NIV2_COST(J)
      ENDDO
      POOL_SIZE=POOL_SIZE-1
      END SUBROUTINE DAGMG_MUMPS_514
      RECURSIVE SUBROUTINE PROCESS_NIV2_MEM_MSG(INODE)
      IMPLICIT NONE
      INTEGER INODE
      EXTERNAL AGMG_MUMPS_330
      INTEGER AGMG_MUMPS_330
      INTEGER NBTOP, i
      IF((INODE.EQ.KEEP_LOAD(20)).OR.                                   &
     &     (INODE.EQ.KEEP_LOAD(38)))THEN
         RETURN
      ENDIF
      IF(NB_SON(STEP_LOAD(INODE)).EQ.-1)THEN
         RETURN
      ELSE
         IF(NB_SON(STEP_LOAD(INODE)).LT.0)THEN
            WRITE(*,*)                                                  &
     &        'Internal error 1 in PROCESS_NIV2_MEM_MSG'
            CALL AGMG_MUMPS_ABORT()
         ENDIF
      ENDIF
      NB_SON(STEP_LOAD(INODE))=                                         &
     &     NB_SON(STEP_LOAD(INODE))-1
      IF(NB_SON(STEP_LOAD(INODE)).EQ.0)THEN
         POOL_NIV2(POOL_SIZE+1)=INODE
         POOL_NIV2_COST(POOL_SIZE+1)=                                   &
     &        DAGMG_MUMPS_543(INODE)
         POOL_SIZE=POOL_SIZE+1
         IF(POOL_NIV2_COST(POOL_SIZE).GT.MAX_M2)THEN
            MAX_M2=POOL_NIV2_COST(POOL_SIZE)
            ID_MAX_M2=POOL_NIV2(POOL_SIZE)
            CALL DAGMG_MUMPS_515(REMOVE_NODE_FLAG_MEM,MAX_M2,COMM_LD)
            NIV2(1+MYID)=MAX_M2
         ENDIF
      ENDIF
      RETURN
      END SUBROUTINE PROCESS_NIV2_MEM_MSG
      RECURSIVE SUBROUTINE PROCESS_NIV2_FLOPS_MSG(INODE)
      IMPLICIT NONE
      INTEGER INODE
      EXTERNAL AGMG_MUMPS_330
      INTEGER AGMG_MUMPS_330
      IF((INODE.EQ.KEEP_LOAD(20)).OR.                                   &
     &     (INODE.EQ.KEEP_LOAD(38)))THEN
         RETURN
      ENDIF
      IF(NB_SON(STEP_LOAD(INODE)).EQ.-1)THEN
         RETURN
      ELSE
         IF(NB_SON(STEP_LOAD(INODE)).LT.0)THEN
            WRITE(*,*)                                                  &
     &        'Internal error 1 in PROCESS_NIV2_FLOPS_MSG'
            CALL AGMG_MUMPS_ABORT()
         ENDIF
      ENDIF
      NB_SON(STEP_LOAD(INODE))=                                         &
     &     NB_SON(STEP_LOAD(INODE))-1
      IF(NB_SON(STEP_LOAD(INODE)).EQ.0)THEN
         POOL_NIV2(POOL_SIZE+1)=INODE
         POOL_NIV2_COST(POOL_SIZE+1)=                                   &
     &        DAGMG_MUMPS_542(INODE)
         POOL_SIZE=POOL_SIZE+1
         MAX_M2=POOL_NIV2_COST(POOL_SIZE)
         ID_MAX_M2=POOL_NIV2(POOL_SIZE)
         CALL DAGMG_MUMPS_515(REMOVE_NODE_FLAG,                              &
     &           POOL_NIV2_COST(POOL_SIZE),                             &
     &        COMM_LD)
         NIV2(MYID+1)=POOL_NIV2_COST(POOL_SIZE)+NIV2(MYID+1)
      ENDIF
      RETURN
      END SUBROUTINE PROCESS_NIV2_FLOPS_MSG
      DOUBLE PRECISION FUNCTION DAGMG_MUMPS_542(INODE)
      INTEGER INODE
      INTEGER NFRONT,NELIM,i,LEVEL
      EXTERNAL AGMG_MUMPS_330
      INTEGER AGMG_MUMPS_330
      DOUBLE PRECISION COST
      i = INODE
      NELIM = 0
   10 CONTINUE
      IF ( i > 0 ) THEN
        NELIM = NELIM + 1
        i = FILS_LOAD(i)
        GOTO 10
      ENDIF
      NFRONT = ND_LOAD( STEP_LOAD(INODE) )
      LEVEL = AGMG_MUMPS_330( STEP_LOAD(INODE), PROCNODE_LOAD, NPROCS )
      COST=dble(0)
      CALL AGMG_MUMPS_511(NFRONT,NELIM,NELIM,                                &
     &                          KEEP_LOAD(50),LEVEL,COST)
      DAGMG_MUMPS_542=COST
      RETURN
      END FUNCTION DAGMG_MUMPS_542
      INTEGER FUNCTION DAGMG_MUMPS_541( INODE )
      IMPLICIT NONE
      INTEGER INODE,NELIM,NFR,SON,IN,i
      INTEGER COST_CB
      EXTERNAL AGMG_MUMPS_330
      INTEGER AGMG_MUMPS_330
      COST_CB=0
      i = INODE
   10 CONTINUE
      IF ( i > 0 ) THEN
        i = FILS_LOAD(i)
        GOTO 10
      ENDIF
      SON=-i
      DO i=1, NE_LOAD(STEP_LOAD(INODE))
         NFR = ND_LOAD( STEP_LOAD(SON) )
         IN=SON
         NELIM = 0
   20    CONTINUE
         IF ( IN > 0 ) THEN
            NELIM = NELIM + 1
            IN = FILS_LOAD(IN)
            GOTO 20
         ENDIF
         COST_CB=COST_CB+((NFR-NELIM)*(NFR-NELIM))
         SON=FRERE_LOAD(STEP_LOAD(SON))
      ENDDO
      DAGMG_MUMPS_541=COST_CB
      RETURN
      END FUNCTION DAGMG_MUMPS_541
      SUBROUTINE DAGMG_MUMPS_533(SLAVEF,NMB_OF_CAND,                         &
     &     TAB_POS, NASS, KEEP,KEEP8, LIST_SLAVES,                      &
     &     NSLAVES,INODE)
      USE DAGMG_MUMPS_COMM_BUFF
      IMPLICIT NONE
      INTEGER, INTENT (IN) :: SLAVEF, NASS, NSLAVES
      INTEGER, INTENT (IN) :: TAB_POS(SLAVEF+2)
      INTEGER, intent(in) :: NMB_OF_CAND
      INTEGER, INTENT (IN) :: LIST_SLAVES( NMB_OF_CAND )
      INTEGER KEEP(500),INODE,NELIM
      INTEGER*8 KEEP8(150)
      INTEGER allocok
      DOUBLE PRECISION MEM_COST,FCT_COST
      DOUBLE PRECISION, DIMENSION(:),ALLOCATABLE ::EMPTY_ARRAY,         &
     &     DELTA_MD,EMPTY_ARRAY2
      INTEGER, DIMENSION(:),ALLOCATABLE ::PROCS
      INTEGER NBROWS_SLAVE,i,WHAT,IERR,NFRONT,NPROCS_LOC
      LOGICAL FORCE_CAND
      MEM_COST=dble(0)
      FCT_COST=dble(0)
      IF ( KEEP(24) == 0 .OR. KEEP(24) == 1 ) THEN
        FORCE_CAND = .FALSE.
        NPROCS_LOC=SLAVEF-1
      ELSE
        FORCE_CAND = (mod(KEEP(24),2).eq.0)
        NPROCS_LOC=NMB_OF_CAND
      END IF
      IF(FORCE_CAND)THEN
         CALL DAGMG_MUMPS_540(INODE,FCT_COST,                                &
     &        MEM_COST,NPROCS_LOC,NASS)
      ELSE
         CALL DAGMG_MUMPS_540(INODE,FCT_COST,                                &
     &        MEM_COST,SLAVEF-1,NASS)
      ENDIF
      DO i=1,SLAVEF
         IDWLOAD(i)=i-1
      ENDDO
      ALLOCATE(EMPTY_ARRAY(NPROCS_LOC),DELTA_MD(NPROCS_LOC),            &
     &     EMPTY_ARRAY2(NPROCS_LOC),                                    &
     &    stat=allocok)
      DO i = 1, NSLAVES
         NBROWS_SLAVE = TAB_POS(i+1) - TAB_POS(i)
         DELTA_MD( i ) = FCT_COST - dble(NBROWS_SLAVE)*                 &
     &        dble(NASS)
      END DO
      IF(FORCE_CAND)THEN
         DO i=NSLAVES+1,NPROCS_LOC
            DELTA_MD( i ) = FCT_COST
         ENDDO
      ELSE
         DO i=NSLAVES+1,SLAVEF-1
            DELTA_MD( i ) = FCT_COST
         ENDDO
      ENDIF
      WHAT=7
  111 CONTINUE
      CALL DAGMG_MUMPS_524(.FALSE., COMM_LD, MYID, SLAVEF,                   &
     &     FUTURE_NIV2,                                                 &
     &     NPROCS_LOC, LIST_SLAVES,0,                                   &
     &     EMPTY_ARRAY,                                                 &
     &     DELTA_MD,EMPTY_ARRAY2,WHAT, IERR)
      IF ( IERR == -1 ) THEN
          CALL DAGMG_MUMPS_467(COMM_LD, KEEP)
          GOTO 111
      ELSE IF ( IERR .NE. 0 ) THEN
         WRITE(*,*) "Internal Error in DAGMG_MUMPS_533",                     &
     &   IERR
         CALL AGMG_MUMPS_ABORT()
      ENDIF
      IF (FUTURE_NIV2(MYID+1) .NE. 0) THEN
        DO i = 1, NSLAVES
           MD_MEM(LIST_SLAVES(i))=MD_MEM(LIST_SLAVES(i))+               &
     &          int(DELTA_MD( i ),8)
           IF(FUTURE_NIV2(LIST_SLAVES(i)+1).EQ.0)THEN
              MD_MEM(LIST_SLAVES(i))=999999999_8
           ENDIF
        ENDDO
      ENDIF
      DEALLOCATE(EMPTY_ARRAY)
      DEALLOCATE(DELTA_MD)
      END SUBROUTINE DAGMG_MUMPS_533
      SUBROUTINE DAGMG_MUMPS_540(INODE,FCT_COST,                             &
     &     MEM_COST,NSLAVES,NELIM)
      IMPLICIT NONE
      INTEGER INODE,NSLAVES,NFR,NELIM,IN
      DOUBLE PRECISION MEM_COST,FCT_COST
      NFR=ND_LOAD(STEP_LOAD(INODE))
      IN = INODE
      FCT_COST=dble(int(dble(NFR-NELIM)/dble(NSLAVES))+1)*              &
     &     dble(NELIM)
      MEM_COST=dble(int(dble(NFR-NELIM)/dble(NSLAVES))+1)*              &
     &     dble(NFR)
      END SUBROUTINE DAGMG_MUMPS_540
      SUBROUTINE CLEAN_POOL_MEM_INFO(INODE)
      IMPLICIT NONE
      INTEGER INODE
      INTEGER i,J,SON,NSLAVES_TEMP,POS_TEMP,K
      INTEGER AGMG_MUMPS_275
      EXTERNAL AGMG_MUMPS_275
      IF((INODE.LT.0).OR.(INODE.GT.N_LOAD))THEN
         RETURN
      ENDIF
      IF(POS_ID.GT.1)THEN
         i=INODE
   10    CONTINUE
         IF ( i > 0 ) THEN
            i = FILS_LOAD(i)
            GOTO 10
         ENDIF
         SON=-i
         IF(POS_ID.LT.NE_LOAD(STEP_LOAD(INODE))*3)THEN
            i=1
         ENDIF
         DO i=1, NE_LOAD(STEP_LOAD(INODE))
            J=1
            DO WHILE (J.LT.POS_ID)
               IF(CB_COST_ID(J).EQ.SON)GOTO 295
               J=J+3
            ENDDO
  295       CONTINUE
            IF(J.GE.POS_ID)THEN
               IF(AGMG_MUMPS_275(                                            &
     &            STEP_LOAD(INODE),PROCNODE_LOAD,NPROCS).EQ.MYID)THEN
                  IF(INODE.EQ.KEEP_LOAD(38))THEN
                     GOTO 666
                  ELSE
                     IF(FUTURE_NIV2(MYID+1).NE.0)THEN
                        WRITE(*,*)MYID,': i did not find ',SON
                        CALL AGMG_MUMPS_ABORT()
                     ENDIF
                     GOTO 666
                  ENDIF
               ELSE
                  GOTO 666
               ENDIF
            ENDIF
            NSLAVES_TEMP=CB_COST_ID(J+1)
            POS_TEMP=CB_COST_ID(J+2)
            DO K=J,POS_ID-1
               CB_COST_ID(K)=CB_COST_ID(K+3)
            ENDDO
            K=POS_TEMP
            DO WHILE (K.LE.POS_MEM-1)
               CB_COST_MEM(K)=CB_COST_MEM(K+2*NSLAVES_TEMP)
               K=K+1
            ENDDO
            POS_MEM=POS_MEM-2*NSLAVES_TEMP
            POS_ID=POS_ID-3
            IF((POS_MEM.LT.1).OR.(POS_ID.LT.1))THEN
               WRITE(*,*)MYID,': negative pos_mem or pos_id'
               CALL AGMG_MUMPS_ABORT()
            ENDIF
  666       CONTINUE
            SON=FRERE_LOAD(STEP_LOAD(SON))
         ENDDO
      ENDIF
      END SUBROUTINE CLEAN_POOL_MEM_INFO
      SUBROUTINE CHECK_MEM_CONST_FOR_POOL(FLAG)
      IMPLICIT NONE
      LOGICAL FLAG
      INTEGER i
      DOUBLE PRECISION MEM
      FLAG=.FALSE.
      DO i=0,NPROCS-1
         MEM=DM_MEM(i)+LU_USAGE(i)
         IF(BDC_SBTR)THEN
            MEM=MEM+SBTR_MEM(i)-SBTR_CUR(i)
         ENDIF
         IF((MEM/dble(TAB_MAXS(i))).GT.0.8d0)THEN
            FLAG=.TRUE.
            GOTO 666
         ENDIF
      ENDDO
  666 CONTINUE
      END SUBROUTINE CHECK_MEM_CONST_FOR_POOL
      SUBROUTINE DAGMG_MUMPS_554(NBINSUBTREE,INSUBTREE,NBTOP,                &
     &           MIN_COST,SBTR)
      IMPLICIT NONE
      INTEGER NBINSUBTREE,INSUBTREE,NBTOP
      DOUBLE PRECISION MIN_COST
      LOGICAL SBTR
      INTEGER i
      DOUBLE PRECISION TMP_COST,TMP_MIN
      TMP_MIN=huge(TMP_MIN)
      DO i=0,NPROCS-1
         IF(i.NE.MYID)THEN
            IF(BDC_SBTR)THEN
               TMP_MIN=min(TMP_MIN,dble(TAB_MAXS(i))-(DM_MEM(i)+        &
     &              LU_USAGE(i))-(SBTR_MEM(i)-SBTR_CUR(i)))
            ELSE
               TMP_MIN=min(TMP_MIN,dble(TAB_MAXS(i))-                   &
     &              (DM_MEM(i)+LU_USAGE(i)))
            ENDIF
         ENDIF
      ENDDO
      IF(NBINSUBTREE.GT.0)THEN
         IF(INSUBTREE.EQ.1)THEN
            TMP_COST=dble(TAB_MAXS(MYID))-(DM_MEM(MYID)+                &
     &           LU_USAGE(MYID))                                        &
     &           -(SBTR_MEM(MYID)-SBTR_CUR(MYID))
         ELSE
            SBTR=.FALSE.
            GOTO 777
         ENDIF
      ENDIF
      TMP_MIN=min(TMP_COST,TMP_MIN)
      IF(TMP_MIN.GT.MIN_COST) SBTR=.TRUE.
  777 CONTINUE
      END SUBROUTINE DAGMG_MUMPS_554
      SUBROUTINE COMPUTE_MAX_MEM(INODE,MAX_MEM,PROC)
      IMPLICIT NONE
      INTEGER INODE,PROC
      INTEGER i,POS,NSLAVES,SLAVE,NCAND,J,NELIM,NCB,NFRONT,SON,K
      INTEGER allocok
      EXTERNAL AGMG_MUMPS_330
      INTEGER  AGMG_MUMPS_330
      DOUBLE PRECISION MAX_MEM,TMP
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: MEM_ON_PROCS,      &
     &     RECV_BUF
      LOGICAL, DIMENSION(:), ALLOCATABLE :: CONCERNED
      DOUBLE PRECISION MAX_SENT_MSG
      IF((FRERE_LOAD(STEP_LOAD(INODE)).EQ.0)                            &
     &           .AND.(INODE.EQ.KEEP_LOAD(38)))THEN
         RETURN
      ENDIF
      ALLOCATE( MEM_ON_PROCS(0:NPROCS-1), stat=allocok)
      IF ( allocok > 0 ) THEN
        WRITE(*,*) 'PB allocation in COMPUTE_MAX_MEM'
        CALL AGMG_MUMPS_ABORT()
      ENDIF
      ALLOCATE( CONCERNED(0:NPROCS-1), stat=allocok)
      IF ( allocok > 0 ) THEN
        WRITE(*,*) 'PB allocation in COMPUTE_MAX_MEM'
        CALL AGMG_MUMPS_ABORT()
      ENDIF
      ALLOCATE( RECV_BUF(0:NPROCS-1), stat=allocok)
      IF ( allocok > 0 ) THEN
        WRITE(*,*) 'PB allocation in COMPUTE_MAX_MEM'
        CALL AGMG_MUMPS_ABORT()
      ENDIF
      RECV_BUF=dble(0)
      MAX_SENT_MSG=dble(0)
      i = INODE
      NELIM = 0
   10 CONTINUE
      IF ( i > 0 ) THEN
        NELIM = NELIM + 1
        i = FILS_LOAD(i)
        GOTO 10
      ENDIF
      SON=-i
      NFRONT=ND_LOAD(STEP_LOAD(INODE))
      NCB=NFRONT-NELIM
      IF(AGMG_MUMPS_330(STEP_LOAD(INODE)                                     &
     &     ,PROCNODE_LOAD,NPROCS).EQ.2)THEN
         NCAND=CAND_LOAD(NPROCS+1, STEP_TO_NIV2_LOAD(STEP_LOAD(INODE)))
      ENDIF
      DO i=0,NPROCS-1
         IF(i.EQ.MYID)THEN
            MEM_ON_PROCS(i)=dble(TAB_MAXS(i))-(DM_MEM(i)+               &
     &           LU_USAGE(i)+                                           &
     &           DAGMG_MUMPS_543(INODE))
            IF(BDC_SBTR)THEN
               MEM_ON_PROCS(i)=MEM_ON_PROCS(i)-(SBTR_MEM(i)-SBTR_CUR(i))
            ENDIF
            CONCERNED(i)=.TRUE.
         ELSE
            MEM_ON_PROCS(i)=dble(TAB_MAXS(i))-(DM_MEM(i)+LU_USAGE(i))
            IF(BDC_SBTR)THEN
               MEM_ON_PROCS(i)=MEM_ON_PROCS(i)-(SBTR_MEM(i)-SBTR_CUR(i))
            ENDIF
            IF(BDC_M2_MEM)THEN
               MEM_ON_PROCS(i)=MEM_ON_PROCS(i)-NIV2(i+1)
            ENDIF
         ENDIF
         IF(AGMG_MUMPS_330(STEP_LOAD(INODE)                                  &
     &        ,PROCNODE_LOAD,NPROCS).EQ.2)THEN
            IF(BDC_MD.AND.(KEEP_LOAD(48).EQ.5))THEN
               DO J=1,NCAND
                  IF(CAND_LOAD(J, STEP_TO_NIV2_LOAD(STEP_LOAD(INODE)))  &
     &                 .EQ.i)THEN
                     MEM_ON_PROCS(i)=MEM_ON_PROCS(i)-                   &
     &                 ((dble(NFRONT)*dble(NCB))/dble(NCAND))
                     CONCERNED(i)=.TRUE.
                     GOTO 666
                  ENDIF
               ENDDO
            ENDIF
         ENDIF
  666    CONTINUE
      ENDDO
      DO K=1, NE_LOAD(STEP_LOAD(INODE))
         i=1
         DO WHILE (i.LE.POS_ID)
            IF(CB_COST_ID(i).EQ.SON)GOTO 295
            i=i+3
         ENDDO
  295    CONTINUE
         IF(i.GE.POS_ID)THEN
            IF(FUTURE_NIV2(MYID+1).NE.0)THEN
               WRITE(*,*)MYID,': ',SON,'has not been found              &
     &              in COMPUTE_MAX_MEM'
               CALL AGMG_MUMPS_ABORT()
            ENDIF
            GOTO 777
         ENDIF
         NSLAVES=CB_COST_ID(i+1)
         POS=CB_COST_ID(i+2)
         DO i=1,NSLAVES
            SLAVE=int(CB_COST_MEM(POS))
            IF(.NOT.CONCERNED(SLAVE))THEN
               MEM_ON_PROCS(SLAVE)=MEM_ON_PROCS(SLAVE)+                 &
     &              dble(CB_COST_MEM(POS+1))
            ENDIF
            DO J=0,NPROCS-1
               IF(CONCERNED(J))THEN
                  IF(SLAVE.NE.J)THEN
                     RECV_BUF(J)=max(RECV_BUF(J),                       &
     &                    dble(CB_COST_MEM(POS+1)))
                  ENDIF
               ENDIF
            ENDDO
            POS=POS+2
         ENDDO
  777    CONTINUE
         SON=FRERE_LOAD(STEP_LOAD(SON))
      ENDDO
      MAX_MEM=huge(MAX_MEM)
      WRITE(*,*)'NPROCS=',NPROCS,MAX_MEM
      DO i=0,NPROCS-1
         IF(MAX_MEM.GT.MEM_ON_PROCS(i))THEN
            PROC=i
         ENDIF
         MAX_MEM=min(MEM_ON_PROCS(i),MAX_MEM)
      ENDDO
      DEALLOCATE(MEM_ON_PROCS)
      DEALLOCATE(CONCERNED)
      DEALLOCATE(RECV_BUF)
      END SUBROUTINE COMPUTE_MAX_MEM
      SUBROUTINE DAGMG_MUMPS_553(MIN_PROC,POOL,                              &
     &                      LPOOL,INODE)
      IMPLICIT NONE
      INTEGER INODE,LPOOL,MIN_PROC
      INTEGER POOL(LPOOL)
      EXTERNAL AGMG_MUMPS_275
      INTEGER AGMG_MUMPS_275
      INTEGER i,NBTOP,INSUBTREE,NBINSUBTREE,NODE,FATHER,SON,J
      INTEGER FIRST_LEAF,SBTR_NB_LEAF,POS,K,allocok,L
      INTEGER, ALLOCATABLE, DIMENSION (:) ::  TMP_SBTR
      NBINSUBTREE = POOL(LPOOL)
      NBTOP       = POOL(LPOOL - 1)
      INSUBTREE   = POOL(LPOOL - 2)
      IF((KEEP_LOAD(47).EQ.4).AND.                                      &
     &     ((NBINSUBTREE.NE.0)))THEN
         DO J=INDICE_SBTR,NB_SUBTREES
            NODE=MY_ROOT_SBTR(J)
            FATHER=DAD_LOAD(STEP_LOAD(NODE))
            i=FATHER
  110       CONTINUE
            IF ( i > 0 ) THEN
               i = FILS_LOAD(i)
               GOTO 110
            ENDIF
            SON=-i
            i=SON
  120       CONTINUE
            IF ( i > 0 ) THEN
               IF(AGMG_MUMPS_275(STEP_LOAD(i),PROCNODE_LOAD,NPROCS).EQ.      &
     &              MIN_PROC)THEN
                  SBTR_NB_LEAF=MY_NB_LEAF(J)
                  POS=SBTR_FIRST_POS_IN_POOL(J)
                  IF(POOL(POS+SBTR_NB_LEAF).NE.MY_FIRST_LEAF(J))THEN
                     WRITE(*,*)MYID,': The first leaf is not ok'
                     CALL AGMG_MUMPS_ABORT()
                  ENDIF
                  ALLOCATE (TMP_SBTR(SBTR_NB_LEAF), stat=allocok)
                  IF (allocok > 0 ) THEN
                     WRITE(*,*)MYID,': Not enough space                 &
     &                    for allocation'
                     CALL AGMG_MUMPS_ABORT()
                  ENDIF
                  POS=SBTR_FIRST_POS_IN_POOL(J)
                  DO K=1,SBTR_NB_LEAF
                     TMP_SBTR(K)=POOL(POS+K-1)
                  ENDDO
                  DO K=POS+1,NBINSUBTREE-SBTR_NB_LEAF
                     POOL(K)=POOL(K+SBTR_NB_LEAF)
                  ENDDO
                  POS=1
                  DO K=NBINSUBTREE-SBTR_NB_LEAF+1,NBINSUBTREE
                     POOL(K)=TMP_SBTR(POS)
                     POS=POS+1
                  ENDDO
                  DO K=INDICE_SBTR,J
                     SBTR_FIRST_POS_IN_POOL(K)=SBTR_FIRST_POS_IN_POOL(K)&
     &                    -SBTR_FIRST_POS_IN_POOL(J)
                  ENDDO
                  SBTR_FIRST_POS_IN_POOL(J)=NBINSUBTREE-SBTR_NB_LEAF
                  POS=MY_FIRST_LEAF(J)
                  L=MY_NB_LEAF(J)
                  DO K=INDICE_SBTR,J
                     MY_FIRST_LEAF(J)=MY_FIRST_LEAF(J+1)
                     MY_NB_LEAF(J)=MY_NB_LEAF(J+1)
                  ENDDO
                  MY_FIRST_LEAF(INDICE_SBTR)=POS
                  MY_NB_LEAF(INDICE_SBTR)=L
                  INODE=POOL(NBINSUBTREE)
                  DEALLOCATE(TMP_SBTR)
                  RETURN
               ENDIF
               i = FRERE_LOAD(STEP_LOAD(i))
               GOTO 120
            ENDIF
         ENDDO
      ENDIF
      DO J=NBTOP,1,-1
         NODE=POOL(LPOOL-2-J)
         FATHER=DAD_LOAD(STEP_LOAD(NODE))
         i=FATHER
   11    CONTINUE
         IF ( i > 0 ) THEN
            i = FILS_LOAD(i)
            GOTO 11
         ENDIF
         SON=-i
         i=SON
   12    CONTINUE
         IF ( i > 0 ) THEN
            IF(AGMG_MUMPS_275(STEP_LOAD(i),PROCNODE_LOAD,NPROCS).EQ.         &
     &           MIN_PROC)THEN
               INODE=NODE
               RETURN
            ENDIF
            i = FRERE_LOAD(STEP_LOAD(i))
            GOTO 12
         ENDIF
      ENDDO
      END SUBROUTINE DAGMG_MUMPS_553
      SUBROUTINE DAGMG_MUMPS_555(POOL, LPOOL,KEEP,KEEP8)
      IMPLICIT NONE
      INTEGER LPOOL,POOL(LPOOL),KEEP(500)
      INTEGER*8 KEEP8(150)
      INTEGER i,POS
      EXTERNAL AGMG_MUMPS_283
      LOGICAL AGMG_MUMPS_283
      IF(.NOT.BDC_SBTR) RETURN
      POS=0
      DO i=NB_SUBTREES,1,-1
         DO WHILE(AGMG_MUMPS_283(STEP_LOAD(POOL(POS+1)),                     &
     &        PROCNODE_LOAD,NPROCS))
            POS=POS+1
         ENDDO
         SBTR_FIRST_POS_IN_POOL(i)=POS+1
         POS=POS+MY_NB_LEAF(i)
      ENDDO
      END SUBROUTINE DAGMG_MUMPS_555
      END MODULE DAGMG_MUMPS_LD
      ! final merge of series 3 with 39 inserted here
!
!  This file is part of MUMPS 4.9.2, built on Thu Nov  5 07:05:08 UTC 20
!
!
!  This version of MUMPS is provided to you free of charge. It is public
!  domain, based on public domain software developed during the Esprit I
!  European project PARASOL (1996-1999) by CERFACS, ENSEEIHT-IRIT and RA
!  Since this first public domain version in 1999, the developments are
!  supported by the following institutions: CERFACS, CNRS, INPT(ENSEEIHT
!  IRIT, and INRIA.
!
!  Current development team includes Patrick Amestoy, Alfredo Buttari,
!  Abdou Guermouche, Jean-Yves L'Excellent, Bora Ucar.
!
!  Up-to-date copies of the MUMPS package can be obtained
!  from the Web pages:
!  http://mumps.enseeiht.fr/  or  http://graal.ens-lyon.fr/MUMPS
!
!
!   THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY
!   EXPRESSED OR IMPLIED. ANY USE IS AT YOUR OWN RISK.
!
!
!  User documentation of any code that uses this software can
!  include this complete notice. You can acknowledge (using
!  references [1] and [2]) the contribution of this package
!  in any scientific publication dependent upon the use of the
!  package. You shall use reasonable endeavours to notify
!  the authors of the package of this publication.
!
!   [1] P. R. Amestoy, I. S. Duff, J. Koster and  J.-Y. L'Excellent,
!   A fully asynchronous multifrontal solver using distributed dynamic
!   scheduling, SIAM Journal of Matrix Analysis and Applications,
!   Vol 23, No 1, pp 15-41 (2001).
!
!   [2] P. R. Amestoy and A. Guermouche and J.-Y. L'Excellent and
!   S. Pralet, Hybrid scheduling for the parallel solution of linear
!   systems. Parallel Computing Vol 32 (2), pp 136-156 (2006).
!
      MODULE AGMG_MUMPS_STATIC_MAP
      IMPLICIT NONE
      PRIVATE
      PUBLIC :: AGMG_MUMPS_369, AGMG_MUMPS_393,                                   &
     &     AGMG_MUMPS_427,AGMG_MUMPS_494
      integer,pointer,dimension(:,:),SAVE::cv_cand
      integer,pointer,dimension(:),SAVE::cv_par2_nodes
      integer,SAVE::cv_slavef,cv_nb_niv2,cv_lp,cv_mp
      DOUBLE PRECISION,SAVE::cv_stack_peak
      integer,SAVE::cv_mem_strat
      integer,parameter::cv_invalid=-9999
      DOUBLE PRECISION,parameter::cv_d_invalid=-9999.D0
      integer,parameter::cv_equilib_flops=1
      integer,parameter::cv_equilib_mem=2
      integer,parameter::cv_error_memalloc = -13
      integer,parameter::cv_error_memdeloc = -96
      integer,dimension(:),allocatable,save :: mem_distribtmp
      integer, dimension(:),allocatable, save :: table_of_process
      integer,dimension(:),allocatable,save :: mem_distribmpi
      integer, save ::ke69,nb_arch_nodes
      logical,dimension(:),allocatable,save :: allowed_nodes
      integer,dimension(:),allocatable,save :: score
      type nodelist
         integer::nodenumber
         type(nodelist),pointer::next
      end type nodelist
      type alloc_arraytype
         integer, pointer, dimension(:)::t2_nodenumbers
         integer, pointer, dimension(:,:)::t2_cand
         DOUBLE PRECISION, pointer, dimension(:)::t2_candcostw(:),      &
     &                                            t2_candcostm(:)
         integer:: nmb_t2s
      end type alloc_arraytype
      type splitting_data
         integer:: new_ison,new_ifather,old_keep2
         DOUBLE PRECISION:: ncostw_oldinode,ncostm_oldinode,            &
     &                      tcostw_oldinode,tcostm_oldinode
      end type splitting_data
      type procs4node_t
         integer, dimension(:), pointer :: ind_proc
      end type procs4node_t
      DOUBLE PRECISION, pointer, dimension(:) ::                        &
     &      cv_proc_workload,                                           &
     &      cv_proc_maxwork,                                            &
     &      cv_proc_memused,                                            &
     &      cv_proc_maxmem
      type(splitting_data)::cv_last_splitting
      integer::cv_n,cv_nsteps,cv_maxlayer,                              &
     &         cv_nbsa,cv_maxnsteps,cv_maxcut,cv_maxdepth,              &
     &         cv_maxnodenmb,cv_total_amalg,cv_total_split,             &
     &         cv_bitsize_of_int,cv_size_ind_proc                       &
     &         ,cv_mixed_strat_bound,cv_dist_L0_mixed_strat_bound       &
     &         ,cv_layerl0_end,cv_layerl0_start
      integer :: layerL0_endforarrangeL0
      DOUBLE PRECISION :: mincostw
      DOUBLE PRECISION:: cv_costw_upper,cv_costm_upper,                 &
     &      cv_costw_layer0,cv_costm_layer0,cv_relax,                   &
     &      cv_costw_total,cv_costm_total,cv_l0wthresh,cv_splitthresh
      logical::cv_constr_work,cv_constr_mem
      integer,pointer,dimension(:):: cv_nodetype,cv_nodelayer,          &
     &          cv_layerl0_array,cv_proc_sorted,cv_potsplit,cv_depth
      integer,dimension(:),pointer::                                    &
     &    cv_ne,cv_nfsiz,cv_frere,cv_fils,cv_keep,cv_info,              &
     &    cv_procnode,cv_ssarbr,cv_icntl
      integer*8,dimension(:),pointer::cv_keep8
      type(alloc_arraytype),pointer,dimension(:)::cv_layer_p2node
      DOUBLE PRECISION,dimension(:),pointer:: cv_ncostw,                &
     & cv_tcostw,cv_ncostm,cv_tcostm,cv_layerworkload,cv_layermemused   &
     & ,cv_layerl0_sorted_costw
      type(procs4node_t),dimension(:),pointer :: cv_prop_map
      contains
      subroutine AGMG_MUMPS_369(n,slavef,icntl,info,                         &
     &                        ne,nfsiz,frere,fils,keep,KEEP8,           &
     &                        procnode,ssarbr,nbsa,peak,istat           &
     &     )
      implicit none
      integer,intent(in)::n,slavef
      integer, intent(inout),TARGET:: ne(n),nfsiz(n),                   &
     &         procnode(n),ssarbr(n),frere(n),fils(n),keep(500),        &
     &         icntl(40),info(40)
      INTEGER*8 KEEP8(150)
      integer,intent(out)::nbsa,istat
      integer ierr,nmb_thislayer,layernmb,mapalgo,allocok,i
      integer,pointer,dimension(:)::thislayer
      integer,parameter::memonly=1,floponly=2,hybrid=3
      DOUBLE PRECISION::                                                &
     &         maxwork,minwork,maxmem,minmem,workbalance,membalance
      DOUBLE PRECISION:: cost_root_node
      DOUBLE PRECISION,dimension(:),allocatable:: work_per_proc
      integer,dimension(:),allocatable::id_son
      logical::cont
      character (len=48):: err_rep,subname
      DOUBLE PRECISION peak
      istat=-1
      subname='DISTRIBUTE'
      cv_lp=icntl(1)
      cv_mp=icntl(3)
      nullify(thislayer)
      err_rep='INITPART1'
      call AGMG_MUMPS_478(n,slavef,                                          &
     &                   frere,fils,nfsiz,ne,keep,KEEP8,icntl,info,     &
     &                   procnode,ssarbr,peak,ierr                      &
     &     )
      if (ierr.ne.0) goto 99999
      err_rep='PROCINIT'
      call AGMG_MUMPS_391(istat=ierr)
      if (ierr.ne.0) goto 99999
      err_rep='CALCCOST'
      call AGMG_MUMPS_417(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='ROOTLIST'
      call AGMG_MUMPS_394(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='LAYERL0'
      call AGMG_MUMPS_381(ierr)
      if (ierr.ne.0) goto 99999
      if(cv_keep(82) .eq. 0) then
         err_rep='FIND_POTSPLIT'
         call AGMG_MUMPS_375(cv_equilib_flops,ierr)
      endif
      if (ierr.ne.0) goto 99999
      err_rep='INITPART2'
      call AGMG_MUMPS_479(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='WORKMEM_'
      call AGMG_MUMPS_408(                                                   &
     &     cv_proc_workload,cv_proc_memused,                            &
     &     maxwork,minwork,maxmem,minmem)
      if(maxwork.gt.0.0D0) then
         workbalance=minwork/maxwork
      else
         workbalance=0.0D0
      endif
      if(maxmem.gt.0.0D0) then
         membalance=minmem/maxmem
      else
         membalance=0.0D0
      endif
      err_rep='mem_alloc'
      allocate(thislayer(cv_maxnodenmb),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 2*cv_maxnsteps+cv_maxnodenmb
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'memory allocation error in ',subname
         ierr = cv_error_memalloc
         goto 99999
      end if
      cont=.TRUE.
      layernmb=0
      mapalgo=floponly
      err_rep='SELECT_TYPE3'
      call AGMG_MUMPS_396(ierr)
      if (ierr.ne.0) goto 99999
      IF (cv_keep(38) .ne. 0 .and. cv_keep(60) .eq. 0 ) THEN
        call AGMG_MUMPS_511(cv_nfsiz(keep(38)),                              &
     &               cv_nfsiz(keep(38)), cv_nfsiz(keep(38)),            &
     &               cv_keep(50), 3, cost_root_node)
        cost_root_node = cost_root_node / dble(cv_slavef)
        do i=1, cv_slavef
          cv_proc_memused(i)=cv_proc_memused(i)+                        &
     &        dble(cv_nfsiz(keep(38)))*dble(cv_nfsiz(keep(38)))/        &
     &        dble(cv_slavef)
          cv_proc_workload(i)=cv_proc_workload(i)+dble(cost_root_node)
        enddo
      ENDIF
      do while((cont).OR.(layernmb.le.cv_maxlayer))
         err_rep='FIND_THIS'
         call AGMG_MUMPS_376(layernmb,thislayer,nmb_thislayer,               &
     &                                ierr)
         if (ierr.ne.0) goto 99999
         err_rep='DO_SPLITTING'
         if(cv_keep(82) .gt. 0) then
            if(layernmb.gt.0) call AGMG_MUMPS_527                            &
     &           (layernmb,thislayer,nmb_thislayer,ierr)
         else
            if(layernmb.gt.0) call AGMG_MUMPS_370                            &
     &           (layernmb,thislayer,nmb_thislayer,ierr)
         endif
         if (ierr.ne.0) goto 99999
         err_rep='ASSIGN_TYPES'
         call AGMG_MUMPS_416(layernmb,thislayer,nmb_thislayer,               &
     &                              ierr)
         if (ierr.ne.0) goto 99999
         if(layernmb.gt.0) then
            if ((cv_keep(24).eq.1).OR.(cv_keep(24).eq.2).OR.            &
     &          (cv_keep(24).eq.4).OR.(cv_keep(24).eq.6)) then
               err_rep='COSTS_LAYER_T2'
               call AGMG_MUMPS_367(layernmb,nmb_thislayer,ierr)
            elseif((cv_keep(24).eq.8).OR.(cv_keep(24).eq.10)            &
     &             .OR.(cv_keep(24).eq.12).OR.(cv_keep(24).eq.14)       &
     &             .OR.(cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
               err_rep='COSTS_LAYER_T2PM'
               call AGMG_MUMPS_489(layernmb,nmb_thislayer,ierr)
            else
               err_rep='wrong strategy for COSTS_LAYER_T2'
               ierr = -9999
            endif
            if (ierr.ne.0) goto 99999
            err_rep='WORKMEM_'
            call AGMG_MUMPS_408(                                             &
     &                          cv_proc_workload,cv_proc_memused,       &
     &                                   maxwork,minwork,maxmem,minmem)
            if(maxwork.gt.0.0D0) then
               workbalance=minwork/maxwork
            else
               workbalance=0.0D0
            endif
            if(maxmem.gt.0.0D0) then
               membalance=minmem/maxmem
            else
               membalance=0.0D0
            endif
            if(mapalgo.eq.memonly) then
               err_rep='MAP_LAYER'
               call AGMG_MUMPS_387(layernmb,thislayer,                       &
     &              nmb_thislayer,cv_equilib_mem,ierr)
               if (ierr.ne.0) goto 99999
            elseif(mapalgo.eq.floponly) then
               err_rep='MAP_LAYER'
               call AGMG_MUMPS_387(layernmb,thislayer,                       &
     &              nmb_thislayer,cv_equilib_flops,ierr)
               if (ierr.ne.0) goto 99999
            elseif(mapalgo.eq.hybrid) then
               if (workbalance <= membalance) then
                  err_rep='MAP_LAYER'
                  call AGMG_MUMPS_387(layernmb,thislayer,                    &
     &                 nmb_thislayer,cv_equilib_flops,ierr)
                  if (ierr.ne.0) goto 99999
               else
                  err_rep='MAP_LAYER'
                  call AGMG_MUMPS_387(layernmb,thislayer,                    &
     &                 nmb_thislayer,cv_equilib_mem,ierr)
                  if (ierr.ne.0) goto 99999
               endif
            else
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Unknown mapalgo in ',subname
               return
            endif
      endif
         layernmb=layernmb+1
         err_rep='HIGHER_LAYER'
         call AGMG_MUMPS_377(layernmb,thislayer,                             &
     &                 nmb_thislayer,cont,ierr)
         if (ierr.ne.0) goto 99999
      end do
       if(cv_slavef.gt.4)                                               &
     &   call AGMG_MUMPS_431()
      err_rep='SETUP_CAND'
      call AGMG_MUMPS_397(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='ENCODE_PROC'
      call AGMG_MUMPS_371(ierr)
      if (ierr.ne.0) goto 99999
      err_rep='STORE_GLOB'
      call AGMG_MUMPS_402(ne,nfsiz,frere,fils,keep,KEEP8,                    &
     &                         info,procnode,ssarbr,nbsa)
      err_rep='mem_dealloc'
      deallocate(thislayer,STAT=ierr)
      if (ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'Memory deallocation error in ',subname
         ierr = cv_error_memdeloc
         goto 99999
      endif
      err_rep='TERMGLOB'
      call AGMG_MUMPS_403(ierr)
      if (ierr.ne.0) goto 99999
      istat=0
      return
99999 continue
      if(cv_lp.gt.0) then
         write(cv_lp,*)'Error in ',subname,', layernmb=',layernmb
         write(cv_lp,*)'procedure reporting the error: ',err_rep
      endif
      if(ierr.eq.cv_error_memalloc) then
         info(1) = cv_info(1)
         info(2) = cv_info(2)
      endif
      istat=ierr
      return
      CONTAINS
      subroutine AGMG_MUMPS_413(                                             &
     &                       map_strat,workload,memused,accepted,       &
     &                              istat)
      implicit none
      integer,intent(in)::map_strat
      DOUBLE PRECISION,dimension(cv_slavef),intent(in)::workload,       &
     &                                                   memused
      logical,intent(out)::accepted
      integer,intent(out)::istat
      DOUBLE PRECISION maxi,mini,mean,stddev
      integer i,nmb
      intrinsic maxval,minval,count,sum
      character (len=48):: subname
      logical alternative_criterion
      DOUBLE PRECISION::                                                &
     &          MINFLOPS , MINMEM,                                      &
     &          CL_RATE, DV_RATE
      istat=-1
      if ( cv_keep(72) .EQ. 1) then
       MINFLOPS = 2.0D0
       MINMEM=50.0D0
       CL_RATE =0.8D0
       DV_RATE=0.2D0
      else
       MINFLOPS = 5.0D7
       MINMEM=5.0D6
       CL_RATE =0.8D0
       DV_RATE=0.2D0
      endif
      subname='ACCEPT_L0'
      accepted=.FALSE.
      alternative_criterion=.FALSE.
      if(map_strat.eq.cv_equilib_flops) then
         maxi=maxval(workload)
         mini=minval(workload)
         if (maxi.lt.MINFLOPS) then
            accepted=.TRUE.
         elseif(maxi.le.(dble(cv_keep(102))/dble(100))*mini)then
            accepted=.TRUE.
         endif
         if ((.NOT.accepted).AND.(alternative_criterion)) then
            mean=sum(workload)/max(dble(cv_slavef),dble(1))
            stddev=dble(0)
            do i=1,cv_slavef
               stddev=stddev+                                           &
     &               (abs(workload(i)-mean)*abs(workload(i)-mean))
            enddo
            stddev=sqrt(stddev/max(dble(cv_slavef),dble(1)))
            nmb=count(mask=abs(workload-mean)<stddev)
            if((dble(nmb)/max(dble(cv_slavef),dble(1)).gt.CL_RATE)      &
     &       .AND.(stddev.lt.DV_RATE*mean)) accepted=.TRUE.
         endif
      elseif(map_strat.eq.cv_equilib_mem) then
         maxi=maxval(memused)
         mini=minval(memused)
         if (maxi.lt.MINMEM) then
            accepted=.TRUE.
         else if(cv_slavef.lt.48) then
            if (maxi.le.dble(2)*mini) accepted=.TRUE.
         else if(cv_slavef.lt.128) then
            if (maxi.le.dble(4)*mini) accepted=.TRUE.
         else if(cv_slavef.lt.256) then
            if (maxi.le.dble(6)*mini) accepted=.TRUE.
         else if(cv_slavef.lt.512) then
            if (maxi.le.dble(8)*mini) accepted=.TRUE.
         else if(cv_slavef.gt.512) then
            if (maxi.le.dble(10)*mini) accepted=.TRUE.
         end if
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_413
      subroutine AGMG_MUMPS_414(ifather,ison,istat)
      implicit none
      integer,intent(in)::ifather,ison
      integer, intent(out)::istat
      integer in,npiv_son,in_father,in_ison,bigbrother,oldest_son,      &
     &        npiv,nfront
      DOUBLE PRECISION costm,costw
      character (len=48):: subname
      istat=-1
      subname='AMALGAMATE_SONFATH'
      if((cv_frere(ison).eq.0).or.(cv_frere(ison).eq.cv_n+1)) return
      in=ifather
      do while (cv_fils(in).gt.0)
         in=cv_fils(in)
      end do
      if (cv_fils(in).eq.0) return
      in_father=in
      in = -cv_fils(in)
      if (in.eq.ison) then
         cv_fils(in_father)=ison
         in=ison
         do while (cv_fils(in).gt.0)
            in=cv_fils(in)
         end do
         if (cv_fils(in).eq.0) then
            if(cv_frere(ison).gt.0) then
               cv_fils(in)=-cv_frere(ison)
            end if
            cv_frere(ison)=cv_n+1
         else
            in=-cv_fils(in)
            do while(cv_frere(in).gt.0)
               in=cv_frere(in)
            end do
            cv_frere(in)=cv_frere(ison)
            cv_frere(ison)=cv_n+1
         end if
      else
         oldest_son=in
         do while((cv_frere(in).ne.ison).and.(cv_frere(in).gt.0))
            in=cv_frere(in)
         end do
         if (cv_frere(in).ne.ison) return
         bigbrother=in
         cv_fils(in_father)=ison
         in=ison
         do while (cv_fils(in).gt.0)
            in=cv_fils(in)
         end do
         in_ison=in
         if (cv_fils(in).eq.0) then
            cv_frere(bigbrother)=cv_frere(ison)
            cv_frere(ison)=cv_n+1
            cv_fils(in_ison)=-oldest_son
         else
            in=-cv_fils(in_ison)
            cv_frere(bigbrother)=in
            do while(cv_frere(in).gt.0)
               in=cv_frere(in)
            end do
            cv_frere(in)=cv_frere(ison)
            cv_frere(ison)=cv_n+1
            cv_fils(in_ison)=-oldest_son
         end if
      end if
      cv_nsteps = cv_nsteps - 1
      cv_ne(ifather)=cv_ne(ifather)+cv_ne(ison)-1
      cv_ne(ison)=0
      in=ison
      npiv_son=0
      do while (in.gt.0)
         in=cv_fils(in)
         npiv_son=npiv_son+1
      end do
      cv_nfsiz(ifather)=cv_nfsiz(ifather)+npiv_son
      cv_nfsiz(ison)=0
      cv_info(5)=max(cv_info(5),cv_nfsiz(ifather))
      in=ifather
      npiv=0
      do while (in.gt.0)
         in=cv_fils(in)
         npiv=npiv+1
      end do
      nfront=cv_nfsiz(ifather)
      call AGMG_MUMPS_418(npiv,nfront,costw,costm)
      cv_ncostw(ifather)=costw
      cv_ncostm(ifather)=costm
      cv_ncostw(ison)=0.0D0
      cv_ncostm(ison)=0.0D0
      if(associated(cv_tcostw)) cv_tcostw(ison)=0.0D0
      if(associated(cv_tcostm)) cv_tcostm(ison)=0.0D0
      cv_total_amalg=cv_total_amalg+1
      call AGMG_MUMPS_436(ison,ifather,ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'PROPMAP4AMALG error in ',subname
         istat = ierr
         return
      end if
      istat = 0
      return
      end subroutine AGMG_MUMPS_414
      subroutine AGMG_MUMPS_415(map_strat,layerL0end,workload,memused,       &
     &                           procnode,istat,respect_prop)
      implicit none
      integer, intent(in)::map_strat, layerL0end
      DOUBLE PRECISION,dimension(cv_slavef),intent(out)::workload,      &
     &                                                   memused
      integer, intent(out)::procnode(cv_n),istat
      logical, intent(in), OPTIONAL:: respect_prop
      integer nmb_listnodes,i,j,ierr,                                   &
     &        nodenumber,proc
      DOUBLE PRECISION tot_work,tot_mem,work,mem
      character (len=48):: err_rep,subname
      istat=-1
      subname='ARRANGEL0'
      if ((.NOT.associated(cv_tcostw)).OR.(.NOT.associated(cv_tcostm))) &
     &   then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:tcost must be allocated in ',subname
         return
      end if
      if((map_strat.ne.cv_equilib_flops).and.                           &
     &   (map_strat.ne.cv_equilib_mem)) return
      do i=1,cv_n
         procnode(i)=cv_invalid
      end do
      do i=1,cv_slavef
         workload(i)=cv_proc_workload(i)
         memused(i)=cv_proc_memused(i)
      end do
      do i=cv_layerl0_start,layerl0end
         nodenumber=cv_layerl0_array(i)
         work=cv_tcostw(nodenumber)
         mem=cv_tcostm(nodenumber)
         err_rep='FIND_BEST_PROC'
         if(present(respect_prop)) then
            call AGMG_MUMPS_374(nodenumber,map_strat,work,mem,               &
     &           workload,memused,proc,ierr,respect_prop)
         else
            call AGMG_MUMPS_374(nodenumber,map_strat,work,mem,               &
     &           workload,memused,proc,ierr)
         endif
         if(ierr.eq.0) then
            procnode(nodenumber)=proc
         else
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            do j=1,cv_slavef
               workload(j)=cv_proc_workload(j)
               memused(j)=cv_proc_memused(j)
            end do
            do j=1,cv_n
               procnode(j)=cv_invalid
            end do
            return
         end if
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_415
      subroutine AGMG_MUMPS_416(                                             &
     &                              layernmb,thislayer,nmb_thislayer,   &
     &                                 istat)
      implicit none
      integer,intent(in)::layernmb,thislayer(cv_maxnodenmb),            &
     &                    nmb_thislayer
      integer,intent(out)::istat
      integer i,in,npiv,nfront,inode,inoderoot,par_nodes_in_layer,      &
     &        dummy,allocok
      character (len=48):: subname
      istat=-1
      subname='ASSIGN_TYPES'
      if((layernmb.lt.0).or.(layernmb.gt.cv_maxlayer)) return
      if(cv_slavef.eq.1) then
         if(layernmb.eq.0) then
            do inode=1,cv_n
               cv_nodetype(inode)=0
            end do
         end if
      else if(layernmb.eq.0) then
         do i=1,nmb_thislayer
            inode=thislayer(i)
            inoderoot=inode
            if(cv_nodetype(inode).ne.cv_invalid) cycle
            cv_nodetype(inode)=0
   30       continue
            in = inode
            do while (in .ne. 0)
               inode = in
               do while (in .gt. 0)
                  in = cv_fils(in)
               end do
               if (in.lt.0) in=-in
            end do
   10       continue
            if ( inode .ne. inoderoot ) then
               cv_nodetype(inode)=-1
               in = cv_frere(inode)
               inode = abs(in)
               if (in .lt. 0) then
                  go to 10
               else
                  go to 30
               end if
            end if
         end do
      else
         do i=1,nmb_thislayer
            inode=thislayer(i)
            in = inode
            npiv = 0
            do while (in.gt.0)
               in = cv_fils(in)
               npiv = npiv + 1
            end do
            nfront = cv_nfsiz(inode)
            if(cv_nodetype(inode).ne.cv_invalid) cycle
            if( ( AGMG_MUMPS_359(nfront,npiv)) .AND.                         &
     &           (in.ne.0)) then
               cv_nodetype(inode)=2
            else
               cv_nodetype(inode)=1
            end if
         end do
      end if
      if(layernmb.gt.0) then
         par_nodes_in_layer=0
         do i=1,nmb_thislayer
            inode=thislayer(i)
            if (cv_nodetype(inode).eq.2)                                &
     &         par_nodes_in_layer=par_nodes_in_layer+1
         enddo
         if(par_nodes_in_layer.gt.0) then
            allocate(                                                   &
     &cv_layer_p2node(layernmb)%t2_nodenumbers(par_nodes_in_layer),     &
     &cv_layer_p2node(layernmb)%t2_cand(par_nodes_in_layer,cv_slavef+1),&
     &cv_layer_p2node(layernmb)%t2_candcostw(par_nodes_in_layer),       &
     &cv_layer_p2node(layernmb)%t2_candcostm(par_nodes_in_layer),       &
     &               STAT=allocok)
            if (allocok.gt.0) then
               cv_info(1) = cv_error_memalloc
               cv_info(2) = (3+cv_slavef+1)*par_nodes_in_layer
               istat = cv_error_memalloc
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'memory allocation error in ',subname
               return
            end if
            cv_layer_p2node(layernmb)%nmb_t2s=par_nodes_in_layer
            dummy=1
            do i=1,nmb_thislayer
               inode=thislayer(i)
               if (cv_nodetype(inode).eq.2) then
                  cv_layer_p2node(layernmb)%t2_nodenumbers(dummy)=inode
                  cv_layer_p2node(layernmb)%t2_cand(dummy,:)=0
                  cv_layer_p2node(layernmb)%t2_candcostw(dummy)         &
     &                                                    =cv_d_invalid
                  cv_layer_p2node(layernmb)%t2_candcostm(dummy)         &
     &                                                    =cv_d_invalid
                  dummy=dummy+1
               endif
            enddo
         else
            nullify(cv_layer_p2node(layernmb)%t2_nodenumbers,           &
     &              cv_layer_p2node(layernmb)%t2_cand,                  &
     &              cv_layer_p2node(layernmb)%t2_candcostw,             &
     &              cv_layer_p2node(layernmb)%t2_candcostm)
         end if
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_416
      function AGMG_MUMPS_480(procs4node,procnumber)
      implicit none
      integer,intent(in)::procs4node(cv_size_ind_proc)
      integer,intent(in)::procnumber
      logical :: AGMG_MUMPS_480
      integer pos1,pos2
      pos1 = (procnumber-1)/cv_bitsize_of_int +1
      pos2 = mod(procnumber-1,cv_bitsize_of_int)
      AGMG_MUMPS_480=btest(procs4node(pos1),pos2)
      return
      end function AGMG_MUMPS_480
      function AGMG_MUMPS_481(inode,procnumber)
      implicit none
      integer, intent(in)::inode,procnumber
      logical :: AGMG_MUMPS_481
      integer pos1,pos2
      AGMG_MUMPS_481=.FALSE.
      if((procnumber.lt.1).or.(procnumber.gt.cv_slavef)) return
      if(.not.associated(cv_prop_map(inode)%ind_proc)) return
      pos1 = (procnumber-1)/cv_bitsize_of_int +1
      pos2 = mod(procnumber-1,cv_bitsize_of_int)
      AGMG_MUMPS_481=btest                                                   &
     &               (cv_prop_map(inode)%ind_proc(pos1),pos2)
      return
      end function AGMG_MUMPS_481
      subroutine AGMG_MUMPS_482(procs4node,procnumber,istat)
      implicit none
      integer, intent(inout)::procs4node(cv_size_ind_proc)
      integer,intent(in)::procnumber
      integer, intent(out)::istat
      integer pos1,pos2
      istat = -1
      if((procnumber.lt.1).or.(procnumber.gt.cv_slavef)) return
      if(cv_bitsize_of_int.le.0) return
      pos1 = (procnumber-1)/cv_bitsize_of_int +1
      pos2 = mod(procnumber-1,cv_bitsize_of_int)
      procs4node(pos1)=ibset(procs4node(pos1),pos2)
      istat = 0
      return
      end subroutine AGMG_MUMPS_482
      subroutine AGMG_MUMPS_417(istat)
      implicit none
      integer,intent(out)::istat
      integer i,ierr
      DOUBLE PRECISION :: maxcostw_root
      character (len=48):: err_rep,subname
      istat = -1
      subname='CALCCOSTS'
      if ((.NOT.associated(cv_tcostw)).OR.(.NOT.associated(cv_tcostm))) &
     &   then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:tcost must be allocated in ',subname
         return
      end if
      maxcostw_root = 0D0
      do i=1,cv_n
         if (cv_frere(i).eq.cv_n+1) then
            cv_tcostw(i)=0.0D0
            cv_ncostw(i)=0.0D0
            cv_tcostm(i)=0.0D0
            cv_ncostm(i)=0.0D0
         elseif (cv_frere(i).eq.0) then
            err_rep='TREECOSTS'
            cv_depth(i)=1
            call AGMG_MUMPS_404(i,ierr)
            maxcostw_root = max(maxcostw_root,cv_tcostw(i))
            if (ierr.ne.0) then
          if(cv_lp.gt.0)                                                &
     &    write(cv_lp,*)'Failure reported by ',err_rep, ' in ',subname
               return
            end if
         end if
      end do
      istat = 0
      mincostw = 1.0D0+maxcostw_root/(dble(cv_maxnsteps)*               &
     &              dble(10*cv_slavef) )
      return
      end subroutine AGMG_MUMPS_417
      subroutine AGMG_MUMPS_418(npiv,nfront,costw,costm)
      implicit none
      integer,intent(in)::npiv,nfront
      DOUBLE PRECISION,intent(out)::costw,costm
      character (len=48):: subname
      subname='CALCNODECOSTS'
      if((npiv.le.1).and.(nfront.le.1)) then
         costw = dble(0)
         costm = dble(1)
      else
         if(cv_keep(50).eq.0) then
            costw= 2.0D0*dble(nfront)*dble(npiv)*dble(nfront-npiv-1)    &
     &      + dble(npiv)*dble(npiv+1)*dble(2*npiv+1)/dble(3)            &
     &           + dble(2*nfront-npiv-1) * dble(npiv) / dble(2)
            costm= dble(npiv)*(dble(2*nfront)-dble(npiv))
         else
            costw= dble(npiv) *                                         &
     &            (dble(nfront)*dble(nfront)+dble(2*nfront) -           &
     &             dble(nfront+1) * dble(npiv+1) +                      &
     &             dble(npiv+1) * dble(2*npiv+1) / dble(6))
            costm= dble(npiv) * dble(nfront)
         end if
      end if
      if((costw.lt.0.0D0).or.(costm.lt.0.0D0)) then
      endif
      return
      end subroutine AGMG_MUMPS_418
      subroutine AGMG_MUMPS_367(layernmb,nmb_thislayer,istat)
      implicit none
      integer,intent(in)::layernmb,nmb_thislayer
      integer,intent(out)::istat
      integer in,inode,j,kmax,npiv,nfront,ncb,ncol,                     &
     &        min_needed,max_needed,more_than_needed,total_nmb_cand,    &
     &        nmb_type2_thislayer,fraction,                             &
     &        total_cand_layer,cand_strat, keep48_loc, WHAT_LOC
      DOUBLE PRECISION flop1,work_type2_thislayer,                      &
     &        relative_weight,workmaster,nrow
      logical force_cand
      intrinsic count,max
      character (len=48):: subname
      integer AGMG_MUMPS_497, AGMG_MUMPS_50,                                      &
     &        AGMG_MUMPS_52
      external AGMG_MUMPS_497, AGMG_MUMPS_50,                                     &
     &        AGMG_MUMPS_52
      istat=-1
      subname='COSTS_LAYER_T2'
      if (cv_keep(24).lt.1) then
          if(cv_lp.gt.0)                                                &
     &    write(cv_lp,*)'Error in ',subname,'. Wrong keep24'
         return
      endif
      force_cand=(mod(cv_keep(24),2).eq.0)
      cand_strat=cv_keep(24)/2
      nmb_type2_thislayer=cv_layer_p2node(layernmb)%nmb_t2s
      if (nmb_type2_thislayer.gt.0) then
         work_type2_thislayer=0.0D0
         do j=1,nmb_type2_thislayer
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(j)
            work_type2_thislayer=work_type2_thislayer+cv_ncostw(inode)
         end do
         if(cv_relax.le.0.0D0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Error in ',subname,'. Wrong cv_relax'
            return
         endif
         total_cand_layer=int(cv_relax*dble(cv_slavef))
         do j=1,nmb_type2_thislayer
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(j)
            nfront=cv_nfsiz(inode)
            npiv=0
            in=inode
            do while(in.gt.0)
               npiv=npiv+1
               in=cv_fils(in)
            end do
            ncb=nfront-npiv
            kmax = AGMG_MUMPS_497(cv_keep8(21),ncb)
            if (force_cand) then
              if (cv_keep(50) ==  0) then
                keep48_loc=0
              else
                keep48_loc=3
              endif
              if (cv_keep(48).EQ.5) keep48_loc = 5
               min_needed = AGMG_MUMPS_50(                                   &
     &             cv_slavef, keep48_loc,cv_keep8(21),                  &
     &             cv_keep(50),nfront,ncb)
               max_needed = AGMG_MUMPS_52(                                   &
     &             cv_slavef, keep48_loc,cv_keep8(21),                  &
     &             cv_keep(50),nfront,ncb)
               if(cand_strat.eq.1) then
                  more_than_needed = 0
               elseif (cand_strat.eq.2) then
                  if(work_type2_thislayer.gt.0.0D0) then
                  relative_weight=cv_ncostw(inode)/work_type2_thislayer
                  else
                     relative_weight = 0.0D0
                  endif
                  fraction=nint(relative_weight *                       &
     &                 dble(total_cand_layer))
                  more_than_needed=min(max(0,cv_slavef-1-min_needed),   &
     &                                 max(0,fraction-min_needed)    )
               elseif (cand_strat.eq.3) then
                  more_than_needed=cv_slavef-1-min_needed
               else
                  if(cv_lp.gt.0)                                        &
     &            write(cv_lp,*)'Unknown cand. strategy in ',subname
                  return
               endif
               total_nmb_cand=min(min_needed+more_than_needed,          &
     &              cv_slavef-1)
               total_nmb_cand=min(total_nmb_cand,max_needed)
            else
               total_nmb_cand=0
            endif
            cv_layer_p2node(layernmb)%t2_cand(j,cv_slavef+1)            &
     &                                 = total_nmb_cand
            if(cv_keep(50).eq.0) then
               flop1=dble(2*npiv)*dble(nfront)-                         &
     &              dble(npiv+nfront)*dble(npiv+1)
               flop1= dble(npiv)*flop1 +                                &
     &          dble(2 * npiv-npiv-1)*dble(npiv)/dble(2)+               &
     &          dble(npiv)*dble(npiv+1)*dble(2*npiv+1)/dble(3)
            else
               flop1=dble(npiv)*                                        &
     &         ( dble(npiv)*dble(npiv)+dble(npiv)-                      &
     &         dble(npiv*npiv+npiv+1) )+                                &
     &         (dble(npiv)*dble(npiv+1)*dble(2*npiv+1))/dble(6)
            endif
            cv_ncostw(inode)=flop1
            if(total_nmb_cand.gt.0) then
               nrow = dble(max(min(dble(ncb)/dble(total_nmb_cand),      &
     &                     dble(kmax)),                                 &
     &                 dble(ncb)/dble(cv_slavef-1)))
            elseif(cv_slavef.gt.1) then
               nrow = dble(max(dble(kmax),                              &
     &                         dble(ncb)/dble(cv_slavef-1)))
            else
               nrow = dble(ncb)
            endif
            if(cv_keep(50).eq.0) then
               flop1 = dble(npiv)*dble(nrow)+                           &
     &                 dble(nrow)*dble(npiv)*dble(2*nfront-npiv-1)
            else
               ncol= nfront
               flop1 = dble(npiv)*dble(nrow)*                           &
     &          (dble(2*ncol)-dble(nrow)-dble(npiv)+dble(1))
               workmaster = dble(npiv*npiv)*dble(npiv)/dble(3)
               if (workmaster.gt.flop1) flop1=workmaster
            endif
            cv_layer_p2node(layernmb)%t2_candcostw(j)=flop1
            if(cv_keep(50).eq.0) then
               cv_ncostm(inode)=dble(npiv)*dble(nfront)
            else
               cv_ncostm(inode)=dble(npiv)*dble(npiv)
            endif
            if(cv_keep(50).eq.0) then
               cv_layer_p2node(layernmb)%t2_candcostm(j)                &
     &                                  =dble(npiv)*dble(nrow)
            else
               cv_layer_p2node(layernmb)%t2_candcostm(j)                &
     &                                  =dble(npiv)*dble(nrow)
            endif
         end do
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_367
      subroutine AGMG_MUMPS_489(layernmb,nmb_thislayer,istat)
      implicit none
      integer,intent(in)::layernmb,nmb_thislayer
      integer,intent(out)::istat
      integer in,inode,j,jj,kmax,npiv,nfront,ncb,ncol,                  &
     &        total_nmb_cand,nmb_type2_thislayer,                       &
     &        total_cand_layer,npropmap,min_needed,                     &
     &        keep48_loc
      DOUBLE PRECISION flop1,work_type2_thislayer,                      &
     &        relative_weight,workmaster,nrow
      intrinsic count,max
      character (len=48):: subname
      integer AGMG_MUMPS_497, AGMG_MUMPS_50
      external AGMG_MUMPS_497, AGMG_MUMPS_50
      istat=-1
      subname='COSTS_LAYER_T2PM'
      if((cv_keep(24).ne.8).AND.(cv_keep(24).ne.10)                     &
     &    .AND.(cv_keep(24).ne.12).AND.(cv_keep(24).ne.14)              &
     &    .AND.(cv_keep(24).ne.16).AND.(cv_keep(24).ne.18)) then
          if(cv_lp.gt.0)                                                &
     &    write(cv_lp,*)'Error in ',subname,'. Wrong keep24'
         return
      endif
      nmb_type2_thislayer=cv_layer_p2node(layernmb)%nmb_t2s
      if (nmb_type2_thislayer.gt.0) then
         total_cand_layer=0
         work_type2_thislayer=0.0D0
         do j=1,nmb_type2_thislayer
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(j)
            work_type2_thislayer=work_type2_thislayer+cv_ncostw(inode)
            npropmap=0
            do jj=1,cv_slavef
               if( AGMG_MUMPS_481(inode,jj))                                 &
     &              npropmap=npropmap+1
            end do
            total_cand_layer=total_cand_layer+npropmap
         end do
         do j=1,nmb_type2_thislayer
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(j)
            nfront=cv_nfsiz(inode)
            npiv=0
            in=inode
            do while(in.gt.0)
               npiv=npiv+1
               in=cv_fils(in)
            end do
            ncb=nfront-npiv
            kmax = AGMG_MUMPS_497(cv_keep8(21),ncb)
            if(kmax.lt.1) then
               kmax = max(kmax,1)
            endif
            if (cv_keep(50) ==  0) then
              keep48_loc=0
            else
              keep48_loc=3
            endif
            if (cv_keep(48).EQ.5) keep48_loc = 5
            min_needed= AGMG_MUMPS_50                                        &
     &          (cv_slavef, keep48_loc,cv_keep8(21),                    &
     &           cv_keep(50),nfront,ncb)
            if(min_needed.lt.1) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname,'.NEG min_needed'
               return
            endif
            if ((cv_keep(24).eq.8).OR.(cv_keep(24).eq.14).OR.           &
     &          (cv_keep(24).eq.18)) then
               npropmap=0
               do jj=1,cv_slavef
                  if( AGMG_MUMPS_481(inode,jj))                              &
     &                 npropmap=npropmap+1
               end do
               total_nmb_cand=max(npropmap-1,min_needed)
            elseif(cv_keep(24).eq.10) then
               if(work_type2_thislayer.gt.0.0D0) then
                  relative_weight=cv_ncostw(inode)/work_type2_thislayer
               else
                  relative_weight = 0.0D0
               endif
               total_nmb_cand=nint(relative_weight *                    &
     &              dble(total_cand_layer))
               total_nmb_cand=max(total_nmb_cand-1,min_needed)
            elseif((cv_keep(24).eq.12).OR.(cv_keep(24).eq.16)) then
               if(layernmb.lt.cv_dist_L0_mixed_strat_bound) then
                  if(cv_mp.gt.0)then
                     write(cv_mp,*)'Strat', cv_keep(24),                &
     &                             ': use 8 on layer',layernmb
                  endif
                  npropmap=0
                  do jj=1,cv_slavef
                     if( AGMG_MUMPS_481(inode,jj))                           &
     &                    npropmap=npropmap+1
                  end do
                  total_nmb_cand=max(npropmap-1,min_needed)
               else
                  if(cv_mp.gt.0)then
                     write(cv_mp,*)'Strat', cv_keep(24),                &
     &                             ': use 10 on layer',layernmb
                  endif
                  if(work_type2_thislayer.gt.0.0D0) then
                  relative_weight=cv_ncostw(inode)/work_type2_thislayer
                  else
                     relative_weight = 0.0D0
                  endif
                  total_nmb_cand=nint(relative_weight *                 &
     &                 dble(total_cand_layer))
                  total_nmb_cand=max(total_nmb_cand-1,min_needed)
               endif
            else
               if(cv_lp.gt.0)                                           &
     &              write(cv_lp,*)'Unknown cand. strategy in ',subname
               return
            endif
            total_nmb_cand=max(total_nmb_cand,1)
            total_nmb_cand=min(total_nmb_cand,cv_slavef-1)
            total_nmb_cand=min(total_nmb_cand,ncb)
            cv_layer_p2node(layernmb)%t2_cand(j,cv_slavef+1)            &
     &                                 = total_nmb_cand
            if(cv_keep(50).eq.0) then
               flop1=dble(2*npiv)*dble(nfront)-                         &
     &              dble(npiv+nfront)*dble(npiv+1)
               flop1= dble(npiv)*flop1 +                                &
     &          dble(2 * npiv-npiv-1)*dble(npiv)/dble(2)+               &
     &          dble(npiv)*dble(npiv+1)*dble(2*npiv+1)/dble(3)
            else
               flop1=dble(npiv)*                                        &
     &         ( dble(npiv)*dble(npiv)+dble(npiv)-                      &
     &         dble(npiv*npiv+npiv+1) )+                                &
     &         (dble(npiv)*dble(npiv+1)*dble(2*npiv+1))/dble(6)
            endif
            cv_ncostw(inode)=flop1
            if(total_nmb_cand.gt.0) then
               nrow = dble(max(min(dble(ncb)/dble(total_nmb_cand),      &
     &                     dble(kmax)),                                 &
     &                 dble(ncb)/dble(cv_slavef-1)))
            elseif(cv_slavef.gt.1) then
               nrow = dble(max(dble(kmax),                              &
     &                         dble(ncb)/dble(cv_slavef-1)))
            else
               nrow = dble(ncb)
            endif
            if(cv_keep(50).eq.0) then
               flop1 = dble(npiv)*dble(nrow)+                           &
     &                 dble(nrow)*dble(npiv)*dble(2*nfront-npiv-1)
            else
               ncol= nfront
               flop1 = dble(npiv)*dble(nrow)*                           &
     &          (dble(2*ncol)-dble(nrow)-dble(npiv)+dble(1))
               workmaster = dble(npiv*npiv)*dble(npiv)/dble(3)
               if (workmaster.gt.flop1) flop1=workmaster
            endif
            cv_layer_p2node(layernmb)%t2_candcostw(j)=flop1
            if(cv_keep(50).eq.0) then
               cv_ncostm(inode)=dble(npiv)*dble(nfront)
            else
               cv_ncostm(inode)=dble(npiv)*dble(npiv)
            endif
            if(cv_keep(50).eq.0) then
               cv_layer_p2node(layernmb)%t2_candcostm(j)                &
     &                                  =dble(npiv)*dble(nrow)
            else
               cv_layer_p2node(layernmb)%t2_candcostm(j)                &
     &                                  =dble(npiv)*dble(nrow)
            endif
         end do
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_489
      subroutine AGMG_MUMPS_527(                                             &
     &     layernmb,thislayer,nmb_thislayer,                            &
     &     istat)
      implicit none
      integer,intent(in)::layernmb,thislayer(cv_maxnodenmb),            &
     &     nmb_thislayer
      integer,intent(out)::istat
      integer i,k1,k2,k3,ierr,inode,in,nfront,npiv,                     &
     &     npiv_son,ison,ifather,                                       &
     &     nmb_type2
      logical doit
      character (len=48):: err_rep,subname
      istat=-1
      subname='NEWSPLITTING'
      if((layernmb.lt.0).or.(layernmb.gt.cv_maxlayer)) return
      if (cv_slavef.eq.1) then
         return
      endif
      do i=1,nmb_thislayer
         ierr=0
         inode=thislayer(i)
         err_rep='GET_SPLIT_INKPART'
         call AGMG_MUMPS_525(inode,                                          &
     &        doit,npiv,nfront,k1,k3,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Error reported by ',                    &
     &           err_rep,' in ',subname
            istat =ierr
            return
         endif
         if(.NOT.doit) then
            cycle
         endif
         if(k3.eq.1) then
            cycle
         endif
         err_rep='GET_MEMSPLIT_INKPART'
         call AGMG_MUMPS_526(inode,                                          &
     &        doit,npiv,nfront,k2,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Error reported by ',                    &
     &           err_rep,' in ',subname
            istat =ierr
            return
         endif
         k1 = max(k1,k2)
         k1 = min(k1,k3)
         if( k1 .eq. 1) cycle
         err_rep='SPLITNODE_INKPART'
         call AGMG_MUMPS_529(inode,nfront,npiv,k1,                           &
     &        ison,ifather,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,                 &
     &           ' in ',subname
            istat = ierr
            return
         endif
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_527
      subroutine AGMG_MUMPS_525(inode,                                       &
     &     doit,npiv,nfront,k1,k3,istat)
      implicit none
      integer,intent(in)::inode
      logical,intent(out)::doit
      integer,intent(out),OPTIONAL::npiv,nfront,istat
      integer,intent(out),OPTIONAL::k1,k3
      integer in,nfr_ifather,npiv_ifather,npiv2,nfront2,npiv_son2
      integer npropmap,j,ncb,kmax,keep48_loc,nslaves_max,               &
     &     nslaves_estim,strat,kk
      DOUBLE PRECISION wk_master,wk_slave,wk_master2,wk_slave2
      integer AGMG_MUMPS_497,                                                &
     &     AGMG_MUMPS_52,                                                    &
     &     AGMG_MUMPS_50
      external AGMG_MUMPS_497
      external AGMG_MUMPS_52
      external AGMG_MUMPS_50
      doit=.FALSE.
      if(present(npiv)) npiv=cv_invalid
      if(present(nfront)) nfront=cv_invalid
      if(present(k1)) k1=1
      if(present(k3)) k3 =1
      if(present(istat)) istat=-1
      doit=.TRUE.
      if (cv_nodetype(inode) .gt. 0) then
         doit=.FALSE.
         istat = 0
         return
      endif
      if ( (cv_frere(inode).eq.0) ) then
         doit=.FALSE.
         istat = 0
         return
      endif
      nfront=cv_nfsiz(inode)
      in=inode
      npiv=0
      do while (in.gt.0)
         in=cv_fils(in)
         npiv=npiv+1
      end do
      npiv_son2 = max(npiv/2,1)
      if(npiv.le.npiv_son2) then
         doit=.FALSE.
         istat = 0
         return
      endif
      if( .not. AGMG_MUMPS_359(nfront,npiv_son2) )then
         doit=.FALSE.
         istat = 0
         return
      endif
      in = inode
      npropmap=0
      do j=1,cv_slavef
         if( AGMG_MUMPS_481(in,j)) then
            npropmap=npropmap+1
         endif
      end do
      ncb = nfront - npiv
      kmax = AGMG_MUMPS_497(cv_keep8(21),ncb)
      if (cv_keep(50) ==  0) then
         keep48_loc=0
      else
         keep48_loc=3
      endif
      if (cv_keep(48).EQ.5) keep48_loc = 5
      if(npropmap .gt. cv_keep(83)) then
         nslaves_max   = AGMG_MUMPS_52(                                      &
     &        cv_slavef, keep48_loc, cv_keep8(21),                      &
     &        cv_keep(50), nfront, ncb )
         nslaves_estim = min(npropmap,nslaves_max)
         nslaves_estim = max(nslaves_estim,1)
      else
         nslaves_max   = AGMG_MUMPS_52(                                      &
     &        cv_slavef, keep48_loc, cv_keep8(21),                      &
     &        cv_keep(50), nfront, ncb )
         nslaves_estim = AGMG_MUMPS_50(                                      &
     &     cv_slavef, keep48_loc,cv_keep8(21),                          &
     &     cv_keep(50), nfront, ncb )
         nslaves_estim = max(nslaves_estim,1)
         nslaves_estim = min(nslaves_estim,nslaves_max)
      endif
      if (cv_keep(50).eq.0) then
         wk_master = (dble(2)/dble(3))*                                 &
     &        dble(npiv)*dble(npiv)*dble(npiv)+                         &
     &        dble(npiv)*dble(npiv)*dble(nfront-npiv)
      else
         wk_master = dble(npiv)*dble(npiv)*dble(npiv)/dble(3)
      end if
      strat = cv_keep(62)
      doit = .TRUE.
      k1 = cv_keep(82)
      k3 = cv_keep(82)
      do kk=1,cv_keep(82)-1
         npiv2 = npiv/kk
         if(npiv2 .eq. 0) then
            k1 = max(1,kk-1)
            exit
         endif
         wk_master2 = wk_master / dble(kk)
         nfront2 = nfront-npiv+npiv2
         if (cv_keep(50).eq.0) then
            wk_slave2  = ( dble(npiv2)*dble(nfront-npiv2) *             &
     &           dble(2*nfront-npiv2) ) / dble(nslaves_estim)
         else
            wk_slave2 =                                                 &
     &           ( dble(npiv2)*dble(nfront2-npiv2)*dble(nfront2) )      &
     &           /   dble(nslaves_estim)
         endif
         if(wk_master2.le.                                              &
     &        (1.0D0 +dble(kk*strat)/dble(100))*wk_slave2) then
            k1 = kk
            exit
         endif
      enddo
      do kk=1,cv_keep(82)-1
         npiv2 = npiv/kk
         if(npiv2 .eq. 0) then
            k3 = max(1,kk-1)
            exit
         endif
         wk_master2 = wk_master / dble(kk)
         nfront2 = nfront
         if (cv_keep(50).eq.0) then
            wk_slave2  = ( dble(npiv2)*dble(nfront-npiv2) *             &
     &           dble(2*nfront-npiv2) ) / dble(nslaves_estim)
         else
            wk_slave2 =                                                 &
     &           ( dble(npiv2)*dble(nfront2-npiv2)*dble(nfront2) )      &
     &           /   dble(nslaves_estim)
         endif
         if(wk_master2.le.wk_slave2) then
            k3 = kk
            exit
         endif
      enddo
      if(k3 .lt. k1) then
         k3 = k1
      endif
      if(present(istat)) istat=0
      return
      end subroutine AGMG_MUMPS_525
      subroutine AGMG_MUMPS_526(inode,                                       &
     &     doit,npiv,nfront,k2,istat)
      implicit none
      integer,intent(in)::inode
      logical,intent(out)::doit
      integer,intent(out),OPTIONAL::npiv,nfront,istat
      integer,intent(out),OPTIONAL::k2
      integer in,nfr_ifather,npiv_ifather,npiv2,nfront2,npiv_son2
      integer npropmap,j,ncb,kmax,keep48_loc,nslaves_max,               &
     &     nslaves_estim,kk
      DOUBLE PRECISION mem_master
      doit=.FALSE.
      if(present(npiv)) npiv=cv_invalid
      if(present(nfront)) nfront=cv_invalid
      if(present(k2)) k2=1
      if(present(istat)) istat=-1
      doit=.TRUE.
      nfront=cv_nfsiz(inode)
      in=inode
      npiv=0
      do while (in.gt.0)
         in=cv_fils(in)
         npiv=npiv+1
      end do
      doit = .TRUE.
      k2 = cv_keep(82)
      do kk=1,cv_keep(82)-1
         npiv2 = npiv/kk
         if(npiv2 .eq. 0) then
            k2 = max(1,kk-1)
            exit
         endif
         if (cv_keep(50).eq.0) then
            mem_master  = dble(npiv2)*dble(nfront)
         else
            mem_master  = dble(npiv2)*dble(npiv2)
         endif
         if(mem_master.le.                                              &
     &        (dble(cv_mem_strat)/dble(100))*cv_stack_peak) then
            k2 = kk
            exit
         endif
      enddo
      if(present(istat)) istat=0
      return
      end subroutine AGMG_MUMPS_526
      subroutine AGMG_MUMPS_529(inode,nfront,npiv,k,                         &
     &     ison,ifather,istat)
      implicit none
      integer, intent(in)::nfront,npiv
      integer, intent(in):: k
      integer inode
      integer,intent(out)::ison,ifather
      integer, intent(out)::istat
      integer i,lev,in,in_son,in_father,in_grandpa,                     &
     &     npiv_son,nfrontk,npivk,d1,f1,e1,dk,fk,next_father
      DOUBLE PRECISION:: ncostm,ncostw,ncostm_ison,ncostw_ison,         &
     &                   ncostm_ifather,ncostw_ifather
      character (len=48):: subname
      istat=-1
      subname='SPLITNODE_INKPART'
      ison=-1
      ifather=-1
      ncostw=cv_ncostw(inode)
      ncostm=cv_ncostm(inode)
      nfrontk = nfront
      npivk = npiv
      npiv_son = npiv/k
      cv_keep(2)=max(cv_keep(2),nfront-npiv_son)
      d1 = inode
      f1 = d1
      e1 = cv_frere(d1)
      do i=1,npiv_son-1
         f1 = cv_fils(f1)
      enddo
      ison = d1
      in_son = f1
      next_father = cv_fils(in_son)
      call AGMG_MUMPS_418(npiv_son,nfrontk,                                  &
     &     ncostw_ison,ncostm_ison)
      cv_ncostw(ison)=ncostw_ison
      cv_ncostm(ison)=ncostm_ison
      if(associated(cv_tcostw)) cv_tcostw(ison) = cv_tcostw(inode)      &
     &     -ncostw +cv_ncostw(ison)
      if(associated(cv_tcostm)) cv_tcostm(ison) = cv_tcostm(inode)      &
     &     -ncostm +cv_ncostm(ison)
      do lev = 1,k-1
         ifather = next_father
         in_father = ifather
         if(lev .eq. k-1) then
            do while (cv_fils(in_father).gt.0)
               in_father=cv_fils(in_father)
            end do
         else
            do i=1,npiv_son-1
               in_father=cv_fils(in_father)
            enddo
         endif
         cv_frere(ison)=-ifather
         next_father = cv_fils(in_father)
         cv_fils(in_father)=-ison
         cv_nfsiz(ison)=nfrontk
         cv_nfsiz(ifather)=nfrontk-npiv_son
         cv_ne(ifather)=1
         cv_keep(61)=cv_keep(61)+1
         call AGMG_MUMPS_418(npiv_son,nfrontk-npiv_son,                      &
     &        ncostw_ifather,ncostm_ifather)
         cv_ncostw(ifather)=ncostw_ifather
         cv_ncostm(ifather)=ncostm_ifather
         if(associated(cv_tcostw))                                      &
     &        cv_tcostw(ifather) = cv_tcostw(ison)+cv_ncostw(ifather)
         if(associated(cv_tcostm))                                      &
     &        cv_tcostm(ifather) = cv_tcostm(ison)+cv_ncostm(ifather)
         cv_total_split=cv_total_split+1
         if(lev .gt. 1) then
            call AGMG_MUMPS_437(inode,ison,ierr)
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &              write(cv_lp,*)'PROPMAP4SPLIT error in ',subname
               istat = ierr
               return
            endif
         endif
         if( AGMG_MUMPS_359(nfrontk-npiv_son,npiv_son) ) then
            cv_nodetype(ifather) = 2
         else
            cv_nodetype(ifather) = 1
         endif
         nfrontk = nfrontk-npiv_son
         npivk = npivk - npiv_son
         ison = ifather
         in_son = in_father
      enddo
      dk = ifather
      fk = in_father
      if( AGMG_MUMPS_359(nfrontk,npivk) ) then
         cv_nodetype(dk) = 2
      else
         cv_nodetype(dk) = 1
      endif
      call AGMG_MUMPS_418(npivk,nfrontk,                                     &
     &     ncostw_ifather,ncostm_ifather)
      cv_ncostw(dk)=ncostw_ifather
      cv_ncostm(dk)=ncostm_ifather
      if(associated(cv_tcostw))                                         &
     &     cv_tcostw(dk) = cv_tcostw(ison)+cv_ncostw(dk)
      if(associated(cv_tcostm))                                         &
     &     cv_tcostm(dk) = cv_tcostm(ison)+cv_ncostm(dk)
      cv_fils(f1) = next_father
      cv_frere(dk) = e1
      in = e1
      do while (in.gt.0)
         in=cv_frere(in)
      end do
      in = -in
      do while(cv_fils(in).gt.0)
         in=cv_fils(in)
      end do
      in_grandpa = in
      if(cv_fils(in_grandpa).eq.-d1) then
         cv_fils(in_grandpa)=-dk
      else
         in=-cv_fils(in_grandpa)
         do while(cv_frere(in) .ne. d1)
            in=cv_frere(in)
         end do
         cv_frere(in) = dk
      end if
      ison = dk
      do lev=1,k
         do while (cv_fils(ison).gt.0)
            ison=cv_fils(ison)
         end do
         ison = -cv_fils(ison)
      enddo
      call AGMG_MUMPS_437(inode,dk,ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'PROPMAP4SPLIT error in ',subname
         istat = ierr
         return
      endif
      cv_nsteps = cv_nsteps + k-1
      istat = 0
      return
      end subroutine AGMG_MUMPS_529
      subroutine AGMG_MUMPS_370(                                             &
     &                              layernmb,thislayer,nmb_thislayer,   &
     &                              istat)
      implicit none
      integer,intent(in)::layernmb,thislayer(cv_maxnodenmb),            &
     &                    nmb_thislayer
      integer,intent(out)::istat
      integer i,ierr,inode,in,nfront,npiv,npiv_son,ison,ifather,        &
     &        nmb_type2
      logical doit
      character (len=48):: err_rep,subname
      istat=-1
      subname='DO_SPLITTING'
      if((layernmb.lt.0).or.(layernmb.gt.cv_maxlayer)) return
      if ((cv_slavef.eq.1).OR.(cv_slavef.lt.nmb_thislayer)) then
         istat=0
         return
      endif
      nmb_type2=0
      do i=1,nmb_thislayer
         inode=thislayer(i)
         nfront=cv_nfsiz(inode)
         in=inode
         npiv=0
         do while (in.gt.0)
            in=cv_fils(in)
            npiv=npiv+1
         end do
         if( AGMG_MUMPS_359(nfront,npiv))                                    &
     &                        nmb_type2=nmb_type2+1
      end do
      do i=1,nmb_thislayer
         ierr=0
         inode=thislayer(i)
         err_rep='SPLIT2HALVES'
         call AGMG_MUMPS_400(inode,                                          &
     &                              doit,npiv,nfront,npiv_son,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat =ierr
            return
         endif
         if(.NOT.doit) cycle
         err_rep='SPLITNODE'
         call AGMG_MUMPS_401(inode,nfront,npiv,npiv_son,                     &
     &                           ison,ifather,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat = ierr
            return
         endif
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_370
      subroutine AGMG_MUMPS_371(istat)
      implicit none
      integer, intent(out)::istat
      integer i,in,inode
      character (len=48):: subname
      istat=-1
      subname='ENCODE_PROCNODE'
      do i=1,cv_nbsa
         inode=cv_ssarbr(i)
         cv_nodetype(inode)=0
         in=cv_fils(inode)
         do while (in>0)
            in=cv_fils(in)
         end do
         in=-in
         do while(in.gt.0)
            call AGMG_MUMPS_406(in)
            in=cv_frere(in)
         enddo
      enddo
      do i=1,cv_n
         if (cv_frere(i).lt.cv_n+1) then
            if(cv_nodetype(i).eq.cv_invalid) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname
               return
            endif
            if (i.eq.cv_keep(38)) then
               cv_nodetype(i)=3
            endif
            cv_procnode(i)=(cv_nodetype(i)-1)*cv_slavef+cv_procnode(i)
            in=cv_fils(i)
            do while (in>0)
               cv_procnode(in)=cv_procnode(i)
               in=cv_fils(in)
            end do
         end if
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_371
      subroutine AGMG_MUMPS_372(ifather,istat)
      implicit none
      integer,intent(in)::ifather
      integer,intent(out)::istat
      integer in,son,ierr,allocok,oldl0end
      logical father_has_sons,fathfound
      character (len=48):: subname
      istat=-1
      subname='FATHSON_REPLACE'
      father_has_sons=.TRUE.
      in=ifather
      do while (in.gt.0)
         in=cv_fils(in)
      end do
      if(in.eq.0) then
         cv_nodelayer(ifather)=1
         father_has_sons=.FALSE.
      end if
      if(cv_layerl0_end-cv_layerl0_start.gt.0) then
         cv_layerl0_start= cv_layerl0_start+1
      elseif(father_has_sons) then
         cv_layerl0_start= cv_layerl0_start+1
      else
         istat=1
         cv_nodelayer(ifather)=0
         return
      endif
      cv_nbsa=cv_nbsa-1
      oldl0end = cv_layerl0_end
      if (father_has_sons) then
         son=-in
         son=-in
   10    continue
         cv_layerl0_end=cv_layerl0_end+1
         if (cv_tcostw(son).GT.mincostw)                                &
     &           layerL0_endforarrangeL0 = layerL0_endforarrangeL0+1
         cv_layerl0_array(cv_layerl0_end)=son
         cv_layerl0_sorted_costw(cv_layerl0_end)=cv_tcostw(son)
         cv_nbsa=cv_nbsa+1
         if((cv_frere(son).gt.0).and.(cv_frere(son).lt.cv_n+1)) then
            son=cv_frere(son)
            goto 10
         end if
      endif
         cv_costw_layer0=cv_costw_layer0 - cv_ncostw(ifather)
         cv_costm_layer0=cv_costm_layer0 - cv_ncostm(ifather)
         cv_costw_upper=cv_costw_upper + cv_ncostw(ifather)
         cv_costm_upper=cv_costm_upper + cv_ncostm(ifather)
         if(cv_layerl0_end.gt.oldl0end) then
            call AGMG_MUMPS_459(cv_layerl0_end-oldl0end,                     &
     &            cv_layerl0_array(oldl0end+1:cv_layerl0_end),          &
     &     cv_layerl0_sorted_costw(oldl0end+1:cv_layerl0_end))
           call AGMG_MUMPS_516(                                              &
     &        cv_layerl0_start,oldl0end,oldl0end-cv_layerl0_start+1,    &
     &        oldl0end+1,cv_layerl0_end,cv_layerl0_end-oldl0end,        &
     &        cv_layerl0_array,                                         &
     &        cv_layerl0_sorted_costw)
         endif
      istat=0
      return
      end subroutine AGMG_MUMPS_372
      subroutine AGMG_MUMPS_374(inode,map_strat,work,mem,                    &
     &                       workload,memused,proc,istat,respect_prop)
!DEC$ NOOPTIMIZE
      implicit none
      integer, intent(in)::inode,map_strat
      DOUBLE PRECISION,intent(in)::work,mem
      DOUBLE PRECISION,dimension(cv_slavef),intent(inout)::workload,    &
     &                                                     memused
      integer,intent(out):: proc,istat
      logical,intent(in),OPTIONAL::respect_prop
      integer i
      logical respect_proportional
      intrinsic huge
      DOUBLE PRECISION dummy
      character (len=48):: subname
      istat=-1
      respect_proportional=.FALSE.
      if(present(respect_prop)) respect_proportional=respect_prop
      subname='FIND_BEST_PROC'
      proc=-1
      if((map_strat.ne.cv_equilib_flops).and.                           &
     &   (map_strat.ne.cv_equilib_mem)) return
      dummy=huge(dummy)
      do i=cv_slavef,1,-1
         if (                                                           &
     &       ((.NOT.respect_proportional)                               &
     &        .OR.                                                      &
     &        (AGMG_MUMPS_481(inode,i).AND.respect_proportional))            &
     &      .AND.                                                       &
     &       (((workload(i).lt.dummy).AND.                              &
     &                         (map_strat.eq.cv_equilib_flops))         &
     &        .OR.                                                      &
     &       ((memused(i).lt.dummy).AND.                                &
     &                         (map_strat.eq.cv_equilib_mem))))then
            if((.not.cv_constr_work).or.                                &
     &         (workload(i)+work.lt.cv_proc_maxwork(i))) then
               if((.not.cv_constr_mem).or.                              &
     &            (memused(i)+mem.lt.cv_proc_maxmem(i))) then
                  proc=i
                  if(map_strat.eq.cv_equilib_flops) then
                     dummy=workload(i)
                  elseif(map_strat.eq.cv_equilib_mem) then
                     dummy=memused(i)
                  endif
               end if
            end if
         end if
      end do
      if (proc.ne.-1) then
         workload(proc)=workload(proc)+work
         memused(proc)=memused(proc)+mem
         istat=0
      end if
      return
      end subroutine AGMG_MUMPS_374
      subroutine AGMG_MUMPS_375(map_strat,istat)
      implicit none
      integer,intent(in)::map_strat
      integer,intent(out)::istat
      DOUBLE PRECISION aux_dbl,current_max,temp_max
      integer i,j,aux_int,inode,in,nfront,npiv,                         &
     &     nslaves_estim,strat,kmax,ncb,npropmap, blsize,               &
     &     nslaves_max, keep48_loc,current_max_node
      DOUBLE PRECISION wk_slave, wk_master
      DOUBLE PRECISION,DIMENSION(:), POINTER ::  costs
      logical doit
      character (len=48):: subname
      integer AGMG_MUMPS_497,                                                &
     &        AGMG_MUMPS_52
      external AGMG_MUMPS_497,                                               &
     &        AGMG_MUMPS_52
      istat=-1
      current_max_node=-9999
      current_max=0.0D0
      subname='FIND_POTSPLIT'
      if(map_strat.eq.cv_equilib_flops) then
         costs=>cv_ncostw
      elseif(map_strat.eq.cv_equilib_mem) then
         costs=>cv_ncostm
      endif
      if((cv_slavef.eq.1).OR.(cv_keep(62).lt.1))then
         cv_potsplit(1:cv_maxcut)=cv_invalid
         istat=0
         return
      end if
      current_max=huge(current_max);
      current_max_node=cv_n+1
      aux_int=1
      do i=1,min(cv_maxcut,cv_n)
         temp_max=0.0D0
         do j=1,cv_n
            if(costs(j).ge.temp_max)then
               if(((costs(j).eq.current_max).and.                       &
     &              (j.lt.current_max_node)).or.                        &
     &              (costs(j).lt.current_max))then
                  temp_max=costs(j)
                  inode=j
               endif
            endif
         end do
         current_max=costs(inode)
         current_max_node=inode
         if((inode.lt.0).OR.(inode.gt.cv_n)) cycle
         doit = .TRUE.
         if (cv_frere(inode).eq.0) cycle
         nfront = cv_nfsiz(inode)
         in = inode
         npropmap=0
         do j=1,cv_slavef
            if( AGMG_MUMPS_481(in,j)) then
               npropmap=npropmap+1
            endif
         end do
         npiv = 0
         do while(in>0)
            in = cv_fils(in)
            npiv = npiv + 1
         end do
         ncb = nfront - npiv
         kmax = AGMG_MUMPS_497(cv_keep8(21),ncb)
         if (cv_keep(50) ==  0) then
           keep48_loc=0
         else
           keep48_loc=3
         endif
         if (cv_keep(48).EQ.5) keep48_loc = 5
         nslaves_max   = AGMG_MUMPS_52(                                      &
     &        cv_slavef, keep48_loc, cv_keep8(21),                      &
     &        cv_keep(50), nfront, ncb )
         nslaves_estim = min(npropmap,nslaves_max)
         nslaves_estim = max(nslaves_estim,1)
         if (cv_keep(50).eq.0) then
            wk_master = dble(2)/dble(3)*                                &
     &                  dble(npiv)*dble(npiv)*dble(npiv)+               &
     &                  dble(npiv)*dble(npiv)*dble(nfront-npiv)
            wk_slave  = dble(npiv)*dble(nfront-npiv) *                  &
     &                  dble(2*nfront-npiv)/dble(nslaves_estim)
         else
            wk_master = dble(npiv)*dble(npiv)*dble(npiv)/dble(3)
            wk_slave  =                                                 &
     &           ( dble(npiv)*dble(nfront-npiv)*dble(nfront) )          &
     &            /   dble(nslaves_estim)
         end if
         strat = cv_keep(62)
         if(wk_master.le.                                               &
     &     ( dble(1)                                                    &
     &      +dble(strat)/dble(100) )                                    &
     &      *wk_slave) doit=.false.
         if ((cv_depth(inode).le.cv_maxdepth).and.doit) then
            cv_potsplit(aux_int)=inode
            aux_int=aux_int+1
         endif
      enddo
      cv_potsplit(aux_int:cv_maxcut)=cv_invalid
      istat=0
      return
      end subroutine AGMG_MUMPS_375
      subroutine AGMG_MUMPS_376(nmb,                                         &
     &                                   thislayer,nmb_thislayer,istat)
      implicit none
      integer, intent(in)::nmb
      integer,intent(out)::thislayer(cv_maxnodenmb),nmb_thislayer,istat
      integer i
      character (len=48):: subname
      istat=-1
      subname='FIND_THISLAYER'
      thislayer=0
      nmb_thislayer=0
      if((nmb.lt.0).or.(nmb.gt.cv_maxlayer)) return
      do i=1,cv_n
         if(cv_nodelayer(i).eq.nmb) then
            nmb_thislayer=nmb_thislayer+1
            if(nmb_thislayer.gt.cv_maxnodenmb) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Problem with nmb_thislayer in ',subname
               return
            endif
            thislayer(nmb_thislayer)=i
         end if
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_376
      subroutine AGMG_MUMPS_377(startlayer,thislayer,                        &
     &                 nmb_thislayer,cont,istat)
      implicit none
      integer,intent(in)::startlayer,nmb_thislayer
      integer,intent(in)::thislayer(cv_maxnodenmb)
      logical,intent(inout)::cont
      integer,intent(out)::istat
      integer :: visited
      integer il,i,current,in,ifather
      logical father_valid,upper_layer_exists
      character (len=48):: subname
      istat=-1
      subname='HIGHER_LAYER'
      if(.NOT.cont) return
      if(startlayer.lt.1) return
      current=startlayer-1
      visited = -current-1
         upper_layer_exists=.FALSE.
         if (current.eq.0) then
          do i=1,cv_n
            if (cv_nodelayer(i).ne.current) then
               if(cv_nodelayer(i).eq.1) then
                  upper_layer_exists=.TRUE.
                  exit
               endif
            endif
          enddo
         endif
         do il=1,nmb_thislayer
            i = thislayer(il)
            if (cv_nodelayer(i).lt.current) cycle
            in=i
            if(cv_frere(in).eq.0) cycle
            cv_nodelayer (in) = visited
               father_valid=.TRUE.
            do while(cv_frere(in).gt.0)
               if (cv_nodelayer(cv_frere(in)).gt.current) then
                  father_valid=.FALSE.
                  in = cv_frere(in)
                  cycle
               endif
               if (cv_nodelayer(cv_frere(in)).eq.visited) exit
               in=cv_frere(in)
               if (cv_nodelayer(in).eq.current)  then
                    cv_nodelayer(in) = visited
               endif
            end do
            if (.not.father_valid .or. cv_frere(in).gt.0) then
                    cycle
            endif
            ifather=-cv_frere(in)
            if(cv_nodelayer(ifather).eq.current+1) then
               cycle
            endif
            in=ifather
            do while (cv_fils(in).gt.0)
               in=cv_fils(in)
            end do
            in=-cv_fils(in)
            if(cv_nodelayer(in).gt.current) then
               father_valid=.FALSE.
            else
               father_valid=.TRUE.
               do while(cv_frere(in).gt.0)
                in=cv_frere(in)
                if(cv_nodelayer(in).gt.current) then
                       father_valid=.FALSE.
                       exit
                endif
                if(cv_nodelayer(in).eq.visited) then
                    exit
                endif
               end do
            endif
            if(father_valid) then
               cv_nodelayer(ifather)=current+1
               upper_layer_exists=.TRUE.
            end if
         end do
         if (upper_layer_exists) then
             current=current+1
             cv_maxlayer=current
             cont=.TRUE.
         else
             cv_maxlayer=current
             cont=.FALSE.
         endif
         do il=1,nmb_thislayer
            i = thislayer(il)
           if (cv_nodelayer(i).eq.visited) cv_nodelayer(i) = -visited-1
         enddo
      istat=0
      return
      end subroutine AGMG_MUMPS_377
      subroutine AGMG_MUMPS_478(n,slavef,                                    &
     &                    frere,fils,nfsiz,ne,keep,KEEP8,icntl,info,    &
     &                    procnode,ssarbr,peak,istat                    &
     &     )
      implicit none
      integer, intent(in)::n,slavef
      integer, intent(in), TARGET:: frere(n),fils(n),nfsiz(n),ne(n),    &
     &  keep(500),icntl(40),info(40),                                   &
     &  procnode(n),ssarbr(n)
      INTEGER*8, intent(in), TARGET:: KEEP8(150)
      integer,intent(out)::istat
      integer i,allocok,rest
      DOUBLE PRECISION peak
      character (len=48):: subname
      intrinsic bit_size,min,max
      istat=-1
      nullify(cv_frere,cv_fils,cv_nfsiz,cv_ne,cv_keep,cv_keep8,         &
     &        cv_icntl,cv_info,cv_procnode,cv_ssarbr)
      nullify(cv_ncostw,cv_tcostw,cv_ncostm,cv_tcostm,                  &
     &        cv_nodelayer,cv_nodetype,cv_depth,cv_potsplit,            &
     &        cv_layerworkload,cv_layermemused,cv_prop_map)
      subname='INITPART1'
      cv_n=n
      cv_slavef=slavef
      cv_stack_peak = peak
      cv_mem_strat = max((300 / cv_slavef),1)
      cv_keep=>keep
      cv_keep8=>KEEP8
      if(cv_keep(82) .lt. 0) then
         write(cv_lp,*)                                                 &
     &        'warning in mumps_static_mapping : keep(82) reset to 0'
         cv_keep(82) = 0
      endif
      if(cv_keep(83) .lt. 0) then
         write(cv_lp,*)                                                 &
     &        'warning in mumps_static_mapping : keep(83) reset to 0'
         cv_keep(83) = 0
      endif
      if(cv_keep(82) .gt. 0) then
         cv_maxcut=-99999
      else
         cv_maxcut =min(slavef,n/2)
      endif
      if(slavef.gt.1) then
         cv_mixed_strat_bound = max(cv_keep(78),1)
         cv_maxdepth = slavef
      else
         cv_maxdepth = 0
         cv_mixed_strat_bound=0
      endif
      cv_bitsize_of_int = bit_size(n)
      if(cv_bitsize_of_int.le.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Problem with bit size in ',subname
         return
      endif
      rest = mod(cv_slavef,cv_bitsize_of_int)
      if (rest.eq.0) then
         cv_size_ind_proc = cv_slavef / cv_bitsize_of_int
      else
         cv_size_ind_proc = cv_slavef / cv_bitsize_of_int + 1
      endif
      allocate(cv_ncostw(n),cv_tcostw(n),cv_ncostm(n),cv_tcostm(n),     &
     &       cv_nodelayer(n),cv_nodetype(n),cv_depth(n),                &
     &       cv_layerworkload(slavef),cv_layermemused(slavef),          &
     &       cv_prop_map(n),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 8*n+2*cv_slavef
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'memory allocation error in ',subname
         return
      end if
      if(cv_keep(82) .eq. 0) then
         allocate(cv_potsplit(cv_maxcut),STAT=allocok)
         if (allocok.gt.0) then
            cv_info(1) = cv_error_memalloc
            cv_info(2) = cv_maxcut
            istat = cv_error_memalloc
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'memory allocation error in ',subname
            return
         end if
      endif
      cv_frere=>frere
      cv_fils=>fils
      cv_nfsiz=>nfsiz
      cv_ne=>ne
      cv_icntl=>icntl
      cv_info=>info
      cv_procnode=>procnode
      cv_ssarbr=>ssarbr
      cv_ssarbr=0
      cv_nodetype=cv_invalid
      cv_nsteps=keep(28)
      if((keep(28).gt.n).OR.(keep(28).lt.0)) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'problem with nsteps in ',subname
         return
      end if
      cv_costw_upper=0.0D0
      cv_costm_upper=0.0D0
      cv_costw_layer0=0.0D0
      cv_costm_layer0=0.0D0
      cv_costw_total=0.0D0
      cv_costm_total=0.0D0
      cv_nodelayer=n+2
      cv_depth=cv_invalid
      if(cv_keep(82) .eq. 0) then
         cv_potsplit=cv_invalid
      endif
      cv_l0wthresh=0.0D0
      cv_splitthresh=0.45D0
      cv_relax=dble(1) + dble(max(0,keep(68)))/dble(100)
      cv_maxlayer=0
      if( cv_keep(82) .gt. 0) then
         cv_maxnsteps= cv_nsteps+1
      else
         cv_maxnsteps=cv_nsteps+cv_maxcut+1
      endif
      cv_layerworkload=dble(0)
      cv_layermemused=dble(0)
      cv_total_amalg=0
      cv_total_split=0
      cv_last_splitting%new_ison=cv_invalid
      cv_last_splitting%new_ifather=cv_invalid
      cv_last_splitting%old_keep2=cv_invalid
      cv_last_splitting%ncostw_oldinode=cv_d_invalid
      cv_last_splitting%ncostm_oldinode=cv_d_invalid
      cv_last_splitting%tcostw_oldinode=cv_d_invalid
      cv_last_splitting%tcostm_oldinode=cv_d_invalid
      do i=1,cv_n
         nullify(cv_prop_map(i)%ind_proc)
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_478
      subroutine AGMG_MUMPS_479(istat)
      implicit none
      integer,intent(out)::istat
      integer i,allocok,inode,in,inoderoot,ierr
      character (len=48):: subname
      type(nodelist),pointer::nodeptr
      istat=-1
      subname='INITPART2'
      if(associated(cv_layerl0_array))deallocate(cv_layerl0_array)
      if(associated(cv_layerl0_sorted_costw))                           &
     &           deallocate(cv_layerl0_sorted_costw)
      deallocate(cv_depth,cv_tcostw,cv_tcostm,STAT=ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      if(cv_maxnsteps.lt.1) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'problem with maxnsteps in ',subname
         return
      end if
      cv_maxnodenmb=cv_maxnsteps
      do i=1,cv_nbsa
         inode=cv_ssarbr(i)
         inoderoot=inode
  300    continue
         in = inode
         do while (in.ne.0)
            inode = in
            do while (in.gt.0)
            in = cv_fils(in)
            end do
            if (in.lt.0) in=-in
         end do
  100    continue
         if (inode.ne.inoderoot) then
            cv_maxnodenmb=cv_maxnodenmb-1
            in = cv_frere(inode)
            inode = abs(in)
            if (in.lt.0) then
               go to 100
            else
               go to 300
            end if
         end if
      end do
      if(cv_keep(82) .gt. 0) then
         cv_maxcut = min((cv_keep(82)-1)*cv_maxnodenmb,cv_n)
         cv_maxnsteps = min(cv_maxnsteps+cv_maxcut,cv_n)
         cv_maxnodenmb = cv_maxnsteps
      endif
      nullify(cv_layer_p2node)
      if(cv_maxnodenmb.lt.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'problem with maxnodenmb in ',subname
         return
      elseif(cv_maxnodenmb.lt.1) then
         cv_maxnodenmb = 1
      end if
      allocate(cv_layer_p2node(cv_maxnodenmb),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = cv_maxnodenmb
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'memory allocation error in ',subname
         return
      end if
      do i=1,cv_maxnodenmb
         nullify(cv_layer_p2node(i)%t2_nodenumbers,                     &
     &           cv_layer_p2node(i)%t2_cand,                            &
     &           cv_layer_p2node(i)%t2_candcostw,                       &
     &           cv_layer_p2node(i)%t2_candcostm)
         cv_layer_p2node(i)%nmb_t2s=0
      enddo
      istat = 0
      end subroutine AGMG_MUMPS_479
      function AGMG_MUMPS_359(nfront,npiv)
      implicit none
      logical::AGMG_MUMPS_359
      integer,intent(in)::nfront,npiv
      AGMG_MUMPS_359=.FALSE.
      if( (nfront - npiv > cv_keep(9))                                  &
     &     .and. ((npiv > cv_keep(4)).or.(.TRUE.))                      &
     &     .and. (cv_icntl(40).eq.0) ) AGMG_MUMPS_359=.TRUE.
      return
      end function AGMG_MUMPS_359
      subroutine AGMG_MUMPS_381(istat)
      implicit none
      integer,intent(out)::istat
      integer i,ierr,inode
      logical accepted,splitting_allowed
      integer,parameter::map_strat=cv_equilib_flops
      character (len=48):: err_rep,subname
      logical use_geist_ng_replace, skiparrangeL0
      INTEGER MINSIZE_L0
      istat=-1
      subname='LAYERL0'
      accepted=.FALSE.
      splitting_allowed=.TRUE.
      splitting_allowed=.FALSE.
      IF (cv_keep(72).EQ.2) THEN
       MINSIZE_L0 = 6*cv_slavef
      ELSE
       MINSIZE_L0 = 3*cv_slavef
      ENDIF
   55 continue
      skiparrangeL0 = .false.
      do while(.not.accepted)
         IF ( (    (layerL0_endforarrangeL0.LT.MINSIZE_L0)              &
     &             .OR. skiparrangeL0                                   &
     &        )                                                         &
     &        .AND.                                                     &
     &           (cv_layerl0_end.LT.cv_maxnsteps/2) ) THEN
          accepted = .false.
         ELSE
          err_rep='ARRANGEL0'
          call AGMG_MUMPS_415(map_strat, layerL0_endforarrangeL0,            &
     &                           cv_layerworkload,cv_layermemused,      &
     &                           cv_procnode,ierr)
          if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat = ierr
            return
          end if
          err_rep='ACCEPT_L0'
          call AGMG_MUMPS_413(map_strat,                                     &
     &                           cv_layerworkload,cv_layermemused,      &
     &                           accepted,ierr)
          if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat = ierr
            return
          end if
         ENDIF
         IF (cv_slavef.GT.16)                                           &
     &        skiparrangeL0 = .NOT.skiparrangeL0
         if (accepted.OR.(cv_costw_total.le.0.0D0)) then
            exit
         elseif(((cv_costw_layer0/cv_costw_total).gt.cv_l0wthresh) .AND.&
     &           (.TRUE.))then
            err_rep='MAX_TCOST_L0'
            inode = cv_layerl0_array(cv_layerl0_start)
            use_geist_ng_replace = .TRUE.
            if(use_geist_ng_replace) then
               err_rep='FATHSON_REPLACE'
               call AGMG_MUMPS_372(inode,ierr)
               if(ierr.eq.1) then
                  accepted=.TRUE.
               elseif(ierr.ne.0) then
                  if(cv_lp.gt.0)                                        &
     &            write(cv_lp,*)                                        &
     &            'Error rep. by ',err_rep,' in ',subname
                  istat = ierr
                  return
               endif
            endif
         else
            accepted=.TRUE.
         end if
      end do
      accepted=.TRUE.
      if (accepted) then
      else
         goto 55
      endif
      err_rep='LIST2LAYER'
      call AGMG_MUMPS_382(ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
         istat = ierr
         return
      end if
      err_rep='MAKE_PROPMAP'
      call AGMG_MUMPS_477(ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
         istat = ierr
         return
      end if
      if ( cv_keep(75).EQ.1 ) then
         call AGMG_MUMPS_415(map_strat, cv_layerl0_end,                      &
     &                     cv_layerworkload,cv_layermemused,            &
     &                     cv_procnode,ierr, respect_prop=.TRUE.)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Error reported by ',err_rep,' in ',subname
            istat = ierr
            return
         end if
      else if (layerL0_endforarrangeL0.LT.cv_layerl0_end) THEN
         call AGMG_MUMPS_415(map_strat, cv_layerl0_end,                      &
     &                     cv_layerworkload,cv_layermemused,            &
     &                     cv_procnode,ierr)
      endif
      call AGMG_MUMPS_386(cv_procnode)
      do i=1,cv_slavef
         cv_proc_workload(i)=cv_layerworkload(i)
         cv_proc_memused(i)=cv_layermemused(i)
      end do
      istat=0
      return
      end subroutine AGMG_MUMPS_381
      subroutine AGMG_MUMPS_382(istat)
      implicit none
      integer, intent(out)::istat
      type(nodelist),pointer::dummy
      character (len=48):: subname
      integer i,inode
      istat=-1
      subname='LIST2LAYER'
      cv_dist_L0_mixed_strat_bound=0
      cv_nbsa=0
      do i=cv_layerl0_start,cv_layerl0_end
         inode=cv_layerl0_array(i)
         if(inode.gt.0) then
         cv_dist_L0_mixed_strat_bound=max(cv_dist_L0_mixed_strat_bound  &
     &        ,max(cv_depth(inode)-cv_mixed_strat_bound,0))
         cv_nodelayer(inode)=0
         cv_nbsa=cv_nbsa+1
         cv_ssarbr(cv_nbsa)=inode
         endif
      enddo
      istat=0
      return
      end subroutine AGMG_MUMPS_382
      subroutine AGMG_MUMPS_477(istat)
      implicit none
      integer,intent(out)::istat
      integer i,pctr,pctr2,ierr,procindex(cv_size_ind_proc)
      istat = -1
      pctr=cv_n
      pctr2=cv_mixed_strat_bound
      do i=1,cv_slavef
         call AGMG_MUMPS_482(procindex,i,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)write(cv_lp,*)                                &
     &           'BIT_SET signalled error to',subname
            istat = ierr
            return
         end if
      end do
      do i=1,cv_n
         if(cv_frere(i).eq.0) then
            if(.NOT.associated(cv_prop_map(i)%ind_proc)) then
               call AGMG_MUMPS_434(i,ierr)
               if(ierr.ne.0) then
                  if(cv_lp.gt.0)                                        &
     &                 write(cv_lp,*)'PROPMAP_INIT signalled error to'  &
     &                 ,subname
                  istat = ierr
                  return
               end if
            endif
            cv_prop_map(i)%ind_proc = procindex
            call AGMG_MUMPS_433(i,pctr,ierr)
            if(ierr.ne.0) then
            if(cv_lp.gt.0)write(cv_lp,*)                                &
     &           'PROPMAP signalled error to',subname
               istat = ierr
               return
            endif
            if((cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
               call AGMG_MUMPS_517(i,pctr2,ierr)
               if(ierr.ne.0) then
                  if(cv_lp.gt.0)write(cv_lp,*)                          &
     &           'MOD_PROPMAP signalled error to',subname
                  istat = ierr
                  return
               endif
            endif
         endif
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_477
      subroutine AGMG_MUMPS_387(layernmb,thislayer,                          &
     &   nmb_thislayer,map_strat,istat)
      implicit none
      integer, intent(in)::layernmb,thislayer(cv_maxnodenmb),           &
     &                     nmb_thislayer,map_strat
      integer,intent(out)::istat
      integer i,inode,j,k,ierr,nmb,aux_int,nmb_cand_needed
      DOUBLE PRECISION aux_dbl,aux_flop,aux_mem
      INTEGER candid(cv_slavef)
      integer sorted_nmb(2*nmb_thislayer)
      DOUBLE PRECISION sorted_costw(2*nmb_thislayer),                   &
     &                 sorted_costm(2*nmb_thislayer),                   &
     &                 old_workload(cv_slavef),old_memused(cv_slavef)
      character (len=48):: err_rep,subname
      logical use_propmap
      istat=-1
      subname='MAP_LAYER'
      if((cv_keep(24).eq.8).OR.(cv_keep(24).eq.10)                      &
     &   .OR.(cv_keep(24).eq.12).OR.(cv_keep(24).eq.14)                 &
     &   .OR.(cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
         use_propmap=.TRUE.
      else
         use_propmap=.FALSE.
      endif
      if((layernmb.lt.0).or.(layernmb.gt.cv_maxlayer)) return
      if((map_strat.ne.cv_equilib_flops).and.                           &
     &   (map_strat.ne.cv_equilib_mem)) return
      do i=1,nmb_thislayer
         inode=thislayer(i)
         if (cv_nodetype(inode).eq.3) then
            cv_procnode(inode)=1
            exit
         end if
      end do
      do i=1,cv_slavef
         old_workload(i)=cv_layerworkload(i)
         old_memused(i)=cv_layermemused(i)
      enddo
      nmb=0
      do i=1,nmb_thislayer
         inode=thislayer(i)
         if(cv_nodetype(inode).eq.1) then
            nmb=nmb+1
            sorted_nmb(nmb)=inode
            sorted_costw(nmb)=cv_ncostw(inode)
            sorted_costm(nmb)=cv_ncostm(inode)
         else if(cv_nodetype(inode).eq.2) then
            nmb=nmb+1
            do j=1,cv_layer_p2node(layernmb)%nmb_t2s
               if(cv_layer_p2node(layernmb)%t2_nodenumbers(j).ne.inode) &
     &            then
                  cycle
               else
                  sorted_costw(nmb)=                                    &
     &                 cv_layer_p2node(layernmb)%t2_candcostw(j)
                  sorted_costm(nmb)=                                    &
     &                 cv_layer_p2node(layernmb)%t2_candcostm(j)
               endif
            enddo
            if((sorted_costw(nmb).eq.cv_d_invalid).OR.                  &
     &           (sorted_costm(nmb).eq.cv_d_invalid)) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname
               return
            end if
            if(sorted_costw(nmb).lt.cv_ncostw(inode))then
               sorted_costw(nmb)=cv_ncostw(inode)
               sorted_costm(nmb)=cv_ncostm(inode)
               sorted_nmb(nmb)=inode
            else
               sorted_nmb(nmb)=-inode
            endif
         else if(cv_nodetype(inode).eq.3) then
            cycle
         else
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Unknown node type. Error in ',subname
            return
         end if
      end do
      if (map_strat.eq.cv_equilib_flops) then
         call AGMG_MUMPS_459(nmb,sorted_nmb(1:nmb),                          &
     &                   sorted_costw(1:nmb),sorted_costm(1:nmb))
      elseif(map_strat.eq.cv_equilib_mem) then
         call AGMG_MUMPS_459(nmb,sorted_nmb(1:nmb),                          &
     &                   sorted_costm(1:nmb),sorted_costw(1:nmb))
      endif
      do i=1,nmb
         aux_int=sorted_nmb(i)
         aux_flop=sorted_costw(i)
         aux_mem=sorted_costm(i)
         k=1
         if (aux_int.lt.0) then
            inode=-aux_int
            err_rep='SORTPROCS'
            if(use_propmap) then
               call AGMG_MUMPS_398(map_strat,                                &
     &              cv_proc_workload,cv_proc_memused,                   &
     &              inode=inode,istat=ierr)
            else
               call AGMG_MUMPS_398(map_strat,                                &
     &              cv_proc_workload,cv_proc_memused,                   &
     &              istat=ierr)
            end if
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)                                           &
     &         'Error reported by ',err_rep,' in ',subname
               istat = ierr
               return
            endif
            nmb_cand_needed=cv_invalid
            do j=1,cv_layer_p2node(layernmb)%nmb_t2s
               if(cv_layer_p2node(layernmb)%t2_nodenumbers(j).ne.inode) &
     &            then
                  cycle
               else
                  nmb_cand_needed=                                      &
     &                  cv_layer_p2node(layernmb)%t2_cand(j,cv_slavef+1)
                  exit
               endif
            enddo
            if(nmb_cand_needed.eq.cv_invalid) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname
               return
            endif
            do while((k.le.cv_slavef).and.(nmb_cand_needed.gt.0))
               if(((.not.cv_constr_work).or.                            &
     &            (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.     &
     &             cv_proc_maxwork(cv_proc_sorted(k))))                 &
     &           .AND.((.not.cv_constr_mem).or.                         &
     &            (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.       &
     &             cv_proc_maxmem(cv_proc_sorted(k))))                  &
     &           .AND.                                                  &
     & (cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k)).eq.0))   &
     &            then
                  cv_proc_workload(cv_proc_sorted(k))=                  &
     &                 cv_proc_workload(cv_proc_sorted(k))+aux_flop
                  cv_proc_memused(cv_proc_sorted(k))=                   &
     &                 cv_proc_memused(cv_proc_sorted(k))+aux_mem
                  cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k))&
     &                                             =inode
                  cv_layerworkload(cv_proc_sorted(k))=                  &
     &                 cv_layerworkload(cv_proc_sorted(k))+aux_flop
                  cv_layermemused(cv_proc_sorted(k))=                   &
     &                 cv_layermemused(cv_proc_sorted(k))+aux_mem
                  nmb_cand_needed=nmb_cand_needed-1
                  k=k+1
               else
                  k=k+1
                  if(k.gt.cv_slavef) then
                     if(cv_lp.gt.0)                                     &
     &                    write(cv_lp,*)'Error in ',subname
                     return
                  endif
               end if
            end do
            if(nmb_cand_needed.gt.0) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Error in ',subname
               return
            endif
            aux_flop=cv_ncostw(inode)
            aux_mem=cv_ncostm(inode)
            do while(k.le.cv_slavef)
               if(((.not.cv_constr_work).or.                            &
     &              (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.   &
     &              cv_proc_maxwork(cv_proc_sorted(k))))                &
     &              .AND.((.not.cv_constr_mem).or.                      &
     &              (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.     &
     &              cv_proc_maxmem(cv_proc_sorted(k))))                 &
     &              .AND.                                               &
     & (cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k)).eq.0))   &
     &              then
                  cv_procnode(inode)=cv_proc_sorted(k)
                  cv_proc_workload(cv_proc_sorted(k))=                  &
     &                 cv_proc_workload(cv_proc_sorted(k))+aux_flop
                  cv_proc_memused(cv_proc_sorted(k))=                   &
     &                 cv_proc_memused(cv_proc_sorted(k))+aux_mem
                  cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k))&
     &                 =-inode
                  cv_layerworkload(cv_proc_sorted(k))=                  &
     &                 cv_layerworkload(cv_proc_sorted(k))+aux_flop
                  cv_layermemused(cv_proc_sorted(k))=                   &
     &                 cv_layermemused(cv_proc_sorted(k))+aux_mem
                  exit
               else
                  k=k+1
                  if(k.gt.cv_slavef) then
                     if(cv_lp.gt.0)                                     &
     &                    write(cv_lp,*)'Error in ',subname
                     return
                  endif
               end if
            end do
         else
            inode=aux_int
            err_rep='SORTPROCS'
            if(use_propmap) then
               call AGMG_MUMPS_398(map_strat,                                &
     &                        cv_proc_workload,cv_proc_memused,         &
     &                        inode=inode,istat=ierr)
            else
               call AGMG_MUMPS_398(map_strat,                                &
     &                        cv_proc_workload,cv_proc_memused,         &
     &                        inode,istat=ierr)
            endif
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &          write(cv_lp,*)                                          &
     &          'Error reported by ',err_rep,' in ',subname
               istat = ierr
               return
            endif
            if (cv_nodetype(inode).eq.1) then
               do while(k.le.cv_slavef)
                  if((.not.cv_constr_work).or.                          &
     &               (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.  &
     &                cv_proc_maxwork(cv_proc_sorted(k)))               &
     &            .AND.((.not.cv_constr_mem).or.                        &
     &               (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.    &
     &                cv_proc_maxmem(cv_proc_sorted(k))))) then
                     cv_procnode(inode)=cv_proc_sorted(k)
                     cv_proc_workload(cv_proc_sorted(k))=               &
     &                    cv_proc_workload(cv_proc_sorted(k))+aux_flop
                     cv_proc_memused(cv_proc_sorted(k))=                &
     &                    cv_proc_memused(cv_proc_sorted(k))+aux_mem
                     cv_layerworkload(cv_proc_sorted(k))=               &
     &                    cv_layerworkload(cv_proc_sorted(k))+aux_flop
                     cv_layermemused(cv_proc_sorted(k))=                &
     &                    cv_layermemused(cv_proc_sorted(k))+aux_mem
                     exit
                  else
                     k=k+1
                     if(k.gt.cv_slavef) then
                        if(cv_lp.gt.0)                                  &
     &                  write(cv_lp,*)'Inconsist data in ',subname
                        return
                     endif
                  end if
               end do
            elseif (cv_nodetype(inode).eq.2)then
               do j=1,cv_layer_p2node(layernmb)%nmb_t2s
                  if(cv_layer_p2node(layernmb)%t2_nodenumbers(j).ne.    &
     &               inode) then
                     cycle
                  else
                     exit
                  endif
               enddo
               do while(k.le.cv_slavef)
                  if(((.not.cv_constr_work).or.                         &
     &               (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.  &
     &                cv_proc_maxwork(cv_proc_sorted(k))))              &
     &              .AND.((.not.cv_constr_mem).or.                      &
     &               (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.    &
     &                cv_proc_maxmem(cv_proc_sorted(k))))               &
     &              .AND.                                               &
     & (cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k)).eq.0))   &
     &               then
                     cv_procnode(inode)=cv_proc_sorted(k)
                     cv_proc_workload(cv_proc_sorted(k))=               &
     &                    cv_proc_workload(cv_proc_sorted(k))+aux_flop
                     cv_proc_memused(cv_proc_sorted(k))=                &
     &                     cv_proc_memused(cv_proc_sorted(k))+aux_mem
                cv_layer_p2node(layernmb)%t2_cand(j,cv_proc_sorted(k))  &
     &                                           =-inode
                     cv_layerworkload(cv_proc_sorted(k))=               &
     &                    cv_layerworkload(cv_proc_sorted(k))+aux_flop
                     cv_layermemused(cv_proc_sorted(k))=                &
     &                    cv_layermemused(cv_proc_sorted(k))+aux_mem
                     exit
                  else
                     k=k+1
                     if(k.gt.cv_slavef) then
                        if(cv_lp.gt.0)                                  &
     &                       write(cv_lp,*)'Error in ',subname
                        return
                     endif
                  end if
               end do
               nmb_cand_needed=cv_invalid
               do j=1,cv_layer_p2node(layernmb)%nmb_t2s
                  if(cv_layer_p2node(layernmb)%t2_nodenumbers(j)        &
     &                 .ne.inode)                                       &
     &                 then
                     cycle
                  else
                     nmb_cand_needed=                                   &
     &                    cv_layer_p2node(layernmb)%                    &
     &                    t2_cand(j,cv_slavef+1)
                     exit
                  endif
               enddo
               if(nmb_cand_needed.eq.cv_invalid) then
                  if(cv_lp.gt.0)                                        &
     &                 write(cv_lp,*)'Error in ',subname
                  return
               endif
               aux_flop=                                                &
     &              cv_layer_p2node(layernmb)%t2_candcostw(j)
               aux_mem=                                                 &
     &              cv_layer_p2node(layernmb)%t2_candcostm(j)
               do while((k.le.cv_slavef).and.(nmb_cand_needed.gt.0))
                  if(((.not.cv_constr_work).or.                         &
     &                 (cv_proc_workload(cv_proc_sorted(k))+aux_flop.lt.&
     &                 cv_proc_maxwork(cv_proc_sorted(k))))             &
     &                 .AND.((.not.cv_constr_mem).or.                   &
     &                 (cv_proc_memused(cv_proc_sorted(k))+aux_mem.lt.  &
     &                 cv_proc_maxmem(cv_proc_sorted(k))))              &
     &                 .AND.                                            &
     &                 (cv_layer_p2node(layernmb)%                      &
     &                 t2_cand(j,cv_proc_sorted(k)).eq.0))              &
     &                 then
                     cv_proc_workload(cv_proc_sorted(k))=               &
     &                    cv_proc_workload(cv_proc_sorted(k))+aux_flop
                     cv_proc_memused(cv_proc_sorted(k))=                &
     &                    cv_proc_memused(cv_proc_sorted(k))+aux_mem
                     cv_layer_p2node(layernmb)%                         &
     &                    t2_cand(j,cv_proc_sorted(k))                  &
     &                    =inode
                     cv_layerworkload(cv_proc_sorted(k))=               &
     &                    cv_layerworkload(cv_proc_sorted(k))+aux_flop
                     cv_layermemused(cv_proc_sorted(k))=                &
     &                    cv_layermemused(cv_proc_sorted(k))+aux_mem
                     nmb_cand_needed=nmb_cand_needed-1
                     k=k+1
                  else
                     k=k+1
                     if(k.gt.cv_slavef) then
                        if(cv_lp.gt.0)                                  &
     &                       write(cv_lp,*)'Error in ',subname
                        return
                     endif
                  end if
               end do
               if(nmb_cand_needed.gt.0) then
                  if(cv_lp.gt.0)                                        &
     &                 write(cv_lp,*)'Error in ',subname
                  return
               endif
            end if
         end if
      end do
      do i=1,cv_layer_p2node(layernmb)%nmb_t2s
         nmb_cand_needed=                                               &
     &                  cv_layer_p2node(layernmb)%t2_cand(i,cv_slavef+1)
         candid= cv_layer_p2node(layernmb)%t2_cand(i,1:cv_slavef)
         cv_layer_p2node(layernmb)%t2_cand(i,1:cv_slavef)=-1
         k=0
         do j=1,cv_slavef
            if(candid(j).gt.0) then
               k=k+1
               cv_layer_p2node(layernmb)%t2_cand(i,k)=j-1
            end if
         end do
         if (k.ne.nmb_cand_needed) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Error in ',subname
            return
         endif
      enddo
      do i=1,cv_slavef
         cv_layerworkload(i)=cv_layerworkload(i)-old_workload(i)
         cv_layermemused(i)=cv_layermemused(i)-old_memused(i)
      enddo
      istat=0
      return
      end subroutine AGMG_MUMPS_387
      recursive subroutine AGMG_MUMPS_385(inode,procnmb,                     &
     &                                       procnode)
      integer,intent(in)::inode,procnmb
      integer,intent(inout)::procnode(cv_n)
      integer in
      procnode(inode)=procnmb
      if (cv_fils(inode).eq.0) return
      in=cv_fils(inode)
      do while(in>0)
         procnode(in)=procnmb
         in=cv_fils(in)
      end do
      in=-in
      do while(in>0)
         call AGMG_MUMPS_385(in,procnmb,procnode)
         in=cv_frere(in)
      end do
      return
      end subroutine AGMG_MUMPS_385
      subroutine AGMG_MUMPS_386(procnode)
      implicit none
      integer,intent(inout)::procnode(cv_n)
      integer i,inode,procnmb
      type(nodelist),pointer::dummy
      do i=cv_layerl0_start,cv_layerl0_end
         inode=cv_layerl0_array(i)
         if(inode.gt.0) then
            procnmb=procnode(inode)
            call AGMG_MUMPS_385(inode,procnmb,procnode)
         endif
      enddo
      return
      end subroutine AGMG_MUMPS_386
      subroutine AGMG_MUMPS_389(map_strat,inode,istat)
      implicit none
      integer, intent(in)::map_strat
      integer,intent(out)::inode,istat
      type(nodelist),pointer::dummy
      character (len=48):: subname
      subname='MAX_TCOST_L0'
      inode=-1
      istat=-1
      if ((.NOT.associated(cv_tcostw)).OR.(.NOT.associated(cv_tcostm))) &
     &   then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:tcost must be allocated in ',subname
         return
      end if
      if((map_strat.ne.cv_equilib_flops).and.                           &
     &   (map_strat.ne.cv_equilib_mem)) return
      inode=cv_layerl0_array(cv_layerl0_start)
      istat=0
      return
      end subroutine AGMG_MUMPS_389
      subroutine AGMG_MUMPS_431()
      implicit none
      integer candid,inode,index,i,j,layernmb,master,nmbcand,swapper,   &
     &        totalnmb,node_of_master,node_of_candid,node_of_swapper
      DOUBLE PRECISION::mastermem,slavemem,maxmem
      logical swapthem,cand_better_master_arch,cand_better_swapper_arch
      intrinsic maxval,minval
      maxmem=maxval(cv_proc_memused(:))
      totalnmb=0
      do layernmb=cv_maxlayer,1,-1
         do i=1,cv_layer_p2node(layernmb)%nmb_t2s
            inode=cv_layer_p2node(layernmb)%t2_nodenumbers(i)
            master=cv_procnode(inode)
            if(ke69 .gt. 1) then
               allowed_nodes = .FALSE.
               call AGMG_MUMPS_476(layernmb,i)
               node_of_master = mem_distribmpi(master-1)
               if (node_of_master .lt. 0 ) then
                if(cv_mp.gt.0) write(cv_mp,*)'node_of_master_not found'
               endif
               node_of_swapper = node_of_master
            endif
            mastermem=cv_proc_memused(master)
            nmbcand=cv_layer_p2node(layernmb)%t2_cand(i,cv_slavef+1)
            swapper=master
            index=0
            do j=1,nmbcand
               candid=cv_layer_p2node(layernmb)%t2_cand(i,j)+1
               slavemem=cv_proc_memused(candid)
               if(ke69 .gt. 1) then
                  node_of_candid = mem_distribmpi(candid-1)
                  if (node_of_candid .lt. 0 ) then
                     if(cv_mp.gt.0) write(cv_mp,*)                      &
     &               'node_of_candid_not found'
                  endif
               endif
               if(ke69 .le. 1) then
                  if((slavemem.lt.mastermem) .and.                      &
     &                 (slavemem.lt.cv_proc_memused(swapper))) then
                     swapper=candid
                     index=j
                  endif
               else
                  cand_better_master_arch = (                           &
     &                    (                                             &
     &                       (slavemem.lt.mastermem) .or.               &
     &                       (.not. allowed_nodes(node_of_master))      &
     &                    )                                             &
     &                    .and. allowed_nodes(node_of_candid)           &
     &                 )
                  cand_better_swapper_arch = (                          &
     &                    (                                             &
     &                       (slavemem.lt.cv_proc_memused(swapper)) .or.&
     &                       (.not. allowed_nodes(node_of_swapper))     &
     &                    )                                             &
     &                    .and. allowed_nodes(node_of_candid)           &
     &                 )
                  if(cand_better_master_arch .and.                      &
     &                 cand_better_swapper_arch  ) then
                     swapper=candid
                     node_of_swapper = node_of_candid
                     index=j
                  endif
               endif
            enddo
            if(swapper.ne.master) then
               swapthem = .FALSE.
               if(0.75D0*mastermem.ge.cv_proc_memused(swapper))         &
     &            swapthem=.TRUE.
               if(mastermem.le.mastermem-cv_ncostm(inode)               &
     &                +cv_layer_p2node(layernmb)%t2_candcostm(i))       &
     &            swapthem=.FALSE.
               if(mastermem.le.cv_proc_memused(swapper)                 &
     &                    +cv_ncostm(inode)                             &
     &                    -cv_layer_p2node(layernmb)%t2_candcostm(i))   &
     &            swapthem=.FALSE.
               if(maxmem.le.mastermem-cv_ncostm(inode)                  &
     &                +cv_layer_p2node(layernmb)%t2_candcostm(i))       &
     &            swapthem=.FALSE.
               if(maxmem.le.cv_proc_memused(swapper)+cv_ncostm(inode)   &
     &                -cv_layer_p2node(layernmb)%t2_candcostm(i))       &
     &            swapthem=.FALSE.
               if(ke69 .gt. 1) then
                  if (.not. allowed_nodes(node_of_master)) then
                     swapthem=.TRUE.
                  endif
               endif
               if(.NOT.swapthem) cycle
               cv_proc_workload(master)=cv_proc_workload(master)        &
     &                -cv_ncostw(inode)                                 &
     &                +cv_layer_p2node(layernmb)%t2_candcostw(i)
               cv_proc_memused(master)=cv_proc_memused(master)          &
     &                -cv_ncostm(inode)                                 &
     &                +cv_layer_p2node(layernmb)%t2_candcostm(i)
               cv_proc_workload(swapper)=cv_proc_workload(swapper)      &
     &                +cv_ncostw(inode)                                 &
     &                -cv_layer_p2node(layernmb)%t2_candcostw(i)
               cv_proc_memused(swapper)=cv_proc_memused(swapper)        &
     &                +cv_ncostm(inode)                                 &
     &                -cv_layer_p2node(layernmb)%t2_candcostm(i)
               cv_layer_p2node(layernmb)%t2_cand(i,index)=master-1
               cv_procnode(inode)=swapper
               maxmem=maxval(cv_proc_memused(:))
               totalnmb = totalnmb+1
            endif
         enddo
      enddo
      end subroutine AGMG_MUMPS_431
      subroutine AGMG_MUMPS_391(maxwork,maxmem,istat)
      implicit none
      DOUBLE PRECISION,intent(in),OPTIONAL::maxwork(cv_slavef),         &
     &                                      maxmem(cv_slavef)
      integer,intent(out)::istat
      integer i,allocok
      intrinsic huge
      DOUBLE PRECISION dummy
      character (len=48):: subname
      istat=-1
      subname='PROCINIT'
      if(present(maxwork)) then
         cv_constr_work=.TRUE.
      else
         cv_constr_work=.FALSE.
      end if
      if(present(maxmem)) then
         cv_constr_mem=.TRUE.
      else
         cv_constr_mem=.FALSE.
      end if
      allocate(cv_proc_workload(cv_slavef),                             &
     &         cv_proc_maxwork(cv_slavef),                              &
     &         cv_proc_memused(cv_slavef),                              &
     &         cv_proc_maxmem(cv_slavef),                               &
     &         cv_proc_sorted(cv_slavef),                               &
     &         STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 2*cv_slavef
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'memory allocation error in ',subname
         return
      end if
      allocate(work_per_proc(cv_slavef),id_son(cv_slavef),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 2*cv_slavef
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'memory allocation error in ',subname
         return
      end if
      do i=1,cv_slavef
         cv_proc_workload(i)=dble(0)
         if(cv_constr_work) then
            cv_proc_maxwork(i)=maxwork(i)
         else
            cv_proc_maxwork(i)=(huge(dummy))
         endif
         cv_proc_memused(i)=dble(0)
         if(cv_constr_mem) then
            cv_proc_maxmem(i)=maxmem(i)
         else
            cv_proc_maxmem(i)=(huge(dummy))
         endif
      end do
      cv_proc_sorted=(/(i,i=1,cv_slavef)/)
      istat=0
      return
      end subroutine AGMG_MUMPS_391
      recursive subroutine AGMG_MUMPS_517                                    &
     &                    (inode,ctr,istat)
      implicit none
      integer, intent(in)::inode,ctr
      integer, intent(inout)::istat
      integer::j,k,in,in1,ierr,son,nmb_procs_inode,nmb_sons_inode,      &
     &           procs4son(cv_size_ind_proc),current,i
      character (len=48):: subname
      DOUBLE PRECISION :: relative_weight,costs_sons
      DOUBLE PRECISION :: loc_relax
      INTEGER :: depth
      logical force_cand
      DOUBLE PRECISION Y
      intrinsic random_number
      integer nmb_propmap,nmb_propmap_strict,share2,procsrest,current2
      integer k69onid
      integer procs_inode(slavef)
      if (ctr.le.0) then
         istat = 0
         return
      endif
      procs_inode=-1
      istat= -1
      if(cv_frere(inode).eq.cv_n+1) return
      subname='MOD_PROPMAP'
      if(.NOT.associated(cv_prop_map(inode)%ind_proc)) return
      nmb_procs_inode = 0
      do j=1,cv_slavef
         if( AGMG_MUMPS_481(inode,j))then
            nmb_procs_inode = nmb_procs_inode + 1
         endif
      end do
      i=0
      do j=1,cv_slavef
            if(ke69 .gt.1) then
               call AGMG_MUMPS_493(j-1,                                      &
     &              k69onid,ierr)
            else
               k69onid = j
            endif
            if(AGMG_MUMPS_481(inode,k69onid))then
               i = i + 1
               procs_inode(i)=k69onid
            endif
      end do
      if(i.ne.nmb_procs_inode)then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      endif
      if(nmb_procs_inode.eq.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      end if
      if ((cv_nodelayer(inode).eq.0).AND.                               &
     &         (cv_frere(inode).ne.cv_n+1)) then
         istat = 0
         return
      endif
      nmb_sons_inode = 0
      costs_sons = dble(0)
      force_cand=(mod(cv_keep(24),2).eq.0)
      in = inode
      do while (cv_fils(in).gt.0)
         in=cv_fils(in)
      end do
      if (cv_fils(in).eq.0) then
         istat = 0
         return
      endif
      in = -cv_fils(in)
      son=in
      do while(in.gt.0)
         nmb_sons_inode = nmb_sons_inode + 1
         if(cv_tcostw(in).le.0.0D0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Subtree costs for ',in,                 &
     &                         ' should be positive in ',subname
            return
         endif
         costs_sons = costs_sons + cv_tcostw(in)
         in=cv_frere(in)
      enddo
      if(costs_sons.le.0D0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      endif
      depth= max(cv_mixed_strat_bound - ctr,0)
      if ((cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
         if(depth.ge.cv_mixed_strat_bound) then
            loc_relax = dble(1)
         else
            loc_relax =  dble(1) +                                      &
     &            max(dble(cv_keep(77))/dble(100), dble(0))
         endif
      else
         loc_relax = dble(1)
      endif
      in=son
      current = 1
      do while(in.gt.0)
         if( (nmb_sons_inode.ge.nmb_procs_inode).AND.                   &
     &       (nmb_procs_inode.LT.4) ) then
            procs4son = cv_prop_map(inode)%ind_proc
         else
            do k=1,cv_size_ind_proc
               do j=0,cv_bitsize_of_int-1
                  procs4son(k)=ibclr(procs4son(k),j)
               end do
            end do
            nmb_propmap_strict=0
            do k=1,cv_slavef
               if( AGMG_MUMPS_481(in,k)) then
                  nmb_propmap_strict=nmb_propmap_strict+1
                  call AGMG_MUMPS_482(procs4son,k,ierr)
               end if
            end do
            if(costs_sons.gt.0.0D0) then
               relative_weight=cv_tcostw(in)/costs_sons
            else
               relative_weight=0.0D0
            endif
            current = nmb_propmap_strict
            share2=                                                     &
     &           max(0,nint(relative_weight*(loc_relax-dble(1))*        &
     &           dble(nmb_procs_inode)))
            procsrest=nmb_procs_inode - nmb_propmap_strict
            share2=min(share2,procsrest)
            CALL RANDOM_NUMBER(Y)
            current2=int(dble(Y)*dble(procsrest))
            k=1
            i=1
            do while((share2.gt.0).and.(i.le.2))
               do j=1,nmb_procs_inode
                  if(share2.le.0) exit
                  k69onid = procs_inode(j)
                  if(( AGMG_MUMPS_481(inode,k69onid)).AND.                   &
     &                 (.NOT.AGMG_MUMPS_480(procs4son,k69onid))) then
                     if(k.ge.current2)then
                        call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                        if(ierr.ne.0) then
                           if(cv_lp.gt.0)write(cv_lp,*)                 &
     &                          'BIT_SET signalled error to',subname
                           istat = ierr
                           return
                        end if
                        share2 = share2 - 1
                     endif
                     k=k+1
                  end if
               enddo
               i=i+1
            enddo
            if(share2.ne.0) then
               if(cv_lp.gt.0) write(cv_lp,*)                            &
     &           'Error reported in ',subname
               return
            end if
         end if
         ierr=0
         in1=in
         cv_prop_map(in1)%ind_proc=procs4son
         call AGMG_MUMPS_517(in1,ctr-1,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0) write(cv_lp,*)                               &
     &           'Error reported in ',subname
            istat=ierr
            return
         endif
         in=cv_frere(in)
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_517
      recursive subroutine AGMG_MUMPS_433(inode,ctr,istat)
      implicit none
      integer, intent(in)::inode,ctr
      integer, intent(inout)::istat
      integer::j,k,in,in1,ierr,son,nmb_procs_inode,nmb_sons_inode,      &
     &           share,procs4son(cv_size_ind_proc),current,offset,      &
     &           in_tmp,nfront,npiv,ncb,                                &
     &           keep48_loc,min_cand_needed
      character (len=48):: subname
      DOUBLE PRECISION :: relative_weight,costs_sons, shtemp
      DOUBLE PRECISION :: costs_sons_real
      DOUBLE PRECISION :: PartofaProc
      LOGICAL          :: SkipSmallNodes
      PARAMETER (PartofaProc=0.01D0)
      DOUBLE PRECISION :: loc_relax
      INTEGER :: depth
      logical force_cand
      integer AGMG_MUMPS_497, AGMG_MUMPS_50
      external AGMG_MUMPS_497, AGMG_MUMPS_50
      DOUBLE PRECISION Y
      intrinsic random_number
      integer nmb_propmap_strict,share2,procsrest,current2
      integer k69onid,nb_free_procs,local_son_indice,nb_procs_for_sons, &
     &     ptr_upper_ro_procs
      logical upper_round_off,are_sons_treated
      DOUBLE PRECISION tmp_cost
      if (ctr.le.0) then
         istat = 0
         return
      endif
      istat= -1
      if(cv_frere(inode).eq.cv_n+1) return
      subname='PROPMAP'
      nmb_procs_inode = 0
      do j=1,cv_slavef
         if( AGMG_MUMPS_481(inode,j))                                        &
     &        nmb_procs_inode = nmb_procs_inode + 1
      end do
      if(nmb_procs_inode.eq.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      end if
      if ((cv_nodelayer(inode).eq.0).AND.                               &
     &         (cv_frere(inode).ne.cv_n+1)) then
         istat = 0
         return
      endif
      ptr_upper_ro_procs=1
      work_per_proc(1:cv_slavef)=0.0D0
      id_son(1:cv_slavef)=0
      nmb_sons_inode = 0
      costs_sons = dble(0)
      force_cand=(mod(cv_keep(24),2).eq.0)
      min_cand_needed=0
      in = inode
      do while (cv_fils(in).gt.0)
         in=cv_fils(in)
      end do
      if (cv_fils(in).eq.0) then
         istat = 0
         return
      endif
      in = -cv_fils(in)
      son=in
      do while(in.gt.0)
         nmb_sons_inode = nmb_sons_inode + 1
         if(cv_tcostw(in).le.0.0D0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Subtree costs for ',in,                 &
     &                         ' should be positive in ',subname
            return
         endif
         costs_sons = costs_sons + cv_tcostw(in)
         in=cv_frere(in)
      enddo
      costs_sons_real = costs_sons
      SkipSmallNodes = .true.
      IF (costs_sons_real.gt.0.0D0) then
       in = son
       do while (in.gt.0)
          relative_weight=cv_tcostw(in)/costs_sons_real
          shtemp = relative_weight*dble(nmb_procs_inode)
          IF (shtemp.lt.PartofaProc) THEN
            costs_sons = costs_sons - cv_tcostw(in)
          ENDIF
          in=cv_frere(in)
       enddo
       IF (costs_sons.LT. PartofaProc*costs_sons_real) THEN
         costs_sons = costs_sons_real
         SkipSmallNodes = .false.
       ENDIF
      ENDIF
      if(costs_sons.le.0.0D0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname                         &
     &        ,subname
         return
      endif
      if(cv_relax.le.0.0D0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname,'. Wrong cv_relax'
         return
      endif
      depth= max(cv_n - ctr,0)
      if(cv_keep(24).eq.8) then
         loc_relax = cv_relax
      elseif ((cv_keep(24).eq.16).OR.(cv_keep(24).eq.18)) then
         loc_relax = cv_relax
      elseif (cv_keep(24).eq.10) then
         loc_relax = cv_relax
      elseif ((cv_keep(24).eq.12).OR.(cv_keep(24).eq.14)) then
         if(depth.ge.cv_mixed_strat_bound) then
            loc_relax = cv_relax
         else
            loc_relax =  cv_relax +                                     &
     &          max(dble(cv_keep(77))/dble(100), dble(0))
         endif
      endif
      in=son
      current = 1
      local_son_indice=1
      nb_procs_for_sons=0
      upper_round_off=.FALSE.
      are_sons_treated=.TRUE.
      do while(in.gt.0)
         if( (nmb_sons_inode.ge.nmb_procs_inode).AND.                   &
     &       (nmb_procs_inode.LT.4) ) then
            procs4son = cv_prop_map(inode)%ind_proc
            are_sons_treated=.FALSE.
            nb_procs_for_sons=nmb_procs_inode
            nmb_propmap_strict=nmb_procs_inode
         elseif(nmb_procs_inode .LE. cv_keep(83)) then
            procs4son = cv_prop_map(inode)%ind_proc
            are_sons_treated=.FALSE.
            nb_procs_for_sons=nmb_procs_inode
            nmb_propmap_strict=nmb_procs_inode
         else
            do k=1,cv_size_ind_proc
               do j=0,cv_bitsize_of_int-1
                  procs4son(k)=ibclr(procs4son(k),j)
               end do
            end do
            if(costs_sons.gt.0.0D0) then
               relative_weight=cv_tcostw(in)/costs_sons
            else
               relative_weight=dble(0)
            endif
            shtemp = relative_weight*dble(nmb_procs_inode)
            IF ( (shtemp.LT.PartofaProc)                                &
     &             .AND. ( SkipSmallNodes ) )  THEN
             share = 1
             do j=current,cv_slavef
              if(ke69 .gt.1) then
               call AGMG_MUMPS_493(j-1,k69onid,ierr)
              else
               k69onid = j
              endif
              if( AGMG_MUMPS_481(inode,k69onid)) then
                  call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                  if(ierr.ne.0) then
                     if(cv_lp.gt.0)write(cv_lp,*)                       &
     &               'BIT_SET signalled error to',subname
                     istat = ierr
                     return
                  end if
                  share = share -1
                  exit
              endif
             enddo
             if (share.gt.0) then
               do j=1,current-1
                if(ke69 .gt.1) then
                 call AGMG_MUMPS_493(j-1,k69onid,ierr)
                else
                 k69onid = j
                endif
                if( AGMG_MUMPS_481(inode,k69onid)) then
                    call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                    if(ierr.ne.0) then
                       if(cv_lp.gt.0)write(cv_lp,*)                     &
     &               'BIT_SET signalled error to',subname
                       istat = ierr
                       return
                    end if
                    share = share -1
                    exit
                endif
               enddo
              endif
              if(share.ne.0) then
                 if(cv_lp.gt.0) write(cv_lp,*)                          &
     &              'Error reported in ',subname
                 return
              end if
              if(.NOT.associated(cv_prop_map(in)%ind_proc)) then
                 call AGMG_MUMPS_434(in,ierr)
              if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &              write(cv_lp,*)'PROPMAP_INIT signalled error to'     &
     &              ,subname
               istat = ierr
               return
              end if
             endif
             current  = j
             cv_prop_map(in)%ind_proc = procs4son
             in = cv_frere(in)
             cycle
            ENDIF
            share  = max(1,nint(shtemp))
            if (dble(share).ge.shtemp) then
               upper_round_off=.TRUE.
            else
               upper_round_off = .FALSE.
            endif
            share=min(share,nmb_procs_inode)
            nmb_propmap_strict=share
            nb_procs_for_sons=nb_procs_for_sons+nmb_propmap_strict
            offset=1
            do j=current,cv_slavef
               if(ke69 .gt.1) then
                  call AGMG_MUMPS_493(j-1,k69onid,ierr)
               else
                  k69onid = j
               endif
               if( AGMG_MUMPS_481(inode,k69onid)) then
                  call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                  if(ierr.ne.0) then
                     if(cv_lp.gt.0)write(cv_lp,*)                       &
     &               'BIT_SET signalled error to',subname
                     istat = ierr
                     return
                  end if
                  share = share-1
                  if(share.le.0) then
                     current = j + offset
                     if(current.gt.cv_slavef) current = 1
                     exit
                  end if
               end if
            end do
            if(share.gt.0) then
               do j=1,current-1
                  if(ke69 .gt.1) then
                     call AGMG_MUMPS_493(j-1,k69onid,ierr)
                  else
                     k69onid = j
                  endif
                  if( AGMG_MUMPS_481(inode,k69onid)) then
                     call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                     if(ierr.ne.0) then
                        if(cv_lp.gt.0)write(cv_lp,*)                    &
     &                  'BIT_SET signalled error to',subname
                        istat = ierr
                        return
                     end if
                     share = share-1
                     if(share.le.0) then
                        current = j + offset
                        if(current.gt.cv_slavef) current = 1
                        exit
                     end if
                  end if
               end do
            endif
            if(share.ne.0) then
               if(cv_lp.gt.0) write(cv_lp,*)                            &
     &              'Error reported in ',subname
               return
            end if
            if(.not.upper_round_off)then
               if(local_son_indice.lt.cv_slavef)then
                  id_son(local_son_indice)=in
                  work_per_proc(local_son_indice)=cv_tcostw(in)/        &
     &                 dble(nmb_propmap_strict)
                  local_son_indice=local_son_indice+1
                  if(local_son_indice.eq.cv_slavef)then
                     CALL AGMG_MUMPS_459(cv_slavef,id_son,                   &
     &                    work_per_proc)
                  endif
               else
                  current2=cv_slavef
                  tmp_cost=cv_tcostw(in)/dble(nmb_propmap_strict)
                  do while(current2.ge.1)
                     if(tmp_cost.lt.work_per_proc(current2))exit
                     current2=current2-1
                  enddo
                  if(current2.ne.cv_slavef)then
                     if(current2.eq.0)then
                        current2=1
                     endif
                     do j=cv_slavef-1,current2,-1
                        id_son(j+1)=id_son(j)
                        work_per_proc(j+1)=work_per_proc(j)
                     enddo
                     id_son(current2)=in
                     work_per_proc(current2)=tmp_cost
                  endif
               endif
            endif
            upper_round_off=.FALSE.
         endif
         if(.NOT.associated(cv_prop_map(in)%ind_proc)) then
            call AGMG_MUMPS_434(in,ierr)
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &              write(cv_lp,*)'PROPMAP_INIT signalled error to'     &
     &              ,subname
               istat = ierr
               return
            end if
         endif
         cv_prop_map(in)%ind_proc = procs4son
         in=cv_frere(in)
      end do
      if(are_sons_treated)then
         if(nb_procs_for_sons.ne.nmb_procs_inode)then
            do j=1,nmb_procs_inode-nb_procs_for_sons
               procs4son=cv_prop_map(id_son(j))%ind_proc
               do while(current.le.cv_slavef)
                  if(ke69 .gt.1) then
                     call AGMG_MUMPS_493(current-1,k69onid,ierr)
                  else
                     k69onid = current
                  endif
                  if(.NOT.AGMG_MUMPS_481(inode,k69onid)) then
                     current=current+1
                  else
                     exit
                  endif
               enddo
               call AGMG_MUMPS_482(procs4son,k69onid,ierr)
               cv_prop_map(id_son(j))%ind_proc=procs4son
            enddo
            ptr_upper_ro_procs=min(j,nmb_procs_inode-nb_procs_for_sons)
         endif
      endif
      in=son
      current = 1
      do while(in.gt.0)
         if( (nmb_sons_inode.ge.nmb_procs_inode).AND.                   &
     &       (nmb_procs_inode.LT.4) ) then
            procs4son = cv_prop_map(inode)%ind_proc
         elseif(nmb_procs_inode .LE. cv_keep(83)) then
            procs4son = cv_prop_map(inode)%ind_proc
         else
            procs4son = cv_prop_map(in)%ind_proc
            in_tmp=in
            nfront=cv_nfsiz(in_tmp)
            npiv=0
            in_tmp=in_tmp
            do while(in_tmp.gt.0)
               npiv=npiv+1
               in_tmp=cv_fils(in_tmp)
            end do
            ncb=nfront-npiv
            if (force_cand) then
               if (cv_keep(50) ==  0) then
                  keep48_loc=0
               else
                  keep48_loc=3
               endif
               if (cv_keep(48).EQ.5) keep48_loc = 5
               min_cand_needed=                                         &
     &              AGMG_MUMPS_50                                            &
     &              (cv_slavef, keep48_loc,cv_keep8(21),                &
     &              cv_keep(50),                                        &
     &              nfront,ncb)
               min_cand_needed=min(cv_slavef,min_cand_needed+1)
            else
               min_cand_needed = 0
            endif
            min_cand_needed = max(min_cand_needed, cv_keep(91))
            if(costs_sons.gt.0.0D0) then
               relative_weight=cv_tcostw(in)/costs_sons
            else
               relative_weight=dble(0)
            endif
            nmb_propmap_strict=0
            do k=1,cv_slavef
               if( AGMG_MUMPS_480(procs4son,k)) then
                  nmb_propmap_strict=nmb_propmap_strict+1
               end if
            end do
            offset=1
            share2=                                                     &
     &          max(0,nint(relative_weight*(loc_relax-dble(1))*         &
     &                                   dble(nmb_procs_inode)))
            share2 = max(share2, min_cand_needed -nmb_propmap_strict,   &
     &                   (cv_keep(83)/2) - nmb_propmap_strict)
            procsrest=nmb_procs_inode - nmb_propmap_strict
            share2=min(share2,procsrest)
            share2 = 0
            CALL RANDOM_NUMBER(Y)
            current2     =int(dble(Y)*dble(procsrest))
            nb_free_procs=1
            do j=1,cv_slavef
               if(share2.le.0) exit
               if(ke69 .gt.1) then
                     call AGMG_MUMPS_493(j-1,k69onid,ierr)
                  else
                     k69onid = j
                  endif
               if(( AGMG_MUMPS_481(inode,k69onid)).AND.                      &
     &           (.NOT.AGMG_MUMPS_480(procs4son,k69onid))) then
                  if(nb_free_procs.ge.current2)then
                     call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                     if(ierr.ne.0) then
                        if(cv_lp.gt.0)write(cv_lp,*)                    &
     &                       'BIT_SET signalled error to',subname
                        istat = ierr
                        return
                     end if
                     share2 = share2 - 1
                  endif
                  nb_free_procs=nb_free_procs+1
               end if
            end do
            if(share2.gt.0) then
               do j=1,cv_slavef
                  if(share2.le.0) exit
                  if(ke69 .gt.1) then
                     call AGMG_MUMPS_493(j-1,k69onid,ierr)
                  else
                     k69onid = j
                  endif
                  if(( AGMG_MUMPS_481(inode,k69onid)).AND.                   &
     &              (.NOT.AGMG_MUMPS_480(procs4son,k69onid))) then
                        call AGMG_MUMPS_482(procs4son,k69onid,ierr)
                        if(ierr.ne.0) then
                           if(cv_lp.gt.0)write(cv_lp,*)                 &
     &                          'BIT_SET signalled error to',subname
                           istat = ierr
                           return
                        end if
                        share2 = share2 - 1
                  end if
               end do
            endif
            if(share2.ne.0) then
               if(cv_lp.gt.0) write(cv_lp,*)                            &
     &              'Error reported in ',subname
               return
            end if
         endif
         ierr=0
         in1=in
         cv_prop_map(in1)%ind_proc = procs4son
         call AGMG_MUMPS_433(in1,ctr-1,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0) write(cv_lp,*)                               &
     &         'Error reported in ',subname
            istat=ierr
            return
         endif
         in=cv_frere(in)
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_433
      subroutine AGMG_MUMPS_434(inode,istat)
      implicit none
      integer, intent(in)::inode
      integer, intent(out)::istat
      integer j,k,allocok
      character (len=48):: subname
      istat = -1
      if(cv_frere(inode).eq.cv_n+1) return
      subname='PROPMAP_INIT'
      if(.not.associated(                                               &
     &     cv_prop_map(inode)%ind_proc)) then
         allocate(cv_prop_map(inode)%ind_proc                           &
     &        (cv_size_ind_proc),STAT=allocok)
         if (allocok.gt.0) then
            cv_info(1) = cv_error_memalloc
            cv_info(2) = cv_size_ind_proc
            istat = cv_error_memalloc
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)                                         &
     &           'memory allocation error in ',subname
            return
         end if
      end if
      do k=1,cv_size_ind_proc
         do j=0,cv_bitsize_of_int-1
            cv_prop_map(inode)%ind_proc(k)=                             &
     &         ibclr(cv_prop_map(inode)%ind_proc(k),j)
         end do
      end do
      istat = 0
      return
      end subroutine AGMG_MUMPS_434
      subroutine AGMG_MUMPS_435(inode,istat)
      integer,intent(in)::inode
      integer,intent(out)::istat
      integer ierr
      character (len=48):: subname
      subname='PROPMAP_TERM'
      istat =-1
      if(associated(cv_prop_map(inode)%ind_proc)) then
        deallocate(cv_prop_map(inode)%ind_proc, STAT=ierr)
        if(ierr.ne.0) then
           if(cv_lp.gt.0)                                               &
     &         write(cv_lp,*)'Memory deallocation error in ', subname
               istat = cv_error_memdeloc
               return
        endif
        nullify(cv_prop_map(inode)%ind_proc)
      end if
      istat =0
      return
      end subroutine AGMG_MUMPS_435
      subroutine AGMG_MUMPS_436(ison,ifather,istat)
      implicit none
      integer,intent(in)::ison,ifather
      integer,intent(out)::istat
      character (len=48):: subname
      istat= -1
      subname='PROPMAP4AMALG'
      call AGMG_MUMPS_435(ison,ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'PROPMAP_TERM signalled error in ',         &
     &        subname
         istat = ierr
         return
      end if
      istat = 0
      return
      end subroutine AGMG_MUMPS_436
      subroutine AGMG_MUMPS_437(inode,ifather,istat)
      implicit none
      integer,intent(in)::inode,ifather
      integer,intent(out)::istat
      character (len=48):: subname
      istat= -1
      subname='PROPMAP4SPLIT'
      if((cv_frere(inode).eq.cv_n+1).OR.(cv_frere(ifather).eq.cv_n+1)   &
     &     .OR.(.NOT.associated(cv_prop_map(inode)%ind_proc))) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'tototo signalled error to'                 &
     &        ,subname
         return
      endif
      if(.NOT.associated(cv_prop_map(ifather)%ind_proc)) then
         call AGMG_MUMPS_434(ifather,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'PROPMAP_INIT signalled error to '       &
     &           ,subname
            istat = ierr
            return
         end if
      endif
      cv_prop_map(ifather)%ind_proc =                                   &
     &                    cv_prop_map(inode)%ind_proc
      istat=0
      return
      end subroutine AGMG_MUMPS_437
      subroutine AGMG_MUMPS_394(istat)
      implicit none
      integer,intent(out)::istat
      integer i,ierr,allocok
      type(nodelist),pointer::nodeptr
      character (len=48):: subname
      istat=-1
      subname='ROOTLIST'
      allocate(cv_layerl0_array(cv_maxnsteps),                          &
     &         cv_layerl0_sorted_costw(cv_maxnsteps),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = 12*cv_maxnsteps
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)                                            &
     &        'memory allocation error in ',subname
         return
      end if
      do i=1,cv_maxnsteps
         cv_layerl0_sorted_costw(i)=dble(0)
         cv_layerl0_array(i)=0
      end do
      cv_layerl0_start        = 0
      cv_layerl0_end          = 0
      layerL0_endforarrangeL0 = 0
      if ((.NOT.associated(cv_tcostw)).OR.(.NOT.associated(cv_tcostm))) &
     &   then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:tcost must be allocated in ',subname
         return
      end if
      cv_nbsa=0
      do i=1,cv_n
         if (cv_frere(i).eq.0) then
            cv_layerl0_start=1
            cv_layerl0_end=cv_layerl0_end+1
            IF (cv_tcostw(i).GT.mincostw)                               &
     &           layerL0_endforarrangeL0 = layerL0_endforarrangeL0+1
            cv_layerl0_array(cv_layerl0_end)=i
            cv_layerl0_sorted_costw(cv_layerl0_end)=cv_tcostw(i)
            cv_costw_layer0=cv_costw_layer0 + cv_tcostw(i)
            cv_costm_layer0=cv_costm_layer0 + cv_tcostm(i)
            cv_nbsa=cv_nbsa+1
         end if
      end do
      if(cv_nbsa.eq.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:no root nodes in ',subname
         return
      end if
      call AGMG_MUMPS_459(cv_layerl0_end-cv_layerl0_start+1,                 &
     &            cv_layerl0_array(cv_layerl0_start:cv_layerl0_end),    &
     &     cv_layerl0_sorted_costw(cv_layerl0_start:cv_layerl0_end))
      cv_costw_total=cv_costw_layer0
      cv_costm_total=cv_costm_layer0
      istat=0
      return
      end subroutine AGMG_MUMPS_394
      subroutine AGMG_MUMPS_396(istat)
      implicit none
      integer,intent(out)::istat
      integer i,nfront
      character (len=48):: subname
      subname='SELECT_TYPE3'
      CALL AGMG_MUMPS_712(cv_n, slavef, cv_mp, cv_icntl(13),                 &
     &     cv_keep(1), cv_frere, cv_nfsiz, istat)
      IF (istat .NE. 0) THEN
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)                                         &
     &           'Error: Can''t select type 3 node in ',subname
      ELSE IF (cv_keep(38) .ne. 0) then
        IF(cv_nodelayer(cv_keep(38)).eq.0) then
          cv_keep(38)=0
        ELSE
          cv_nodetype(cv_keep(38))=3
        ENDIF
      ENDIF
      RETURN
      end subroutine AGMG_MUMPS_396
      subroutine AGMG_MUMPS_397(istat)
      integer,intent(out)::istat
      integer i,dummy,layernmb,allocok
      character (len=48):: subname
      istat=-1
      subname='SETUP_CAND'
      cv_nb_niv2=0
      do i=1,cv_n
         if(cv_nodetype(i).eq.2) cv_nb_niv2=cv_nb_niv2+1
      end do
      cv_keep(56)=cv_nb_niv2
      nullify(cv_par2_nodes,cv_cand)
      allocate(cv_par2_nodes(cv_nb_niv2),                               &
     &         cv_cand(cv_nb_niv2,cv_slavef+1),STAT=allocok)
      if (allocok.gt.0) then
         cv_info(1) = cv_error_memalloc
         cv_info(2) = cv_nb_niv2*(cv_slavef+2)
         istat = cv_error_memalloc
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)                                            &
     &        'memory allocation error in ',subname
         return
      end if
      cv_par2_nodes=0
      cv_cand(:,:)=0
      dummy=1
      do layernmb=1,cv_maxlayer
         do i=1,cv_layer_p2node(layernmb)%nmb_t2s
            cv_par2_nodes(dummy)=                                       &
     &                  cv_layer_p2node(layernmb)%t2_nodenumbers(i)
            cv_cand(dummy,:)=cv_layer_p2node(layernmb)%t2_cand(i,:)
            dummy=dummy+1
         enddo
      enddo
      if(dummy.ne.cv_nb_niv2+1) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error in ',subname,                        &
     &        ' : dummy =',dummy,'nbniv2 =',cv_nb_niv2
         return
      endif
      istat=0
      return
      end subroutine AGMG_MUMPS_397
      subroutine AGMG_MUMPS_398(map_strat,workload,memused,                  &
     &                       inode,istat)
      implicit none
      integer,intent(in)::map_strat
      DOUBLE PRECISION,dimension(cv_slavef),intent(in)::workload,       &
     &                                                   memused
      integer, optional::inode,istat
      integer i,j,aux_int,nmb_procs,proc,pos
      character (len=48):: subname
      logical enforce_prefsort
      logical use_propmap
      logical,SAVE::init1 = .FALSE.
      logical,SAVE::init2 = .FALSE.
      logical,SAVE::init3 = .FALSE.
      subname='SORTPROCS'
      enforce_prefsort=.TRUE.
      use_propmap=present(inode)
      if(present(istat))istat=-1
      if((map_strat.ne.cv_equilib_flops).and.                           &
     &   (map_strat.ne.cv_equilib_mem)) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'error in ',subname
         return
      endif
      cv_proc_sorted=(/(i,i=1,cv_slavef)/)
      if (.not.present(inode)) then
         if(.NOT.init1) then
            init1=.TRUE.
         end if
         do i=1,cv_slavef-1
            do j=i+1,cv_slavef
               if(((workload(cv_proc_sorted(j)).lt.                     &
     &                     workload(cv_proc_sorted(i))).AND.            &
     &             (map_strat.eq.cv_equilib_flops))                     &
     &            .OR.                                                  &
     &            ((memused(cv_proc_sorted(j)).lt.                      &
     &                     memused(cv_proc_sorted(i))).AND.             &
     &             (map_strat.eq.cv_equilib_mem)))then
                  aux_int=cv_proc_sorted(j)
                  cv_proc_sorted(j)=cv_proc_sorted(i)
                  cv_proc_sorted(i)=aux_int
               end if
            end do
         end do
      else if(present(inode)) then
         if (use_propmap) then
            if(.NOT.init2) then
               init2=.TRUE.
            end if
            nmb_procs=0
            do pos=1,cv_slavef
               if( AGMG_MUMPS_481(inode,pos)) then
                  if (pos.le.nmb_procs) then
                     exit
                  else
                     nmb_procs=nmb_procs+1
                     aux_int=cv_proc_sorted(pos)
                     cv_proc_sorted(pos)=                               &
     &                    cv_proc_sorted(nmb_procs)
                     cv_proc_sorted(nmb_procs)=aux_int
                     cycle
                  end if
               end if
            end do
         end if
         do i=1,nmb_procs-1
            do j=i+1,nmb_procs
               if(((workload(cv_proc_sorted(j)).lt.                     &
     &                     workload(cv_proc_sorted(i))).AND.            &
     &             (map_strat.eq.cv_equilib_flops))                     &
     &            .OR.                                                  &
     &            ((memused(cv_proc_sorted(j)).lt.                      &
     &                     memused(cv_proc_sorted(i))).AND.             &
     &             (map_strat.eq.cv_equilib_mem)))then
                  aux_int=cv_proc_sorted(j)
                  cv_proc_sorted(j)=cv_proc_sorted(i)
                  cv_proc_sorted(i)=aux_int
               end if
            end do
         end do
         do i=nmb_procs+1,cv_slavef-1
            do j=i+1,cv_slavef
               if(((workload(cv_proc_sorted(j)).lt.                     &
     &                     workload(cv_proc_sorted(i))).AND.            &
     &             (map_strat.eq.cv_equilib_flops))                     &
     &            .OR.                                                  &
     &            ((memused(cv_proc_sorted(j)).lt.                      &
     &                     memused(cv_proc_sorted(i))).AND.             &
     &             (map_strat.eq.cv_equilib_mem)))then
                  aux_int=cv_proc_sorted(j)
                  cv_proc_sorted(j)=cv_proc_sorted(i)
                  cv_proc_sorted(i)=aux_int
               end if
            end do
         end do
         if(.NOT.enforce_prefsort) then
            if(((2.0D0*workload(cv_proc_sorted(nmb_procs+1)).lt.        &
     &           workload(cv_proc_sorted(1))).AND.                      &
     &          (map_strat.eq.cv_equilib_flops))                        &
     &         .OR.                                                     &
     &         ((2.0D0*memused(cv_proc_sorted(nmb_procs+1)).lt.         &
     &           memused(cv_proc_sorted(1))).AND.                       &
     &          (map_strat.eq.cv_equilib_mem)))then
               do i=1,cv_slavef-1
                  do j=i+1,cv_slavef
                     if(((workload(cv_proc_sorted(j)).lt.               &
     &                    workload(cv_proc_sorted(i))).AND.             &
     &                    (map_strat.eq.cv_equilib_flops))              &
     &                    .OR.                                          &
     &                    ((memused(cv_proc_sorted(j)).lt.              &
     &                    memused(cv_proc_sorted(i))).AND.              &
     &                    (map_strat.eq.cv_equilib_mem)))then
                        aux_int=cv_proc_sorted(j)
                        cv_proc_sorted(j)=cv_proc_sorted(i)
                        cv_proc_sorted(i)=aux_int
                     end if
                  end do
               end do
            endif
         end if
      endif
      if(present(istat))istat=0
      return
      end subroutine AGMG_MUMPS_398
      subroutine AGMG_MUMPS_400(inode,                                       &
     &                                 doit,npiv,nfront,npiv_son,istat)
      implicit none
      integer,intent(in)::inode
      logical,intent(out)::doit
      integer,intent(out),OPTIONAL::npiv,nfront,npiv_son,istat
      integer in,nfr_ifather,npiv_ifather,npiv2,nfront2,npiv_son2
      doit=.FALSE.
      if(present(npiv)) npiv=cv_invalid
      if(present(nfront)) nfront=cv_invalid
      if(present(npiv_son)) npiv_son=cv_invalid
      if(present(istat)) istat=-1
      if(cv_frere(inode).eq.cv_n+1) return
      doit=.TRUE.
      if ((cv_frere(inode).eq.0).AND.                                   &
     &     ((cv_keep(60).ne.0).OR.(cv_keep(53).le.0))) doit=.FALSE.
      if(cv_nodetype(inode).eq.3) doit=.FALSE.
      nfront2=cv_nfsiz(inode)
      in=inode
      npiv2=0
      do while (in.gt.0)
         in=cv_fils(in)
         npiv2=npiv2+1
      end do
      if(in.eq.0) doit=.FALSE.
      if(cv_total_split.ge.cv_maxcut) doit=.FALSE.
      if(.NOT.any(cv_potsplit.eq.inode)) doit=.FALSE.
      npiv_son2 = max(npiv2/2,1)
      if(npiv2.le.npiv_son2) doit=.FALSE.
      if(.NOT.( AGMG_MUMPS_359(nfront2,npiv_son2)))                          &
     &   doit=.FALSE.
      nfr_ifather=nfront2-npiv_son2
      npiv_ifather=npiv2-npiv_son2
      if(.NOT.                                                          &
     &     ( AGMG_MUMPS_359(nfr_ifather,npiv_ifather)))then
         doit=.FALSE.
      endif
      if ((dble(nfront2)-dble(npiv2)/dble(2)).le.dble(cv_keep(9)))      &
     &   doit=.FALSE.
      if(cv_nsteps.ge.cv_maxnsteps) doit=.FALSE.
      if(.NOT.doit) then
         if(present(npiv)) npiv=cv_invalid
         if(present(nfront)) nfront=cv_invalid
         if(present(npiv_son)) npiv_son=cv_invalid
      else
         if(present(npiv)) npiv=npiv2
         if(present(nfront)) nfront=nfront2
         if(present(npiv_son)) npiv_son=npiv_son2
      endif
      if(present(istat)) istat=0
      end subroutine AGMG_MUMPS_400
      subroutine AGMG_MUMPS_401(inode,nfront,npiv,npiv_son,                  &
     &                              ison,ifather,istat)
      implicit none
      integer, intent(in)::inode,nfront,npiv,npiv_son
      integer,intent(out)::ison,ifather
      integer, intent(out)::istat
      integer i,in,in_son,in_father,in_grandpa
      DOUBLE PRECISION:: ncostm,ncostw,ncostm_ison,ncostw_ison,         &
     &                   ncostm_ifather,ncostw_ifather
      character (len=48):: subname
      istat=-1
      subname='SPLITNODE'
      ison=-1
      ifather=-1
      ncostw=cv_ncostw(inode)
      ncostm=cv_ncostm(inode)
      if((npiv_son.ge.npiv).OR.(npiv.ge.nfront)) return
      ison=inode
      in_son = ison
      do i = 1,npiv_son-1
         in_son = cv_fils(in_son)
         if (in_son.le.0) return
      end do
      ifather = cv_fils(in_son)
      if(ifather.le.0) return
      in_father=ifather
      do while (cv_fils(in_father).gt.0)
         in_father=cv_fils(in_father)
      end do
      cv_nsteps = cv_nsteps + 1
      cv_frere(ifather)=cv_frere(inode)
      cv_frere(ison)=-ifather
      cv_fils(in_son)=cv_fils(in_father)
      cv_fils(in_father)=-ison
      in=cv_frere(ifather)
      do while (in.gt.0)
         in=cv_frere(in)
      end do
      in=-in
      do while(cv_fils(in).gt.0)
         in=cv_fils(in)
      end do
      in_grandpa = in
      if(cv_fils(in_grandpa).eq.-inode) then
         cv_fils(in_grandpa)=-ifather
      else
         in=-cv_fils(in_grandpa)
         do while(cv_frere(in).gt.0)
            if(cv_frere(in).eq.inode) then
               cv_frere(in)=ifather
            else
               in=cv_frere(in)
            end if
         end do
      end if
      cv_last_splitting%new_ison=ison
      cv_last_splitting%new_ifather=ifather
      cv_last_splitting%old_keep2=cv_keep(2)
      cv_last_splitting%ncostw_oldinode=ncostw
      cv_last_splitting%ncostm_oldinode=ncostm
      if(associated(cv_tcostw))                                         &
     &   cv_last_splitting%tcostw_oldinode=cv_tcostw(inode)
      if(associated(cv_tcostm))                                         &
     &   cv_last_splitting%tcostm_oldinode=cv_tcostm(inode)
      cv_nfsiz(ison)=nfront
      cv_nfsiz(ifather)=nfront-npiv_son
      cv_keep(2)=max(cv_keep(2),nfront-npiv_son)
      cv_ne(ifather)=1
      cv_keep(61)=cv_keep(61)+1
      call AGMG_MUMPS_418(npiv_son,nfront,                                   &
     &                            ncostw_ison,ncostm_ison)
      call AGMG_MUMPS_418(npiv-npiv_son,nfront-npiv_son,                     &
     &                            ncostw_ifather,ncostm_ifather)
      cv_ncostw(ison)=ncostw_ison
      cv_ncostm(ison)=ncostm_ison
      cv_ncostw(ifather)=ncostw_ifather
      cv_ncostm(ifather)=ncostm_ifather
      if(associated(cv_tcostw)) cv_tcostw(ison) = cv_tcostw(inode)      &
     &                 -ncostw +cv_ncostw(ison)
      if(associated(cv_tcostm)) cv_tcostm(ison) = cv_tcostm(inode)      &
     &                 -ncostm +cv_ncostm(ison)
      if(associated(cv_tcostw))                                         &
     &   cv_tcostw(ifather) = cv_tcostw(ison)+cv_ncostw(ifather)
      if(associated(cv_tcostm))                                         &
     &   cv_tcostm(ifather) = cv_tcostm(ison)+cv_ncostm(ifather)
      cv_total_split=cv_total_split+1
      call AGMG_MUMPS_437(inode,ifather,ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'PROPMAP4SPLIT error in ',subname
         istat = ierr
         return
      end if
      istat = 0
      return
      end subroutine AGMG_MUMPS_401
      subroutine AGMG_MUMPS_402(ne,nfsiz,frere,fils,keep,KEEP8,              &
     &                                info,procnode,ssarbr,nbsa)
      implicit none
      integer,dimension(cv_n),intent(inout)::ne,nfsiz,frere,fils,       &
     &                                     procnode,ssarbr
      integer, intent(inout):: keep(500),info(40),nbsa
      INTEGER*8 KEEP8(150)
      ne=cv_ne
      nfsiz=cv_nfsiz
      frere=cv_frere
      fils=cv_fils
      keep(2) =cv_keep(2)
      keep(20)=cv_keep(20)
      keep(28)=cv_nsteps
      keep(38)=cv_keep(38)
      keep(56)=cv_keep(56)
      keep(61)=cv_keep(61)
      info(5)=cv_info(5)
      info(6)=cv_nsteps
      procnode=cv_procnode
      ssarbr=cv_ssarbr
      nbsa=cv_nbsa
      end subroutine AGMG_MUMPS_402
      subroutine AGMG_MUMPS_403(istat)
      implicit none
      integer,intent(out)::istat
      integer i,ierr,layernmb
      character (len=48):: subname
      istat=-1
      subname='TERMGLOB'
      if(cv_keep(82) .eq. 0) then
         deallocate(cv_potsplit,STAT=ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'Memory deallocation error in ',subname
            istat = cv_error_memdeloc
            return
         end if
      endif
      nullify(cv_frere,cv_fils,cv_nfsiz,cv_ne,cv_keep,cv_keep8,         &
     &        cv_icntl,cv_info,cv_procnode,cv_ssarbr)
      deallocate(cv_proc_workload,cv_proc_maxwork,cv_proc_memused,      &
     &    cv_proc_maxmem,cv_nodetype,                                   &
     &    cv_nodelayer,cv_proc_sorted,                                  &
     &    cv_ncostw,cv_ncostm,                                          &
     &    cv_layerworkload,cv_layermemused,                             &
     &    STAT=ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      deallocate(work_per_proc,id_son,STAT=ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      do layernmb=1,cv_maxlayer
         if(cv_layer_p2node(layernmb)%nmb_t2s.gt.0) then
            deallocate(cv_layer_p2node(layernmb)%t2_nodenumbers,        &
     &                 cv_layer_p2node(layernmb)%t2_cand,               &
     &                 cv_layer_p2node(layernmb)%t2_candcostw,          &
     &                 cv_layer_p2node(layernmb)%t2_candcostm,          &
     &                 STAT=ierr)
            if(ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Memory deallocation error in ',           &
     &                        subname
               istat = cv_error_memdeloc
               return
            end if
         endif
      enddo
      if(associated(cv_layer_p2node)) then
         deallocate(cv_layer_p2node,STAT=ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &      write(cv_lp,*)'Memory deallocation error in ',subname
            istat = cv_error_memdeloc
            return
         end if
      end if
      do i=1,cv_n
         call AGMG_MUMPS_435(i,ierr)
         if(ierr.ne.0) then
            if(cv_lp.gt.0)                                              &
     &           write(cv_lp,*)'PROPMAP_TERM signalled error in ',      &
     &           subname
            istat = ierr
            return
         end if
      end do
      if(associated(cv_prop_map))deallocate(cv_prop_map,STAT=ierr)
      if(ierr.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      istat=0
      return
      end subroutine AGMG_MUMPS_403
      recursive subroutine AGMG_MUMPS_404(pos,istat)
      implicit none
      integer,intent(in)::pos
      integer, intent(out)::istat
      integer i,nfront,npiv,nextpos,ierr
      DOUBLE PRECISION costw,costm
      character (len=48):: subname
      istat=-1
      subname='TREECOSTS'
      if ((.NOT.associated(cv_tcostw)).OR.(.NOT.associated(cv_tcostm))) &
     &   then
         if(cv_lp.gt.0)                                                 &
     &        write(cv_lp,*)'Error:tcost must be allocated in ',subname
         return
      end if
      nfront=cv_nfsiz(pos)
      npiv=1
      nextpos=cv_fils(pos)
      do
         if(nextpos.le.0) then
            exit
         else
            npiv=npiv+1
            nextpos=cv_fils(nextpos)
         end if
      end do
      call AGMG_MUMPS_418(npiv,nfront,costw,costm)
      cv_ncostw(pos)=costw
      cv_ncostm(pos)=costm
      if (cv_ne(pos).ne.0) then
         nextpos=cv_fils(pos)
         do while(nextpos.gt.0)
            nextpos=cv_fils(nextpos)
         end do
         nextpos=-nextpos
         do i=1,cv_ne(pos)
            cv_depth(nextpos)=cv_depth(pos)+1
            call AGMG_MUMPS_404(nextpos,ierr)
            if (ierr.ne.0) then
               if(cv_lp.gt.0)                                           &
     &         write(cv_lp,*)'Failure in recursive call to ',subname
               return
            end if
            costw=costw+cv_tcostw(nextpos)
            costm=costm+cv_tcostm(nextpos)
            nextpos=cv_frere(nextpos)
         end do
      endif
      cv_tcostw(pos) = costw
      cv_tcostm(pos) = costm
      istat = 0
      end subroutine AGMG_MUMPS_404
      recursive subroutine AGMG_MUMPS_406(inode)
      implicit none
      integer, intent(in)::inode
      integer in
      cv_nodetype(inode)=-1
      in=cv_fils(inode)
      do while (in>0)
         in=cv_fils(in)
      end do
      in=-in
      do while(in.gt.0)
         call AGMG_MUMPS_406(in)
         in=cv_frere(in)
      enddo
      end subroutine AGMG_MUMPS_406
      subroutine AGMG_MUMPS_408(workload,memused,                            &
     &                                    maxwork,minwork,maxmem,minmem)
      implicit none
      DOUBLE PRECISION,dimension(cv_slavef),intent(in)::workload,       &
     &                                                  memused
      DOUBLE PRECISION,intent(out)::maxwork,minwork,maxmem,minmem
      intrinsic maxval,minval
      maxwork=maxval(workload)
      minwork=minval(workload, mask= workload > dble(0))
      maxmem=maxval(memused)
      minmem=minval(memused, mask= memused > dble(0))
      end subroutine AGMG_MUMPS_408
      subroutine AGMG_MUMPS_476(layernumber,nodenumber)
      implicit none
      integer layernumber,nodenumber
      integer i
      integer inode
      integer current_max,current_proc
      current_max = 0
      score = 0
      allowed_nodes = .FALSE.
      inode=cv_layer_p2node(layernumber)%t2_nodenumbers(nodenumber)
      do i=1,cv_layer_p2node(layernumber)%t2_cand(nodenumber,           &
     &     cv_slavef+1)
         current_proc=cv_layer_p2node(layernumber)%t2_cand(nodenumber,i)
         if ( current_proc .ge. 0) then
            score(mem_distribmpi(current_proc)) =                       &
     &           score(mem_distribmpi(current_proc)) + 1
         endif
      enddo
      current_proc = cv_procnode(inode) - 1
      score(mem_distribmpi(current_proc)) =                             &
     &     score(mem_distribmpi(current_proc)) + 1
      do i=0,nb_arch_nodes - 1
         if ( score(i) .gt. current_max ) then
            current_max = score(i)
            allowed_nodes = .FALSE.
            allowed_nodes(i) = .TRUE.
         else
            if(score(i) .eq. current_max) then
               allowed_nodes(i) = .TRUE.
            endif
         endif
      enddo
      return
      end subroutine AGMG_MUMPS_476
      end subroutine AGMG_MUMPS_369
      subroutine AGMG_MUMPS_393(par2_nodes,cand,istat)
      integer, intent(out)::par2_nodes(cv_nb_niv2),                     &
     &                      cand(cv_slavef+1,cv_nb_niv2),istat
      character (len=48):: subname
      integer iloop
      istat=-1
      subname='MUMPS_393'
      par2_nodes=cv_par2_nodes
      do iloop=1, cv_slavef+1
        cand(iloop,:)=cv_cand(:,iloop)
      enddo
      deallocate(cv_par2_nodes,cv_cand,STAT=istat)
      if(istat.ne.0) then
         if(cv_lp.gt.0)                                                 &
     &   write(cv_lp,*)'Memory deallocation error in ',subname
         istat = cv_error_memdeloc
         return
      end if
      istat = 0
      return
      end subroutine AGMG_MUMPS_393
      subroutine AGMG_MUMPS_427(                                             &
     &     total_comm,working_comm,keep69,par,                          &
     &     nbslaves,mem_distrib,informerr)
      implicit none
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
      integer nbslaves
      integer, dimension(0:nbslaves-1) :: mem_distrib
      integer total_comm,working_comm,keep69,par
      integer, dimension(40) ::informerr
      integer myrank
      integer host,i,ierr
      integer,dimension(:),allocatable :: buffer_memdistrib
      ierr = 0
      myrank = -1
      host = -1
      ke69 = keep69
      cv_slavef = nbslaves
      if (ke69 .eq. 1) then
         return
      endif
      if ( allocated(mem_distribtmp) ) deallocate(mem_distribtmp )
      allocate( mem_distribtmp( 0:cv_slavef-1 ),                        &
     &          buffer_memdistrib( 0:cv_slavef-1 ), stat=ierr )
      if ( ierr .gt. 0 ) then
         if(cv_mp.gt.0) write(cv_mp,*) 'pb allocation mem_dist'
         informerr(1) = -13
         informerr(2) = cv_slavef
         return
      end if
      mem_distribtmp = -1
      call AGMG_SETIERR2( host, ierr )
      if ((par .eq. 1) .or. (host .ne. 0)) then
         call AGMG_SETIERR2( myrank, ierr )
         call AGMG_MUMPS_430(ierr,myrank,                                    &
     &        working_comm,mem_distrib(0))
         if ( ierr .ne. 0 ) then
            if(cv_mp.gt.0)                                              &
     &      write(cv_mp,*) 'pb in mumps_init_arch_parameters'
            informerr(1) = -13
            informerr(2) = cv_slavef
            return
         end if
         mem_distribtmp = mem_distrib
         call AGMG_MUMPS_429(ierr)
         if ( ierr .ne. 0 ) then
            if(cv_mp.gt.0) write(cv_mp,*)                               &
     &'pb in mumps_init_arch_parameters'
            informerr(1) = -13
            informerr(2) = cv_slavef
            return
         endif
      endif
      if(ke69 .le. 0) then
         deallocate(mem_distribtmp)
         deallocate(buffer_memdistrib)
         return
      endif
      call AGMG_COPY(mem_distribtmp(0),buffer_memdistrib(0),                     cv_slavef,AGMG_MPF_INTEGER,ierr)
      mem_distribtmp = buffer_memdistrib
      deallocate (buffer_memdistrib)
      call AGMG_MUMPS_492()
        if((cv_slavef/nb_arch_nodes) .le. 4) then
          do i = 0, cv_slavef-1
            if ( mem_distrib(i) .NE. 1 ) then
              mem_distrib(i)=max(ke69/2,2)
            endif
          enddo
        endif
        if((nb_arch_nodes .eq. 1) .or.                                  &
     &     (nb_arch_nodes .eq. cv_slavef)) then
         ke69 = 1
         keep69 = 1
         deallocate(mem_distribtmp)
         return
      endif
      if (host .eq. 0) then
         if ( allocated(mem_distribmpi) ) deallocate(mem_distribmpi )
         allocate( mem_distribmpi( 0:cv_slavef-1 ), stat=ierr )
         if ( ierr .gt. 0 ) then
            if(cv_mp.gt.0) write(cv_mp,*) 'pb allocation mem_dist'
            informerr(1) = -13
            informerr(2) = cv_slavef
            return
         endif
         call AGMG_MUMPS_495(ierr)
         if(ierr .ne. 0 ) then
            return
         endif
         mem_distribmpi = mem_distribtmp
         call AGMG_MUMPS_428(ierr)
         if ( ierr .ne. 0 ) then
            if(cv_mp.gt.0)                                              &
     &      write(cv_mp,*) 'pb in mumps_init_arch_parameters'
            informerr(1) = -13
            informerr(2) = cv_slavef
            return
         endif
      else
         deallocate(mem_distribtmp)
      endif
      return
      end subroutine AGMG_MUMPS_427
      subroutine AGMG_MUMPS_492()
      implicit none
      integer i
      nb_arch_nodes = 0
      do i=0,cv_slavef-1
         if(mem_distribtmp(i) .eq. i) then
            nb_arch_nodes = nb_arch_nodes + 1
         endif
      enddo
      return
      end subroutine AGMG_MUMPS_492
      subroutine AGMG_MUMPS_428(ierr)
      implicit none
      external AGMG_MUMPS_463
      integer i,precnode,nodecount
      integer sizesmp
      integer ierr
      ierr = 0
      sizesmp = 0
      if ( allocated(table_of_process) )                                &
     &     deallocate(table_of_process  )
      allocate( table_of_process(0:cv_slavef-1), stat=ierr )
      if ( ierr .gt. 0 ) then
         if(cv_mp.gt.0) write(cv_mp,*)                                  &
     &   'pb allocation in MUMPS_428'
         return
      end if
      do i=0,cv_slavef - 1
         table_of_process(i) = i
      enddo
      call AGMG_MUMPS_463(cv_slavef,mem_distribtmp(0),                       &
     &     table_of_process(0))
      precnode = 0
      nodecount = 0
      do i=0,cv_slavef-1
         if(mem_distribtmp(i) .eq. precnode) then
            sizesmp = sizesmp + 1
            mem_distribtmp(i) = nodecount
            mem_distribmpi(table_of_process(i)) = nodecount
         else
            score(nodecount) = sizesmp
            sizesmp = 1
            nodecount = nodecount + 1
            precnode = mem_distribtmp(i)
            mem_distribtmp(i) = nodecount
            mem_distribmpi(table_of_process(i)) = nodecount
         endif
      enddo
      score(nodecount) = sizesmp
      do i=0,cv_slavef-1
         mem_distribtmp(i) = score(mem_distribtmp(i))
      enddo
      CALL AGMG_MUMPS_466(cv_slavef,mem_distribtmp(0),                       &
     &     table_of_process(0))
      ierr = 0
      return
      end subroutine AGMG_MUMPS_428
      subroutine AGMG_MUMPS_429(ierr)
      implicit none
      integer i,j,ierr
      integer idmaster
      idmaster = -1
      ierr = 0
      do i=0,cv_slavef-1
         if (mem_distribtmp(i) .eq. 1) then
            idmaster = i
            do j=i,cv_slavef-1
               if (mem_distribtmp(j) .eq. 1) then
                  mem_distribtmp(j) = idmaster
               else
                  mem_distribtmp(j) = 0
               endif
            enddo
            return
         else
            mem_distribtmp(i) = 0
         endif
      enddo
      if(cv_mp.gt.0) write(cv_mp,*)'problem in MUMPS_429:               &
     &     cannot find a master'
      ierr = 1
      return
      end subroutine AGMG_MUMPS_429
      subroutine AGMG_MUMPS_430(ierr,myrank,working_comm,                    &
     &     mem_distrib)
      implicit none
!
      INTEGER AGMG_MPF_2DOUBLE_PRECISION
      INTEGER AGMG_MPF_2INTEGER
      INTEGER AGMG_MPF_2REAL
      INTEGER AGMG_MPF_ANY_SOURCE
      INTEGER AGMG_MPF_ANY_TAG
      INTEGER AGMG_MPF_BYTE
      INTEGER AGMG_MPF_CHARACTER
      INTEGER AGMG_MPF_COMM_NULL
      INTEGER AGMG_MPF_COMM_WORLD
      INTEGER AGMG_MPF_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_COMPLEX
      INTEGER AGMG_MPF_DOUBLE_PRECISION
      INTEGER AGMG_MPF_INTEGER
      INTEGER AGMG_MPF_LOGICAL
      INTEGER AGMG_MPF_MAX
      INTEGER AGMG_MPF_MAX_PROCESSOR_NAME
      INTEGER AGMG_MPF_MAXLOC
      INTEGER AGMG_MPF_MIN
      INTEGER AGMG_MPF_MINLOC
      INTEGER AGMG_MPF_PACKED
      INTEGER AGMG_MPF_PROD
      INTEGER AGMG_MPF_REAL
      INTEGER AGMG_MPF_REPLACE
      INTEGER AGMG_MPF_REQUEST_NULL
      INTEGER AGMG_MPF_SOURCE
      INTEGER AGMG_MPF_STATUS_SIZE
      INTEGER AGMG_MPF_SUM
      INTEGER AGMG_MPF_TAG
      INTEGER AGMG_MPF_UNDEFINED
      INTEGER AGMG_MPF_WTIME_IS_GLOBAL
      INTEGER AGMG_MPF_LOR
      INTEGER AGMG_MPF_LAND
      INTEGER AGMG_MPF_INTEGER8
      INTEGER AGMG_MPF_REAL8
      INTEGER AGMG_MPF_BSEND_OVERHEAD
      PARAMETER (AGMG_MPF_2DOUBLE_PRECISION=1)
      PARAMETER (AGMG_MPF_2INTEGER=2)
      PARAMETER (AGMG_MPF_2REAL=3)
      PARAMETER (AGMG_MPF_ANY_SOURCE=4)
      PARAMETER (AGMG_MPF_ANY_TAG=5)
      PARAMETER (AGMG_MPF_BYTE=6)
      PARAMETER (AGMG_MPF_CHARACTER=7)
      PARAMETER (AGMG_MPF_COMM_NULL=8)
      PARAMETER (AGMG_MPF_COMM_WORLD=9)
      PARAMETER (AGMG_MPF_COMPLEX=10)
      PARAMETER (AGMG_MPF_DOUBLE_COMPLEX=11)
      PARAMETER (AGMG_MPF_DOUBLE_PRECISION=12)
      PARAMETER (AGMG_MPF_INTEGER=13)
      PARAMETER (AGMG_MPF_LOGICAL=14)
      PARAMETER (AGMG_MPF_MAX=15)
      PARAMETER (AGMG_MPF_MAX_PROCESSOR_NAME=31)
      PARAMETER (AGMG_MPF_MAXLOC=16)
      PARAMETER (AGMG_MPF_MIN=17)
      PARAMETER (AGMG_MPF_MINLOC=18)
      PARAMETER (AGMG_MPF_PACKED=19)
      PARAMETER (AGMG_MPF_PROD=20)
      PARAMETER (AGMG_MPF_REAL=21)
      PARAMETER (AGMG_MPF_REPLACE=22)
      PARAMETER (AGMG_MPF_REQUEST_NULL=23)
      PARAMETER (AGMG_MPF_SOURCE=1)
      PARAMETER (AGMG_MPF_STATUS_SIZE=2)
      PARAMETER (AGMG_MPF_SUM=26)
      PARAMETER (AGMG_MPF_TAG=2)
      PARAMETER (AGMG_MPF_UNDEFINED=28)
      PARAMETER (AGMG_MPF_WTIME_IS_GLOBAL=30)
      PARAMETER (AGMG_MPF_LOR=31)
      PARAMETER (AGMG_MPF_LAND=32)
      PARAMETER (AGMG_MPF_INTEGER8=33)
      PARAMETER (AGMG_MPF_REAL8=34)
      PARAMETER (AGMG_MPF_BSEND_OVERHEAD=0)
      DOUBLE PRECISION AGMG_MPF_WTIME
      EXTERNAL AGMG_MPF_WTIME
      INTERFACE AGMG_COPY
      SUBROUTINE AGMG_COPY_INTEGER( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_INTEGER
      SUBROUTINE AGMG_COPY_INTEGER1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R(N)
      END SUBROUTINE AGMG_COPY_INTEGER1
      SUBROUTINE AGMG_COPY_INTEGER11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S,R
      END SUBROUTINE AGMG_COPY_INTEGER11
      SUBROUTINE AGMG_COPY_INTEGER1_81( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER(8) :: S
      INTEGER:: R
      END SUBROUTINE AGMG_COPY_INTEGER1_81
      SUBROUTINE AGMG_COPY_INTEGER10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      INTEGER :: S(N),R
      END SUBROUTINE AGMG_COPY_INTEGER10
      SUBROUTINE AGMG_COPY_INTEGER21( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N(*), DATATYPE, IERR
      INTEGER :: S(N(1),1),R(N(1))
      END SUBROUTINE AGMG_COPY_INTEGER21
      SUBROUTINE AGMG_COPY_REAL( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_REAL
      SUBROUTINE AGMG_COPY_REAL1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R(N)
      END SUBROUTINE AGMG_COPY_REAL1
      SUBROUTINE AGMG_COPY_REAL11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S,R
      END SUBROUTINE AGMG_COPY_REAL11
      SUBROUTINE AGMG_COPY_REAL10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      REAL :: S(N),R
      END SUBROUTINE AGMG_COPY_REAL10
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION1
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION11
      SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE PRECISION :: S(N),R
      END SUBROUTINE AGMG_COPY_DOUBLE_PRECISION10
      SUBROUTINE AGMG_COPY_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_COMPLEX
      SUBROUTINE AGMG_COPY_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_COMPLEX1
      SUBROUTINE AGMG_COPY_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_COMPLEX11
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S(*),R(*)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R(N)
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX1
      SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11( S, R, N, DATATYPE, IERR )
      IMPLICIT NONE
      INTEGER :: N, DATATYPE, IERR
      DOUBLE COMPLEX :: S,R
      END SUBROUTINE AGMG_COPY_DOUBLE_COMPLEX11
      END INTERFACE
      integer ierr,resultlen,myrank,i,working_comm
      integer , dimension(0:cv_slavef-1) :: mem_distrib
      integer allocok
      character(len=AGMG_MPF_MAX_PROCESSOR_NAME) name
      integer, dimension(:),allocatable :: namercv
      integer, dimension(:),allocatable :: myname
      integer lenrcv
      external AGMG_MUMPS_438
      logical AGMG_MUMPS_438
      ierr = 0
      call AGMG_MPF_get_processor_name(name,resultlen,ierr)
      allocate(myname(resultlen),stat=allocok)
      if ( allocok .gt. 0 ) then
        if(cv_mp.gt.0) write(cv_mp,*)                                   &
     &  'pb allocation in compute_dist for myname'
        ierr = 1
        return
      end if
      do i=1, resultlen
         myname(i) = ichar(name(i:i))
      enddo
      do i=0, cv_slavef-1
         if(myrank .eq. i) then
            lenrcv = resultlen
         else
            lenrcv = 0
         endif
         call AGMG_SETIERR(ierr)
         allocate(namercv(lenrcv),stat=allocok)
         if ( allocok .gt. 0 ) then
            if(cv_mp.gt.0) write(cv_mp,*)                               &
     &      'pb allocation in compute_dist for namercv'
            ierr = 1
            return
         end if
         if(myrank .eq. i) then
            namercv = myname
         endif
         call AGMG_SETIERR(ierr)
         if( AGMG_MUMPS_438(myname,namercv,                                  &
     &        resultlen,lenrcv)) then
            mem_distrib(i)=1
         else
            mem_distrib(i)=ke69
         endif
         deallocate(namercv)
      enddo
      deallocate(myname)
      ierr = 0
      return
      end subroutine AGMG_MUMPS_430
      subroutine AGMG_MUMPS_493(current_proc,idarch,ierr)
      implicit none
      integer current_proc
      integer idarch,ierr
      ierr = 0
      if (current_proc .ge. cv_slavef) then
         ierr = -1
         return
      endif
      if (current_proc .lt. 0) then
         idarch = 1
         return
      else
         idarch = table_of_process(current_proc) + 1
      endif
      return
      end subroutine AGMG_MUMPS_493
      subroutine AGMG_MUMPS_494()
      if (allocated(table_of_process)) deallocate(table_of_process)
      if (allocated(allowed_nodes)) deallocate(allowed_nodes)
      if (allocated(score)) deallocate(score)
      if (allocated(mem_distribtmp)) deallocate(mem_distribtmp)
      if (allocated(mem_distribmpi)) deallocate(mem_distribmpi)
      return
      end subroutine AGMG_MUMPS_494
      subroutine AGMG_MUMPS_495(ierr)
      integer ierr
      ierr = 0
      if (allocated(allowed_nodes)) deallocate(allowed_nodes)
      allocate( allowed_nodes(0:nb_arch_nodes-1),stat=ierr)
      if ( ierr .gt. 0 ) then
         if(cv_mp.gt.0) write(cv_mp,*)                                  &
     &   'pb allocation MUMPS_495'
         ierr = -13
         return
      end if
      allowed_nodes = .FALSE.
       if (allocated(score)) deallocate(score)
      allocate( score(0:nb_arch_nodes-1),stat=ierr)
      if ( ierr .gt. 0 ) then
         if(cv_mp.gt.0) write(cv_mp,*)                                  &
     &   'pb allocation MUMPS_495'
         ierr = -13
         return
      end if
      score = 0
      ierr = 0
      return
      end subroutine AGMG_MUMPS_495
      subroutine AGMG_MUMPS_496(idproc,thenode)
      implicit none
      integer idproc,thenode
      thenode = mem_distribmpi(idproc)
      return
      end subroutine AGMG_MUMPS_496
      SUBROUTINE AGMG_MUMPS_516(start1st,end1st,dim1,                        &
     &                             start2nd,end2nd,dim2,                &
     &                             indx,                                &
     &                             val)
      implicit none
      integer, intent(in):: start1st,end1st,dim1,start2nd,end2nd,dim2
      integer, intent(inout):: indx(end2nd)
      DOUBLE PRECISION, intent(inout):: val(end2nd)
      integer::index(dim1+dim2)
      DOUBLE PRECISION ::dummy1(dim1+dim2)
      integer:: a,b,c
      a=start1st
      b=start2nd
      c=1
      do while((a.LT.end1st+1).AND.(b.LT.end2nd+1))
         if(val(a).GT.val(b))then
            index(c)=indx(a)
            dummy1(c)=val(a)
            a=a+1
            c=c+1
         else
            index(c)=indx(b)
            dummy1(c)=val(b)
            b=b+1
            c=c+1
         endif
      end do
      if(a.LT.end1st+1) then
         do while(a.LT.end1st+1)
            index(c)=indx(a)
            dummy1(c)=val(a)
            a=a+1
            c=c+1
         enddo
      elseif(b.LT.end2nd+1) then
         do while(b.LT.end2nd+1)
            index(c)=indx(b)
            dummy1(c)=val(b)
            b=b+1
            c=c+1
         enddo
      endif
      indx(start1st:end1st)=index(1:dim1)
      val(start1st:end1st)=dummy1(1:dim1)
      indx(start2nd:end2nd)=index(dim1+1:dim1+dim2)
      val(start2nd:end2nd)=dummy1(dim1+1:dim1+dim2)
      end SUBROUTINE AGMG_MUMPS_516
      SUBROUTINE AGMG_MUMPS_459(dim,indx,val1,val2)
      implicit none
      integer, intent(in):: dim
      integer, intent(inout):: indx(dim)
      DOUBLE PRECISION, intent(inout):: val1(dim)
      DOUBLE PRECISION, intent(inout),optional:: val2(dim)
      integer::index(dim),dummy1(dim)
      DOUBLE PRECISION ::dummy2(dim)
      integer, parameter :: ss = 35
      integer:: a,b,c,i,j,k,l,r,s,stackl(ss),stackr(ss)
      DOUBLE PRECISION :: w,x
      do i=1,dim
         index(i)=i
      enddo
      s = 1
      stackl(1) = 1
      stackr(1) = dim
 5511 CONTINUE
      l = stackl(s)
      r = stackr(s)
      k = (l+r) / 2
      if(l.LT.k) then
         if(s.GE.ss) stop 'maxsize of stack reached'
         s = s + 1
         stackl(s) = l
         stackr(s) = k
         goto 5511
      endif
 5512 CONTINUE
      l = stackl(s)
      r = stackr(s)
      k = (l+r) / 2
      if(k+1.LT.r) then
         if(s.GE.ss) stop 'maxsize of stack reached'
         s = s + 1
         stackl(s) = k+1
         stackr(s) = r
         goto 5511
      endif
 5513 CONTINUE
      l = stackl(s)
      r = stackr(s)
      k = (l+r) / 2
      a=l
      b=k+1
      c=1
      do while((a.LT.k+1).AND.(b.LT.r+1))
         if(val1(index(a)).GT.val1(index(b)))then
            dummy1(c)=index(a)
            a=a+1
            c=c+1
         else
            dummy1(c)=index(b)
            b=b+1
            c=c+1
         endif
      end do
      if(a.LT.k+1) then
         dummy1(c:r-l+1)=index(a:k)
      elseif(b.LT.r+1) then
         dummy1(c:r-l+1)=index(b:r)
      endif
      index(l:r)=dummy1(1:r-l+1)
      if(s.GT.1) then
         s = s - 1
         if(l.EQ.stackl(s)) goto 5512
         if(r.EQ.stackr(s)) goto 5513
      endif
      do i=1,dim
         dummy1(i)=indx(index(i))
      enddo
      indx=dummy1
      do i=1,dim
         dummy2(i)=val1(index(i))
      enddo
      val1=dummy2
      if(present(val2)) then
         do i=1,dim
            dummy2(i)=val2(index(i))
         enddo
         val2=dummy2
      endif
      return
      end subroutine AGMG_MUMPS_459
      END MODULE AGMG_MUMPS_STATIC_MAP
